Package: netrics
Title: Many Ways to Measure and Classify Membership for Networks,
        Nodes, and Ties
Version: 0.2.1
Date: 2026-04-04
Description: Many tools for calculating network, node, or tie
   marks, measures, motifs and memberships of many different types of networks.
   Marks identify structural positions, measures quantify network properties,
   memberships classify nodes into groups, and motifs tabulate substructure participation.
   All functions operate with all classes of network data covered in 'manynet',
   and on directed, undirected, multiplex, multimodal, signed, and other networks.
URL: https://stocnet.github.io/netrics/
BugReports: https://github.com/stocnet/netrics/issues
License: MIT + file LICENSE
Language: en-GB
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 3.6.0), manynet
Imports: dplyr, igraph (>= 2.1.0)
Suggests: autograph, sna, testthat (>= 3.0.0)
Authors@R: 
    c(person(given = "James",
             family = "Hollway",
             role = c("cre", "aut", "ctb"),
             email = "james.hollway@graduateinstitute.ch",
             comment = c("IHEID", ORCID = "0000-0002-8361-9647"))
    )
Config/Needs/build: roxygen2, devtools
Config/Needs/check: covr, lintr, spelling
Config/Needs/website: pkgdown
Config/testthat/parallel: true
Config/testthat/edition: 3
Config/testthat/start-first: measure_net
NeedsCompilation: no
Packaged: 2026-04-04 08:25:06 UTC; hollway
Author: James Hollway [cre, aut, ctb] (IHEID, ORCID:
    <https://orcid.org/0000-0002-8361-9647>)
Maintainer: James Hollway <james.hollway@graduateinstitute.ch>
Repository: CRAN
Date/Publication: 2026-04-06 05:10:16 UTC
Built: R 4.6.0; ; 2026-04-29 21:48:38 UTC; unix

# Generated by roxygen2: do not edit by hand

export("%>%")
export(cluster_concor)
export(cluster_cosine)
export(cluster_hierarchical)
export(k_elbow)
export(k_gap)
export(k_silhouette)
export(k_strict)
export(net_by_adhesion)
export(net_by_assortativity)
export(net_by_balance)
export(net_by_betweenness)
export(net_by_closeness)
export(net_by_cohesion)
export(net_by_components)
export(net_by_congruency)
export(net_by_connectedness)
export(net_by_core)
export(net_by_degree)
export(net_by_density)
export(net_by_diameter)
export(net_by_diversity)
export(net_by_efficiency)
export(net_by_eigenvector)
export(net_by_equivalency)
export(net_by_factions)
export(net_by_harmonic)
export(net_by_heterophily)
export(net_by_homophily)
export(net_by_immunity)
export(net_by_indegree)
export(net_by_independence)
export(net_by_infection_complete)
export(net_by_infection_peak)
export(net_by_infection_total)
export(net_by_length)
export(net_by_modularity)
export(net_by_outdegree)
export(net_by_reach)
export(net_by_reciprocity)
export(net_by_recovery)
export(net_by_reproduction)
export(net_by_richclub)
export(net_by_richness)
export(net_by_scalefree)
export(net_by_smallworld)
export(net_by_spatial)
export(net_by_strength)
export(net_by_toughness)
export(net_by_transitivity)
export(net_by_transmissibility)
export(net_by_upperbound)
export(net_by_waves)
export(net_x_brokerage)
export(net_x_change)
export(net_x_correlation)
export(net_x_dyad)
export(net_x_hazard)
export(net_x_hierarchy)
export(net_x_mixed)
export(net_x_stability)
export(net_x_tetrad)
export(net_x_triad)
export(node_by_adopt_exposure)
export(node_by_adopt_recovery)
export(node_by_adopt_threshold)
export(node_by_adopt_time)
export(node_by_alpha)
export(node_by_authority)
export(node_by_betweenness)
export(node_by_bridges)
export(node_by_brokering_activity)
export(node_by_brokering_exclusivity)
export(node_by_closeness)
export(node_by_constraint)
export(node_by_coreness)
export(node_by_deg)
export(node_by_degree)
export(node_by_distance)
export(node_by_diversity)
export(node_by_eccentricity)
export(node_by_efficiency)
export(node_by_effsize)
export(node_by_eigenvector)
export(node_by_equivalency)
export(node_by_flow)
export(node_by_harmonic)
export(node_by_heterophily)
export(node_by_hierarchy)
export(node_by_homophily)
export(node_by_hub)
export(node_by_indegree)
export(node_by_induced)
export(node_by_information)
export(node_by_kcoreness)
export(node_by_leverage)
export(node_by_multidegree)
export(node_by_neighbours_degree)
export(node_by_outdegree)
export(node_by_pagerank)
export(node_by_posneg)
export(node_by_power)
export(node_by_randomwalk)
export(node_by_reach)
export(node_by_reciprocity)
export(node_by_redundancy)
export(node_by_richness)
export(node_by_stress)
export(node_by_subgraph)
export(node_by_transitivity)
export(node_by_vitality)
export(node_in_adopter)
export(node_in_automorphic)
export(node_in_betweenness)
export(node_in_brokering)
export(node_in_community)
export(node_in_component)
export(node_in_core)
export(node_in_eigen)
export(node_in_equivalence)
export(node_in_fluid)
export(node_in_greedy)
export(node_in_infomap)
export(node_in_leiden)
export(node_in_louvain)
export(node_in_optimal)
export(node_in_partition)
export(node_in_regular)
export(node_in_roulette)
export(node_in_spinglass)
export(node_in_strong)
export(node_in_structural)
export(node_in_walktrap)
export(node_in_weak)
export(node_is_core)
export(node_is_cutpoint)
export(node_is_exposed)
export(node_is_fold)
export(node_is_independent)
export(node_is_infected)
export(node_is_isolate)
export(node_is_latent)
export(node_is_max)
export(node_is_mean)
export(node_is_mentor)
export(node_is_min)
export(node_is_neighbor)
export(node_is_pendant)
export(node_is_random)
export(node_is_recovered)
export(node_is_universal)
export(node_x_brokerage)
export(node_x_dyad)
export(node_x_exposure)
export(node_x_path)
export(node_x_tetrad)
export(node_x_tie)
export(node_x_triad)
export(tie_by_betweenness)
export(tie_by_closeness)
export(tie_by_cohesion)
export(tie_by_degree)
export(tie_by_eigenvector)
export(tie_is_bridge)
export(tie_is_cyclical)
export(tie_is_feedback)
export(tie_is_imbalanced)
export(tie_is_loop)
export(tie_is_max)
export(tie_is_min)
export(tie_is_multiple)
export(tie_is_path)
export(tie_is_random)
export(tie_is_reciprocated)
export(tie_is_simmelian)
export(tie_is_transitive)
export(tie_is_triangular)
export(tie_is_triplet)
importFrom(dplyr,"%>%")
importFrom(igraph,V)
importFrom(igraph,adhesion)
importFrom(igraph,all_shortest_paths)
importFrom(igraph,alpha_centrality)
importFrom(igraph,articulation_points)
importFrom(igraph,assortativity_degree)
importFrom(igraph,cohesion)
importFrom(igraph,components)
importFrom(igraph,decompose)
importFrom(igraph,degree)
importFrom(igraph,delete_edges)
importFrom(igraph,delete_vertices)
importFrom(igraph,diameter)
importFrom(igraph,distances)
importFrom(igraph,edge_betweenness)
importFrom(igraph,edge_density)
importFrom(igraph,feedback_arc_set)
importFrom(igraph,fit_power_law)
importFrom(igraph,graph_from_incidence_matrix)
importFrom(igraph,is_bipartite)
importFrom(igraph,ivs_size)
importFrom(igraph,knn)
importFrom(igraph,largest_ivs)
importFrom(igraph,make_ego_graph)
importFrom(igraph,mean_distance)
importFrom(igraph,power_centrality)
importFrom(igraph,reciprocity)
importFrom(igraph,transitivity)
importFrom(igraph,triad_census)
importFrom(igraph,triangles)
importFrom(igraph,vcount)
importFrom(igraph,which_loop)
importFrom(igraph,which_multiple)
importFrom(igraph,which_mutual)
importFrom(manynet,as_igraph)
importFrom(manynet,is_complex)
importFrom(manynet,is_twomode)
importFrom(manynet,is_weighted)
importFrom(manynet,tie_weights)
importFrom(stats,as.dist)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,cutree)
importFrom(stats,hclust)
importFrom(stats,median)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# netrics

<img src="man/figures/logo.png" align="right" alt="netrics logo" width="150"/>

<!-- badges: start -->

[![Lifecycle:
maturing](https://img.shields.io/badge/lifecycle-maturing-blue.svg)](https://lifecycle.r-lib.org/articles/stages.html#maturing)
![CRAN/METACRAN](https://img.shields.io/cran/v/netrics) ![GitHub release
(latest by
date)](https://img.shields.io/github/v/release/stocnet/netrics) ![GitHub
Release
Date](https://img.shields.io/github/release-date/stocnet/netrics)
[![Codecov test
coverage](https://codecov.io/gh/stocnet/netrics/branch/main/graph/badge.svg)](https://app.codecov.io/gh/stocnet/netrics?branch=main)
<!-- badges: end -->

## About the package

`{netrics}` is the analytic engine of the
[stocnet](https://github.com/stocnet) ecosystem. It provides *many*
tools for marking, measuring, and identifying nodes’ motifs and
memberships in *many* (if not most) types and kinds of networks. All
functions operate on matrices, `{igraph}`, `{network}`, `{tidygraph}`,
and `mnet` class objects, and recognise directed, weighted, multiplex,
multimodal, signed, and other types of networks.

`{netrics}` depends on
[`{manynet}`](https://stocnet.github.io/manynet/), which handles working
with different network classes, converting between them, and modifying
them. Network-level logical tests (e.g. `is_directed()`, `is_twomode()`)
remain in `{manynet}`, while node- and tie-level analytic marks
(e.g. `node_is_*()`, `tie_is_*()`) are in `{netrics}`. For graph
drawing, see [`{autograph}`](https://stocnet.github.io/autograph/), and
for further testing and modelling capabilities see
[`{migraph}`](https://stocnet.github.io/migraph/).

- [Marking](#marking)
- [Measures](#measures)
- [Memberships](#memberships)
- [Motifs](#motifs)
- [Analysis](#analysis)
- [Installation](#installation)
  - [Stable](#stable)
  - [Development](#development)
- [Relationship to other packages](#relationship-to-other-packages)
- [Funding details](#funding-details)

## Marking

`{netrics}` includes four special groups of functions, each with their
own pretty `print()` and `plot()` methods: marks, measures, motifs, and
memberships. Marks are logical scalars or vectors, measures are numeric,
memberships categorical, and motifs result in tabular outputs.

`{netrics}`’s `node_is_*()` and `tie_is_*()` functions offer fast
logical tests of node- and tie-level properties. `node_is_*()` returns a
logical vector the length of the number of nodes in the network, and
`tie_is_*()` returns a logical vector the length of the number of ties
in the network. Note that network-level tests such as `is_directed()`
and `is_twomode()` are in `{manynet}`.

- `node_is_core()`, `node_is_cutpoint()`, `node_is_exposed()`,
  `node_is_fold()`, `node_is_independent()`, `node_is_infected()`,
  `node_is_isolate()`, `node_is_latent()`, `node_is_max()`,
  `node_is_mean()`, `node_is_mentor()`, `node_is_min()`,
  `node_is_neighbor()`, `node_is_pendant()`, `node_is_random()`,
  `node_is_recovered()`, `node_is_universal()`
- `tie_is_bridge()`, `tie_is_cyclical()`, `tie_is_feedback()`,
  `tie_is_imbalanced()`, `tie_is_loop()`, `tie_is_max()`,
  `tie_is_min()`, `tie_is_multiple()`, `tie_is_path()`,
  `tie_is_random()`, `tie_is_reciprocated()`, `tie_is_simmelian()`,
  `tie_is_transitive()`, `tie_is_triangular()`, `tie_is_triplet()`

The `*is_max()` and `*is_min()` functions are used to identify the
maximum or minimum, respectively, node or tie according to some measure
(see below).

## Measures

`{netrics}`’s `*_by_*()` functions offer numeric measures at the
network, node, and tie level. These include:

- `net_by_adhesion()`, `net_by_assortativity()`, `net_by_balance()`,
  `net_by_betweenness()`, `net_by_closeness()`, `net_by_cohesion()`,
  `net_by_components()`, `net_by_congruency()`,
  `net_by_connectedness()`, `net_by_core()`, `net_by_degree()`,
  `net_by_density()`, `net_by_diameter()`, `net_by_diversity()`,
  `net_by_efficiency()`, `net_by_eigenvector()`, `net_by_equivalency()`,
  `net_by_factions()`, `net_by_harmonic()`, `net_by_heterophily()`,
  `net_by_homophily()`, `net_by_immunity()`, `net_by_indegree()`,
  `net_by_independence()`, `net_by_infection_complete()`,
  `net_by_infection_peak()`, `net_by_infection_total()`,
  `net_by_length()`, `net_by_modularity()`, `net_by_outdegree()`,
  `net_by_reach()`, `net_by_reciprocity()`, `net_by_recovery()`,
  `net_by_reproduction()`, `net_by_richclub()`, `net_by_richness()`,
  `net_by_scalefree()`, `net_by_smallworld()`, `net_by_spatial()`,
  `net_by_strength()`, `net_by_toughness()`, `net_by_transitivity()`,
  `net_by_transmissibility()`, `net_by_upperbound()`, `net_by_waves()`,
  `node_by_adopt_threshold()`, `node_by_adopt_time()`,
  `node_by_alpha()`, `node_by_authority()`, `node_by_betweenness()`,
  `node_by_bridges()`, `node_by_brokering_activity()`,
  `node_by_brokering_exclusivity()`, `node_by_closeness()`,
  `node_by_constraint()`, `node_by_coreness()`, `node_by_deg()`,
  `node_by_degree()`, `node_by_distance()`, `node_by_diversity()`,
  `node_by_eccentricity()`, `node_by_efficiency()`, `node_by_effsize()`,
  `node_by_eigenvector()`, `node_by_equivalency()`,
  `node_by_exposure()`, `node_by_flow()`, `node_by_harmonic()`,
  `node_by_heterophily()`, `node_by_hierarchy()`, `node_by_homophily()`,
  `node_by_hub()`, `node_by_indegree()`, `node_by_induced()`,
  `node_by_information()`, `node_by_kcoreness()`, `node_by_leverage()`,
  `node_by_multidegree()`, `node_by_neighbours_degree()`,
  `node_by_outdegree()`, `node_by_pagerank()`, `node_by_posneg()`,
  `node_by_power()`, `node_by_randomwalk()`, `node_by_reach()`,
  `node_by_reciprocity()`, `node_by_recovery()`, `node_by_redundancy()`,
  `node_by_richness()`, `node_by_stress()`, `node_by_subgraph()`,
  `node_by_transitivity()`, `node_by_vitality()`,
  `tie_by_betweenness()`, `tie_by_closeness()`, `tie_by_cohesion()`,
  `tie_by_degree()`, `tie_by_eigenvector()`

The measures are organised into several broad categories, including:
*Centrality*, *Cohesion*, *Hierarchy*, *Innovation* (structural holes),
*Diversity* (heterogeneity), *Topology* (features), and *Diffusion*.
Each measure recognises whether the network is directed or undirected,
weighted or unweighted, one-mode or two-mode, and returns normalized
values wherever possible. We recommend you explore [the list of
functions](https://stocnet.github.io/netrics/reference/index.html) to
find out more.

## Memberships

`{netrics}`‘s `*_in_*()` functions identify nodes’ membership in some
grouping, such as a community or component. These functions always
return a character vector, indicating e.g. that the first node is a
member of group “A”, the second in group “B”, etc.

- `node_in_adopter()`, `node_in_automorphic()`, `node_in_betweenness()`,
  `node_in_brokering()`, `node_in_community()`, `node_in_component()`,
  `node_in_core()`, `node_in_eigen()`, `node_in_equivalence()`,
  `node_in_fluid()`, `node_in_greedy()`, `node_in_infomap()`,
  `node_in_leiden()`, `node_in_louvain()`, `node_in_optimal()`,
  `node_in_partition()`, `node_in_regular()`, `node_in_roulette()`,
  `node_in_spinglass()`, `node_in_strong()`, `node_in_structural()`,
  `node_in_walktrap()`, `node_in_weak()`

For example `node_in_brokering()` returns the frequency of nodes’
participation in Gould-Fernandez brokerage roles for a one-mode network,
and the Jasny-Lubell brokerage roles for a two-mode network.

These can be analysed alone, or used as a profile for establishing
equivalence. `{netrics}` offers both HCA and CONCOR algorithms, as well
as elbow, silhouette, and strict methods for *k*-cluster selection.

<img src="https://www.jameshollway.com/post/migraph/dendroPlot.png" alt="Plot of a dendrogram of structural equivalence"/>

`{netrics}` also includes functions for establishing membership on other
bases, such as typical community detection algorithms, as well as
component and core-periphery partitioning algorithms.

## Motifs

`{netrics}`‘s `*_x_*()` functions tabulate nodes’ and networks’
frequency in various motifs. These include:

- `net_x_brokerage()`, `net_x_change()`, `net_x_correlation()`,
  `net_x_dyad()`, `net_x_hazard()`, `net_x_hierarchy()`,
  `net_x_mixed()`, `net_x_stability()`, `net_x_tetrad()`,
  `net_x_triad()`, `node_x_brokerage()`, `node_x_dyad()`,
  `node_x_exposure()`, `node_x_path()`, `node_x_tetrad()`,
  `node_x_tie()`, `node_x_triad()`

## Analysis

The functions in `{netrics}` are designed to answer a wide variety of
analytic questions about networks. For example, you might want to know
about:

- *Centrality*: `net_by_betweenness()`, `net_by_closeness()`,
  `net_by_degree()`, `net_by_eigenvector()`, `net_by_indegree()`,
  `net_by_outdegree()`, `node_by_betweenness()`, `node_by_closeness()`,
  `node_by_degree()`, `node_by_eigenvector()`, `node_by_indegree()`,
  `node_by_multidegree()`, `node_by_neighbours_degree()`,
  `node_by_outdegree()`, `node_in_betweenness()`,
  `tie_by_betweenness()`, `tie_by_closeness()`, `tie_by_degree()`,
  `tie_by_eigenvector()`
- *Cohesion*: `net_by_congruency()`, `net_by_density()`,
  `net_by_equivalency()`, `net_by_reciprocity()`,
  `net_by_transitivity()`, `node_by_equivalency()`,
  `node_by_reciprocity()`, `node_by_transitivity()`
- *Hierarchy*: `net_by_connectedness()`, `net_by_efficiency()`,
  `net_by_reciprocity()`, `net_by_upperbound()`, `net_x_hierarchy()`,
  `node_by_efficiency()`, `node_by_hierarchy()`, `node_by_reciprocity()`
- *Topology*: `net_by_balance()`, `net_by_core()`, `net_by_factions()`,
  `net_by_modularity()`, `net_by_richclub()`, `net_by_smallworld()`,
  `node_by_coreness()`, `node_by_kcoreness()`, `node_in_core()`,
  `node_is_core()`, `tie_is_imbalanced()`
- *Resilience*: `net_by_adhesion()`, `net_by_cohesion()`,
  `node_by_bridges()`, `node_is_cutpoint()`, `tie_by_cohesion()`,
  `tie_is_bridge()`
- *Brokerage*: `net_x_brokerage()`, `node_by_brokering_activity()`,
  `node_by_brokering_exclusivity()`, `node_by_constraint()`,
  `node_by_effsize()`, `node_by_redundancy()`, `node_in_brokering()`,
  `node_x_brokerage()`
- *Diversity*: `net_by_assortativity()`, `net_by_diversity()`,
  `net_by_heterophily()`, `net_by_homophily()`, `net_by_richness()`,
  `node_by_diversity()`, `node_by_heterophily()`, `node_by_homophily()`,
  `node_by_richness()`
- *Diffusion*: `net_by_infection_complete()`, `net_by_infection_peak()`,
  `net_by_infection_total()`, `node_by_adopt_threshold()`,
  `node_by_adopt_time()`, `node_by_exposure()`, `node_in_adopter()`,
  `node_is_exposed()`, `node_is_infected()`, `node_x_exposure()`

## Installation

### Stable

The easiest way to install the latest stable version of `{netrics}` is
via CRAN. Simply open the R console and enter:

`install.packages('netrics')`

`library(netrics)` will then load the package and make the functions
contained within the package available.

### Development

For the latest development version, for slightly earlier access to new
features or for testing, you may wish to download and install the
binaries from Github or install from source locally. The latest binary
releases for all major OSes – Windows, Mac, and Linux – can be found
[here](https://github.com/stocnet/netrics/releases/latest). Download the
appropriate binary for your operating system, and install using an
adapted version of the following commands:

- For Windows:
  `install.packages("~/Downloads/netrics_winOS.zip", repos = NULL)`
- For Mac:
  `install.packages("~/Downloads/netrics_macOS.tgz", repos = NULL)`
- For Unix:
  `install.packages("~/Downloads/netrics_linuxOS.tar.gz", repos = NULL)`

To install from source the latest main version of `{netrics}` from
Github, please install the `{remotes}` package from CRAN and then:

- For latest stable version:
  `remotes::install_github("stocnet/netrics")`
- For latest development version:
  `remotes::install_github("stocnet/netrics@develop")`

### Other sources

Those using Mac computers may also install using Macports:

`sudo port install R-netrics`

## Relationship to other packages

`{netrics}` is part of the [stocnet](https://github.com/stocnet)
ecosystem of R packages for network analysis. The packages are designed
to be modular, with clear roles and dependencies:

- [`{manynet}`](https://stocnet.github.io/manynet/): The foundation
  package for working with network data. It handles network classes
  (matrices, `{igraph}`, `{network}`, `{tidygraph}`, `mnet`), coercion
  between them, modification, and network-level logical tests (`is_*()`
  functions).
- **`{netrics}`**: The analytic package containing all measures
  (`*_by_*()` functions), memberships (`*_in_*()` functions), motifs
  (`*_x_*()` functions), and node- and tie-level marks (`node_is_*()`,
  `tie_is_*()` functions). `{netrics}` depends on `{manynet}`.
- [`{autograph}`](https://stocnet.github.io/autograph/): The graph
  drawing package. `{autograph}` depends on both `{manynet}` (for
  network classes) and `{netrics}` (for analytic results to visualise),
  since it would typically be used with both.
- [`{migraph}`](https://stocnet.github.io/migraph/): The modelling and
  testing package, building on both `{manynet}` and `{netrics}`.

Node- and tie-level marks such as `node_is_cutpoint()` and
`tie_is_bridge()` are kept in `{netrics}` rather than `{manynet}`
because they are analytic functions that identify structural positions
in the network. Network-level property tests like `is_directed()` remain
in `{manynet}` because they describe the type of data rather than an
analytic result.

## Funding details

Development on this package has been funded by the Swiss National
Science Foundation (SNSF) [Grant Number
188976](https://data.snf.ch/grants/grant/188976): “Power and Networks
and the Rate of Change in Institutional Complexes” (PANARCHIC).

#' Memberships in maximally diverse cliques
#' @description 
#'   These functions create a vector of nodes' memberships in
#'   cliques:
#'   
#'   - `node_in_roulette()` assigns nodes to maximally diverse groups.
#'   
#' @section Maximally diverse grouping problem: 
#'   This well known computational problem is a NP-hard problem
#'   with a number of relevant applications, 
#'   including the formation of groups of students that have encountered
#'   each other least or least recently.
#'   Essentially, the aim is to return a membership of nodes in cliques
#'   that minimises the sum of their previous (weighted) ties:
#'   
#'   \deqn{\sum_{g=1}^{m} \sum_{i=1}^{n-1} \sum_{j=i+1}^{n} x_{ij} y_{ig} y_{jg}}
#'   
#'   where \eqn{y_{ig} = 1} if node \eqn{i} is in group \eqn{g}, and 0 otherwise.
#'   
#'   \eqn{x_{ij}} is the existing network data.
#'   If this is an empty network, the function will just return cliques.
#'   To run this repeatedly, one can join a clique network of the membership result
#'   with the original network, using this as the network data for the next round.
#' 
#'   A form of the Lai and Hao (2016) iterated maxima search (IMS) is used here.
#'   This performs well for small and moderately sized networks.
#'   It includes both weak and strong perturbations to an initial solution
#'   to ensure that a robust solution from the broader state space is identified.
#'   The user is referred to Lai and Hao (2016) and Lai et al (2021) for more details.
#' @template param_data
#' @template node_member
#' @name member_cliques
NULL

#' @rdname member_cliques
#' @param num_groups An integer indicating the number of groups desired.
#' @param group_size An integer indicating the desired size of most of the groups.
#'   Note that if the number of nodes is not divisible into groups of equal size,
#'   there may be some larger or smaller groups.
#' @param times An integer of the number of search iterations the algorithm should complete.
#'   By default this is the number of nodes in the network multiplied by the number of groups.
#'   This heuristic may be insufficient for small networks and numbers of groups,
#'   and burdensome for large networks and numbers of groups, but can be overwritten.
#'   At every 10th iteration, a stronger perturbation of a number of successive changes,
#'   approximately the number of nodes divided by the number of groups,
#'   will take place irrespective of whether it improves the objective function.
#' @references
#' ## On the maximally diverse grouping problem
#' Lai, Xiangjing, and Jin-Kao Hao. 2016. 
#' “Iterated Maxima Search for the Maximally Diverse Grouping Problem.” 
#' _European Journal of Operational Research_ 254(3):780–800. 
#' \doi{10.1016/j.ejor.2016.05.018}.
#' 
#' Lai, Xiangjing, Jin-Kao Hao, Zhang-Hua Fu, and Dong Yue. 2021. 
#' “Neighborhood Decomposition Based Variable Neighborhood Search and Tabu Search for Maximally Diverse Grouping.” 
#' _European Journal of Operational Research_ 289(3):1067–86. 
#' \doi{10.1016/j.ejor.2020.07.048}.
#' @export
node_in_roulette <- function(.data, num_groups, group_size, times = NULL){
  .data <- manynet::expect_nodes(.data)
  if(missing(num_groups) & missing(group_size)){
    manynet::snet_abort(paste("Either `num_groups` must indicate number of groups desired",
               "or `group_size` must indicate the desired average size of groups."))
  }
  n <- manynet::net_nodes(.data)
  my_vec <- sample(seq.int(n))
  # Initial partition
  if(!missing(num_groups)){
    out <- cut(seq_along(my_vec), num_groups, labels = FALSE)[my_vec]
  } else {
    out <- ceiling(seq_along(my_vec) / group_size)[my_vec]
  }
  if(is.null(times)) times <- n * max(out)
  # Get fitness
  mat <- manynet::as_matrix(.data)
  fit <- sum(.to_cliques(out) * mat)
  soln <- out
  for(t in seq.int(times)){
    soln <- .weakPerturb(soln)
    new_fit <- sum(.to_cliques(soln) * mat)
    if(new_fit < fit){
      out <- soln
      fit <- new_fit
    } 
    if(t %% 10) soln <- .strongPerturb(soln)
  }
  make_node_member(out, .data)
}

.to_cliques <- function(member){
  (member == t(matrix(member, length(member), length(member))))*1
}

.weakPerturb <- function(soln){
  gsizes <- table(soln)
  evens <- all(gsizes == max(gsizes))
  if(evens){
    soln <- .swapMove(soln)
  } else {
    if(stats::runif(1)<0.5) soln <- .swapMove(soln) else 
      soln <- .oneMove(soln)
  }
  soln
}

.swapMove <- function(soln){
  from <- sample(seq.int(length(soln)), 1)
  to <- sample(which(soln != soln[from]), 1)
  soln[c(to,from)] <- soln[c(from,to)]
  soln
}

.oneMove <- function(soln){
  gsizes <- table(soln)
  maxg <- which(gsizes == max(gsizes))
  from <- sample(which(soln %in% maxg), 1)
  soln[from] <- sample(which(gsizes != max(gsizes)), 1)
  soln
}

.strongPerturb <- function(soln, strength = 1){
  times <- ceiling(strength * length(soln)/max(soln))
  for (t in seq.int(times)){
    soln <- .weakPerturb(soln)
  }
  soln
}

test_that("node_in_roulette works", {
  res <- node_in_roulette(ison_adolescents, num_groups = 3)
  expect_s3_class(res, "node_member")
  expect_length(res, net_nodes(ison_adolescents))
  expect_false(res[1] == res[2])
  expect_output(print(res), "3 groups")
  expect_output(print(summary(res)), "Class A:")
})

#' Memberships in components
#' @description 
#'   These functions create a vector of nodes' memberships in components:
#'   
#'   - `node_in_component()` assigns nodes' component membership
#'   using edge direction where available.
#'   - `node_in_weak()` assigns nodes' component membership
#'   ignoring edge direction.
#'   - `node_in_strong()` assigns nodes' component membership
#'   based on edge direction.
#'   
#'   In graph theory, components, sometimes called connected components, 
#'   are induced subgraphs from partitioning the nodes into disjoint sets.
#'   All nodes that are members of the same partition as _i_ are reachable
#'   from _i_.
#'   
#'   For directed networks, 
#'   strongly connected components consist of subgraphs where there are paths
#'   in each direction between member nodes.
#'   Weakly connected components consist of subgraphs where there is a path
#'   in either direction between member nodes.
#'   
#' @template param_data
#' @template node_member
#' @name member_components
NULL

#' @rdname member_components 
#' @importFrom igraph components
#' @examples
#' ison_monks |> to_uniplex("esteem") |>
#'   mutate_nodes(comp = node_in_component())
#' @export
node_in_component <- function(.data){
  .data <- manynet::expect_nodes(.data)
  if(!manynet::is_graph(.data)) .data <- manynet::as_igraph(.data) # nocov
  make_node_member(igraph::components(.data, mode = "strong")$membership,
              .data)
}

#' @rdname member_components 
#' @importFrom igraph components
#' @export
node_in_weak <- function(.data){
  .data <- manynet::expect_nodes(.data)
  if(!manynet::is_graph(.data)) .data <- manynet::as_igraph(.data) # nocov
  make_node_member(igraph::components(.data, mode = "weak")$membership,
                 .data)
}

#' @rdname member_components 
#' @importFrom igraph components
#' @export
node_in_strong <- function(.data){
  .data <- manynet::expect_nodes(.data)
  if(!manynet::is_graph(.data)) .data <- manynet::as_igraph(.data) # nocov
  make_node_member(igraph::components(.data, mode = "strong")$membership,
                 .data)
}

test_that("node_in_component works", {
  comp <- ison_monks %>% to_uniplex("esteem") %>%
    node_in_component()
  expect_s3_class(comp, "node_member")
  expect_equal(length(unique(comp)), 
               c(net_by_components(to_uniplex(ison_monks, "esteem"))))
  expect_equal(length(unique(comp)), 
               length(unique(node_in_strong(to_uniplex(ison_monks, "esteem")))))
  comp <- ison_monks %>% to_uniplex("esteem") %>%
    to_undirected() %>% 
    node_in_component()
  expect_equal(length(unique(comp)), 
               length(unique(node_in_weak(to_uniplex(ison_monks, "esteem")))))
})

test_that("node_in_component works for two-mode networks", {
  expect_output(print(node_in_component(ison_southern_women)), "1 group")
})

# Structural properties ####

#' Marking nodes based on structural properties
#' @name mark_nodes
#' @description 
#'   These functions return logical vectors the length of the 
#'   nodes in a network identifying which hold certain properties or positions in the network.
#'   
#'   - `node_is_independent()` marks nodes that are members of the largest independent set,
#'   aka largest internally stable set.
#'   - `node_is_cutpoint()` marks nodes that cut or act as articulation points in a network,
#'   increasing the number of connected components when removed.
#'   - `node_is_fold()` marks nodes that are in a structural fold between two or more
#'   triangles that are only connected by that node.
#'   - `node_is_mentor()` marks a proportion of high indegree nodes as 'mentors' (see details).
#'   - `node_is_neighbor()` marks nodes that are neighbours of a given node.
#' @template param_data
#' @template node_mark
NULL

#' @rdname mark_nodes
#' @importFrom igraph largest_ivs
#' @references
#' ## On independent sets
#' Tsukiyama, Shuji, Mikio Ide, Hiromu Ariyoshi, and Isao Shirawaka. 1977.
#' "A new algorithm for generating all the maximal independent sets". 
#' _SIAM Journal on Computing_, 6(3):505–517.
#' \doi{10.1137/0206036}
#' @examples
#' node_is_independent(ison_adolescents)
#' @export
node_is_independent <- function(.data){
  .data <- manynet::expect_nodes(.data)
  if(manynet::is_twomode(.data)){
    samp <- igraph::largest_ivs(manynet::to_mode1(.data))
    if(manynet::is_labelled(.data)){
      out <- manynet::node_names(.data) %in% 
        attr(samp[[sample(1:length(samp), 1)]], 
             "names")
      names(out) <- manynet::node_names(.data)
    } else {
      out <- 1:manynet::net_nodes(.data) %in% 
        samp[[sample(1:length(samp), 1)]]
    }
  } else {
    samp <- igraph::largest_ivs(manynet::to_undirected(as_igraph(.data)))
    if(manynet::is_labelled(.data)){
      out <- manynet::node_names(.data) %in% 
        attr(samp[[sample(1:length(samp), 1)]], 
             "names")
      names(out) <- manynet::node_names(.data)
    } else {
      out <- 1:manynet::net_nodes(.data) %in% 
        samp[[sample(1:length(samp), 1)]]
    }
  }
  make_node_mark(out, .data)
}

#' @rdname mark_nodes
#' @importFrom igraph articulation_points
#' @references
#' ## On articulation or cut-points
#' Tarjan, Robert E. and Uzi Vishkin. 1985.
#' "An Efficient Parallel Biconnectivity Algorithm",
#' _SIAM Journal on Computing_ 14(4): 862-874.
#' \doi{10.1137/0214061}
#' @examples 
#' node_is_cutpoint(ison_brandes)
#' @export
node_is_cutpoint <- function(.data){
  .data <- manynet::expect_nodes(.data)
  if(manynet::is_labelled(.data)){
    out <- manynet::node_names(.data) %in% 
      attr(igraph::articulation_points(as_igraph(.data)), 
           "names")
    names(out) <- manynet::node_names(.data)
  } else {
    out <- 1:manynet::net_nodes(.data) %in% 
      igraph::articulation_points(as_igraph(.data))
  }
  make_node_mark(out, .data)
}

#' @rdname mark_nodes
#' @references
#' ## On structural folds
#' Vedres, Balazs, and David Stark. 2010.
#' "Structural folds: Generative disruption in overlapping groups",
#' _American Journal of Sociology_ 115(4): 1150-1190.
#' \doi{10.1086/649497}
#' @examples
#' node_is_fold(create_explicit(A-B, B-C, A-C, C-D, C-E, D-E))
#' @export
node_is_fold <- function(.data){
  .data <- manynet::expect_nodes(.data)
  mult_tri <- igraph::count_triangles(.data)>1
  tris <- igraph::triangles(.data)
  tris <- matrix(tris, length(tris)/3, 3, byrow = TRUE)
  out <- vapply(seq_along(mult_tri), function(x){
    if(!mult_tri[x]) FALSE else {
      tri_neigh <- unique(c(tris[apply(tris, 1, function(r) any(x %in% r)),] ))
      tri_neigh <- tri_neigh[tri_neigh != x]
      all(rowSums(igraph::distances(.data, tri_neigh, tri_neigh)==2)>=2)
    }
  }, FUN.VALUE = logical(1) )
  make_node_mark(out, .data)
}

#' @rdname mark_nodes
#' @param elites The proportion of nodes to be selected as mentors.
#'   By default this is set at 0.1.
#'   This means that the top 10% of nodes in terms of degree,
#'   or those equal to the highest rank degree in the network,
#'   whichever is the higher, will be used to select the mentors.
#'   
#'   Note that if nodes are equidistant from two mentors,
#'   they will choose one at random.
#'   If a node is without a path to a mentor,
#'   for example because they are an isolate,
#'   a tie to themselves (a loop) will be created instead.
#'   Note that this is a different default behaviour than that
#'   described in Valente and Davis (1999).
#' @references
#' ## On mentoring
#' Valente, Thomas, and Rebecca Davis. 1999.
#' "Accelerating the Diffusion of Innovations Using Opinion Leaders",
#' _Annals of the American Academy of Political and Social Science_ 566: 56-67.
#' @export
node_is_mentor <- function(.data, elites = 0.1){
  .data <- manynet::expect_nodes(.data)
  indegs <- colSums(manynet::as_matrix(.data)) # get rank order of indegrees
  out <- indegs == max(indegs)
  if(sum(out) < length(indegs)*elites){
    out <- indegs %in% unique(sort(indegs, decreasing=TRUE)[seq_len(length(indegs)*elites)])
  }
  make_node_mark(out, .data)
}

#' @rdname mark_nodes
#' @param node A node ID or name for which neighbors are to be marked.
#' @export
node_is_neighbor <- function(.data, node){
  .data <- manynet::expect_nodes(.data)
  .data <- manynet::as_igraph(.data)
  out <- igraph::V(.data) %in% igraph::neighbors(.data, v = node)
  make_node_mark(out, .data)
}

# Degree properties ####

#' Marking nodes based on degree properties
#' @name mark_degree
#' @description
#'   These functions return logical vectors the length of the 
#'   nodes in a network identifying which hold certain properties or positions in the network.
#'   
#'   - `node_is_isolate()` marks nodes that are isolates,
#'   with neither incoming nor outgoing ties.
#'   - `node_is_pendant()` marks nodes that are pendants,
#'   with exactly one incoming or outgoing tie.
#'   - `node_is_universal()` identifies whether nodes are adjacent to all other
#'   nodes in the network.
#' @template param_data
#' @family degree
#' @template node_mark
NULL

#' @rdname mark_degree
#' @examples 
#' node_is_isolate(ison_brandes)
#' @export
node_is_isolate <- function(.data){
  .data <- manynet::expect_nodes(.data)
  mat <- abs(manynet::as_matrix(.data))
  if(manynet::is_twomode(.data)){
    out <- c(rowSums(mat)==0, colSums(mat)==0)
  } else {
    out <- rowSums(mat)==0 & colSums(mat)==0
  }
  names(out) <- manynet::node_names(.data)
  make_node_mark(out, .data)
}

#' @rdname mark_degree
#' @export
node_is_pendant <- function(.data){
  .data <- manynet::expect_nodes(.data)
  mat <- abs(manynet::as_matrix(.data))
  if(manynet::is_twomode(.data)){
    out <- c(rowSums(mat)==1, colSums(mat)==1)
  } else {
    out <- rowSums(mat)==1 & colSums(mat)==1
  }
  make_node_mark(out, .data)
}

#' @rdname mark_degree
#' @section Universal/dominating node: 
#'   A universal node is adjacent to all other nodes in the network.
#'   It is also sometimes called the dominating vertex because it represents
#'   a one-element dominating set.
#'   A network with a universal node is called a cone, and its universal node
#'   is called the apex of the cone.
#'   A classic example of a cone is a star graph,
#'   but friendship, wheel, and threshold graphs are also cones.
#' @examples
#' node_is_universal(create_star(11))
#' @export
node_is_universal <- function(.data){
  .data <- manynet::expect_nodes(.data)
  net <- manynet::to_undirected(manynet::to_unweighted(.data))
  make_node_mark(node_by_deg(net)==(manynet::net_nodes(net)-1), .data)
}

# Diffusion properties ####

#' Marking nodes based on diffusion properties
#' 
#' @description 
#'   These functions return logical vectors the length of the 
#'   nodes in a network identifying which hold certain properties or positions in the network.
#'   
#'   - `node_is_infected()` marks nodes that are infected by a particular time point. 
#'   - `node_is_exposed()` marks nodes that are exposed to a given (other) mark.
#'   - `node_is_latent()` marks nodes that are latent at a particular time point.
#'   - `node_is_recovered()` marks nodes that are recovered at a particular time point.
#' @inheritParams mark_nodes
#' @template node_mark
#' @family diffusion
#' @name mark_diff
NULL

#' @rdname mark_diff 
#' @examples
#'   # To mark nodes that are latent by a particular time point
#'   node_is_latent(play_diffusion(create_tree(6), latency = 1), time = 1)
#' @export
node_is_latent <- function(.data, time = 0){
  .data <- manynet::expect_nodes(.data)
  if(manynet::is_changing(.data)){
    t <- time
    latent <- manynet::as_changelist(.data) |> 
      dplyr::filter(time <= t & value %in% c("E", "I")) |>
      dplyr::group_by(node) |>
      dplyr::mutate(n = dplyr::n()) |>
      dplyr::filter(n == 1 & value == "E")
    if (manynet::is_labelled(.data)) {
      out <- seq_len(manynet::net_nodes(.data)) %in% latent$node
      names(out) <- manynet::node_names(.data)
    } else {
      out <- seq_len(manynet::net_nodes(.data)) %in% latent$node
    }
    make_node_mark(out, .data)
  } else if(inherits(.data, "diff_model")){
    latent <- summary(.data) |>
      dplyr::filter(t <= time & event %in% c("E", "I")) |>
      dplyr::group_by(nodes) |>
      dplyr::mutate(n = dplyr::n()) |>
      dplyr::filter(n == 1 & event == "E")
    net <- attr(.data, "network")
    if (manynet::is_labelled(net)) {
      out <- seq_len(manynet::net_nodes(net)) %in% latent$nodes
      names(out) <- manynet::node_names(net)
    } else {
      out <- seq_len(manynet::net_nodes(net)) %in% latent$nodes
    }
    make_node_mark(out, net)
  } else {
    out <- manynet::to_time(.data, time)
    out <- manynet::node_attribute(out, "diffusion") == "E"
    make_node_mark(out, .data)
  }
}

#' @rdname mark_diff 
#' @param time A time step at which nodes are identified.
#' @examples
#'   # To mark nodes that are infected by a particular time point
#'   node_is_infected(play_diffusion(create_tree(6)), time = 1)
#' @export
node_is_infected <- function(.data, time = 0) {
  .data <- manynet::expect_nodes(.data)
  if(manynet::is_changing(.data)){
    t <- time
    infected <- manynet::as_changelist(.data) |> 
      dplyr::filter(time <= t & value %in% c("I", "R")) |>
      dplyr::group_by(node) |>
      dplyr::mutate(n = dplyr::n()) |>
      dplyr::filter(n == 1 & value == "I")
    if (manynet::is_labelled(.data)) {
      out <- seq_len(manynet::net_nodes(.data)) %in% infected$node
      names(out) <- manynet::node_names(.data)
    } else {
      out <- seq_len(manynet::net_nodes(.data)) %in% infected$node
    }
    make_node_mark(out, .data)
  } else if(inherits(.data, "diff_model")){
    infected <- summary(.data) |> 
      dplyr::filter(t <= time & event %in% c("I", "R")) |>
      dplyr::group_by(nodes) |>
      dplyr::mutate(n = dplyr::n()) |>
      dplyr::filter(n == 1 & event == "I")
    net <- attr(.data, "network")
    if (manynet::is_labelled(net)) {
      out <- seq_len(manynet::net_nodes(net)) %in% infected$nodes
      names(out) <- manynet::node_names(net)
    } else {
      out <- seq_len(manynet::net_nodes(net)) %in% infected$nodes
    }
    make_node_mark(out, net)
  } else {
    out <- manynet::to_time(.data, time)
    out <- manynet::node_attribute(out, "diffusion") == "I"
    make_node_mark(out, .data)
  }
}

#' @rdname mark_diff 
#' @examples
#'   # To mark nodes that are recovered by a particular time point
#'   node_is_recovered(play_diffusion(create_tree(6), recovery = 0.5), time = 3)
#' @export
node_is_recovered <- function(.data, time = 0){
  .data <- manynet::expect_nodes(.data)
  if(manynet::is_changing(.data)){
    t <- time
    recovered <- manynet::as_changelist(.data) |> 
      dplyr::filter(time <= t & value %in% c("R")) |>
      dplyr::group_by(node) |>
      dplyr::mutate(n = dplyr::n()) |>
      dplyr::filter(n == 1 & value == "R")
    if (manynet::is_labelled(.data)) {
      out <- seq_len(manynet::net_nodes(.data)) %in% recovered$node
      names(out) <- manynet::node_names(.data)
    } else {
      out <- seq_len(manynet::net_nodes(.data)) %in% recovered$node
    }
    make_node_mark(out, .data)
  } else if(inherits(.data, "diff_model")){
    recovered <- summary(.data) |> 
      dplyr::filter(t <= time & event == "R") |>
      dplyr::group_by(nodes) |>
      dplyr::mutate(n = dplyr::n()) |>
      dplyr::filter(n == 1)
    net <- attr(.data, "network")
    if (manynet::is_labelled(net)) {
      out <- seq_len(manynet::net_nodes(net)) %in% recovered$nodes
      names(out) <- manynet::node_names(net)
    } else {
      out <- seq_len(manynet::net_nodes(net)) %in% recovered$nodes
    }
    make_node_mark(out, net)
  } else {
    out <- manynet::to_time(.data, time)
    out <- manynet::node_attribute(out, "diffusion") == "R"
    make_node_mark(out, .data)
  }
}

#' @rdname mark_diff 
#' @param mark A valid 'node_mark' object or
#'   logical vector (TRUE/FALSE) of length equal to 
#'   the number of nodes in the network.
#' @section Exposed:
#'   `node_is_exposed()` is similar to `node_exposure()`,
#'   but returns a mark (TRUE/FALSE) vector indicating which nodes
#'   are currently exposed to the diffusion content.
#'   This diffusion content can be expressed in the 'mark' argument.
#'   If no 'mark' argument is provided,
#'   and '.data' is a diff_model object,
#'   then the function will return nodes exposure to the seed nodes
#'   in that diffusion.
#' @param mark vector denoting which nodes are infected
#' @importFrom dplyr filter select group_by mutate
#' @examples
#'   # To mark which nodes are currently exposed
#'   (expos <- node_is_exposed(manynet::create_tree(14), mark = c(1,3)))
#'   which(expos)
#' @export
node_is_exposed <- function(.data, mark, time = 0){
  .data <- manynet::expect_nodes(.data)
  if (missing(mark)){
    if(manynet::is_changing(.data)){
      t <- time
      return(make_node_mark(node_by_adopt_exposure(.data, time = t)>0, .data))
    } else if(inherits(.data, "diff_model")){
      mark <- summary(.data) |> 
        dplyr::filter(t == 0 & event == "I") |> 
        dplyr::select(nodes) |> unlist()
      .data <- attr(.data, "network")
    }    
  }
  if(is.logical(mark)) mark <- which(mark)
  out <- rep(F, manynet::net_nodes(.data))
  out[unique(setdiff(unlist(igraph::neighborhood(.data, nodes = mark)),
                     mark))] <- TRUE
  make_node_mark(out, .data)
}

# Selection properties ####

#' Marking nodes based on measures
#' @name mark_select_node
#' @description 
#'   These functions return logical vectors the length of the 
#'   nodes in a network identifying which hold certain properties or positions in the network.
#'   
#'   - `node_is_random()` marks one or more nodes at random.
#'   - `node_is_max()` and `node_is_min()` are more generally useful
#'   for converting the results from some node measure into a mark-class object.
#'   They can be particularly useful for highlighting which node or nodes
#'   are key because they minimise or, more often, maximise some measure.
#' @template param_data
#' @family selection
#' @template node_mark
NULL

#' @rdname mark_select_node
#' @template param_select
#' @examples 
#' node_is_random(ison_brandes, 2)
#' @export
node_is_random <- function(.data, select = 1){
  .data <- manynet::expect_nodes(.data)
  n <- manynet::net_nodes(.data)
  out <- rep(FALSE, n)
  out[sample.int(n, select)] <- TRUE
  make_node_mark(out, .data)
}

#' @rdname mark_select_node
#' @param node_measure An object created by a `node_` measure.
#' @param ranks The number of ranks of max or min to return.
#'   For example, `ranks = 3` will return TRUE for nodes with
#'   scores equal to any of the top (or, for `node_is_min()`, bottom)
#'   three scores.
#'   By default, `ranks = 1`.
#' @examples 
#' node_is_max(node_by_degree(ison_brandes))
#' @export
node_is_max <- function(node_measure, ranks = 1){
  if(!inherits(node_measure, "node_measure"))
    manynet::snet_abort("This function expects an object of class `node_measure`")
  if(any(attr(node_measure, "mode"))){
    mode1 <- as.numeric(node_measure)[!as.logical(attr(node_measure, "mode"))]
    max1 <- mode1[order(mode1, decreasing = TRUE)[1:ranks]]
    mode2 <- as.numeric(node_measure)[as.logical(attr(node_measure, "mode"))]
    max2 <- mode2[order(mode2, decreasing = TRUE)[1:ranks]]
    out <- ((as.numeric(node_measure) %in% max1 & 
               !as.logical(attr(node_measure, "mode"))) | 
              (as.numeric(node_measure) %in% max2 & 
                 as.logical(attr(node_measure, "mode"))))
    attr(out, "mode") <- attr(node_measure, "mode")
  } else {
    out <- node_measure %in% node_measure[order(node_measure,
                                                decreasing = TRUE)[1:ranks]]
  }
  names(out) <- attr(node_measure, "names")
  class(out) <- c("node_mark", class(out))
  out
}

#' @rdname mark_select_node
#' @examples 
#' node_is_min(node_by_degree(ison_brandes))
#' @export
node_is_min <- function(node_measure, ranks = 1){
  if(!inherits(node_measure, "node_measure"))
    manynet::snet_abort("This function expects an object of class `node_measure`")
  if(any(attr(node_measure, "mode"))){
    mode1 <- as.numeric(node_measure)[!as.logical(attr(node_measure, "mode"))]
    max1 <- mode1[order(mode1, decreasing = FALSE)[1:ranks]]
    mode2 <- as.numeric(node_measure)[as.logical(attr(node_measure, "mode"))]
    max2 <- mode2[order(mode2, decreasing = FALSE)[1:ranks]]
    out <- ((as.numeric(node_measure) %in% max1 & 
               !as.logical(attr(node_measure, "mode"))) | 
              (as.numeric(node_measure) %in% max2 & 
                 as.logical(attr(node_measure, "mode"))))
    attr(out, "mode") <- attr(node_measure, "mode")
  } else {
    out <- node_measure %in% node_measure[order(node_measure,
                                                decreasing = FALSE)[1:ranks]]
  }
  names(out) <- attr(node_measure, "names")
  class(out) <- c("node_mark", class(out))
  out
}

#' @rdname mark_select_node
#' @examples 
#' node_is_mean(node_by_degree(ison_brandes))
#' @export
node_is_mean <- function(node_measure, ranks = 1){
  if(!inherits(node_measure, "node_measure"))
    manynet::snet_abort("This function expects an object of class `node_measure`")
  if(any(attr(node_measure, "mode"))){
    mode1 <- node_measure[!as.logical(attr(node_measure, "mode"))]
    out <- mode1 == sort(abs(mode1 - mean(mode1)))[ranks]
    mode2 <- node_measure[as.logical(attr(node_measure, "mode"))]
    out <- c(out, mode2 == sort(abs(mode2 - mean(mode2)))[ranks])
    attr(out, "mode") <- as.logical(attr(node_measure, "mode"))
  } else {
    out <- node_measure == sort(abs(node_measure - mean(node_measure)))[ranks]
  }
  class(out) <- c("node_mark", class(out))
  out
}

set.seed(1234)

node_marks <- funs_objs[grepl("node_is_", names(funs_objs))]
for(fn in names(node_marks)) {
  for (ob in names(data_objs)) { 
    test_that(paste(fn, "works on", ob), {
      if(fn == "node_is_exposed"){
        expect_s3_class(node_marks[[fn]](data_objs[[ob]], mark = c(1,3)), 
                        "node_mark")
      } else if(fn == "node_is_core"){
        skip_if(fn == "node_is_core" && manynet::is_directed(data_objs[[ob]]))
        expect_s3_class(node_marks[[fn]](data_objs[[ob]]), "node_mark")
      } else if(fn == "node_is_neighbor"){
        expect_s3_class(node_marks[[fn]](data_objs[[ob]], node = 1), 
                        "node_mark")
      } else if(grepl("recovered|latent|infected", fn)){
        if(ob == "diffusion")
        expect_s3_class(node_marks[[fn]](data_objs[[ob]]), 
                        "node_mark") else succeed("Only used for diffusion objects")
      } else if(grepl("min|max|mean", fn)){
          expect_s3_class(node_marks[[fn]](node_by_deg(data_objs[[ob]])), 
                          "node_mark")
      } else {
        expect_s3_class(node_marks[[fn]](data_objs[[ob]]), "node_mark")
      }
    })
  }
}

test_that("node_is_cutpoint", {
  expect_length(node_is_cutpoint(ison_southern_women),
                c(net_nodes(ison_southern_women)))
})

test_that("node_is_isolate", {
  f <- node_is_isolate
  test <- f(ison_brandes)
  test_that("returns correct values", {
    expect_equal(length(test), c(net_nodes(ison_brandes)))
  })
  test_that("returns correct type", {
    expect_s3_class(test, "logical")
  })
})

test_that("node_is_fold works", {
  test <- node_is_fold(create_explicit(A-B, B-C, A-C, C-D, C-E, D-E))
  expect_equal(as.logical(test), c(F,F,T,F,F))
})

test_that("node_is_neighbor works", {
  expect_equal(names(which(node_is_neighbor(ison_adolescents, "Betty"))), "Sue")
})

test_that("node_is_max works", {
  # skip_on_cran()
  # skip_on_ci()
  expect_equal(length(node_is_max(node_by_betweenness(ison_brandes))),
               c(net_nodes(ison_brandes)))
  expect_equal(sum(node_is_max(node_by_betweenness(ison_brandes)) == TRUE), 1)
  expect_s3_class(node_is_max(node_by_betweenness(ison_brandes)), "logical")
})

test_that("node_is_min works", {
  # skip_on_cran()
  # skip_on_ci()
  expect_equal(length(node_is_min(node_by_betweenness(ison_brandes))),
               c(net_nodes(ison_brandes)))
  expect_equal(sum(node_is_min(node_by_betweenness(ison_brandes)) == TRUE), 4)
  expect_s3_class(node_is_min(node_by_betweenness(ison_brandes)), "logical")
})

test_that("node_is_mean works", {
  expect_s3_class(node_is_mean(node_by_betweenness(ison_brandes)), "logical")
})

test_that("additional node mark functions work", {
  expect_mark(node_is_independent(ison_adolescents), c(TRUE,FALSE,F))
  expect_mark(node_is_core(ison_adolescents), c(F,T,T))
  expect_mark(node_is_mentor(ison_adolescents), c(F,T,T))
  expect_mark(node_is_latent(play_diffusion(create_tree(6), latency = 1), time = 1),
               c(F,T,T))
  expect_mark(node_is_infected(play_diffusion(create_tree(6)), time = 1),
               c(T,T,T,F,F,F), top = 6)
  set.seed(123)
  expect_mark(node_is_recovered(play_diffusion(create_tree(12), recovery = 0.3), time = 3),
               c(T,T,T,F,F,F), top = 6)
  expect_mark(node_is_exposed(manynet::create_tree(6), mark = c(1,3)),
               c(F,T,F))
  set.seed(1234)
  expect_mark(node_is_random(ison_adolescents, 2), c(F,T,F))
})

test_that("node_is_pendant correctly identifies pendant nodes", {
  # Apply the function
  result <- node_is_pendant(create_star(5))
  
  # The center of the star (node 1) is not pendant, others are
  expect_type(result, "logical")
  expect_length(result, 5)
  expect_equal(as.logical(result), c(FALSE, TRUE, TRUE, TRUE, TRUE))
  expect_output(print(result), "V1")
})

test_that("node infection, exposure, and recovery works", {
  skip_on_cran()
  skip_on_ci()
  set.seed(1234)
  .data <- play_diffusion(create_tree(15),
                          seeds = 10, latency = 1, steps = 10)
  expect_true(which(node_is_infected(.data, time = 0))==10)
  expect_true(which(node_is_latent(.data, time = 2))==5)
})

# Nodal holes ####

#' Measuring nodes brokerage
#' @name measure_broker_node
#' @description
#'   These function provide different measures of the degree to which nodes
#'   fill structural holes, as outlined in Burt (1992):
#'   
#'   - `node_by_bridges()` measures the sum of bridges to which each node
#'   is adjacent.
#'   - `node_by_redundancy()` measures the redundancy of each nodes' contacts.
#'   - `node_by_effsize()` measures nodes' effective size.
#'   - `node_by_efficiency()` measures nodes' efficiency.
#'   - `node_by_constraint()` measures nodes' constraint scores for one-mode networks
#'   according to Burt (1992) and for two-mode networks according to Hollway et al (2020). 
#'   - `node_by_hierarchy()` measures nodes' exposure to hierarchy,
#'   where only one or two contacts are the source of closure.
#'   - `node_by_neighbours_degree()` measures nodes' average nearest neighbors degree,
#'   or \eqn{knn}, a measure of the type of local environment a node finds itself in
#'   
#'   Burt's theory holds that while those nodes embedded in dense clusters
#'   of close connections are likely exposed to the same or similar ideas and information,
#'   those who fill structural holes between two otherwise disconnected groups
#'   can gain some comparative advantage from that position.
#' @details
#'   A number of different ways of measuring these structural holes are available.
#'   Note that we use Borgatti's reformulation for unweighted networks in
#'   `node_redundancy()` and `node_effsize()`.
#'   Redundancy is thus \eqn{\frac{2t}{n}}, 
#'   where \eqn{t} is the sum of ties and \eqn{n} the sum of nodes in each node's neighbourhood,
#'   and effective size is calculated as \eqn{n - \frac{2t}{n}}.
#'   Node efficiency is the node's effective size divided by its degree.
#' @references 
#' ## On structural holes
#' Burt, Ronald S. 1992. 
#' _Structural Holes: The Social Structure of Competition_. 
#' Cambridge, MA: Harvard University Press.
#' @template param_data
#' @family brokerage
#' @template node_measure
NULL

#' @rdname measure_broker_node 
#' @examples 
#' node_by_bridges(ison_adolescents)
#' node_by_bridges(ison_southern_women)
#' @export
node_by_bridges <- function(.data){
  .data <- manynet::expect_nodes(.data)
  g <- manynet::as_igraph(.data)
  .inc <- NULL
  out <- vapply(igraph::V(g), function(ego){
    length(igraph::E(g)[.inc(ego) & tie_is_bridge(g)==1])
  }, FUN.VALUE = numeric(1))
  make_node_measure(out, .data)
}

#' @rdname measure_broker_node 
#' @references 
#' Borgatti, Steven. 1997. 
#' “\href{http://www.analytictech.com/connections/v20(1)/holes.htm}{Structural Holes: Unpacking Burt’s Redundancy Measures}” 
#' _Connections_ 20(1):35-38.
#' 
#' Burchard, Jake, and Benjamin Cornwell. 2018. 
#' “Structural Holes and Bridging in Two-Mode Networks.” 
#' _Social Networks_ 55:11–20. 
#' \doi{10.1016/j.socnet.2018.04.001}
#' @examples 
#' node_by_redundancy(ison_adolescents)
#' node_by_redundancy(ison_southern_women)
#' @export
node_by_redundancy <- function(.data){
  .data <- manynet::expect_nodes(.data)
  if(manynet::is_twomode(.data)){
    mat <- manynet::as_matrix(.data)
    out <- c(.redund2(mat), .redund2(t(mat)))
  } else {
    out <- .redund(manynet::as_matrix(.data))
  }
  make_node_measure(out, .data)
}

.redund <- function(.mat){
  n <- nrow(.mat)
  qs <- .twopath_matrix(.mat > 0)
  piq <- .mat/rowSums(.mat)
  mjq <- .mat/matrix(do.call("pmax",data.frame(.mat)),n,n)
  out <- rowSums(qs * piq * mjq)
  out
}

.redund2 <- function(.mat){
  sigi <- .mat %*% t(.mat)
  diag(sigi) <- 0
  vapply(seq.int(nrow(sigi)), 
         function(x){
           xvec <- sigi[x,] #> 0
           if(manynet::is_weighted(.mat)){
             wt <- colMeans((.mat[x,] > 0 * t(.mat[xvec > 0,])) * t(.mat[xvec > 0,]) + .mat[x,]) * 2  
           } else wt <- 1
           sum(colSums(xvec > 0 & t(sigi[xvec > 0,])) * xvec[xvec > 0] / 
                 (sum(xvec) * wt))
         }, FUN.VALUE = numeric(1))
}

#' @rdname measure_broker_node 
#' @examples 
#' node_by_effsize(ison_adolescents)
#' node_by_effsize(ison_southern_women)
#' @export
node_by_effsize <- function(.data){
  .data <- manynet::expect_nodes(.data)
  if(manynet::is_twomode(.data)){
    mat <- manynet::as_matrix(.data)
    out <- c(rowSums(manynet::as_matrix(manynet::to_mode1(.data))>0), 
             rowSums(manynet::as_matrix(manynet::to_mode2(.data))>0)) - node_by_redundancy(.data)
  } else {
    mat <- manynet::as_matrix(.data)
    out <- rowSums(mat>0) - .redund(mat)
  }
  make_node_measure(out, .data)
}

.twopath_matrix <- function(.data){
  .data <- manynet::as_matrix(.data)
  qs <- .data %*% t(.data)
  diag(qs) <- 0
  qs
}


#' @rdname measure_broker_node 
#' @examples 
#' node_by_efficiency(ison_adolescents)
#' node_by_efficiency(ison_southern_women)
#' @export
node_by_efficiency <- function(.data){
  .data <- manynet::expect_nodes(.data)
  out <- node_by_effsize(.data) / node_by_degree(.data, normalized = FALSE)
  make_node_measure(as.numeric(out), .data)
}

#' @rdname measure_broker_node 
#' @references
#' Hollway, James, Jean-Frédéric Morin, and Joost Pauwelyn. 2020.
#' "Structural conditions for novelty: The introduction of new environmental clauses to the trade regime complex."
#' _International Environmental Agreements: Politics, Law and Economics_ 20 (1): 61–83.
#' \doi{10.1007/s10784-019-09464-5}
#' @examples
#' node_by_constraint(ison_southern_women)
#' @export 
node_by_constraint <- function(.data) {
  .data <- manynet::expect_nodes(.data)
  if (manynet::is_twomode(.data)) {
    get_constraint_scores <- function(mat) {
      inst <- colnames(mat)
      rowp <- mat * matrix(1 / rowSums(mat), nrow(mat), ncol(mat))
      colp <- mat * matrix(1 / colSums(mat), nrow(mat), ncol(mat), byrow = T)
      res <- vector()
      for (i in inst) {
        ci <- 0
        membs <- names(which(mat[, i] > 0))
        for (a in membs) {
          pia <- colp[a, i]
          oth <- membs[membs != a]
          pbj <- 0
          if (length(oth) == 1) {
            for (j in inst[mat[oth, ] > 0 & inst != i]) {
              pbj <- sum(pbj, sum(colp[oth, i] * rowp[oth, j] * colp[a, j]))
            }
          } else {
            for (j in inst[colSums(mat[oth, ]) > 0 & inst != i]) {
              pbj <- sum(pbj, sum(colp[oth, i] * rowp[oth, j] * colp[a, j]))
            }
          }
          cia <- (pia + pbj)^2
          ci <- sum(ci, cia)
        }
        res <- c(res, ci)
      }
      names(res) <- inst
      res
    }
    inst.res <- get_constraint_scores(manynet::as_matrix(.data))
    actr.res <- get_constraint_scores(t(manynet::as_matrix(.data)))
    res <- c(actr.res, inst.res)
  } else {
    res <- igraph::constraint(manynet::as_igraph(.data), 
                              nodes = igraph::V(.data), 
                              weights = NULL)
  }
  make_node_measure(res, .data)
}

#' @rdname measure_broker_node 
#' @examples 
#' node_by_hierarchy(ison_adolescents)
#' node_by_hierarchy(ison_southern_women)
#' @export
node_by_hierarchy <- function(.data){
  .data <- manynet::expect_nodes(.data)
  cs <- node_by_constraint(.data)
  g <- manynet::as_igraph(.data)
  out <- vapply(igraph::V(g), function(ego){
    n = igraph::neighbors(g, ego)
    N <- length(n)
    css <- cs[n]
    CN <- mean(css)
    rj <- css/CN
    sum(rj*log(rj)) / (N * log(N))
  }, FUN.VALUE = numeric(1))
  out[is.nan(out)] <- 0
  make_node_measure(out, .data)
}

#' @rdname measure_broker_node 
#' @importFrom igraph knn
#' @references
#' ## On neighbours average degree
#' Barrat, Alain, Marc Barthelemy, Romualdo Pastor-Satorras, and Alessandro Vespignani. 2004.
#' "The architecture of complex weighted networks",
#' _Proc. Natl. Acad. Sci._ 101: 3747.
#' @export
node_by_neighbours_degree <- function(.data){
  .data <- manynet::expect_nodes(.data)
  out <- igraph::knn(manynet::as_igraph(.data),
                              mode = "out")$knn
  make_node_measure(out, .data)
}

# Tie holes ####

#' Measuring ties brokerage
#' @name measure_broker_tie
#' @description
#'   `tie_by_cohesion()` measures the ratio between common neighbors to ties'
#'   adjacent nodes and the total number of adjacent nodes,
#'   where high values indicate ties' embeddedness in dense local environments.
#'   
#' @template param_data
#' @family brokerage
#' @template tie_measure
NULL

#' @rdname measure_broker_tie 
#' @export
tie_by_cohesion <- function(.data){
  .data <- manynet::expect_ties(.data)
  ties <- igraph::E(.data)
  coins <- data.frame(heads = igraph::head_of(.data, ties),
                      tails = igraph::tail_of(.data, ties))
  out <- apply(coins, 1, 
        function(x){
          neigh1 <- igraph::neighbors(.data, x[1])
          neigh2 <- igraph::neighbors(.data, x[2])
          shared_nodes <- sum(c(neigh1 %in% neigh2, 
                                neigh2 %in% neigh1))/2
          neigh_nodes <- length(unique(c(neigh1, neigh2)))-2
          shared_nodes / neigh_nodes
        } )
  make_tie_measure(out, .data)
}

test_that("redundancy is reported correctly", {
  expect_equal(as.numeric(length(node_by_redundancy(ison_brandes))),
               as.numeric(net_nodes(ison_brandes)))
  expect_equal(as.numeric(length(node_by_redundancy(ison_southern_women))),
               as.numeric(net_nodes(ison_southern_women)))
  expect_named(node_by_redundancy(ison_southern_women))
})

test_that("effective size is calculated and reported correctly", {
  expect_equal(as.numeric(length(node_by_effsize(ison_brandes))),
               as.numeric(net_nodes(ison_brandes)))
  expect_equal(length(node_by_effsize(ison_southern_women)),
               c(net_nodes(ison_southern_women)))
  expect_named(node_by_effsize(ison_southern_women))
  expect_equal(unname(node_by_effsize(ison_southern_women)[1:3]), c(2.5,1.38,2.46), tolerance = 0.01)
})

test_that("efficiency is reported correctly", {
  expect_equal(length(node_by_efficiency(ison_brandes)), c(net_nodes(ison_brandes)))
  expect_equal(length(node_by_efficiency(ison_southern_women)),
               c(net_nodes(ison_southern_women)))
})

test_that("constraint scores are reported correctly for two-mode networks",{
  res <- node_by_constraint(ison_southern_women)
  expect_equal(top3(res), c(0.2782, 0.3071, 0.2965))
  expect_output(print(res), "Evelyn")
  # expect_named(node_constraint(ison_southern_women)[1:3], c("Evelyn", "Laura", "Theresa"))
})

test_that("constraint scores are reported correctly for one-mode notworks",{
  res <- node_by_constraint(ison_adolescents)
  expect_equal(round(unname(res[1:3]),2), c(1, .43, .57))
  expect_output(print(res), "Alice")
})

test_that("hierarchy is reported correctly", {
  expect_s3_class(node_by_hierarchy(ison_brandes), "node_measure")
  expect_s3_class(node_by_hierarchy(ison_southern_women), "node_measure")
  expect_equal(length(node_by_hierarchy(ison_brandes)), c(net_nodes(ison_brandes)))
  expect_equal(length(node_by_hierarchy(ison_southern_women)),
               c(net_nodes(ison_southern_women)))
  expect_named(node_by_hierarchy(ison_southern_women))
})

test_that("node_neighbours_degree works", {
  expect_equal(top3(node_by_neighbours_degree(ison_adolescents)), c(4,2.75,3))
})

test_that("tie_cohesion works", {
  expect_equal(top3(tie_by_cohesion(ison_adolescents)), c(0,0.5,0.3333))
})

# nocov start
.onAttach <- function(...) {

  # suppressMessages(suppressPackageStartupMessages(library("manynet", warn.conflicts = FALSE)))
  if (!interactive()) return()
  
  # options(snet_verbosity = getOption("snet_verbosity", "verbose"))
  options(manynet_theme = getOption("manynet_theme", "default"))
  
  # pkgs <- as.data.frame(utils::available.packages(utils::contrib.url(getOption("repos"))))
  # 
  # cran_version <- pkgs[pkgs$Package == "manynet","Version"]

  local_version <- utils::packageVersion("netrics")
  manynet::snet_info("You are using {.pkg netrics} version {.version {local_version}}.")
  old.list <- as.data.frame(utils::old.packages())
  behind_cran <- "netrics" %in% old.list$Package
  
  greet_startup_cli <- function() {
    tips <- c(
      "i" = "Contribute to {.pkg netrics} at {.url https://github.com/stocnet/netrics/}.",
      "i" = "Share bugs, issues, or feature requests at {.url https://github.com/stocnet/netrics/issues}.",
      # "i" = "To suppress package startup messages, use: `suppressPackageStartupMessages(library({.pkg netrics}))`.",
      # "i" = "Changing the theme of all your graphs is straightforward with `set_manynet_theme()`",
      # "i" = "If too many messages appear in the console, run `options(snet_verbosity = 'quiet')`",
      "i" = "Explore changes since the last version with {.code news(package = 'netrics')}.",
      "i" = "Visit the website to learn more: {.url https://stocnet.github.io/netrics/}.",
      "i" = "Discover new analytic opportunities: {.url https://stocnet.github.io/netrics/reference/index.html}."
      # "i" = "Discover {.emph stocnet} R packages at {.url https://github.com/stocnet/}."
      # "i" = "Star me at {.url https://github.com/users/follow?target=jhollway}.",
      # "i" = "You can list all the tutorials available in {.pkg netrics} using {.fn run_tute}, and run them too!"
    )
    manynet::snet_info(sample(tips, 1))
  }

  if (interactive()) {
    if (behind_cran) {
      msg <- "A new version of netrics is available with bug fixes and new features."
      packageStartupMessage(msg, "\nWould you like to install it?")
      if (utils::menu(c("Yes", "No")) == 1) {
        utils::update.packages("netrics")
      }
    } else {
      greet_startup_cli()
      # packageStartupMessage(paste(strwrap(tip), collapse = "\n"))
    }
  }

}

# nocov end

# Cohesion ####

#' Measures of network cohesion
#' @name measure_cohesion
#' @description
#'   These functions return values or vectors relating to how cohesive a network is:
#'   
#'   - `net_by_density()` measures the ratio of ties to the number
#'   of possible ties.
#'   - `net_by_components()` measures the number of (strong) components 
#'   in the network.
#'   - `net_by_independence()` measures the independence number, 
#'   or size of the largest independent set in the network.
#'   
#' @template param_data
#' @family cohesion
#' @template net_measure
NULL

#' @rdname measure_cohesion
#' @importFrom igraph edge_density
#' @examples 
#' net_by_density(ison_adolescents)
#' net_by_density(ison_southern_women)
#' @export
net_by_density <- function(.data) {
  .data <- manynet::expect_nodes(.data)
  if (manynet::is_twomode(.data)) {
    mat <- manynet::as_matrix(.data)
    out <- sum(mat) / (nrow(mat) * ncol(mat))
  } else {
    out <- igraph::edge_density(manynet::as_igraph(.data))
  }
  make_network_measure(out, .data, call = deparse(sys.call()))
}

#' @rdname measure_cohesion
#' @section Components: 
#'   To get the 'weak' components of a directed graph, 
#'   please use `manynet::to_undirected()` first.
#' @importFrom igraph components
#' @examples
#'   net_by_components(fict_thrones)
#'   net_by_components(to_undirected(fict_thrones))
#' @export
net_by_components <- function(.data){
  .data <- manynet::expect_nodes(.data)
  object <- manynet::as_igraph(.data)
  make_network_measure(igraph::components(object, mode = "strong")$no,
                       object, call = deparse(sys.call()))
}

#' @rdname measure_cohesion 
#' @importFrom igraph ivs_size
#' @examples 
#' net_by_independence(ison_adolescents)
#' @export
net_by_independence <- function(.data){
  .data <- manynet::expect_nodes(.data)
  if(manynet::is_twomode(.data)){
    out <- igraph::ivs_size(manynet::to_mode1(manynet::as_igraph(.data)))
  } else {
    out <- igraph::ivs_size(manynet::to_undirected(manynet::as_igraph(.data)))
  }
  make_network_measure(out, .data, call = deparse(sys.call()))
}

# Breadth ####

#' Measures of network breadth
#' @name measure_breadth
#' @description
#'   These functions return values or vectors relating to how broad a network is.
#'   
#'   - `net_by_diameter()` measures the maximum path length in the network.
#'   - `net_by_length()` measures the average path length in the network.
#'   
#' @template param_data
#' @family cohesion
#' @template net_measure
NULL

#' @rdname measure_breadth 
#' @importFrom igraph diameter
#' @examples 
#' net_by_diameter(fict_marvel)
#' net_by_diameter(to_giant(fict_marvel))
#' @export
net_by_diameter <- function(.data){
  .data <- manynet::expect_nodes(.data)
  object <- manynet::as_igraph(.data)
  make_network_measure(igraph::diameter(object, 
                                        directed = manynet::is_directed(object)),
                       object, call = deparse(sys.call()))
}

#' @rdname measure_breadth 
#' @importFrom igraph mean_distance
#' @examples 
#' net_by_length(fict_marvel)
#' net_by_length(to_giant(fict_marvel))
#' @export
net_by_length <- function(.data){
  .data <- manynet::expect_nodes(.data)
  object <- manynet::as_igraph(.data)
  make_network_measure(igraph::mean_distance(object,
                                             directed = manynet::is_directed(object)),
                       object, call = deparse(sys.call()))
}

# Fragmentation ####

#' Measures of network fragmentation
#' @name measure_fragmentation
#' @description
#'   These functions return values relating to how connected a network is
#'   and the number of nodes or edges to remove that would increase fragmentation.
#'   
#'   - `net_by_cohesion()` measures the minimum number of nodes to remove
#'   from the network needed to increase the number of components.
#'   - `net_by_toughness()` measures the number of nodes that would need to be
#'   removed from a network to increase its number of components.
#'   - `net_by_adhesion()` measures the minimum number of ties to remove
#'   from the network needed to increase the number of components.
#'   - `net_by_strength()` measures the number of ties that would need to be
#'   removed from a network to increase its number of components.
#'   
#' @template param_data
#' @family cohesion
#' @template net_measure
NULL

#' @rdname measure_fragmentation 
#' @importFrom igraph cohesion
#' @references
#' ## On cohesion
#' White, Douglas R and Frank Harary. 2001. 
#' "The Cohesiveness of Blocks In Social Networks: Node Connectivity and Conditional Density." 
#' _Sociological Methodology_ 31(1): 305-59.
#' \doi{10.1111/0081-1750.00098}
#' @examples 
#' net_by_cohesion(fict_marvel)
#' net_by_cohesion(to_giant(fict_marvel))
#' @export
net_by_cohesion <- function(.data){
  .data <- manynet::expect_nodes(.data)
  make_network_measure(igraph::cohesion(manynet::as_igraph(.data)), 
                       .data, call = deparse(sys.call()))
}

#' @rdname measure_fragmentation 
#' @importFrom igraph adhesion
#' @examples 
#' net_by_adhesion(fict_marvel)
#' net_by_adhesion(to_giant(fict_marvel))
#' @export
net_by_adhesion <- function(.data){
  .data <- manynet::expect_nodes(.data)
  make_network_measure(igraph::adhesion(manynet::as_igraph(.data)), 
                       .data, call = deparse(sys.call()))
}

#' @rdname measure_fragmentation 
#' @examples 
#' net_by_strength(ison_adolescents)
#' @export
net_by_strength <- function(.data){
  .data <- manynet::expect_nodes(.data)
  n <- manynet::net_ties(.data)
  seties <- unlist(lapply(1:n, utils::combn, x = 1:n, simplify = FALSE), recursive = FALSE)
  out <- vapply(seties, function(x) length(x)/net_by_components(manynet::delete_ties(.data, x)), 
                FUN.VALUE = numeric(1))
  make_network_measure(min(out), .data, call = deparse(sys.call()))
}

#' @rdname measure_fragmentation 
#' @examples 
#' net_by_toughness(ison_adolescents)
#' @export
net_by_toughness <- function(.data){
  .data <- manynet::expect_nodes(.data)
  n <- manynet::net_nodes(.data)
  seties <- unlist(lapply(1:n, utils::combn, x = 1:n, simplify = FALSE), recursive = FALSE)
  out <- vapply(seties, function(x) length(x)/net_by_components(manynet::delete_nodes(.data, x)), 
                FUN.VALUE = numeric(1))
  make_network_measure(min(out), .data, call = deparse(sys.call()))
}

test_that("network components works", {
  expect_equal(as.numeric(net_by_components(ison_adolescents)), 1)
})

test_that("network cohesion works", {
  expect_equal(as.numeric(net_by_cohesion(ison_southern_women)), 2)
})

test_that("network adhesion works", {
  expect_equal(as.numeric(net_by_adhesion(ison_southern_women)), 2)
})

test_that("network diameter works", {
  expect_equal(as.numeric(net_by_diameter(ison_southern_women)), 4)
})

test_that("network length works", {
  expect_equal(as.numeric(net_by_length(ison_southern_women)), 2.306, 
               tolerance = 0.001)
})

test_that("net_independence works", {
  expect_values(net_by_independence(ison_adolescents), 4)
})

test_that("net_strength works", {
  expect_values(net_by_strength(ison_adolescents), 0.5)
})

test_that("net_toughness works", {
  expect_values(net_by_toughness(ison_adolescents), 0.5)
})

# Network diversity ####

#' Measures of network diversity
#' @name measure_diverse_net
#' @description
#'   These functions offer ways to measure the heterogeneity of an attribute
#'   across a network, within groups of a network, or the distribution of ties
#'   across this attribute:
#'   
#'   - `net_by_richness()` measures the number of unique categories 
#'   in a network attribute.
#'   - `net_by_diversity()` measures the heterogeneity of ties across a network.
#'   
#' @template param_data
#' @template param_attr
#' @family diversity
#' @template net_measure
#' @param diversity Which method to use for `*_diversity()`.
#'   Either "blau" (Blau's index) or "teachman" (Teachman's index) for categorical attributes, 
#'   or "variation" (coefficient of variation) or "gini" (Gini coefficient) for numeric attributes.
#'   Default is "blau". 
#'   If an incompatible method is chosen for the attribute type,
#'   a suitable alternative will be used instead with a message.
NULL

#' @rdname measure_diverse_net
#' @section Richness:
#'   Richness is a simple count of the number of different categories
#'   present for a given attribute.
#' @references
#' ## On richness
#'   Magurran, Anne E. 1988.
#'   _Ecological Diversity and Its Measurement_.
#'   Princeton: Princeton University Press.
#'   \doi{10.1007/978-94-015-7358-0}
#' @examples
#' net_by_richness(ison_networkers)
#' @export
net_by_richness <- function(.data, attribute){
  .data <- manynet::expect_nodes(.data)
  make_network_measure(length(unique(manynet::node_attribute(.data, attribute))),
                       .data, call = deparse(sys.call()))
}

#' @rdname measure_diverse_net 
#' @section Diversity:
#'    Blau's index (1977) uses a formula known also in other disciplines
#'    by other names 
#'    (Gini-Simpson Index, Gini impurity, Gini's diversity index, 
#'    Gibbs-Martin index, and probability of interspecific encounter (PIE)): 
#'    \deqn{1 - \sum\limits_{i = 1}^k {p_i^2 }} 
#'    where \eqn{p_i} is the proportion of group members in \eqn{i}th category 
#'    and \eqn{k} is the number of categories for an attribute of interest. 
#'    This index can be interpreted as the probability that two members 
#'    randomly selected from a group would be from different categories. 
#'    This index finds its minimum value (0) when there is no variety, 
#'    i.e. when all individuals are classified in the same category. 
#'    The maximum value depends on the number of categories and 
#'    whether nodes can be evenly distributed across categories.
#'    
#'    Teachman's index (1980) is based on information theory
#'    and is calculated as:
#'    \deqn{- \sum\limits_{i = 1}^k {p_i \log(p_i)}}
#'    where \eqn{p_i} is the proportion of group members in \eqn{i}th category 
#'    and \eqn{k} is the number of categories for an attribute of interest.
#'    This index finds its minimum value (0) when there is no variety, 
#'    i.e. when all individuals are classified in the same category. 
#'    The maximum value depends on the number of categories and 
#'    whether nodes can be evenly distributed across categories.
#'    It thus shares similar properties to Blau's index,
#'    but includes also a notion of richness that tends to give more weight to 
#'    rare categories and thus tends to highlight imbalances more.
#'
#'    The coefficient of variation (CV) is a standardised measure of dispersion
#'    of a probability distribution or frequency distribution.
#'    It is defined as the ratio of the standard deviation \eqn{\sigma}
#'    to the mean \eqn{\mu}:
#'    \deqn{CV = \frac{\sigma}{\mu}}
#'    It is often expressed as a percentage.
#'    The CV is useful because the standard deviation of data must always be understood
#'    in the context of the mean of the data.
#'    The CV is particularly useful when comparing the degree of variation
#'    from one data series to another,
#'    even if the means are drastically different from each other.
#'
#'    The Gini coefficient is a measure of statistical dispersion
#'    that is intended to represent the income or wealth distribution
#'    of a nation's residents,
#'    and is commonly used as a measure of inequality.
#'    It is defined as a ratio with values between 0 and 1,
#'    where 0 corresponds with perfect equality
#'    (everyone has the same income) and 1 corresponds with perfect inequality
#'    (one person has all the income, and everyone else has zero income).
#'    The Gini coefficient can be calculated from the Lorenz curve,
#'    which plots the proportion of the total income of the population
#'    that is cumulatively earned by the bottom x% of the population.
#'    The Gini coefficient is defined as the area between the line of equality
#'    and the Lorenz curve,
#'    divided by the total area under the line of equality.
#' @references 
#' ## On diversity
#'   Blau, Peter M. 1977. 
#'   _Inequality and heterogeneity_. 
#'   New York: Free Press.
#'   
#'   Teachman, Jay D. 1980. 
#'   Analysis of population diversity: Measures of qualitative variation. 
#'   _Sociological Methods & Research_, 8:341-362.
#'   \doi{10.1177/004912418000800305}
#'   
#'   Page, Scott E. 2010. 
#'   _Diversity and Complexity_. 
#'   Princeton: Princeton University Press. 
#'   \doi{10.1515/9781400835140}
#' @examples
#' marvel_friends <- to_unsigned(to_uniplex(fict_marvel, "relationship"), "positive")
#' net_by_diversity(marvel_friends, "Gender")
#' net_by_diversity(marvel_friends, "Appearances")
#' @export
net_by_diversity <- function(.data, attribute, 
                        diversity = c("blau","teachman","variation","gini")){
  .data <- manynet::expect_nodes(.data)
  blau <- function(features) { 1 - sum((table(features)/length(features))^2) }
  teachman <- function(features) {
    p <- table(features)/length(features)
    -sum(p * log(p))
  }
  cv <- function(values) { 
    stats::sd(values, na.rm = TRUE) / mean(values, na.rm = TRUE) }
  gini <- function(values) {
    x <- sort(values)
    n <- length(x)
    G <- sum(x * (1:n))
    return((2 * G) / (n * sum(x)) - (n + 1) / n)
  }
  attr <- manynet::node_attribute(.data, attribute)
  diversity <- match.arg(diversity)
  if(is.numeric(attr) && diversity %in% c("blau","teachman")){
    manynet::snet_info("{.val {diversity}} index is not appropriate for numeric attributes.")
    manynet::snet_info("Using {.val variation} coefficient instead",
              "({.val gini} coefficient also available).")
    diversity <- "variation"
  }
  if(is.character(attr) && diversity %in% c("variation","gini")){
    manynet::snet_info("{.val {diversity}} coefficient is not appropriate for categorical attributes.")
    manynet::snet_info("Using {.val blau} index instead",
              "({.val teachman} index also available).")
    diversity <- "blau"
  }
  
  out <- switch(diversity,
                blau = blau(attr),
                teachman = teachman(attr),
                variation = cv(attr),
                gini = gini(attr))
  make_network_measure(out, .data, call = deparse(sys.call()))
}

# Nodal diversity ####

#' Measures of nodes diversity
#' @name measure_diverse_node
#' @description
#'   These functions offer ways to measure the heterogeneity of an attribute
#'   across a network, within groups of a network, or the distribution of ties
#'   across this attribute:
#'   
#'   - `node_by_richness()` measures the number of unique categories 
#'   of an attribute to which each node is connected.
#'   - `node_by_diversity()` measures the heterogeneity of each node's
#'   local neighbourhood.
#'   
#' @template param_data
#' @template param_attr
#' @family diversity
#' @template node_measure
#' @inheritParams measure_diverse_net
NULL

#' @rdname measure_diverse_node 
#' @examples
#' node_by_richness(ison_networkers, "Discipline")
#' @export
node_by_richness <- function(.data, attribute){
  .data <- manynet::expect_nodes(.data)
  out <- vapply(manynet::to_egos(.data, min_dist = 1), 
                function(x) length(unique(manynet::node_attribute(x, attribute))),
                FUN.VALUE = numeric(1))
  make_node_measure(out, .data)
}

#' @rdname measure_diverse_node 
#' @examples 
#' marvel_friends <- to_unsigned(to_uniplex(fict_marvel, "relationship"), "positive")
#' node_by_diversity(marvel_friends, "Gender")
#' node_by_diversity(marvel_friends, "Attractive")
#' @export
node_by_diversity <- function(.data, attribute, 
                              diversity = c("blau","teachman","variation","gini")){
  .data <- manynet::expect_nodes(.data)
  attr <- manynet::node_attribute(.data, attribute)
  diversity <- match.arg(diversity)
  if(is.numeric(attr) && diversity %in% c("blau","teachman")){
    manynet::snet_info("{.val {method}} index is not appropriate for numeric attributes.")
    manynet::snet_info("Using {.val variation} coefficient instead",
                       "({.val gini} coefficient also available).")
    diversity <- "variation"
  }
  if(is.character(attr) && diversity %in% c("variation","gini")){
    manynet::snet_info("{.val {diversity}} coefficient is not appropriate for categorical attributes.")
    manynet::snet_info("Using {.val blau} index instead",
                       "({.val teachman} index also available).")
    diversity <- "blau"
  }
  out <- vapply(igraph::ego(manynet::as_igraph(.data)),
                function(x) net_by_diversity(
                  igraph::induced_subgraph(manynet::as_igraph(.data), x),
                  attribute, diversity = diversity),
                FUN.VALUE = numeric(1))
  make_node_measure(out, .data)
}

# Network assortativity ####

#' Measures of network assortativity
#' @name measure_assort_net
#' @description
#'   These functions offer ways to measure the distribution or assortativity 
#'   of ties in a network:
#'   
#'   - `net_by_heterophily()` measures how embedded nodes in the network
#'   are within groups of nodes with the same attribute.
#'   - `net_by_homophily()` measures how embedded nodes in the network
#'   are within groups of nodes with the same attribute, but with more options for method.
#'   - `net_by_assortativity()` measures the degree assortativity in a network.
#'   - `net_by_spatial()` measures the spatial association/autocorrelation 
#'   (global Moran's I) in a network.
#'   
#'   Note that for two-mode networks, 
#'   homophily is calculated on the mode with the attribute of interest,
#'   and the other mode is ignored.
#'   If the attribute is present on both modes, 
#'   homophily is calculated on the first mode by default,
#'   but a message is given and the user can choose to calculate homophily on the
#'   other mode instead by subsetting the attribute vector and 
#'   converting the network to a one-mode network.
#' @template param_data
#' @template param_attr
#' @param assortativity Which method to use for `*_homophily()`.
#'   Either "ie" (negative E-I index), "ei" (E-I index), 
#'   "yule" (Yule's Q), or "geary" (Geary's C).
#'   Default is "ie".
#'   The E-I index is the number of ties between (or _external_) nodes
#'   grouped in some mutually exclusive categories
#'   minus the number of ties within (or _internal_) these groups
#'   divided by the total number of ties.
#'   This value can range from 1 to -1,
#'   where 1 indicates ties only between categories/groups and -1 ties only within categories/groups.
#'   
#'   Yule's Q is a measure of association for two binary variables, calculated as:
#'   \deqn{Q = \frac{ad - bc}{ad + bc}}
#'   where \eqn{a} is the number of ties between nodes in the same category,
#'   \eqn{b} is the number of ties between nodes in different categories,
#'   \eqn{c} is the number of non-ties between nodes in the same category,
#'   and \eqn{d} is the number of non-ties between nodes in different categories.
#'   This value can range from -1 to 1, where 1 indicates perfect association
#'   (all ties are between nodes in the same category), -1 indicates perfect disassociation
#'   (all ties are between nodes in different categories), and 0 indicates no association.
#'   
#'   Geary's C is a measure of spatial autocorrelation, calculated as:
#'   \deqn{C = \frac{(n - 1) \sum
#'   \limits_{i=1}^n \sum\limits_{j=1}^n w_{ij} (x_i - x_j)^2}{2W \sum\limits_{i=1}^n (x_i - \bar{x})^2}}
#'   where \eqn{n} is the number of nodes, \eqn{
#'   w_{ij}} is the weight of the tie between nodes \eqn{i} and \eqn{j},
#'   \eqn{x_i} is the attribute value of node \eqn{i},
#'   \eqn{\bar{x}} is the mean attribute value across all nodes, and \eqn{W} is the sum of all tie weights.
#'   This value can range from 0 to 2, where values less than 1
#'   indicate positive autocorrelation (similar values are more likely to be connected),
#'   values greater than 1 indicate negative autocorrelation (dissimilar values are more likely
#'   to be connected), and a value of 1 indicates no autocorrelation.
#'   If an incompatible method is chosen for the attribute type,
#'   a suitable alternative will be used instead with a message.
#' @family diversity
#' @template net_measure
NULL

#' @rdname measure_assort_net 
#' @section Heterophily:
#'   Given a partition of a network into a number of mutually exclusive groups then 
#'   The E-I index is the number of ties between (or _external_) nodes 
#'   grouped in some mutually exclusive categories
#'   minus the number of ties within (or _internal_) these groups
#'   divided by the total number of ties. 
#'   This value can range from 1 to -1,
#'   where 1 indicates ties only between categories/groups and -1 ties only within categories/groups.
#' @references 
#' ## On heterophily
#'   Krackhardt, David, and Robert N. Stern. 1988. 
#'   Informal networks and organizational crises: an experimental simulation. 
#'   _Social Psychology Quarterly_ 51(2): 123-140.
#'   \doi{10.2307/2786835}
#'   
#'   McPherson, Miller, Lynn Smith-Lovin, and James M. Cook. 2001.
#'   "Birds of a Feather: Homophily in Social Networks".
#'   _Annual Review of Sociology_, 27(1): 415-444.
#'   \doi{10.1146/annurev.soc.27.1.415}
#' @examples 
#' marvel_friends <- to_unsigned(to_uniplex(fict_marvel, "relationship"), "positive")
#' net_by_heterophily(marvel_friends, "Gender")
#' net_by_heterophily(marvel_friends, "Attractive")
#' @export
net_by_heterophily <- function(.data, attribute){
  .data <- manynet::expect_nodes(.data)
  m <- manynet::as_matrix(.data)
  if (length(attribute) == 1 && is.character(attribute)) {
    attribute <- manynet::node_attribute(.data, attribute)
  }
  if (is.character(attribute) | is.numeric(attribute)) {
    attribute <- as.factor(attribute)
  }
  same <- outer(attribute, attribute, "==")
  nInternal <- sum(m * same, na.rm = TRUE)
  nExternal <- sum(m, na.rm = TRUE) - nInternal
  ei <- (nExternal - nInternal) / sum(m, na.rm = TRUE)
  make_network_measure(ei, .data, call = deparse(sys.call()))
}

#' @rdname measure_assort_net
#' @examples 
#' net_by_homophily(marvel_friends, "Gender")
#' @export
net_by_homophily <- function(.data, attribute,
                             assortativity = c("ie","ei","yule","geary")){
  .data <- manynet::expect_nodes(.data)
  # mode <- attr_mode(.data, attribute)
  # if(is_twomode(.data) && !is.null(mode)){
  #   if(mode){
  #     snet_info("Attribute only present on second mode of two-mode network.")
  #     snet_info("Calculating homophily on first mode instead.")
  #     attribute <- manynet::node_attribute(.data, attribute)[
  #       !manynet::node_is_mode(.data)]
  #   } else {
  #     snet_info("Attribute only present on first mode of two-mode network.")
  #     snet_info("Calculating homophily on second mode instead.")
  #     attribute <- manynet::node_attribute(.data, attribute)[
  #       manynet::node_is_mode(.data)]
  #   }
  #   .data <- manynet::to_mode(.data, mode = mode)
  # }
  if (length(attribute) == 1 && is.character(attribute)) {
    attribute <- manynet::node_attribute(.data, attribute)
  }
  assortativity <- match.arg(assortativity)
  if(is.numeric(attribute) && assortativity %in% c("ie","ei","yule")){
    manynet::snet_info("{.val {assortativity}} index is not appropriate for numeric attributes.")
    manynet::snet_info("Using {.val geary}'s C instead.")
    assortativity <- "geary"
  }
  if(!is.numeric(attribute) && assortativity == "geary"){
    manynet::snet_info("{.val {assortativity}} index is not appropriate for categorical attributes.")
    manynet::snet_info("Using {.val ie} index instead.")
    assortativity <- "ie"
  }
  
  m <- manynet::as_matrix(manynet::to_unweighted(.data))
  
  ei <- function(m, attribute){
    same <- outer(attribute, attribute, "==")
    nInternal <- sum(m * same, na.rm = TRUE)
    nExternal <- sum(m, na.rm = TRUE) - nInternal
    (nExternal - nInternal) / sum(m, na.rm = TRUE)
  }
  
  yule <- function(m, attribute){
    same <- outer(attribute, attribute, "==")
    a <- sum(m * same, na.rm = TRUE)
    b <- sum(m * (!same), na.rm = TRUE)
    c <- sum((1 - m) * same, na.rm = TRUE)
    d <- sum((1 - m) * (!same), na.rm = TRUE)
    (a*d - b*c)/(a*d + b*c)
  }
  
  geary <- function(m, attribute){
    # identify valid nodes
    valid <- !is.na(attribute)
    attr_valid <- attribute[valid]
    m_valid <- m[valid, valid, drop = FALSE]
    
    # recompute n and mean on valid nodes
    n <- length(attr_valid)
    xbar <- mean(attr_valid, na.rm = TRUE)
    
    # pairwise squared differences
    diffsq <- (outer(attr_valid, attr_valid, "-"))^2
    
    # weight sum
    W <- sum(m_valid, na.rm = TRUE)
    
    # numerator and denominator
    num <- (n - 1) * sum(m_valid * diffsq, na.rm = TRUE)
    den <- 2 * W * sum((attr_valid - xbar)^2, na.rm = TRUE)
    
    if (den == 0) return(NA_real_)
    num / den
  }
  
  res <- switch(match.arg(assortativity),
                ie = -ei(m, attribute),
                ei = ei(m, attribute),
                yule = yule(m, attribute),
                geary = geary(m, attribute))
  
  make_network_measure(res, .data, call = deparse(sys.call()))
}


attr_mode <- function(.data, attribute){
  if(manynet::is_twomode(.data)){
    miss <- is.na(manynet::node_attribute(.data, attribute))
    mode <- manynet::node_is_mode(.data)
    if(all(miss[mode])) return(FALSE) # attribute only on first (FALSE) mode
    if(all(miss[!mode])) return(TRUE) # attribute only on second (TRUE) mode
  } else NULL
}

#' @rdname measure_assort_net 
#' @importFrom igraph assortativity_degree
#' @references
#' ## On assortativity
#' Newman, Mark E.J. 2002.
#' "Assortative mixing in networks".
#' _Physical Review Letters_, 89(20): 208701.
#' \doi{10.1103/physrevlett.89.208701} 
#' @examples 
#' net_by_assortativity(ison_networkers)
#' @export
net_by_assortativity <- function(.data){
  .data <- manynet::expect_nodes(.data)
  make_network_measure(igraph::assortativity_degree(manynet::as_igraph(.data), 
                                                    directed = manynet::is_directed(.data)),
                       .data, call = deparse(sys.call()))
}

#' @rdname measure_assort_net
#' @references
#' ## On spatial autocorrelation
#'   Moran, Patrick Alfred Pierce. 1950.
#'   "Notes on continuous stochastic phenomena".
#'   _Biometrika_ 37(1): 17-23.
#'   \doi{10.2307/2332142}
#' @examples 
#' net_by_spatial(ison_lawfirm, "age")
#' @export
net_by_spatial <- function(.data, attribute){
  .data <- manynet::expect_nodes(.data)
  N <- manynet::net_nodes(.data)
  x <- manynet::node_attribute(.data, attribute)
  stopifnot(is.numeric(x))
  x_bar <- mean(x, na.rm = TRUE)
  w <- manynet::as_matrix(.data)
  W <- sum(w, na.rm = TRUE)
  I <- (N/W) * 
    (sum(w * matrix(x - x_bar, N, N) * matrix(x - x_bar, N, N, byrow = TRUE)) / 
       sum((x - x_bar)^2))
  make_network_measure(I, .data, 
                       call = deparse(sys.call()))
}

# Network assortativity ####

#' Measures of nodes assortativity
#' @name measure_assort_node
#' @description
#'   These functions offer ways to measure nodes' assortativity in a network:
#'   
#'   - `node_by_heterophily()` measures each node's embeddedness within groups
#'   of nodes with the same attribute.
#'   - `node_by_homophily()` measures each node's embeddedness within groups
#'   of nodes with the same attribute, using a variety of methods.
#'   
#' @template param_data
#' @template param_attr
#' @family diversity
#' @template node_measure
#' @inheritParams measure_assort_net
NULL

#' @rdname measure_assort_node 
#' @examples 
#' marvel_friends <- to_unsigned(to_uniplex(fict_marvel, "relationship"), "positive")
#' node_by_heterophily(marvel_friends, "Gender")
#' node_by_heterophily(marvel_friends, "Attractive")
#' @export
node_by_heterophily <- function(.data, attribute){
  .data <- manynet::expect_nodes(.data)
  m <- manynet::as_matrix(.data)
  if (length(attribute) == 1 && is.character(attribute)) {
    attribute <- manynet::node_attribute(.data, attribute)
  }
  if (is.character(attribute) | is.numeric(attribute)) {
    attribute <- as.factor(attribute)
  }
  if(anyNA(attribute)){
    m[is.na(attribute),] <- NA
    m[,is.na(attribute)] <- NA
  }
  same <- outer(attribute, attribute, "==")
  nInternal <- rowSums(m * same, na.rm = TRUE)
  nInternal[is.na(attribute)] <- NA
  nExternal <- rowSums(m, na.rm = TRUE) - nInternal
  ei <- (nExternal - nInternal) / rowSums(m, na.rm = TRUE)
  make_node_measure(ei, .data)
}

#' @rdname measure_assort_node
#' @export
node_by_homophily <- function(.data, attribute,
                              assortativity = c("ie","ei","yule","geary")){
  .data <- manynet::expect_nodes(.data)
  # if (length(attribute) == 1 && is.character(attribute)) {
  #   attribute <- manynet::node_attribute(.data, attribute)
  # }
  assortativity <- match.arg(assortativity)
  if(is.numeric(attribute) && assortativity %in% c("ie","ei","yule")){
    manynet::snet_info("{.val {assortativity}} index is not appropriate for numeric attributes.")
    manynet::snet_info("Using {.val geary}'s C instead.")
    assortativity <- "geary"
  }
  if(!is.numeric(attribute) && assortativity == "geary"){
    manynet::snet_info("{.val {assortativity}} index is not appropriate for categorical attributes.")
    manynet::snet_info("Using {.val ie} index instead.")
    assortativity <- "ie"
  }
  idat <- manynet::as_igraph(.data)
  out <- vapply(igraph::ego(idat),
                function(x) {
                  subattr <- if (length(attribute) == 1 && is.character(attribute))
                    attribute else attribute[as.integer(x)]
                  net_by_homophily(
                    igraph::induced_subgraph(idat, x),
                    subattr, assortativity = assortativity)
                },
                FUN.VALUE = numeric(1))
  make_node_measure(out, .data)
}

#*************** Test the heterogeneity family of functions ******************#

test_that("diversity functions works", {
  expect_equal(as.numeric(net_by_diversity(to_uniplex(fict_marvel,"relationship"), "Gender")), 
               0.306, tolerance = 0.001)
  expect_equal(top3(node_by_diversity(ison_lawfirm, "gender")),
               c(0.285, 0.375,0), tolerance = 0.01)
})

test_that("heterophily function works", {
  expect_equal(as.numeric(net_by_heterophily(ison_networkers, "Discipline")), .1704, tolerance = 0.001)
  expect_length(node_by_heterophily(ison_networkers, "Discipline"),
                net_nodes(ison_networkers))
  expect_s3_class(node_by_heterophily(ison_networkers, "Discipline"), "node_measure")
})

test_that("assortativity function works", {
  expect_length(net_by_assortativity(ison_networkers), 1)
})

test_that("richness function works", {
  expect_length(net_by_richness(ison_networkers), 1)
  expect_equal(as.numeric(net_by_richness(ison_networkers)), 3)
  expect_length(node_by_richness(ison_networkers, "type"), 32)
})

test_that("net_spatial works", {
  expect_values(net_by_spatial(ison_lawfirm, "age"), 0.126)
})

test_that("node_by_homophily works with vector attribute (e.g. community assignment)", {
  attr_vec <- node_in_community(data_objs[["attribute"]])
  result <- node_by_homophily(data_objs[["attribute"]], attr_vec)
  expect_s3_class(result, "node_measure")
  expect_length(result, manynet::net_nodes(data_objs[["attribute"]]))
})

node_meas <- funs_objs[grepl("node_by_", names(funs_objs))]
for(fn in names(node_meas)) {
  for (ob in names(data_objs)) { 
    test_that(paste(fn, "works on", ob), {
      skip_if(grepl("multideg", fn))
      skip_if(grepl("equivalency", fn) && ob == "labelled")
      if(grepl("diversity|richness|heterophily|homophily", fn)){
        if(ob == "attribute")
          expect_s3_class(node_meas[[fn]](data_objs[[ob]], "group"), "node_measure") else
            succeed("Only used for attribute objects")
      } else if(grepl("adopt|recovery|exposure",fn)){
        if(ob == "diffusion")
          expect_s3_class(node_meas[[fn]](data_objs[[ob]]), "node_measure") else
            succeed("Only used for diffusion objects")
      } else if(grepl("posneg",fn)){
        if(ob == "signed")
          expect_s3_class(node_meas[[fn]](data_objs[[ob]]), "node_measure") else
            succeed("Only used for signed objects")
      } else if(grepl("distance",fn)){
          expect_s3_class(node_meas[[fn]](data_objs[[ob]], 1, 2), "node_measure")
      } else {
        expect_s3_class(node_meas[[fn]](data_objs[[ob]]), "node_measure")
      }
    })
  }
}

# Betweenness centrality ####

#' Measuring nodes betweenness-like centrality
#' @name measure_central_between
#' @description
#'   These functions calculate common betweenness-related centrality measures for one- and two-mode networks:
#'   
#'   - `node_by_betweenness()` measures the betweenness centralities of nodes in a network.
#'   - `node_by_induced()` measures the induced betweenness centralities of nodes in a network.
#'   - `node_by_flow()` measures the flow betweenness centralities of nodes in a network,
#'   which uses an electrical current model for information spreading 
#'   in contrast to the shortest paths model used by normal betweenness centrality.
#'   - `node_by_stress()` measures the stress centrality of nodes in a network.
#'   - `tie_by_betweenness()` measures the number of shortest paths going through a tie.
#'   
#'   All measures attempt to use as much information as they are offered,
#'   including whether the networks are directed, weighted, or multimodal.
#'   If this would produce unintended results, 
#'   first transform the salient properties using e.g. [to_undirected()] functions.
#'   All centrality and centralization measures return normalized measures by default,
#'   including for two-mode networks.
#' @template param_data
#' @template param_norm
#' @family betweenness
#' @family centrality
#' @template node_measure
#' @param cutoff The maximum path length to consider when calculating betweenness.
#'   If negative or NULL (the default), there's no limit to the path lengths considered.
NULL

#' @rdname measure_central_between
#' @section Betweenness centrality: 
#'   Betweenness centrality is based on the number of shortest paths between
#'   other nodes that a node lies upon:
#'   \deqn{C_B(i) = \sum_{j,k:j \neq k, j \neq i, k \neq i} \frac{g_{jik}}{g_{jk}}}
#' @references
#' ## On betweenness centrality
#' Freeman, Linton. 1977. 
#' "A set of measures of centrality based on betweenness". 
#' _Sociometry_, 40(1): 35–41. 
#' \doi{10.2307/3033543}
#' @examples
#' node_by_betweenness(ison_southern_women)
#' @export 
node_by_betweenness <- function(.data, normalized = TRUE, 
                                cutoff = NULL){
  
  .data <- manynet::expect_nodes(.data)
  weights <- `if`(manynet::is_weighted(.data), 
                  manynet::tie_weights(.data), NA)
  graph <- manynet::as_igraph(.data)
  
  # Do the calculations
  if (manynet::is_twomode(graph) & normalized){
    betw_scores <- igraph::betweenness(graph = graph, v = igraph::V(graph), 
                                       directed = manynet::is_directed(graph))
    other_set_size <- ifelse(igraph::V(graph)$type, sum(!igraph::V(graph)$type), sum(igraph::V(graph)$type))
    set_size <- ifelse(igraph::V(graph)$type, sum(igraph::V(graph)$type), sum(!igraph::V(graph)$type))
    out <- ifelse(set_size > other_set_size, 
                  betw_scores/(2*(set_size-1)*(other_set_size-1)), 
                  betw_scores/(1/2*other_set_size*(other_set_size-1)+1/2*(set_size-1)*(set_size-2)+(set_size-1)*(other_set_size-1)))
  } else {
    if (is.null(cutoff)) {
      out <- igraph::betweenness(graph = graph, v = igraph::V(graph), 
                                 directed = manynet::is_directed(graph), weights = weights, 
                                 normalized = normalized)
    } else {
      out <- igraph::betweenness(graph = graph, v = igraph::V(graph), 
                                 directed = manynet::is_directed(graph), 
                                 cutoff = cutoff, 
                                 weights = weights)
    }
  }
  out <- make_node_measure(out, .data)
  out
}

#' @rdname measure_central_between 
#' @section Induced centrality: 
#'   Induced centrality or vitality centrality concerns the change in 
#'   total betweenness centrality between networks with and without a given node:
#'   \deqn{C_I(i) = C_B(G) - C_B(G\ i)}
#' @references
#' ## On induced centrality
#' Everett, Martin and Steve Borgatti. 2010.
#' "Induced, endogenous and exogenous centrality"
#' _Social Networks_, 32: 339-344.
#' \doi{10.1016/j.socnet.2010.06.004}
#' @examples
#' node_by_induced(ison_adolescents)
#' @export 
node_by_induced <- function(.data, normalized = TRUE, 
                            cutoff = NULL){
  .data <- manynet::expect_nodes(.data)
  endog <- sum(node_by_betweenness(.data, normalized = normalized, cutoff = cutoff),
               na.rm = TRUE)
  exog <- vapply(seq.int(manynet::net_nodes(.data)),
                 function(x) sum(node_by_betweenness(manynet::delete_nodes(.data, x),
                                                     normalized = normalized, cutoff = cutoff),
                                 na.rm = TRUE),
                 FUN.VALUE = numeric(1))
  out <- endog - exog
  make_node_measure(out, .data)
}

#' @rdname measure_central_between 
#' @section Flow betweenness centrality: 
#'   Flow betweenness centrality concerns the total maximum flow, \eqn{f},
#'   between other nodes \eqn{j,k} in a network \eqn{G} that a given node mediates:
#'   \deqn{C_F(i) = \sum_{j,k:j\neq k, j\neq i, k\neq i} f(j,k,G) - f(j,k,G\ i)}
#'   When normalized (by default) this sum of differences is divided by the
#'   sum of flows \eqn{f(i,j,G)}.
#' @references
#' ## On flow centrality
#' Freeman, Lin, Stephen Borgatti, and Douglas White. 1991. 
#' "Centrality in Valued Graphs: A Measure of Betweenness Based on Network Flow". 
#' _Social Networks_, 13(2), 141-154.
#' 
#' Koschutzki, D., K.A. Lehmann, L. Peeters, S. Richter, D. Tenfelde-Podehl, and O. Zlotowski. 2005. 
#' "Centrality Indices". 
#' In U. Brandes and T. Erlebach (eds.), _Network Analysis: Methodological Foundations_. 
#' Berlin: Springer.
#' @export 
node_by_flow <- function(.data, normalized = TRUE){
  .data <- manynet::expect_nodes(.data)
  thisRequires("sna")
  out <- sna::flowbet(manynet::as_network(.data),
                      gmode = ifelse(manynet::is_directed(.data), "digraph", "graph"),
                      diag = manynet::is_complex(.data),
                      cmode = ifelse(normalized, "normflow", "rawflow"))
  make_node_measure(out, .data)
}

#' @rdname measure_central_between 
#' @section Stress centrality: 
#'   Stress centrality is the number of all shortest paths or geodesics, \eqn{g}, 
#'   between other nodes that a given node mediates:
#'   \deqn{C_S(i) = \sum_{j,k:j \neq k, j \neq i, k \neq i} g_{jik}}
#'   High stress nodes lie on a large number of shortest paths between other
#'   nodes, and thus associated with bridging or spanning boundaries.
#' @references
#' ## On stress centrality
#'   Shimbel, A. 1953.
#'   "Structural Parameters of Communication Networks".
#'   _Bulletin of Mathematical Biophysics_, 15:501-507.
#'   \doi{10.1007/BF02476438}
#' @export 
node_by_stress <- function(.data, normalized = TRUE){
  .data <- manynet::expect_nodes(.data)
  thisRequires("sna")
  out <- sna::stresscent(manynet::as_network(.data),
                         gmode = ifelse(manynet::is_directed(.data), "digraph", "graph"),
                         diag = manynet::is_complex(.data),
                         rescale = normalized)
  make_node_measure(out, .data)
}

# Tie betweenness centrality ####

#' Measuring ties betweenness-like centrality
#' @name measure_centralities_between
#' @description
#'   `tie_by_betweenness()` measures the number of shortest paths going through a tie.
#'   
#'   All measures attempt to use as much information as they are offered,
#'   including whether the networks are directed, weighted, or multimodal.
#'   If this would produce unintended results, 
#'   first transform the salient properties using e.g. [to_undirected()] functions.
#'   All centrality and centralization measures return normalized measures by default,
#'   including for two-mode networks.
#' @template param_data
#' @template param_norm
#' @family betweenness
#' @family centrality
#' @template tie_measure
NULL

#' @rdname measure_centralities_between
#' @importFrom igraph edge_betweenness
#' @examples
#' (tb <- tie_by_betweenness(ison_adolescents))
#' ison_adolescents |> mutate_ties(weight = tb)
#' @export
tie_by_betweenness <- function(.data, normalized = TRUE){
  .data <- manynet::expect_ties(.data)
  .data <- manynet::as_igraph(.data)
  eddies <- manynet::as_edgelist(.data)
  eddies <- paste(eddies[["from"]], eddies[["to"]], sep = "-")
  out <- igraph::edge_betweenness(.data)
  names(out) <- eddies
  make_tie_measure(out, .data)
}

# Betweenness centralisation ####

#' Measuring networks betweenness-like centralisation
#' @name measure_centralisation_between
#' @description
#'   - `net_by_betweenness()` measures the betweenness centralization for a
#'   network as a single score.
#'   - `mode_by_betweenness()` measures betweenness centralization separately for
#'   each mode of a two-mode network, returning one score per mode
#'   (following Borgatti and Everett, 1997).
#'
#'   All measures attempt to use as much information as they are offered,
#'   including whether the networks are directed, weighted, or multimodal.
#'   If this would produce unintended results,
#'   first transform the salient properties using e.g. [to_undirected()] functions.
#'   All centrality and centralization measures return normalized measures by default,
#'   including for two-mode networks.
#'
#'   For two-mode networks the two modes have different theoretical maxima, so
#'   `net_by_betweenness()` reports a single network-level score by applying
#'   Freeman's general centralization index over the normalized node betweenness
#'   scores, whereas `mode_by_betweenness()` reports the per-mode scores directly.
#' @template param_data
#' @template param_norm
#' @template param_dir
#' @family betweenness
#' @family centrality
#' @references
#'   Borgatti, Stephen P., and Martin G. Everett. 1997.
#'   "Network analysis of 2-mode data."
#'   _Social Networks_ 19(3): 243-269.
#'   \doi{10.1016/S0378-8733(96)00301-2}
#' @returns
#'   `net_by_betweenness()` returns a `network_measure` scalar;
#'   `mode_by_betweenness()` returns a `mode_measure` numeric vector of length two,
#'   giving one centralization score per mode.
NULL

#' @rdname measure_centralisation_between
#' @examples
#' net_by_betweenness(ison_southern_women, direction = "in")
#' @export
net_by_betweenness <- function(.data, normalized = TRUE,
                               direction = c("all", "out", "in")) {
  .data <- manynet::expect_nodes(.data)
  direction <- match.arg(direction)
  graph <- manynet::as_igraph(.data)

  if (manynet::is_twomode(.data)) {
    # Two-mode betweenness centralization is intrinsically per mode
    # (see `mode_by_betweenness()`, following Borgatti and Everett, 1997).
    # For a single network-level score we apply Freeman's general
    # centralization index over the whole node set, using the normalized node
    # betweenness scores (each in [0, 1]); the numerator's maximum is (n - 1).
    nc <- node_by_betweenness(graph, normalized = TRUE)
    out <- sum(max(nc) - nc) / (length(nc) - 1)
  } else {
    out <- igraph::centr_betw(graph = graph,
                              normalized = normalized)$centralization
  }
  out <- make_network_measure(out, .data, call = deparse(sys.call()))
  out
}

#' @rdname measure_centralisation_between
#' @examples
#' mode_by_betweenness(ison_southern_women, direction = "in")
#' @export
mode_by_betweenness <- function(.data, normalized = TRUE,
                                direction = c("all", "out", "in")) {
  .data <- manynet::expect_nodes(.data)
  direction <- match.arg(direction)
  graph <- manynet::as_igraph(.data)

  if (!manynet::is_twomode(.data))
    manynet::snet_abort("`mode_by_betweenness()` is only defined for two-mode networks; use `net_by_betweenness()` for one-mode networks.")

  {
    becent <- node_by_betweenness(graph, normalized = FALSE)
    mode <- igraph::V(graph)$type
    mode1 <- length(mode) - sum(mode)
    mode2 <- sum(mode)
    out <- list()
    if (direction == "all") {
      if (!normalized) {
        out$nodes1 <- sum(max(becent[!mode]) - becent) / ((1/2 * mode2 * (mode2 - 1) + 1/2 * (mode1 - 1)*(mode1 - 2) + (mode1 - 1) * (mode2 - 2))*(mode1 + mode2 - 1) + (mode1 - 1))
        out$nodes2 <- sum(max(becent[mode]) - becent) / ((1/2 * mode1 * (mode1 - 1) + 1/2 * (mode2 - 1)*(mode2 - 2) + (mode2 - 1) * (mode1 - 2))*(mode2 + mode1 - 1) + (mode2 - 1))
        if (mode1 > mode2) {
          out$nodes1 <- sum(max(becent[!mode]) - becent) / (2 * (mode1 - 1) * (mode2 - 1) * (mode1 + mode2 - 1) - (mode2 - 1) * (mode1 + mode2 - 2) - 1/2 * (mode1 - mode2) * (mode1 + 3*mode2 - 3))
        }
        if (mode2 > mode1) {
          out$nodes2 <- sum(max(becent[mode]) - becent) / (2 * (mode2 - 1) * (mode1 - 1) * (mode2 + mode1 - 1) - (mode1 - 1) * (mode2 + mode1 - 2) - 1/2 * (mode2 - mode1) * (mode2 + 3*mode1 - 3))
        }
      } else if (normalized) {
        out$nodes1 <- sum(max(becent[!mode]) - becent) / ((1/2 * mode2 * (mode2 - 1) + 1/2 * (mode1 - 1)*(mode1 - 2) + (mode1 - 1) * (mode2 - 2))*(mode1 + mode2 - 1) + (mode1 - 1))
        out$nodes2 <- sum(max(becent[mode]) - becent) / ((1/2 * mode1 * (mode1 - 1) + 1/2 * (mode2 - 1)*(mode2 - 2) + (mode2 - 1) * (mode1 - 2))*(mode2 + mode1 - 1) + (mode2 - 1))
        if (mode1 > mode2) {
          becent <- node_by_betweenness(graph, normalized = TRUE)
          out$nodes1 <- sum(max(becent[!mode]) - becent) / ((mode1 + mode2 - 1) - (((mode2 - 1)*(mode1 + mode2 - 2) + 1/2*(mode1 - mode2)*(mode1 + (3*mode2) - 3)) / (1/2*(mode1*(mode1 - 1)) + 1/2*(mode2 - 1) * (mode2 - 2) + (mode1 - 1) * (mode2 - 1))))
        }
        if (mode2 > mode1) {
          becent <- node_by_betweenness(graph, normalized = TRUE)
          out$nodes2 <- sum(max(becent[mode]) - becent) / ((mode1 + mode2 - 1)*((mode1 - 1)*(mode1 + mode2 - 2) / 2*(mode1 - 1)*(mode2 - 1)))
        }
      }
    } else if (direction == "in") {
      out$nodes1 <- sum(max(becent[!mode]) - becent[!mode])/((mode1 - 1)*(1/2*mode2*(mode2 - 1) + 1/2*(mode1 - 1)*(mode1 - 2) + (mode1 - 1)*(mode2 - 1)))
      out$nodes2 <- sum(max(becent[mode]) - becent[mode])/((mode2 - 1)*(1/2*mode1*(mode1 - 1) + 1/2 * (mode2 - 1) * (mode2 - 2) + (mode2 - 1) * (mode1 - 1)))
      if (mode1 > mode2) {
        out$nodes1 <- sum(max(becent[!mode]) - becent[!mode]) / (2 * (mode1 - 1)^2 * (mode2 - 1))
      }
      if (mode2 > mode1) {
        out$nodes2 <- sum(max(becent[mode]) - becent[mode]) / (2 * (mode2 - 1)^2 * (mode1 - 1))
      }
    }
    out <- c("Mode 1" = out$nodes1, "Mode 2" = out$nodes2)
  }
  out <- make_mode_measure(out, .data, call = deparse(sys.call()))
  out
}

test_tbl <- as_tidygraph(ison_southern_women)
test_igr <- ison_southern_women
test_mat <- as_matrix(ison_southern_women)

test_that("one mode degree centrality calculated correctly",{
  expect_equal(top5(node_by_degree(ison_adolescents, normalized = FALSE)), c(1,4,4,2,3))
})

test_that("one mode strength centrality calculated correctly",{
  expect_equal(top5(node_by_degree(to_unweighted(ison_networkers), direction = "in", normalized = FALSE)), 
               c(29, 24, 11, 18, 8))
  expect_equal(top5(node_by_degree(ison_networkers, direction = "in", normalized = FALSE, alpha = 1)), 
               c(2495, 1212, 101, 322, 89))
})

test_that("two mode degree centrality calculated correctly",{
  expect_equal(top5(node_by_degree(test_mat, normalized = FALSE)), c(8,7,8,7,4))
  expect_equal(top5(node_by_degree(test_igr, normalized = FALSE)), c(8,7,8,7,4))
  expect_equal(top5(with_graph(test_tbl, node_by_degree(normalized = FALSE))), c(8,7,8,7,4))
  expect_equal(bot3(node_by_degree(test_mat, normalized = FALSE)), c(6,3,3))
  expect_equal(bot3(node_by_degree(test_igr, normalized = FALSE)), c(6,3,3))
  expect_equal(bot3(with_graph(test_tbl, node_by_degree(normalized = FALSE))), c(6,3,3))
  expect_equal(top5(node_by_degree(test_mat, normalized = TRUE)), c(0.5714, .5, .5714, .5, .2857))
  expect_equal(top5(node_by_degree(test_igr, normalized = TRUE)), c(0.5714, .5, .5714, .5, .2857))
  expect_equal(top5(with_graph(test_tbl, node_by_degree(normalized = TRUE))), c(0.5714, .5, .5714, .5, .2857))
  expect_equal(bot3(node_by_degree(test_mat, normalized = TRUE)), c(.3333, .1667, .1667))
  expect_equal(bot3(node_by_degree(test_igr, normalized = TRUE)), c(.3333, .1667, .1667))
  expect_equal(bot3(with_graph(test_tbl, node_by_degree(normalized = TRUE))), c(.3333, .1667, .1667))
})

test_that("one mode closeness centrality calculated correctly",{
  expect_equal(top3(node_by_closeness(ison_adolescents, normalized = FALSE)), c(0.059, 0.091, 0.091), tolerance = 0.01)
})

test_that("two mode closeness centrality calculated correctly",{
  expect_equal(top5(node_by_closeness(test_mat, normalized = FALSE)), c(0.0167, 0.0152, 0.0167, 0.0152, 0.0125))
  expect_equal(top5(node_by_closeness(test_igr, normalized = FALSE)), c(0.0167, 0.0152, 0.0167, 0.0152, 0.0125))
  expect_equal(top5(with_graph(test_tbl, node_by_closeness(normalized = FALSE))), c(0.0167, 0.0152, 0.0167, 0.0152, 0.0125))
  expect_equal(bot3(node_by_closeness(test_mat, normalized = FALSE)), c(0.0128, 0.0119, 0.0119))
  expect_equal(bot3(node_by_closeness(test_igr, normalized = FALSE)), c(0.0128, 0.0119, 0.0119))
  expect_equal(bot3(with_graph(test_tbl, node_by_closeness(normalized = FALSE))), c(0.0128, 0.0119, 0.0119))
  expect_equal(top5(node_by_closeness(test_mat, normalized = TRUE)), c(0.8000, 0.7273, 0.8000, 0.7273, 0.6000))
  expect_equal(top5(node_by_closeness(test_igr, normalized = TRUE)), c(0.8000, 0.7273, 0.8000, 0.7273, 0.6000))
  expect_equal(top5(with_graph(test_tbl, node_by_closeness(normalized = TRUE))), c(0.8000, 0.7273, 0.8000, 0.7273, 0.6000))
  expect_equal(bot3(node_by_closeness(test_mat, normalized = TRUE)), c(0.5641, 0.5238, 0.5238))
  expect_equal(bot3(node_by_closeness(test_igr, normalized = TRUE)), c(0.5641, 0.5238, 0.5238))
  expect_equal(bot3(with_graph(test_tbl, node_by_closeness(normalized = TRUE))), c(0.5641, 0.5238, 0.5238))
})

test_that("one mode betweenness centrality calculated correctly",{
  expect_equal(top3(node_by_betweenness(ison_adolescents, normalized = FALSE)), c(0, 7.5, 5.5), tolerance = 0.001)
})

test_that("two mode betweenness centrality calculated correctly",{
  expect_equal(top5(node_by_betweenness(test_mat, normalized = FALSE)), c(42.9802, 22.8541, 38.9796, 22.0215, 4.7153))
  expect_equal(top5(node_by_betweenness(test_igr, normalized = FALSE)), c(42.9802, 22.8541, 38.9796, 22.0215, 4.7153))
  expect_equal(top5(with_graph(test_tbl, node_by_betweenness(normalized = FALSE))), c(42.9802, 22.8541, 38.9796, 22.0215, 4.7153))
  expect_equal(bot3(node_by_betweenness(test_mat, normalized = FALSE)), c(8.1786, 1.0128, 1.0128))
  expect_equal(bot3(node_by_betweenness(test_igr, normalized = FALSE)), c(8.1786, 1.0128, 1.0128))
  expect_equal(bot3(with_graph(test_tbl, node_by_betweenness(normalized = FALSE))), c(8.1786, 1.0128, 1.0128))
  expect_equal(top3(node_by_betweenness(test_mat, normalized = TRUE),4), c(0.0972, 0.0517, 0.0882))
  expect_equal(top3(node_by_betweenness(test_igr, normalized = TRUE),4), c(0.0972, 0.0517, 0.0882))
  expect_equal(top3(with_graph(test_tbl, node_by_betweenness(normalized = TRUE)),4), c(0.0972, 0.0517, 0.0882))
  expect_equal(bot3(node_by_betweenness(test_mat, normalized = TRUE),4), c(0.0181, 0.0022, 0.0022))
  expect_equal(bot3(node_by_betweenness(test_igr, normalized = TRUE),4), c(0.0181, 0.0022, 0.0022))
  expect_equal(bot3(with_graph(test_tbl, node_by_betweenness(normalized = TRUE)),4), c(0.0181, 0.0022, 0.0022))
})

test_that("one mode eigenvector centrality calculated correctly",{
  # expect_equal(top3(node_eigenvector(ison_adolescents, normalized = FALSE)), c(0.16, 0.491, 0.529), tolerance = 0.001)
  expect_equal(top3(node_by_eigenvector(ison_adolescents)), c(0.303, 0.928, 1), tolerance = 0.001)
})

test_that("two mode eigenvector centrality calculated correctly",{
  expect_equal(top3(node_by_eigenvector(test_mat)), c(0.9009, 0.8497, 1))
  expect_equal(bot3(node_by_eigenvector(test_mat)), c(0.4764, 0.2907, 0.2907))
  expect_equal(top3(node_by_eigenvector(test_igr)), c(0.9009, 0.8497, 1))
  expect_equal(bot3(node_by_eigenvector(test_igr)), c(0.4764, 0.2907, 0.2907))
})

test_that("summary node measure works", {
  expect_equal(names(summary(node_by_degree(ison_adolescents))), 
               c("Minimum","Maximum","Mean","StdDev","Missing"))
})

test_that("summary net measure works", {
  expect_match(summary(net_by_degree(ison_adolescents)), "z =")
})

# ####### Centralization

test_that("one-mode centralisation is calculated correctly", {
  expect_equal(as.numeric(net_by_degree(ison_adolescents)), 0.2142, tolerance = 0.001)
  expect_equal(as.numeric(net_by_closeness(ison_adolescents)), 0.3195, tolerance = 0.001)
  expect_equal(as.numeric(net_by_betweenness(ison_adolescents)), 0.3401, tolerance = 0.001)
  expect_equal(as.numeric(net_by_eigenvector(ison_adolescents)), 0.5479, tolerance = 0.001)
})

test_that("two mode per-mode degree centralisation calculated correctly", {
  expect_equal(as.numeric(mode_by_degree(ison_southern_women, normalized = FALSE)), c(0.2021, 0.5253), tolerance = 0.001)
  expect_equal(as.numeric(mode_by_degree(ison_southern_women, direction = "in")), c(0.249, 0.484), tolerance = 0.001)
  expect_equal(as.numeric(mode_by_degree(ison_southern_women, normalized = TRUE)), c(0.245, 0.493), tolerance = 0.001)
})

test_that("two mode per-mode closeness centralisation calculated correctly", {
  expect_equal(as.numeric(mode_by_closeness(ison_southern_women, normalized = TRUE)), c(0.293, 0.452), tolerance = 0.001)
  expect_equal(as.numeric(mode_by_closeness(ison_southern_women, direction = "in")), c(0.224, 0.537), tolerance = 0.001)
})

test_that("two mode per-mode betweenness centralisation calculated correctly", {
  expect_equal(as.numeric(mode_by_betweenness(ison_southern_women, normalized = FALSE)), c(0.0733, 0.2113), tolerance = 0.001)
  expect_equal(as.numeric(mode_by_betweenness(ison_southern_women, direction = "in")), c(0.082, 0.202), tolerance = 0.001)
  expect_equal(as.numeric(mode_by_betweenness(ison_southern_women, normalized = TRUE)), c(0.0739, 0.2113), tolerance = 0.001)
})

test_that("net_by_*() returns a single scalar for two-mode networks", {
  expect_length(net_by_degree(ison_southern_women), 1L)
  expect_length(net_by_closeness(ison_southern_women), 1L)
  expect_length(net_by_betweenness(ison_southern_women), 1L)
  expect_length(net_by_eigenvector(ison_southern_women), 1L)
  expect_equal(as.numeric(net_by_degree(ison_southern_women)), 0.4383, tolerance = 0.001)
  expect_equal(as.numeric(net_by_closeness(ison_southern_women)), 0.1930, tolerance = 0.001)
  expect_equal(as.numeric(net_by_betweenness(ison_southern_women)), 0.2050, tolerance = 0.001)
  expect_equal(as.numeric(net_by_eigenvector(ison_southern_women)), 0.4186, tolerance = 0.001)
  # single network-level score is valid input to summary()
  expect_match(summary(net_by_degree(ison_southern_women)), "z =")
  # mode_by_*() errors on one-mode input
  expect_error(mode_by_degree(ison_adolescents), "two-mode")
})

test_that("net_measure class works", {
  expect_output(print(net_by_degree(ison_algebra)))
})

# ####### Edge centrality
test_that("tie_betweenness works", {
  expect_equal(unname(tie_by_betweenness(ison_adolescents)[1:3]),
               c(7,3,5), tolerance = 0.001)
})

test_that("tie_closeness works", {
  expect_equal(unname(tie_by_closeness(ison_adolescents)[1:3]),
               c(0.562,0.692,0.600), tolerance = 0.001)
})

# nocov start
#' Functions that have been renamed, superseded, or are no longer working
#' 
#' `r lifecycle::badge("deprecated")`
#' Generally these functions have been superseded or renamed.
#' Upon using them, a message is provided directing the user to the new function.
#' However, at this stage of package development,
#' we generally clear older defunct functions at each minor release,
#' and so you are strongly encouraged to use the new functions/names/syntax
#' wherever possible and update your scripts accordingly.
#' @name defunct
#' @keywords internal
#' @returns Results as expected 
#'   along with a warning to use new function naming in the future.
NULL

# nocov end

# Node path ####

#' Motifs of nodes pathing
#' @name motif_path
#' @description
#'   These functions include ways to take a census of the positions of nodes
#'   in a network: 
#'   
#'   - `node_x_tie()` returns a census of the ties in a network.
#'   For directed networks, out-ties and in-ties are bound together.
#'   For multiplex networks, the various types of ties are bound together.
#'   - `node_x_path()` returns the shortest path lengths
#'   of each node to every other node in the network.
#'   
#' @template param_data
#' @template node_motif
#' @importFrom igraph vcount make_ego_graph delete_vertices triad_census
NULL

#' @rdname motif_path 
#' @examples
#' task_eg <- to_named(to_uniplex(ison_algebra, "tasks"))
#' (tie_cen <- node_x_tie(task_eg))
#' @export
node_x_tie <- function(.data){
  .data <- manynet::expect_nodes(.data)
  object <- manynet::as_igraph(.data)
  # edge_names <- net_tie_attributes(object)
  if (manynet::is_directed(object)) {
    if (manynet::is_multiplex(.data)) {
      mat <- do.call(rbind, lapply(unique(manynet::tie_attribute(object, "type")), 
                                   function(x){
                                     rc <- manynet::as_matrix(manynet::to_uniplex(object, x))
                                     rbind(rc, t(rc))
                                   }))
    } else if (manynet::is_longitudinal(object)){
      mat <- do.call(rbind, lapply(unique(manynet::tie_attribute(object, "wave")), 
                                   function(x){
                                     rc <- manynet::as_matrix(manynet::to_waves(object)[[x]])
                                     rbind(rc, t(rc))
                                   }))
      
    } else {
      rc <- manynet::as_matrix(object)
      mat <- rbind(rc, t(rc))
    }
  } else {
    if (manynet::is_multiplex(.data)) {
      mat <- do.call(rbind, lapply(unique(manynet::tie_attribute(object, "type")), 
                                   function(x){
                                     manynet::as_matrix(manynet::to_uniplex(object, x))
                                   }))
    } else if (manynet::is_longitudinal(object)){
      mat <- do.call(rbind, lapply(unique(manynet::tie_attribute(object, "wave")), 
                                   function(x){
                                     manynet::as_matrix(manynet::to_waves(object)[[x]])
                                   }))
    } else if (manynet::is_twomode(.data)) {
      mat <- manynet::as_matrix(manynet::to_multilevel(object))
    } else {
      mat <- manynet::as_matrix(object)
    }
  }
  if(manynet::is_labelled(object) & manynet::is_directed(object))
    if(manynet::is_multiplex(.data)){
      rownames(mat) <- apply(expand.grid(c(paste0("from", manynet::node_names(object)),
                                           paste0("to", manynet::node_names(object))),
                                         unique(manynet::tie_attribute(object, "type"))), 
                             1, paste, collapse = "_")
    } else if (manynet::is_longitudinal(object)){
      rownames(mat) <- apply(expand.grid(c(paste0("from", manynet::node_names(object)),
                                           paste0("to", manynet::node_names(object))),
                                         unique(manynet::tie_attribute(object, "wave"))), 
                             1, paste, collapse = "_wave")
    } else {
      rownames(mat) <- rep(c(paste0("from", manynet::node_names(object)),
                             paste0("to", manynet::node_names(object))))
    }
  make_node_motif(t(mat), object)
}

#' @rdname motif_path 
#' @importFrom igraph distances
#' @references 
#' ## On paths
#' Dijkstra, Edsger W. 1959. 
#' "A note on two problems in connexion with graphs". 
#' _Numerische Mathematik_ 1, 269-71.
#' \doi{10.1007/BF01386390}.
#' 
#' Opsahl, Tore, Filip Agneessens, and John Skvoretz. 2010.
#' "Node centrality in weighted networks: Generalizing degree and shortest paths". 
#' _Social Networks_ 32(3): 245-51.
#' \doi{10.1016/j.socnet.2010.03.006}.
#' @examples 
#' node_x_path(ison_adolescents)
#' node_x_path(ison_southern_women)
#' @export
node_x_path <- function(.data){
  .data <- manynet::expect_nodes(.data)
  if(manynet::is_weighted(.data)){
    tore <- manynet::as_matrix(.data)/mean(manynet::as_matrix(.data))
    out <- 1/tore
  } else out <- igraph::distances(manynet::as_igraph(.data))
  diag(out) <- 0
  make_node_motif(out, .data)
}

# Node cohesion ####

#' Motifs of nodes cohesion
#' @name motif_node
#' @description
#'   These functions include ways to take a census of the positions of nodes
#'   in a network: 
#'   
#'   - `node_x_tie()` returns a census of the ties in a network.
#'   For directed networks, out-ties and in-ties are bound together.
#'   For multiplex networks, the various types of ties are bound together.
#'   - `node_x_triad()` returns a census of the triad configurations
#'   nodes are embedded in.
#'   - `node_x_tetrad()` returns a census of nodes' positions
#'   in motifs of four nodes.
#'   - `node_x_path()` returns the shortest path lengths
#'   of each node to every other node in the network.
#'   
#' @template param_data
#' @family cohesion
#' @template node_motif
#' @importFrom igraph vcount make_ego_graph delete_vertices triad_census
NULL

#' @rdname motif_node 
#' @references
#' ## On the dyad census
#' Holland, Paul W., and Samuel Leinhardt. 1970. 
#' "A Method for Detecting Structure in Sociometric Data". 
#' _American Journal of Sociology_, 76: 492-513.
#' \doi{10.1016/B978-0-12-442450-0.50028-6}
#' @examples 
#' node_x_dyad(ison_networkers)
#' @export
node_x_dyad <- function(.data) {
  .data <- manynet::expect_nodes(.data)
  if(is_weighted(.data)){
    .data <- manynet::to_unweighted(.data)
    manynet::snet_info("Ignoring tie weights.")
  }
  mat <- manynet::as_matrix(.data)
  out <- t(vapply(seq_nodes(.data), function(x){
    vec <- mat[x,] + mat[,x]
    c(sum(vec==2), sum(vec==1), sum(vec==0))
  }, FUN.VALUE = numeric(3)))
  colnames(out) <- c("Mutual", "Asymmetric", "Null")
  if (!manynet::is_directed(.data)) out <- out[,c(1, 3)]
  make_node_motif(out, .data)
}

#' @rdname motif_node 
#' @references 
#' ## On the triad census
#' Davis, James A., and Samuel Leinhardt. 1967. 
#' “\href{https://files.eric.ed.gov/fulltext/ED024086.pdf}{The Structure of Positive Interpersonal Relations in Small Groups}.” 55.
#' @examples 
#' task_eg <- to_named(to_uniplex(ison_algebra, "tasks"))
#' (triad_cen <- node_x_triad(task_eg))
#' @export
node_x_triad <- function(.data){
  .data <- manynet::expect_nodes(.data)
  out <- t(sapply(seq.int(manynet::net_nodes(.data)), 
                  function(x) net_x_triad(.data) - net_x_triad(manynet::delete_nodes(.data, x))))
  make_node_motif(out, .data)
}

# #' @rdname motif_node
# #' @section Quad census:
# #'   The quad census uses the `{oaqc}` package to do
# #'   the heavy lifting of counting the number of each orbits.
# #'   See `vignette('oaqc')`.
# #'   However, our function relabels some of the motifs
# #'   to avoid conflicts and improve some consistency with
# #'   other census-labelling practices.
# #'   The letter-number pairing of these labels indicate
# #'   the number and configuration of ties.
# #'   For now, we offer a rough translation:
# #' 
# #' | migraph | Ortmann and Brandes      
# #' | ------------- |------------- |
# #' | E4  | co-K4
# #' | I40, I41  | co-diamond
# #' | H4  | co-C4
# #' | L42, L41, L40 | co-paw
# #' | D42, D40 | co-claw
# #' | U42, U41 | P4
# #' | Y43, Y41 | claw
# #' | P43, P42, P41 | paw
# #' | 04 | C4
# #' | Z42, Z43 | diamond
# #' | X4 | K4
# #' 
# #' See also [this list of graph classes](https://www.graphclasses.org/smallgraphs.html#nodes4).

#' @rdname motif_node
#' @section Tetrad census:
#'   The nodal tetrad census counts the number of four-node configurations
#'   that each node is embedded in.
#'   The function returns a matrix with a special naming convention:
#'   - E4 (aka co-K4): This is an empty set of four nodes; no ties
#'   - I4 (aka co-diamond): This is a set of four nodes with just one tie
#'   - H4 (aka co-C4): This set of four nodes includes two non-adjacent ties
#'   - L4 (aka co-paw): This set of four nodes includes two adjacent ties
#'   - D4 (aka co-claw): This set of four nodes includes three adjacent ties,
#'   in the form of a triangle with one isolate
#'   - U4 (aka P4, four-actor line): This set of four nodes includes three ties 
#'   arranged in a line
#'   - Y4 (aka claw): This set of four nodes includes three ties all adjacent
#'   to a single node
#'   - P4 (aka paw, kite): This set of four nodes includes four ties arranged
#'   as a triangle with an extra tie hanging off of one of the nodes
#'   - C4 (aka bifan): This is a symmetric box or 4-cycle or set of shared choices
#'   - Z4 (aka diamond): This resembles C4 but with an extra tie cutting across the box
#'   - X4 (aka K4): This resembles C4 but with two extra ties cutting across the box;
#'   a realisation of all possible ties
#'   
#'   Graphs of these motifs can be shown using 
#'   `plot(node_by_tetrad(ison_southern_women))`.
#' @references
#' ## On the tetrad census
#'  Ortmann, Mark, and Ulrik Brandes. 2017. 
#'  “Efficient Orbit-Aware Triad and Quad Census in Directed and Undirected Graphs.” 
#'  \emph{Applied Network Science} 2(1):13. 
#'  \doi{10.1007/s41109-017-0027-2}.
#'  
#'  McMillan, Cassie, and Diane Felmlee. 2020.
#'  "Beyond Dyads and Triads: A Comparison of Tetrads in Twenty Social Networks".
#'  _Social Psychology Quarterly_ 83(4): 383-404.
#'  \doi{10.1177/0190272520944151}
#' @examples 
#' node_x_tetrad(ison_southern_women)
#' @export
node_x_tetrad <- function(.data){
  .data <- manynet::expect_nodes(.data)
  cmbs <- utils::combn(1:manynet::net_nodes(.data), 4)
  mat <- manynet::as_matrix(manynet::to_onemode(.data))
  dd <- apply(cmbs, 2, function(x) c(sum(mat[x,x]), 
                                     max(rowSums(mat[x,x]))))
  
  types <- rep(NA, ncol(cmbs))
  types[dd[1,] == 0] <- "E4"
  types[dd[1,] == 2] <- "I4"
  types[dd[1,] == 4 & dd[2,] == 1] <- "H4"
  types[dd[1,] == 4 & dd[2,] == 2] <- "L4"
  types[dd[1,] == 6 & dd[2,] == 2] <- "D4"
  types[dd[1,] == 6 & dd[2,] == 1] <- "U4"
  types[dd[1,] == 6 & dd[2,] == 3] <- "Y4"
  types[dd[1,] == 8 & dd[2,] == 3] <- "P4"
  types[dd[1,] == 8 & dd[2,] == 2] <- "C4"
  types[dd[1,] == 10] <- "Z4"
  types[dd[1,] == 12] <- "X4"
  
  appears <- sapply(seq.int(manynet::net_nodes(.data)), 
         function(x) types[which(cmbs == x, arr.ind = TRUE)[,2]])
  out <- apply(appears, 2, table)

  if(is.list(out)){
    out <- as.matrix(dplyr::bind_rows(out))
  } else out <- as.matrix(as.data.frame(t(out)))
  out.order <- c("E4","I4","H4","L4","D4","U4","Y4","P4","C4","Z4","X4")
  out <- out[,match(out.order, colnames(out))]
  colnames(out) <- out.order
  out[is.na(out)] <- 0

  make_node_motif(out, .data)
}

# https://stackoverflow.com/questions/26828301/faster-version-of-combn#26828486
# comb2.int <- function(n, choose = 2){
#   # e.g. n=3 => (1,2), (1,3), (2,3)
#   x <- rep(1:n,(n:1)-1)
#   i <- seq_along(x)+1
#   o <- c(0,cumsum((n-2):1))
#   y <- i-o[x]
#   return(cbind(x,y))
# }
  
# #' @export
# node_igraph_census <- function(.data, normalized = FALSE){
#     out <- igraph::motifs(manynet::as_igraph(.data), 4)
#     if(manynet::is_labelled(.data))
#       rownames(out) <- manynet::node_names(.data)
#     colnames(out) <- c("co-K4",
#                        "co-diamond",
#                        "co-C4",
#                        "co-paw",
#                        "co-claw",
#                        "P4",
#                        "claw",
#                        "paw",
#                        "C4",
#                        "diamond",
#                        "K4")
#     make_node_motif(out, .data)
# }

# Network cohesion ####

#' Motifs of network cohesion
#' @name motif_net
#' @description
#'   These functions include ways to take a census of the graphlets
#'   in a network: 
#'   
#'   - `net_x_dyad()` returns a census of dyad motifs in a network.
#'   - `net_x_triad()` returns a census of triad motifs in a network.
#'   - `net_x_tetrad()` returns a census of tetrad motifs in a network.
#'   - `net_x_mixed()` returns a census of triad motifs that span
#'   a one-mode and a two-mode network.
#'   
#'   See also \href{https://www.graphclasses.org/smallgraphs.html}{graph classes}.
#'   
#' @template param_data
#' @family cohesion
#' @template net_motif
#' @param object2 A second, two-mode network object.
NULL

#' @rdname motif_net
#' @section Dyad census: 
#'   The dyad census counts the number of mutual, asymmetric, and null dyads 
#'   in a network.
#'   For directed networks, 
#'   - Mutual dyads have ties in both directions
#'   - Asymmetric dyads have a tie in one direction only
#'   - Null dyads have no ties
#'   
#'   Note that for undirected and two-mode networks,
#'   only mutual and null dyads are possible, 
#'   as the concept of an asymmetric dyad does not apply.
#' @references
#' ## On the dyad census
#' Holland, Paul W., and Samuel Leinhardt. 1970. 
#' "A Method for Detecting Structure in Sociometric Data". 
#' _American Journal of Sociology_, 76: 492-513.
#' \doi{10.1016/B978-0-12-442450-0.50028-6}
#' 
#' Wasserman, Stanley, and Katherine Faust. 1994. 
#' "Social Network Analysis: Methods and Applications". 
#' Cambridge: Cambridge University Press.
#' @examples 
#' net_x_dyad(manynet::ison_algebra)
#' @export
net_x_dyad <- function(.data) {
  .data <- manynet::expect_nodes(.data)
  out <- suppressWarnings(igraph::dyad_census(manynet::as_igraph(.data)))
  out <- unlist(out)
  names(out) <- c("Mutual", "Asymmetric", "Null")
  if (!manynet::is_directed(.data)) out <- out[c(1, 3)]
  make_network_motif(out, .data)
}

#' @rdname motif_net
#' @section Triad census:
#'  The triad census counts the number of three-node configurations in the network.
#'  The function returns a matrix with a special naming convention:
#'  - 003: This is an empty triad; no ties
#'  - 012: This triad includes one tie
#'  - 102: This triad includes two ties, but they are not reciprocated
#'  - 021D: This triad includes two ties, one of which is reciprocated, and the other is directed towards the reciprocated tie
#'  - 021U: This triad includes two ties, one of which is
#'  reciprocated, and the other is directed away from the reciprocated tie
#'  - 021C: This triad includes two ties, one of which is reciprocated, and the other is directed between the two non-reciprocated nodes
#'  - 111D: This triad includes three ties, two of which are
#'  reciprocated, and the other is directed towards the reciprocated ties
#'  - 111U: This triad includes three ties, two of which are
#'  reciprocated, and the other is directed away from the reciprocated ties
#'  - 030T: This triad includes three ties, all of which are
#'  directed in a transitive manner (i.e. A->B, B->C, A->C)
#'  - 030C: This triad includes three ties, all of which are
#'  directed in a cyclic manner (i.e. A->B, B->C
#'  A->C)
#'  - 201: This triad includes three ties, all of which are reciproc
#'  ated (i.e. A<->B, B<->C, A<->C)
#'  - 120D: This triad includes four ties, three of which are
#'  reciprocated, and the other is directed towards the reciprocated ties
#'  - 120U: This triad includes four ties, three of which are
#'  reciprocated, and the other is directed away from the reciprocated ties
#'  - 120C: This triad includes four ties, three of which are
#'  reciprocated, and the other is directed between the two non-reciprocated
#'  - 210: This triad includes five ties, four of which are reciprocated, and the other is directed between the two non-reciprocated
#'  - 300: This triad includes six ties, all of which are reciprocated
#'  
#'  Note that for undirected and two-mode networks, only 003, 102, and 201 are possible,
#'  as the other configurations rely on the concept of directionality.
#' @references 
#' ## On the triad census
#' Davis, James A., and Samuel Leinhardt. 1967. 
#' “\href{https://files.eric.ed.gov/fulltext/ED024086.pdf}{The Structure of Positive Interpersonal Relations in Small Groups}.” 55.
#' @examples 
#' net_x_triad(manynet::ison_adolescents)
#' @export
net_x_triad <- function(.data) {
  .data <- manynet::expect_nodes(.data)
  if (manynet::is_twomode(.data)) {
    manynet::snet_abort("A twomode or multilevel option for a triad census is not yet implemented.")
  } else {
    out <- suppressWarnings(igraph::triad_census(as_igraph(.data)))
    names(out) <- c("003", "012", "102", "021D",
                    "021U", "021C", "111D", "111U",
                    "030T", "030C", "201", "120D",
                    "120U", "120C", "210", "300")
    if (!manynet::is_directed(.data)) out <- out[c(1, 2, 3, 11, 15, 16)]
    make_network_motif(out, .data)
  }
}

#' @rdname motif_net
#' @section Tetrad census:
#'   The tetrad census counts the number of four-node configurations in the network.
#'   The function returns a matrix with a special naming convention:
#'   - E4 (aka co-K4): This is an empty set of four nodes; no ties
#'   - I4 (aka co-diamond): This is a set of four nodes with just one tie
#'   - H4 (aka co-C4): This set of four nodes includes two non-adjacent ties
#'   - L4 (aka co-paw): This set of four nodes includes two adjacent ties
#'   - D4 (aka co-claw): This set of four nodes includes three adjacent ties,
#'   in the form of a triangle with one isolate
#'   - U4 (aka P4, four-actor line): This set of four nodes includes three ties 
#'   arranged in a line
#'   - Y4 (aka claw): This set of four nodes includes three ties all adjacent
#'   to a single node
#'   - P4 (aka paw, kite): This set of four nodes includes four ties arranged
#'   as a triangle with an extra tie hanging off of one of the nodes
#'   - C4 (aka bifan): This is a symmetric box or 4-cycle or set of shared choices
#'   - Z4 (aka diamond): This resembles C4 but with an extra tie cutting across the box
#'   - X4 (aka K4): This resembles C4 but with two extra ties cutting across the box;
#'   a realisation of all possible ties
#'   
#'   Graphs of these motifs can be shown using 
#'   `plot(net_x_tetrad(ison_southern_women))`.
#' @references
#' ## On the tetrad census
#'  Ortmann, Mark, and Ulrik Brandes. 2017. 
#'  “Efficient Orbit-Aware Triad and Quad Census in Directed and Undirected Graphs.” 
#'  \emph{Applied Network Science} 2(1):13. 
#'  \doi{10.1007/s41109-017-0027-2}.
#'  
#'  McMillan, Cassie, and Diane Felmlee. 2020.
#'  "Beyond Dyads and Triads: A Comparison of Tetrads in Twenty Social Networks".
#'  _Social Psychology Quarterly_ 83(4): 383-404.
#'  \doi{10.1177/0190272520944151}
#' @examples 
#' net_x_tetrad(ison_southern_women)
#' @export
net_x_tetrad <- function(.data){
  .data <- manynet::expect_nodes(.data)
  cmbs <- utils::combn(1:manynet::net_nodes(.data), 4)
  mat <- manynet::as_matrix(manynet::to_onemode(.data))
  dens <- apply(cmbs, 2, function(x) sum(mat[x,x]))
  
  E4 <- sum(dens == 0)
  I4 <- sum(dens == 1)
  
  if(any(dens==2)){
    if(sum(dens==2)>1){
      twosies <- apply(cmbs[,dens==2], 2, function(x) max(rowSums(mat[x,x])))
    } else twosies <- max(rowSums(mat[cmbs[,dens==2], cmbs[,dens==2]]))
    H4 <- sum(twosies==1)
    L4 <- sum(twosies==2)
  } else H4 <- L4 <- 0
  
  if(any(dens==3)){
    if(sum(dens==3)>1){
      threesies <- apply(cmbs[,dens==3], 2, function(x) max(rowSums(mat[x,x])))
    } else threesies <- max(rowSums(mat[cmbs[,dens==3], cmbs[,dens==3]]))
    D4 <- sum(threesies==2)
    U4 <- sum(threesies==1)
    Y4 <- sum(threesies==3)
  } else D4 <- U4 <- Y4 <- 0
  
  if(any(dens==4)){
    if(sum(dens==4)>1){
      foursies <- apply(cmbs[,dens==4], 2, function(x) max(rowSums(mat[x,x])))
    } else foursies <- max(rowSums(mat[cmbs[,dens==4], cmbs[,dens==4]]))
    P4 <- sum(foursies==3)
    C4 <- sum(foursies==2)
  } else P4 <- C4 <- 0
  
  Z4 <- sum(dens == 5)
  X4 <- sum(dens == 6)
  
  out <- c(E4 = E4, I4 = I4, H4 = H4, L4 = L4, D4 = D4, U4 = U4, Y4 = Y4, 
           P4 = P4, C4 = C4, Z4 = Z4, X4 = X4)
  make_network_motif(out, .data)
}

#' @rdname motif_net 
#' @source Alejandro Espinosa 'netmem'
#' @references 
#' ## On the mixed census
#' Hollway, James, Alessandro Lomi, Francesca Pallotti, and Christoph Stadtfeld. 2017.
#' “Multilevel Social Spaces: The Network Dynamics of Organizational Fields.” 
#' _Network Science_ 5(2): 187–212.
#' \doi{10.1017/nws.2017.8}
#' @examples 
#' net_x_mixed(fict_marvel)
#' @export
net_x_mixed <- function (.data, object2) {
  .data <- manynet::expect_nodes(.data)
  if(missing(object2) && manynet::is_multiplex(.data)) {
    object2 <- manynet::to_uniplex(.data, unique(manynet::tie_attribute(.data, "type"))[2])
    .data <- manynet::to_uniplex(.data, unique(manynet::tie_attribute(.data, "type"))[1])
  }
  if(manynet::is_twomode(.data))
    manynet::snet_abort("First object should be a one-mode network")
  if(!manynet::is_twomode(object2))
    manynet::snet_abort("Second object should be a two-mode network")
  if(manynet::net_dims(.data)[1] != manynet::net_dims(object2)[1])
    manynet::snet_abort("Non-conformable arrays")
  m1 <- manynet::as_matrix(.data)
  m2 <- manynet::as_matrix(object2)
  cp <- function(m) (-m + 1)
  onemode.reciprocal <- m1 * t(m1)
  onemode.forward <- m1 * cp(t(m1))
  onemode.backward <- cp(m1) * t(m1)
  onemode.null <- cp(m1) * cp(t(m1))
  diag(onemode.forward) <- 0
  diag(onemode.backward) <- 0
  diag(onemode.null) <- 0
  bipartite.twopath <- m2 %*% t(m2)
  bipartite.null <- cp(m2) %*% cp(t(m2))
  bipartite.onestep1 <- m2 %*% cp(t(m2))
  bipartite.onestep2 <- cp(m2) %*% t(m2)
  diag(bipartite.twopath) <- 0
  diag(bipartite.null) <- 0
  diag(bipartite.onestep1) <- 0
  diag(bipartite.onestep2) <- 0
  res <- c("22" = sum(onemode.reciprocal * bipartite.twopath) / 2,
           "21" = sum(onemode.forward * bipartite.twopath) / 2 + sum(onemode.backward * bipartite.twopath) / 2,
           "20" = sum(onemode.null * bipartite.twopath) / 2,
           "12" = sum(onemode.reciprocal * bipartite.onestep1) / 2 + sum(onemode.reciprocal * bipartite.onestep2) / 2,
           "11D" = sum(onemode.forward * bipartite.onestep1) / 2 + sum(onemode.backward * bipartite.onestep2) / 2,
           "11U" = sum(onemode.forward * bipartite.onestep2) / 2 + sum(onemode.backward * bipartite.onestep1) / 2,
           "10" = sum(onemode.null * bipartite.onestep2) / 2 + sum(onemode.null * bipartite.onestep1) / 2,
           "02" = sum(onemode.reciprocal * bipartite.null) / 2,
           "01" = sum(onemode.forward * bipartite.null) / 2 + sum(onemode.backward * bipartite.null) / 2,
           "00" = sum(onemode.null * bipartite.null) / 2)  
  make_network_motif(res, .data)
}

# Exposure ####

#' Motifs of nodes exposure
#' @name motif_exposure
#' @description
#'   `node_x_exposure()` produces a motif matrix of nodes' exposure to 
#'   infection/adoption by time step.
#' 
#' @template param_data
#' @family diffusion
#' @template node_motif
NULL

#' @rdname motif_exposure
#' @examples
#' node_x_exposure(play_diffusion(create_tree(12)))
#' @export
node_x_exposure <- function(.data){
  if(inherits(.data, "diff_model")){
    diff_model <- manynet::as_tidygraph(.data)
    times <- diff_model$t
    out <- sapply(times, function(x){
      inf <- node_is_infected(diff_model, time = x)
      if(sum(inf)==1) manynet::as_matrix(.data)[inf,] else
        colSums(manynet::as_matrix(.data)[inf,])
    })
  } else {
    times <- manynet::as_diffusion(.data)$time
    out <- sapply(times, function(x){
      inf <- node_is_infected(.data, time = x)
      if(sum(inf)==1) manynet::as_matrix(.data)[inf,] else
        colSums(manynet::as_matrix(.data)[inf,])
    })
  }
  colnames(out) <- paste0("t",times)
  make_node_motif(out, .data)
}

# Hazard ####

#' Motifs of network hazard
#' @name motif_hazard
#' @description
#'   `net_x_hazard()` measures the hazard rate or instantaneous probability that
#'   nodes will adopt/become infected at that time.
#' 
#' @template param_data
#' @family diffusion
#' @template net_motif
NULL

#' @rdname motif_hazard
#' @section Hazard rate: 
#' The hazard rate is the instantaneous probability of adoption/infection 
#' at each time point (Allison 1984).
#' In survival analysis, hazard rate is formally defined as:
#'
#' \deqn{%
#' \lambda(t)=\lim_{h\to +0}\frac{F(t+h)-F(t)}{h}\frac{1}{1-F(t)} %
#' }{%
#' \lambda(t-1)= lim (t -> +0) [F(t+h)-F(t)]/h * 1/[1-F(t)] %
#' }
#'
#' By approximating \eqn{h=1}, we can rewrite the equation as
#'
#' \deqn{%
#' \lambda(t)=\frac{F(t+1)-F(t)}{1-F(t)} %
#' }{%
#' \lambda(t-1)= [F(t+1)-F(t)]/[1-F(t)] %
#' }
#'
#' If we estimate \eqn{F(t)}, 
#' the probability of not having adopted the innovation in time \eqn{t}, 
#' from the proportion of adopters in that time, 
#' such that \eqn{F(t) \sim q_t/n}{F(t) ~ q(t)/n}, we now have (ultimately for \eqn{t>1}):
#'
#' \deqn{%
#' \lambda(t)=\frac{q_{t+1}/n-q_t/n}{1-q_t/n} = \frac{q_{t+1} - q_t}{n - q_t} = \frac{q_t - q_{t-1}}{n - q_{t-1}} %
#' }{%
#' \lambda(t-1)= [q(t+1)/n-q(t)/n]/[1-q(t)/n] = [q(t+1) - q(t)]/[n - q(t)] = [q(t) - q(t-1)]/[n - q(t-1)] %
#' }
#' 
#' where \eqn{q_i}{q(i)} is the number of adopters in time \eqn{t}, 
#' and \eqn{n} is the number of vertices in the graph.
#'
#' The shape of the hazard rate indicates the pattern of new adopters over time.
#' Rapid diffusion with convex cumulative adoption curves will have 
#' hazard functions that peak early and decay over time. 
#' Slow concave cumulative adoption curves will have 
#' hazard functions that are low early and rise over time.
#' Smooth hazard curves indicate constant adoption whereas 
#' those that oscillate indicate variability in adoption behavior over time.
#' @source `{netdiffuseR}`
#' @references
#' ## On hazard rates
#' Allison, Paul D. 1984. 
#' _Event history analysis: Regression for longitudinal event data_. 
#' London: Sage Publications.
#' \doi{10.4135/9781412984195}
#'
#' Wooldridge, Jeffrey M. 2010. 
#' _Econometric Analysis of Cross Section and Panel Data_ (2nd ed.). 
#' Cambridge: MIT Press.
#' @examples
#' # To calculate the hazard rates at each time point
#'   smeg <- generate_smallworld(15, 0.025)
#' net_x_hazard(play_diffusion(smeg, transmissibility = 0.3))
#' @export
net_x_hazard <- function(.data){
  diff_model <- manynet::as_diffusion(.data)
  out <- (diff_model$I - dplyr::lag(diff_model$I)) / 
    (diff_model$n - dplyr::lag(diff_model$I))
  if(inherits(.data, "diff_model")) 
    net <- attr(.data, "network") else 
      net <- .data
  names(out) <- paste0("t", diff_model$t)
  make_network_motif(out, net)
}

test_that("cluster_cosine works", {
  expect_s3_class(cluster_cosine(node_x_triad(ison_monks), distance = "euclidean"), "hclust")
})

test_that("cluster_cosine works", {
  unlab_2mode <- generate_random(c(6,6))
  expect_s3_class(cluster_concor(unlab_2mode, node_x_tetrad(unlab_2mode)), "hclust")
})

node_membs <- funs_objs[grepl("node_in_", names(funs_objs))]
for(fn in names(node_membs)) {
  for (ob in names(data_objs)) { 
    test_that(paste(fn, "works on", ob), {
      if(grepl("roulette", fn)){
        if(ob != "twomode")
          expect_s3_class(node_membs[[fn]](data_objs[[ob]], num_groups = 3), 
                          "node_member") else
                            succeed("Roulette doesn't work on two-mode objects")
      } else if(grepl("adopter", fn)){
        if(ob == "diffusion")
          expect_s3_class(node_membs[[fn]](data_objs[[ob]]), "node_member") else
                            succeed("Only works on diffusion objects")
      } else if(grepl("equivalence", fn)){
          expect_s3_class(node_membs[[fn]](data_objs[[ob]], 
                                           node_x_tie(data_objs[[ob]])), 
                          "node_member")
      } else {
        expect_s3_class(node_membs[[fn]](data_objs[[ob]]), "node_member")
      }
    })
  }
}

node_motifs <- funs_objs[grepl("node_x_", names(funs_objs))]
for(fn in names(node_motifs)) {
  for (ob in names(data_objs)) { 
    test_that(paste(fn, "works on", ob), {
      skip_if(grepl("triad|dyad", fn) && is_twomode(data_objs[[ob]]))
      if(grepl("brokerage", fn)){
        if(ob == "attribute")
          expect_s3_class(node_motifs[[fn]](data_objs[[ob]], "group"), "node_motif") else
            succeed("Only used for attribute objects")
      } else if(grepl("exposure", fn)){
        if(ob == "diffusion")
          expect_s3_class(node_motifs[[fn]](data_objs[[ob]]), "node_motif") else
            succeed("Only used for diffusion objects")
      } else {
        expect_s3_class(node_motifs[[fn]](data_objs[[ob]]), "node_motif")
      }
    })
  }
}

# # Census function family tests
set.seed(123)
task_eg <- to_named(to_uniplex(ison_algebra, "tasks"))

test_that("node_x_tie census works", {
  test <- node_x_tie(task_eg)
  expect_equal(test[1:4], rep(0, 4))
  expect_output(print(test), "fromA")
  expect_equal(nrow(summary(test, membership = node_in_roulette(task_eg, 3))),
               3)
})

test_that("node_x_dyad census works", {
  test <- node_x_dyad(ison_adolescents)
  expect_equal(colnames(test)[1:2], c("Mutual", "Null"))
})

test_that("node_x_triad census works", {
  test <- node_x_triad(task_eg)
  expect_equal(top3(test[,16]), c(7,8,6))
  expect_equal(colnames(test)[1:3], c("003", "012", "102"))
})

test_that("net_x_dyad census works", {
  test <- net_x_dyad(ison_adolescents)
  expect_equal(test[[1]], 10)
  expect_equal(test[[2]], 18)
  expect_equal(names(test), c("Mutual", "Null"))
  expect_output(print(test), "Mutual")
})

test_that("net_x_triad census works", {
  test <- net_x_triad(ison_adolescents)
  expect_equal(test[[1]], 13)
  expect_equal(test[[3]], 29)
  expect_equal(names(test), c("003", "012", "102", "201", "210", "300"))
  expect_equal(names(summary(test)), c("003", "012", "102", "201", "210", "300"))
  # Error
  expect_error(net_x_triad(ison_southern_women))
})

test_that("net_x_tetrad census works", {
  test <- net_x_tetrad(ison_southern_women)
  expect_values(c(test)[1], 12388)
})

test_that("node_x_tetrad census works", {
  test <- node_x_tetrad(ison_southern_women)
  expect_equal(test[1,1], 1241)
})

test_that("net_mixed census works", {
  marvel_friends <- to_unsigned(to_uniplex(fict_marvel, "relationship"), "positive")
  test <- net_x_mixed(marvel_friends, to_uniplex(fict_marvel, "affiliation"))
  expect_equal(unname(test[1]), 1137)
  expect_equal(names(test[1]), "22")
  # Errors
  expect_error(net_x_mixed(ison_southern_women,
                            to_uniplex(fict_marvel, "affiliation")))
  expect_error(net_x_mixed(to_uniplex(fict_marvel, "affiliation"),
                                    ison_southern_women))
  expect_error(net_x_mixed(ison_karateka,
                            to_uniplex(fict_marvel, "affiliation")))
})

test <- node_x_path(ison_southern_women)
test_that("node path census works", {
  expect_equal(c(net_nodes(ison_adolescents)),
               nrow(node_x_path(ison_adolescents)))
  expect_true(nrow(node_x_path(ison_southern_women)) ==
                ncol(node_x_path(ison_southern_women)))
})

test_that("node_x_brokerage works", {
  test <- node_x_brokerage(ison_networkers, "Discipline")
  expect_equal(dim(test), c(32,6))
})

test_that("net_x_brokerage works", {
  test <- net_x_brokerage(ison_networkers, "Discipline")
  expect_equal(top3(names(test)), c("Coordinator","Itinerant","Gatekeeper"))
})

# Structural properties ####

#' Marking ties based on structural properties
#' @description 
#'   These functions return logical vectors the length of the ties
#'   in a network identifying which hold certain properties or positions in the network.
#'   
#'   - `tie_is_loop()` marks ties that are loops.
#'   - `tie_is_feedback()` marks ties that are feedback arcs causing the network to not be acyclic.
#'   - `tie_is_bridge()` marks ties that cut or act as articulation points in a network.
#'   - `tie_is_path()` marks ties on a path from one node to another.
#'   
#'   They are most useful in highlighting parts of the network that
#'   are particularly well- or poorly-connected.
#' @template param_data
#' @template tie_mark
#' @name mark_ties
NULL

#' @rdname mark_ties
#' @importFrom igraph which_loop
#' @examples 
#' tie_is_loop(fict_marvel)
#' @export
tie_is_loop <- function(.data){
  .data <- manynet::expect_ties(.data)  
  make_tie_mark(igraph::which_loop(manynet::as_igraph(.data)), .data)
}

#' @rdname mark_ties
#' @importFrom igraph feedback_arc_set
#' @examples 
#' tie_is_feedback(ison_algebra)
#' @export
tie_is_feedback <- function(.data){
  .data <- manynet::expect_ties(.data)  
  .data <- manynet::as_igraph(.data)
  make_tie_mark(igraph::E(.data) %in% igraph::feedback_arc_set(.data), 
                .data)
}

#' @rdname mark_ties
#' @importFrom igraph decompose delete_edges
#' @examples 
#' tie_is_bridge(ison_brandes)
#' @export
tie_is_bridge <- function(.data){
  .data <- manynet::expect_ties(.data)  
  num_comp <- length( igraph::decompose(manynet::as_igraph(.data)) )
  out <- vapply(seq_len(manynet::net_ties(.data)), function(x){
    length( igraph::decompose(igraph::delete_edges(.data, x)) ) > num_comp
  }, FUN.VALUE = logical(1))
  if(manynet::is_labelled(.data)) 
    names(out) <- attr(igraph::E(.data), "vnames")
  make_tie_mark(out, .data)
}

#' @rdname mark_ties
#' @param from The index or name of the node from which the path should start.
#' @param to The index or name of the node to which the path should be traced.
#' @param all_paths Whether to return a list of paths or sample just one.
#'   By default FALSE, sampling just a single path.
#' @importFrom igraph all_shortest_paths
#' @examples 
#' ison_adolescents |>
#'   mutate_ties(route = tie_is_path(from = "Jane", to = 7)) 
#' @export
tie_is_path <- function(.data, from, to, all_paths = FALSE){
  .data <- manynet::expect_ties(.data)  
  out <- igraph::all_shortest_paths(.data, from = from, to = to,
                                     mode = "out")$epath
  if(all_paths){
    out <- igraph::E(.data) %in% unique(unlist(out))
  } else out <- igraph::E(.data) %in% out[[sample(length(out),1)]]
  make_tie_mark(out, .data)
}

# Dyadic properties ####

#' Marking ties based on dyadic properties
#' 
#' @description 
#'   These functions return logical vectors the length of the ties
#'   in a network identifying which are embedded within particular dyads.
#'   
#'   - `tie_is_multiple()` marks ties that are multiples.
#'   - `tie_is_reciprocated()` marks ties that are mutual/reciprocated.
#'   
#'   They are most useful in highlighting parts of the network where
#'   relationships are denser.
#' @template param_data
#' @template tie_mark
#' @name mark_dyads
NULL

#' @rdname mark_dyads
#' @importFrom igraph which_multiple
#' @examples 
#' tie_is_multiple(fict_marvel)
#' @export
tie_is_multiple <- function(.data){
  .data <- manynet::expect_ties(.data)  
  make_tie_mark(igraph::which_multiple(manynet::as_igraph(.data)), .data)
}

#' @rdname mark_dyads
#' @importFrom igraph which_mutual
#' @examples 
#' tie_is_reciprocated(ison_algebra)
#' @export
tie_is_reciprocated <- function(.data){
  .data <- manynet::expect_ties(.data)  
  make_tie_mark(igraph::which_mutual(manynet::as_igraph(.data)), .data)
}

# Triangular properties ####

#' Marking ties based on triangular properties
#' 
#' @description 
#'   These functions return logical vectors the length of the ties
#'   in a network identifying which hold certain properties or positions in the network.
#'   
#'   - `tie_is_triangular()` marks ties that are part of triangles.
#'   - `tie_is_cyclical()` marks ties that are part of cycles.
#'   - `tie_is_triplet()` marks ties that are part of transitive triplets.
#'   - `tie_is_simmelian()` marks ties that are both in a triangle 
#'   and fully reciprocated.
#'   - `tie_is_imbalanced()` marks ties that are part of imbalanced triads.
#'   - `tie_is_transitive()` marks ties that complete transitive closure.
# #'   - `tie_is_forbidden()` marks ties that complete forbidden triads.
#'   
#'   They are most useful in highlighting parts of the network that
#'   are cohesively connected.
#' @template param_data
#' @template tie_mark
#' @family cohesion
#' @name mark_triangles
NULL

#' @rdname mark_triangles
#' @importFrom igraph triangles
#' @examples 
#' ison_monks |> to_uniplex("like") |> 
#'   mutate_ties(tri = tie_is_triangular())
#' @export
tie_is_triangular <- function(.data){
  .data <- manynet::expect_ties(.data)  
  out <- .triangle_ties(.data)
  ties <- manynet::as_edgelist(manynet::to_unnamed(.data))[,c("from","to")]
  out <- do.call(paste, ties) %in% do.call(paste, as.data.frame(out))
  make_tie_mark(out, .data)
}

.triangle_ties <- function(.data){
  out <- t(matrix(igraph::triangles(manynet::as_igraph(.data)), nrow = 3))
  out <- rbind(out[,c(1,2)],out[,c(2,3)],out[,c(3,1)],
               out[,c(1,3)],out[,c(3,2)],out[,c(2,1)])
  out
}

#' @rdname mark_triangles
#' @examples 
#' ison_adolescents |> to_directed() |> 
#'   mutate_ties(trans = tie_is_transitive())
#' @export
tie_is_transitive <- function(.data){
  .data <- manynet::expect_ties(.data)  
  nodes <- manynet::as_edgelist(manynet::to_unnamed(.data))
  out <- vapply(seq_len(manynet::net_ties(.data)), function(x){
    igraph::distances(manynet::delete_ties(.data, x), 
                      v = nodes[x,1], to = nodes[x,2], 
                      mode = "out") == 2
  }, FUN.VALUE = logical(1))
  make_tie_mark(out, .data)
}

#' @rdname mark_triangles
#' @examples 
#' ison_adolescents |> to_directed() |> 
#'   mutate_ties(trip = tie_is_triplet())
#' @export
tie_is_triplet <- function(.data){
  .data <- manynet::expect_ties(.data)  
  nodes <- manynet::as_edgelist(manynet::to_unnamed(.data))
  trans <- tie_is_transitive(.data)
  altpath <- unlist(lapply(which(trans), function(x){
    tie_is_path(manynet::delete_ties(.data, x),
                      from = nodes[x,1], to = nodes[x,2],
                      all_paths = TRUE)
  }))
  if(!is.null(names(altpath)))
    names(altpath) <- gsub("^.*\\.", "", names(altpath))
  altpath <- altpath[altpath]
  trans[names(trans) %in% names(altpath)] <- TRUE
  make_tie_mark(trans, .data)
}

#' @rdname mark_triangles
#' @examples 
#' ison_adolescents |> to_directed() |> 
#'   mutate_ties(cyc = tie_is_cyclical())
#' @export
tie_is_cyclical <- function(.data){
  .data <- manynet::expect_ties(.data)  
  out <- vapply(seq_len(manynet::net_ties(.data)), function(x){
    nodes <- manynet::as_edgelist(manynet::to_unnamed(.data))[x,]
    igraph::distances(manynet::delete_ties(.data, x), 
                      v = nodes[2], to = nodes[1], 
                      mode = "out") == 2
  }, FUN.VALUE = logical(1))
  make_tie_mark(out, .data)
}

#' @rdname mark_triangles
#' @examples 
#' ison_monks |> to_uniplex("like") |> 
#'   mutate_ties(simmel = tie_is_simmelian())
#' @export
tie_is_simmelian <- function(.data){
  .data <- manynet::expect_ties(.data)  
  recip <- manynet::filter_ties(.data, tie_is_reciprocated())
  simmel <- manynet::filter_ties(recip, tie_is_triangular())
  ties <- manynet::as_edgelist(manynet::to_unnamed(.data))[,c("from","to")]
  simmel <- manynet::as_edgelist(manynet::to_unnamed(simmel))[,c("from","to")]
  out <- do.call(paste, ties) %in% do.call(paste, simmel)
  make_tie_mark(out, .data)
}

# #' @rdname mark_triangles
# #' @examples 
# #' generate_random(8, directed = TRUE) |> 
# #'   mutate_ties(forbid = tie_is_forbidden())
# #' @export
# tie_is_forbidden <- function(.data){
#   .data <- manynet::expect_ties(.data)
#   if(!manynet::is_weighted(.data)) 
#     snet_abort("This function only works with weighted networks.")
#   dists <- igraph::distances(.data, mode = "out")==2
#   ends <- which(dists * t(dists)==1, arr.ind = TRUE)
#   ends <- t(apply(ends, 1, function(x) sort(x)))
#   ends <- ends[!duplicated(ends),]
#   tris <- apply(ends, 1, function(x){
#     y <- unlist(igraph::all_shortest_paths(.data, x[1], x[2], mode = "out")$res)
#     y <- matrix(y, ncol = 3, byrow = TRUE)
#     y <- do.call("paste", c(as.data.frame(y)[, , drop = FALSE], sep = "-"))
#     z <- unlist(igraph::all_shortest_paths(.data, x[1], x[2], mode = "in")$res)
#     z <- matrix(z, ncol = 3, byrow = TRUE)
#     z <- do.call("paste", c(as.data.frame(z)[, , drop = FALSE], sep = "-"))
#     as.numeric(unlist(strsplit(y[y %in% z], "-")))
#   })
#   out <- matrix(unlist(tris), ncol = 3, byrow = TRUE)
#   out <- unique(c(apply(out, 1, function(x){
#     c(paste0(x[1],"->",x[2]),
#     paste0(x[2],"->",x[1]),
#     paste0(x[2],"->",x[3]),
#     paste0(x[3],"->",x[2]))
#   } )))
#   out <- names(tie_is_reciprocated(.data)) %in% out
#   make_tie_mark(out, .data)
# }

#' @rdname mark_triangles
#' @examples
#' fict_marvel |> to_uniplex("relationship") |> tie_is_imbalanced()
#' @export
tie_is_imbalanced <- function(.data){
  .data <- manynet::expect_ties(.data)  
  
  # identify_imbalanced_ties <- function(adj_matrix) {
  adj_matrix <- manynet::as_matrix(.data)
  
  # Check if the input is a square matrix
  if (!is.matrix(adj_matrix) || nrow(adj_matrix) != ncol(adj_matrix)) {
    manynet::snet_unavailable("This function only works with undirected one-mode networks.")
  }
  
  # Get the number of nodes
  n <- manynet::net_nodes(.data)
  
  # Identify all edges in the upper triangle (non-zero)
  edges <- manynet::as_edgelist(manynet::to_unnamed(.data))
  num_edges <- nrow(edges)
  
  # Initialize a logical vector to store the result for each edge
  is_imbalanced <- logical(num_edges)
  
  # Loop over each edge to check if it participates in an imbalanced triad
  for (e in 1:num_edges) {
    i <- unname(unlist(edges[e, 1]))
    j <- unname(unlist(edges[e, 2]))
    
    # Variable to track if the current edge is part of any imbalanced triad
    imbalanced_found <- FALSE
    
    # Check all possible third nodes k to form a triad (i, j, k)
    for (k in setdiff(1:n, c(i, j))) {
      # Get the ties for the triad (i, j, k)
      a <- adj_matrix[i, j]
      b <- adj_matrix[j, k]
      c <- adj_matrix[i, k]
      
      # Check if the triad is complete (no zeros)
      if (b != 0 && c != 0) {
        # Count the number of positive ties in the triad
        positive_count <- sum(c(a, b, c) > 0)
        
        # Determine if the triad is imbalanced
        if (positive_count == 2 || positive_count == 0) {
          # Mark as imbalanced and exit loop for this edge
          imbalanced_found <- TRUE
          break
        }
      }
    }
    
    # Store the result for the current edge
    is_imbalanced[e] <- imbalanced_found
  }
  
  make_tie_mark(is_imbalanced, .data)
}

# Selection properties ####

#' Marking ties based on measures
#' @name mark_select_tie
#' @description 
#'   These functions return logical vectors the length of the ties in a network:
#'   
#'   - `tie_is_random()` marks one or more ties at random.
#'   - `tie_is_max()` and `tie_is_min()` are more useful
#'   for converting the results from some tie measure into a mark-class object.
#'   They can be particularly useful for highlighting which tie or ties
#'   are key because they minimise or, more often, maximise some measure.
#' @template param_data
#' @template tie_mark
#' @family selection
NULL

#' @rdname mark_select_tie
#' @template param_select
#' @export
tie_is_random <- function(.data, select = 1){
  .data <- manynet::expect_ties(.data)  
  n <- manynet::net_ties(.data)
  out <- rep(FALSE, n)
  out[sample.int(n, select)] <- TRUE
  make_tie_mark(out, .data)
}

#' @rdname mark_select_tie
#' @param tie_measure An object created by a `tie_` measure.
#' @examples 
#' tie_is_max(tie_by_betweenness(ison_brandes))
#' @export
tie_is_max <- function(tie_measure){
  out <- as.numeric(tie_measure) == max(as.numeric(tie_measure))
  class(out) <- c("tie_mark", class(out))
  out
}

#' @rdname mark_select_tie
#' @examples 
#' tie_is_min(tie_by_betweenness(ison_brandes))
#' @export
tie_is_min <- function(tie_measure){
  out <- as.numeric(tie_measure) == min(as.numeric(tie_measure))
  class(out) <- c("tie_mark", class(out))
  out
}

tie_marks <- funs_objs[grepl("tie_is_", names(funs_objs))]
for(fn in names(tie_marks)) {
  for (ob in names(data_objs)) { 
    test_that(paste(fn, "works on", ob), {
      skip_if(grepl("tie_is_imbalanced", fn) && ob == "twomode")
      if(fn == "tie_is_path"){
        expect_s3_class(tie_marks[[fn]](data_objs[[ob]], 1, 2), "tie_mark")
      } else if(grepl("infected|recovered", fn)){
        if(ob == "diffusion")
        expect_s3_class(tie_marks[[fn]](data_objs[[ob]]), "tie_mark") else
          success("Only used for diffusion objects")
      } else if(grepl("max|min", fn)){
        skip_if_not(packageVersion("manynet") >= "1.7.3")
        expect_s3_class(tie_marks[[fn]](tie_by_degree(data_objs[[ob]])), "tie_mark")
      } else {
        expect_s3_class(tie_marks[[fn]](data_objs[[ob]]), "tie_mark")
      }
    })
  }
}

graph1 <- igraph::make_directed_graph(c(1,2,1,5,2,3,2,4,3,5,4,5,5,1))
graph2 <- igraph::make_undirected_graph(c(1,1,1,2,2,4,3,4,3,4))

test_that("tie_is_reciprocated works", {
  expect_length(tie_is_reciprocated(graph1), 7)
  expect_true(tie_is_reciprocated(graph1)[2], tie_is_reciprocated(graph1)[7])
})

test_that("tie_is_multiple works", {
  expect_true(tie_is_multiple(graph2)[5])
})

test_that("tie_is_loop works", {
  expect_true(tie_is_loop(graph2)[1])
  expect_false(tie_is_loop(graph2)[2], tie_is_loop(graph2)[3], tie_is_loop(graph2)[4])
})

test_that("tie_is_bridge works", {
  expect_equal(length(tie_is_bridge(graph1)), c(net_ties(graph1)))
})

test_that("tie_is_path works", {
  expect_equal(sum(tie_is_path(ison_adolescents, "Betty", "Alice")), 2)
})

test_that("tie_is_triangular works", {
  expect_equal(sum(tie_is_triangular(ison_adolescents)), 7)
  expect_output(print(tie_is_triangular(ison_adolescents)), "Betty-Sue")
})

test_that("directed triangle tie marks work", {
  acy_adol <- to_acyclic(ison_adolescents)
  expect_true(sum(tie_is_transitive(acy_adol)) >= 3)
  expect_true(sum(tie_is_triplet(acy_adol)) >= 3)
  expect_false(any(tie_is_cyclical(acy_adol)))
  expect_false(any(tie_is_simmelian(acy_adol)))
})

test_that("tie_is_max works", {
  skip_on_ci()
  skip_on_cran()
  expect_equal(length(tie_is_max(tie_by_betweenness(graph1))),
               c(net_ties(graph1)))
  expect_equal(sum(tie_is_max(tie_by_betweenness(graph1)) == TRUE), 1)
  expect_s3_class(tie_is_max(tie_by_betweenness(graph1)), "logical")
})

test_that("tie_is_min works", {
  skip_on_ci()
  skip_on_cran()
  expect_equal(length(tie_is_min(tie_by_betweenness(ison_brandes))),
               c(net_ties(ison_brandes)))
  expect_equal(sum(tie_is_min(tie_by_betweenness(ison_brandes)) == TRUE), 1)
  expect_s3_class(tie_is_min(tie_by_betweenness(ison_brandes)), "logical")
})

test_that("tie_is_feedback() mark functions work", {
  expect_equal(as.character(tie_is_feedback(ison_adolescents)),
               c("FALSE", "FALSE", "FALSE", "FALSE", "TRUE", "TRUE", "FALSE",
                 "TRUE", "FALSE", "FALSE"))
})

# Change measures ####

#' Measures of network change
#' @name measure_periods
#' @description
#'   `net_by_waves()` measures the number of waves in longitudinal network data.
#' 
#' @template param_data
#' @family change
#' @template net_measure
NULL

#' @rdname measure_periods 
#' @export
net_by_waves <- function(.data){
  .data <- manynet::expect_nodes(.data)
  tie_waves <- length(unique(manynet::tie_attribute(.data, "wave")))
  if(manynet::is_changing(.data)){
    chltime <- manynet::as_changelist(.data)$time
    chg_waves <- (max(chltime)+1) - max(min(chltime)-1, 0)
  } else chg_waves <- 1
  make_network_measure(max(tie_waves, chg_waves),
                       .data, call = deparse(sys.call()))
}

# Change motifs ####

#' Motifs of network change
#' @name motif_periods
#' @description
#'   These functions measure certain topological features of networks:
#'   
#'   - `net_x_change()` measures the Hamming distance between two or more networks.
#'   - `net_x_stability()` measures the Jaccard index of stability between two or more networks.
#'   - `net_x_correlation()` measures the product-moment correlation between two networks.
#' 
#'   These `net_*()` functions return a numeric vector the length of the number
#'   of networks minus one. E.g., the periods between waves.
#' @template param_data
#' @family change
#' @template net_motif
NULL

#' @rdname motif_periods 
#' @param object2 A network object.
#' @export
net_x_change <- function(.data, object2){
  net <- manynet::expect_nodes(.data)
  if(!missing(object2)){
    net <- list(net, object2)
  } else if(manynet::is_longitudinal(net)){
    net <- manynet::to_waves(net)
  }
  if(!manynet::is_list(net))
    manynet::snet_abort("`.data` must be a list of networks or a second network must be provided.")
  periods <- length(net)-1
  out <- vapply(seq.int(periods), function(x){
    net1 <- manynet::as_matrix(net[[x]])
    net2 <- manynet::as_matrix(net[[x+1]])
    sum(net1 != net2)
  }, FUN.VALUE = numeric(1))
  make_network_motif(out, .data)
}

#' @rdname motif_periods 
#' @export
net_x_stability <- function(.data, object2){
  net <- manynet::expect_nodes(.data)
  if(!missing(object2)){
    net <- list(net, object2)
  } else if(manynet::is_longitudinal(net)){
    net <- manynet::to_waves(net)
  }
  if(!manynet::is_list(net))
    manynet::snet_abort("`.data` must be a list of networks or a second network must be provided.")
  periods <- length(net)-1
  out <- vapply(seq.int(periods), function(x){
    net1 <- manynet::as_matrix(net[[x]])
    net2 <- manynet::as_matrix(net[[x+1]])
    n11 <- sum(net1 * net2)
    n01 <- sum(net1==0 * net2)
    n10 <- sum(net1 * net2==0)
    n11 / (n01 + n10 + n11)
  }, FUN.VALUE = numeric(1))
  make_network_motif(out, .data)
}

#' @rdname motif_periods 
#' @export
net_x_correlation <- function(.data, object2){
  .data <- manynet::expect_nodes(.data)
  comp1 <- manynet::as_matrix(.data)
  comp2 <- manynet::as_matrix(object2)
  if(!manynet::is_complex(.data)){
    diag(comp1) <- NA
  }
  if(!manynet::is_directed(.data)){
    comp1[upper.tri(comp1)] <- NA
  }
  out <- cor(c(comp1), c(comp2), use = "complete.obs")
  make_network_motif(out, .data)
}

# Closeness-like centralities ####

#' Measuring nodes closeness-like centrality
#' @name measure_central_close
#' @description
#'   These functions calculate common closeness-related centrality measures 
#'   that rely on path-length for one- and two-mode networks:
#'   
#'   - `node_by_closeness()` measures the closeness centrality of nodes in a 
#'   network.
#'   - `node_by_harmonic()` measures nodes' harmonic centrality or valued 
#'   centrality, which is thought to behave better than reach centrality 
#'   for disconnected networks.
#'   - `node_by_reach()` measures nodes' reach centrality,
#'   or how many nodes they can reach within _k_ steps.
#'   - `node_by_information()` measures nodes' information centrality or 
#'   current-flow closeness centrality.
#'   - `node_by_eccentricity()` measures nodes' eccentricity or maximum distance
#'   from another node in the network.
#'   - `node_by_distance()` measures nodes' geodesic distance from or to a 
#'   given node.
#'   - `node_by_vitality()` measures a network's closeness vitality centrality,
#'   or the change in closeness centrality between networks with and without a
#'   given node.
#'   
#'   All measures attempt to use as much information as they are offered,
#'   including whether the networks are directed, weighted, or multimodal.
#'   If this would produce unintended results, 
#'   first transform the salient properties using e.g. [to_undirected()] functions.
#'   All centrality and centralization measures return normalized measures by default,
#'   including for two-mode networks.
#' @template param_data
#' @template param_norm
#' @template param_dir
#' @family closeness
#' @family centrality
#' @template node_measure
NULL

#' @rdname measure_central_close
#' @param cutoff Maximum path length to use during calculations.
#' @section Closeness centrality: 
#'   Closeness centrality, status centrality, or barycenter centrality is 
#'   defined as the reciprocal of the farness or distance, \eqn{d}, 
#'   from a node to all other nodes in the network:
#'   \deqn{C_C(i) = \frac{1}{\sum_j d(i,j)}}
#'   When (more commonly) normalised, the numerator is instead \eqn{N-1}.
#' @references
#' ## On closeness centrality
#' Bavelas, Alex. 1950. 
#' "Communication Patterns in Task‐Oriented Groups". 
#' _The Journal of the Acoustical Society of America_, 22(6): 725–730.
#' \doi{10.1121/1.1906679}
#' 
#' Harary, Frank. 1959. 
#' "Status and Contrastatus". 
#' _Sociometry_, 22(1): 23–43. 
#' \doi{10.2307/2785610}
#' @examples
#' node_by_closeness(ison_southern_women)
#' @export
node_by_closeness <- function(.data, normalized = TRUE, 
                              direction = "out", cutoff = NULL){
  
  .data <- manynet::expect_nodes(.data)
  weights <- `if`(manynet::is_weighted(.data), 
                  manynet::tie_weights(.data), NA)
  graph <- manynet::as_igraph(.data)
  
  # Do the calculations
  if (manynet::is_twomode(graph) & normalized){
    # farness <- rowSums(igraph::distances(graph = graph))
    closeness <- igraph::closeness(graph = graph, vids = igraph::V(graph), mode = direction)
    other_set_size <- ifelse(igraph::V(graph)$type, sum(!igraph::V(graph)$type), sum(igraph::V(graph)$type))
    set_size <- ifelse(igraph::V(graph)$type, sum(igraph::V(graph)$type), sum(!igraph::V(graph)$type))
    out <- closeness/(1/(other_set_size+2*set_size-2))
  } else {
    cutoff <- if (is.null(cutoff)) -1 else cutoff
    out <- igraph::closeness(graph = graph, vids = igraph::V(graph), mode = direction, 
                             cutoff = cutoff, weights = weights, normalized = normalized)
  }
  out <- make_node_measure(out, .data)
  out
} 

#' @rdname measure_central_close 
#' @section Harmonic centrality:
#'   Harmonic centrality or valued centrality reverses the sum and reciprocal 
#'   operations compared to closeness centrality:
#'   \deqn{C_H(i) = \sum_{i, i \neq j} \frac{1}{d(i,j)}}
#'   where \eqn{\frac{1}{d(i,j)} = 0} where there is no path between \eqn{i} and
#'   \eqn{j}. Normalization is by \eqn{N-1}. 
#'   Since the harmonic mean performs better than the arithmetic mean on
#'   unconnected networks, i.e. networks with infinite distances,
#'   harmonic centrality is to be preferred in these cases.
#' @references
#'   ## On harmonic centrality
#'   Marchiori, Massimo, and Vito Latora. 2000. 
#'   "Harmony in the small-world".
#'   _Physica A_ 285: 539-546.
#'   \doi{10.1016/S0378-4371(00)00311-3}
#'   
#'   Dekker, Anthony. 2005.
#'   "Conceptual distance in social network analysis".
#'   _Journal of Social Structure_ 6(3).
#' @export
node_by_harmonic <- function(.data, normalized = TRUE, cutoff = -1){
  .data <- manynet::expect_nodes(.data)
  out <- igraph::harmonic_centrality(as_igraph(.data), # weighted if present
                                     normalized = normalized, cutoff = cutoff)
  out <- make_node_measure(out, .data)
  out
}

#' @rdname measure_central_close 
#' @section Reach centrality: 
#'   In some cases, longer path lengths are irrelevant and 'closeness' should
#'   be defined as how many others are in a local neighbourhood.
#'   How many steps out this neighbourhood should be defined as is given by 
#'   the 'cutoff' parameter. 
#'   This is usually termed \eqn{k} or \eqn{m} in equations,
#'   which is why this is sometimes called (\eqn{m}- or) 
#'   \eqn{k}-step reach centrality:
#'   \deqn{C_R(i) = \sum_j d(i,j) \leq k}
#'   The maximum reach score is \eqn{N-1}, achieved when the node can reach all
#'   other nodes in the network in \eqn{k} steps or less,
#'   but the normalised version, \eqn{\frac{C_R}{N-1}}, is more common.
#'   Note that if \eqn{k = 1} (i.e. cutoff = 1), then this returns the node's degree.
#'   At higher cutoff reach centrality returns the size of the node's component.
#' @references
#' ## On reach centrality
#' Borgatti, Stephen P., Martin G. Everett, and J.C. Johnson. 2013. 
#' _Analyzing social networks_. 
#' London: SAGE Publications Limited.
#' @examples
#' node_by_reach(ison_adolescents)
#' @export
node_by_reach <- function(.data, normalized = TRUE, cutoff = 2){
  .data <- manynet::expect_nodes(.data)
  if(manynet::is_weighted(.data)){
    tore <- manynet::as_matrix(.data)/mean(manynet::as_matrix(.data))
    out <- 1/tore
  } else out <- igraph::distances(manynet::as_igraph(.data))
  diag(out) <- 0
  out <- rowSums(out <= cutoff)
  if(normalized) out <- out/(manynet::net_nodes(.data)-1)
  out <- make_node_measure(out, .data)
  out
}

#' @rdname measure_central_close 
#' @section Information centrality: 
#'   Information centrality, also known as current-flow centrality, 
#'   is a hybrid measure relating to both path-length and walk-based measures. 
#'   The information centrality of a node is the harmonic average of the 
#'   “bandwidth” or inverse path-length for all paths originating from the node.
#'   
#'   As described in the `{sna}` package, 
#'   information centrality works on an undirected but potentially weighted 
#'   network excluding isolates (which take scores of zero).
#'   It is defined as:
#'   \deqn{C_I = \frac{1}{T + \frac{\sum T - 2 \sum C_1}{|N|}}}
#'   where \eqn{C = B^-1} with \eqn{B} is a pseudo-adjacency matrix replacing
#'   the diagonal of \eqn{1-A} with \eqn{1+k},
#'   and \eqn{T} is the trace of \eqn{C} and \eqn{S_R} an arbitrary row sum 
#'   (all rows in \eqn{C} have the same sum). 
#'   
#'   Nodes with higher information centrality have a large number of short paths
#'   to many others in the network, and are thus considered to have greater
#'   control of the flow of information.
#' @references
#' ## On information centrality
#' Stephenson, Karen, and Marvin Zelen. 1989.
#' "Rethinking centrality: Methods and examples". 
#' _Social Networks_ 11(1):1-37. 
#' \doi{10.1016/0378-8733(89)90016-6}
#' 
#' Brandes, Ulrik, and Daniel Fleischer. 2005. 
#' "Centrality Measures Based on Current Flow". 
#' _Proc. 22nd Symp. Theoretical Aspects of Computer Science_ LNCS 3404: 533-544. 
#' \doi{10.1007/978-3-540-31856-9_44}
#' @export
node_by_information <- function(.data, normalized = TRUE){
  .data <- manynet::expect_nodes(.data)
  thisRequires("sna")
  out <- sna::infocent(manynet::as_network(.data),
                       gmode = ifelse(manynet::is_directed(.data), "digraph", "graph"),
                       diag = manynet::is_complex(.data),
                       rescale = normalized)
  make_node_measure(out, .data)
}

#' @rdname measure_central_close
#' @section Eccentricity centrality: 
#'   Eccentricity centrality, graph centrality, or the Koenig number,
#'   is the (if normalized, inverse of) the distance to the furthest node:
#'   \deqn{C_E(i) = \frac{1}{max_{j \in N} d(i,j)}}
#'   where the distance from \eqn{i} to \eqn{j} is \eqn{\infty} if unconnected.
#'   As such it is only well defined for connected networks.
#' @references
#' ## On eccentricity centrality
#'   Hage, Per, and Frank Harary. 1995.
#'   "Eccentricity and centrality in networks".
#'   _Social Networks_, 17(1): 57-63.
#'   \doi{10.1016/0378-8733(94)00248-9}
#' @export
node_by_eccentricity <- function(.data, normalized = TRUE){
  .data <- manynet::expect_nodes(.data)
  if(!manynet::is_connected(.data)) 
    manynet::snet_unavailable("Eccentricity centrality is only available for connected networks.")
  disties <- igraph::distances(as_igraph(.data))
  out <- apply(disties, 1, max)
  if(normalized) out <- 1/out
  make_node_measure(out, .data)
}

#   - `node_eccentricity()` measures nodes' eccentricity or Koenig number,
#   a measure of farness based on number of links needed to reach 
#   most distant node in the network.
# #' @rdname measure_holes 
# #' @importFrom igraph eccentricity
# #' @export
# cnode_eccentricity <- function(.data){
#  if(missing(.data)) {expect_nodes(); .data <- .G()}
#  out <- igraph::eccentricity(manynet::as_igraph(.data),
#                              mode = "out")
#  make_node_measure(out, .data)
# }

#' @rdname measure_central_close 
#' @param from,to Index or name of a node to calculate distances from or to.
#' @export
node_by_distance <- function(.data, from, to, normalized = TRUE){
  .data <- manynet::expect_nodes(.data)
  if(missing(from) && missing(to)) manynet::snet_abort("Either 'from' or 'to' must be specified.")
  if(!missing(from)) out <- igraph::distances(manynet::as_igraph(.data), v = from) else 
    if(!missing(to)) out <- igraph::distances(manynet::as_igraph(.data), to = to)
    if(normalized) out <- out/max(out)
    make_node_measure(out, .data)
}

#' @rdname measure_central_close 
#' @section Closeness vitality centrality: 
#'   The closeness vitality of a node is the change in the sum of all distances
#'   in a network, also known as the Wiener Index, when that node is removed.
#'   Note that the closeness vitality may be negative infinity if
#'   removing that node would disconnect the network.
#'   Formally:
#'   \deqn{C_V(i) = \sum_{j,k} d(j,k) - \sum_{j,k} d(j,k,G\ i)}
#'   where \eqn{d(j,k,G\ i)} is the distance between nodes \eqn{j} and \eqn{k}
#'   in the network with node \eqn{i} removed.
#' @references
#' ## On closeness vitality centrality
#'   Koschuetzki, Dirk, Katharina Lehmann, Leon Peeters, Stefan Richter,
#'   Dagmar Tenfelde-Podehl, and Oliver Zlotowski. 2005.
#'   "Centrality Indices", in
#'   Brandes, Ulrik, and Thomas Erlebach (eds.). 
#'   _Network Analysis: Methodological Foundations_. 
#'   Springer: Berlin, pp. 16-61.
#' @export
node_by_vitality <- function(.data, normalized = TRUE){
  .data <- manynet::expect_nodes(.data)
  .data <- manynet::as_igraph(.data)
  out <- vapply(manynet::snet_progress_nodes(.data), function(x){
    sum(igraph::distances(.data)) - 
      sum(igraph::distances(manynet::delete_nodes(.data, x)))
  }, FUN.VALUE = numeric(1))
  if(normalized) out <- out/max(out)
  make_node_measure(out, .data)
}

#' @rdname measure_central_close 
#' @section Random walk closeness centrality: 
#'   Random walk closeness centrality is based on the average length of
#'   random walks starting at all other nodes to reach a given node.
#'   It is defined as the inverse of the average hitting time to a node.
#'   This means that higher values are given to nodes that can be reached
#'   more quickly on average by random walks starting at other nodes.
#'   Formally:
#'   \deqn{C_{RW}(i) = \frac{1}{\frac{1}{N-1} \sum_{j \neq i} H_{ji}}}
#'   where \eqn{H_{ji}} is the hitting time from node \eqn{j} to node \eqn{i}.
#' @references
#' ## On random walk closeness centrality
#'   Noh, J.D. and R. Rieger. 2004.
#'   "Random Walks on Complex Networks".
#'   _Physical Review Letters_, 92(11): 118701.
#'   \doi{10.1103/PhysRevLett.92.118701}
#' @export
node_by_randomwalk <- function(.data, normalized = TRUE){
  .data <- manynet::expect_nodes(.data)
  # adjacency and degree matrices
  A <- manynet::as_matrix(manynet::to_multilevel(.data))
  degs <- node_by_deg(.data)
  D <- diag(degs)
  
  # Laplacian
  L <- D - A
  
  # pseudoinverse of Laplacian
  Lplus <- .ginv(L)
  
  volG <- sum(degs)  # total degree (2 * edges)
  n <- manynet::net_nodes(.data)
  
  out <- numeric(n)
  
  for (i in 1:n) {
    # hitting time from j to i
    hitting_times <- sapply(1:n, function(j) {
      if (i == j) return(0)
      volG * (Lplus[j, j] + Lplus[i, i] - 2 * Lplus[i, j])
    })
    # average hitting time to node i
    avg_ht <- mean(hitting_times[-i])
    out[i] <- 1 / avg_ht
  }
  
  make_node_measure(out, .data)
}

# This is a helper function to compute the Moore-Penrose generalized inverse
# of a matrix using its singular value decomposition (SVD).
.ginv <- function (X, tol = sqrt(.Machine$double.eps)){
  if (length(dim(X)) > 2L || !(is.numeric(X) || is.complex(X))) 
    stop("'X' must be a numeric or complex matrix")
  if (!is.matrix(X)) 
    X <- as.matrix(X)
  Xsvd <- svd(X)
  if (is.complex(X)) 
    Xsvd$u <- Conj(Xsvd$u)
  Positive <- Xsvd$d > max(tol * Xsvd$d[1L], 0)
  if (all(Positive)) 
    Xsvd$v %*% (1/Xsvd$d * t(Xsvd$u))
  else if (!any(Positive)) 
    array(0, dim(X)[2L:1L])
  else Xsvd$v[, Positive, drop = FALSE] %*% ((1/Xsvd$d[Positive]) * 
                                               t(Xsvd$u[, Positive, drop = FALSE]))
}

# Tie closeness centrality ####

#' Measuring ties closeness-like centrality
#' @name measure_centralities_close
#' @description
#'   `tie_by_closeness()` measures the closeness of each tie to other ties 
#'   in the network.
#'   
#'   All measures attempt to use as much information as they are offered,
#'   including whether the networks are directed, weighted, or multimodal.
#'   If this would produce unintended results, 
#'   first transform the salient properties using e.g. [to_undirected()] functions.
#'   All centrality and centralization measures return normalized measures by default,
#'   including for two-mode networks.
#' @template param_data
#' @template param_norm
#' @family closeness
#' @family centrality
#' @template tie_measure
NULL

#' @rdname measure_centralities_close 
#' @examples
#' (ec <- tie_by_closeness(ison_adolescents))
#' ison_adolescents |> mutate_ties(weight = ec)
#' @export
tie_by_closeness <- function(.data, normalized = TRUE){
  .data <- manynet::expect_ties(.data)
  edge_adj <- manynet::to_ties(.data)
  out <- node_by_closeness(edge_adj, normalized = normalized)
  class(out) <- "numeric"
  make_tie_measure(out, .data)
}

# Closeness centralisation ####

#' Measuring networks closeness-like centralisation
#' @name measure_centralisation_close
#' @description
#'   - `net_by_closeness()` measures a network's closeness centralization as a
#'   single score.
#'   - `mode_by_closeness()` measures closeness centralization separately for each
#'   mode of a two-mode network, returning one score per mode
#'   (following Borgatti and Everett, 1997).
#'   - `net_by_reach()` measures a network's reach centralization.
#'   - `net_by_harmonic()` measures a network's harmonic centralization.
#'
#'   All measures attempt to use as much information as they are offered,
#'   including whether the networks are directed, weighted, or multimodal.
#'   If this would produce unintended results,
#'   first transform the salient properties using e.g. [to_undirected()] functions.
#'   All centrality and centralization measures return normalized measures by default,
#'   including for two-mode networks.
#'
#'   For two-mode networks the two modes have different theoretical maxima, so
#'   `net_by_closeness()` reports a single network-level score by applying
#'   Freeman's general centralization index over the normalized node closeness
#'   scores, whereas `mode_by_closeness()` reports the per-mode scores directly.
#' @template param_data
#' @template param_norm
#' @template param_dir
#' @family closeness
#' @family centrality
#' @references
#'   Borgatti, Stephen P., and Martin G. Everett. 1997.
#'   "Network analysis of 2-mode data."
#'   _Social Networks_ 19(3): 243-269.
#'   \doi{10.1016/S0378-8733(96)00301-2}
#' @param cutoff The maximum path length to consider when calculating betweenness.
#'   If negative or NULL (the default), there's no limit to the path lengths considered.
#' @returns
#'   `net_by_*()` functions return a `network_measure` scalar;
#'   `mode_by_closeness()` returns a `mode_measure` numeric vector of length two,
#'   giving one centralization score per mode.
NULL

#' @rdname measure_centralisation_close
#' @examples
#' net_by_closeness(ison_southern_women, direction = "in")
#' @export
net_by_closeness <- function(.data, normalized = TRUE,
                             direction = c("all", "out", "in")){

  .data <- manynet::expect_nodes(.data)
  direction <- match.arg(direction)
  graph <- manynet::as_igraph(.data)

  if (manynet::is_twomode(.data)) {
    # Two-mode closeness centralization is intrinsically per mode
    # (see `mode_by_closeness()`, following Borgatti and Everett, 1997).
    # For a single network-level score we apply Freeman's general
    # centralization index over the whole node set, using the normalized node
    # closeness scores (each in [0, 1]); the numerator's maximum is (n - 1).
    nc <- node_by_closeness(graph, normalized = TRUE, direction = direction)
    out <- sum(max(nc) - nc) / (length(nc) - 1)
  } else {
    out <- igraph::centr_clo(graph = graph,
                             mode = direction,
                             normalized = normalized)$centralization
  }
  out <- make_network_measure(out, .data, call = deparse(sys.call()))
  out
}

#' @rdname measure_centralisation_close
#' @examples
#' mode_by_closeness(ison_southern_women, direction = "in")
#' @export
mode_by_closeness <- function(.data, normalized = TRUE,
                              direction = c("all", "out", "in")){

  .data <- manynet::expect_nodes(.data)
  direction <- match.arg(direction)
  graph <- manynet::as_igraph(.data)

  if (!manynet::is_twomode(.data))
    manynet::snet_abort("`mode_by_closeness()` is only defined for two-mode networks; use `net_by_closeness()` for one-mode networks.")

  {
    clcent <- node_by_closeness(graph, normalized = TRUE, direction = direction)
    mode <- igraph::V(graph)$type
    mode1 <- length(mode) - sum(mode)
    mode2 <- sum(mode)
    out <- list()
    if (direction == "in") {
      out$nodes1 <- sum(max(clcent[!mode]) - clcent[!mode])/(((mode1 - 2)*(mode1 - 1))/(2 * mode1 - 3))
      out$nodes2 <- sum(max(clcent[mode]) - clcent[mode])/(((mode2 - 2)*(mode2 - 1))/(2 * mode2 - 3))
      if (mode1 > mode2) { #28.43
        lhs <- ((mode2 - 1)*(mode1 - 2) / (2 * mode1 - 3))
        rhs <- ((mode2 - 1)*(mode1 - mode2) / (mode1 + mode2 - 2))
        out$nodes1 <- sum(max(clcent[!mode]) - clcent[!mode])/( lhs +  rhs) # 0.2135
      }
      if (mode2 > mode1) {
        lhs <- ((mode1 - 1)*(mode2 - 2) / (2 * mode2 - 3))
        rhs <- ((mode1 - 1)*(mode2 - mode1) / (mode2 + mode1 - 2))
        out$nodes2 <- sum(max(clcent[mode]) - clcent[mode])/( lhs +  rhs)
      }
    } else {
      term1 <- 2*(mode1 - 1) * (mode2 + mode1 - 4)/(3*mode2 + 4*mode1 - 8)
      term2 <- 2*(mode1 - 1) * (mode1 - 2)/(2*mode2 + 3*mode1 - 6)
      term3 <- 2*(mode1 - 1) * (mode2 - mode1 + 1)/(2*mode2 + 3*mode1 - 4)
      out$nodes1 <- sum(max(clcent[!mode]) - clcent) / sum(term1, term2, term3)
      term1 <- 2*(mode2 - 1) * (mode1 + mode2 - 4)/(3*mode1 + 4*mode2 - 8)
      term2 <- 2*(mode2 - 1) * (mode2 - 2)/(2*mode1 + 3*mode2 - 6)
      term3 <- 2*(mode2 - 1) * (mode1 - mode2 + 1)/(2*mode1 + 3*mode2 - 4)
      out$nodes2 <- sum(max(clcent[mode]) - clcent) / sum(term1, term2, term3)
      
      if (mode1 > mode2) {
        term1 <- 2*(mode2 - 1) * (mode2 + mode1 - 2) / (3 * mode2 + 4 * mode1 - 8)
        term2 <- 2*(mode1 - mode2) * (2 * mode2 - 1) / (5 * mode2 + 2 * mode1 - 6)
        term3 <- 2*(mode2 - 1) * (mode1 - 2) / (2 * mode2 + 3 * mode1 - 6)
        term4 <- 2 * (mode2 - 1) / (mode1 + 4 * mode2 - 4)
        out$nodes1 <- sum(max(clcent[!mode]) - clcent) / sum(term1, term2, term3, term4)
      }
      if (mode2 > mode1) {
        term1 <- 2*(mode1 - 1) * (mode1 + mode2 - 2) / (3 * mode1 + 4 * mode2 - 8)
        term2 <- 2*(mode2 - mode1) * (2 * mode1 - 1) / (5 * mode1 + 2 * mode2 - 6)
        term3 <- 2*(mode1 - 1) * (mode2 - 2) / (2 * mode1 + 3 * mode2 - 6)
        term4 <- 2 * (mode1 - 1) / (mode2 + 4 * mode1 - 4)
        out$nodes2 <- sum(max(clcent[mode]) - clcent) / sum(term1, term2, term3, term4)
      }
    }
    out <- c("Mode 1" = out$nodes1, "Mode 2" = out$nodes2)
  }
  out <- make_mode_measure(out, .data, call = deparse(sys.call()))
  out
}

#' @rdname measure_centralisation_close
#' @export
net_by_reach <- function(.data, normalized = TRUE, cutoff = 2){
  .data <- manynet::expect_nodes(.data)
  reaches <- node_by_reach(.data, normalized = FALSE, cutoff = cutoff)
  out <- sum(max(reaches) - reaches)
  if(normalized) out <- out / sum(manynet::net_nodes(.data) - reaches)
  make_network_measure(out, .data, call = deparse(sys.call()))
}

#' @rdname measure_centralisation_close
#' @export
net_by_harmonic <- function(.data, normalized = TRUE, cutoff = 2){
  .data <- manynet::expect_nodes(.data)
  harm <- node_by_harmonic(.data, normalized = FALSE, cutoff = cutoff)
  out <- sum(max(harm) - harm)
  if(normalized) out <- out / sum(manynet::net_nodes(.data) - harm)
  make_network_measure(out, .data, call = deparse(sys.call()))
}

# Network closure ####

#' Measuring network closure
#' @name measure_closure
#' @description
#'   These functions offer methods for summarising the closure in configurations 
#'   in one-, two-, and three-mode networks:
#'   
#'   - `net_by_reciprocity()` measures reciprocity in a (usually directed) network.
#'   - `net_by_transitivity()` measures transitivity in a network.
#'   - `net_by_equivalency()` measures equivalence or reinforcement 
#'   in a (usually two-mode) network.
#'   - `net_by_congruency()` measures congruency across two two-mode networks.
#'   
#' @details 
#' For one-mode networks, shallow wrappers of igraph versions exist via 
#' `net_reciprocity` and `net_transitivity`.
#' 
#' For two-mode networks, `net_equivalency` calculates the proportion of three-paths in the network
#' that are closed by fourth tie to establish a "shared four-cycle" structure.
#' 
#' For three-mode networks, `net_congruency` calculates the proportion of three-paths 
#' spanning two two-mode networks that are closed by a fourth tie to establish a 
#' "congruent four-cycle" structure.
#' @template param_data
#' @template net_measure
#' @param object2 Optionally, a second (two-mode) matrix, igraph, or tidygraph
#' @param method For reciprocity, either `default` or `ratio`.
#'   See `?igraph::reciprocity`
NULL

#' @rdname measure_closure 
#' @importFrom igraph reciprocity
#' @examples
#' net_by_reciprocity(ison_southern_women)
#' @export
net_by_reciprocity <- function(.data, method = "default") {
  .data <- manynet::expect_nodes(.data)
  make_network_measure(igraph::reciprocity(manynet::as_igraph(.data), mode = method), 
                       .data, call = deparse(sys.call()))
}

#' @rdname measure_closure 
#' @importFrom igraph transitivity
#' @examples
#' net_by_transitivity(ison_adolescents)
#' @export
net_by_transitivity <- function(.data) {
  .data <- manynet::expect_nodes(.data)
  make_network_measure(igraph::transitivity(manynet::as_igraph(.data)), 
                       .data, call = deparse(sys.call()))
}

#' @rdname measure_closure
#' @section Equivalency: 
#'   The `net_by_equivalency()` function calculates the Robins and Alexander (2004) 
#'   clustering coefficient for two-mode networks.
#'   Note that for weighted two-mode networks, the result is divided by the average tie weight.
#' @references 
#' ## On equivalency or four-cycles
#' Robins, Garry L, and Malcolm Alexander. 2004. 
#' Small worlds among interlocking directors: Network structure and distance in bipartite graphs. 
#' \emph{Computational & Mathematical Organization Theory} 10(1): 69–94.
#' \doi{10.1023/B:CMOT.0000032580.12184.c0}.
#' @examples
#' net_by_equivalency(ison_southern_women)
#' @export
net_by_equivalency <- function(.data) {
  .data <- manynet::expect_nodes(.data)
  if(manynet::is_twomode(.data)){
    mat <- manynet::as_matrix(.data)
    c <- ncol(mat)
    indegrees <- colSums(mat)
    twopaths <- crossprod(mat)
    diag(twopaths) <- 0
    out <- sum(twopaths * (twopaths - 1)) /
      (sum(twopaths * (twopaths - 1)) +
         sum(twopaths *
               (matrix(indegrees, c, c) - twopaths)))
    if (is.nan(out)) out <- 1
    if(manynet::is_weighted(.data)) out <- out / mean(mat[mat>0])
  } else {
    out <- rowSums(vapply(manynet::snet_progress_nodes(.data), function(i){
      threepaths <- igraph::all_simple_paths(.data, i, cutoff = 3,
                                             mode = "all")
      onepaths <- threepaths[vapply(threepaths, length,
                                    FUN.VALUE = numeric(1))==2]
      threepaths <- threepaths[vapply(threepaths, length,
                                      FUN.VALUE = numeric(1))==4]
      c(sum(sapply(threepaths,"[[",4) %in% sapply(onepaths,"[[",2)),
        length(threepaths))
    }, FUN.VALUE = numeric(2)))
    out <- out[1]/out[2]
  }
  make_network_measure(out, .data, call = deparse(sys.call()))
}

#' @rdname measure_closure 
#' @references 
#' ## On congruency
#' Knoke, David, Mario Diani, James Hollway, and Dimitris C Christopoulos. 2021. 
#' \emph{Multimodal Political Networks}. 
#' Cambridge University Press. Cambridge University Press.
#' \doi{10.1017/9781108985000}
#' @export
net_by_congruency <- function(.data, object2){
  .data <- manynet::expect_nodes(.data)
  if(missing(.data) | missing(object2)) 
    manynet::snet_abort("This function expects two two-mode networks")
  if(!manynet::is_twomode(.data) | !manynet::is_twomode(object2)) 
    manynet::snet_abort("This function expects two two-mode networks")
  if(manynet::net_dims(.data)[2] != manynet::net_dims(object2)[1]) 
    manynet::snet_abort(paste("This function expects the number of nodes",
                              "in the second mode of the first network", "to be the same as the number of nodes",
                              "in the first mode of the second network."))
  mat1 <- manynet::as_matrix(.data)
  mat2 <- manynet::as_matrix(object2)
  connects <- ncol(mat1)
  twopaths1 <- crossprod(mat1)
  indegrees <- diag(twopaths1)
  diag(twopaths1) <- 0
  twopaths2 <- tcrossprod(mat2)
  outdegrees <- diag(twopaths2)
  diag(twopaths2) <- 0
  twopaths <- twopaths1 + twopaths2
  degrees <- indegrees + outdegrees
  output <- sum(twopaths * (twopaths - 1)) /
    (sum(twopaths * (twopaths - 1)) +
       sum(twopaths *
             (matrix(degrees, connects, connects) - twopaths)))
  if (is.nan(output)) output <- 1
  make_network_measure(output, .data, call = deparse(sys.call()))
}

# Nodal closure ####

#' Measuring node closure
#' @name measure_closure_node
#' @description
#'   These functions offer methods for summarising the closure in configurations 
#'   in one- and two-mode networks:
#'   
#'   - `node_by_reciprocity()` measures nodes' reciprocity.
#'   - `node_by_transitivity()` measures nodes' transitivity.
#'   - `node_by_equivalency()` measures nodes' equivalence or reinforcement 
#'   in a (usually two-mode) network.
#'   
#' @details 
#' For one-mode networks, shallow wrappers of igraph versions exist via 
#' `node_by_reciprocity` and `node_by_transitivity`.
#' 
#' For two-mode networks, `node_by_equivalency` calculates the proportion of three-paths in the network
#' that are closed by fourth tie to establish a "shared four-cycle" structure.
#' @template param_data
#' @template node_measure
NULL

#' @rdname measure_closure_node 
#' @examples
#' node_by_reciprocity(to_unweighted(ison_networkers))
#' @export
node_by_reciprocity <- function(.data) {
  .data <- manynet::expect_nodes(.data)
  out <- manynet::as_matrix(.data)
  make_node_measure(rowSums(out * t(out))/rowSums(out), 
                    .data)
}

#' @rdname measure_closure_node 
#' @examples
#' node_by_transitivity(ison_adolescents)
#' @export
node_by_transitivity <- function(.data) {
  .data <- manynet::expect_nodes(.data)
  make_node_measure(igraph::transitivity(manynet::as_igraph(.data), 
                                         type = "local"), 
                    .data)
}

#' @rdname measure_closure_node
#' @export
node_by_equivalency <- function(.data) {
  .data <- manynet::expect_nodes(.data)
  # if(is_weighted(.data))
  #   snet_info("Using unweighted form of the network.")
  out <- vapply(manynet::snet_progress_seq(.data), function(i){
    threepaths <- igraph::all_simple_paths(.data, i, cutoff = 3,
                                          mode = "all")
    onepaths <- threepaths[vapply(threepaths, length, 
                                  FUN.VALUE = numeric(1))==2]
    threepaths <- threepaths[vapply(threepaths, length, 
                                    FUN.VALUE = numeric(1))==4]
    mean(sapply(threepaths,"[[",4) %in% sapply(onepaths,"[[",2))
  }, FUN.VALUE = numeric(1))
  if (any(is.nan(out))) out[is.nan(out)] <- 0
  make_node_measure(out, .data)
}

test_that("network density works", {
  expect_equal(as.numeric(net_by_density(create_empty(10))), 0)
  expect_equal(as.numeric(net_by_density(create_empty(c(10,6)))), 0)
  expect_equal(as.numeric(net_by_density(create_filled(10))), 1)
  expect_equal(as.numeric(net_by_density(create_filled(c(10,6)))), 1)
  expect_output(print(net_by_density(create_filled(10))))
})

test_that("one-mode object clustering is reported correctly",{
  expect_equal(as.numeric(net_by_transitivity(ison_algebra)),
               0.69787, tolerance = 0.001)
})

test_that("two-mode object clustering is reported correctly",{
  expect_equal(as.numeric(net_by_equivalency(ison_southern_women)),
               0.4677, tolerance = 0.001)
  expect_values(net_by_equivalency(ison_adolescents), 0.258)
})

test_that("three-mode clustering calculated correctly",{
  mat1 <- manynet::create_ring(c(10,5))
  mat2 <- manynet::create_ring(c(5,8))
  expect_equal(as.numeric(net_by_congruency(mat1, mat2)),
               0.3684, tolerance = 0.001)
})

# Community clustering ####

#' Memberships in communities
#' @name member_community
#' @description
#'   `node_in_community()` runs through all available community detection algorithms 
#'   for a given network type, finds the algorithm that returns the
#'   largest modularity score, and returns the corresponding membership
#'   partition.
#'   Where feasible (a small enough network), the optimal problem solving
#'   technique is used to ensure the maximal modularity partition.
#'   For larger networks, it identifies the applicable algorithms and 
#'   finds the algorithm that maximises modularity and 
#'   returns that membership vector.
#'   
#' @template param_data
#' @family community
#' @template node_member
NULL

#' @rdname member_community
#' @export
node_in_community <- function(.data){
  .data <- manynet::expect_nodes(.data)
  if(manynet::net_nodes(.data)<100){
    # don't use node_in_betweenness because slow and poorer quality to optimal
    manynet::snet_success("{.fn node_in_optimal} available and", 
                          "will return the highest modularity partition.")
    netrics::node_in_optimal(.data)
  } else {
    manynet::snet_info("Excluding {.fn node_in_optimal} because network rather large.")
    poss_algs <- c("node_in_infomap",
                   "node_in_spinglass",
                   "node_in_fluid",
                   "node_in_louvain",
                   "node_in_leiden",
                   "node_in_greedy",
                   "node_in_eigen",
                   "node_in_walktrap")
    if(!manynet::is_connected(.data)){
      notforconnected <- c("node_in_spinglass", 
                           "node_in_fluid")
      manynet::snet_info("Excluding {.fn {notforconnected}} because network unconnected.")
      poss_algs <- setdiff(poss_algs, notforconnected)
    }
    if(manynet::is_directed(.data)){
      notfordirected <- c("node_in_louvain", 
                          "node_in_leiden",
                          "node_in_eigen")
      manynet::snet_info("Excluding {.fn {notfordirected}} because network directed.")
      poss_algs <- setdiff(poss_algs, notfordirected)
    }
    manynet::snet_info("Considering each of {.fn {poss_algs}}.")
    candidates <- lapply(manynet::snet_progress_along(poss_algs), function(comm){
      memb <- get(poss_algs[comm])(.data)
      mod <- net_by_modularity(.data, memb)
      list(memb, mod)
    })
    mods <- unlist(sapply(candidates, "[", 2))
    maxmod <- which.max(mods)
    manynet::snet_success("{.fn {poss_algs[maxmod]}} returns the highest modularity ({round(mods[maxmod],3)}).")
    candidates[[maxmod]][[1]]
  }
}

# #' @rdname member_community_hier 
# #' @section Ensemble:
# #'   Ensemble-based community detection runs community detection
# #'   algorithms over multilayer or multiplex networks.
# #' @references
# #' ## On ensemble-based community detection
# #' Tagarelli, Andrea, Alessia Amelio, and Francesco Gullo. 2017.
# #' "Ensemble-based Community Detection in Multilayer Networks".
# #' _Data Mining and Knowledge Discovery_, 31: 1506-1543.
# #' \doi{10.1007/s10618-017-0528-8}
# #' @examples
# #' node_in_ensemble(ison_adolescents)
# #' @export
# node_in_ensemble <- function(.data, linkage_constraint = TRUE){
#   if(missing(.data)) {expect_nodes(); .data <- .G()}
#   clust <- igraph::cluster_walktrap(manynet::as_igraph(.data))
#   out <- clust$membership
#   make_node_member(out, .data)
#   out <- make_node_member(out, .data)
#   attr(out, "hc") <- stats::as.hclust(clust, 
#                                       use.modularity = igraph::is_connected(.data))
#   attr(out, "k") <- max(clust$membership)
#   out
# }

# Non-hierarchical community clustering ####

#' Memberships in non-hierarchical communities
#' @name member_community_non
#' @description
#'   These functions offer algorithms for partitioning
#'   networks into sets of communities:
#' 
#'   - `node_in_optimal()` is a problem-solving algorithm that seeks to maximise 
#'   modularity over all possible partitions.
#'   - `node_in_partition()` is a greedy, iterative, deterministic
#'   partitioning algorithm that results in two equally-sized communities.
#'   - `node_in_infomap()` is an algorithm based on the information in random walks.
#'   - `node_in_spinglass()` is a greedy, iterative, probabilistic algorithm, 
#'   based on analogy to model from statistical physics.
#'   - `node_in_fluid()` is a propogation-based partitioning algorithm,
#'   based on analogy to model from fluid dynamics.
#'   - `node_in_louvain()` is an agglomerative multilevel algorithm that seeks to maximise 
#'   modularity over all possible partitions.
#'   - `node_in_leiden()` is an agglomerative multilevel algorithm that seeks to maximise 
#'   the Constant Potts Model over all possible partitions.
#'  
#'   The different algorithms offer various advantages in terms of computation time,
#'   availability on different types of networks, ability to maximise modularity,
#'   and their logic or domain of inspiration.
#'   
#' @template param_data
#' @family community
#' @template node_member
NULL

#' @rdname member_community_non 
#' @section Optimal:
#'   The general idea is to calculate the modularity of all possible partitions,
#'   and choose the community structure that maximises this modularity measure.
#'   Note that this is an NP-complete problem with exponential time complexity.
#'   The guidance in the igraph package is networks of <50-200 nodes is probably fine.
#' @references
#' ## On optimal community detection
#' Brandes, Ulrik, Daniel Delling, Marco Gaertler, Robert Gorke, Martin Hoefer, Zoran Nikoloski, Dorothea Wagner. 2008.
#' "On Modularity Clustering", 
#' _IEEE Transactions on Knowledge and Data Engineering_ 20(2):172-188.
#' @examples
#' node_in_optimal(ison_adolescents)
#' @export
node_in_optimal <- function(.data){
  .data <- manynet::expect_nodes(.data)
  if(manynet::net_nodes(.data)>100) 
    manynet::snet_warn("This algorithm may take some time", 
    "or even run out of memory on such a large network.")
  out <- igraph::cluster_optimal(manynet::as_igraph(.data)
  )$membership
  make_node_member(out, .data)
}

#' @rdname member_community_non 
#' @section Partition:
#'   The general idea is to assign nodes to two groups, and then iteratively 
#'   swap pairs of nodes (one from each group) that give a positive sum of net tie costs,
#'   where the net tie cost of a node is the difference between the sum 
#'   of the weights of ties to nodes in the other group (external costs) and 
#'   the sum of the weights of ties to nodes in the same group (internal costs).
#'   This is a deterministic algorithm that will always return the same partition 
#'   for a given network, but it is not guaranteed to maximise modularity.
#'   Note that this algorithm is only applicable to undirected, unipartite networks, 
#'   and will always return two communities of equal size (or as close to equal as possible).
#' @references
#' ## On partitioning community detection
#' Kernighan, Brian W., and Shen Lin. 1970.
#' "An efficient heuristic procedure for partitioning graphs."
#' _The Bell System Technical Journal_ 49(2): 291-307.
#' \doi{10.1002/j.1538-7305.1970.tb01770.x}
#' @examples
#' node_in_partition(ison_adolescents)
#' node_in_partition(ison_southern_women)
#' @export
node_in_partition <- function(.data){
  .data <- manynet::expect_nodes(.data)
  # assign groups arbitrarily
  n <- manynet::net_nodes(.data)
  group_size <- ifelse(n %% 2 == 0, n/2, (n+1)/2)
  
  # count internal and external costs of each node
  g <- manynet::as_matrix(manynet::to_multilevel(.data))
  g1 <- g[1:group_size, 1:group_size]
  g2 <- g[(group_size+1):n, (group_size+1):n]
  intergroup <- g[1:group_size, (group_size+1):n]
  
  g2.intcosts <- rowSums(g2)
  g2.extcosts <- colSums(intergroup)
  
  g1.intcosts <- rowSums(g1)
  g1.extcosts <- rowSums(intergroup)
  
  # count edge costs of each nodes
  g1.net <- g1.extcosts - g1.intcosts
  g2.net <- g2.extcosts - g2.intcosts
  
  g1.net <- sort(g1.net, decreasing = TRUE)
  g2.net <- sort(g2.net, decreasing = TRUE)
  
  # swap pairs of nodes (one from each group) that give a positive sum of net tie costs
  if(length(g1.net)!=length(g2.net)) {
    g2.net <- c(g2.net,0)
  } else {g2.net}
  
  sums <- as.integer(unname(g1.net + g2.net))
  # positions in sequence of names at which sum >= 0
  index <- which(sums >= 0 %in% sums)
  g1.newnames <- g1.names <- names(g1.net)
  g2.newnames <- g2.names <- names(g2.net)
  # make swaps based on positions in sequence
  for (i in index) {
    g1.newnames[i] <- g2.names[i]
    g2.newnames[i] <- g1.names[i]
  }
  
  # extract names of vertices in each group after swaps
  out <- ifelse(manynet::node_names(.data) %in% g1.newnames, 1, 2)
  make_node_member(out, .data)
}

#' @rdname member_community_non 
#' @section Infomap:
#'   Motivated by information theoretic principles, this algorithm tries to build 
#'   a grouping that provides the shortest description length for a random walk,
#'   where the description length is measured by the expected number of bits 
#'   per node required to encode the path.
#' @param times Integer indicating number of simulations/walks used.
#'   By default, `times=50`.
#' @references
#' ## On infomap community detection
#' Rosvall, M, and C. T. Bergstrom. 2008.
#' "Maps of information flow reveal community structure in complex networks", 
#' _PNAS_ 105:1118.
#' \doi{10.1073/pnas.0706851105}
#' 
#' Rosvall, M., D. Axelsson, and C. T. Bergstrom. 2009.
#' "The map equation", 
#' _Eur. Phys. J. Special Topics_ 178: 13. 
#' \doi{10.1140/epjst/e2010-01179-1}
#' @examples
#' node_in_infomap(ison_adolescents)
#' @export
node_in_infomap <- function(.data, times = 50){
  .data <- manynet::expect_nodes(.data)
  out <- igraph::cluster_infomap(manynet::as_igraph(.data), 
                                 nb.trials = times
  )$membership
  make_node_member(out, .data)
}

#' @rdname member_community_non 
#' @param max_k Integer constant, the number of spins to use as an upper limit
#'   of communities to be found. Some sets can be empty at the end.
#' @param resolution The Reichardt-Bornholdt “gamma” resolution parameter for modularity.
#'   By default 1, making existing and non-existing ties equally important.
#'   Smaller values make existing ties more important,
#'   and larger values make missing ties more important.
#' @section Spin-glass:
#'   This is motivated by analogy to the Potts model in statistical physics.
#'   Each node can be in one of _k_ "spin states",
#'   and ties (particle interactions) provide information about which pairs of nodes 
#'   want similar or different spin states.
#'   The final community definitions are represented by the nodes' spin states
#'   after a number of updates.
#'   A different implementation than the default is used in the case of signed networks,
#'   such that nodes connected by negative ties will be more likely found in separate communities.
#' @references
#' ## On spinglass community detection
#' Reichardt, Jorg, and Stefan Bornholdt. 2006.
#' "Statistical Mechanics of Community Detection"
#' _Physical Review E_, 74(1): 016110–14.
#' \doi{10.1073/pnas.0605965104}
#' 
#' Traag, Vincent A., and Jeroen Bruggeman. 2009.
#' "Community detection in networks with positive and negative links".
#' _Physical Review E_, 80(3): 036115.
#' \doi{10.1103/PhysRevE.80.036115}
#' @examples
#' node_in_spinglass(ison_adolescents)
#' @export
node_in_spinglass <- function(.data, max_k = 200, resolution = 1){
  .data <- manynet::expect_nodes(.data)
  if(!igraph::is_connected(.data)) # note manynet::is_connected will return false
    manynet::snet_unavailable("This algorithm only works for connected networks.",
                     "We suggest using `to_giant()`", 
                     "to select the largest component.") else {
      out <- igraph::cluster_spinglass(manynet::as_igraph(.data), 
                                       spins = max_k, gamma = resolution,
                                       implementation = ifelse(manynet::is_signed(.data), "neg", "orig")
      )$membership
      make_node_member(out, .data)
    }
}

#' @rdname member_community_non 
#' @section Fluid:
#'   The general idea is to observe how a discrete number of fluids interact, expand and contract, 
#'   in a non-homogenous environment, i.e. the network structure.
#'   Unlike the `{igraph}` implementation that this function wraps,
#'   this function iterates over all possible numbers of communities and returns the membership
#'   associated with the highest modularity.
#' @references
#' ## On fluid community detection
#' Parés Ferran, Dario Garcia Gasulla, Armand Vilalta, Jonatan Moreno, Eduard Ayguade, Jesus Labarta, Ulises Cortes, and Toyotaro Suzumura. 2018. 
#' "Fluid Communities: A Competitive, Scalable and Diverse Community Detection Algorithm". 
#' In: _Complex Networks & Their Applications VI_
#' Springer, 689: 229.
#' \doi{10.1007/978-3-319-72150-7_19}
#' @examples
#' node_in_fluid(ison_adolescents)
#' @export
node_in_fluid <- function(.data) {
  .data <- manynet::expect_nodes(.data)
  .data <- manynet::as_igraph(.data)
  if (!igraph::is_connected(.data)) {
    manynet::snet_unavailable("This algorithm only works for connected networks.",
                     "We suggest using `to_giant()`", 
                     "to select the largest component.")
  } else {
    if(manynet::is_complex(.data)){
      manynet::snet_info("This algorithm only works for simple networks.", 
                      "Converting to simplex.")
      .data <- manynet::to_simplex(.data)
    }
    if(manynet::is_directed(.data)){
      manynet::snet_info("This algorithm only works for undirected networks.", 
                      "Converting to undirected")
      .data <- manynet::to_undirected(.data)
    }
    mods <- vapply(seq_nodes(.data), function(x)
      igraph::modularity(.data, membership = igraph::membership(
        igraph::cluster_fluid_communities(.data, x))),
      FUN.VALUE = numeric(1))
    out <- igraph::membership(igraph::cluster_fluid_communities(
      .data, no.of.communities = which.max(mods)))
    make_node_member(out, .data)
  }
}

#' @rdname member_community_non 
#' @section Louvain:
#'   The general idea is to take a hierarchical approach to optimising the modularity criterion.
#'   Nodes begin in their own communities and are re-assigned in a local, greedy way:
#'   each node is moved to the community where it achieves the highest contribution to modularity.
#'   When no further modularity-increasing reassignments are possible, 
#'   the resulting communities are considered nodes (like a reduced graph),
#'   and the process continues.
#' @references
#' ## On Louvain community detection
#' Blondel, Vincent, Jean-Loup Guillaume, Renaud Lambiotte, Etienne Lefebvre. 2008.
#' "Fast unfolding of communities in large networks",
#' _J. Stat. Mech._ P10008.
#' @examples
#' node_in_louvain(ison_adolescents)
#' @export
node_in_louvain <- function(.data, resolution = 1){
  .data <- manynet::expect_nodes(.data)
  if(manynet::is_directed(.data)){
    manynet::snet_info("This algorithm only works for undirected networks.", 
              "Converting to undirected")
    .data <- manynet::to_undirected(.data)
  }
  out <- igraph::cluster_louvain(manynet::as_igraph(.data), 
                                resolution = resolution
  )$membership
  make_node_member(out, .data)
}

#' @rdname member_community_non 
#' @section Leiden:
#'   The general idea is to optimise the Constant Potts Model, 
#'   which does not suffer from the resolution limit, instead of modularity.
#'   As outlined in the `{igraph}` package, 
#'   the Constant Potts Model object function is:
#'   
#'   \deqn{\frac{1}{2m} \sum_{ij}(A_{ij}-\gamma n_i n_j)\delta(\sigma_i, \sigma_j)}
#'   
#'   where _m_ is the total tie weight, 
#'   \eqn{A_{ij}} is the tie weight between _i_ and _j_,
#'   \eqn{\gamma} is the so-called resolution parameter,
#'   \eqn{n_i} is the node weight of node _i_,
#'   and \eqn{\delta(\sigma_i, \sigma_j) = 1} if and only if
#'   _i_ and _j_ are in the same communities and 0 otherwise.
#'   Compared to the Louvain method, the Leiden algorithm additionally
#'   tries to avoid unconnected communities.
#' @references
#' ## On Leiden community detection
#' Traag, Vincent A., Ludo Waltman, and Nees Jan van Eck. 2019. 
#' "From Louvain to Leiden: guaranteeing well-connected communities", 
#' _Scientific Reports_, 9(1):5233. 
#' \doi{10.1038/s41598-019-41695-z}
#' @examples
#' node_in_leiden(ison_adolescents)
#' @export
node_in_leiden <- function(.data, resolution = 1){
  .data <- manynet::expect_nodes(.data)
  if(manynet::is_directed(.data)){
    manynet::snet_info("This algorithm only works for undirected networks.", 
              "Converting to undirected")
    .data <- manynet::to_undirected(.data)
  }
  if(manynet::is_weighted(.data)){ # Traag resolution default
    n <- manynet::net_nodes(.data)
    resolution <- sum(manynet::tie_weights(.data))/(n*(n - 1)/2)
  }
  out <- igraph::cluster_leiden(manynet::as_igraph(.data), 
                                resolution = resolution
  )$membership
  make_node_member(out, .data)
}

# Hierarchical community clustering ####

#' Memberships in hierarchical communities
#' @name member_community_hier
#' @description
#'   These functions offer algorithms for hierarchically clustering
#'   networks into communities. Since all of the following are hierarchical,
#'   their dendrograms can be plotted:
#' 
#'   - `node_in_betweenness()` is a hierarchical, decomposition algorithm
#'   where edges are removed in decreasing order of the number of
#'   shortest paths passing through the edge.
#'   - `node_in_greedy()` is a hierarchical, agglomerative algorithm, 
#'   that tries to optimize modularity in a greedy manner.
#'   - `node_in_eigen()` is a top-down, hierarchical algorithm.
#'   - `node_in_walktrap()` is a hierarchical, agglomerative algorithm based on random walks.
#'  
#'   The different algorithms offer various advantages in terms of computation time,
#'   availability on different types of networks, ability to maximise modularity,
#'   and their logic or domain of inspiration.
#'   
#' @template param_data
#' @template node_member
#' @family community
NULL

#' @rdname member_community_hier 
#' @section Edge-betweenness:
#'   This is motivated by the idea that edges connecting different groups 
#'   are more likely to lie on multiple shortest paths when they are the 
#'   only option to go from one group to another. 
#'   This method yields good results but is very slow because of 
#'   the computational complexity of edge-betweenness calculations and 
#'   the betweenness scores have to be re-calculated after every edge removal. 
#'   Networks of ~700 nodes and ~3500 ties are around the upper size limit 
#'   that are feasible with this approach. 
#' @references
#' ## On edge-betweenness community detection
#' Newman, Mark, and Michelle Girvan. 2004.
#' "Finding and evaluating community structure in networks." 
#' _Physical Review E_ 69: 026113.
#' \doi{10.1103/PhysRevE.69.026113}
#' @examples
#' node_in_betweenness(ison_adolescents)
#' @export
node_in_betweenness <- function(.data){
  .data <- manynet::expect_nodes(.data)
  if(manynet::net_nodes(.data)>100) 
    manynet::snet_warn("This algorithm may take some time", 
                                "or even run out of memory on such a large network.")
  clust <- suppressWarnings(igraph::cluster_edge_betweenness(
    manynet::as_igraph(.data)))
  out <- clust$membership
  out <- make_node_member(out, .data)
  attr(out, "hc") <- stats::as.hclust(clust, 
                                      use.modularity = igraph::is_connected(.data))
  attr(out, "k") <- max(clust$membership)
  out
}

#' @rdname member_community_hier 
#' @section Fast-greedy:
#'   Initially, each node is assigned a separate community.
#'   Communities are then merged iteratively such that each merge
#'   yields the largest increase in the current value of modularity,
#'   until no further increases to the modularity are possible.
#'   The method is fast and recommended as a first approximation 
#'   because it has no parameters to tune. 
#'   However, it is known to suffer from a resolution limit.
#' @references
#' ## On fast-greedy community detection
#' Clauset, Aaron, Mark E.J. Newman, and Cristopher Moore. 2004.
#' "Finding community structure in very large networks."
#' _Physical Review E_, 70: 066111.
#' \doi{10.1103/PhysRevE.70.066111}
#' @examples
#' node_in_greedy(ison_adolescents)
#' @export
node_in_greedy <- function(.data){
  .data <- manynet::expect_nodes(.data)
  clust <- igraph::cluster_fast_greedy(manynet::to_undirected(manynet::as_igraph(.data)))
  out <- clust$membership
  make_node_member(out, .data)
  out <- make_node_member(out, .data)
  attr(out, "hc") <- stats::as.hclust(clust, 
                                      use.modularity = igraph::is_connected(.data))
  attr(out, "k") <- max(clust$membership)
  out
}

#' @rdname member_community_hier 
#' @section Leading eigenvector:
#'   In each step, the network is bifurcated such that modularity increases most.
#'   The splits are determined according to the leading eigenvector of the modularity matrix.
#'   A stopping condition prevents tightly connected groups from being split further.
#'   Note that due to the eigenvector calculations involved,
#'   this algorithm will perform poorly on degenerate networks,
#'   but will likely obtain a higher modularity than fast-greedy (at some cost of speed).
#' @references
#' ## On leading eigenvector community detection
#' Newman, Mark E.J. 2006.
#' "Finding community structure using the eigenvectors of matrices"
#' _Physical Review E_ 74:036104.
#' \doi{10.1103/PhysRevE.74.036104}
#' @examples
#' node_in_eigen(ison_adolescents)
#' @export
node_in_eigen <- function(.data){
  .data <- manynet::expect_nodes(.data)
  if(manynet::is_directed(.data)){
    manynet::snet_info("This algorithm only works for undirected networks.", 
              "Converting to undirected")
    .data <- manynet::to_undirected(.data)
  }
  clust <- igraph::cluster_leading_eigen(as_igraph(.data))
  out <- clust$membership
  make_node_member(out, .data)
  out <- make_node_member(out, .data)
  attr(out, "hc") <- stats::as.hclust(clust)
  attr(out, "k") <- max(clust$membership)
  out
}

#' @rdname member_community_hier 
#' @section Walktrap:
#'   The general idea is that random walks on a network are more likely to stay 
#'   within the same community because few edges lead outside a community.
#'   By repeating random walks of 4 steps many times,
#'   information about the hierarchical merging of communities is collected.
#' @param times Integer indicating number of simulations/walks used.
#'   By default, `times=50`.
#' @references
#' ## On walktrap community detection
#' Pons, Pascal, and Matthieu Latapy. 2005.
#' "Computing communities in large networks using random walks".
#' 1-20.
#' \doi{10.48550/arXiv.physics/0512106}
#' @examples
#' node_in_walktrap(ison_adolescents)
#' @export
node_in_walktrap <- function(.data, times = 50){
  .data <- manynet::expect_nodes(.data)
  clust <- igraph::cluster_walktrap(manynet::as_igraph(.data))
  out <- clust$membership
  make_node_member(out, .data)
  out <- make_node_member(out, .data)
  attr(out, "hc") <- stats::as.hclust(clust, 
                                      use.modularity = igraph::is_connected(.data))
  attr(out, "k") <- max(clust$membership)
  out
}

test_that("node_kernighanlin algorithm works", {
  expect_s3_class(node_in_partition(ison_adolescents), "node_member")
  expect_length(node_in_partition(ison_adolescents), 
                net_nodes(ison_adolescents))
  expect_false(any(node_in_partition(ison_adolescents) > "B"))
})

test_that("node_edge_betweenness algorithm works", {
  expect_s3_class(node_in_betweenness(ison_adolescents), "node_member")
  expect_length(node_in_betweenness(ison_adolescents), 
                net_nodes(ison_adolescents))
})

test_that("node_fast_greedy algorithm works", {
  expect_s3_class(node_in_greedy(ison_southern_women), "node_member")
  expect_length(node_in_greedy(ison_southern_women), 
                net_nodes(ison_southern_women))
})

test_that("node_walktrap algorithm works", {
  expect_s3_class(node_in_walktrap(ison_southern_women), "node_member")
  expect_length(node_in_walktrap(ison_southern_women), 
                net_nodes(ison_southern_women))
})

test_that("node_in_community uses node_in_optimal on small networks", {
  skip_if(format(Sys.time(), "%H") >= "09")
  options(manynet_verbosity = "verbose")
  options(snet_verbosity = "verbose")
  expect_message(node_in_community(manynet::create_ring(10)), "optimal")
  expect_message(node_in_community(manynet::create_ring(200)), "xcluding")
  options(manynet_verbosity = "quiet")
  options(snet_verbosity = "quiet")
})

# Eigenvector-like centralities ####

#' Measuring nodes eigenvector-like centrality
#' @name measure_central_eigen
#' @description
#'   These functions calculate common eigenvector-related centrality 
#'   measures, or walk-based eigenmeasures, for one- and two-mode networks:
#'   
#'   - `node_by_eigenvector()` measures the eigenvector centrality of nodes 
#'   in a network.
#'   - `node_by_power()` measures the Bonacich, beta, or power centrality of 
#'   nodes in a network.
#'   - `node_by_alpha()` measures the alpha or Katz centrality of nodes in a 
#'   network.
#'   - `node_by_pagerank()` measures the pagerank centrality of nodes in a network.
#'   - `node_by_hub()` measures how well nodes in a network serve as hubs pointing 
#'   to many authorities.
#'   - `node_by_authority()` measures how well nodes in a network serve as 
#'   authorities from many hubs.
#'   
#'   All measures attempt to use as much information as they are offered,
#'   including whether the networks are directed, weighted, or multimodal.
#'   If this would produce unintended results, 
#'   first transform the salient properties using e.g. [to_undirected()] functions.
#'   All centrality and centralization measures return normalized measures 
#'   by default, including for two-mode networks.
#' @template param_data
#' @template param_norm
#' @family eigenvector
#' @family centrality
#' @template node_measure
NULL

#' @rdname measure_central_eigen
#' @section Eigenvector centrality:
#'   Eigenvector centrality operates as a measure of a node's influence in a network.
#'   The idea is that being connected to well-connected others results in a higher score.
#'   Each node's eigenvector centrality can be defined as:
#'   \deqn{x_i = \frac{1}{\lambda} \sum_{j \in N} a_{i,j} x_j}
#'   where \eqn{a_{i,j} = 1} if \eqn{i} is linked to \eqn{j} and 0 otherwise,
#'   and \eqn{\lambda} is a constant representing the principal eigenvalue.
#'   Rather than performing this iteration,
#'   most routines solve the eigenvector equation \eqn{Ax = \lambda x}.
#'   Note that since `{igraph}` v2.1.1,
#'   the values will always be rescaled so that the maximum is 1.
#' @param scale Logical scalar, whether to rescale the vector so the maximum score is 1. 
#' @details
#'   We use `{igraph}` routines behind the scenes here for consistency and because they are often faster.
#'   For example, `igraph::eigencentrality()` is approximately 25% faster than `sna::evcent()`.
#' @references 
#'   ## On eigenvector centrality
#'   Bonacich, Phillip. 1991. 
#'   “Simultaneous Group and Individual Centralities.” 
#'   _Social Networks_ 13(2):155–68. 
#'   \doi{10.1016/0378-8733(91)90018-O}
#' @examples
#' node_by_eigenvector(ison_southern_women)
#' @export 
node_by_eigenvector <- function(.data, normalized = TRUE, scale = TRUE){
  
  .data <- manynet::expect_nodes(.data)
  weights <- `if`(manynet::is_weighted(.data), 
                  manynet::tie_weights(.data), NA)
  graph <- manynet::as_igraph(.data)
  
  if(!normalized) manynet::snet_info("This function always returns a normalized value now.")
  if(!scale) manynet::snet_info("This function always returns a scaled value now.")
  
  if(!manynet::is_connected(.data)) 
    manynet::snet_warn("Unconnected networks will only allow nodes from one component to have non-zero eigenvector scores.")
  
  # Do the calculations
  if (!manynet::is_twomode(graph)){
    out <- igraph::eigen_centrality(graph = graph, 
                                    directed = manynet::is_directed(graph),
                                    options = igraph::arpack_defaults())$vector
  } else {
    eigen1 <- manynet::to_mode1(graph)
    eigen1 <- igraph::eigen_centrality(graph = eigen1, 
                                       directed = manynet::is_directed(eigen1),
                                       options = igraph::arpack_defaults())$vector
    eigen2 <- manynet::to_mode2(graph)
    eigen2 <- igraph::eigen_centrality(graph = eigen2, 
                                       directed = manynet::is_directed(eigen2),
                                       options = igraph::arpack_defaults())$vector
    out <- c(eigen1, eigen2)
  }
  out <- make_node_measure(out, .data)
  out
}

#' @rdname measure_central_eigen
#' @param exponent Decay rate or attentuation factor for 
#'   the Bonacich power centrality score.
#'   Can be positive or negative.
#' @section Power or beta (or Bonacich) centrality:
#'   Power centrality includes an exponent that weights contributions to a node's
#'   centrality based on how far away those other nodes are.
#'   \deqn{c_b(i) = \sum A(i,j) (\alpha = \beta c(j))}
#'   Where \eqn{\beta} is positive, this means being connected to central people
#'   increases centrality.
#'   Where \eqn{\beta} is negative, this means being connected to central people
#'   decreases centrality 
#'   (and being connected to more peripheral actors increases centrality).
#'   When \eqn{\beta = 0}, this is the outdegree.
#'   \eqn{\alpha} is calculated to make sure the root mean square equals 
#'   the network size.
#' @references 
#' ## On power centrality
#'   Bonacich, Phillip. 1987. 
#'   “Power and Centrality: A Family of Measures.” 
#'   _The American Journal of Sociology_, 92(5): 1170–82.
#' \doi{10.1086/228631}.
#' @importFrom igraph power_centrality
#' @examples
#' node_by_power(ison_southern_women, exponent = 0.5)
#' @export 
node_by_power <- function(.data, normalized = TRUE, scale = FALSE, exponent = 1){
  
  .data <- manynet::expect_nodes(.data)
  weights <- `if`(manynet::is_weighted(.data), 
                  manynet::tie_weights(.data), NA)
  graph <- manynet::as_igraph(.data)
  
  if(var(node_by_deg(graph))==0){
    manynet::snet_minor_info("All nodes have the same degree, so power centrality equals degree centrality.")
    exponent <- 0
  }
  
  # Do the calculations
  if (!manynet::is_twomode(graph)){
    out <- igraph::power_centrality(graph = graph, 
                                    exponent = exponent,
                                    rescale = scale)
    if (normalized) out <- out / sqrt(1/2)
  } else {
    eigen1 <- manynet::to_mode1(graph)
    eigen1 <- igraph::power_centrality(graph = eigen1, 
                                       exponent = exponent,
                                       rescale = scale)
    eigen2 <- manynet::to_mode2(graph)
    eigen2 <- igraph::power_centrality(graph = eigen2, 
                                       exponent = exponent,
                                       rescale = scale)
    out <- c(eigen1, eigen2)
    if (normalized) out <- out / sqrt(1/2)
  }
  out <- make_node_measure(out, .data)
  out
}

#' @rdname measure_central_eigen 
#' @param alpha A constant that trades off the importance of external influence against the importance of connection.
#'   When \eqn{\alpha = 0}, only the external influence matters.
#'   As \eqn{\alpha} gets larger, only the connectivity matters and we reduce to eigenvector centrality.
#'   By default \eqn{\alpha = 0.85}.
#' @section Alpha centrality:
#'   Alpha or Katz (or Katz-Bonacich) centrality operates better than 
#'   eigenvector centrality for directed networks because eigenvector centrality 
#'   will return 0s for all nodes not in the main strongly-connected component.
#'   Each node's alpha centrality can be defined as:
#'   \deqn{x_i = \frac{1}{\lambda} \sum_{j \in N} a_{i,j} x_j + e_i}
#'   where \eqn{a_{i,j} = 1} if \eqn{i} is linked to \eqn{j} and 0 otherwise,
#'   \eqn{\lambda} is a constant representing the principal eigenvalue,
#'   and \eqn{e_i} is some external influence used to ensure that even nodes beyond the main
#'   strongly connected component begin with some basic influence.
#'   Note that many equations replace \eqn{\frac{1}{\lambda}} with \eqn{\alpha},
#'   hence the name.
#'
#'   For example, if \eqn{\alpha = 0.5}, then each direct connection (or alter) would be worth \eqn{(0.5)^1 = 0.5},
#'   each secondary connection (or tertius) would be worth \eqn{(0.5)^2 = 0.25},
#'   each tertiary connection would be worth \eqn{(0.5)^3 = 0.125}, and so on.
#'
#'   Rather than performing this iteration though,
#'   most routines solve the equation \eqn{x = (I - \frac{1}{\lambda} A^T)^{-1} e}.
#' @importFrom igraph alpha_centrality
#' @references 
#' ## On alpha centrality
#'   Katz, Leo 1953. 
#'   "A new status index derived from sociometric analysis". 
#'   _Psychometrika_. 18(1): 39–43.
#' 
#'   Bonacich, P. and Lloyd, P. 2001. 
#'   “Eigenvector-like measures of centrality for asymmetric relations” 
#'   _Social Networks_. 23(3):191-201.
#' @export 
node_by_alpha <- function(.data, alpha = 0.85){
  .data <- manynet::expect_nodes(.data)
  make_node_measure(igraph::alpha_centrality(manynet::as_igraph(.data), 
                                             alpha = alpha),
                    .data)
}

#' @rdname measure_central_eigen 
#' @references 
#' ## On pagerank centrality
#'   Brin, Sergey and Page, Larry. 1998.
#'   "The anatomy of a large-scale hypertextual web search engine".
#'   _Proceedings of the 7th World-Wide Web Conference_. Brisbane, Australia.
#' @export 
node_by_pagerank <- function(.data){
  .data <- manynet::expect_nodes(.data)
  make_node_measure(igraph::page_rank(manynet::as_igraph(.data))$vector,
                    .data)
}

#' @rdname measure_central_eigen 
#' @references 
#' ## On hub and authority centrality
#'   Kleinberg, Jon. 1999.
#'   "Authoritative sources in a hyperlinked environment". 
#'   _Journal of the ACM_ 46(5): 604–632.
#'   \doi{10.1145/324133.324140}
#' @export 
node_by_authority <- function(.data){
  .data <- manynet::expect_nodes(.data)
  make_node_measure(igraph::hits_scores(manynet::as_igraph(.data))$authority,
                    .data)
}

#' @rdname measure_central_eigen 
#' @export 
node_by_hub <- function(.data){
  .data <- manynet::expect_nodes(.data)
  make_node_measure(igraph::hits_scores(manynet::as_igraph(.data))$hub,
                    .data)
}

#' @rdname measure_central_eigen 
#' @section Subgraph centrality:
#'   Subgraph centrality measures the participation of a node in all subgraphs
#'   in the network, giving higher weight to smaller subgraphs.
#'   It is defined as:
#'   \deqn{C_S(i) = \sum_{k=0}^{\infty} \frac{(A^k)_{ii}}{k!}}
#'   where \eqn{(A^k)_{ii}} is the \eqn{i}th diagonal element of the \eqn{k}th power
#'   of the adjacency matrix \eqn{A}, representing the number of closed walks
#'   of length \eqn{k} starting and ending at node \eqn{i}.
#'   Weighting by \eqn{\frac{1}{k!}} ensures that shorter walks contribute more
#'   to the centrality score than longer walks.
#'   
#'   Subgraph centrality is a good choice of measure when the focus is on
#'   local connectivity and clustering around a node,
#'   as it captures the extent to which a node is embedded in tightly-knit
#'   groups within the network.
#'   Note though that because of the way spectral decomposition is used to
#'   calculate this measure, this is not a good measure for very large graphs.
#' @references
#' ## On subgraph centrality
#'   Estrada, Ernesto and Rodríguez-Velázquez, Juan A. 2005.
#'   "Subgraph centrality in complex networks".
#'   _Physical Review E_ 71(5): 056103.
#'   \doi{10.1103/PhysRevE.71.056103}
#' @export 
node_by_subgraph <- function(.data){
  .data <- manynet::expect_nodes(.data)
  make_node_measure(igraph::subgraph_centrality(manynet::as_igraph(.data)),
                    .data)
}

# Eigenvector-like centralities ####

#' Measuring ties eigenvector-like centrality
#' @name measure_centralities_eigen
#' @description
#'   `tie_by_eigenvector()` measures the eigenvector centrality of ties in a 
#'   network.
#'   
#'   All measures attempt to use as much information as they are offered,
#'   including whether the networks are directed, weighted, or multimodal.
#'   If this would produce unintended results, 
#'   first transform the salient properties using e.g. [to_undirected()] functions.
#'   All centrality and centralization measures return normalized measures 
#'   by default, including for two-mode networks.
#' @template param_data
#' @template param_norm
#' @family eigenvector
#' @family centrality
#' @template tie_measure
NULL

#' @rdname measure_centralities_eigen
#' @examples 
#' tie_by_eigenvector(ison_adolescents)
#' @export
tie_by_eigenvector <- function(.data, normalized = TRUE){
  .data <- manynet::expect_ties(.data)
  edge_adj <- manynet::to_ties(.data)
  out <- node_by_eigenvector(edge_adj, normalized = normalized)
  class(out) <- "numeric"
  make_tie_measure(out, .data)
}

# Eigenvector centralisation ####

#' Measuring networks eigenvector-like centralisation
#' @name measure_centralisation_eigen
#' @description
#'   - `net_by_eigenvector()` measures the eigenvector centralization for a
#'   network as a single score.
#'   - `mode_by_eigenvector()` measures eigenvector centralization separately for
#'   each mode of a two-mode network (via projection to each mode), returning one
#'   score per mode (following Borgatti and Everett, 1997).
#'
#'   All measures attempt to use as much information as they are offered,
#'   including whether the networks are directed, weighted, or multimodal.
#'   If this would produce unintended results,
#'   first transform the salient properties using e.g. [to_undirected()] functions.
#'   All centrality and centralization measures return normalized measures
#'   by default, including for two-mode networks.
#'
#'   For two-mode networks the two modes have different theoretical maxima, so
#'   `net_by_eigenvector()` reports a single network-level score by applying
#'   Freeman's general centralization index over the normalized node eigenvector
#'   scores, whereas `mode_by_eigenvector()` reports the per-mode scores directly.
#' @template param_data
#' @template param_norm
#' @family eigenvector
#' @family centrality
#' @references
#'   Borgatti, Stephen P., and Martin G. Everett. 1997.
#'   "Network analysis of 2-mode data."
#'   _Social Networks_ 19(3): 243-269.
#'   \doi{10.1016/S0378-8733(96)00301-2}
#' @returns
#'   `net_by_eigenvector()` returns a `network_measure` scalar;
#'   `mode_by_eigenvector()` returns a `mode_measure` numeric vector of length two,
#'   giving one centralization score per mode.
NULL

#' @rdname measure_centralisation_eigen
#' @examples
#' net_by_eigenvector(ison_southern_women)
#' @export
net_by_eigenvector <- function(.data, normalized = TRUE){
  .data <- manynet::expect_nodes(.data)
  if (manynet::is_twomode(.data)) {
    # Two-mode eigenvector centralization is intrinsically per mode
    # (see `mode_by_eigenvector()`, following Borgatti and Everett, 1997).
    # For a single network-level score we apply Freeman's general
    # centralization index over the whole node set, using the normalized node
    # eigenvector scores (each in [0, 1]); the numerator's maximum is (n - 1).
    nc <- node_by_eigenvector(.data, normalized = TRUE)
    out <- sum(max(nc) - nc) / (length(nc) - 1)
  } else {
    out <- igraph::centr_eigen(manynet::as_igraph(.data),
                               normalized = normalized)$centralization
  }
  out <- make_network_measure(out, .data, call = deparse(sys.call()))
  out
}

#' @rdname measure_centralisation_eigen
#' @examples
#' mode_by_eigenvector(ison_southern_women)
#' @export
mode_by_eigenvector <- function(.data, normalized = TRUE){
  .data <- manynet::expect_nodes(.data)
  if (!manynet::is_twomode(.data))
    manynet::snet_abort("`mode_by_eigenvector()` is only defined for two-mode networks; use `net_by_eigenvector()` for one-mode networks.")
  out <- c("Mode 1" = igraph::centr_eigen(manynet::as_igraph(manynet::to_mode1(.data)),
                                          normalized = normalized)$centralization,
           "Mode 2" = igraph::centr_eigen(manynet::as_igraph(manynet::to_mode2(.data)),
                                          normalized = normalized)$centralization)
  out <- make_mode_measure(out, .data, call = deparse(sys.call()))
  out
}

# net_diffusion ####

#' Measures of network diffusion
#' @name measure_diffusion_net
#' @description
#'   These functions allow measurement of various features of
#'   a diffusion process at the network level:
#'   
#'   - `net_by_transmissibility()` measures the average transmissibility observed
#'   in a diffusion simulation, or the number of new infections over
#'   the number of susceptible nodes.
#'   - `net_by_recovery()` measures the average number of time steps 
#'   nodes remain infected once they become infected.
#'   - `net_by_reproduction()` measures the observed reproductive number
#'   in a diffusion simulation as the network's transmissibility over
#'   the network's average infection length.
#'   - `net_by_immunity()` measures the proportion of nodes that would need
#'   to be protected through vaccination, isolation, or recovery for herd immunity to be reached.
#'   
#' @param .data Network data with nodal changes,
#'   as created by `play_diffusion()`,
#'   or a valid network diffusion model,
#'   as created by `as_diffusion()`.
#' @template param_norm
#' @template net_measure
#' @family diffusion
#' @examples
#'   smeg <- generate_smallworld(15, 0.025)
#'   smeg_diff <- play_diffusion(smeg, recovery = 0.2)
#'   plot(smeg_diff)
#' @references
#' ## On epidemiological models
#'   Kermack, William O., and Anderson Gray McKendrick. 1927. 
#'   "A contribution to the mathematical theory of epidemics". 
#'   _Proc. R. Soc. London A_ 115: 700-721.
#'   \doi{10.1098/rspa.1927.0118}
NULL

#' @rdname measure_diffusion_net 
#' @section Transmissibility: 
#'   `net_transmissibility()` measures how many directly susceptible nodes
#'   each infected node will infect in each time period, on average.
#'   That is:
#'   \deqn{T = \frac{1}{n}\sum_{j=1}^n \frac{i_{j}}{s_{j}}}
#'   where \eqn{i} is the number of new infections in each time period, \eqn{j \in n},
#'   and \eqn{s} is the number of nodes that could have been infected in that time period
#'   (note that \eqn{s \neq S}, or 
#'   the number of nodes that are susceptible in the population).
#'   \eqn{T} can be interpreted as the proportion of susceptible nodes that are
#'   infected at each time period.
#' @examples
#'   # To calculate the average transmissibility for a given diffusion model
#'   net_by_transmissibility(smeg_diff)
#' @export
net_by_transmissibility <- function(.data){
  diff_model <- manynet::as_diffusion(.data)
  out <- diff_model$I_new/diff_model$s
  out <- out[-1]
  out <- out[!is.infinite(out)]
  out <- out[!is.nan(out)]
  if(inherits(.data, "diff_model")) 
    net <- attr(.data, "network") else 
      net <- .data
  make_network_measure(mean(out, na.rm = TRUE),
                       net,
                       call = deparse(sys.call()))
}

#' @rdname measure_diffusion_net 
#' @param censor Where some nodes have not yet recovered by the end
#'   of the simulation, right censored values can be replaced by the number of steps. 
#'   By default TRUE.
#'   Note that this will likely still underestimate recovery.
#' @section Recovery time: 
#'   `net_recovery()` measures the average number of time steps that
#'   nodes in a network remain infected.
#'   Note that in a diffusion model without recovery, average infection length
#'   will be infinite.
#'   This will also be the case where there is right censoring.
#'   The longer nodes remain infected, the longer they can infect others.
#' @examples
#'   # To calculate the average infection length for a given diffusion model
#'   net_by_recovery(smeg_diff)
#' @export
net_by_recovery <- function(.data, censor = TRUE){
  diff_model <- manynet::as_diffusion(.data)
  recovs <- node_by_adopt_recovery(.data)
  if(censor && any(!is.infinite(recovs) & !is.na(recovs)))
    recovs[is.infinite(recovs)] <- nrow(diff_model)
  if(inherits(.data, "diff_model")) 
    net <- attr(.data, "network") else 
      net <- .data
  make_network_measure(mean(recovs, na.rm = TRUE),
                       net,
                       call = deparse(sys.call()))
}

#' @rdname measure_diffusion_net 
#' @section Reproduction number: 
#'   `net_reproduction()` measures a given diffusion's reproductive number.
#'   Here it is calculated as:
#'   \deqn{R = \min\left(\frac{T}{1/L}, \bar{k}\right)}
#'   where \eqn{T} is the observed transmissibility in a diffusion
#'   and \eqn{L} is the observed recovery length in a diffusion.
#'   Since \eqn{L} can be infinite where there is no recovery
#'   or there is right censoring,
#'   and since network structure places an upper limit on how many
#'   nodes each node may further infect (their degree),
#'   this function returns the minimum of \eqn{R_0}
#'   and the network's average degree.
#'   
#'   Interpretation of the reproduction number is oriented around R = 1.
#'   Where \eqn{R > 1}, the 'disease' will 'infect' more and more
#'   nodes in the network.
#'   Where \eqn{R < 1}, the 'disease' will not sustain itself and eventually
#'   die out.
#'   Where \eqn{R = 1}, the 'disease' will continue as endemic,
#'   if conditions allow.
#' @references
#' ## On the basic reproduction number
#' Diekmann, Odo, Hans J.A.P. Heesterbeek, and Hans J.A.J. Metz. 1990. 
#' "On the definition and the computation of the basic reproduction ratio R0 in models for infectious diseases in heterogeneous populations". 
#' _Journal of Mathematical Biology_, 28(4): 365–82.
#' \doi{10.1007/BF00178324}
#' 
#' Kenah, Eben, and James M. Robins. 2007. 
#' "Second look at the spread of epidemics on networks". 
#' _Physical Review E_, 76(3 Pt 2): 036113.
#' \doi{10.1103/PhysRevE.76.036113}
#' @examples
#'   # To calculate the reproduction number for a given diffusion model
#'   net_by_reproduction(smeg_diff)
#' @export
net_by_reproduction <- function(.data){
  # diff_model <- as_diffusion(.data)
  if(inherits(.data, "diff_model")) 
    net <- attr(.data, "network") else 
      net <- .data
    out <- net_by_transmissibility(.data)/
      (1/net_by_recovery(.data))
    out <- min(out, mean(node_by_deg(net)))
  make_network_measure(out, net,
                       call = deparse(sys.call()))
}

#' @rdname measure_diffusion_net 
#' @section Herd immunity: 
#'   `net_immunity()` estimates the proportion of a network
#'   that need to be protected from infection for herd immunity
#'   to be achieved.
#'   This is known as the Herd Immunity Threshold or HIT:
#'   \deqn{1 - \frac{1}{R}}
#'   where \eqn{R} is the reproduction number from `net_reproduction()`.
#'   The HIT indicates the threshold at which
#'   the reduction of susceptible members of the network means
#'   that infections will no longer keep increasing.
#'   Note that there may still be more infections after this threshold has been reached,
#'   but there should be fewer and fewer.
#'   These excess infections are called the _overshoot_.
#'   This function does _not_ take into account the structure
#'   of the network, instead using the average degree.
#'   
#'   Interpretation is quite straightforward.
#'   A HIT or immunity score of 0.75 would mean that 75% of the nodes in the network
#'   would need to be vaccinated or otherwise protected to achieve herd immunity.
#'   To identify how many nodes this would be, multiply this proportion with the number
#'   of nodes in the network.
#' @references
#' ## On herd immunity
#' Garnett, G.P. 2005.
#' "Role of herd immunity in determining the effect of vaccines against sexually transmitted disease".
#' _The Journal of Infectious Diseases_, 191(1): S97-106.
#' \doi{10.1086/425271}
#' @examples
#'   # Calculating the proportion required to achieve herd immunity
#'   net_by_immunity(smeg_diff)
#'   # To find the number of nodes to be vaccinated
#'   net_by_immunity(smeg_diff, normalized = FALSE)
#' @export
net_by_immunity <- function(.data, normalized = TRUE){
  # diff_model <- as_diffusion(.data)
  if(inherits(.data, "diff_model")) 
    net <- attr(.data, "network") else 
      net <- .data
  out <- 1 - 1/net_by_reproduction(.data)
  if(!normalized) out <- ceiling(out * manynet::net_nodes(net))
  make_network_measure(out, net,
                       call = deparse(sys.call()))
}

# net_infection ####

#' Measures of network infection
#' @name measure_diffusion_infection
#' @description
#'   These functions allow measurement of various features of
#'   a diffusion process at the network level:
#'   
#'   - `net_by_infection_complete()` measures the number of time steps until
#'   (the first instance of) complete infection.
#'   For diffusions that are not observed to complete,
#'   this function returns the value of `Inf` (infinity).
#'   This makes sure that at least ordinality is respected.
#'   - `net_by_infection_total()` measures the proportion or total number of nodes
#'   that are infected/activated at some time by the end of the diffusion process.
#'   This includes nodes that subsequently recover.
#'   Where reinfection is possible, the proportion may be higher than 1.
#'   - `net_by_infection_peak()` measures the number of time steps until the
#'   highest infection rate is observed.
#'   
#' @inheritParams measure_diffusion_net
#' @family diffusion
#' @template net_measure

#' @rdname measure_diffusion_infection 
#' @examples
#'   smeg <- generate_smallworld(15, 0.025)
#'   smeg_diff <- play_diffusion(smeg)
#'   net_by_infection_complete(smeg_diff)
#' @export
net_by_infection_complete <- function(.data){
  diff_model <- manynet::as_diffusion(.data)
  out <- which(diff_model$I == diff_model$n)[1]
  if(is.na(out)) out <- Inf
  if(inherits(.data, "diff_model")) 
    net <- attr(.data, "network") else 
      net <- .data
  make_network_measure(out, net,
                       call = deparse(sys.call()))
}

#' @rdname measure_diffusion_infection 
#' @examples
#'   net_by_infection_total(smeg_diff)
#' @export
net_by_infection_total <- function(.data, normalized = TRUE){
  if(inherits(.data, "diff_model")){
    diff_model <- manynet::as_diffusion(.data)
    out <- sum(diff_model$I_new)
    if(normalized) out <- out / diff_model$n[length(diff_model$n)]
    make_network_measure(out, attr(diff_model, "network"),
                         call = deparse(sys.call()))
  } else {
    out <- sum(manynet::as_changelist(.data)$value == "I")
    if(normalized) out <- out / manynet::net_nodes(.data)
    make_network_measure(out, .data,
                         call = deparse(sys.call()))
  }
}

#' @rdname measure_diffusion_infection 
#' @examples
#'   net_by_infection_peak(smeg_diff)
#' @export
net_by_infection_peak <- function(.data){
  diff_model <- manynet::as_diffusion(.data)
  if(inherits(.data, "diff_model")) 
    net <- attr(.data, "network") else 
      net <- .data
  out <- which(diff_model$I_new == max(diff_model$I_new))[1]
  make_network_measure(out, net,
                       call = deparse(sys.call()))
}

# node_diffusion ####

#' Measures of nodes in a diffusion
#' @name measure_diffusion_node
#' @description
#'   These functions allow measurement of various features of
#'   a diffusion process:
#'   
#'   - `node_by_adopt_time()`: Measures the number of time steps until
#'   nodes adopt/become infected
#'   - `node_by_adopt_threshold()`: Measures nodes' thresholds from the amount
#'   of exposure they had when they became infected
#'   - `node_by_adopt_recovery()`: Measures the average length nodes that become
#'   infected remain infected in a compartmental model with recovery
#'   - `node_by_adopt_exposure()`: Measures how many exposures nodes have to 
#'   a given mark
#'   
#' @template param_data
#' @template param_norm
#' @family diffusion
#' @template node_measure
#' @examples
#'   smeg <- generate_smallworld(15, 0.025)
#'   smeg_diff <- play_diffusion(smeg, recovery = 0.2)
#'   plot(smeg_diff)
#' @references
#' ## On diffusion measures
#'   Valente, Tom W. 1995. _Network models of the diffusion of innovations_
#'   (2nd ed.). Cresskill N.J.: Hampton Press.
NULL

#' @rdname measure_diffusion_node 
#' @section Adoption time: 
#'   `node_by_adopt_time()` measures the time units it took 
#'   until each node became infected.
#'   Note that an adoption time of 0 indicates that this was a seed node.
#' @examples
#'   # To measure when nodes adopted a diffusion/were infected
#'   (times <- node_by_adopt_time(smeg_diff))
#' @export
node_by_adopt_time <- function(.data){

  if(inherits(.data, "diff_model")){
    net <- attr(.data, "network") 
    out <- summary(.data) |> dplyr::filter(event == "I") |> 
      dplyr::distinct(nodes, .keep_all = TRUE) |> 
      dplyr::select(nodes,t)
    if(!manynet::is_labelled(net))
      out <- dplyr::arrange(out, nodes) else if (is.numeric(out$nodes))
        out$nodes <- manynet::node_names(net)[out$nodes]
    out <- stats::setNames(out$t, out$nodes)
    if(length(out) != manynet::net_nodes(net)){
      full <- rep(Inf, manynet::net_nodes(net))
      names(full) <- `if`(manynet::is_labelled(net), 
                          manynet::node_names(net), 
                          as.character(seq_len(manynet::net_nodes(net))))
      full[match(names(out), names(full))] <- out
      out <- `if`(manynet::is_labelled(net), full, unname(full))
    }
  } else {
    net <- .data
    out <- manynet::as_changelist(.data) |> dplyr::filter(value == "I") |> 
      dplyr::distinct(node, .keep_all = TRUE) |> 
      dplyr::select(node,time)
    if(!manynet::is_labelled(net))
      out <- dplyr::arrange(out, node) else if (is.numeric(out$node))
        out$node <- manynet::node_names(net)[out$node]
    out <- stats::setNames(out$time, out$node)
    if(length(out) != manynet::net_nodes(net)){
      full <- rep(Inf, manynet::net_nodes(net))
      names(full) <- `if`(manynet::is_labelled(net), 
                          manynet::node_names(net), 
                          as.character(seq_len(manynet::net_nodes(net))))
      full[match(names(out), names(full))] <- out
      out <- `if`(manynet::is_labelled(net), full, unname(full))
    }
  }
      
  if(!manynet::is_labelled(net)) out <- unname(out)
  make_node_measure(out, net)
}

#' @rdname measure_diffusion_node 
#' @param lag The number of time steps back upon which the thresholds are
#'   inferred.
#' @section Thresholds:
#'   `node_by_adopt_threshold()` infers nodes' thresholds based on how much
#'   exposure they had when they were infected.
#'   This inference is of course imperfect,
#'   especially where there is a sudden increase in exposure,
#'   but it can be used heuristically.
#'   In a threshold model, 
#'   nodes activate when \eqn{\sum_{j:\text{active}} w_{ji} \geq \theta_i},
#'   where \eqn{w} is some (potentially weighted) matrix,
#'   \eqn{j} are some already activated nodes,
#'   and \eqn{theta} is some pre-defined threshold value.
#'   Where a fractional threshold is used, the equation is 
#'   \eqn{\frac{\sum_{j:\text{active}} w_{ji}}{\sum_{j} w_{ji}} \geq \theta_i}.
#'   That is, \eqn{theta} is now a proportion,
#'   and works regardless of whether \eqn{w} is weighted or not.
#' @examples
#'   # To infer nodes' thresholds
#'   node_by_adopt_threshold(smeg_diff)
#' @export
node_by_adopt_threshold <- function(.data, normalized = TRUE, lag = 1){
  if(inherits(.data, "diff_model")){
    net <- attr(.data, "network") 
    diff_model <- manynet::as_diffusion(.data)
    out <- summary(diff_model)
    if(!"exposure" %in% names(out)){
      out[,'exposure'] <- NA_integer_
      for(v in unique(out$t)){
        out$exposure[out$t == v] <- node_by_adopt_exposure(diff_model, 
                                                  time = v-lag)[out$nodes[out$t == v]]
      }
    }
    if(any(out$event == "E")) 
      out <- out |> dplyr::filter(event == "E") else 
        out <- out |> dplyr::filter(event == "I")
    out <- out |> dplyr::distinct(nodes, .keep_all = TRUE) |> 
      dplyr::select(nodes, exposure)
    if(manynet::is_labelled(net))
      out <- stats::setNames(out$exposure, manynet::node_names(net)[out$nodes]) else
        out <- stats::setNames(out$exposure, out$nodes)
  } else {
    net <- .data
    diff_model <- manynet::as_diffusion(.data)
    out <- manynet::as_changelist(.data)
    if(!"exposure" %in% names(out)){
      out[,'exposure'] <- NA_integer_
      for(v in unique(out$time)){
        out$exposure[out$time == v] <- node_by_adopt_exposure(.data, 
                                                  time = v-lag)[out$node[out$time == v]]
      }
    }
    if(any(out$value == "E")) 
      out <- out |> dplyr::filter(value == "E") else 
        out <- out |> dplyr::filter(value == "I")
    out <- out |> dplyr::distinct(node, .keep_all = TRUE) |> 
      dplyr::select(node, exposure)
    if(manynet::is_labelled(net))
      out <- stats::setNames(out$exposure, manynet::node_names(net)[out$node]) else
        out <- stats::setNames(out$exposure, out$node)
  }
  if(length(out) != manynet::net_nodes(net)){
    if(manynet::is_labelled(net)){
      full <- stats::setNames(rep(Inf, manynet::net_nodes(net)), 
                              manynet::node_names(net))
    } else {
      full <- stats::setNames(rep(Inf, manynet::net_nodes(net)), 
                              1:manynet::net_nodes(net))
    } 
    full[match(names(out), names(full))] <- out
    out <- full
  }
  if(manynet::is_labelled(net))
    out <- out[match(manynet::node_names(net), names(out))] else {
      out <- unname(out[order(as.numeric(names(out)))])
    }
  if(normalized) out <- out / node_by_deg(net)
  make_node_measure(out, net)
}

#' @rdname measure_diffusion_node 
#' @section Recovery:
#'   `node_by_adopt_recovery()` measures the average length of time that nodes 
#'   that become infected remain infected in a compartmental model with recovery.
#'   Infections that are not concluded by the end of the study period are
#'   calculated as infinite.
#' @examples
#'   # To measure how long each node remains infected for
#'   node_by_adopt_recovery(smeg_diff)
#' @export
node_by_adopt_recovery <- function(.data){
  if(inherits(.data, "diff_model")){
    net <- attr(.data, "network")
    events <- attr(.data, "events")
    out <- vapply(seq_len(.data$n[1]), 
                  function(x) ifelse("I" %in% dplyr::filter(events, nodes == x)$event,
                                     ifelse("R" %in% dplyr::filter(events, nodes == x)$event,
                                            mean(diff(dplyr::filter(events, nodes == x)$t)),
                                            Inf),
                                     NA),
                  FUN.VALUE = numeric(1))
  } else {
    net <- .data
    events <- manynet::as_changelist(.data)
    out <- vapply(seq_len(manynet::net_nodes(net)), 
                  function(x) ifelse("I" %in% dplyr::filter(events, node == x)$value,
                                     ifelse("R" %in% dplyr::filter(events, node == x)$value,
                                            mean(diff(dplyr::filter(events, node == x)$time)),
                                            Inf),
                                     NA),
                  FUN.VALUE = numeric(1))
  }
  make_node_measure(out, net)
}

#' @rdname measure_diffusion_node
#' @param mark A valid 'node_mark' object or
#'   logical vector (TRUE/FALSE) of length equal to 
#'   the number of nodes in the network.
#' @param time A time point until which infections/adoptions should be
#'   identified. By default `time = 0`.
#' @section Exposure:
#'   `node_exposure()` calculates the number of infected/adopting nodes
#'   to which each susceptible node is exposed.
#'   It usually expects network data and 
#'   an index or mark (TRUE/FALSE) vector of those nodes which are currently infected,
#'   but if a diff_model is supplied instead it will return
#'   nodes exposure at \eqn{t = 0}.
#' @examples
#'   # To measure how much exposure nodes have to a given mark
#'   node_by_adopt_exposure(smeg, mark = c(1,3))
#'   node_by_adopt_exposure(smeg_diff)
#' @export
node_by_adopt_exposure <- function(.data, mark, time = 0){
  .data <- manynet::expect_nodes(.data)
  if(missing(mark)){ 
    if(inherits(.data, "diff_model")){
      mark <- node_is_infected(.data, time = time)
      .data <- attr(.data, "network")
    } else {
      mark <- node_is_infected(.data, time = time)
    }
  }
  .data <- manynet::as_tidygraph(.data)
  if(manynet::is_weighted(.data) || manynet::is_signed(.data)){
    if(is.numeric(mark)){
      mk <- rep(FALSE, manynet::net_nodes(.data))
      mk[mark] <- TRUE
    } else mk <- mark
    out <- manynet::as_matrix(.data)
    out <- colSums(out * matrix(mk, nrow(out), ncol(out)) * 
                     matrix(!mk, nrow(out), ncol(out), byrow = TRUE))
  } else {
    if(is.logical(mark)) mark <- which(mark)
    if(manynet::is_twomode(.data)){
      if(mark[1]>manynet::net_dims(.data)[1]){ 
        dat <- manynet::to_mode2(.data)
        mark <- mark - manynet::net_dims(.data)[1]
        contacts <- unlist(lapply(igraph::neighborhood(dat, nodes = mark, mode = "out"),
                                  function(x) setdiff(x, mark)))
        contacts <- contacts + manynet::net_dims(.data)[1]
      } else {
        dat <- manynet::to_mode1(.data)
        contacts <- unlist(lapply(igraph::neighborhood(dat),
                                  function(x) setdiff(x, mark)))
        contacts <- contacts + manynet::net_dims(.data)[2]
      }
    } else {
      dat <- .data
      contacts <- unlist(lapply(igraph::neighborhood(dat, nodes = mark, mode = "out"),
                                function(x) setdiff(x, mark)))
    }
    # count exposures for each node:
    tabcontact <- table(contacts)
    out <- rep(0, manynet::net_nodes(.data))
    out[as.numeric(names(tabcontact))] <- unname(tabcontact)
  }
  make_node_measure(out, .data)
}

# Diffusion membership ####

#' Memberships in a diffusion process
#' @description
#'   `node_in_adopter()` classifies membership of nodes into diffusion categories
#'   by where on the distribution of adopters they fell.
#'   Valente (1995) defines five memberships:
#'   
#'   - _Early adopter_: those with an adoption time less than 
#'   the average adoption time minus one standard deviation of adoptions times
#'   - _Early majority_: those with an adoption time between
#'   the average adoption time and 
#'   the average adoption time minus one standard deviation of adoptions times
#'   - _Late majority_: those with an adoption time between
#'   the average adoption time and 
#'   the average adoption time plus one standard deviation of adoptions times
#'   - _Laggard_: those with an adoption time greater than 
#'   the average adoption time plus one standard deviation of adoptions times
#'   - _Non-adopter_: those without an adoption time,
#'   i.e. never adopted
#'   
#' @template param_data
#' @template node_member
#' @family diffusion
#' @name member_diffusion
#' @references
#' ## On adopter classes
#'   Valente, Tom W. 1995. 
#'   _Network models of the diffusion of innovations_
#'   (2nd ed.). Cresskill N.J.: Hampton Press.
NULL

#' @rdname member_diffusion
#' @examples
#'   smeg <- generate_smallworld(15, 0.025)
#'   smeg_diff <- play_diffusion(smeg, recovery = 0.2)
#'   # To classify nodes by their position in the adoption curve
#'   (adopts <- node_in_adopter(smeg_diff))
#'   summary(adopts)
#' @export
node_in_adopter <- function(.data){
  toa <- node_by_adopt_time(.data)
  toa[is.infinite(toa)] <- NA
  avg <- mean(toa, na.rm = TRUE)
  sdv <- stats::sd(toa, na.rm = TRUE)
  out <- ifelse(toa < (avg - sdv) | toa == 0, "Early Adopter", 
                ifelse(toa > (avg + sdv), "Laggard",
                       ifelse((avg - sdv) < toa & toa <= avg, "Early Majority", 
                              ifelse(avg < toa & toa <= avg + sdv, "Late Majority", 
                                     "Non-Adopter"))))
  out[is.na(out)] <- "Non-Adopter"
  if(inherits(.data, "diff_model")) 
    net <- attr(.data, "network") else 
      net <- .data
  make_node_member(out, net)
}

test <- play_diffusion(create_tree(12), steps = 2)

test_that("infection total works", {
  expect_values(net_by_infection_total(test), 0.583)
})

test_that("infection complete works", {
  expect_values(net_by_infection_complete(test), Inf)
})

# Topological features ####

#' Measuring network topological features
#' @name measure_features
#' @description
#'   These functions measure certain topological features of networks:
#'   
#'   - `net_by_core()` measures the correlation between a network
#'   and a core-periphery model with the same dimensions.
#'   - `net_by_richclub()` measures the rich-club coefficient of a network.
#'   - `net_by_factions()` measures the correlation between a network
#'   and a component model with the same dimensions.
#'   If no 'membership' vector is given for the data, 
#'   `node_partition()` is used to partition nodes into two groups.
#'   - `net_by_modularity()` measures the modularity of a network 
#'   based on nodes' membership in defined clusters.
#'   - `net_by_smallworld()` measures the small-world coefficient for one- or 
#'   two-mode networks. Small-world networks can be highly clustered and yet
#'   have short path lengths.
#'   - `net_by_scalefree()` measures the exponent of a fitted
#'   power-law distribution. An exponent between 2 and 3 usually indicates 
#'   a power-law distribution.
#'   - `net_by_balance()` measures the structural balance index on 
#'   the proportion of balanced triangles,
#'   ranging between `0` if all triangles are imbalanced and 
#'   `1` if all triangles are balanced.
#' 
#' @template param_data
#' @template param_memb
#' @template net_measure
NULL

#' @rdname measure_features
#' @param mark A logical vector indicating which nodes belong to the core.
#' @param method Which method of the following to use to calculate the fit of
#'   the core assignment to a core-periphery model.
#'   "correlation" calculates the correlation between the empirical network and
#'   an ideal typical network, and "ident" calculates the Euclidean distances
#'   between the same.
#'   "ndiff", however, calculates how distinct the core and periphery groups are
#'   based on the difference in coreness scores between the least core-like
#'   member of the core and the most core-like member of the periphery.
#'   "diff" is similar to "ndiff", but multiplies the raw "ndiff" score by the
#'   square root of the size of the core, thus penalising large cores.
#' @section Core-Periphery: 
#'   `net_core()` calculates the Pearson correlation between the given network, 
#'   where the nodes in the core are assigned by some given mark, and an ideal
#'   typical core-periphery network with the same number of nodes in the core
#'   and the periphery.
#' @references 
#' ## On core-periphery
#' Borgatti, Stephen P., and Martin G. Everett. 2000. 
#' “Models of Core/Periphery Structures.” 
#' _Social Networks_ 21(4):375–95.
#' \doi{10.1016/S0378-8733(99)00019-2}
#' @examples 
#' net_by_core(ison_adolescents)
#' net_by_core(ison_southern_women)
#' @export
net_by_core <- function(.data,
                     mark = NULL,
                     method = c("correlation","ident","ndiff", "diff")){
  .data <- manynet::expect_nodes(.data)
  if(is.null(mark)) mark <- node_is_core(.data)
  
  method <- match.arg(method)
  if(method == "correlation"){
    out <- stats::cor(c(manynet::as_matrix(.data)), 
                      c(manynet::as_matrix(manynet::create_core(.data, mark = mark))))
  } else if(method == "ident"){
    out <- sqrt(sum((manynet::as_matrix(.data) - 
                       manynet::as_matrix(manynet::create_core(.data, mark = mark)))^2))
  } else if(method %in% c("ndiff","diff")){
    # Sort nodes by coreness
    c_scores <- node_by_coreness(.data)
    core <- c_scores[mark]
    periphery <- c_scores[!mark]
    
    min_core <- min(core)
    max_periphery <- max(periphery)
    
    diff1 <- sum(min_core - periphery)
    diff2 <- sum(core - max_periphery)
    
    if(method == "ndiff"){
      out <- (diff1 + diff2) / length(c_scores)  # Normalize
    } else if(method == "diff"){
      out <- (diff1 + diff2) * sqrt(sum(mark))
    } 
  } else manynet::snet_unavailable(method)
  make_network_measure(out, .data, call = deparse(sys.call()))
}

#' @rdname measure_features
#' @references
#' ## On the rich-club coefficient
#' Zhou, Shi, and Raul J. Mondragon. 2004.
#' "The Rich-Club Phenomenon in the Internet Topology".
#' _IEEE Communications Letters_, 8(3): 180-182.
#' \doi{10.1109/lcomm.2004.823426}
#' @examples
#' net_by_richclub(ison_adolescents)
#' @export
net_by_richclub <- function(.data){
  .data <- manynet::expect_nodes(.data)
  coefs <- vector()
  temp <- .data
  for(k in seq_len(max(node_by_deg(temp)))){
    richclub <- manynet::to_subgraph(temp, node_by_deg(temp) >= k)
    nk <- manynet::net_nodes(richclub)
    ek <- ifelse(manynet::is_directed(temp),
                 manynet::net_ties(richclub), 
                 2*manynet::net_ties(richclub))
    coefs <- c(coefs, (ek)/(nk*(nk-1)))
  }
  
  .elbow_finder <- function(x_values, y_values) {
    # Max values to create line
    # if(min(x_values)==1) x_values <- x_values[2:length(x_values)]
    # if(min(y_values)==0) y_values <- y_values[2:length(y_values)]
    max_df <- data.frame(x = c(1, min(which(y_values == 1))), 
                         y = c(min(y_values), max(y_values)))
    # Creating straight line between the max values
    fit <- stats::lm(max_df$y ~ max_df$x)
    # Distance from point to line
    distances <- vector()
    for (i in seq_len(length(x_values))) {
      distances <- c(distances,
                     abs(stats::coef(fit)[2]*x_values[i] -
                           y_values[i] +
                           coef(fit)[1]) /
                       sqrt(stats::coef(fit)[2]^2 + 1^2))
    }
    # Max distance point
    x_max_dist <- x_values[which.max(distances)]
    x_max_dist
  }
  
  coefs[is.nan(coefs)] <- 1
  if(length(which(coefs == 1)) == 0) out <- 0 else
    out <- coefs[.elbow_finder(seq_along(coefs), coefs)]
  # max(coefs, na.rm = TRUE)
  make_network_measure(out, .data, call = deparse(sys.call()))
}

#' @rdname measure_features 
#' @examples 
#'   net_by_factions(ison_southern_women)
#' @export
net_by_factions <- function(.data,
                       membership = NULL){
  .data <- manynet::expect_nodes(.data)
  membership <- .resolve_membership(.data, membership)
  if(is.null(membership)){
    manynet::snet_info("No membership vector assigned.",
              "Partitioning the network using {.fn node_in_partition}.")
    membership <- node_in_partition(.data)
  }
  out <- stats::cor(c(manynet::as_matrix(.data)), 
                    c(manynet::as_matrix(manynet::create_components(.data,
                                                  membership = membership))))
  make_network_measure(out, .data, call = deparse(sys.call()))
}

#' @rdname measure_features
#' @section Modularity:
#'   Modularity measures the difference between the number of ties within each community
#'   from the number of ties expected within each community in a random graph
#'   with the same degrees, and ranges between -1 and +1.
#'   Modularity scores of +1 mean that ties only appear within communities,
#'   while -1 would mean that ties only appear between communities.
#'   A score of 0 would mean that ties are half within and half between communities,
#'   as one would expect in a random graph.
#'   
#'   Modularity faces a difficult problem known as the resolution limit 
#'   (Fortunato and Barthélemy 2007).
#'   This problem appears when optimising modularity,
#'   particularly with large networks or depending on the degree of interconnectedness,
#'   can miss small clusters that 'hide' inside larger clusters.
#'   In the extreme case, this can be where they are only connected
#'   to the rest of the network through a single tie.
#'   To help manage this problem, a `resolution` parameter is added. 
#'   Please see the argument definition for more details.
#' @param resolution A proportion indicating the resolution scale.
#'   By default 1, which returns the original definition of modularity.
#'   The higher this parameter, the more smaller communities will be privileged.
#'   The lower this parameter, the fewer larger communities are likely to be found.
#' @examples 
#' net_by_modularity(ison_adolescents, 
#'   node_in_partition(ison_adolescents))
#' net_by_modularity(ison_southern_women, 
#'   node_in_partition(ison_southern_women))
#' @references 
#' ## On modularity
#' Newman, Mark E.J. 2006.
#' "Modularity and community structure in networks",
#' _Proceedings of the National Academy of Sciences_ 103(23): 8577-8696.
#' \doi{https://doi.org/10.1073/pnas.0601602103}
#' 
#' Murata, Tsuyoshi. 2010. 
#' "Modularity for Bipartite Networks". 
#' In: Memon, N., Xu, J., Hicks, D., Chen, H. (eds) 
#' _Data Mining for Social Network Data. Annals of Information Systems_, Vol 12. 
#' Springer, Boston, MA. 
#' \doi{10.1007/978-1-4419-6287-4_7}
#' @export
net_by_modularity <- function(.data, 
                             membership = NULL, 
                             resolution = 1){
  .data <- manynet::expect_nodes(.data)
  membership <- .resolve_membership(.data, membership)
  if(is.null(membership)){
    manynet::snet_info("Since no membership argument has been provided,",
              "a partition of the network into two will be calculated and used.")
    membership <- node_in_partition(.data)
  }
  if(!is.numeric(membership)) membership <- as.numeric(as.factor(membership))
  if(!manynet::is_graph(.data)) .data <- as_igraph(.data)
  if(manynet::is_twomode(.data)){
    make_network_measure(igraph::modularity(manynet::to_multilevel(.data), 
                                          membership = membership,
                                          resolution = resolution), 
                         .data, call = deparse(sys.call()))
  } else make_network_measure(igraph::modularity(.data, 
                                               membership = membership,
                                               resolution = resolution),
                              .data, call = deparse(sys.call()))
}

#' @rdname measure_features 
#' @param times Integer of number of simulations.
#' @param method There are three small-world measures implemented:
#'   - "sigma" is the original equation from Watts and Strogatz (1998),
#'     \deqn{\frac{\frac{C}{C_r}}{\frac{L}{L_r}}}, 
#'     where \eqn{C} and \eqn{L} are the observed 
#'     clustering coefficient and path length, respectively,
#'     and \eqn{C_r} and \eqn{L_r} are the averages obtained from
#'     random networks of the same dimensions and density.
#'     A \eqn{\sigma > 1} is considered to be small-world,
#'     but this measure is highly sensitive to network size.
#'  -  "omega" (the default) is an update from Telesford et al. (2011),
#'     \deqn{\frac{L_r}{L} - \frac{C}{C_l}},
#'     where \eqn{C_l} is the clustering coefficient for a lattice graph
#'     with the same dimensions.
#'     \eqn{\omega} ranges between 0 and 1, 
#'     where 1 is as close to a small-world as possible.
#'  -  "SWI" is an alternative proposed by Neal (2017),
#'     \deqn{\frac{L - L_l}{L_r - L_l} \times \frac{C - C_r}{C_l - C_r}},
#'     where \eqn{L_l} is the average path length for a lattice graph
#'     with the same dimensions.
#'     \eqn{SWI} also ranges between 0 and 1 with the same interpretation, 
#'     but where there may not be a network for which \eqn{SWI = 1}.
#' @seealso [net_by_transitivity()] and [net_by_equivalency()]
#'   for how clustering is calculated
#' @references 
#' ## On small-worldliness
#' Watts, Duncan J., and Steven H. Strogatz. 1998. 
#'   “Collective Dynamics of ‘Small-World’ Networks”. 
#'   _Nature_ 393(6684):440–42.
#'   \doi{10.1038/30918}
#' 
#' Telesford QK, Joyce KE, Hayasaka S, Burdette JH, Laurienti PJ. 2011. 
#'   "The ubiquity of small-world networks". 
#'   _Brain Connectivity_ 1(5): 367–75.
#'   \doi{10.1089/brain.2011.0038}
#'   
#' Neal, Zachary P. 2017. 
#'   "How small is it? Comparing indices of small worldliness". 
#'   _Network Science_. 5 (1): 30–44.
#'   \doi{10.1017/nws.2017.5}
#' @examples
#' net_by_smallworld(ison_brandes)
#' net_by_smallworld(ison_southern_women)
#' @export
net_by_smallworld <- function(.data, 
                               method = c("omega", "sigma", "SWI"),
                               times = 100) {
  
  .data <- manynet::expect_nodes(.data)
  method <- match.arg(method)
  
  if(manynet::is_twomode(.data)){
    co <- net_by_equivalency(.data)
    cr <- mean(vapply(1:times, 
                      function(x) net_by_equivalency(manynet::generate_random(.data)),
                      FUN.VALUE = numeric(1)))
    if(method %in% c("omega", "SWI")){
      cl <- net_by_equivalency(manynet::create_ring(.data))
    }
  } else {
    co <- net_by_transitivity(.data)
    cr <- mean(vapply(1:times, 
                            function(x) net_by_transitivity(manynet::generate_random(.data)),
                            FUN.VALUE = numeric(1)))
    if(method %in% c("omega", "SWI")){
      cl <- net_by_transitivity(manynet::create_lattice(.data))
    }
  }
  
  lo <- net_by_length(.data)
  lr <- mean(vapply(1:times, 
                         function(x) net_by_length(manynet::generate_random(.data)),
                         FUN.VALUE = numeric(1)))
  if(method == "SWI"){
    ll <- net_by_length(manynet::create_ring(.data))
  }
  
  out <- switch(method,
                "omega" = (lr/lo - co/cl),
                "sigma" = (co/cr)/(lo/lr),
                "SWI" = ((lo - ll)/(lr - ll))*((co - cr)/(cl - cr)))
  make_network_measure(out,
                       .data, call = deparse(sys.call()))
}

#' @rdname measure_features 
#' @importFrom igraph fit_power_law
#' @references
#' ## On scale-free networks
#' Barabasi, Albert-Laszlo, and Reka Albert. 1999.
#' "Emergence of scaling in random networks",
#' _Science_, 286(5439): 509-512.
#' \doi{10.1126/science.286.5439.509}
#' 
#' Clauset, Aaron, Cosma Rohilla Shalizi, and Mark E.J. Newman. 2009.
#' "Power-law distributions in empirical data",
#' _SIAM Review_, 51(4): 661-703.
#' \doi{10.1137/070710111}
#' 
#' Stumpf, Michael P.H., and Mason Porter. 2012.
#' "Critical truths about power laws",
#' _Science_, 335(6069): 665-666.
#' \doi{10.1126/science.1216142}
#' 
#' Holme, Petter. 2019.
#' "Rare and everywhere: Perspectives on scale-free networks",
#' _Nature Communications_, 10(1): 1016.
#' \doi{10.1038/s41467-019-09038-8}
#' @examples 
#' net_by_scalefree(ison_adolescents)
#' net_by_scalefree(generate_scalefree(50, 1.5))
#' net_by_scalefree(create_lattice(100))
#' @export
net_by_scalefree <- function(.data){
  .data <- manynet::expect_nodes(.data)
  out <- igraph::fit_power_law(node_by_deg(.data))
  if ("KS.p" %in% names(out) && !is.null(out$KS.p) && !is.na(out$KS.p) && out$KS.p < 0.05) 
    manynet::snet_info("Note: Kolmogorov-Smirnov test that data could have been drawn",
                       "from a power-law distribution rejected.")
  make_network_measure(out$alpha, .data, 
                       call = deparse(sys.call()))
}

#' @rdname measure_features 
#' @source `{signnet}` by David Schoch
#' @references
#' ## On balance theory
#' Heider, Fritz. 1946.
#' "Attitudes and cognitive organization".
#' _The Journal of Psychology_, 21: 107-112.
#' \doi{10.1080/00223980.1946.9917275}
#' 
#' Cartwright, D., and Frank Harary. 1956.
#' "Structural balance: A generalization of Heider's theory".
#' _Psychological Review_, 63(5): 277-293.
#' \doi{10.1037/h0046049}
#' @examples
#' net_by_balance(to_uniplex(fict_marvel, "relationship"))
#' @export
net_by_balance <- function(.data) {
  
  .data <- manynet::expect_nodes(.data)
  .count_signed_triangles <- function(.data){
    g <- manynet::as_igraph(.data)
    if (!"sign" %in% igraph::edge_attr_names(g)) {
      manynet::snet_abort("network does not have a sign edge attribute")
    }
    if (igraph::is_directed(g)) {
      manynet::snet_abort("g must be undirected")
    }
    eattrV <- igraph::edge_attr(g, "sign")
    if (!all(eattrV %in% c(-1, 1))) {
      manynet::snet_abort("sign may only contain -1 and 1")
    }
    tmat <- t(matrix(igraph::triangles(g), nrow = 3))
    if (nrow(tmat) == 0) {
      warning("g does not contain any triangles")
      return(c(`+++` = 0, `++-` = 0, `+--` = 0, `---` = 0))
    }
    emat <- t(apply(tmat, 1, function(x) c(igraph::get_edge_ids(g, 
                                                                x[1:2]), 
                                           igraph::get_edge_ids(g, x[2:3]), 
                                           igraph::get_edge_ids(g, 
                                                                                                                               x[c(3, 1)]))))
    emat[, 1] <- eattrV[emat[, 1]]
    emat[, 2] <- eattrV[emat[, 2]]
    emat[, 3] <- eattrV[emat[, 3]]
    emat <- t(apply(emat, 1, sort))
    emat_df <- as.data.frame(emat)
    res <- stats::aggregate(list(count = rep(1, nrow(emat_df))), 
                            emat_df, length)
    tri_counts <- c(`+++` = 0, `++-` = 0, `+--` = 0, `---` = 0)
    tmp_counts <- res[, 4]
    if (nrow(res) == 1) {
      names(tmp_counts) <- paste0(c("+", "-")[(rev(res[1:3]) == 
                                                 -1) + 1], collapse = "")
    }
    else {
      names(tmp_counts) <- apply(res[, 1:3], 1, function(x) paste0(c("+", 
                                                                     "-")[(rev(x) == -1) + 1], collapse = ""))
    }
    tri_counts[match(names(tmp_counts), names(tri_counts))] <- tmp_counts
    tri_counts
  }
  
  if (!manynet::is_signed(.data)) {
    manynet::snet_abort("network does not have a sign edge attribute")
  }
  if (manynet::is_directed(.data)) {
    manynet::snet_abort("object must be undirected")
  }
  g <- manynet::as_igraph(.data)
  eattrV <- igraph::edge_attr(g, "sign")
  if (!all(eattrV %in% c(-1, 1))) {
    manynet::snet_abort("sign may only contain -1 and 1")
  }
  tria_count <- .count_signed_triangles(g)
  make_network_measure(unname((tria_count["+++"] + tria_count["+--"])/sum(tria_count)),
                       .data, 
                       call = deparse(sys.call()))
}

set.seed(123)

# test_that("net_balance works", {
#   out <- net_balance(ison_marvel_relationships)
#   expect_s3_class(out, "network_measure")
#   expect_equal(as.numeric(out), 0.668, tolerance = 0.01)
#   expect_length(out, 1)
#   expect_error(net_balance(ison_adolescents))
# })

test_that("net_modularity works for two mode networks", {
  out <- net_by_modularity(ison_southern_women,
                 node_in_partition(ison_southern_women))
  expect_length(out, 1)
})

test_that("net_core works", {
  out <- net_by_core(ison_adolescents)
  expect_values(out, -0.133)
  expect_values(net_by_core(ison_adolescents, method = "ident"), 6.481)
  expect_values(net_by_core(ison_adolescents, method = "diff"), 6.094)
})

test_that("net_richclub works", {
  out <- net_by_richclub(ison_adolescents)
  expect_values(out, 0.833)
})

test_that("net_scalefree works", {
  out <- net_by_scalefree(ison_adolescents)
  expect_values(out,3.689)
})

test_that("net_balance works", {
  out <- net_by_balance(irps_wwi)
  expect_values(out,1)
})

wavenet <- ison_adolescents %>%
  mutate_ties(wave = c(1, 1, 1, 1, 2, 2, 2, 3, 3, 3))

test_that("net_waves works", {
  # expect_equal(net_waves(ison_adolescents), 1)
  expect_values(net_by_waves(wavenet), 3)
})

# nocov start

# defining global variables more centrally
utils::globalVariables(c(".data", "obs",
                         "from", "to", "name", "weight","sign","wave",
                         "from_memb","to_memb","to.y",
                         "nodes","event","exposure",
                         "student","students","colleges",
                         "node","value","var","active","time",
                         "A","B","C","D",
                         "type",
                         "n"))

# Helper function for declaring available methods
available_methods <- function(fun_vctr) {
  out <- lapply(fun_vctr, function(f) regmatches(utils::.S3methods(f),
                                                 regexpr("\\.", utils::.S3methods(f)),
                                                 invert = TRUE))
  out <- out[lapply(out,length)>0]
  out <- t(as.data.frame(out))
  colnames(out) <- c("from","to")
  rownames(out) <- NULL
  out <- as.data.frame(out)
  manynet::as_matrix(out)
}

# Helper function for checking and downloading packages
thisRequires <- function(pkgname){
  if (!requireNamespace(pkgname, quietly = TRUE) & interactive()) {
    if(utils::askYesNo(msg = paste("The", pkgname, 
                                   "package is required to run this function. Would you like to install", pkgname, "from CRAN?"))) {
      utils::install.packages(pkgname)
    } else {
      manynet::snet_abort(paste("Please install", pkgname, "from CRAN to run this function."))
    }
  }
}

seq_nodes <- function(.data){
  seq.int(manynet::net_nodes(.data))
}

# Resolve membership to a vector:
# if a single character string naming a network attribute is provided,
# retrieve that attribute as a vector; otherwise return the value as-is.
.resolve_membership <- function(.data, membership) {
  if (is.character(membership) && length(membership) == 1 &&
      membership %in% manynet::net_node_attributes(.data)) {
    manynet::node_attribute(.data, membership)
  } else {
    membership
  }
}

# nocov end

#' Memberships in equivalent classes
#' @description 
#'   These functions combine an appropriate `node_x_*()` function
#'   together with methods for calculating the hierarchical clusters
#'   provided by a certain distance calculation.
#'   
#'   - `node_in_equivalence()` assigns nodes membership based on their equivalence 
#'   with respective to some motif/class.
#'   The following functions call this function, together with an appropriate motif.
#'   - `node_in_structural()` assigns nodes membership based on their
#'   having equivalent ties to the same other nodes.
#'   - `node_in_regular()` assigns nodes membership based on their
#'   having equivalent patterns of ties.
#'   - `node_in_automorphic()` assigns nodes membership based on their
#'   having equivalent distances to other nodes.
#'   
#'   A `plot()` method exists for investigating the dendrogram
#'   of the hierarchical cluster and showing the returned cluster
#'   assignment.
#' @name member_equivalence
#' @template param_data
#' @template param_motf
#' @template node_member
#' @param k Typically a character string indicating which method
#'   should be used to select the number of clusters to return.
#'   By default `"silhouette"`, other options include `"elbow"` and `"strict"`.
#'   `"strict"` returns classes with members only when strictly equivalent.
#'   `"silhouette"` and `"elbow"` select classes based on the distance between
#'   clusters or between nodes within a cluster.
#'   Fewer, identifiable letters, e.g. `"e"` for elbow, is sufficient.
#'   Alternatively, if `k` is passed an integer, e.g. `k = 3`,
#'   then all selection routines are skipped in favour of this number of clusters.
#' @param cluster Character string indicating whether clusters should be 
#'   clustered hierarchically (`"hierarchical"`) or 
#'   through convergence of correlations (`"concor"`). 
#'   Fewer, identifiable letters, e.g. `"c"` for CONCOR, is sufficient.
#' @param distance Character string indicating which distance metric
#'   to pass on to `stats::dist`.
#'   By default `"euclidean"`, but other options include
#'   `"maximum"`, `"manhattan"`, `"canberra"`, `"binary"`, and `"minkowski"`.
#'   Fewer, identifiable letters, e.g. `"e"` for Euclidean, is sufficient.
#' @param Kmax Integer indicating the maximum number of (k) clusters
#'   to evaluate.
#'   Ignored when `k = "strict"` or a discrete number is given for `k`.
#' @importFrom stats as.dist hclust cutree coef cor median
#' @source \url{https://github.com/aslez/concoR}
NULL

#' @rdname member_equivalence 
#' @export
node_in_equivalence <- function(.data, motif,
                                k = c("silhouette", "elbow", "strict"),
                                cluster = c("hierarchical", "concor", "cosine"),
                                distance = c("euclidean", "maximum", "manhattan", 
                                             "canberra", "binary", "minkowski"),
                                Kmax = 8L){
  .data <- manynet::expect_nodes(.data)
  hc <- switch(match.arg(cluster),
               hierarchical = cluster_hierarchical(motif,
                                                   match.arg(distance)),
               concor = cluster_concor(.data, motif),
               cosine = cluster_cosine(motif, 
                                       match.arg(distance)))
  
  if(!is.numeric(k))
    k <- switch(match.arg(k),
                strict = k_strict(hc, .data),
                elbow = k_elbow(hc, .data, motif, Kmax),
                silhouette = k_silhouette(hc, .data, Kmax))
  if(length(k)==0) k <- 1 # in the case of all nodes being in the same cluster
  
  out <- make_node_member(stats::cutree(hc, k), .data)
  attr(out, "hc") <- hc
  attr(out, "k") <- k
  out
}

#' @rdname member_equivalence
#' @examples
#' (nse <- node_in_structural(ison_algebra))
#' @export
node_in_structural <- function(.data,
                               k = c("silhouette", "elbow", "strict"),
                               cluster = c("hierarchical", "concor","cosine"),
                               distance = c("euclidean", "maximum", "manhattan", 
                                            "canberra", "binary", "minkowski"),
                               Kmax = 8L){
  .data <- manynet::expect_nodes(.data)
  mat <- node_x_tie(.data)
  if(any(colSums(t(mat))==0)){
    mat <- cbind(mat, (colSums(t(mat))==0))
  } 
  node_in_equivalence(.data, mat, 
                      k = k, cluster = cluster, distance = distance, 
                      Kmax = Kmax)
}

#' @rdname member_equivalence
#' @examples
#' (nre <- node_in_regular(ison_southern_women,
#'   cluster = "concor"))
#' @export
node_in_regular <- function(.data, 
                            k = c("silhouette", "elbow", "strict"),
                            cluster = c("hierarchical", "concor","cosine"),
                            distance = c("euclidean", "maximum", "manhattan", 
                                         "canberra", "binary", "minkowski"),
                            Kmax = 8L){
  .data <- manynet::expect_nodes(.data)
  if(manynet::is_twomode(.data)){
    manynet::snet_info("Since this is a two-mode network,", 
              "using {.fn node_x_tetrad} to", 
              "profile nodes' embedding in local structures.")
    mat <- as.matrix(node_x_tetrad(.data))
  } else {
    manynet::snet_info("Since this is a one-mode network,", 
              "using {.fn node_x_triad} to", 
              "profile nodes' embedding in local structures.")
    mat <- node_x_triad(.data)
  }
  if(any(colSums(mat) == 0)) mat <- mat[,-which(colSums(mat) == 0)]
  node_in_equivalence(.data, mat, 
                   k = k, cluster = cluster, distance = distance, Kmax = Kmax)
}

#' @rdname member_equivalence
#' @examples
#' if(require("sna", quietly = TRUE)){
#' (nae <- node_in_automorphic(ison_southern_women,
#'   k = "elbow"))
#' }
#' @export
node_in_automorphic <- function(.data,
                                k = c("silhouette", "elbow", "strict"),
                                cluster = c("hierarchical", "concor","cosine"),
                                distance = c("euclidean", "maximum", "manhattan", 
                                             "canberra", "binary", "minkowski"),
                                Kmax = 8L){
  .data <- manynet::expect_nodes(.data)
  mat <- node_x_path(.data)
  node_in_equivalence(.data, mat, 
                   k = k, cluster = cluster, distance = distance, Kmax = Kmax)
}

# # Equivalence clustering tests

test_that("equivalence clustering returns the right class", {
  expect_s3_class(node_in_structural(ison_adolescents, "strict", "hier"), "node_member")
  expect_s3_class(node_in_regular(ison_adolescents), "node_member")
  expect_s3_class(node_in_automorphic(ison_adolescents), "node_member")
  testthat::skip_if_not_installed("sna")
  expect_s3_class(node_in_structural(ison_adolescents, "elbow", "hier"), "node_member")
  expect_s3_class(node_in_structural(ison_adolescents, "elbow", "concor"), "node_member")
})

test_that("equivalence clustering works", {
  expect_equal(node_in_structural(ison_adolescents, "silhouette", "hier"), node_in_structural(ison_adolescents))
  expect_equal(node_in_regular(ison_adolescents), node_in_regular(ison_adolescents, "silhouette", "hier"))
  expect_equal(c(net_nodes(ison_adolescents)), length(node_in_structural(ison_adolescents, "silhouette", "concor")))
  expect_equal(c(net_nodes(ison_adolescents)), length(node_in_structural(ison_adolescents, k = 3, "hier")))
  expect_equal(c(net_nodes(ison_adolescents)), length(node_in_structural(ison_adolescents, "strict", "concor")))
  expect_equal(c(net_nodes(ison_adolescents)), length(node_in_regular(ison_adolescents, cluster = "concor")))
  expect_equal(c(net_nodes(ison_adolescents)), length(node_in_regular(ison_adolescents, "strict")))
  expect_equal(c(net_nodes(ison_southern_women)), length(node_in_automorphic(ison_southern_women, "strict", distance = "binary")))
  expect_equal(c(net_nodes(ison_southern_women)), length(node_in_automorphic(ison_southern_women, distance = "maximum")))
  expect_true("C" %in% node_in_structural(ison_adolescents, k = 3, "concor"))
  expect_true("B" %in% node_in_regular(ison_adolescents, 2))
  expect_true("D" %in% node_in_automorphic(ison_southern_women, 4))
  testthat::skip_if_not_installed("sna")
  expect_equal(c(net_nodes(ison_adolescents)), length(node_in_regular(ison_adolescents, "elbow")))
})

#' Methods for equivalence clustering
#' 
#' @description
#'   These functions are used to cluster some motif census object:
#'   
#'   - `cluster_hierarchical()` returns a hierarchical clustering object
#'   created by `stats::hclust()`.
#'   - `cluster_concor()` returns a hierarchical clustering object
#'   created from a convergence of correlations procedure (CONCOR).
#'   - `cluster_cosine()` returns a hierarchical clustering object
#'   created by `stats::hclust()` on cosine dissimilarities,
#'   rather than correlations, as created by `to_cosine()`.
#' 
#'   These functions are not intended to be called directly,
#'   but are called within `node_in_equivalence()` and related functions.
#'   They are exported and listed here to provide more detailed documentation.
#' @name method_cluster
#' @inheritParams member_equivalence
#' @returns 
#'   A hierarchical clustering object created by `stats::hclust()`,
#'   with an additional `distances` attribute containing the distance matrix 
#'   used for clustering.
NULL

#' @rdname method_cluster
#' @section Hierarchical clustering:
#'  This method uses `stats::hclust()` to create a hierarchical clustering object
#'  from a distance matrix created from the correlations between nodes' profiles 
#'  in the given motif census.
#'  First a matrix of Pearson correlation coefficients between each pair of 
#'  nodes' profiles in the given motif census is created.
#'  Then a distance matrix is created by subtracting these correlations from `1`,
#'  and this is given to `stats::hclust()` to enable dendrogram construction etc.
#' @export
cluster_hierarchical <- function(motif, distance){
  correlations <- manynet::to_correlation(t(motif))
  dissimilarity <- 1 - correlations
  distances <- stats::dist(dissimilarity, method = distance)
  hc <- stats::hclust(distances)
  hc$distances <- distances
  hc
}

#' @rdname method_cluster
#' @section Cosine similarity: 
#'   This method is similar to the hierarchical clustering method, 
#'   but uses cosine similarity rather than correlation as the clustering basis.
#'   First a matrix of cosine similarities between each pair of nodes' profiles 
#'   in the given census is created. 
#'   Then a distance matrix is created by subtracting these similarities from `1`, 
#'   and this is given to `stats::hclust` to enable dendrogram construction etc.
#' @export
cluster_cosine <- function(motif, distance){
  cosines <- manynet::to_cosine(motif)
  dissimilarity <- 1 - cosines
  distances <- stats::dist(dissimilarity, method = distance)
  hc <- stats::hclust(distances)
  hc$distances <- distances
  hc
}

# cluster_concor(ison_adolescents)
# cluster_concor(ison_southern_women)
# https://github.com/bwlewis/hclust_in_R/blob/master/hc.R

#' @rdname method_cluster 
#' @section CONCOR:
#'   First a matrix of Pearson correlation coefficients between each pair of 
#'   nodes' profiles in the given motif census is created. 
#'   Then, again, we find the correlations of this square, symmetric matrix,
#'   and continue to do this iteratively until each entry is either `1` or `-1`.
#'   These values are used to split the data into two partitions,
#'   with members either holding the values `1` or `-1`.
#'   This procedure from census to convergence is then repeated within each block,
#'   allowing further partitions to be found.
#'   Unlike UCINET, partitions are continued until there are single members in
#'   each partition.
#'   Then a distance matrix is constructed from records of in which partition phase
#'   nodes were separated, 
#'   and this is given to `stats::hclust()` so that dendrograms etc can be returned.
#' @importFrom stats complete.cases
#' @references 
#' ## On CONCOR clustering
#' Breiger, Ronald L., Scott A. Boorman, and Phipps Arabie. 1975.  
#'   "An Algorithm for Clustering Relational Data with Applications to 
#'   Social Network Analysis and Comparison with Multidimensional Scaling". 
#'   _Journal of Mathematical Psychology_, 12: 328-83.
#'   \doi{10.1016/0022-2496(75)90028-0}.
#' @export
cluster_concor <- function(.data, motif){
  .data <- manynet::expect_nodes(.data)
  split_cor <- function(m0, cutoff = 1) {
    if (ncol(m0) < 2 | all(manynet::to_correlation(m0)==1)) list(m0)
    else {
      mi <- manynet::to_correlation(m0)
      while (any(abs(mi) <= cutoff)) {
        mi <- stats::cor(mi)
        cutoff <- cutoff - 0.0001
      }
      group <- mi[, 1] > 0
      if(all(group)){
       list(m0) 
      } else {
        list(m0[, group, drop = FALSE], 
           m0[, !group, drop = FALSE])
      }
    }
  }
  p_list <- list(t(motif))
  if(is.null(colnames(p_list[[1]]))) 
    colnames(p_list[[1]]) <- paste0("V",1:ncol(p_list[[1]]))
  p_group <- list()
  if(manynet::is_twomode(.data)){
    p_list <- list(p_list[[1]][, !manynet::node_is_mode(.data), drop = FALSE],
                   p_list[[1]][, manynet::node_is_mode(.data), drop = FALSE])
    p_group[[1]] <- lapply(p_list, function(z) colnames(z))
    i <- 2
  } else i <- 1
  while(!all(vapply(p_list, function(x) ncol(x)==1, logical(1)))){
    p_list <- unlist(lapply(p_list,
                            function(y) split_cor(y)),
                     recursive = FALSE)
    p_group[[i]] <- lapply(p_list, function(z) colnames(z))
    if(i > 2 && length(p_group[[i]]) == length(p_group[[i-1]])) break
    i <- i+1
  }
  
  if(manynet::is_labelled(.data)){
    merges <- sapply(rev(1:(i-1)), 
                     function(p) lapply(p_group[[p]], 
                                        function(s){
                                          g <- match(s, manynet::node_names(.data))
                                          if(length(g)==1) c(g, 0, p) else 
                                            if(length(g)==2) c(g, p) else
                                              c(t(cbind(t(utils::combn(g, 2)), p)))
                                        } ))
  } else {
    merges <- sapply(rev(1:(i-1)), 
                     function(p) lapply(p_group[[p]], 
                                        function(s){
                                          g <- as.numeric(gsub("^V","",s))
                                          if(length(g)==1) c(g, 0, p) else 
                                            if(length(g)==2) c(g, p) else
                                              c(t(cbind(t(utils::combn(g, 2)), p)))
                                        } ))
  }
  merges <- c(merges, 
              list(c(t(cbind(t(utils::combn(seq_len(manynet::net_nodes(.data)), 2)), 0)))))
  merged <- matrix(unlist(merges), ncol = 3, byrow = TRUE)
  merged <- merged[!duplicated(merged[,1:2]),]
  merged[,3] <- abs(merged[,3] - max(merged[,3]))
  merged[merged == 0] <- NA
  merged <- merged[stats::complete.cases(merged),]
  merged <- as.data.frame(merged)
  names(merged) <- c("from","to","weight")
  
  distances <- manynet::as_matrix(manynet::as_igraph(merged))
  distances <- distances + t(distances)
  # distances <- distances[-which(rownames(distances)==0),-which(colnames(distances)==0)]
  if(manynet::is_labelled(.data))
    rownames(distances) <- colnames(distances) <- manynet::node_names(.data)
  hc <- hclust(d = as.dist(distances))
  hc$method <- "concor"
  hc$distances <- distances
  hc  
}

# Motifs ####

#' Motifs of nodes brokerage
#' @description
#'   `node_x_brokerage()` returns the Gould-Fernandez brokerage
#'   roles played by nodes in a network.
#'   
#' @name motif_brokerage_node
#' @template param_data
#' @template param_memb
#' @family brokerage
#' @template node_motif
#' @param standardized Whether the score should be standardized
#'   into a _z_-score indicating how many standard deviations above
#'   or below the average the score lies.
NULL

#' @rdname motif_brokerage_node 
#' @references 
#' ## On brokerage motifs
#' Gould, Roger V., and Roberto M. Fernandez. 1989. 
#' “Structures of Mediation: A Formal Approach to Brokerage in Transaction Networks.” 
#' _Sociological Methodology_, 19: 89-126.
#' \doi{10.2307/270949}
#' 
#' Jasny, Lorien, and Mark Lubell. 2015. 
#' “Two-Mode Brokerage in Policy Networks.” 
#' _Social Networks_ 41:36–47. 
#' \doi{10.1016/j.socnet.2014.11.005}
#' @examples 
#' node_x_brokerage(ison_networkers, "Discipline")
#' @export
node_x_brokerage <- function(.data, membership, standardized = FALSE){
  thisRequires("sna")
  .data <- manynet::expect_nodes(.data)
  membership <- .resolve_membership(.data, membership)
  if(!manynet::is_twomode(.data)){
    out <- sna::brokerage(manynet::as_network(.data),
                          membership)
    out <- if(standardized) out$z.nli else out$raw.nli
    colnames(out) <- c("Coordinator", "Itinerant", "Gatekeeper", 
                       "Representative", "Liaison", "Total")
  } else {
    out <- suppressWarnings(sna::brokerage(manynet::as_network(manynet::to_mode1(.data)),
                                           membership))
    out <- if(standardized) out$z.nli else out$raw.nli
    out <- out[,-4]
    colnames(out) <- c("Coordinator", "Itinerant", "Gatekeeper", 
                       "Liaison", "Total")
  }
  make_node_motif(out, .data)
}

#' Motifs of network brokerage
#' @description
#'   `net_x_brokerage()` returns the Gould-Fernandez brokerage
#'   roles in a network.
#'   
#' @name motif_brokerage_net
#' @template param_data
#' @template param_memb
#' @family brokerage
#' @template net_motif
#' @param standardized Whether the score should be standardized
#'   into a _z_-score indicating how many standard deviations above
#'   or below the average the score lies.
NULL

#' @rdname motif_brokerage_net 
#' @examples 
#' net_x_brokerage(ison_networkers, "Discipline")
#' @export
net_x_brokerage <- function(.data, membership, standardized = FALSE){
  thisRequires("sna")
  .data <- manynet::expect_nodes(.data)
  membership <- .resolve_membership(.data, membership)
  if(!manynet::is_twomode(.data)){
    out <- sna::brokerage(manynet::as_network(.data),
                          membership)
    out <- if(standardized) out$z.gli else out$raw.gli
    names(out) <- c("Coordinator", "Itinerant", "Gatekeeper", 
                    "Representative", "Liaison", "Total")
  } else {
    out <- suppressWarnings(sna::brokerage(manynet::as_network(manynet::to_mode1(.data)),
                                           membership))
    out <- if(standardized) out$z.gli else out$raw.gli
    names(out) <- c("Coordinator", "Itinerant", "Gatekeeper", 
                    "Representative", "Liaison", "Total")
  }
  make_network_motif(out, .data)
}

# Measures ####

#' Measures of brokerage
#' @description
#'   These functions include ways to measure nodes' brokerage activity and
#'   exclusivity in a network: 
#'   
#'   - `node_by_brokering_activity()` measures nodes' brokerage activity.
#'   - `node_by_brokering_exclusivity()` measures nodes' brokerage exclusivity. 
#'   
#' @name measure_brokerage
#' @template param_data
#' @template param_memb
#' @template node_measure
#' @family brokerage
NULL

#' @rdname measure_brokerage 
#' @references
#' ## On brokerage activity and exclusivity
#'   Hamilton, Matthew, Jacob Hileman, and Orjan Bodin. 2020.
#'   "Evaluating heterogeneous brokerage: New conceptual and methodological approaches
#'   and their application to multi-level environmental governance networks"
#'   _Social Networks_ 61: 1-10.
#'   \doi{10.1016/j.socnet.2019.08.002}
#' @export
node_by_brokering_activity <- function(.data, membership){
  .data <- manynet::expect_nodes(.data)
  twopaths <- .to_twopaths(.data)
  if(!missing(membership)){
    membership <- .resolve_membership(.data, membership)
    twopaths$from_memb <- membership[`if`(manynet::is_labelled(.data),
                                                                          match(twopaths$from, manynet::node_names(.data)),
                                                                          twopaths$from)]
    twopaths$to_memb <- membership[`if`(manynet::is_labelled(.data),
                                                                        match(twopaths$to.y, manynet::node_names(.data)),
                                                                        twopaths$to.y)]
    twopaths <- dplyr::filter(twopaths, from_memb != to_memb)
  }
  # tabulate brokerage
  out <- c(table(twopaths$to))
  # correct ordering
  if(manynet::is_labelled(.data)) out <- out[match(manynet::node_names(.data), names(out))] else {
    temp <- rep(0, manynet::net_nodes(.data))
    temp[as.numeric(names(out))] <- out
    out <- temp
  }
  # missings should be none
  out[is.na(out)] <- 0
  make_node_measure(out, .data)
}

#' @rdname measure_brokerage 
#' @examples
#' node_by_brokering_exclusivity(ison_networkers, "Discipline")
#' @export
node_by_brokering_exclusivity <- function(.data, membership){
  .data <- manynet::expect_nodes(.data)
  twopaths <- .to_twopaths(.data)
  if(!missing(membership)){
    membership <- .resolve_membership(.data, membership)
    twopaths$from_memb <- membership[`if`(manynet::is_labelled(.data),
                                                                          match(twopaths$from, manynet::node_names(.data)),
                                                                          twopaths$from)]
    twopaths$to_memb <- membership[`if`(manynet::is_labelled(.data),
                                                                        match(twopaths$to.y, manynet::node_names(.data)),
                                                                        twopaths$to.y)]
    twopaths <- dplyr::filter(twopaths, from_memb != to_memb)
  }
  # get only exclusive paths
  out <- twopaths |> dplyr::group_by(from, to.y) |> dplyr::filter(dplyr::n()==1)
  # tabulate brokerage
  out <- c(table(out$to))
  # correct ordering for named data
  if(manynet::is_labelled(.data)) out <- out[match(manynet::node_names(.data), names(out))] else {
    temp <- rep(0, manynet::net_nodes(.data))
    temp[as.numeric(names(out))] <- out
    out <- temp
  }
  # missings should be none
  out[is.na(out)] <- 0
  make_node_measure(out, .data)
}

# Memberships ####

#' Memberships in brokerage positions
#' 
#' @description
#'   `node_in_brokerage()` returns nodes membership as a powerhouse,
#'   connector, linchpin, or sideliner according to Hamilton et al. (2020).
#'   
#' @name member_brokerage
#' @template param_data
#' @template param_memb
#' @family brokerage
#' @references
#' ## On brokerage activity and exclusivity
#'   Hamilton, Matthew, Jacob Hileman, and Orjan Bodin. 2020.
#'   "Evaluating heterogeneous brokerage: New conceptual and methodological approaches
#'   and their application to multi-level environmental governance networks"
#'   _Social Networks_ 61: 1-10.
#'   \doi{10.1016/j.socnet.2019.08.002}
#' @template node_member
NULL

#' @rdname member_brokerage 
#' @export
node_in_brokering <- function(.data, membership){
  .data <- manynet::expect_nodes(.data)
  activ <- node_by_brokering_activity(.data, membership)
  exclusiv <- node_by_brokering_exclusivity(.data, membership)
  activ <- activ - mean(activ)
  exclusiv <- exclusiv - mean(exclusiv)
  out <- dplyr::case_when(activ > 0 & exclusiv > 0 ~ "Powerhouse",
                          activ > 0 & exclusiv < 0 ~ "Connectors",
                          activ < 0 & exclusiv > 0 ~ "Linchpins",
                          activ < 0 & exclusiv < 0 ~ "Sideliners")
  make_node_member(out, .data)
}

.to_twopaths <- function(.data){
  to <- from <- to.y <- NULL
  if(!manynet::is_directed(.data)){
    el <- manynet::as_edgelist(manynet::to_reciprocated(.data)) 
  } else el <- manynet::as_edgelist(.data)
  twopaths <- dplyr::full_join(el, el, 
                               by = dplyr::join_by(to == from), 
                               relationship = "many-to-many")
  # remove non two-paths
  twopaths <- dplyr::filter(twopaths, !(is.na(from) | is.na(to.y)))
  # remove reciprocated paths
  twopaths <- dplyr::filter(twopaths, from != to.y)
  # remove triads
  twopaths <- dplyr::filter(twopaths, !paste(from, to.y) %in% paste(from, to))
  twopaths
}

test_that("node_by_brokering_activity works", {
  test <- node_by_brokering_activity(ison_networkers, "Discipline")
  expect_s3_class(test, "node_measure")
  expect_equal(c(net_nodes(ison_networkers)), length(test))
  expect_equal(top3(test), c(333,207,3))
})

test_that("node_brokering_exclusivity works", {
  test <- node_by_brokering_exclusivity(ison_networkers, "Discipline")
  expect_s3_class(test, "node_measure")
  expect_equal(c(net_nodes(ison_networkers)), length(test))
  expect_equal(top3(test), c(1,0,0))
})

test_that("node_in_brokering works", {
  test <- node_in_brokering(ison_networkers, "Discipline")
  expect_s3_class(test, "node_member")
  expect_equal(c(net_nodes(ison_networkers)), length(test))
  expect_equal(top3(test), c("Powerhouse","Connectors","Sideliners"))
  expect_output(print(node_in_brokering(to_uniplex(fict_marvel, "affiliation"))), "4 groups")
  expect_output(print(summary(node_in_brokering(to_uniplex(fict_marvel, "affiliation")))), "Connectors")
})

# Motifs ####

#' Motifs of network hierarchy
#' @name motif_hierarchy
#' @description
#'   `net_x_hierarchy()` collects the measures of hierarchy into a single motif,
#'   which can be used to compare the relative hierarchy of different networks.
#'   The measures of hierarchy are:
#'   - `net_by_connectedness()` measures the proportion of dyads in the network
#'   that are reachable to one another,
#'   or the degree to which network is a single component.
#'   - `net_by_efficiency()` measures the Krackhardt efficiency score.
#'   - `net_by_upperbound()` measures the Krackhardt (least) upper bound
#'   score.
#'   - `net_by_reciprocity()` measures the proportion of ties in the network that 
#'   are reciprocated,
#'   which is a measure of the degree to which the network is non-hierarchical.
#' 
#' @template param_data
#' @family hierarchy
#' @template net_motif
#' @references
#' ## On hierarchy
#' Krackhardt, David. 1994. 
#' Graph theoretical dimensions of informal organizations. 
#' In Carley and Prietula (eds) _Computational Organizational Theory_, 
#' Hillsdale, NJ: Lawrence Erlbaum Associates. Pp. 89-111. 
#' 
#' Everett, Martin, and David Krackhardt. 2012. 
#' “A second look at Krackhardt's graph theoretical dimensions of informal organizations.”
#' _Social Networks_, 34: 159-163.
#' \doi{10.1016/j.socnet.2011.10.006}
#' @examples 
#' net_x_hierarchy(ison_networkers)
NULL

#' @rdname motif_hierarchy 
#' @export
net_x_hierarchy <- function(.data){
  .data <- manynet::expect_nodes(.data)
  out <- data.frame(Connectedness = net_by_connectedness(.data),
                    InvReciprocity = 1 - net_by_reciprocity(.data),
                    Efficiency = net_by_efficiency(.data),
                    LeastUpperBound = net_by_upperbound(.data))
  make_network_motif(out, .data)
}

# Measures ####

#' Measures of hierarchy
#' @name measure_hierarchy
#' @description
#'   These functions, together with `net_reciprocity()`, are used jointly to
#'   measure how hierarchical a network is:
#'   
#'   - `net_by_connectedness()` measures the proportion of dyads in the network
#'   that are reachable to one another, 
#'   or the degree to which network is a single component.
#'   - `net_by_efficiency()` measures the Krackhardt efficiency score.
#'   - `net_by_upperbound()` measures the Krackhardt (least) upper bound score.
#' 
#' @template param_data
#' @template net_measure
#' @family hierarchy
#' @references
#' ## On hierarchy
#' Krackhardt, David. 1994. 
#' Graph theoretical dimensions of informal organizations. 
#' In Carley and Prietula (eds) _Computational Organizational Theory_, 
#' Hillsdale, NJ: Lawrence Erlbaum Associates. Pp. 89-111. 
#' 
#' Everett, Martin, and David Krackhardt. 2012. 
#' “A second look at Krackhardt's graph theoretical dimensions of informal organizations.”
#' _Social Networks_, 34: 159-163.
#' \doi{10.1016/j.socnet.2011.10.006}
#' @examples 
#' net_by_connectedness(ison_networkers)
#' 1 - net_by_reciprocity(ison_networkers)
#' net_by_efficiency(ison_networkers)
#' net_by_upperbound(ison_networkers)
NULL

#' @rdname measure_hierarchy 
#' @export
net_by_connectedness <- function(.data){
  .data <- manynet::expect_nodes(.data)
  dists <- igraph::distances(manynet::as_igraph(.data))
  make_network_measure(1 - sum(dists==Inf)/sum(dists!=0),
                       .data, 
                       call = deparse(sys.call()))
}

#' @rdname measure_hierarchy 
#' @export
net_by_efficiency <- function(.data) {
  .data <- manynet::expect_nodes(.data)
  degs <- node_by_indegree(.data, normalized = FALSE)
  out <- (manynet::net_nodes(.data)-1)/sum(degs)
  make_network_measure(out, .data, 
                       call = deparse(sys.call()))
}

#' @rdname measure_hierarchy 
#' @export
net_by_upperbound <- function(.data) {
  .data <- manynet::expect_nodes(.data)
  dists <- igraph::distances(.data, mode = "in")
  dists[is.infinite(dists)] <- 0
  dists <- dists[order(rowSums(dists)), order(rowSums(dists))]
  if (max(colSums(dists > 0)) / (manynet::net_nodes(.data)-1) == 1){
    out <- 1
  } else {
    out <- apply(utils::combn(2:nrow(dists), 2), 2, 
                 function(x){
                   ubs <- dists[x,]>0
                   any(ubs[1,]*ubs[2,]==1)
                 })
    out <- sum(out)/length(out)
  }
  make_network_measure(out, .data, 
                       call = deparse(sys.call()))
}

# Test hierarchy measures

test_that("net_connectedness works correctly", {
  connect_judo <- net_by_connectedness(ison_judo_moves)
  # Basic functionality tests

  # Return type and range tests
  expect_true(is.numeric(as.numeric(connect_judo)))
  expect_true(as.numeric(connect_judo) >= 0)
  expect_true(as.numeric(connect_judo) <= 1)
  
  # Test with complete graph (should be 1)
  expect_equal(round(as.numeric(net_by_connectedness(create_filled(5))), 4), 1)
  expect_equal(round(as.numeric(net_by_connectedness(create_empty(5))), 4), 0)
  
  # Test edge case: single node
  expect_false(is.finite(as.numeric(net_by_connectedness(create_empty(1)))))
})

test_that("net_efficiency works correctly", {
  # Basic functionality tests
  effic_judo <- net_by_efficiency(ison_judo_moves)

  # Return type tests
  expect_true(is.numeric(as.numeric(effic_judo)))
  expect_true(as.numeric(effic_judo) > 0)
})

test_that("net_upperbound works correctly", {
  # Basic functionality tests
  upper_judo <- net_by_efficiency(ison_judo_moves)

  # Return type and range tests
  expect_true(is.numeric(as.numeric(upper_judo)))
  expect_true(as.numeric(upper_judo) >= 0)
  expect_true(as.numeric(upper_judo) <= 1)
  
  # Test with perfect hierarchy (should approach 1)
  # Create a tournament-like structure
  perfect_hierarchy <- create_tree(5)
  expect_equal(as.numeric(net_by_upperbound(perfect_hierarchy)), 1)
})

test_that("net_x_hierarchy works correctly", {
  result <- net_x_hierarchy(ison_judo_moves)
  # Basic functionality tests

  # Check that it returns a data frame with correct columns
  expect_true(is.data.frame(result))
  expect_named(result, c("Connectedness", "InvReciprocity", "Efficiency", "LeastUpperBound"))
  
  # Check that all values are numeric and within expected ranges
  expect_true(all(sapply(result, is.numeric)))
  expect_true(result$Connectedness >= 0 && result$Connectedness <= 1)
  expect_true(result$InvReciprocity >= 0 && result$InvReciprocity <= 1)
  expect_true(result$Efficiency > 0)  # Should be positive (can be > 1)
  expect_true(result$LeastUpperBound >= 0 && result$LeastUpperBound <= 1)
})

# Marking core ####

#' Marking nodes as core or periphery
#' @name mark_core
#' @description
#'   `node_is_core()` identifies whether nodes belong to the core of the 
#'   network, as opposed to the periphery.
#' 
#' @template param_data
#' @family core-periphery
#' @template node_mark
#' @param centrality Which centrality measure to use to identify cores and periphery.
#'   By default this is "degree", 
#'   which relies on the heuristic that high degree nodes are more likely to be in the core.
#'   An alternative is "eigenvector", which instead begins with high eigenvector nodes.
#'   Other methods, such as a genetic algorithm, CONCOR, and Rombach-Porter,
#'   can be added if there is interest.
NULL

#' @rdname mark_core
#' @section Core-periphery: 
#'   This function is used to identify which nodes should belong to the core,
#'   and which to the periphery.
#'   It seeks to minimize the following quantity:
#'   \deqn{Z(S_1) = \sum_{(i<j)\in S_1} \textbf{I}_{\{A_{ij}=0\}} + \sum_{(i<j)\notin S_1} \textbf{I}_{\{A_{ij}=1\}}}
#'   where nodes \eqn{\{i,j,...,n\}} are ordered in descending degree,
#'   \eqn{A} is the adjacency matrix,
#'   and the indicator function is 1 if the predicate is true or 0 otherwise.
#'   Note that minimising this quantity maximises density in the core block
#'   and minimises density in the periphery block;
#'   it ignores ties between these blocks.
#' @references
#' ## On core-periphery partitioning
#' Borgatti, Stephen P., & Everett, Martin G. 1999. 
#' "Models of core /periphery structures". 
#' _Social Networks_, 21, 375–395. 
#' \doi{10.1016/S0378-8733(99)00019-2}
#' 
#' Lip, Sean Z. W. 2011. 
#' “A Fast Algorithm for the Discrete Core/Periphery Bipartitioning Problem.”
#' \doi{10.48550/arXiv.1102.5511}
#' @examples 
#' node_is_core(ison_adolescents)
#' ison_adolescents |> 
#'    mutate(corep = node_is_core())
#' @export
node_is_core <- function(.data, centrality = c("degree", "eigenvector")){
  .data <- manynet::expect_nodes(.data)
  centrality <- match.arg(centrality)
  if(manynet::is_directed(.data)) warning("Asymmetric core-periphery not yet implemented.")
  if(centrality == "degree"){
    degi <- node_by_degree(.data, normalized = FALSE, 
                           alpha = ifelse(manynet::is_weighted(.data), 1, 0))
  } else if (centrality == "eigenvector") {
    degi <- node_by_eigenvector(.data, normalized = FALSE)
  } else manynet::snet_abort("This function expects either 'degree' or 'eigenvector' method to be specified.")
  nord <- order(degi, decreasing = TRUE)
  zbest <- manynet::net_nodes(.data)*3
  kbest <- 0
  z <- 1/2*sum(degi)
  for(k in 1:(manynet::net_nodes(.data)-1)){
    z <- z + k - 1 - degi[nord][k]
    if(z < zbest){
      zbest <- z
      kbest <- k
    }
  }
  out <- ifelse(seq_len(manynet::net_nodes(.data)) %in% nord[seq_len(kbest)],
                1,2)
  make_node_mark(out==1, .data)
}

# Measuring core ####

#' Measuring nodes' coreness
#' @name measure_core
#' @description
#'   These functions identify nodes belonging to (some level of) the core of a network:
#'   
#'   - `node_by_coreness()` returns a continuous measure of how closely each node
#'   resembles a typical core node.
#'   - `node_by_kcoreness()` assigns nodes to their level of k-coreness.
#' 
#' @template param_data
#' @family core-periphery
#' @template node_measure
NULL

#' @rdname measure_core
#' @section k-coreness:
#'   k-coreness captures the maximal subgraphs in which each vertex has at least
#'   degree _k_, where _k_ is also the order of the subgraph.
#'   As described in `igraph::coreness`,
#'   a node's coreness is _k_ if it belongs to the _k_-core
#'   but not to the (_k_+1)-core.
#' @references
#' ## On k-coreness
#' Seidman, Stephen B. 1983. 
#' "Network structure and minimum degree". 
#' _Social Networks_, 5(3), 269-287.
#' \doi{10.1016/0378-8733(83)90028-X}
#' 
#' Batagelj, Vladimir, and Matjaz Zaversnik. 2003. 
#' "An O(m) algorithm for cores decomposition of networks". 
#' _arXiv preprint_ cs/0310049.
#' \doi{10.48550/arXiv.cs/0310049}
#' @examples
#' node_by_kcoreness(ison_adolescents)
#' @export
node_by_kcoreness <- function(.data){
  .data <- manynet::expect_nodes(.data)
  if(!manynet::is_graph(.data)) .data <- manynet::as_igraph(.data)
  out <- igraph::coreness(.data)
  make_node_measure(out, .data)
}

#' @rdname measure_core
#' @examples
#' node_by_coreness(ison_adolescents)
#' @export
node_by_coreness <- function(.data) {
  .data <- manynet::expect_nodes(.data)
  A <- manynet::as_matrix(.data)
  n <- nrow(A)
  obj_fun <- function(c) {
    ideal <- outer(c, c)
    val <- suppressWarnings(cor(as.vector(A), as.vector(ideal)))
    if (!is.finite(val)) return(1e6)  # Penalize non-finite values
    return(-val)  # Negative for maximization
  }
  # Initial guess: all nodes have coreness 0.5
  init <- rep(0.5, n)
  result <- stats::optim(init, obj_fun, method = "L-BFGS-B", 
                         lower = 0, upper = 1)
  make_node_measure(result$par, .data)
}

# Membering core ####

#' Memberships in core-periphery categories
#' @name member_core
#' @description
#'   `node_in_core()` categorizes nodes into two or more core/periphery
#'   categories based on their coreness.
#' 
#' @template param_data
#' @family core-periphery
#' @template node_member
NULL

#' @rdname member_core
#' @param groups Number of categories to create. Must be at least 2 and at most
#'   the number of nodes in the network. Default is 3.
#' @param cluster_by Method to use to create the categories.
#'   One of "bins" (equal-width bins), "quantiles" (quantile-based bins),
#'   or "kmeans" (k-means clustering). Default is "bins".
#' @section Core-periphery categories:
#'   This function categorizes nodes based on their coreness into a specified
#'   number of groups. The groups are labeled as "Core", "Semi-core",
#'   "Semi-periphery", and "Periphery" depending on the number of groups
#'   specified.
#'   The categorization can be done using different methods: equal-width bins,
#'   quantile-based bins, or k-means clustering.
#' @references
#' ## On core-periphery categorization
#' Wallerstein, Immanuel. 1974.
#' "Dependence in an Interdependent World: The Limited Possibilities of Transformation Within the Capitalist World Economy."
#' _African Studies Review_, 17(1), 1-26.
#' \doi{https://doi.org/10.2307/523574}
#' @examples
#' node_in_core(ison_adolescents)
#' @export
node_in_core <- function(.data, groups = 3,
                         cluster_by = c("bins","quantiles","kmeans")) {
  if (groups < 2) manynet::snet_abort("Number of categories must be at least 2")
  if (groups > manynet::net_nodes(.data)) manynet::snet_abort("There cannot be more categories than nodes.")
  .data <- manynet::expect_nodes(.data)
  contin <- node_by_coreness(.data)
  cluster_by <- match.arg(cluster_by)
  out <- switch(cluster_by,
                bins = cut(as.numeric(contin), breaks = groups, labels = FALSE),
                quantiles = as.numeric(cut(as.numeric(contin),
                                           breaks = stats::quantile(as.numeric(contin),
                                                                    probs = seq(0, 1, length.out = groups + 1)),
                                           include.lowest = TRUE, labels = FALSE)),
                kmeans = stats::kmeans(as.numeric(contin), centers = groups)$cluster
  )
  
  if (groups == 2) core_labels <- c("Core", "Periphery")
  if (groups == 3) core_labels <- c("Core", "Semi-periphery", "Periphery")
  if (groups == 4) core_labels <- c("Core", "Semi-core", "Semi-periphery", "Periphery")
  if (groups >= 5){
    n_middle <- groups - 2
    middle <- character(n_middle)
    
    for (i in seq_len(n_middle)) {
      if (i %% 2 == 1) {
        middle[i] <- paste0("Semi-periphery-", (i + 1) %/% 2)
      } else {
        middle[i] <- paste0("Semi-core-", i %/% 2)
      }
    }
    middle <- middle[order(middle)]
    
    core_labels <- c("Core", middle, "Periphery")
    if(groups == 5) core_labels[2] <- "Semi-core"
  } 
  out <- rev(core_labels)[out]
  make_node_member(out, .data)
}

test_that("node_is_universal works", {
  expect_true(any(node_is_universal(create_star(11))))
})

test_that("node_kcoreness works", {
  expect_equal(top3(node_by_kcoreness(ison_adolescents)), c(1,2,2))
})

test_that("node_in_core works", {
  expect_equal(top3(node_in_core(ison_adolescents)), c("Periphery","Core","Core"))
  expect_output(print(node_in_core(ison_adolescents, groups = 5)), "Semi-periphery-1")
})

#' Methods for selecting clusters
#' 
#' @description
#'   Finding the optimal number of clusters is generally a balance between
#'   optimal fit statistics, parsimony, and interpretability.
#'   These functions help select the number of clusters to return from `hc`,
#'   some hierarchical clustering object:
#'   
#'   - `k_strict()` selects a number of clusters in which there is no
#'   distance between cluster members.
#'   - `k_elbow()` selects a number of clusters in which there is 
#'   a fair trade-off between parsimony and fit according to the elbow method.
#'   - `k_silhouette()` selects a number of clusters that
#'   maximises the silhouette score.
#' 
#'   These functions are generally not user-facing but used internally
#'   in e.g. the `*_equivalence()` functions.
#'   
#' @inheritParams mark_nodes
#' @param hc A hierarchical clustering object.
#' @returns
#'   A single integer indicating the number of clusters to return.
#' @name method_kselect
NULL

#' @rdname method_kselect 
#' @section Strict method:
#'   The strict method selects the number of clusters in which there is no 
#'   distance between cluster members. 
#'   This is a very conservative method that may be appropriate when the goal is 
#'   to identify clusters of nodes that are exactly the same. 
#'   However, it may not be appropriate in cases where the data is noisy or 
#'   when the clusters are not well-defined, 
#'   as it may result in a large number of small clusters.
#' @export
k_strict <- function(hc, .data){
  zero_merged <- hc$merge[round(hc$height,4) == 0,]
  k <- nrow(zero_merged) + manynet::net_nodes(.data) - sum(zero_merged < 0) + sum(zero_merged > 0)
  k
}

#' @rdname method_kselect 
#' @param motif A motif census object.
#' @param Kmax An integer indicating the maximum number of options to consider.
#'   The minimum of this and the number of nodes in the network is used.
#' @section Elbow method:
#'   The elbow method is a heuristic used in cluster analysis to determine the optimal number of clusters.
#'   It is based on the idea of plotting the within cluster correlation
#'   as a function of the number of clusters and looking for an "elbow"
#'   where there is a significant decrease in the rate of improvement in 
#'   correlation as the number of clusters increases.
#'   The point at which the elbow occurs is often considered a good choice for 
#'   the number  of clusters, as it represents a balance between 
#'   model complexity and fit to the data.
#' @references 
#' ## On the elbow method
#'  Thorndike, Robert L. 1953. 
#'    "Who Belongs in the Family?". 
#'    _Psychometrika_, 18(4): 267–76. 
#'    \doi{10.1007/BF02289263}.
#' @export
k_elbow <- function(hc, .data, motif, Kmax){
  
  thisRequires("sna")
  
  clusterCorr <- function(observed_cor_matrix, cluster_vector) {
    num_vertices = nrow(observed_cor_matrix)
    cluster_cor_mat <- observed_cor_matrix
    
    obycor <- function(i, j) 
      mean(observed_cor_matrix[which(cluster_vector[row(observed_cor_matrix)] ==
                                       cluster_vector[i] &
                                       cluster_vector[col(observed_cor_matrix)] ==
                                       cluster_vector[j])])
    obycor_v <- Vectorize(obycor)
    cluster_cor_mat <- outer(1:num_vertices,
                             1:num_vertices,
                             obycor_v)
    dimnames(cluster_cor_mat) <- dimnames(observed_cor_matrix)
    cluster_cor_mat
  }
  
  elbow_finder <- function(x_values, y_values) {
    # Max values to create line
    if(min(x_values)==1) x_values <- x_values[2:length(x_values)]
    if(min(y_values)==0) y_values <- y_values[2:length(y_values)]
    max_df <- data.frame(x = c(min(x_values), max(x_values)), 
                         y = c(min(y_values), max(y_values)))
    # Creating straight line between the max values
    fit <- stats::lm(max_df$y ~ max_df$x)
    # Distance from point to line
    distances <- vector()
    for (i in seq_len(length(x_values))) {
      distances <- c(distances,
                     abs(stats::coef(fit)[2]*x_values[i] -
                           y_values[i] +
                           coef(fit)[1]) /
                       sqrt(stats::coef(fit)[2]^2 + 1^2))
    }
    # Max distance point
    x_max_dist <- x_values[which.max(distances)]
    x_max_dist
  }
  
  vertices <- manynet::net_nodes(.data)
  observedcorrelation <- cor(t(motif))
  
  resultlist <- list()
  correlations <- vector()
  for (i in 2:min(Kmax, vertices)) {
    cluster_result <- list(label = NA, clusters = NA, correlation = NA)
    cluster_result$label <- paste("number of clusters: ", 
                                  i)
    clusters <- stats::cutree(hc, k = i)
    cluster_result$clusters <- clusters
    cluster_cor_mat <- clusterCorr(observedcorrelation, clusters)
    clustered_observed_cors <- sna::gcor(cluster_cor_mat, observedcorrelation)
    cluster_result$correlation <- (clustered_observed_cors)
    resultlist <- c(resultlist, cluster_result)
    correlations <- c(correlations, clustered_observed_cors)
  }
  
  resultlist$correlations <- c(correlations)
  dafr <- data.frame(clusters = 2:min(Kmax, vertices), 
                     correlations = c(correlations))
  correct <- NULL # to satisfy the error god
  
  # k identification method
  elbow_finder(dafr$clusters, dafr$correlations)
}

#' @rdname method_kselect 
#' @section Silhouette method: 
#'   The silhouette method is based on the concept of cohesion and separation. 
#'   Cohesion refers to how closely related the nodes within a cluster are, 
#'   while separation refers to how distinct the clusters are from each other. 
#'   The silhouette score combines these two concepts into a single metric that 
#'   can be used to evaluate the quality of a clustering solution. 
#'   The silhouette score is calculated as follows:
#'   For each node, calculate the average distance to all other nodes in the same cluster (a) 
#'   and the average distance to all other nodes in the next nearest cluster (b). 
#'   The silhouette score for each node is then calculated as:
#'   \deqn{S(i) = \frac{b - a}{\max(a, b)}}
#'   A higher silhouette score indicates that the node is well-matched to its 
#'   own cluster and poorly matched to neighboring clusters.
#'   The silhouette score for the entire clustering is the average silhouette score across all nodes. 
#'   Maximizing the silhouette score across a range of potential clusterings 
#'   allows researchers to identify the number of clusters that best captures 
#'   the underlying structure of the data.
#'   It is particularly useful when the clusters are well-separated.
#' @references 
#' ## On the silhouette method
#' Rousseeuw, Peter J. 1987. 
#'   “Silhouettes: A Graphical Aid to the Interpretation and Validation of Cluster Analysis.” 
#'   _Journal of Computational and Applied Mathematics_, 20: 53–65. 
#'   \doi{10.1016/0377-0427(87)90125-7}.
#' @export
k_silhouette <- function(hc, .data, Kmax){
  if(missing(Kmax)) Kmax <- length(hc$order) else
    Kmax <- min(Kmax, length(hc$order))
  kcs <- 2:min(Kmax, manynet::net_nodes(.data))
  ns <- seq_len(manynet::net_nodes(.data))
  distances <- hc$distances
  ks <- vector()
  for(kc in kcs){
    cand <- stats::cutree(hc, kc)
    ai <- vector()
    bi <- vector()
    for(i in ns){
      wig <- which(cand == cand[i])
      wig <- wig[wig != i]
      ai <- c(ai, 
              ifelse(length(wig)==0,
                     0, mean(as.matrix(distances)[i, wig])))
      wog <- which(cand != cand[i])
      bi <- c(bi, min(vapply(unique(cand[wog]), function(b){
        mean(as.matrix(distances)[i, wog[cand[wog]==b]])
      }, FUN.VALUE = numeric(1))))
    }
    si <- (bi - ai)/
      apply(data.frame(ai, bi), 1, max)
    ks <- c(ks, mean(si))
  }
  k <- which(ks == max(ks)) + 1
  k
}

#' @rdname method_kselect
#' @param sims Integer of how many simulations should be generated as a
#'   reference distribution.
#' @export
k_gap <- function(hc, motif, Kmax, sims = 100) {
  
  if(missing(Kmax)) Kmax <- length(hc$order) else
    Kmax <- min(Kmax, length(hc$order))
  
  # --- helper: within-cluster dispersion Wk ---
  within_disp <- function(motif, clusters) {
    sum(sapply(split(motif, clusters), function(group) {
      group <- matrix(group, ncol = ncol(motif))
      if (nrow(group) <= 1) return(0)
      center <- colMeans(group)
      sum(rowSums((group - center)^2))
    }))
  }
  
  n <- nrow(motif)
  p <- ncol(motif)
  
  # bounding box for reference datasets
  mins <- apply(motif, 2, min)
  maxs <- apply(motif, 2, max)
  
  # storage
  logW <- numeric(Kmax)
  logW_ref <- matrix(0, nrow = sims, ncol = Kmax)
  
  # --- real data W_k ---
  for (k in 1:Kmax) {
    cl <- cutree(hc, k)
    logW[k] <- log(within_disp(motif, cl))
  }
  
  # --- reference datasets ---
  for (b in 1:sims) {
    ref <- matrix(stats::runif(n * p, mins, maxs), nrow = n)
    d_ref <- stats::dist(ref)
    hc_ref <- hclust(d_ref, method = hc$method)
    
    for (k in 1:Kmax) {
      cl_ref <- cutree(hc_ref, k)
      logW_ref[b, k] <- log(within_disp(ref, cl_ref))
    }
  }
  
  # --- gap statistic ---
  gap <- colMeans(logW_ref) - logW
  se  <- sqrt(1 + 1/B) * apply(logW_ref, 2, stats::sd)
  
  # --- Tibshirani 1-SE rule ---
  k <- which(gap[-Kmax] >= gap[-1] - se[-1])[1]
  k
}

# Degree-like centralities ####

#' Measuring nodes degree-like centrality
#' @name measure_central_degree
#' @description
#'   These functions calculate common degree-related centrality measures for one- and two-mode networks:
#'   
#'   - `node_by_degree()` measures the degree centrality of nodes in an unweighted network,
#'   or weighted degree/strength of nodes in a weighted network; 
#'   there are several related shortcut functions:
#'     - `node_by_deg()` returns the unnormalised results.
#'     - `node_by_indegree()` returns the `direction = 'in'` results.
#'     - `node_by_outdegree()` returns the `direction = 'out'` results.
#'   - `node_by_multidegree()` measures the ratio between types of ties in a multiplex network.
#'   - `node_by_posneg()` measures the PN (positive-negative) centrality of a signed network.
#'   - `node_by_leverage()` measures the leverage centrality of nodes in a network.
#'   
#'   All measures attempt to use as much information as they are offered,
#'   including whether the networks are directed, weighted, or multimodal.
#'   If this would produce unintended results, 
#'   first transform the salient properties using e.g. [manynet::to_undirected()] functions.
#'   All centrality and centralization measures return normalized measures by default,
#'   including for two-mode networks.
#' @template param_data
#' @template param_norm
#' @template param_dir
#' @family degree
#' @family centrality
#' @template node_measure
#' @param alpha Numeric scalar, the positive tuning parameter introduced in
#'   Opsahl et al (2010) for trading off between degree and strength centrality measures.
#'   By default, `alpha = 0`, which ignores tie weights and the measure is solely based
#'   upon degree (the number of ties).
#'   `alpha = 1` ignores the number of ties and provides the sum of the tie weights 
#'   as strength centrality.
#'   Values between 0 and 1 reflect different trade-offs in the relative contributions of
#'   degree and strength to the final outcome, with 0.5 as the middle ground.
#'   Values above 1 penalise for the number of ties.
#'   Of two nodes with the same sum of tie weights, the node with fewer ties will obtain
#'   the higher score.
#'   This argument is ignored except in the case of a weighted network.
#' @importFrom igraph graph_from_incidence_matrix is_bipartite degree V
#' @references 
#' ## On multimodal centrality
#' Faust, Katherine. 1997. 
#' "Centrality in affiliation networks." 
#' _Social Networks_ 19(2): 157-191.
#' \doi{10.1016/S0378-8733(96)00300-0}
#' 
#' Borgatti, Stephen P., and Martin G. Everett. 1997. 
#' "Network analysis of 2-mode data." 
#' _Social Networks_ 19(3): 243-270.
#' \doi{10.1016/S0378-8733(96)00301-2}
#' 
#' Borgatti, Stephen P., and Daniel S. Halgin. 2011. 
#' "Analyzing affiliation networks." 
#' In _The SAGE Handbook of Social Network Analysis_, 
#' edited by John Scott and Peter J. Carrington, 417–33. 
#' London, UK: Sage.
#' \doi{10.4135/9781446294413.n28}
#' 
#' ## On strength centrality
#' Opsahl, Tore, Filip Agneessens, and John Skvoretz. 2010. 
#' "Node centrality in weighted networks: Generalizing degree and shortest paths." 
#' _Social Networks_ 32, 245-251.
#' \doi{10.1016/j.socnet.2010.03.006}
#' @examples
#' node_by_degree(ison_southern_women)
NULL

#' @rdname measure_central_degree 
#' @section Degree centrality: 
#'   `r manynet:::glossies$degree`
#'   It is also sometimes called the valency of a node, \eqn{d(v)}.
#'   The maximum degree in a network is often denoted \eqn{\Delta (G)} and
#'   the minimum degree in a network \eqn{\delta (G)}.
#'   The total degree of a network is the sum of all degrees, \eqn{\sum_v d(v)}.
#'   The degree sequence is the set of all nodes' degrees,
#'   ordered from largest to smallest.
#'   Directed networks discriminate between 
#'   outdegree (degree of outgoing ties) and
#'   indegree (degree of incoming ties).
#' @importFrom manynet as_igraph is_weighted tie_weights is_twomode is_complex
#' @export
node_by_degree <- function (.data, normalized = TRUE, alpha = 1,
                         direction = c("all","out","in")){
  .data <- manynet::expect_nodes(.data)
  graph <- manynet::as_igraph(.data)
  weights <- `if`(manynet::is_weighted(.data), 
                  manynet::tie_weights(.data), NA)
  direction <- match.arg(direction)
  
  # Do the calculations
  if (manynet::is_twomode(graph) & normalized){
    degrees <- igraph::degree(graph = graph, 
                              v = igraph::V(graph), 
                              mode = direction, 
                              loops = manynet::is_complex(.data))
    other_set_size <- ifelse(igraph::V(graph)$type, 
                             sum(!igraph::V(graph)$type), 
                             sum(igraph::V(graph)$type))
    out <- degrees/other_set_size
  } else {
    if (all(is.na(weights))) {
      out <- igraph::degree(graph = graph, v = igraph::V(graph), 
                     mode = direction, 
                     loops = manynet::is_complex(.data),
                     normalized = normalized)
    }
    else {
      ki <- igraph::degree(graph = graph, v = igraph::V(graph), 
                     mode = direction, 
                     loops = manynet::is_complex(.data))
      si <- igraph::strength(graph = graph, vids = igraph::V(graph), 
                       mode = direction,
                       loops = manynet::is_complex(.data), weights = weights)
      out <- ki * (si/ki)^alpha
      out[is.nan(out)] <- 0
      if(normalized) out <- out/max(out)
    }
  }
  out <- make_node_measure(out, .data)
  out
}

#' @rdname measure_central_degree
#' @export
node_by_deg <- function (.data, alpha = 0, direction = c("all","out","in")){
  .data <- manynet::expect_nodes(.data)
  node_by_degree(.data, normalized = FALSE, alpha = alpha, direction = direction)
}

#' @rdname measure_central_degree
#' @export
node_by_outdegree <- function (.data, normalized = TRUE, alpha = 0){
  .data <- manynet::expect_nodes(.data)
  node_by_degree(.data, normalized = normalized, alpha = alpha, direction = "out")
}

#' @rdname measure_central_degree
#' @export
node_by_indegree <- function (.data, normalized = TRUE, alpha = 0){
  .data <- manynet::expect_nodes(.data)
  node_by_degree(.data, normalized = normalized, alpha = alpha, direction = "in")
}

#' @rdname measure_central_degree
#' @param tie1 Character string indicating the first uniplex network.
#' @param tie2 Character string indicating the second uniplex network.
#' @export
node_by_multidegree <- function (.data, tie1, tie2){
  .data <- manynet::expect_nodes(.data)
  stopifnot(manynet::is_multiplex(.data))
  out <- node_by_degree(manynet::to_uniplex(.data, tie1)) - 
    node_by_degree(manynet::to_uniplex(.data, tie2))
  make_node_measure(out, .data)
}

#' @rdname measure_central_degree
#' @references
#' ## On signed centrality
#' Everett, Martin G., and Stephen P. Borgatti. 2014. 
#' “Networks Containing Negative Ties.” 
#' _Social Networks_ 38:111–20. 
#' \doi{10.1016/j.socnet.2014.03.005}
#' @export
node_by_posneg <- function(.data){
  .data <- manynet::expect_nodes(.data)
  stopifnot(manynet::is_signed(.data))
  pos <- manynet::as_matrix(manynet::to_unsigned(.data, keep = "positive"))
  neg <- manynet::as_matrix(manynet::to_unsigned(.data, keep = "negative"))
  nn <- manynet::net_nodes(.data)
  pn <- pos-neg*2
  diag(pn) <- 0
  idmat <- diag(nn)
  v1 <- matrix(1,nn,1)
  out <- solve(idmat - ((pn%*%t(pn))/(4*(nn-1)^2))) %*% (idmat+( pn/(2*(nn-1)) )) %*% v1
  make_node_measure(out, .data)
}

#' @rdname measure_central_degree
#' @section Leverage centrality: 
#'   Leverage centrality concerns the degree of a node compared with that of its
#'   neighbours, \eqn{J}:
#'   \deqn{C_L(i) = \frac{1}{d(i)} \sum_{j \in J(i)} \frac{d(i) - d(j)}{d(i) + d(j)}}
#' @references
#' ## On leverage centrality
#' Joyce, Karen E., Paul J. Laurienti, Jonathan H. Burdette, and Satoru Hayasaka. 2010.
#' "A New Measure of Centrality for Brain Networks". 
#' _PLoS ONE_ 5(8): e12200.
#' \doi{10.1371/journal.pone.0012200}
#' @export
node_by_leverage <- function(.data){
  .data <- manynet::expect_nodes(.data)
  out <- (node_by_deg(.data) - node_by_neighbours_degree(.data))/
    (node_by_deg(.data) + node_by_neighbours_degree(.data))
  make_node_measure(out, .data)
}

# Degree-like centralities ####

#' Measuring ties degree-like centrality
#' @name measure_centralities_degree
#' @description
#'   `tie_by_degree()` measures the degree centrality of ties in a network
#'   
#'   All measures attempt to use as much information as they are offered,
#'   including whether the networks are directed, weighted, or multimodal.
#'   If this would produce unintended results, 
#'   first transform the salient properties using e.g. [to_undirected()] functions.
#'   All centrality and centralization measures return normalized measures 
#'   by default, including for two-mode networks.
#' @template param_data
#' @template param_norm
#' @family degree
#' @family centrality
#' @template tie_measure
NULL

#' @rdname measure_centralities_degree
#' @examples 
#' tie_by_degree(ison_adolescents)
#' @export
tie_by_degree <- function(.data, normalized = TRUE){
  .data <- manynet::expect_ties(.data)
  edge_adj <- manynet::to_ties(.data)
  out <- node_by_degree(edge_adj, normalized = normalized)
  class(out) <- "numeric"
  make_tie_measure(out, .data)
}

# Degree centralisation ####

#' Measuring networks degree-like centralisation
#' @name measure_centralisation_degree
#' @description
#'   - `net_by_degree()` measures a network's degree centralization as a single
#'   score; there are several related shortcut functions:
#'     - `net_by_indegree()` returns the `direction = 'in'` results.
#'     - `net_by_outdegree()` returns the `direction = 'out'` results.
#'   - `mode_by_degree()` measures degree centralization separately for each mode
#'   of a two-mode network, returning one score per mode
#'   (following Borgatti and Everett, 1997); it has the same shortcuts:
#'     - `mode_by_indegree()` returns the `direction = 'in'` results.
#'     - `mode_by_outdegree()` returns the `direction = 'out'` results.
#'
#'   All measures attempt to use as much information as they are offered,
#'   including whether the networks are directed, weighted, or multimodal.
#'   If this would produce unintended results,
#'   first transform the salient properties using e.g. [to_undirected()] functions.
#'   All centrality and centralization measures return normalized measures
#'   by default, including for two-mode networks.
#'
#'   For two-mode networks, the two modes have different theoretical maxima,
#'   so `net_by_degree()` reports a single network-level score by applying
#'   Freeman's general centralization index over the mode-normalized node
#'   degrees, whereas `mode_by_degree()` reports the per-mode centralization
#'   scores directly. For the per-mode scores, "all" uses as numerator the sum
#'   of differences between the maximum centrality score for the mode
#'   against all other centrality scores in the network,
#'   whereas "in" uses as numerator the sum of differences
#'   between the maximum centrality score for the mode
#'   against only the centrality scores of the other nodes in that mode.
#' @template param_data
#' @template param_norm
#' @template param_dir
#' @family degree
#' @family centrality
#' @references
#'   Borgatti, Stephen P., and Martin G. Everett. 1997.
#'   "Network analysis of 2-mode data."
#'   _Social Networks_ 19(3): 243-269.
#'   \doi{10.1016/S0378-8733(96)00301-2}
#' @returns
#'   `net_by_*()` functions return a `network_measure` scalar;
#'   `mode_by_*()` functions return a `mode_measure` numeric vector of length two,
#'   giving one centralization score per mode.
NULL

#' @rdname measure_centralisation_degree
#' @examples
#' net_by_degree(ison_southern_women, direction = "in")
#' @export
net_by_degree <- function(.data, normalized = TRUE,
                           direction = c("all", "out", "in")){

  .data <- manynet::expect_nodes(.data)
  direction <- match.arg(direction)

  if (manynet::is_twomode(.data)) {
    # For two-mode networks the two modes have different theoretical maxima,
    # so degree centralization is intrinsically defined per mode (see
    # `mode_by_degree()`, following Borgatti and Everett (1997)). To return a
    # single network-level score we apply Freeman's general centralization index
    # over the whole node set, using the mode-normalized node degrees (each in
    # [0, 1]); the theoretical maximum of the numerator is then (n - 1).
    nc <- node_by_degree(.data, normalized = TRUE, direction = direction)
    out <- sum(max(nc) - nc) / (length(nc) - 1)
  } else {
    out <- igraph::centr_degree(graph = .data, mode = direction,
                                normalized = normalized)$centralization
  }
  out <- make_network_measure(out, .data, call = deparse(sys.call()))
  out
}

#' @rdname measure_centralisation_degree
#' @examples
#' mode_by_degree(ison_southern_women, direction = "in")
#' @export
mode_by_degree <- function(.data, normalized = TRUE,
                           direction = c("all", "out", "in")){

  .data <- manynet::expect_nodes(.data)
  direction <- match.arg(direction)

  if (!manynet::is_twomode(.data))
    manynet::snet_abort("`mode_by_degree()` is only defined for two-mode networks; use `net_by_degree()` for one-mode networks.")

  mat <- manynet::as_matrix(.data)
  mode <- c(rep(FALSE, nrow(mat)), rep(TRUE, ncol(mat)))

  out <- list()
  if (direction == "all") {
    if (!normalized) {
      allcent <- c(rowSums(mat), colSums(mat))
      out$nodes1 <- sum(max(allcent[!mode]) - allcent)/((nrow(mat) + ncol(mat))*ncol(mat) - 2*(ncol(mat) + nrow(mat) - 1))
      out$nodes2 <- sum(max(allcent[mode]) - allcent)/((nrow(mat) + ncol(mat))*nrow(mat) - 2*(ncol(mat) + nrow(mat) - 1))
    } else if (normalized) {
      allcent <- node_by_degree(.data, normalized = TRUE)
      out$nodes1 <- sum(max(allcent[!mode]) - allcent)/((nrow(mat) + ncol(mat) - 1) - (ncol(mat) - 1) / nrow(mat) - (ncol(mat) + nrow(mat) - 1)/nrow(mat))
      out$nodes2 <- sum(max(allcent[mode]) - allcent)/((ncol(mat) + nrow(mat) - 1) - (nrow(mat) - 1) / ncol(mat) - (nrow(mat)  + ncol(mat) - 1)/ncol(mat))
    }
  } else if (direction == "in" | direction == "out") {
    out$nodes1 <- sum(max(rowSums(mat)) - rowSums(mat))/((ncol(mat) - 1)*(nrow(mat) - 1))
    out$nodes2 <- sum(max(colSums(mat)) - colSums(mat))/((ncol(mat) - 1)*(nrow(mat) - 1))
  }
  out <- c("Mode 1" = out$nodes1, "Mode 2" = out$nodes2)
  out <- make_mode_measure(out, .data, call = deparse(sys.call()))
  out
}

#' @rdname measure_centralisation_degree
#' @export
net_by_outdegree <- function(.data, normalized = TRUE){
  .data <- manynet::expect_nodes(.data)
  net_by_degree(.data, normalized = normalized, direction = "out")
}

#' @rdname measure_centralisation_degree
#' @export
net_by_indegree <- function(.data, normalized = TRUE){
  .data <- manynet::expect_nodes(.data)
  net_by_degree(.data, normalized = normalized, direction = "in")
}

#' @rdname measure_centralisation_degree
#' @export
mode_by_outdegree <- function(.data, normalized = TRUE){
  .data <- manynet::expect_nodes(.data)
  mode_by_degree(.data, normalized = normalized, direction = "out")
}

#' @rdname measure_centralisation_degree
#' @export
mode_by_indegree <- function(.data, normalized = TRUE){
  .data <- manynet::expect_nodes(.data)
  mode_by_degree(.data, normalized = normalized, direction = "in")
}

make_node_mark <- function(out, .data) {
  class(out) <- c("node_mark", class(out))
  if (is.null(names(out)) && manynet::is_labelled(.data))
    names(out) <- manynet::node_names(.data)
  attr(out, "mode") <- manynet::node_is_mode(.data)
  out
}

make_tie_mark <- function(out, .data) {
  class(out) <- c("tie_mark", class(out))
  if(manynet::is_labelled(.data)){
    tie_names <- attr(igraph::E(.data), "vnames")
    if(manynet::is_directed(.data)) 
      names(out) <- gsub("\\|", "->", tie_names) else 
        names(out) <- gsub("\\|", "-", tie_names)
  } else {
    ties <- manynet::as_edgelist(.data)[,1:2]
    if(manynet::is_directed(.data)) 
      names(out) <- paste0(ties$from, "->", ties$to) else 
        names(out) <- paste0(ties$from, "-", ties$to)
  }
  out
}

make_node_measure <- function(out, .data) {
  if(manynet::is_labelled(.data)) names(out) <- manynet::node_names(.data)
  class(out) <- c("node_measure", class(out))
  attr(out, "mode") <- manynet::node_is_mode(.data)
  out
}

make_tie_measure <- function(out, .data) {
  class(out) <- c("tie_measure", class(out))
  if(manynet::is_labelled(.data)){
    tie_names <- attr(igraph::E(.data), "vnames")
    if(manynet::is_directed(.data)) 
      names(out) <- gsub("\\|", "->", tie_names) else 
        names(out) <- gsub("\\|", "-", tie_names)
  } else {
    ties <- manynet::as_edgelist(.data)[,1:2]
    if(manynet::is_directed(.data)) 
      names(out) <- paste0(ties$from, "->", ties$to) else 
        names(out) <- paste0(ties$from, "-", ties$to)
  }
  out
}

make_network_measure <- function(out, .data, call) {
  class(out) <- c("network_measure", class(out))
  attr(out, "mode") <- manynet::net_dims(.data)
  attr(out, "call") <- call
  out
}

make_mode_measure <- function(out, .data, call) {
  class(out) <- c("mode_measure", "network_measure", class(out))
  attr(out, "mode") <- manynet::net_dims(.data)
  attr(out, "call") <- call
  out
}

make_node_member <- function(out, .data) {
  if(is.numeric(out))
    out <- MORELETTERS[out]
  if (manynet::is_labelled(.data)) names(out) <- manynet::node_names(.data)
  class(out) <- c("node_member", class(out))
  attr(out, "mode") <- manynet::node_is_mode(.data)
  out
}

MORELETTERS <- c(LETTERS, sapply(LETTERS, function(x) paste0(x, LETTERS)))

make_node_motif <- function(out, .data) {
  class(out) <- c("node_motif", class(out))
  if(manynet::is_twomode(.data)) attr(out, "mode") <- manynet::node_is_mode(.data)
  if(manynet::is_labelled(.data)) attr(out, "dimnames")[[1]] <- manynet::node_names(.data)
  out
}

make_network_motif <- function(out, .data) {
  class(out) <- c("network_motif", class(out))
  attr(out, "mode") <- manynet::net_dims(.data)
  attr(out, "call") <- deparse(sys.calls())
  out
}

options(manynet_verbosity = "quiet")
options(snet_verbosity = "quiet")

expect_values <- function(object, ref, toler = 3) {
  # 1. Capture object and label
  # act <- quasi_label(rlang::enquo(object), arg = "object")
  act <- list(val = object, label = deparse(substitute(object)))
  
  # 2. Call expect()
  act$n <- round(c(unname(unlist(act$val))), toler)
  ref <- round(c(unname(unlist(ref))), toler)
  expect(
    act$n == ref,
    sprintf("%s has values %f, not values %f.", act$lab, act$n, ref)
  )
  
  # 3. Invisibly return the value
  invisible(act$val)
}

expect_mark <- function(object, ref, top = 3) {
  # 1. Capture object and label
  # act <- quasi_label(rlang::enquo(object), arg = "object")
  act <- list(val = object, label = deparse(substitute(object)))
  
  # 2. Call expect()
  act$n <- as.character(c(unname(unlist(act$val)))[1:top])
  ref <- as.character(c(unname(unlist(ref)))[1:top])
  expect(
    all(act$n == ref),
    sprintf("%s has values %s, not values %s.", act$lab, 
            paste(act$n, collapse = ", "), paste(ref, collapse = ", "))
  )
  
  # 3. Invisibly return the value
  invisible(act$val)
}


top3 <- function(res, dec = 4){
  if(is.numeric(res)){
    unname(round(res, dec))[1:3]  
  } else unname(res)[1:3]
}

bot3 <- function(res, dec = 4){
  lr <- length(res)
  if(is.numeric(res)){
    unname(round(res, dec))[(lr-2):lr]
  } else unname(res)[(lr-2):lr]
}

top5 <- function(res, dec = 4){
  if(is.numeric(res)){
    unname(round(res, dec))[1:5]
  } else unname(res)[1:3]
}

bot5 <- function(res, dec = 4){
  lr <- length(res)
  if(is.numeric(res)){
    unname(round(res, dec))[(lr-4):lr]
  } else unname(res)[(lr-2):lr]
}

collect_functions <- function(pattern, package = "netrics"){
  getNamespaceExports(package)[grepl(pattern, getNamespaceExports(package))]
}
funs_objs <- mget(ls("package:netrics"), inherits = TRUE)

# data_objs <- mget(ls("package:manynet"), inherits = TRUE)
# # Filter to relevant objects 
# # data_objs <- data_objs[grepl("ison_|fict_|irps_|mpn_", names(data_objs))]
# # data_objs <- data_objs[!grepl("starwars|physicians|potter", names(data_objs))]
# objs <- table_data() %>% dplyr::filter(!grepl("starwars|physicians|potter", dataset)) %>% 
#   dplyr::distinct(directed, weighted, twomode, labelled, signed, multiplex, longitudinal, dynamic, changing, .keep_all = TRUE) %>% 
#   dplyr::pull(dataset) %>% as.character()
# data_objs <- data_objs[objs]

set.seed(1234)
data_objs <- list(directed = generate_random(12, directed = TRUE),
                  twomode = generate_random(c(6,6)),
                  labelled = to_signed(add_node_attribute(create_wheel(12), "name", 
                                                LETTERS[1:12])),
                  attribute = add_node_attribute(create_ring(12), "group", 
                                                 rep(c("A","B"), each = 6)),
                  weighted = add_tie_attribute(create_ring(12), "weight", 
                                               rep(c(1,2), each = 6)),
                  diffusion = play_diffusion(create_ring(12), seeds = 1, 
                                             steps = 5, latency = 0.75, 
                                             recovery = 0.25))

find_pkg_tutorial_paths <- function(pkg) {
  tute_folders <- list.dirs(system.file("tutorials", package = pkg),
                             recursive = F)
  tute_files <- unlist(lapply(tute_folders, function(folder) {
    list.files(folder, pattern = "*.Rmd", full.names = TRUE)
  }))
  tute_files
}

# Extract the R code from an R Markdown / learnr tutorial's `{r}` chunks, in
# document order, dropping any chunk marked `purl = FALSE`. This replicates the
# only part of knitr's purl() we rely on, so that {knitr} need not be a package
# dependency (it was otherwise used solely to run these tutorial tests). These
# tutorials use no child documents, chunk references, or non-R engines, so this
# simple line scanner is sufficient and matches purl()'s output expression set.
extract_rmd_code <- function(path) {
  lines <- readLines(path, warn = FALSE)
  open_re  <- "^\\s*```+\\s*\\{[rR][\\s,}]"
  close_re <- "^\\s*```+\\s*$"
  code <- character()
  i <- 1L
  n <- length(lines)
  while (i <= n) {
    if (grepl(open_re, lines[i], perl = TRUE)) {
      keep <- !grepl("purl\\s*=\\s*(FALSE|F)\\b", lines[i])
      i <- i + 1L
      while (i <= n && !grepl(close_re, lines[i])) {
        if (keep) code <- c(code, lines[i])
        i <- i + 1L
      }
    }
    i <- i + 1L
  }
  code
}

# The tutorials' code chunks are extracted to a script and evaluated expression
# by expression, so that any chunk that errors or raises a deprecation warning
# fails the suite. Rendering the learnr tutorials themselves is deliberately not
# tested (that check was fragile and added no coverage value), mirroring
# {autograph}'s tests/testthat/helper-tutorials.R.
check_tute_functions <- function(path, skip = "ergm\\("){
  exprs <- parse(text = extract_rmd_code(path))
  env <- new.env(parent = globalenv())

  is_skipped_call <- function(expr) {
    any(grepl(skip, deparse(expr)))
  }

  for (i in seq_along(exprs)) {
    # Stop at the first slow call: it and any later (dependent) expressions
    # are skipped, but we return normally so the caller's loop over the
    # remaining tutorials continues. Using skip() here would unwind to the
    # enclosing test_that() and abort every subsequent tutorial too.
    if (is_skipped_call(exprs[[i]])) {
      break
    }

    w <- NULL
    e <- NULL
    m <- NULL
    
    not_out <- withCallingHandlers(
      tryCatch(
        eval(exprs[[i]], envir = env),
        error = function(err) {
          e <<- err
          NULL
        }
      ),
      warning = function(wrn) {
        w <<- wrn
        invokeRestart("muffleWarning")
      },
      message = function(msg) {
        m <<- c(m, conditionMessage(msg))
        invokeRestart("muffleMessage")
      }
    )
    
    # If there *was* a warning, check if it's a deprecated/defunct one
    if (!is.null(w)) {
      msg <- conditionMessage(w)
      
      # Only fail if it's a deprecated/defunct warning
      if (!grepl("deprecate|defunct|moved", msg, ignore.case = TRUE)) {
        w <- NULL
      }
    }
    
    # Now test what happened
    expect_null(
      e,
      info = paste0("Error in expression ", i,
                    " of ", basename(path), ": ", deparse(exprs[[i]]))
    )
    
    expect_null(
      w,
      info = paste("Warning in expression", i, ":", deparse(exprs[[i]]))
    )
  }
}

net_meas <- funs_objs[grepl("net_by_", names(funs_objs))]
for(fn in names(net_meas)) {
  for (ob in names(data_objs)) { 
    test_that(paste(fn, "works on", ob), {
      skip_if(grepl("core|spatial", fn))
      skip_if(grepl("net_by_factions", fn) && ob == "twomode")
      if(grepl("diversity|heterophily|homophily", fn)){
        if(ob == "attribute")
          expect_s3_class(net_meas[[fn]](data_objs[[ob]], "group"), "network_measure") else
            succeed("Only used for attribute objects")
      } else if(grepl("balance", fn)){
        if(ob == "labelled")
          expect_s3_class(net_meas[[fn]](data_objs[[ob]]), "network_measure") else
            succeed("Only used for signed objects")
      } else if(grepl("congruency", fn)){
        if(ob == "twomode")
          expect_s3_class(net_meas[[fn]](data_objs[[ob]], data_objs[[ob]]), "network_measure") else
            succeed("Only used for multiple two-mode objects")
      } else if(grepl("strength|toughness", fn)){ # why is this so slow??
        if(ob == "weighted")
          expect_s3_class(net_meas[[fn]](data_objs[[ob]]), "network_measure") else
            succeed("Testing only once because slow")
      } else if(grepl("infection|immunity|recovery|reproduction|transmiss", fn)){
        if(ob == "diffusion")
          expect_s3_class(net_meas[[fn]](data_objs[[ob]]), "network_measure") else
            succeed("Only used for diffusion objects")
      } else {
        expect_s3_class(net_meas[[fn]](data_objs[[ob]]), "network_measure")
      }
    })
  }
}

net_motifs <- funs_objs[grepl("net_x_", names(funs_objs))]
for(fn in names(net_motifs)) {
  for (ob in names(data_objs)) { 
    test_that(paste(fn, "works on", ob), {
      skip_if(grepl("exposure|mixed|hazard", fn))
      skip_if(grepl("triad", fn) && is_twomode(data_objs[[ob]]))
      if(grepl("brokerage", fn)){
        if(ob == "attribute")
          expect_s3_class(net_motifs[[fn]](data_objs[[ob]], "group"), "network_motif") else
            succeed("Only used for attribute objects")
      } else if(grepl("correlation|change|stability", fn)){
        if(ob == "labelled")
          expect_s3_class(net_motifs[[fn]](data_objs[[ob]], data_objs[[ob]]), "network_motif") else
            succeed("Only used for multi objects")
      } else {
        expect_s3_class(net_motifs[[fn]](data_objs[[ob]]), "network_motif")
      }
    })
  }
}

tie_meas <- funs_objs[grepl("tie_by_", names(funs_objs))]
for(fn in names(tie_meas)) {
  for (ob in names(data_objs)) { 
    test_that(paste(fn, "works on", ob), {
      skip_if_not(packageVersion("manynet") >= "1.7.3")
      expect_s3_class(tie_meas[[fn]](data_objs[[ob]]), "tie_measure")
    })
  }
}

test_that("netrics tutorial code runs without warnings or errors", {
  skip_if_not_installed("netrics", minimum_version = "0.2.2")
  for(tute in find_pkg_tutorial_paths("netrics")){
    expect_null(check_tute_functions(tute), 
                info = paste("Error in tutorial", basename(tute)))
  }
})

# tie_motifs <- funs_objs[grepl("tie_x_", names(funs_objs))]
# for(fn in names(tie_motifs)) {
#   for (ob in names(data_objs)) { 
#     test_that(paste(fn, "works on", ob), {
#       if(fn == "x"){
#       } else {
#         expect_s3_class(tie_motifs[[fn]](data_objs[[ob]]), "tie_motif")
#       }
#     })
#   }
# }

---
title: "Topology and Resilience"
author: "by James Hollway"
output: 
  learnr::tutorial:
    theme: cosmo
runtime: shiny_prerendered
description: >
  This tutorial aims to teach you how to create deterministic networks,
  generate probabilistic networks, identify core-periphery structures,
  and measure network resilience.
---

```{r pkgs, include = FALSE, eval=TRUE}
library(manynet)
library(autograph)
```

```{r setup, include = FALSE, purl=FALSE, eval=TRUE}
library(learnr)
knitr::opts_chunk$set(echo = FALSE)
clear_glossary()
learnr::random_phrases_add(language = "fr",
                           praise = c("C'est génial!",
                                      "Beau travail",
                                      "Excellent travail!",
                                      "Bravo!",
                                      "Super!",
                                      "Bien fait",
                                      "Bien joué",
                                      "Tu l'as fait!",
                                      "Je savais que tu pouvais le faire.",
                                      "Ça a l'air facile!",
                                      "C'était un travail de première classe.",
                                      "C'est ce que j'appelle un bon travail!"),
                          encouragement = c("Bon effort",
                                            "Vous l'avez presque maîtrisé!",
                                            "Ça avance bien.",
                                            "Continuez comme ça.",
                                            "Continuez à travailler dur!",
                                            "Vous apprenez vite!",
                                            "Vous faites un excellent travail aujourd'hui."))
learnr::random_phrases_add(language = "en", 
                           praise = c("C'est génial!",
                                      "Beau travail!",
                                      "Bravo!",
                                      "Super!"),
                           encouragement = c("Bon effort"))
```

## Overview

<img src="https://64.media.tumblr.com/tumblr_lvpgquhGbq1r54h4xo1_400.gif"/>

In this tutorial, we'll explore:

- how to create or generate different network topologies
- the core-periphery structure of a network
- features of a network related to its resilience

This tutorial covers a range of different network topologies:
trees, lattices, random, small-world, scale-free, and core-periphery
networks.
These ideal networks exaggerate centrality, clustering, and randomness features, 
and are thus great for theory-building and investigating the relationship between rules and structure.

<img src="https://media.tenor.com/images/e26e64a8cce94a6ee491f0ddeecc8b88/tenor.gif"/>

In this practical, we're going to create/generate
a number of ideal-typical network topologies and plot them.
We'll first look at some deterministic algorithms for _creating_ networks
of different structures, 
and then look at how the introduction of some randomness can _generate_ a variety of network structures.

## Creating networks

To begin with, let's create a few 'empty' and full/'complete' graphs.
You will want to use some of the `create_*()` group of functions from `{manynet}`,
because they create graphs following some strict rule(s).
The two functions you will want to use here are `create_empty()` and `create_filled()`.
`create_empty()` creates an empty graph with the given number of nodes, 
in this case 50 nodes.
For `create_filled()` we're creating a full graph,
where all of the nodes are connected to all of the other nodes.

Let's say that we want to explore networks of fifty nodes in this script.
Graph one empty and one complete network with 50 nodes each,
give them an informative title, and plot the graphs together.
What would a complete network with half the nodes look like?
Add that too.

```{r empty, exercise=TRUE, purl = FALSE}

```

```{r empty-solution}
(graphr(create_empty(50), "circle") + ggtitle("Empty graph"))
(graphr(create_filled(50)) + ggtitle("Complete graph"))
(graphr(create_filled(50/2)) + ggtitle("Complete graph (smaller)"))
```

#### Stars

In a `r gloss("star", "star")` network, there is one node to which all other nodes are connected,
otherwise known as the `r gloss("universal", "universal")` node.
There is no transitivity.
The maximum path length is two.
And `r gloss("degree", "degree")` centrality is maximised!^[In fact, all centrality measures are maximised, as one node acts as the sole bridge connecting one part of the network to the other.]

Use the `create_star()` function to graph three star networks:

- an undirected star network
- a out-directed star network
- and an in-directed star network

```{r star, exercise = TRUE, purl = FALSE}

```

```{r star-solution}
(graphr(create_star(50)) + ggtitle("Star graph"))
(graphr(create_star(50, directed = TRUE)) + ggtitle("Star out"))
(graphr(to_redirected(create_star(50, directed = TRUE))) + ggtitle("Star in"))
```

#### Trees

Trees, or regular trees, are networks with branching nodes.
They can be directed or undirected, and tend to indicate strong hierarchy.
Again graph three networks:

- one undirected with 2 branches per node
- a directed network with 2 branches per node
- the same as above, but graphed using the "tree" layout

```{r tree, exercise = TRUE, purl = FALSE}

```

```{r tree-solution}
# width argument specifies the breadth of the branches
(graphr(create_tree(50, width = 2)) + ggtitle("Tree graph"))
(graphr(create_tree(50, width = 2, directed = TRUE)) + ggtitle("Tree out"))
(graphr(create_tree(50, width = 2, directed = TRUE), "tree") + ggtitle("Tree layout"))
```

Try varying the `width` argument to see the result.

#### Lattices

`r gloss("Lattices","lattice")` reflect highly clustered networks
where there is a high likelihood that interaction partners also interact.
They are used to show how clustering facilitates or limits diffusion
or makes pockets of behaviour stable.

```{r lat-qa, echo=FALSE, purl = FALSE}
question("Why are lattices considered highly clustered?",
         answer("Because neighbours are likely also neighbours of each other",
                message = learnr::random_praise(),
                correct = TRUE),
         answer("Because all nodes are directly connected to each other",
                message = learnr::random_encouragement()),
         answer("Because there is a single component",
                message = learnr::random_encouragement()),
         answer("Because there is a single community",
                message = learnr::random_encouragement()),
         random_answer_order = TRUE,
         allow_retry = TRUE)
```

Note that `create_lattice()` in `{manynet}` works a little differently
to how it works in `{igraph}`.
In `{igraph}` the number or vector passed to the function indicates
the length of each dimension.
So `c(50)` would be a one-dimensional lattice,
essentially a chain of 50 nodes connected to their neighbours.
`c(50,50)` would be a two-dimensional lattice,
of 50 nodes long and 50 nodes wide.
`c(50,50,50)` would be a three-dimensional lattice,
of 50 nodes long, 50 nodes wide, and 50 nodes deep, etc.

_But_ this doesn't help us when we want to see what a lattice representation
with the same order (number of nodes) as a given network would be.
For example, perhaps we just want to know what a lattice with 50 nodes
would look like.
So `{manynet}` instead tries to find the most even or balanced 
two-dimensional representation with a given number of nodes.

Graph two lattices, one with 50 nodes, 
and another with half the number of nodes.

```{r lattices, exercise = TRUE, purl = FALSE}

```

```{r lattices-solution}
(graphr(create_lattice(50)) + ggtitle("One-mode lattice graph"))
(graphr(create_lattice(50/2)) + ggtitle("Smaller lattice graph"))
```

#### Rings

This creates a graph where each node has two separate neighbours
which creates a ring graph.
Graph three ring networks:

- one with 50 nodes
- one with 50 nodes where they are connected to neighbours two steps away,
  on a "circle" layout
- the same as above, but on a "stress" layout

```{r rings, exercise = TRUE, purl = FALSE}

```

```{r rings-solution}
(graphr(create_ring(50)) + ggtitle("Ring graph", subtitle = "Starring Naomi Watts"))
# width argument specifies the width of the ring
(graphr(create_ring(50, width = 2), "circle") + ggtitle("The Ring Two", subtitle = "No different?"))
(graphr(create_ring(50, width = 2), "stress") + ggtitle("The Ring Two v2.0"))
```

## Generating networks

Next we are going to take a look at some probabilistic graphs.
These involve some random element, perhaps in addition to specific rules,
to stochastically 'generate' networks of certain types of topologies.
As such, we'll be using the `generate_*()` group of functions from `{manynet}`.

#### Random graphs

An Erdös-Renyi graph is simply a random graph.
You will need to specify the probability of a tie
in addition to the number of nodes.
An Erdos-Renyi graph on the vertex set $V$ is a random graph
which connects each pair of nodes ${i,j}$ with probability $p$, independent.
Note that for a “sparse” ER graphs, $p$ must decrease as $N$ goes up.
Generate three random networks of 50 nodes and a density of 0.08:

```{r random, exercise = TRUE, purl = FALSE}

```

```{r random-solution}
(graphr(generate_random(50, 0.08)) + ggtitle("Random 1 graph"))
(graphr(generate_random(50, 0.08)) + ggtitle("Random 2 graph"))
(graphr(generate_random(50, 0.08)) + ggtitle("Random 3 graph"))
```

Keep going if you like... it will be a little different every time.
Note that you can also pass the second argument an integer,
in which case the function will interpret that as the number of ties/edges rather than the probability that a tie is present.
Try generating a random graph with 200 edges/ties now:

```{r randomno, exercise = TRUE, purl = FALSE}

```

```{r randomno-solution}
(erdren4 <- graphr(generate_random(50, 200)) + ggtitle("Random 1 graph"))
```

#### Small-world graphs

Remember the ring graph from above?
What if we rewire (change) some of the edges at a certain probability?
This is how `r gloss("small-world","smallworld")` networks are generated.
Graph three small-world networks, all with 50 nodes and a rewiring probability of 0.025.

```{r smallw, exercise = TRUE, purl = FALSE}

```

```{r smallw-solution}
(graphr(generate_smallworld(50, 0.025)) + ggtitle("Smallworld 1 graph"))
(graphr(generate_smallworld(50, 0.025)) + ggtitle("Smallworld 2 graph"))
(graphr(generate_smallworld(50, 0.025)) + ggtitle("Smallworld 3 graph"))
```

With on average 2.5 ties randomly rewired, does the structure look different?
This is a small-world network, where clustering/transitivity remains high
but path lengths are much lower than they would otherwise be.
Remember that in a small-world network, the shortest-path  distance between nodes
increases sufficiently slowly as a function of the number of nodes in the network.
You can also call these networks a Watts–Strogatz toy network.
If you want to review this, go back to the reading by Watts (2004).

There is also such a thing as a network's small-world coefficient.
See the help page for more details,
but with the default equation ('omega'),
the coefficient typically ranges between 0 and 1,
where 1 is as close to a small-world as possible.
Try it now on a small-world generated network,
but with a rewiring probability of 0.25:

```{r smallwtest, exercise = TRUE, purl = FALSE}

```

```{r smallwtest-solution}
net_smallworld(generate_smallworld(50, 0.25))
```

#### Scale-free graphs

There is another famous model in network science: the `r gloss("scale-free","scalefree")` model.
Remember:
"In many real-world networks, the distribution of the number of network neighbours
the degree distribution is typically right-skewed with a "heavy tail".
A majority of the nodes have less-than-average `r gloss("degree","degree")` and
a small fraction of hubs are many times better connected than average (2004, p. 250).

The following generates a scale-free graph according to the Barabasi-Albert (BA) model
that rests upon the mechanism of preferential attachment.
More on this in the Watts paper (2005, p.51) and Merton (1968).
The BA model rests on two mechanisms: 
population growth and preferential attachment.
Population growth: real networks grow in time as new members join the population.
Preferential/cumulative attachment means that newly arriving nodes will tend to
connect to already well-connected nodes rather than poorly connected ones.

Generate and graph three scale-free networks,
with alpha parameters of 0.5, 1, and 1.5.

```{r scalef, exercise = TRUE, purl = FALSE}

```

```{r scalef-solution}
(graphr(generate_scalefree(50, 0.5)) +
    ggtitle("Scalefree 1 graph", subtitle = "Power = .5"))
(graphr(generate_scalefree(50, 1)) +
    ggtitle("Scalefree 2 graph", subtitle = "Power = 1"))
(graphr(generate_scalefree(50, 1.5)) +
    ggtitle("Scalefree 3 graph", subtitle = "Power = 1.5"))
```

You can also the degree to which a network has a degree distribution that fits
a power-law distribution.
With an alpha/power-law exponent between 2 and 3, 
one generally cannot reject the hypothesis that the observed data 
comes from a power-law distribution.

```{r scaleftest, exercise = TRUE, purl = FALSE}

```

```{r scaleftest-solution}
net_scalefree(generate_scalefree(50, 2))
```

## Core-Periphery

### Core-periphery graphs

Lastly, we'll take a look at some `r gloss("core-periphery","core")` graphs.
The most common definition of a core-periphery network
is one in which the network can be partitioned into two groups
such that one group of nodes (the core) has 
dense interactions among themselves,
moderately dense interactions with the second group,
and the second group (the periphery) has
sparse interactions among themselves.

```{r corevcomm-qa, echo=FALSE, purl = FALSE}
question("Can a single network have both a community structure and a core-periphery structure?",
         answer("No", message = learnr::random_encouragement()),
         answer("Yes",
                message = "That's right. For example, the core and periphery might represent two or more communities.",
                correct = TRUE),
         random_answer_order = TRUE,
         allow_retry = TRUE)
```

We can visualise extreme versions of such a network
using the `create_core()` function.
Graph a core-periphery network of 50 nodes
(which, unless a core-periphery membership assignment is given,
will be split evenly between core and periphery partitions).

```{r core, exercise=TRUE, purl = FALSE, fig.width=9}

```

```{r core-solution}
(graphr(create_core(50)) + ggtitle("Core"))
```

### Core-periphery assignment

Let's consider identifying the core and peripheral nodes in a network.
Let's use the `ison_lawfirm` dataset from `{manynet}`.
This dataset involves relations between partners in a corporate law firm in New England.
First of all, graph the data and see whether you can guess which nodes
might be part of the core and which are part of the periphery.
Color the nodes by Gender, Office, Practice, and School.
Any you might think correlate with core status?

```{r gnet, exercise=TRUE, purl = FALSE, fig.width=9}
lawfirm <- ison_lawfirm |> to_uniplex("friends") |> to_undirected()
```

```{r gnet-solution}
lawfirm <- ison_lawfirm |> to_uniplex("friends") |> to_undirected()
graphr(lawfirm, node_color = "school", edge_color = "darkgray")
graphr(lawfirm, node_color = "gender", edge_color = "darkgray")
graphr(lawfirm, node_color = "office", edge_color = "darkgray")
graphr(lawfirm, node_color = "practice", edge_color = "darkgray")
```

Next, let's assign nodes to the core and periphery blocks
using the `node_is_core()` function.
It works pretty straightforwardly.
By default it runs down the rank order of nodes by their degree,
at each step working out whether including the next highest degree node
in the core will maximise the core-periphery structure of the network.

```{r nodecore, exercise=TRUE, purl = FALSE, fig.width=9, exercise.setup="gnet"}

```

```{r nodecore-solution}
lawfirm %>% 
  mutate(nc = node_is_core()) %>% 
  graphr(node_color = "nc", edge_color = "gray")
```

This graph suggests that there is a core and a periphery.
There might even be two cores here,
one on the left and one on the right.

But is it really all that much of a core-periphery structure?
We can establish how correlated our network is compared to
a core-periphery model of the same dimension using `net_core()`.

```{r netcore, exercise=TRUE, purl = FALSE, exercise.setup="gnet"}

```

```{r netcore-solution}
net_core(lawfirm, node_is_core(lawfirm))
```

```{r corecorr-qa, echo=FALSE, purl = FALSE}
question("What can we say about this correlation.",
         answer("It is a perfect negative relationship",
                message = learnr::random_encouragement()),
         answer("It is fairly strong",
                message = learnr::random_encouragement()),
         answer("It is positive",
                message = learnr::random_encouragement()),
         answer("There is absolutely no correlation",
                message = learnr::random_encouragement()),
         answer("None of the above", correct = TRUE,
                message = learnr::random_praise()),
         allow_retry = TRUE
)
```

Note that `node_is_core()` also includes a method that descends through
the rank order of nodes' eigenvector centralities instead of degree centralities.
Why might that not be such a good choice here?

Now let's see whether our core-periphery membership vector correlates with any of the three
categorical attributes we looked at before.
Since we're doing this on categorical variables, we'll use the Chi-squared test in base R.
Take a look and see whether there is a statistically significant association
between gender and core (or periphery) status.

```{r chisq, exercise=TRUE, purl=FALSE, exercise.setup="gnet"}
chisq.test(as.factor(node_is_core(lawfirm)), 
           as.factor(node_attribute(lawfirm, "gender")))
```

```{r chisq-solution}
chisq.test(as.factor(node_is_core(lawfirm)), 
           as.factor(node_attribute(lawfirm, "gender")))
chisq.test(as.factor(node_is_core(lawfirm)), 
           as.factor(node_attribute(lawfirm, "office")))
chisq.test(as.factor(node_is_core(lawfirm)), 
           as.factor(node_attribute(lawfirm, "school")))
chisq.test(as.factor(node_is_core(lawfirm)), 
           as.factor(node_attribute(lawfirm, "practice")))
```

```{r chisq-qa, echo=FALSE, purl = FALSE}
question("There a statistically significant association between the core assignment and...",
         answer("gender.",
                message = learnr::random_encouragement()),
         answer("office.",
                message = learnr::random_encouragement()),
         answer("school.",
                message = learnr::random_encouragement()),
         answer("practice.",
                message = learnr::random_encouragement()),
         answer("none of the above variables.", correct = TRUE,
                message = "That's right. The p-value for office is close, but no cigar."),
         allow_retry = TRUE
)
```

### Coreness

An alternative route is to identify 'core' nodes
depending on their `r gloss("k-coreness","kcoreness")`.
In `{manynet}`, we can return nodes _k_-coreness
with `node_coreness()` instead of 
the `node_is_core()` used for core-periphery.

```{r nodecoren, exercise=TRUE, purl = FALSE, exercise.setup="gnet"}
lawfirm %>% 
  mutate(ncn = node_kcoreness()) %>% 
  graphr(node_color = "ncn")
```

```{r dich-qa, echo=FALSE, purl = FALSE}
question("Which has more than two classes/groups.",
         answer("node_kcoreness()", correct = TRUE,
                message = learnr::random_praise()),
         answer("node_is_core()",
                message = learnr::random_encouragement()),
         random_answer_order = TRUE,
         allow_retry = TRUE
)
```

```{r ness-qa, echo=FALSE, purl = FALSE}
question("Select the correct definitions:",
         answer("The k-core of a network is a maximal subgraph in which each vertex has at least degree k.", correct = TRUE,
                message = learnr::random_praise()),
         answer("The coreness of a node is k if it belongs to the k-core but not to the (k+1)-core.", correct = TRUE),
         answer("The coreness of a node is equal to its degree.",
                message = learnr::random_encouragement()),
         random_answer_order = TRUE,
         allow_retry = TRUE
)
```

## Hierarchy

What sometimes drives our interest in whether a network resembles a 
core-periphery network is that it offers a generalisation of hierarchy:
the core are the rule-makers; the periphery are the rule-takers.
But where we have a directed network, 
we may be able to measure hierarchy explicitly.

### Graph theoretic dimensions of hierarchy

Recall that measuring hierarchy directly is difficult, 
as the concept incorporates several different aspects.
Can you recall which of the following are aspects of
the graph theoretic dimensions of hierarchy?

```{r gtdh-qa, echo=FALSE, purl = FALSE}
question("Select the measures included in the graph theoretic dimensions of hierarchy:",
         answer("reciprocity", correct = TRUE,
                message = learnr::random_praise()),
         answer("connectedness", correct = TRUE),
         answer("efficiency", correct = TRUE),
         answer("least upper bound", correct = TRUE),
         answer("components", message = learnr::random_encouragement()),
         answer("degree", message = learnr::random_encouragement()),
         answer("density", message = learnr::random_encouragement()),
         random_answer_order = TRUE,
         allow_retry = TRUE
)
```

### Measuring GTDH

Ok, so let's now take a closer look at how to investigate the degree of
hierarchy in a given network.
The classic example would be to look at a tree network,
like the one constructed earlier.
Let's try the function `net_by_hierarchy()` on a tree network of 12 nodes.

```{r treeh, exercise = TRUE, purl = FALSE}
treeleven <- create_tree(11, directed = TRUE)
net_by_hierarchy(treeleven)
rowMeans(net_by_hierarchy(treeleven))
```

We see here four different measures of hierarchy:

- **connectedness**: the proportion of dyads that can reach each other
- **reciprocity**: the proportion of ties that are reciprocated (inverse)
- **efficiency**: the minimum indegrees required to connect the network over
the actual indegrees present
- **least upper bound**: all nodes in a hierarchy should have a node that can
reach both of them

Note that to get an overall score, we can take the average of these four measures.

### Comparing GTDH

Because some of these measures (reciprocity), only make sense for a directed
network, we'll switch to a couple of directed networks for this exercise.
Which one would you consider is more hierarchical:
`ison_emotions`, a directed network of emotional transitions, 
or `fict_thrones`, a directed network of kinship ties?

```{r hierarchy, exercise = TRUE, warning=FALSE}
graphr(ison_emotions)
graphr(fict_thrones)
```

```{r hierarchy-solution}
graphr(ison_emotions)
net_by_hierarchy(ison_emotions)
graphr(fict_thrones)
net_by_hierarchy(fict_thrones)
```

Actually, these two networks have the same average hierarchy score of around 0.525.
But they have quite different profiles.
Can you make sense of these results?

```{r gtdhcompare-qa, echo=FALSE, purl = FALSE}
question("Compared to the emotional transitions network, the Game of Thrones kinship network has:",
         answer("more nodes able to reach each other",
                message = learnr::random_encouragement()),
         answer("fewer reciprocated ties", correct = TRUE,
                message = learnr::random_praise()),
         answer("fewer redundant ties", correct = TRUE),
         answer("at least one boss for all pairs"),
         random_answer_order = TRUE,
         allow_retry = TRUE
)
```

## Resilience

### How cohesive is the network?

When investigating a network's resilience,
we might think of whether the network will remain connected despite some nodes or ties dropping out.
Let's explore how resilient a (core) network of adolescents (`ison_adolescents`) might be.
First, we might be interested in whether the network is `r gloss("connected","connected")` at all.

```{r connected, exercise=TRUE, purl = FALSE}

```

```{r connected-solution}
net_connectedness(ison_adolescents)
```

This measure gets at the proportion of dyads that can reach each other in the network.
In this case, the proportion is 1, i.e. all nodes can reach every other node.
Another way to get at this would be to see how many `r gloss("components","component")` there are in the network.

```{r connect-qa, echo=FALSE, purl=FALSE}
question("But counting the number of components instead of connectedness can overemphasise:",
         answer("Isolates", correct = TRUE,
                message = learnr::random_praise()),
         answer("Small components", correct = TRUE,
                message = learnr::random_encouragement()),
         answer("Density",
                message = learnr::random_encouragement()),
         random_answer_order = TRUE,
         allow_retry = TRUE
)
```

A dropped tie can have severe consequences to the topology of a network
if it is a bridge, say.
The more ties you would need to remove to fragment the network,
the greater the `r gloss("adhesion","adhesion")` of the network.
But a dropped node can be even more consequential, as it will take any ties it has with it.
Find out the `r gloss("cohesion","cohesion")`, 
or how many dropped nodes it would take to (further) fragment the network.

```{r cohesion, exercise=TRUE, purl = FALSE}

```

```{r cohesion-solution}
net_cohesion(ison_adolescents)
```

```{r cohesion-qa, echo=FALSE, purl = FALSE}
question("The result of this function represents...",
         answer("the minimum number of nodes necessary to remove from the network to increase the number of components.", correct = TRUE,
                message = learnr::random_praise()),
         answer("the number of strong components in the network.",
                message = learnr::random_encouragement()),
         answer("the minimum number of ties necessary to remove from the network to increase the number of components.",
                message = "This is actually the definition of `node_adhesion()`."),
         random_answer_order = TRUE,
         allow_retry = TRUE
)
```

```{r res-qa, echo=FALSE, purl = FALSE}
question("The higher the minimum number of nodes to remove...",
         answer("the more resilient is the network.", correct = TRUE,
                message = learnr::random_praise()),
         answer("the less resilient the network.",
                message = learnr::random_encouragement()),
         random_answer_order = TRUE,
         allow_retry = TRUE
)
```

### Identifying cutpoints

But which are these nodes? Is there more than one? 
Nodes that endanger fragmentation of the network are called `r gloss("cutpoints","cutpoint")`.
Find and use a function to identify which, if any, of the nodes in the `ison_adolescents`
network are cutpoints.

```{r idcuts, exercise = TRUE, purl=FALSE}

```

```{r idcuts-solution}
node_is_cutpoint(ison_adolescents)
```

Ok, so this results in a vector identifying which nodes are cutpoints (TRUE) or not (FALSE).
Somewhat more useful though would be to highlight these nodes on the network.
Can you add a node attribute that highlights which nodes are cutpoints?

```{r closerlook, exercise = TRUE, purl=FALSE}

```

```{r closerlook-solution}
ison_adolescents |> mutate(cut = node_is_cutpoint(ison_adolescents)) |> 
  graphr(node_color = "cut")
```

### Identifying bridges

Let's do something similar now, but with respect to ties rather than nodes.
Here we are interested in identifying which ties are `r gloss("bridges","bridge")`.

```{r tieside, exercise = TRUE, purl=FALSE}

```

```{r tieside-solution}
net_adhesion(ison_adolescents)
ison_adolescents |> mutate_ties(cut = tie_is_bridge(ison_adolescents)) |> 
  graphr(edge_color = "cut")
```

We could also investigate the opposite of a bridge,
the degree to which ties are deeply embedded in triangles.
This is called (rather confusingly) tie cohesion.

```{r tiecoh, exercise = TRUE, purl=FALSE}

```

```{r tiecoh-solution}
ison_adolescents |> mutate_ties(coh = tie_cohesion(ison_adolescents)) |> 
  graphr(edge_size = "coh")
```

Where would you target your efforts if you wanted to fragment this network?

## Free play

```{r freeplay, exercise = TRUE}

```

## Glossary

Here are some of the terms that we have covered in this module:

`r print_glossary()`

---
title: "Position and Equivalence"
author: "by James Hollway"
output: 
  learnr::tutorial:
    theme: united
runtime: shiny_prerendered
description: >
  This tutorial aims to teach you how to measure structural holes and
  identify and interpret structurally equivalent nodes.
---

```{r pkgs, include = FALSE, eval=TRUE}
library(manynet)
library(autograph)
```

```{r setup, include = FALSE, purl=FALSE, eval=TRUE}
library(learnr)
knitr::opts_chunk$set(echo = FALSE)
clear_glossary()
learnr::random_phrases_add(language = "fr",
                           praise = c("C'est génial!",
                                      "Beau travail",
                                      "Excellent travail!",
                                      "Bravo!",
                                      "Super!",
                                      "Bien fait",
                                      "Bien joué",
                                      "Tu l'as fait!",
                                      "Je savais que tu pouvais le faire.",
                                      "Ça a l'air facile!",
                                      "C'était un travail de première classe.",
                                      "C'est ce que j'appelle un bon travail!"),
                          encouragement = c("Bon effort",
                                            "Vous l'avez presque maîtrisé!",
                                            "Ça avance bien.",
                                            "Continuez comme ça.",
                                            "Continuez à travailler dur!",
                                            "Vous apprenez vite!",
                                            "Vous faites un excellent travail aujourd'hui."))
learnr::random_phrases_add(language = "en", 
                           praise = c("C'est génial!",
                                      "Beau travail!",
                                      "Bravo!",
                                      "Super!"),
                           encouragement = c("Bon effort"))
friends <- to_uniplex(ison_algebra, "friends")
social <- to_uniplex(ison_algebra, "social")
tasks <- to_uniplex(ison_algebra, "tasks")
```


## Setting up

<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExMTVtNTJqZmFsdmFpOGRjd24zeHZ1NTM0bDhxbWxrMWdja202NXo1eSZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/fL1EXX8pQs0dd9zqLR/giphy.gif" alt="gif of rick and morty characters hanging out with themselves"/>

For this session, we're going to use the "ison_algebra" dataset included in the `{manynet}` package.
Do you remember how to call the data?
Can you find out some more information about it via its help file?

```{r data, exercise = TRUE, purl = FALSE}

```

```{r data-hint-1, purl = FALSE}
# Let's call and load the 'ison_algebra' dataset
data("ison_algebra", package = "manynet")
# Or you can retrieve like this:
ison_algebra <- manynet::ison_algebra
```

```{r data-hint-2, purl = FALSE}
# If you want to learn more about the 'ison_algebra' dataset, use the following function (below)
?manynet::ison_algebra
```

```{r data-solution}
data("ison_algebra", package = "manynet")
?manynet::ison_algebra
# If you want to see the network object, you can run the name of the object
# ison_algebra
# or print the code with brackets at the front and end of the code
# (ison_algebra <- manynet::ison_algebra)
```

We can see that the dataset is multiplex,
meaning that it contains several different types of ties: 
friendship (friends), social (social) and task interactions (tasks).

### Separating multiplex networks

As a `r gloss("multiplex")` network, 
there are actually three different types of ties in this network.
We can extract them and investigate them separately using `to_uniplex()`.
Within the parentheses, put the multiplex object's name,
and then as a second argument put the name of the tie attribute in quotation marks.
Once you have extracted all three networks,
graph them and add a descriptive title.

```{r separatingnets, exercise=TRUE, exercise.setup = "data", purl = FALSE}

```

```{r separatingnets-hint-1, purl = FALSE}
# Here's the basic idea/code syntax you will need to extract each type of network
# You will want to replace
____ <- to_uniplex(ison_algebra, _____)
```

```{r separatingnets-hint-4, purl = FALSE}
graphr(friends) + ggtitle("Friendship")
```

```{r separatingnets-solution}
friends <- to_uniplex(ison_algebra, "friends")
graphr(friends) + ggtitle("Friendship")

social <- to_uniplex(ison_algebra, "social")
graphr(social) + ggtitle("Social")

tasks <- to_uniplex(ison_algebra, "tasks")
graphr(tasks) + ggtitle("Task")
```

Note also that these are `r gloss("weighted")` networks.
`graphr()` automatically recognises these different weights and plots them.

```{r strongties-qa, echo=FALSE, purl = FALSE}
question("If we interpret ties with higher weights as strong ties, and lesser weights as weak ties, then, according to network theory, where would we expect novel information to come from?",
         answer("Weak ties",
                correct = TRUE,
                message = learnr::random_praise()),
         answer("Strong ties",
                message = learnr::random_encouragement()),
         answer("Isolates",
                message = learnr::random_encouragement()),
         answer("Highest degree nodes",
                message = learnr::random_encouragement()),
  random_answer_order = TRUE,
  allow_retry = TRUE
)
```

## Structural Holes

Our first question for this network, is where innovation and creative ideas
might be expected to appear.
There are a number of theories that associate innovation or novelty with 
structural position.

```{r structinnov-qa, echo=FALSE, purl = FALSE}
question("Which network concepts are associated with innovation?",
         answer("Structural holes",
                correct = TRUE,
                message = "Being positioned in a structural hole is said to be a brokerage position that brings with it information arbitrage possibilities."),
         answer("Structural folds",
                correct = TRUE,
                message = "Being positioned in a structural fold, with in-group membership in multiple groups, can provide not only information arbitrage possibilities, but also the standing in the target group to introduce novelty successfully."),
         answer("Structural balance",
                message = learnr::random_encouragement()),
         answer("Structural equivalence",
                message = '<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExdmY4ejhpMnZoZmh3OGUxbTdjbXQwMTcwcmg0ZHo5dmJ6NzI3c3FxNyZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9dg/2E7VU4zGVm8H1mFSAz/giphy.gif" alt="gif of a morty assassinating another morty"/>'),
         answer("Structuralism",
                message = learnr::random_encouragement()),
  random_answer_order = TRUE,
  allow_retry = TRUE
)
```

### Measuring structural holes

```{r shmeasures-qa, echo=FALSE, purl = FALSE}
question("There are a number of measures that might be used to approximate the concept of structural holes. Select all that apply.",
         answer("Constraint",
                correct = TRUE,
                message = learnr::random_praise()),
         answer("Effective size",
                correct = TRUE),
         answer("Bridges",
                correct = TRUE),
         answer("Redundancy",
                correct = TRUE),
         answer("Efficiency",
                correct = TRUE),
         answer("Hierarchy",
                correct = TRUE),
  random_answer_order = TRUE,
  allow_retry = TRUE
)
```

### Bridges

One common way of thinking about how innovations and ideas might flow across
the network is to think about where the bottlenecks are.
Ties that are the bottlenecks for information to flow from one part of the
network to another are called `r gloss("bridges", "bridge")`.
Remember, this means that only ties can be bridges,
though a node-based measure that tracks the count of bridges to which a node
is adjacent is available.
Unfortunately, if we take a closer look at the friends network,
there are no bridges.

```{r bridges, exercise = TRUE, exercise.setup = "objects-setup"}
sum(tie_is_bridge(friends))
any(node_bridges(friends)>0)
```

### Constraint

But some nodes do seem more deeply embedded in the network than others.
Let's take a look at which actors are least `r gloss("constrained", "constraint")`
by their position in the *task* network.
`{manynet}` makes this easy enough with the `node_constraint()` function.

```{r objects-setup, purl=FALSE}
alge <- to_named(ison_algebra)
friends <- to_uniplex(alge, "friends")
social <- to_uniplex(alge, "social")
tasks <- to_uniplex(alge, "tasks")
```

```{r constraint, exercise = TRUE, exercise.setup = "objects-setup", purl = FALSE}

```

```{r constraint-hint, purl = FALSE}
node_constraint(____)
# Don't forget we want to look at which actors are least constrained by their position in the 'tasks' network
```

```{r constraint-solution}
node_constraint(tasks)
```

This function returns a vector of constraint scores that can range between 0 and 1.
Let's graph the network again, sizing the nodes according to this score.
We can also identify the node with the minimum constraint score using `node_is_min()`.

```{r constraintplot, exercise=TRUE, exercise.setup = "objects-setup", purl = FALSE}

```

```{r constraintplot-hint-1, purl = FALSE}
tasks <- tasks %>% 
  mutate(constraint = node_constraint(____),
         low_constraint = node_is_min(node_constraint(____)))

# Don't forget, we are still looking at the 'tasks' network
```

```{r constraintplot-hint-3, purl = FALSE}
# Now, let's graph the network
# Note 1: we are looking at the 'tasks' network
# Note 2: we are interested in the actors 'least constrained' by their position

graphr(____, node_color = "____")
```

```{r constraintplot-hint-4, purl = FALSE}
graphr(tasks, node_size = "constraint", node_color = "low_constraint")
```

```{r constraintplot-solution}
tasks <- tasks %>% 
  mutate(constraint = node_constraint(tasks), 
         low_constraint = node_is_min(node_constraint(tasks)))
graphr(tasks, node_size = "constraint", node_color = "low_constraint")
```

Why minimum? 
Because constraint measures how well connected each node's partners are,
with the implication that having few partners that are already connected to each 
other puts a node in an advantageous position to identify and 
share novel solutions to problems.
So what can we learn from this plot
about where innovation might occur within this network?

```{r constraintQ, echo=FALSE, purl = FALSE}
question("Do bridges innovate?",
         answer("No",
                correct = TRUE,
                message = learnr::random_praise()),
         answer("Yes",
                message = "Bridges can be important conduits for information, but ties do not typically have agency. Nodes do. Nodes in structural holes are called brokers. Nodes that are located in structural holes that would be there but for the node and their ties can be identified through various measures such as the number of bridges they are adjacent to, or their constraint score."),
  random_answer_order = TRUE,
  allow_retry = TRUE
)
```

## Structural Equivalence

<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExNGhvaTZ2Z3NxOW0yeTJ6ZjVoZ2ZqM2NqMjlxbXp1YnBmb2VtdTBldiZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/XcFqv8EzBiAlzeEID9/giphy.gif" alt="gif of rick multiplying"/>

Next we might ask ourselves what (other) roles there are in the network?
We want to know who plays what role in this algebra class.
Let us begin with `r gloss("structural equivalence", "structeq")`.

```{r equiv-qa, echo=FALSE, purl = FALSE}
question("Structural equivalence means identifying classes of nodes with...",
         answer("same/similar tie partners.",
                correct = TRUE,
                message = learnr::random_praise()),
         answer("same/similar pattern of ties.",
                message = "This is the definition for regular equivalence."),
         answer("same/similar distance from all others.",
                message = "This is the definition for automorphic equivalence."),
         allow_retry = TRUE,
         random_answer_order = TRUE
)
```

We're going to identify structurally equivalent positions
across all the data that we have, including 'task', 'social', and 'friend' ties.
So that is, we are using the multiplex `ison_algebra` dataset again and not
a uniplex subgraph thereof.

### Finding structurally equivalent classes

In `{manynet}`, finding how the nodes of a network can be partitioned
into structurally equivalent classes can be as easy as:

```{r find-se, exercise = TRUE, exercise.setup = "data"}
node_in_structural(ison_algebra)

ison_algebra %>% 
  mutate(se = node_in_structural(ison_algebra)) %>% 
  graphr(node_color = "se")
```

That is all.
But actually, a lot is going on behind the scenes here that we can unpack.
Understanding what is going on behind the scenes is important for understanding
how these classes are identified and how to interpret them.

### Step one: starting with a census

All equivalence classes are based on nodes' similarity across some profile of motifs.
In `{manynet}`, we call these motif *censuses*.
Any kind of census can be used, and `{manynet}` includes a few options,
but `node_in_structural()` is based off of the census of all the nodes' ties,
both outgoing and incoming ties, to characterise their relationships to tie partners.

```{r construct-cor, exercise = TRUE, exercise.setup = "data", purl = FALSE}

```

```{r construct-cor-hint-1, purl = FALSE}
# Let's use the node_by_tie() function
# The function accepts an object such as a dataset
# Hint: Which dataset are we using in this tutorial?
node_by_tie(____)
```

```{r construct-cor-hint-2, purl = FALSE}
node_by_tie(ison_algebra)
```

```{r construct-cor-hint-3, purl = FALSE}
# Now, let's get the dimensions of an object via the dim() function
dim(node_by_tie(ison_algebra))
```

```{r construct-cor-solution}
node_by_tie(ison_algebra)
dim(node_by_tie(ison_algebra))
```

We can see that the result is a matrix of 16 rows and 96 columns, 
because we want to catalogue or take a census of all the different 
incoming/outgoing partners our 16 nodes might have across these three networks.
Note also that the result is a weighted matrix; 
what would you do if you wanted it to be binary?

```{r construct-binary, exercise = TRUE, exercise.setup = "data", purl = FALSE}

```

```{r construct-binary-hint, purl = FALSE}
# we could convert the result using as.matrix, returning the ties 
as.matrix((node_by_tie(ison_algebra)>0)+0)

```

```{r construct-binary-solution}
# But it's easier to simplify the network by removing the classification into different types of ties.
# Note that this also reduces the total number of possible paths between nodes
ison_algebra %>%
  select_ties(-type) %>%
  node_by_tie()
```

Note that `node_by_tie()` does not need to be passed to `node_in_structural()` ---
this is done automatically!
However, the more generic `node_in_equivalence()` is available and can be used 
with whichever census (`node_by_*()` output) is desired.
Feel free to explore using some of the other censuses available in `{manynet}`,
though some common ones are already used in the other equivalence convenience functions,
e.g. `node_by_triad()` in `node_in_regular()` 
and `node_by_path()` in `node_in_automorphic()`.

### Step two: growing a tree of similarity

The next part takes this census and creates a `r gloss("dendrogram")` based on 
distance or dissimilarity among the nodes' census profiles.
This is all done internally within e.g. `node_in_structural()`,
though there are two important parameters that can be set to obtain different results.

First, users can set the type of distance measure used.
For enthusiasts, this is passed on to `stats::dist()`, 
so that help page should be consulted for more details.
By default `"euclidean"` is used.

Second, we can also set the type of clustering algorithm employed.
By default, `{manynet}`'s equivalence functions use `r gloss("hierarchical clustering","hierclust")`, `"hier"`,
but for compatibility and enthusiasts, we also offer `"concor"`,
which implements a CONCOR (CONvergence of CORrelations) algorithm.

We can see the difference from varying the clustering algorithm and/or distance
by plotting the dendrograms (hidden) in the output from `node_in_structural()`:

```{r varyclust, exercise = TRUE, exercise.setup = "data"}
alge <- to_named(ison_algebra) # fake names to make comparison clearer
plot(node_in_structural(alge, cluster = "hier", distance = "euclidean"))

# changing the type of distance used
plot(node_in_structural(alge, cluster = "hier", distance = "manhattan"))

# changing the clustering algorithm
plot(node_in_structural(alge, cluster = "concor", distance = "euclidean"))
```

```{r scale-interp, echo = FALSE, purl = FALSE}
question("Do you see any differences?",
         answer("Yes", correct = TRUE, message = learnr::random_praise()),
         answer("No"),
        allow_retry = TRUE)
```

So plotting a `membership` vector from `{manynet}` returns a dendrogram
with the names of the nodes on the _y_-axis and the distance between them on the _x_-axis.
Using the census as material, the distances between the nodes
is used to create a dendrogram of (dis)similarity among the nodes.
Basically, as we move to the right, we're allowing for
more and more dissimilarity among those we cluster together.
A fork or branching point indicates the level of dissimilarity
at which those two or more nodes would be said to be equivalent.
Where two nodes' branches join/fork represents the maximum distance among all their leaves,
so more similar nodes' branches fork closer to the tree's canopy,
and less similar (groups of) nodes don't join until they form basically the trunk.

Note that with the results using the hierarchical clustering algorithm,
the distance directly affects the structure of the tree (and the results).

The CONCOR dendrogram operates a bit differently to hierarchical clustering though.
Instead it represents how converging correlations repeatedly bifurcate 
the nodes into one of two partitions.
As such the 'distance' is really just the (inverse) number of steps
of bifurcations until nodes belong to the same class.

### Step three: identifying the number of clusters

Another bit of information represented in the dendrogram
is where the tree should be cut (the dashed red line) and
how the nodes are assigned to the branches (clusters) present at that cut-point.

But where does this red line come from?
Or, more technically, how do we identify the number of clusters
into which to assign nodes?

`{manynet}` includes several different ways of establishing `k`,
or the number of clusters.
Remember, the further to the right the red line is 
(the lower on the tree the cut point is)
the more dissimilar we're allowing nodes in the same cluster to be.
We could set this ourselves by just passing `k` an integer, say `k = 2`.
What we are saying here is that we want to partition the nodes
into two clusters, no matter how dissimilar they are.

```{r k-discrete, exercise = TRUE, exercise.setup = "varyclust"}
plot(node_in_structural(alge, k = 2))
```

But we're really just guessing. Maybe 2 is not the best `k`?
To establish what the best `k` is for this clustering exercise, 
we need to iterate through a number of potential `k`
and consider their fitness by some metric.
There are a couple of options here.

One is to consider, for each `k`, 
how correlated this partition is with the observed network.
When there is one cluster for each vertex in the network, 
cell values will be identical to the observed correlation matrix, 
and when there is one cluster for the whole network, 
the values will all be equal to the average correlation 
across the observed matrix.
So the correlations in each by-cluster matrix are correlated with the observed 
correlation matrix to see how well each by-cluster matrix fits the data.

Of course, the perfect partition would then be 
where all nodes are in their own cluster,
which is hardly 'clustering' at all.
Also, increasing `k` will always improve the correlation.
But if one were to plot these correlations as a line graph,
then we might expect there to be a relatively rapid increase
in correlation as we move from, for example, 3 clusters to 4 clusters,
but a relatively small increase from, for example, 13 clusters to 14 clusters.
By identifying the inflection point in this line graph,
`{manynet}` selects a number of clusters that represents a trade-off
between fit and parsimony.
This is the `k = "elbow"` method.

The other option is to evaluate a candidate for `k` based
not on correlation but on a metric of 
how similar each node in a cluster is to others in its cluster
_and_ how dissimilar each node is to those in a neighbouring cluster.
When averaged over all nodes and all clusters, 
this provides a 'silhouette coefficient' for a candidate of `k`.
Choosing the number of clusters that maximizes this coefficient,
which is what `k = "silhouette"` does,
can return a somewhat different result to the elbow method.
See what we have here, with all other arguments held the same:

```{r elbowsil, exercise = TRUE, exercise.setup = "varyclust"}
plot(node_in_structural(alge, k = "elbow"))
plot(node_in_structural(alge, k = "silhouette"))
```

Ok, so it looks like the elbow method returns `k == 3` as a good trade-off
between fit and parsimony.
The silhouette method, by contrast, sees `k == 4` as maximising cluster 
similarity and dissimilarity.
Either is probably fine here, 
and there is much debate around how to select the number of clusters anyway.
However, the silhouette method seems to do a better job of identifying 
how unique the 16th node is.
The silhouette method is also the default in `{manynet}`.

Note that there is a somewhat hidden parameter here, `range`.
Since testing across all possible numbers of clusters can get 
computationally expensive (not to mention uninterpretable) for large networks,
`{manynet}` only considers up to 8 clusters by default.
This however can be modified to be higher or lower, e.g. `range = 16`.

Finally, one last option is `k = "strict"`,
which only assigns nodes to the same partition 
if there is zero distance between them.
This is quick and rigorous solution,
however oftentimes this misses the point in finding clusters of nodes that, 
despite some variation, can be considered as similar on some dimension.

```{r strict, exercise = TRUE, exercise.setup = "varyclust"}
plot(node_in_structural(alge, k = "strict"))
```

Here for example, no two nodes have precisely the same tie-profile,
otherwise their branches would join/fork at a distance of 0.
As such, `k = "strict"` partitions the network into 16 clusters.
Where networks have a number of nodes with strictly the same profiles,
such a k-selection method might be helpful to recognise nodes in exactly 
the same structural position,
but here it essentially just reports nodes' identity.

## Blockmodelling

<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExeHVzdGlhOGkyOWg2Z3BnMmE2eDgxMnlhZHhwaWx0bGZ3dTNmZnYyaSZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/dsKkAGVnds7aXQKBI7/giphy.gif" alt="gif of mortys in a block parade"/>

### Summarising profiles

Ok, so now we have a result from establishing nodes' membership in 
structurally equivalent classes.
We can graph this of course, as above:

```{r strplot, exercise = TRUE, exercise.setup = "varyclust"}
alge %>% 
  mutate(se = node_in_structural(alge)) %>% 
  graphr(node_color = "se")
```

While this plot adds the structurally equivalent classes information to our 
earlier graph, it doesn't really help us understand how the classes relate.
That is, we might be less interested in how the individuals in the different 
classes relate, and more interested in how the different classes relate in aggregate.

<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExdmRlczhzaDJzaTBma3Q0c3pydDJjZTFkY20yMmhzbW13YmU3cnl6ZSZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9dg/QIEol1DjEwSOc9PvLu/giphy.gif" alt="gif of mortys learning about roles"/>

One option that can be useful for characterising what
the profile of ties (partners) is for each position/equivalence class
is to use `summary()`.
It summarises some census result by a partition (equivalence/membership) assignment.
By default it takes the average of ties (values),
but this can be tweaked by assigning some other summary statistic as `FUN = `.

```{r summ, exercise = TRUE, exercise.setup = "strplot", purl = FALSE}

```

```{r summ-hint, purl = FALSE}
# Let's wrap node_by_tie inside the summary() function
# and pass it a membership result
summary(node_by_tie(____),
        membership = ____)
```

```{r summ-solution}
summary(node_by_tie(alge),
        membership = node_in_structural(alge))
```

This node census produces 96 columns, 
$16 \text{nodes} * 2 \text{directions} * 3 \text{edge types}$,
it takes a bit to look through what varies between the different classes
as 'blocked'.
But only four rows (the four structurally equivalent classes, according to the default).

Another way to do this is to plot the `gloss("blockmodel")` as a whole.
Passing the `plot()` function an adjacency/incidence matrix
along with a membership vector allows the matrix to be sorted and framed
(without the membership vector, just the adjacency/incidence matrix is plotted):

```{r block, exercise = TRUE, exercise.setup = "strplot", purl = FALSE}

```

```{r block-hint, purl = FALSE}
# Let's plot the blockmodel using the plot() function we used for the dendrograms
# Instead of node_tie_census() let's us as_matrix()

plot(as_matrix(____),
     membership = ____)
```

```{r block-solution}
# plot the blockmodel for the whole network
plot(as_matrix(alge),
     membership = node_in_structural(alge))

# plot the blockmodel for the friends, tasks, and social networks separately
plot(as_matrix(to_uniplex(alge, "friends")),
     membership = node_in_structural(alge))
plot(as_matrix(to_uniplex(alge, "tasks")),
     membership = node_in_structural(alge))
plot(as_matrix(to_uniplex(alge, "social")),
     membership = node_in_structural(alge))
```

By passing the membership argument our structural equivalence results,
the matrix is re-sorted to cluster or 'block' nodes from the same class together.
This can help us interpret the general relationships between classes.
For example, when we plot the friends, tasks, and social networks using the structural equivalence results,
we might characterise them like so:

- The first group (of 6) are a bit of a mix: there seem to be two popular friends,
one that strongly reciprocates and the other that nominates no friendships but
seems to nominate others in the group as social contacts instead.
The first group rely heavily on the nerd for advice.
- The second group (of 5) seem to be strongly reciprocal in friendship and
social together, lightly advise each other but mostly go to the nerd for advice.
- The third group (of 4) are also strongly reciprocal in friendship,
but also sometimes nominate some in groups one and two as friends too.
There is at least a pair that often hang out together socially,
but this group do not hang out with the nerd much nor ask them for advice
as much as members of the other groups.
- The nerd is a loner, no friends,
but everyone hangs out with them for task advice.

## Reduced graphs

Lastly, we can consider how _classes_ of nodes relate to one another in a blockmodel.
Let's use the 4-cluster solution on the valued network (though binary is possible too)
to create a `r gloss("reduced graph","reduced")`.
A reduced graph is a transformation of a network such that 
the nodes are no longer the individual nodes but the groups of one or more nodes as a class,
and the ties between these blocked nodes can represent the sum or average tie between these classes.
Of course, this means that there can be self-ties or loops,
because even if the original network was simple (not `r gloss("complex")`), 
any within-class ties will end up becoming loops and thus the network will be complex.

```{r structblock, exercise = TRUE, exercise.setup = "varyclust", warning=FALSE}
(bm <- to_blocks(alge, node_in_structural(alge)))

bm <- bm %>% as_tidygraph %>% 
  mutate(name = c("Freaks", "Squares", "Nerds", "Geek"))
graphr(bm)
```

## Free play

Now try to find the regularly equivalent classes in the `ison_lawfirm` dataset.
As this is a multiplex network, you can make this a uniplex network first.

```{r freeplay, exercise = TRUE}

```

An extension can be to also explore `r gloss("regularly equivalent","regulareq")` 
or `r gloss("automorphically equivalent","automorphiceq")` classes.

## Glossary

Here are some of the terms that we have covered in this module:

`r print_glossary()`

---
title: "Cohesion and Community"
author: "by James Hollway"
output: 
  learnr::tutorial:
    theme: spacelab
runtime: shiny_prerendered
description: >
  This tutorial aims to teach you how to calculate various cohesion measures,
  as well as detect communities using a variety of algorithms.
---

```{r pkgs, include = FALSE, eval=TRUE}
library(manynet)
library(autograph)
```

```{r setup, include = FALSE, purl=FALSE, eval=TRUE}
library(learnr)
knitr::opts_chunk$set(echo = FALSE)
clear_glossary()
learnr::random_phrases_add(language = "fr",
                           praise = c("C'est génial!",
                                      "Beau travail",
                                      "Excellent travail!",
                                      "Bravo!",
                                      "Super!",
                                      "Bien fait",
                                      "Bien joué",
                                      "Tu l'as fait!",
                                      "Je savais que tu pouvais le faire.",
                                      "Ça a l'air facile!",
                                      "C'était un travail de première classe.",
                                      "C'est ce que j'appelle un bon travail!"),
                          encouragement = c("Bon effort",
                                            "Vous l'avez presque maîtrisé!",
                                            "Ça avance bien.",
                                            "Continuez comme ça.",
                                            "Continuez à travailler dur!",
                                            "Vous apprenez vite!",
                                            "Vous faites un excellent travail aujourd'hui."))
learnr::random_phrases_add(language = "en", 
                           praise = c("C'est génial!",
                                      "Beau travail!",
                                      "Bravo!",
                                      "Super!"),
                           encouragement = c("Bon effort"))
friends <- to_uniplex(ison_algebra, "friends")
social <- to_uniplex(ison_algebra, "social")
tasks <- to_uniplex(ison_algebra, "tasks")
```


## Setting up

In the last tutorial, we looked at network centrality and centralisation.
Nodal centrality is often thought to be an expression of structural inequality.
But nodes don't always seek to differentiate themselves in a network.
Sometimes they are interested in being part of a group.
In this tutorial, we're going to consider various ways in which 'groupness'
might be examined.

<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExbDNsNmZnNzJ2YzRyYzE5cDl4M2pxeGU4d2M1bnMwZm1qd3JvMWw3MyZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/N7AOIqqUpjXUc/giphy.gif" alt="gif of man not knowing how to study"/>

<!-- <img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExeDBpd3BsdzVjMmNpYmloM2NjcTdvdWh1YWxwaDYya2hpbWw5Y3llMCZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/YKypDIKxXXfDq/giphy.gif" alt="gif of I'll allow it"/> -->

<!-- <img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExZXFscHVkY3hxYWM4aXNoNHNrOXNpa2Jkenc5aGgwemhxNWNkbmJ6dyZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/WXVWHMW1Ki7C/giphy.gif" alt="gif of oh no scream"/> -->

<!-- <img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExMTNjOTFqMGdvc2NmN2VuaGQydXExOW96czZwamY1OHNkMmg4dW0ybyZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/z8gtBVdZVrH20/giphy.gif" alt="gif of huzzah"/> -->

<!-- <img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExZDJxanoxNGh4aWE4YXdzaGdsdG9kNDkzOHdjMThsNmVkNWU2MnJvdSZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/14rgZN1O9BxIBO/giphy.gif" alt="gif of troy's thoughts"/> -->

The data we're going to use here, "ison_algebra", is included in the `{manynet}` package.
Do you remember how to call the data?
Can you find out some more information about it?

```{r data, exercise = TRUE, purl = FALSE}

```

```{r data-hint-1, purl = FALSE}
# Let's call and load the 'ison_algebra' dataset
data("ison_algebra", package = "manynet")
# Or you can retrieve like this:
ison_algebra <- manynet::ison_algebra
```

```{r data-hint-2, purl = FALSE}
# If you want to learn more about the 'ison_algebra' dataset, use the following function (below)
?manynet::ison_algebra
```

```{r data-solution}
data("ison_algebra", package = "manynet")
?manynet::ison_algebra
# If you want to see the network object, you can run the name of the object
ison_algebra
# or print the code with brackets at the front and end of the code
(ison_algebra <- manynet::ison_algebra)
```

We can see after printing the object that the dataset is multiplex,
meaning that it contains several different types of ties: 
friendship (friends), social (social) and task interactions (tasks).

### Adding names

The network is also anonymous, but I think it would be nice to add some names,
even if it's just pretend. 
Luckily, `{manynet}` has a function for this, `to_named()`.
This makes plotting the network just a wee bit more accessible and interpretable.
Let's try adding names and graphing the network now:

```{r addingnames, exercise=TRUE, exercise.setup = "data", purl = FALSE}

```

```{r addingnames-hint-1, purl = FALSE}
ison_algebra <- to_named(ison_algebra)
```

```{r addingnames-hint-2, purl = FALSE}
graphr(ison_algebra)
```

```{r addingnames-solution}
ison_algebra <- to_named(ison_algebra)
graphr(ison_algebra)
```

Note that you will likely get a different set of names,
as they are assigned randomly from a pool of (American) first names.

### Separating multiplex networks

As a multiplex network, 
there are actually three different types of ties (friends, social, and tasks)
in this network.
We can extract them and graph them separately using `to_uniplex()`:

```{r separatingnets, exercise=TRUE, exercise.setup = "data", purl = FALSE}

```

```{r separatingnets-hint-1, purl = FALSE}
# to_uniplex extracts ties of a single type,
# focusing on the 'friends' tie attribute here
friends <- to_uniplex(ison_algebra, "friends")
gfriend <- graphr(friends) + ggtitle("Friendship")
```

```{r separatingnets-hint-2, purl = FALSE}
# now let's focus on the 'social' tie attribute
social <- to_uniplex(ison_algebra, "social")
gsocial <- graphr(social) + ggtitle("Social")
```

```{r separatingnets-hint-3, purl = FALSE}
# and the 'tasks' tie attribute
tasks <- to_uniplex(ison_algebra, "tasks")
gtask <- graphr(tasks) + ggtitle("Task")
```

```{r separatingnets-hint-4, purl = FALSE}
# now, let's compare each attribute's graph, side-by-side
gfriend + gsocial + gtask
# if you get an error here, you may need to install and load
# the package 'patchwork'.
# It's highly recommended for assembling multiple plots together.
# Otherwise you can just plot them separately on different lines.
```

```{r separatingnets-solution}
friends <- to_uniplex(ison_algebra, "friends")
gfriend <- graphr(friends) + ggtitle("Friendship")

social <- to_uniplex(ison_algebra, "social")
gsocial <- graphr(social) + ggtitle("Social")

tasks <- to_uniplex(ison_algebra, "tasks")
gtask <- graphr(tasks) + ggtitle("Task")

# We now have three separate networks depicting each type of tie from the ison_algebra network:
gfriend + gsocial + gtask
```

Note also that these are weighted networks.
`graphr()` automatically recognises these different weights and plots them.
Where useful (less dense directed networks),
`graphr()` also bends reciprocated arcs.
What (else) can we say about these three networks?

## Cohesion

Let's concentrate on the **task** network for now and calculate a few basic measures of cohesion: 
density, reciprocity, transitivity, and components.

### Density

Density represents a generalised measure of cohesion,
characterising how cohesive the network is in terms of how many
potential ties (i.e. dyads) are actualised.
Recall that there are different equations depending on the type of network.
Below are three equations:

$$A: \frac{|T|}{|N|(|N|-1)}$$
$$B: \frac{2|T|}{|N|(|N|-1)}$$
$$C: \frac{|T|}{|N||M|}$$

where $|T|$ is the number of ties in the network,
and $|N|$ and $|M|$ are the number of nodes in the first and second mode respectively.

```{r densq, echo=FALSE, purl = FALSE}
question("Which equation is used for measuring density for a directed network:",
         answer("A",
                correct = TRUE,
                message = '<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExb2M5cTNmZm1kdmszdmR6Yzc3MnczcWxvb3MzdHRic2N0ZncwYml2MSZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/YcMs3OGd89Pxu/giphy.gif" alt="gif of a thumbs up"/>'
),
         answer("B",
                message = "This is the equation for an undirected network."),
         answer("C",
                message = "This is the equation for a two-mode network."),
        allow_retry = TRUE
)
```

One can calculate the density of the network using the number of nodes
and the number of ties using the functions `net_nodes()` and `net_ties()`,
respectively:

```{r dens-explicit, exercise=TRUE, exercise.setup = "separatingnets", purl = FALSE}

```

```{r dens-explicit-solution}
# calculating network density manually according to equation
net_ties(tasks)/(net_nodes(tasks)*(net_nodes(tasks)-1))
```

but we can also just use the `{manynet}` function for calculating the density,
which always uses the equation appropriate for the type of network...

```{r dens, exercise=TRUE, exercise.setup = "separatingnets", purl = FALSE}

```

```{r dens-solution}
net_density(tasks)
```

Note that the various measures in `{manynet}` print results to three decimal points
by default, but the underlying result retains the same recurrence.
So same result...

```{r dens-qa, echo=FALSE, purl = FALSE}
question("Is this network's density high or low in absolute terms?",
         answer("High",
                message = "The closer the value is to 1, the more dense the network and the more cohesive the network is as a whole."),
         answer("Low",
                correct = TRUE,
                message = 'The closer the value is to 0, the sparser the network and the less cohesive the network is as a whole. But this is still quite typical density for a relatively small, social network like this one.  <br><br>    <img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExOWs4Y292OXg5YmlvNTF3OWp4YWxmNjNpdm11c2kyMm55Z2U4eTJkeiZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/6agGMtS4rEVSo/giphy.gif" alt="gif of i knew it"/>'),
        allow_retry = TRUE
)
```

Density offers an important baseline measure for characterising
the network as a whole.

## Closure

In this section we're going to move from generalised measures of cohesion,
like density, to more localised measures of cohesion.
These are not only measures of cohesion though,
but are also often associated with certain mechanisms of closure.
Closure involves ties being more likely because other ties are present.
There are two common examples of this in the literature:
`gloss("reciprocity")`, where a directed tie is often likely to prompt a reciprocating tie,
and `gloss("transitivity")`, where a directed two-path is likely to be shortened
by an additional arc connecting the first and third nodes on that path.

<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExZ29idmxwMGNpb3Z6YzV0NGdremNoZ3A3OXdjeHlsd3UwNDdwY3ZwOCZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/dxVdo4ca7FkME/giphy.gif" alt="gif of ah ha gotcha"/>

### Reciprocity

First, let's calculate 
`r gloss("reciprocity")`
in the task network.
While one could do this by hand,
it's more efficient to do this using the `{manynet}` package.
Can you guess the correct name of the function?

```{r recip, exercise=TRUE, exercise.setup = "separatingnets", purl = FALSE}

```

```{r recip-solution}
net_reciprocity(tasks)
# this function calculates the amount of reciprocity in the whole network
```

Wow, this seems quite high based on what we observed visually!
But if we look closer, this makes sense.
We can use `tie_is_reciprocated()` to identify those ties that are
reciprocated and not.

```{r recip-explanation, exercise = TRUE}
tasks %>% mutate_ties(rec = tie_is_reciprocated(tasks)) %>% graphr(edge_color = "rec")
net_indegree(tasks)
```

So we can see that indeed there are very few asymmetric ties,
and yet node 16 is both the sender and receiver of most of the task activity.
So our reciprocity measure has taught us something about this network
that might not have been obvious visually.

### Transitivity

And let's calculate 
`r gloss("transitivity")`
in the task network.
Again, can you guess the correct name of this function?

```{r trans, exercise=TRUE, exercise.setup = "separatingnets", purl = FALSE}

```

```{r trans-solution}
net_transitivity(tasks)
# this function calculates the amount of transitivity in the whole network
```

```{r trans-interp, echo=FALSE, purl = FALSE}
question("What can we say about task closure in this network? Choose all that apply.",
  answer("Transitivity for the task network is 0.568",
         correct = TRUE,
         message = '<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExZXBpNDlsZzNraTJ1bHN6MW1scjA4aDVzdXo1NW11bGI5bWF0dzYxMyZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/Hg7gPa05K6bAc/giphy.gif" alt="gif of cool cool cool"/>'),
  answer("Transitivity for the task network is -0.568", 
         message = "Transivitity must be between 0 and 1."),
  answer("Transitivity is quite low in this network", 
         message = "Transitivity is often around 0.3 in most social networks."),
  answer("Transitivity is quite high in this network", 
         correct = TRUE),
  answer("Transitivity is likely higher in the task network than the friendship network", 
         correct = TRUE),
  random_answer_order = TRUE,
  allow_retry = TRUE
)
```

## Projection

### A two-mode network

The next dataset, 'ison_southern_women', is also available in `{manynet}`.
It's a classic two-mode network about the attendance of 18 women
at 14 social events in the 1930s American South.
For more details see `?ison_southern_women` for documentation in your R/RStudio.
Let's load and graph the data.

```{r setup-women, exercise=TRUE, exercise.setup = "data", purl = FALSE}

```

```{r setup-women-hint-1, purl = FALSE}
# let's load the data and analyze it
data("ison_southern_women")
ison_southern_women
```

```{r setup-women-hint-2, purl = FALSE}
graphr(ison_southern_women, node_color = "type")
graphr(ison_southern_women, "railway", node_color = "type")
```

```{r setup-women-solution}
data("ison_southern_women")
ison_southern_women
graphr(ison_southern_women, node_color = "type")
```

### Project two-mode network into two one-mode networks

Now what if we are only interested in one part of the network?
For that, we can obtain a 'projection' of the two-mode network.
There are two ways of doing this.
The hard way...

```{r hardway, exercise=TRUE, exercise.setup = "setup-women", purl = FALSE}

```

```{r hardway-solution}
twomode_matrix <- as_matrix(ison_southern_women)
women_matrix <- twomode_matrix %*% t(twomode_matrix)
event_matrix <- t(twomode_matrix) %*% twomode_matrix
```

Or the easy way:

```{r easyway, exercise=TRUE, exercise.setup = "setup-women"}
# women-graph
# to_mode1(): Results in a weighted one-mode object that retains the row nodes from
# a two-mode object, and weights the ties between them on the basis of their joint
# ties to nodes in the second mode (columns)

women_graph <- to_mode1(ison_southern_women)
graphr(women_graph)

# note that projection `to_mode1` involves keeping one type of nodes
# this is different from to_uniplex above, which keeps one type of ties in the network

# event-graph
# to_mode2(): Results in a weighted one-mode object that retains the column nodes from
# a two-mode object, and weights the ties between them on the basis of their joint ties
# to nodes in the first mode (rows)

event_graph <- to_mode2(ison_southern_women)
graphr(event_graph)
```

### Projection options

`{manynet}` also includes several other options for how to construct the projection.
The default ("count") might be interpreted as indicating the degree or amount of
shared ties to the other mode.
"jaccard" divides this count by the number of nodes in the other mode 
to which either of the nodes are tied.
It can thus be interpreted as opportunity weighted by participation.
"rand" instead counts both shared ties and shared absences,
and can thus be interpreted as the degree of behavioural mirroring between the
nodes.
Lastly, "pearson" (Pearson's coefficient) and "yule" (Yule's Q) produce
correlations in ties for valued and binary data respectively.

```{r otherway, exercise=TRUE, exercise.setup = "setup-women", purl = FALSE}

```

```{r otherway-solution}
tie_weights(to_mode1(ison_southern_women, similarity = "count"))
tie_weights(to_mode1(ison_southern_women, similarity = "jaccard"))
tie_weights(to_mode1(ison_southern_women, similarity = "rand"))
tie_weights(to_mode1(ison_southern_women, similarity = "pearson"))
tie_weights(to_mode1(ison_southern_women, similarity = "yule"))
```

Practically speaking, most projection is conducted using the "count" method,
as it reflects the absolute overlap, the frequency of co-occurrence.
As such, it is most commonly used in social and organisational network analysis,
reflecting shared group membership or collaboration intensity.
But it is sensitive to asymmetries in the number of nodes in each node set and
the degree distribution of each node set, and ignores the absence of ties as
uninformative.
To normalise for these effects, "jaccard" and "rand" are often used.
"pearson" and "yule" are less commonly used, but can be useful when
interested in the correlation of ties between nodes.

### Closure in two-mode networks

Let's return to the question of closure.
First try one of the closure measures we have already treated that gives us
a sense of shared partners for one-mode networks.
Then compare this with `net_equivalency()`, which can be used on the original
two-mode network.

```{r twomode-cohesion, exercise=TRUE, exercise.setup = "easyway", purl = FALSE}

```

```{r twomode-cohesion-hint-1, purl = FALSE}
# net_transitivity(): Calculate transitivity in a network

net_transitivity(women_graph)
net_transitivity(event_graph)
```

```{r twomode-cohesion-hint-2, purl = FALSE}
# net_equivalency(): Calculate equivalence or reinforcement in a (usually two-mode) network

net_equivalency(ison_southern_women)
```

```{r twomode-cohesion-solution}
net_transitivity(women_graph)
net_transitivity(event_graph)
net_equivalency(ison_southern_women)
```

```{r equil-interp, echo=FALSE, purl = FALSE}
question("What do we learn from this? Choose all that apply.",
  answer("Transitivity in the women projection is very high.", 
         correct = TRUE),
  answer("Transitivity in the event projection is very high.", 
         correct = TRUE),
  answer("Equivalence for the two-mode Southern Women dataset is moderate.",
         correct = TRUE,
         message = '<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExZmdqZHFraGR5Ym9udTBrYzA1ejhwd3ljOXdma3ppNWp2cXozMTh0bCZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/xqjZLKH1iTBe0/giphy.gif" alt="gif of you just wrinkled my brain"/>'),
  answer("Transitivity will often be very high in projected networks.", 
         correct = TRUE),
  answer("Projection obscures which women are members of which events and vice versa.", 
         correct = TRUE),
  # random_answer_order = TRUE,
  allow_retry = TRUE
)
```

Try to explain in no more than a paragraph why projection can lead to misleading transitivity measures and what some consequences of this might be.

<!-- Explain in no more than a paragraph how structural balance might lead to group identity. -->

## Components

Now let's look at the friendship network, 'friends'.
We're interested here in how many 
`r gloss("components", "component")`
there are.
By default, the `net_components()` function will
return the number of _strong_ components for directed networks.
For _weak_ components, you will need to first make the network 
`r gloss("undirected")`.
Remember the difference between weak and strong components?

```{r weak-strong, echo = FALSE, purl = FALSE}
question("Weak components...",
  answer("don't care about tie direction when establishing components.", 
         correct = TRUE),
  answer("care about tie direction when establishing components."),
  allow_retry = TRUE
)
```

```{r comp-no, exercise=TRUE, exercise.setup = "separatingnets", purl = FALSE}

```

```{r comp-no-hint-1, purl = FALSE}
net_components(friends)
# note that friends is a directed network
# you can see this by calling the object 'friends'
# or by running `is_directed(friends)`
```

```{r comp-no-hint-2, purl = FALSE}
# Now let's look at the number of components for objects connected by an undirected edge
# Note: to_undirected() returns an object with all tie direction removed, 
# so any pair of nodes with at least one directed edge 
# will be connected by an undirected edge in the new network.
net_components(to_undirected(friends))
```

```{r comp-no-solution}
# note that friends is a directed network
net_components(friends)
net_components(to_undirected(friends))
```

```{r comp-interp, echo = FALSE, purl = FALSE}
question("How many components are there?",
  answer("2", 
         message = "There are more than 2 components."),
  answer("3", 
         message = "There are 3 _weak_ components.",
         correct = TRUE),
  answer("4", 
         message = 'There are 4 _strong_ components. <img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExMDB5bzhjcGo3NHozZm5jdHcyMWk5emhnajNyZnFxc3Z2cmJnbGN0ZyZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/YpS0oFFC2VJ84/giphy.gif" alt="gif of two thumbs up"/>',
         correct = TRUE),
  answer("5", 
         message = "There are fewer than 5 components."),
  allow_retry = TRUE
)
```

So we know how many components there are,
but maybe we're also interested in which nodes are members of which components?
`node_components()` returns a membership vector
that can be used to color nodes in `graphr()`:

```{r comp-memb, exercise=TRUE, exercise.setup = "separatingnets", purl = FALSE}

```

```{r comp-memb-hint-1, purl = FALSE}
friends <- friends %>% 
  mutate(weak_comp = node_components(to_undirected(friends)),
         strong_comp = node_components(friends))
# node_components returns a vector of nodes' memberships to components in the network
# here, we are adding the nodes' membership to components as an attribute in the network
# alternatively, we can also use the function `add_node_attribute()`
# eg. `add_node_attribute(friends, "weak_comp", node_components(to_undirected(friends)))`
```

```{r comp-memb-hint-2, purl = FALSE}
graphr(friends, node_color = "weak_comp") + ggtitle("Weak components") +
graphr(friends, node_color = "strong_comp") + ggtitle("Strong components")
# by using the 'node_color' argument, we are telling graphr to colour 
# the nodes in the graph according to the values of the 'weak_comp' attribute in the network 
```

```{r comp-memb-solution}
friends <- friends %>% 
  mutate(weak_comp = node_components(to_undirected(friends)),
         strong_comp = node_components(friends))
graphr(friends, node_color = "weak_comp") + ggtitle("Weak components") +
graphr(friends, node_color = "strong_comp") + ggtitle("Strong components")
```

```{r node-comp-interp, echo = FALSE, purl = FALSE}
question("Why is there a difference between the weak and strong components results?",
  answer("Because one node has only incoming ties.",
         correct = TRUE),
  answer("Because three nodes cannot reach any other nodes.",
         correct = TRUE),
  answer("Because there is an extra isolate."),
  answer("Because the tie strength matters."),
  random_answer_order = TRUE,
  allow_retry = TRUE
)
```

## Factions

Components offer a precise way of understanding groups in a network.
However, they can also ignore some 'groupiness' that is obvious to even a
cursory examination of the graph.
The `irps_blogs` network concerns the url links between political blogs 
in the 2004 election.
It is a big network (you can check below).
In our experience, it can take a few seconds 

```{r blogsize, exercise = TRUE}
# This is a large network
net_nodes(irps_blogs)
# Let's concentrate on just a sample of 240 
# (by deleting a random sample of 1250)
blogs <- delete_nodes(irps_blogs, sample(1:1490, 1250))
graphr(blogs)
```

But are they all actually linked?
Even among the smaller sample, there seems to be a number of isolates.
We can calculate the number of isolates by simply summing `node_is_isolate()`.

```{r blogisolates, exercise = TRUE, exercise.setup = "blogsize"}
sum(node_is_isolate(blogs))
```

Since there are many isolates, there will be many components,
even if we look at weak components and not just strong components.

```{r blogcomp, exercise = TRUE, exercise.setup = "blogsize"}
net_components(blogs)
net_components(to_undirected(blogs))
```

### Giant component

So, it looks like most of the (weak) components are due to isolates!
How do we concentrate on the main component of this network?
Well, the main/largest component in a network is called the
`r gloss("giant component", "giant")`.

```{r blogtogiant, exercise=TRUE, warning=FALSE, fig.width=9, exercise.setup = "blogsize"}
blogs <- blogs %>% to_giant()
sum(node_is_isolate(blogs))
graphr(blogs)
```

Finally, we have a single 'giant' component to examine.
However, now we have a different kind of challenge: 
everything is one big hairball.
And yet, if we think about what we might expect of the structure of a network
of political blogs, we might not think it is so undifferentiated.
We might hypothesise that, despite the _graphical_ presentation of a hairball,
there is actually a reasonable partition of the network into two factions.

### Finding a partition

To find a partition in a network, we use the `node_in_partition()` function.
All `node_in_*()` functions return a string vector the length of the number 
of nodes in the network.
It is a string vector because this is how a categorical result is obtained.
We can assign the result of this function to the nodes in the network
(because it is the length of the nodes in the network),
and graph the network using this result.

```{r bloggraph, exercise=TRUE, exercise.setup = "blogtogiant", warning=FALSE, fig.width=9}
blogs %>% mutate_nodes(part = node_in_partition()) %>% 
  graphr(node_color = "part")
```

We see from this graph that indeed there seems to be an obvious separation
between the left and right 'hemispheres' of the network.

## Modularity

But what is the 'fit' of this assignment of the blog nodes into two partitions?
The most common measure of the fit of a community assignment in a network is
`r gloss("modularity")`.

```{r blogmod, exercise=TRUE, exercise.setup = "blogtogiant"}
net_modularity(blogs, membership = node_in_partition(blogs))
```

Remember that modularity ranges between 1 and -1.
How can we interpret this result?

```{r blogQ, echo = FALSE, purl = FALSE}
question("Which statements about modularity are true?",
  answer("A community assignment with modularity > 0 means there are more ties within communities than between communities.",
         correct = TRUE),
  answer("A community assignment with modularity = 0 means any tie is as likely to occur within a community as between assigned communities.",
         correct = TRUE),
  answer("In a community assignment with modularity = 1, every community must have an internal density of 1 and an external density of 0."),
  answer("A community assignment with modularity = -1 is impossible."),
  random_answer_order = TRUE,
  allow_retry = TRUE
)
```

While the partition algorithm is useful for deriving a partition of the network
into the number of factions assigned,
it is still an algorithm that tries to maximise modularity.
Other times we might instead have an empirically collected grouping,
and we are keen to see how 'modular' the network is around this attribute.
This only works on categorical attributes, of course,
but is otherwise quite flexible.

```{r blogmodassign, exercise=TRUE, exercise.setup = "blogtogiant", warning=FALSE, fig.width=9}
graphr(blogs, node_color = "Leaning")
net_modularity(blogs, membership = node_attribute(blogs, "Leaning"))
```

<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExcXFwZWF2azV0YTlwdnM0bTZncXpuOTdxdHFyMjJqcXJod2V3b29vYiZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/PsBRTPKG71YVq/giphy.gif" alt="gif of Chevy Chase saying plot twist"/>

How interesting.
Perhaps the partitioning algorithm is not the algorithm that maximises
modularity after all...
Perhaps we need to look further and see whether there is another solution here
that returns an even greater modularity criterion.

## Communities

Ok, the friendship network has 3-4 components, but how many 'groups' are there?
Visually, it looks like there are two denser clusters within the main component.

Today we'll use the 'friends' subgraph for exploring community detection methods.
For clarity and simplicity, 
we will concentrate on the main component (the so-called 'giant' component)
and consider friendship undirected.

```{r manip-fri, exercise=TRUE, exercise.setup = "separatingnets", purl = FALSE}
# to_giant() returns an object that includes only the main component without any smaller components or isolates
(friends <- to_giant(friends))
(friends <- to_undirected(friends))
graphr(friends)
```

Comparing `friends` before and after these operations, 
you'll notice the number of ties decreases as reciprocated directed ties
are consolidated into single undirected ties, 
and the number of nodes decreases as two isolates are removed.

There is no one single best community detection algorithm. 
Instead there are several, each with their strengths and weaknesses. 
Since this is a rather small network, we'll focus on the following methods: 
walktrap, edge betweenness, and fast greedy.
(Others are included in `{manynet}`/`{igraph}`)
As you use them, consider how they portray communities and consider which one(s) 
afford a sensible view of the social world as cohesively organized.

### Walktrap

This algorithm detects communities through a series of short random walks, 
with the idea that nodes encountered on any given random `r gloss("walk")` 
are more likely to be within a community than not.
It was proposed by Pons and Latapy (2005).

The algorithm initially treats all nodes as communities of their own, then 
merges them into larger communities, still larger communities, and so on.
In each step a new community is created from two other communities, 
and its ID will be one larger than the largest community ID so far. 
This means that before the first merge we have n communities 
(the number of vertices in the graph) numbered from zero to n-1.
The first merge creates community n, the second community n+1, etc. 
This merge history is returned by the function: 
` # ?igraph::cluster_walktrap`

Note the "steps=" argument that specifies the length of the random walks.
While `{igraph}` sets this to 4 by default, 
which is what is recommended by Pons and Latapy,
Waugh et al (2009) found that for many groups (Congresses), 
these lengths did not provide the maximum modularity score. 
To be thorough in their attempts to optimize modularity, they ran the walktrap 
algorithm 50 times for each group (using random walks of lengths 1–50) and 
selected the network partition with the highest modularity value from those 50. 
They call this the "maximum modularity partition" and insert the parenthetical 
"(though, strictly speaking, this cannot be proven to be the optimum without 
computationally-prohibitive exhaustive enumeration (Brandes et al. 2008))."

So let's try and get a community classification using the walktrap algorithm, `node_in_walktrap()`,
with path lengths of the random walks specified to be 50.

```{r walk, exercise=TRUE, exercise.setup = "separatingnets"}
friend_wt <- node_in_walktrap(friends, times=50)
```

```{r walk-hint-1, purl = FALSE}
friend_wt # note that it prints pretty, but underlying its just a vector:
c(friend_wt)
```

```{r walk-hint-2, purl = FALSE}
# This says that dividing the graph into 2 communities maximises modularity,
# one with the nodes 
which(friend_wt == 1)
# and the other 
which(friend_wt == 2)
```

```{r walk-hint-3, purl = FALSE}
# resulting in a modularity of 
net_modularity(friends, friend_wt)
```

```{r walk-solution}
friend_wt <- node_in_walktrap(friends, times=50)
# results in a modularity of 
net_modularity(friends, friend_wt)
```

We can also visualise the clusters on the original network
How does the following look? Plausible?

```{r walkplot, exercise=TRUE, exercise.setup = "walk", purl = FALSE}
```

```{r walkplot-hint-1, purl = FALSE}
# plot 1: groups by node color

friends <- friends %>% 
  mutate(walk_comm = friend_wt)
graphr(friends, node_color = "walk_comm")
```

```{r walkplot-hint-2, purl = FALSE}
#plot 2: groups by borders

# to be fancy, we could even draw the group borders around the nodes using the node_group argument
graphr(friends, node_group = "walk_comm")
```

```{r walkplot-hint-3, purl = FALSE}
# plot 3: group and node colors

# or both!
graphr(friends,
       node_color = "walk_comm",
       node_group = "walk_comm") +
  ggtitle("Walktrap",
    subtitle = round(net_modularity(friends, friend_wt), 3))
# the function `round()` rounds the values to a specified number of decimal places
# here, we are telling it to round the net_modularity score to 3 decimal places,
# but the score is exactly 0.27 so only two decimal places are printed.
```

```{r walkplot-solution}
friends <- friends %>% 
  mutate(walk_comm = friend_wt)
graphr(friends, node_color = "walk_comm")
# to be fancy, we could even draw the group borders around the nodes using the node_group argument
graphr(friends, node_group = "walk_comm")
# or both!
graphr(friends,
       node_color = "walk_comm",
       node_group = "walk_comm") +
  ggtitle("Walktrap",
    subtitle = round(net_modularity(friends, friend_wt), 3))
```

This can be helpful when polygons overlap to better identify membership
Or you can use node color and size to indicate other attributes...

### Edge Betweenness

Edge betweenness is like `r gloss("betweenness")` centrality but for ties not nodes.
The edge-betweenness score of an edge measures the number of
shortest paths from one vertex to another that go through it.

The idea of the edge-betweenness based community structure detection is that 
it is likely that edges connecting separate clusters have high edge-betweenness, 
as all the shortest paths from one cluster to another must traverse through them. 
So if we iteratively remove the edge with the highest edge-betweenness score 
we will get a hierarchical map (`r gloss("dendrogram")`) of the communities in the graph. 

The following works similarly to walktrap, but no need to set a step length. 

```{r eb, exercise=TRUE, exercise.setup = "separatingnets"}
friend_eb <- node_in_betweenness(friends)
friend_eb
```

How does community membership differ here from that found by walktrap?

We can see how the edge betweenness community detection method works
here: http://jfaganuk.github.io/2015/01/24/basic-network-analysis/

To visualise the result:

```{r ebplot, exercise=TRUE, exercise.setup = "eb", purl = FALSE}

```

```{r ebplot-hint-1, purl = FALSE}
# create an object

friends <- friends %>% 
  mutate(eb_comm = friend_eb)
```

```{r ebplot-hint-2, purl = FALSE}
# create a graph with a title and subtitle returning the modularity score

graphr(friends,
       node_color = "eb_comm",
       node_group = "eb_comm") +
  ggtitle("Edge-betweenness",
    subtitle = round(net_modularity(friends, friend_eb), 3))
```

```{r ebplot-solution}
friends <- friends %>% 
  mutate(eb_comm = friend_eb)
graphr(friends,
       node_color = "eb_comm",
       node_group = "eb_comm") +
  ggtitle("Edge-betweenness",
    subtitle = round(net_modularity(friends, friend_eb), 3))
```

For more on this algorithm, see M Newman and M Girvan: Finding and 
evaluating community structure in networks, Physical Review E 69, 026113
(2004), https://arxiv.org/abs/cond-mat/0308217. 

### Fast Greedy

This algorithm is the Clauset-Newman-Moore algorithm. 
Whereas edge betweenness was `r gloss("divisive")` (top-down), 
the fast greedy algorithm is `r gloss("agglomerative")` (bottom-up).

At each step, the algorithm seeks a merge that would most increase modularity.
This is very fast, but has the disadvantage of being a greedy algorithm, 
so it might not produce the best overall community partitioning, 
although I personally find it both useful and in many cases quite "accurate". 

```{r fg, exercise=TRUE, exercise.setup = "separatingnets", purl = FALSE}

```

```{r fg-hint-1, purl = FALSE}
friend_fg <- node_in_greedy(friends)
friend_fg # Does this result in a different community partition?
net_modularity(friends, friend_fg) # Compare this to the edge betweenness procedure
```

```{r fg-hint-2, purl = FALSE}
# Again, we can visualise these communities in different ways:
friends <- friends %>% 
  mutate(fg_comm = friend_fg)
graphr(friends,
       node_color = "fg_comm",
       node_group = "fg_comm") +
  ggtitle("Fast-greedy",
    subtitle = round(net_modularity(friends, friend_fg), 3))
# 
```

```{r fg-solution}
friend_fg <- node_in_greedy(friends)
friend_fg # Does this result in a different community partition?
net_modularity(friends, friend_fg) # Compare this to the edge betweenness procedure

# Again, we can visualise these communities in different ways:
friends <- friends %>% 
  mutate(fg_comm = friend_fg)
graphr(friends,
       node_color = "fg_comm",
       node_group = "fg_comm") +
  ggtitle("Fast-greedy",
    subtitle = round(net_modularity(friends, friend_fg), 3))
```

See A Clauset, MEJ Newman, C Moore: 
Finding community structure in very large networks, 
https://arxiv.org/abs/cond-mat/0408187

```{r comm-comp, echo=FALSE, purl = FALSE}
question("What is the difference between communities and components?",
         answer("Communities and components are just different terms for the same thing"),
         answer("Communities are a stricter form of component"),
         answer("Components are about paths whereas communities are about the relationship between within-group and between-group ties",
                correct = TRUE),
  random_answer_order = TRUE,
         allow_retry = TRUE)
```

### Communities

Lastly, `{manynet}` includes a function to run through and find the membership
assignment that maximises `r gloss("modularity")` across any of the applicable community
detection procedures.
This is helpfully called `node_in_community()`.
This function pays attention to the size and other salient network properties,
such as whether it is directed, weighted, or connected.
It either uses the algorithm that maximises modularity by default 
(`node_in_optimal()`) for smaller networks, or surveys the approximations 
provided by other algorithms for the result that maximises the modularity.

```{r incomm, exercise = TRUE, exercise.setup = "separatingnets"}
node_in_community(friends)
```

It is important to name _which_ algorithm produced the membership assignment
that is then subsequently analysed though.

```{r alg-comp, echo=FALSE, purl = FALSE}
question("Which algorithm provides the 'best' membership assignment here?",
         answer("Walktrap"),
         answer("Edge betweenness"),
         answer("Leiden"),
         answer("Louvain"),
         answer("Infomap"),
         answer("Fluid"),
         answer("Fast Greedy"),
         answer("Eigenvector"),
         answer("Spinglass"),
         answer("Partition"),
         answer("Optimal",
                correct = TRUE),
  random_answer_order = TRUE,
         allow_retry = TRUE)
```

## Free play

<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExc3Z1ZmlueXJhNzU5eHQ4eW14bHc3YWRiemdubnhkdzR5YzFvOHN4dCZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/mnLSsJi13Sgmc/giphy.gif" alt="gif of two dancing"/>

We've looked here at the `irps_blogs` dataset.
Now have a go at the `irps_books` dataset.
What is the density? Does it make sense to investigate reciprocity,
transitivity, or equivalence? How can we interpret the results?
How many components in the network? Is there a strong factional structure?
Which community detection algorithm returns the highest modularity score,
or corresponds best to what is in the data or what you see in the graph?

```{r freeplay, exercise = TRUE, fig.width=9}
irps_books
```

<!-- <img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExZXlkZWQ2d3NvM3NhbHFyMDBhc25meXhvOTU2YTU4dWQ0cXhlbGlsNyZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/eobYwOL3jdtqo/giphy.gif" alt="gif of a movie reference"/> -->

## Glossary

<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExMTQ4bWhkbGJ1emlpaDd4a3RocDg2OWxzNnB3czdpZXBoZjdsYWhqNSZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/10VoRuQWt2mb28/giphy.gif" alt="gif of annie raising her hand"/>

Here are some of the terms that we have covered in this module:

`r print_glossary()`

---
title: "Centrality"
author: "by James Hollway"
output: 
  learnr::tutorial:
    theme: journal
runtime: shiny_prerendered
description: >
  This tutorial aims to show how to measure and map degree, betweenness,
  closeness, eigenvector, and other types of centrality and centralization.
---

```{r pkgs, include = FALSE, eval=TRUE}
library(manynet)
library(autograph)
```

```{r setup, include = FALSE, purl=FALSE, eval=TRUE}
library(learnr)
knitr::opts_chunk$set(echo = FALSE)
clear_glossary()
learnr::random_phrases_add(language = "fr",
                           praise = c("C'est génial!",
                                      "Beau travail",
                                      "Excellent travail!",
                                      "Bravo!",
                                      "Super!",
                                      "Bien fait",
                                      "Bien joué",
                                      "Tu l'as fait!",
                                      "Je savais que tu pouvais le faire.",
                                      "Ça a l'air facile!",
                                      "C'était un travail de première classe.",
                                      "C'est ce que j'appelle un bon travail!"),
                          encouragement = c("Bon effort",
                                            "Vous l'avez presque maîtrisé!",
                                            "Ça avance bien.",
                                            "Continuez comme ça.",
                                            "Continuez à travailler dur!",
                                            "Vous apprenez vite!",
                                            "Vous faites un excellent travail aujourd'hui."))
learnr::random_phrases_add(language = "en", 
                           praise = c("C'est génial!",
                                      "Beau travail!",
                                      "Bravo!",
                                      "Super!"),
                           encouragement = c("Bon effort"))
ison_brandes2 <- ison_brandes %>% rename(type = twomode_type)
```

## Today's target

The aim of this tutorial is to show how we can measure and map
degree, betweenness, closeness, eigenvector, and other types of centrality,
explore their distributions and calculate the corresponding centralisation.

<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExbG5oYmVtYmcxMWs1d3MzOGh3Zng5dTZhc241ZHR2NTE0YThiZWp4NiZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/970Sr8vpwEbXG/giphy.gif" height="480px"/>

### Setting up

For this exercise, we'll use the `ison_brandes` dataset in `{manynet}`.
This dataset is in a 'tidygraph' format, 
but `manynet` makes it easy to coerce this into other forms
to be compatible with other packages.
We can create a two-mode version of the dataset
by renaming the nodal attribute "twomode_type" to just "type".
Let's begin by graphing these datasets using `graphr()`.

```{r coercion, exercise = TRUE, purl = FALSE}

```

```{r coercion-hint-1, purl = FALSE}
# Let's graph the one-mode version
graphr(____)
```

```{r coercion-hint-2, purl = FALSE}
# Now, let's create a two-mode version 'ison_brandes2' and graph it.
ison_brandes2 <- ison_brandes %>% rename(type = twomode_type)
graphr(____)
```

```{r coercion-solution}
# plot the one-mode version
graphr(ison_brandes)
ison_brandes2 <- ison_brandes %>% rename(type = twomode_type)
# plot the two-mode version
graphr(ison_brandes2, layout = "bipartite")
```

The network is anonymous, but I think it would be nice to add some names,
even if it's just pretend.
Luckily, `{manynet}` has a function for this: `to_named()`.
This makes plotting the network just a wee bit more accessible and interpretable:

```{r addingnames, exercise = TRUE, purl = FALSE}
ison_brandes <- to_named(ison_brandes)
```

```{r addingnames-hint-1, purl = FALSE}
# Now, let's graph using the object names: "ison_brandes"
graphr(____)
```

```{r addingnames-solution}
ison_brandes <- to_named(ison_brandes)
# plot network with names
graphr(ison_brandes)
```

Note that you will likely get a different set of names (though still alphabetical),
as they are assigned randomly from a pool of (American) first names.

## Degree centrality

<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExM3d6eGd2OHRxb2FvbHYzc2oyenFzOG5qcjhzMm82Mm1ncTVpOHhreiZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/uVd5gVHAq63SjtjAOe/giphy.gif" height="480px"/>

Let's start with calculating `r gloss("degree")`.
Remember that degree centrality is just the number of incident edges/ties to each node.
It is therefore easy to calculate yourself.
Just sum the rows or columns of the matrix!

```{r degreesum, exercise = TRUE, exercise.setup = "addingnames", purl = FALSE}

```

```{r degreesum-hint-1, purl = FALSE}
# We can calculate degree centrality like this:
(mat <- as_matrix(ison_brandes))
(degrees <- rowSums(mat))
rowSums(mat) == colSums(mat)
```

```{r degreesum-hint-2, purl = FALSE}
# Or by using a built in command in manynet like this:
node_degree(ison_brandes, normalized = FALSE)
```

```{r degreesum-solution}
# manually calculate degree centrality
mat <- as_matrix(ison_brandes)
degrees <- rowSums(mat)
rowSums(mat) == colSums(mat)
# You can also just use a built in command in manynet though:
node_degree(ison_brandes, normalized = FALSE)
```

```{r degreesum-Q, echo=FALSE, purl = FALSE}
question("Are the row sums the same as the column sums?",
  answer("Yes",
         correct = TRUE,
         message = "That's right, that's because this is an undirected network."),
  answer("No"),
  allow_retry = FALSE
)
```

```{r degreeinterpQ, purl = FALSE}
question("In what ways are higher degree nodes more 'central'?",
         answer("They have more power than other nodes",
                message = "Not necessarily"),
         answer("They are more active than other nodes",
                correct = TRUE, message = learnr::random_praise()),
         answer("They would be located in the centre of the graph",
                message = "Not necessarily"),
         answer("They would be able to control information flow in the network",
                message = "Not necessarily"),
         allow_retry = TRUE,
         random_answer_order = TRUE)
```

## Degree distributions

<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExanZ4OWhwM2k0eWU1ancza3NtYXlpZDFkdTJub2RvMnFkYnlwbWZ4dCZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/3o7aTHbH39h8xYePza/giphy.gif" height="400px"/>

Often we are interested in the distribution of (degree) centrality in a network.
`{autograph}` offers a way to get a pretty good first look at this `r gloss("distribution")`,
though there are more elaborate ways to do this in base and grid graphics.

```{r distrib, exercise = TRUE, exercise.setup = "addingnames", purl = FALSE}

```

```{r distrib-solution}
# distribution of degree centrality scores of nodes
plot(node_degree(ison_brandes))
```

What's plotted here by default is both the degree distribution as a histogram,
as well as a density plot overlaid on it in red.

```{r degdistshapeQ, purl = FALSE}
question("What kind of shape does this degree distribution have?",
         answer("Exponential"),
         answer("Hard to tell",
                correct = TRUE, message = learnr::random_praise()),
         answer("Uniform"),
         answer("Normal"),
         allow_retry = TRUE,
         random_answer_order = TRUE)
```

```{r degdistinterpQ, purl = FALSE}
question("What can the degree distribution tell us?",
         answer("Whether the network is centralized around a few key nodes",
                correct = TRUE, message = learnr::random_praise()),
         answer("Whether there are a few highly connected nodes and many poorly connected nodes",
                correct = TRUE, message = learnr::random_praise()),
         answer("Whether the network is hierarchical",
                message = "Not necessarily"),
         answer("Whether the network is robust to random failure",
                message = "Not necessarily"),
         allow_retry = TRUE,
         random_answer_order = TRUE)
```

## Other centralities

Other measures of centrality can be a little trickier to calculate by hand.
Fortunately, we can use functions from `{manynet}` to help calculate the
`r gloss("betweenness")`, `r gloss("closeness")`, and `r gloss("eigenvector")` centralities for each node in the network.
Let's collect the vectors of these centralities for the `ison_brandes` dataset:

```{r micent, exercise = TRUE, exercise.setup = "addingnames", purl = FALSE}

```

```{r micent-hint-1, purl = FALSE}
# Use the node_betweenness() function to calculate the
# betweenness centralities of nodes in a network
node_betweenness(ison_brandes)
```

```{r micent-hint-2, purl = FALSE}
# Use the node_closeness() function to calculate the 
# closeness centrality of nodes in a network
node_closeness(ison_brandes)
```

```{r micent-hint-3, purl = FALSE}
# Use the node_eigenvector() function to calculate 
# the eigenvector centrality of nodes in a network
node_eigenvector(ison_brandes)
```

```{r micent-solution}
node_betweenness(ison_brandes)
node_closeness(ison_brandes)
node_eigenvector(ison_brandes)
```

What is returned here are vectors of betweenness, closeness, and eigenvector 
scores for the nodes in the network.
But what do they mean?

```{r closeinterpQ, purl = FALSE}
question("In what way is a node with the smallest sum of geodesic distances to all other nodes 'central'?",
         answer("They have more power than other nodes",
                message = "Not necessarily"),
         answer("They can distribute a message to all other nodes in the network most quickly",
                correct = TRUE, message = learnr::random_praise()),
         answer("They would be located in the centre of the graph",
                message = "Not necessarily"),
         answer("They would be able to control information flow in the network",
                message = "Not necessarily"),
         allow_retry = TRUE,
         random_answer_order = TRUE)
```

```{r betinterpQ, purl = FALSE}
question("Why would a node lying 'between' many other nodes be 'central'?",
         answer("They have more power than other nodes",
                message = "Not necessarily"),
         answer("They would be able to control information flow in the network",
                correct = TRUE, message = learnr::random_praise()),
         answer("They would be located in the centre of the graph",
                message = "Not necessarily"),
         answer("They can distribute a message to all other nodes in the network most quickly",
                message = "Not necessarily"),
         allow_retry = TRUE,
         random_answer_order = TRUE)
```

```{r eiginterpQ, purl = FALSE}
question("Which of the following is true of eigenvector centrality?",
         answer("It is the only centrality measure that is correlated with power",
                message = "Not necessarily"),
         answer("It is a measure of influence, not activity",
                correct = TRUE, message = learnr::random_praise()),
         answer("It always gives the highest score to the node with the highest degree",
                message = "Not necessarily"),
         answer("It is only meaningful in networks with nodes of similar degree",
                message = "Not necessarily"),
         allow_retry = TRUE,
         random_answer_order = TRUE)
```

Try to answer the following questions for yourself:

- what does Bonacich mean when he says that `r gloss("power")` and influence are not the same thing?
- can you think of a real-world example when an actor might be central but not powerful, or powerful but not central?

Note that all centrality measures in `{manynet}` return normalized
scores by default -- 
for the raw scores, include `normalized = FALSE` in the function as an extra argument.

Now, can you create degree distributions for each of these?

```{r otherdist, exercise=TRUE, purl=FALSE}

```

```{r otherdist-solution}
plot(node_betweenness(ison_brandes))
plot(node_closeness(ison_brandes))
plot(node_eigenvector(ison_brandes))
```

## Plotting centrality

It is straightforward in `{autograph}` to highlight nodes and ties 
with maximum or minimum (e.g. degree) scores.
If the vector is numeric (i.e. a "measure"),
then this can be easily converted into a logical vector that
identifies the node/tie with the maximum/minimum score using
e.g. `node_is_max()` or `tie_is_min()`.
By passing this attribute to the `graphr()` argument "node_color"
we can highlight which node or nodes hold the maximum score in red.

```{r ggid, exercise = TRUE, exercise.setup = "addingnames", purl = FALSE}

```

```{r ggid-solution}
# plot the network, highlighting the node with the highest centrality score with a different colour
ison_brandes %>%
  mutate_nodes(color = node_is_max(node_degree())) %>%
  graphr(node_color = "color")

ison_brandes %>%
  mutate_nodes(color = node_is_max(node_betweenness())) %>%
  graphr(node_color = "color")

ison_brandes %>%
  mutate_nodes(color = node_is_max(node_closeness())) %>%
  graphr(node_color = "color")

ison_brandes %>%
  mutate_nodes(color = node_is_max(node_eigenvector())) %>%
  graphr(node_color = "color")
```

How neat! Try it with the two-mode version.
What can you see?

```{r ggid_twomode, exercise = TRUE, purl = FALSE}
# Instead of "ison_brandes", use "ison_brandes2"

```

```{r ggid_twomode-solution}
ison_brandes2 %>%
  add_node_attribute("color", node_is_max(node_degree(ison_brandes2))) %>%
  graphr(node_color = "color", layout = "bipartite")

ison_brandes2 %>%
  add_node_attribute("color", node_is_max(node_betweenness(ison_brandes2))) %>%
  graphr(node_color = "color", layout = "bipartite")

ison_brandes2 %>%
  add_node_attribute("color", node_is_max(node_closeness(ison_brandes2))) %>%
  graphr(node_color = "color", layout = "bipartite")

ison_brandes2 %>%
  add_node_attribute("color", node_is_max(node_eigenvector(ison_brandes2))) %>%
  graphr(node_color = "color", layout = "bipartite")
```

```{r brandes2quiz, purl = FALSE}
question("Select all that are true for the two-mode Brandes network.",
         answer("Only one node is selected in each plot."),
         answer("The maximum degree triangle has a higher degree than the maximum degree circle(s).",
                correct = TRUE),
         answer("No node is ever the most central according to two or more different centrality measures."),
         allow_retry = TRUE,
         random_answer_order = TRUE)
```

## Centralization

<img src="https://media.makeameme.org/created/centralization-here-i.jpg"/>

`{manynet}` also implements network `r gloss("centralization")` functions.
Here we are no longer interested in the level of the node,
but in the level of the whole network, 
so the syntax replaces `node_` with `net_`:

```{r centzn, exercise = TRUE, exercise.setup = "addingnames", purl = FALSE}

```

```{r centzn-solution}
net_degree(ison_brandes)
net_betweenness(ison_brandes)
net_closeness(ison_brandes)
print(net_eigenvector(ison_brandes), digits = 5)
```

By default, scores are printed up to 3 decimal places,
but this can be modified and, in any case, 
the unrounded values are retained internally.
This means that even if rounded values are printed (to respect console space),
as much precision as is available is used in further calculations.

Note that for centralization in two-mode networks,
two values are given (as a named vector),
since normalization typically depends on the (asymmetric)
number of nodes in each mode.

What if we want to have a single image/figure with multiple plots?
This can be a little tricky with gg-based plots,
but fortunately the `{patchwork}` package is here to help.

```{r multiplot, exercise = TRUE, exercise.setup = "addingnames", purl = FALSE, fig.width=9}

```

```{r multiplot-solution}
ison_brandes <- ison_brandes %>%
  add_node_attribute("degree", node_is_max(node_degree(ison_brandes))) %>%
  add_node_attribute("betweenness", node_is_max(node_betweenness(ison_brandes))) %>%
  add_node_attribute("closeness", node_is_max(node_closeness(ison_brandes))) %>%
  add_node_attribute("eigenvector", node_is_max(node_eigenvector(ison_brandes)))
gd <- graphr(ison_brandes, node_color = "degree") + 
  ggtitle("Degree", subtitle = round(net_degree(ison_brandes), 2))
gc <- graphr(ison_brandes, node_color = "closeness") + 
  ggtitle("Closeness", subtitle = round(net_closeness(ison_brandes), 2))
gb <- graphr(ison_brandes, node_color = "betweenness") + 
  ggtitle("Betweenness", subtitle = round(net_betweenness(ison_brandes), 2))
ge <- graphr(ison_brandes, node_color = "eigenvector") + 
  ggtitle("Eigenvector", subtitle = round(net_eigenvector(ison_brandes), 2))
(gd | gb) / (gc | ge)
# ggsave("brandes-centralities.pdf")
```


```{r centzdq, purl = FALSE}
question("How centralized is the ison_brandes network? Select all that apply.",
         answer("It is more degree centralised than betweenness centralised.",
                message = "Degree centralisation is at 0.18 for this network whereas betweenness centralisation is at 0.32. In other words, the network is better characterised as having 1 or 2 nodes lying on the shortest paths between others than one where 1 or 2 nodes have many more ties than the others."),
         answer("It is more closeness centralised than betweenness centralised.",
                message = "Closeness centralisation is at 0.23 for this network whereas betweenness centralisation is at 0.32. In other words, the network is better characterised as having 1 or 2 nodes lying on the shortest paths between others than one where 1 or 2 nodes can reach or access most other nodes."),
         answer("It is more eigenvector centralised than betweenness centralised.",
                correct = TRUE,
                message = "That's right, eigenvector centralisation is at 0.48 for this network whereas betweenness centralisation is at 0.32. In other words, the network is better characterised as having a core (or cores) of well-connected nodes rather than a wide network with only 1 or 2 nodes lying on the shortest paths between others."),
         random_answer_order = TRUE, 
         allow_retry = TRUE)
```

```{r centvcent, echo=FALSE, purl = FALSE}
question("What is the difference between centrality and centralisation according to the literature?",
  answer("Centrality is for nodes and centralisation is for networks",
         correct = TRUE),
  answer("Centrality is a state and centralisation is a process"),
  answer("Centrality is a '-ity' and centralisation is a '-isation'"),
  answer("Centrality is to centralisation what polarity is to polarisation"),
  allow_retry = TRUE,
  random_answer_order = TRUE
)
```

## Free play

<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExejh4dXBha3JhNW0yenJwMDd5aGpwd3Brc201ZXoxZnY2aWFna3Z4cCZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/7ROgX60TtTVK0mZwkJ/giphy.gif" height="400px"/>

Choose another dataset included in `{manynet}`.
Name a plausible research question you could ask of the dataset relating to each 
of the four main centrality measures (degree, betweenness, closeness, eigenvector).
Plot the network with nodes sized by each centrality measure,
using titles or subtitles to record the question and/or centralization measure.

```{r freeplayend, exercise=TRUE, purl=FALSE, fig.width=9}

```

## Glossary

Here are some of the terms that we have covered in this module:

`r print_glossary()`

---
title: "Topology and Resilience"
author: "by James Hollway"
output:
  learnr::tutorial:
    theme: flatly
runtime: shiny_prerendered
description: >
  This tutorial covers how to create deterministic networks,
  generate probabilistic networks, identify core-periphery structures,
  measure hierarchy, and assess network resilience.
---

```{r pkgs, include = FALSE, eval=TRUE}
library(netrics)
library(autograph)
library(patchwork)
```

```{r setup, include = FALSE, purl=FALSE, eval=TRUE}
library(learnr)
knitr::opts_chunk$set(echo = FALSE)
library(netrics)
library(autograph)
stocnet_theme("default")
clear_glossary()
learnr::random_phrases_add(language = "fr",
                           praise = c("C'est génial!",
                                      "Beau travail",
                                      "Excellent travail!",
                                      "Bravo!",
                                      "Super!",
                                      "Bien fait",
                                      "Bien joué",
                                      "Tu l'as fait!",
                                      "Je savais que tu pouvais le faire.",
                                      "Ça a l'air facile!",
                                      "C'était un travail de première classe.",
                                      "C'est ce que j'appelle un bon travail!"),
                          encouragement = c("Bon effort",
                                            "Vous l'avez presque maîtrisé!",
                                            "Ça avance bien.",
                                            "Continuez comme ça.",
                                            "Continuez à travailler dur!",
                                            "Vous apprenez vite!",
                                            "Vous faites un excellent travail aujourd'hui."))
learnr::random_phrases_add(language = "en",
                           praise = c("That's brilliant!",
                                      "Great work!",
                                      "Well done!",
                                      "Awesome!"),
                           encouragement = c("Good effort"))
```

```{r detect-language, context="server-start", purl=FALSE}
sys_locale <- Sys.getlocale("LC_CTYPE")
if (grepl("fr", sys_locale, ignore.case = TRUE)) {
  options(tutorial.language = "fr")
} else {
  options(tutorial.language = "en")
}
```

<style>
html { font-size: 115%; }
.callout {
  border-left: 3px solid #5B9C78;
  padding: 0.1em 1em;
  margin: 1.2em 0;
}
.callout-label { color: #5B9C78; }
.page-toc { font-style: italic; }
.page-toc-link { color: #5B9C78; cursor: pointer; text-decoration: underline; }
.tutorialTitle { color: #5B9C78 !important; }
#tutorial-topic .nav-link { color: #5B9C78 !important; }
.section a:not(.btn) { color: #5B9C78; }
.section a:not(.btn):hover { color: #487E61; }
.btn-tutorial-run,
.btn-tutorial-run:hover,
.btn-tutorial-run:focus,
.btn-tutorial-run:active {
  background-color: #5B9C78;
  border-color: #5B9C78;
  color: #fff;
}
</style>

## Today's target

So far the `{netrics}` tutorials have measured networks from the inside out:
which `r gloss("nodes","node")` are central, which cluster into communities,
and which share positions.
This tutorial turns the telescope around and asks about the _shape_ of the
whole `r gloss("network")` — its topology.
We will _create_ ideal-typical networks by deterministic rule
(stars, trees, lattices, and rings),
_generate_ others by probabilistic recipe
(random, small-world, and scale-free networks),
and then use these ideals as yardsticks for empirical questions:
Does this network have a core and a periphery?
How hierarchical is it?
And how many nodes or `r gloss("ties","tie")` could it lose before falling apart?

<img src="https://64.media.tumblr.com/tumblr_lvpgquhGbq1r54h4xo1_400.gif"/>

These ideal networks exaggerate centrality, clustering, and randomness features,
and are thus great for theory-building and investigating
the relationship between rules and structure.

::: {.callout}
<span class="callout-label">**Catching up**:</span>
This tutorial assumes you can already load, manipulate, and draw a network in R,
and that you have met the earlier `{netrics}` tutorials.
If any of that is hazy, work through the earlier `{stocnet}` tutorials first:
run `run_tute("Making")` and `run_tute("Manipulating")` for network data,
`run_tute("Visualising")` for graphing,
and `run_tute("Centrality")`, `run_tute("Community")`, and
`run_tute("Position")` for measuring networks,
or read their static versions on the
[manynet](https://stocnet.github.io/manynet/),
[autograph](https://stocnet.github.io/autograph/), and
[netrics](https://stocnet.github.io/netrics/) websites.
:::

::: {.callout}
<span class="callout-label">**New to network vocabulary?**:</span>
Throughout this tutorial, key terms are italicised:
hover over them for a definition,
and a full glossary of the terms used appears at the end of the tutorial.
:::

### Aims

By the end of this tutorial, you should be able to:

- [ ] &nbsp; Create deterministic network topologies with `create_*()`: empty, complete, star, tree, lattice, and ring networks
- [ ] &nbsp; Generate probabilistic networks with `generate_*()`: random, small-world, and scale-free networks
- [ ] &nbsp; Measure and interpret a network's small-world coefficient and power-law exponent
- [ ] &nbsp; Partition a network into core and periphery with `node_is_core()`, and judge the fit with `net_by_core()`
- [ ] &nbsp; Peel a network into _k_-cores with `node_by_kcoreness()`
- [ ] &nbsp; Measure how hierarchical a directed network is with `net_x_hierarchy()`
- [ ] &nbsp; Assess a network's resilience via its connectedness, cohesion, and adhesion, and locate its cutpoints and bridges

**Choose your own data**: The first half of this tutorial creates and generates
its own toy networks, and the worked examples later use `ison_lawfirm`,
`ison_adolescents`, and other datasets bundled with `{manynet}`.
But wherever there is an exercise box, you are encouraged to swap in
a network that interests _you_.
Remember the three flavours of bundled data as a rough difficulty ladder —
**Classic** (`ison_*`, small & tidy),
**Fiction** (`fict_*`, mid-sized & fun),
**Real-world** (`irps_*`, larger & realistic) —
and that you can browse the full list with `table_data()`.

## Creating networks

<span class="page-toc">On this page:
<span class="page-toc-link" onclick="document.getElementById('section-empty-and-complete').scrollIntoView({behavior:'auto',block:'start'});">Empty & complete</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-stars').scrollIntoView({behavior:'auto',block:'start'});">Stars</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-trees').scrollIntoView({behavior:'auto',block:'start'});">Trees</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-lattices').scrollIntoView({behavior:'auto',block:'start'});">Lattices</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-rings').scrollIntoView({behavior:'auto',block:'start'});">Rings</span></span>

<img src="https://media.tenor.com/images/e26e64a8cce94a6ee491f0ddeecc8b88/tenor.gif"/>

In this practical, we're going to create/generate
a number of ideal-typical network topologies and plot them.
We'll first look at some deterministic algorithms for _creating_ networks
of different structures,
and then look at how the introduction of some randomness can
_generate_ a variety of network structures.

::: {.callout}
<span class="callout-label">**Beginner note**:</span>
All the `create_*()` functions take the number of nodes to create
as their first argument.
You can even pass them an existing network instead,
in which case they will create a network of the same dimensions —
handy for comparing an empirical network with its ideal-typical counterpart.
Many also accept a `directed = TRUE` argument.
:::

### Empty and complete {#empty-and-complete}

To begin with, let's create a few '`r gloss("empty")`' and
full/'`r gloss("complete")`' graphs.
You will want to use some of the `create_*()` group of functions from `{manynet}`,
because they create graphs following some strict rule(s).
The two functions you will want to use here are `create_empty()` and `create_filled()`.
`create_empty()` creates an empty graph with the given number of nodes,
in this case 50 nodes.
For `create_filled()` we're creating a full graph,
where all of the nodes are connected to all of the other nodes.

Let's say that we want to explore networks of fifty nodes in this script.
**Graph one empty and one complete network with 50 nodes each,
give them an informative title, and plot the graphs together.
What would a complete network with half the nodes look like?
Add that too.**

```{r empty, exercise=TRUE, purl = FALSE}

```

```{r empty-solution}
(graphr(create_empty(50), "circle") + ggtitle("Empty graph"))
(graphr(create_filled(50)) + ggtitle("Complete graph"))
(graphr(create_filled(50/2)) + ggtitle("Complete graph (smaller)"))
```

These two extremes bookend every topology to come:
an empty network has a `r gloss("density")` of 0
and a complete network a density of 1,
and every real network lies somewhere in between —
usually much closer to the empty end.

### Stars {#stars}

In a `r gloss("star", "star")` network, there is one node to which all other nodes are connected,
otherwise known as the `r gloss("universal", "universal")` node.
There is no transitivity.
The maximum path length is two.
And `r gloss("degree", "degree")` centrality is maximised!^[In fact, all centrality measures are maximised, as one node acts as the sole bridge connecting one part of the network to the other.]

**Use the `create_star()` function to graph three star networks:**

- an undirected star network
- a out-directed star network
- and an in-directed star network

```{r star, exercise = TRUE, purl = FALSE}

```

```{r star-solution}
(graphr(create_star(50)) + ggtitle("Star graph"))
(graphr(create_star(50, directed = TRUE)) + ggtitle("Star out"))
(graphr(to_redirected(create_star(50, directed = TRUE))) + ggtitle("Star in"))
```

Substantively, a star is what maximal centralisation looks like:
everything depends on the hub.
That makes stars very efficient at spreading anything from the centre,
but also maximally fragile — remove the universal node and
the network shatters into isolates.

### Trees {#trees}

Trees, or regular trees, are networks with branching nodes.
They can be directed or undirected, and tend to indicate strong hierarchy:
each node reports up to a single parent,
branching down from a root to the `r gloss("pendant","pendant")` 'leaf' nodes
at the bottom.
**Again graph three networks:**

- one undirected with 2 branches per node
- a directed network with 2 branches per node
- the same as above, but graphed using the "tree" layout

```{r tree, exercise = TRUE, purl = FALSE}

```

```{r tree-solution}
# width argument specifies the breadth of the branches
(graphr(create_tree(50, width = 2)) + ggtitle("Tree graph"))
(graphr(create_tree(50, width = 2, directed = TRUE)) + ggtitle("Tree out"))
(graphr(create_tree(50, width = 2, directed = TRUE), "tree") + ggtitle("Tree layout"))
```

Try varying the `width` argument to see the result.
We will return to trees when we measure hierarchy later in this tutorial.

### Lattices {#lattices}

`r gloss("Lattices","lattice")` reflect highly clustered networks
where there is a high likelihood that interaction partners also interact.
They are used to show how clustering facilitates or limits diffusion
or makes pockets of behaviour stable.

```{r lat-qa, echo=FALSE, purl = FALSE}
question("Why are lattices considered highly clustered?",
         answer("Because neighbours are likely also neighbours of each other",
                message = learnr::random_praise(),
                correct = TRUE),
         answer("Because all nodes are directly connected to each other",
                message = learnr::random_encouragement()),
         answer("Because there is a single component",
                message = learnr::random_encouragement()),
         answer("Because there is a single community",
                message = learnr::random_encouragement()),
         random_answer_order = TRUE,
         allow_retry = TRUE)
```

Note that `create_lattice()` in `{manynet}` works a little differently
to how it works in `{igraph}`.
In `{igraph}` the number or vector passed to the function indicates
the length of each dimension.
So `c(50)` would be a one-dimensional lattice,
essentially a chain of 50 nodes connected to their neighbours.
`c(50,50)` would be a two-dimensional lattice,
of 50 nodes long and 50 nodes wide.
`c(50,50,50)` would be a three-dimensional lattice,
of 50 nodes long, 50 nodes wide, and 50 nodes deep, etc.

_But_ this doesn't help us when we want to see what a lattice representation
with the same order (number of nodes) as a given network would be.
For example, perhaps we just want to know what a lattice with 50 nodes
would look like.
So `{manynet}` instead tries to find the most even or balanced
two-dimensional representation with a given number of nodes.

**Graph two lattices, one with 50 nodes,
and another with half the number of nodes.**

```{r lattices, exercise = TRUE, purl = FALSE}

```

```{r lattices-solution}
(graphr(create_lattice(50)) + ggtitle("One-mode lattice graph"))
(graphr(create_lattice(50/2)) + ggtitle("Smaller lattice graph"))
```

We can put numbers on what makes a lattice special.
Its density is low, but its `r gloss("transitivity","transitivity")` —
the tendency for two nodes with a shared neighbour to be tied themselves —
is high.
**Measure both for a 50-node lattice.**

```{r latmeasures, exercise = TRUE}
net_by_density(create_lattice(50))
net_by_transitivity(create_lattice(50))
```

So with only around an eighth of all possible ties present,
nearly half of all open two-paths are closed into triangles:
lots of local clustering despite few ties overall.

### Rings {#rings}

This creates a graph where each node has two separate neighbours
which creates a ring graph.
**Graph three ring networks:**

- one with 50 nodes
- one with 50 nodes where they are connected to neighbours two steps away,
  on a "circle" layout
- the same as above, but on a "stress" layout

```{r rings, exercise = TRUE, purl = FALSE}

```

```{r rings-solution}
(graphr(create_ring(50)) + ggtitle("Ring graph", subtitle = "Starring Naomi Watts"))
# width argument specifies the width of the ring
(graphr(create_ring(50, width = 2), "circle") + ggtitle("The Ring Two", subtitle = "No different?"))
(graphr(create_ring(50, width = 2), "stress") + ggtitle("The Ring Two v2.0"))
```

The price a ring pays for its regularity is distance.
The `r gloss("diameter")` of a network is the length of its longest
shortest path (`r gloss("geodesic")`),
and `net_by_length()` returns the _average_ shortest path length.
**Check how far apart nodes in a ring really are,
and what widening the ring does about it.**

```{r ringmeasures, exercise = TRUE}
net_by_diameter(create_ring(50))
net_by_length(create_ring(50))
net_by_diameter(create_ring(50, width = 2))
```

In a 50-node ring, a message travelling neighbour-to-neighbour
needs 25 steps to reach the far side of the circle —
and about 13 on average to reach anyone.
Doubling the width halves these distances, but they remain long:
in rings and lattices, distances grow with the number of nodes.
Hold that thought for the small-world networks on the next page.

::: {.callout}
<span class="callout-label">**In brief**:</span> The `create_*()` functions build deterministic
topologies from rules: `create_empty()` and `create_filled()` set the two
density extremes, `create_star()` maximises centralisation around one
universal node, `create_tree()` branches like a hierarchy,
`create_lattice()` clusters neighbours into grids, and `create_ring()`
chains them into circles. `net_by_density()`, `net_by_transitivity()`,
`net_by_diameter()`, and `net_by_length()` put numbers on the resulting
shapes.
:::

## Generating networks

<span class="page-toc">On this page:
<span class="page-toc-link" onclick="document.getElementById('section-random-graphs').scrollIntoView({behavior:'auto',block:'start'});">Random</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-small-world-graphs').scrollIntoView({behavior:'auto',block:'start'});">Small-world</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-scale-free-graphs').scrollIntoView({behavior:'auto',block:'start'});">Scale-free</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-degree-mixing').scrollIntoView({behavior:'auto',block:'start'});">Degree mixing</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-more-generators').scrollIntoView({behavior:'auto',block:'start'});">More generators</span></span>

Next we are going to take a look at some probabilistic graphs.
These involve some `r gloss("random")` element, perhaps in addition to specific rules,
to stochastically 'generate' networks of certain types of topologies.
As such, we'll be using the `generate_*()` group of functions from `{manynet}`.

::: {.callout}
<span class="callout-label">**Beginner note**:</span>
Because these functions are stochastic,
you will get a slightly different network every time you run them.
That is the point!
If you need the same network twice — say, to reproduce a figure —
call `set.seed()` with some number of your choice first.
:::

### Random graphs {#random-graphs}

An Erdös-Renyi graph is simply a random graph.
You will need to specify the probability of a tie
in addition to the number of nodes.
An Erdos-Renyi graph on the vertex set $V$ is a random graph
which connects each pair of nodes ${i,j}$ with probability $p$, independent.
Note that for a `r gloss("sparse","sparse")` ER graph, $p$ must decrease as $N$ goes up.
**Generate three random networks of 50 nodes and a density of 0.08.**

```{r random, exercise = TRUE, purl = FALSE}

```

```{r random-solution}
(graphr(generate_random(50, 0.08)) + ggtitle("Random 1 graph"))
(graphr(generate_random(50, 0.08)) + ggtitle("Random 2 graph"))
(graphr(generate_random(50, 0.08)) + ggtitle("Random 3 graph"))
```

Keep going if you like... it will be a little different every time.
Note that you can also pass the second argument an integer,
in which case the function will interpret that as the number of ties/edges rather than the probability that a tie is present.
**Try generating a random graph with 200 edges/ties now.**

```{r randomno, exercise = TRUE, purl = FALSE}

```

```{r randomno-solution}
(erdren4 <- graphr(generate_random(50, 200)) + ggtitle("Random 1 graph"))
```

Random graphs are the mirror image of the lattice:
their paths are short, but they have next to no clustering —
ties are sprinkled independently,
so triangles arise only by coincidence.
They serve as the customary _null model_ in network analysis:
the baseline of what a network with no structuring principle at all
would look like.

### Small-world graphs {#small-world-graphs}

Remember the ring graph from above?
What if we rewire (change) some of the edges at a certain probability?
This is how `r gloss("small-world","smallworld")` networks are generated.
**Graph three small-world networks, all with 50 nodes and a rewiring probability of 0.025.**

```{r smallw, exercise = TRUE, purl = FALSE}

```

```{r smallw-solution}
(graphr(generate_smallworld(50, 0.025)) + ggtitle("Smallworld 1 graph"))
(graphr(generate_smallworld(50, 0.025)) + ggtitle("Smallworld 2 graph"))
(graphr(generate_smallworld(50, 0.025)) + ggtitle("Smallworld 3 graph"))
```

With on average 2.5 ties randomly rewired, does the structure look different?
This is a small-world network, where clustering/transitivity remains high
but path lengths are much lower than they would otherwise be.
Remember that in a small-world network, the shortest-path distance between nodes
increases sufficiently slowly as a function of the number of nodes in the network.
You can also call these networks a Watts–Strogatz toy network.
If you want to review this, go back to the reading by Watts (2004).
**Check both properties for a generated small-world network.**

```{r swcompare, exercise = TRUE}
sw <- generate_smallworld(50, 0.025)
net_by_transitivity(sw)
net_by_length(sw)
```

Compare these against the pages before:
transitivity close to the lattice's (and far above a random graph's),
yet an average path length a fraction of the ring's.
A handful of rewired ties act as shortcuts spanning the circle,
which is why "six degrees of separation" can hold
even in enormous, highly clustered social networks.

There is also such a thing as a network's small-world coefficient.
See the help page for more details,
but with the default equation ('omega'),
the coefficient typically ranges between 0 and 1,
where 1 is as close to a small-world as possible.
**Try it now on a small-world generated network,
but with a rewiring probability of 0.25.**

```{r smallwtest, exercise = TRUE, purl = FALSE}

```

```{r smallwtest-solution}
net_by_smallworld(generate_smallworld(50, 0.25))
```

Substantively, a coefficient near 1 says the network offers
the best of both worlds:
neighbourhoods cohesive enough to sustain trust and local norms,
yet paths short enough for information or contagion
to traverse the whole network in a few steps.
A coefficient near 0 means the network behaves more like
a pure lattice (clustered but slow) or
a pure random graph (fast but unclustered).

### Scale-free graphs {#scale-free-graphs}

There is another famous model in network science: the `r gloss("scale-free","scalefree")` model.
Remember:
"In many real-world networks, the distribution of the number of network neighbours
the degree `r gloss("distribution","distribution")` is typically right-skewed with a "heavy tail".
A majority of the nodes have less-than-average degree and
a small fraction of `r gloss("hubs","hub")` are many times better connected than average (2004, p. 250).

The following generates a scale-free graph according to the Barabasi-Albert (BA) model
that rests upon the mechanism of preferential attachment.
More on this in the Watts paper (2005, p.51) and Merton (1968).
The BA model rests on two mechanisms:
population growth and preferential attachment.
Population growth: real networks grow in time as new members join the population.
Preferential/cumulative attachment means that newly arriving nodes will tend to
connect to already well-connected nodes rather than poorly connected ones.

**Generate and graph three scale-free networks,
with alpha parameters of 0.5, 1, and 1.5.**

```{r scalef, exercise = TRUE, purl = FALSE}

```

```{r scalef-solution}
(graphr(generate_scalefree(50, 0.5)) +
    ggtitle("Scalefree 1 graph", subtitle = "Power = .5"))
(graphr(generate_scalefree(50, 1)) +
    ggtitle("Scalefree 2 graph", subtitle = "Power = 1"))
(graphr(generate_scalefree(50, 1.5)) +
    ggtitle("Scalefree 3 graph", subtitle = "Power = 1.5"))
```

You can also measure the degree to which a network has a degree distribution that fits
a power-law distribution.
With an alpha/power-law exponent between 2 and 3,
one generally cannot reject the hypothesis that the observed data
comes from a power-law distribution.
**Fit the power-law exponent to a scale-free network generated with an alpha of 2.**

```{r scaleftest, exercise = TRUE, purl = FALSE}

```

```{r scaleftest-solution}
net_by_scalefree(generate_scalefree(50, 2))
```

Why does this matter?
A power-law degree distribution means the network is dominated by a few hubs,
and such networks behave distinctively:
they are remarkably robust to _random_ failure
(a random node is almost surely peripheral),
yet fragile to _targeted_ attack on their hubs —
a theme we return to on the Resilience page.

::: {.callout}
<span class="callout-label">**Going further**:</span>
The exponent reported by `net_by_scalefree()` is fitted to the empirical
degree distribution, so it is most informative for networks large enough to
have a meaningful tail. For a visual check, `plot()` the network's degree
distribution (see the Centrality tutorial) and look for the heavy right tail.
:::

### Degree mixing {#degree-mixing}

Once a network has hubs, two further topological questions arise about how
degrees are arranged with respect to one another.
_Do the hubs stick together?_ — the **rich-club** coefficient
(`net_by_richclub()`) asks whether high-degree nodes are more densely tied to
_each other_ than their numbers alone would predict.
And _do similar nodes attach to similar nodes?_ — degree **assortativity**
(`net_by_assortativity()`) correlates the degrees at the two ends of each tie:
positive means high-degree nodes pair with high-degree nodes (assortative),
negative means hubs mostly attach to the periphery (disassortative).
Both are purely structural, needing no node attributes.
**Contrast a scale-free network with a random one.**

```{r degmix, exercise = TRUE}
set.seed(123)
sf <- generate_scalefree(50, 2)
net_by_richclub(sf)          # hubs interconnected?
net_by_assortativity(sf)     # do like degrees attach?
net_by_assortativity(generate_random(50, 0.08))
```

The scale-free network is strongly _disassortative_ (a negative score):
its preferential-attachment hubs are joined mostly to the many low-degree
nodes, not to each other — the signature of technological and biological
networks. Many _social_ networks, by contrast, are mildly assortative, as
popular people tend to know one another. A random graph sits near zero,
having no degree-mixing tendency at all.

### More generators {#more-generators}

Beyond the three classics above, `{manynet}` offers a few further generative
models worth knowing about, each motivated by how real networks _grow_:

- `generate_fire()` implements the **forest-fire** model, in which each
arriving node "burns" outward from a random contact along existing ties,
producing the heavy tails, high clustering, and shrinking diameters seen in
many evolving networks (Leskovec et al. 2007).
- `generate_islands()` stitches together several dense **islands** (à la
communities) joined by a few `bridges` — handy for pairing this tutorial's
topology measures with the community structure of the earlier tutorial.
- `generate_citations()` grows a **citation** network in which new nodes cite
existing ones with a recency bias, the classic model of directed,
(near-)acyclic reference networks.

**Generate and glance at all three.**

```{r moregen, exercise = TRUE, fig.width=9}
set.seed(123)
graphr(generate_fire(50)) + ggtitle("Forest fire")
graphr(generate_islands(50, islands = 3, bridges = 2)) + ggtitle("Islands")
graphr(generate_citations(50)) + ggtitle("Citations")
```

Each model leaves its own visual fingerprint:
the forest-fire network grows a few dominant hubs (like a scale-free network,
but with more clustering around them),
the islands network shows several dense blobs joined by a handful of lone
bridging ties, and the citation network fans out as a directed, tree-like
cascade from the earliest nodes.
These are exactly the ideal patterns you have learned to _measure_ in this
tutorial — so you could, for instance, confirm the islands model with
`net_by_modularity()` (from the community tutorial) or the citation model's
one-directional flow with `net_by_reciprocity()`.

::: {.callout}
<span class="callout-label">**Going further**:</span>
These three are more advanced and we will not dwell on them in class.
Each takes further arguments controlling its growth
(e.g. `islands`, `bridges`, or the forest-fire burn probabilities);
see `?generate_fire`, `?generate_islands`, and `?generate_citations`,
and their underlying `igraph` samplers, for the details.
:::

::: {.callout}
<span class="callout-label">**In brief**:</span> The `generate_*()` functions add randomness:
`generate_random()` places ties by probability or count (Erdös-Renyi),
`generate_smallworld()` rewires a ring so paths shorten while clustering
survives (Watts–Strogatz), and `generate_scalefree()` grows hubs through
preferential attachment (Barabasi-Albert);
`generate_fire()`, `generate_islands()`, and `generate_citations()` add
forest-fire, modular, and citation growth models.
`net_by_richclub()` and `net_by_assortativity()` then describe how a
network's degrees mix — whether hubs cluster together, and whether like
attaches to like.
:::

## Core-Periphery

<span class="page-toc">On this page:
<span class="page-toc-link" onclick="document.getElementById('section-core-periphery-graphs').scrollIntoView({behavior:'auto',block:'start'});">Ideal graphs</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-core-periphery-assignment').scrollIntoView({behavior:'auto',block:'start'});">Assignment</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-coreness').scrollIntoView({behavior:'auto',block:'start'});">Coreness</span></span>

### Core-periphery graphs {#core-periphery-graphs}

Lastly, we'll take a look at some `r gloss("core-periphery","core")` graphs.
The most common definition of a core-periphery network
is one in which the network can be partitioned into two groups
such that one group of nodes (the core) has
dense interactions among themselves,
moderately dense interactions with the second group,
and the second group (the periphery) has
sparse interactions among themselves.

```{r corevcomm-qa, echo=FALSE, purl = FALSE}
question("Can a single network have both a community structure and a core-periphery structure?",
         answer("No", message = learnr::random_encouragement()),
         answer("Yes",
                message = "That's right. For example, the core and periphery might represent two or more communities.",
                correct = TRUE),
         random_answer_order = TRUE,
         allow_retry = TRUE)
```

We can visualise extreme versions of such a network
using the `create_core()` function.
**Graph a core-periphery network of 50 nodes**
(which, unless a core-periphery membership assignment is given,
will be split evenly between core and periphery partitions).

```{r core, exercise=TRUE, purl = FALSE, fig.width=9}

```

```{r core-solution}
(graphr(create_core(50)) + ggtitle("Core"))
```

Notice the tell-tale shape: a tight knot of mutually-connected core nodes in
the middle, a scatter of peripheral nodes around the edge that connect to the
core but rarely to each other. This is the ideal that the fitting measures
below compare real networks against.

### Core-periphery assignment {#core-periphery-assignment}

Let's consider identifying the core and peripheral nodes in a network.
Let's use the `ison_lawfirm` dataset from `{manynet}`.
This dataset involves relations between partners in a corporate law firm in New England.
First of all, **graph the data and see whether you can guess which nodes
might be part of the core and which are part of the periphery.
Colour the nodes by Gender, Office, Practice, and School.**
Any you might think correlate with core status?

```{r gnet, exercise=TRUE, fig.width=9}
lawfirm <- ison_lawfirm |> to_uniplex("friends") |> to_undirected()
```

```{r gnet-solution}
lawfirm <- ison_lawfirm |> to_uniplex("friends") |> to_undirected()
graphr(lawfirm, node_color = "school", edge_color = "darkgray")
graphr(lawfirm, node_color = "gender", edge_color = "darkgray")
graphr(lawfirm, node_color = "office", edge_color = "darkgray")
graphr(lawfirm, node_color = "practice", edge_color = "darkgray")
```

Next, let's assign nodes to the core and periphery blocks
using the `node_is_core()` function.
It works pretty straightforwardly.
By default it runs down the rank order of nodes by their degree,
at each step working out whether including the next highest degree node
in the core will maximise the core-periphery structure of the network.
**Assign core/periphery membership and graph the network coloured by it.**

```{r nodecore, exercise=TRUE, purl = FALSE, fig.width=9, exercise.setup="gnet"}

```

```{r nodecore-solution}
lawfirm |>
  mutate_nodes(nc = node_is_core()) |>
  graphr(node_color = "nc", edge_color = "gray")
```

This graph suggests that there is a core and a periphery.
There might even be two cores here,
one on the left and one on the right.

But is it really all that much of a core-periphery structure?
We can establish how correlated our network is compared to
a core-periphery model of the same dimension using `net_by_core()`.
A value close to 1 would mean the observed ties line up almost perfectly
with the ideal blockmodel — dense within the core, sparse within the periphery —
while a value near 0 means the partition fits no better than chance.
**Measure how well a core-periphery model fits this network with `net_by_core()`.**

```{r netcore, exercise=TRUE, purl = FALSE, exercise.setup="gnet"}

```

```{r netcore-solution}
net_by_core(lawfirm, node_is_core(lawfirm))
```

```{r corecorr-qa, echo=FALSE, purl = FALSE}
question("What can we say about this correlation.",
         answer("It is a perfect negative relationship",
                message = learnr::random_encouragement()),
         answer("It is fairly strong",
                message = learnr::random_encouragement()),
         answer("It is positive",
                message = learnr::random_encouragement()),
         answer("There is absolutely no correlation",
                message = learnr::random_encouragement()),
         answer("None of the above", correct = TRUE,
                message = learnr::random_praise()),
         allow_retry = TRUE
)
```

So despite what the eye suggested,
this friendship network is only weakly — indeed slightly _negatively_ —
characterised by a single core-periphery structure.
That is substantively interesting in itself:
the firm's friendships seem to cluster into groups
(remember the two suspected "cores")
rather than stratify into insiders and outsiders.

Note that `node_is_core()` also includes a method that descends through
the rank order of nodes' eigenvector centralities instead of degree centralities.
Why might that not be such a good choice here?

::: {.callout}
<span class="callout-label">**Going further**:</span>
A two-way split can be too blunt. `node_in_core()` classifies nodes into
_three_ blocks — "Core", "Semi-periphery", and "Periphery" — for when the
world-systems flavour of the concept fits better.
:::

Now let's see whether our core-periphery membership vector correlates with any of the three
categorical attributes we looked at before.
Since we're doing this on categorical variables, we'll use the Chi-squared test in base R.
**Take a look and see whether there is a statistically significant association
between gender and core (or periphery) status.**

```{r chisq, exercise=TRUE, purl=FALSE, exercise.setup="gnet"}
chisq.test(as.factor(node_is_core(lawfirm)),
           as.factor(node_attribute(lawfirm, "gender")))
```

```{r chisq-solution}
chisq.test(as.factor(node_is_core(lawfirm)),
           as.factor(node_attribute(lawfirm, "gender")))
chisq.test(as.factor(node_is_core(lawfirm)),
           as.factor(node_attribute(lawfirm, "office")))
chisq.test(as.factor(node_is_core(lawfirm)),
           as.factor(node_attribute(lawfirm, "school")))
chisq.test(as.factor(node_is_core(lawfirm)),
           as.factor(node_attribute(lawfirm, "practice")))
```

```{r chisq-qa, echo=FALSE, purl = FALSE}
question("There a statistically significant association between the core assignment and...",
         answer("gender.",
                message = learnr::random_encouragement()),
         answer("office.",
                message = learnr::random_encouragement()),
         answer("school.",
                message = learnr::random_encouragement()),
         answer("practice.",
                message = learnr::random_encouragement()),
         answer("none of the above variables.", correct = TRUE,
                message = "That's right. The p-value for office is close, but no cigar."),
         allow_retry = TRUE
)
```

### Coreness {#coreness}

An alternative route is to identify 'core' nodes
depending on their `r gloss("k-coreness","kcoreness")`.
In `{manynet}`, we can return nodes _k_-coreness
with `node_by_kcoreness()` instead of
the `node_is_core()` used for core-periphery.
**Run the code to colour the law-firm network by each node's _k_-coreness.**

```{r nodecoren, exercise=TRUE, exercise.setup="gnet"}
lawfirm |>
  mutate_nodes(ncn = node_by_kcoreness()) |>
  graphr(node_color = "ncn")
```

Where `node_is_core()` forces a yes/no answer,
_k_-coreness grades how deep each node sits in the network:
a node with coreness _k_ survives even after all nodes of degree
less than _k_ have been successively peeled away.
High-coreness nodes are thus embedded in a densely interlocked middle,
which matters for processes like diffusion —
what starts in a high _k_-core is far more likely to spread widely
than what starts among the peelable outer layers.

```{r dich-qa, echo=FALSE, purl = FALSE}
question("Which has more than two classes/groups.",
         answer("node_kcoreness()", correct = TRUE,
                message = learnr::random_praise()),
         answer("node_is_core()",
                message = learnr::random_encouragement()),
         random_answer_order = TRUE,
         allow_retry = TRUE
)
```

```{r ness-qa, echo=FALSE, purl = FALSE}
question("Select the correct definitions:",
         answer("The k-core of a network is a maximal subgraph in which each vertex has at least degree k.", correct = TRUE,
                message = learnr::random_praise()),
         answer("The coreness of a node is k if it belongs to the k-core but not to the (k+1)-core.", correct = TRUE,
                message = "Yes -- a node's coreness is the highest k for which it still sits inside the k-core, so it tells you how deep in the network's core the node lies."),
         answer("The coreness of a node is equal to its degree.",
                message = paste(learnr::random_encouragement(), "Coreness is usually less than or equal to degree: a node needs at least degree k to be in the k-core, but many high-degree nodes have lower coreness because their neighbours don't all qualify.")),
         random_answer_order = TRUE,
         allow_retry = TRUE
)
```

::: {.callout}
<span class="callout-label">**In brief**:</span> `create_core()` draws the ideal core-periphery
network; `node_is_core()` assigns each node to core or periphery, and
`net_by_core()` reports how well that bipartition actually fits the network
(1 = perfectly, ~0 = not at all). `node_by_kcoreness()` offers a graded
alternative, peeling the network into successively deeper _k_-cores.
:::

## Hierarchy

<span class="page-toc">On this page:
<span class="page-toc-link" onclick="document.getElementById('section-graph-theoretic-dimensions-of-hierarchy').scrollIntoView({behavior:'auto',block:'start'});">Dimensions</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-measuring-gtdh').scrollIntoView({behavior:'auto',block:'start'});">Measuring</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-comparing-gtdh').scrollIntoView({behavior:'auto',block:'start'});">Comparing</span></span>

What sometimes drives our interest in whether a network resembles a
core-periphery network is that it offers a generalisation of hierarchy:
the core are the rule-makers; the periphery are the rule-takers.
But where we have a directed network,
we may be able to measure hierarchy explicitly.

### Graph theoretic dimensions of hierarchy {#graph-theoretic-dimensions-of-hierarchy}

Recall that measuring hierarchy directly is difficult,
as the concept incorporates several different aspects.
Can you recall which of the following are aspects of
the graph theoretic dimensions of hierarchy?

```{r gtdh-qa, echo=FALSE, purl = FALSE}
question("Select the measures included in the graph theoretic dimensions of hierarchy:",
         answer("reciprocity", correct = TRUE,
                message = paste(learnr::random_praise(), "The hierarchy dimension is measured through (a lack of) reciprocity: in a perfect hierarchy relations run one way, never back.")),
         answer("connectedness", correct = TRUE,
                message = "Yes -- connectedness asks whether every pair of nodes is joined by some (undirected) path, the first of Krackhardt's four dimensions."),
         answer("efficiency", correct = TRUE,
                message = "Yes -- efficiency asks whether the structure uses no more ties than necessary (no redundant links beyond those needed to hold it together)."),
         answer("least upper bound", correct = TRUE,
                message = "Yes -- least upper bound asks whether every pair of nodes shares a common superior they both ultimately report up to."),
         answer("components", message = learnr::random_encouragement()),
         answer("degree", message = learnr::random_encouragement()),
         answer("density", message = learnr::random_encouragement()),
         random_answer_order = TRUE,
         allow_retry = TRUE
)
```

### Measuring GTDH {#measuring-gtdh}

Ok, so let's now take a closer look at how to investigate the degree of
hierarchy in a given network.
The classic example would be to look at a tree network,
like the one constructed earlier.
**Run the code to try the function `net_x_hierarchy()` on a directed tree of 11 nodes.**

```{r treeh, exercise = TRUE}
treeleven <- create_tree(11, directed = TRUE)
net_x_hierarchy(treeleven)
rowMeans(net_x_hierarchy(treeleven))
```

We see here four different measures of hierarchy:

- **connectedness**: the proportion of dyads that can reach each other
- **reciprocity**: the proportion of ties that are reciprocated (inverse) —
in a perfect hierarchy no `r gloss("reciprocity")` is expected,
since deference flows one way
- **efficiency**: the minimum indegrees required to connect the network over
the actual indegrees present
- **least upper bound**: all nodes in a hierarchy should have a node that can
reach both of them

Note that to get an overall score, we can take the average of these four measures.
A directed tree is the perfect hierarchy —
it scores 1 on all four dimensions —
which is exactly why organisational charts are drawn as trees.

::: {.callout}
<span class="callout-label">**Going further**:</span>
Each dimension can also be obtained on its own:
`net_by_connectedness()`, `net_by_reciprocity()`,
`net_by_efficiency()`, and `net_by_upperbound()`.
See `?measure_hierarchy` for definitions and the Krackhardt references.
:::

### Comparing GTDH {#comparing-gtdh}

Because some of these measures (reciprocity), only make sense for a directed
network, we'll switch to a couple of directed networks for this exercise.
Which one would you consider is more hierarchical:
`ison_emotions`, a directed network of emotional transitions,
or `fict_thrones`, a directed network of kinship ties?
**Graph each network, then add a `net_x_hierarchy()` call for both and compare their profiles.**

```{r hierarchy, exercise = TRUE}
graphr(ison_emotions)
graphr(fict_thrones)
```

```{r hierarchy-solution}
graphr(ison_emotions)
net_x_hierarchy(ison_emotions)
graphr(fict_thrones)
net_x_hierarchy(fict_thrones)
```

Actually, these two networks have the same average hierarchy score of around 0.525.
But they have quite different profiles.
Can you make sense of these results?

```{r gtdhcompare-qa, echo=FALSE, purl = FALSE}
question("Compared to the emotional transitions network, the Game of Thrones kinship network has:",
         answer("more nodes able to reach each other",
                message = learnr::random_encouragement()),
         answer("fewer reciprocated ties", correct = TRUE,
                message = learnr::random_praise()),
         answer("fewer redundant ties", correct = TRUE,
                message = "Yes -- that shows up as a higher efficiency score: the kinship structure carries fewer redundant ties than the emotional transitions network."),
         answer("at least one boss for all pairs",
                message = "Not here -- that would be the least-upper-bound property, but not every pair in the kinship network shares a common ancestor they both report up to."),
         random_answer_order = TRUE,
         allow_retry = TRUE
)
```

This is the payoff of a multidimensional measure:
a single "hierarchy score" would have called these two networks identical,
but their profiles show they are hierarchical in quite different _ways_ —
the emotions network is fully connected but saturated with reciprocated,
redundant ties, while the kinship network is sparser and more one-directional
but does not integrate everyone under common superiors.

::: {.callout}
<span class="callout-label">**In brief**:</span> `net_x_hierarchy()` returns Krackhardt's four
graph-theoretic dimensions of hierarchy — connectedness, (inverse)
reciprocity, efficiency, and least upper bound. Average them (e.g. with
`rowMeans()`) for an overall score, but read the whole profile to see _how_ a
network is hierarchical. A directed tree scores perfectly on all four.
:::

## Resilience

<span class="page-toc">On this page:
<span class="page-toc-link" onclick="document.getElementById('section-how-cohesive-is-the-network').scrollIntoView({behavior:'auto',block:'start'});">Cohesion</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-identifying-cutpoints').scrollIntoView({behavior:'auto',block:'start'});">Cutpoints</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-identifying-bridges').scrollIntoView({behavior:'auto',block:'start'});">Bridges</span></span>

### How cohesive is the network? {#how-cohesive-is-the-network}

When investigating a network's resilience,
we might think of whether the network will remain connected despite some nodes or ties dropping out.
Let's explore how resilient a (core) network of adolescents (`ison_adolescents`) might be.
First, we might be interested in whether the network is `r gloss("connected","connected")` at all.
**Check whether `ison_adolescents` is connected.**

```{r connected, exercise=TRUE, purl = FALSE}

```

```{r connected-solution}
net_by_connectedness(ison_adolescents)
```

This measure gets at the proportion of dyads that can reach each other in the network.
In this case, the proportion is 1, i.e. all nodes can reach every other node.
Another way to get at this would be to see how many `r gloss("components","component")` there are in the network.

```{r connect-qa, echo=FALSE, purl=FALSE}
question("But counting the number of components instead of connectedness can overemphasise:",
         answer("Isolates", correct = TRUE,
                message = learnr::random_praise()),
         answer("Small components", correct = TRUE,
                message = learnr::random_encouragement()),
         answer("Density",
                message = learnr::random_encouragement()),
         random_answer_order = TRUE,
         allow_retry = TRUE
)
```

A dropped tie can have severe consequences to the topology of a network
if it is a bridge, say.
The more ties you would need to remove to fragment the network,
the greater the `r gloss("adhesion","adhesion")` of the network.
But a dropped node can be even more consequential, as it will take any ties it has with it.
**Find out the `r gloss("cohesion","cohesion")`,
or how many dropped nodes it would take to (further) fragment the network.**

```{r cohesion, exercise=TRUE, purl = FALSE}

```

```{r cohesion-solution}
net_by_cohesion(ison_adolescents)
```

```{r cohesion-qa, echo=FALSE, purl = FALSE}
question("The result of this function represents...",
         answer("the minimum number of nodes necessary to remove from the network to increase the number of components.", correct = TRUE,
                message = learnr::random_praise()),
         answer("the number of strong components in the network.",
                message = learnr::random_encouragement()),
         answer("the minimum number of ties necessary to remove from the network to increase the number of components.",
                message = "This is actually the definition of `node_adhesion()`."),
         random_answer_order = TRUE,
         allow_retry = TRUE
)
```

```{r res-qa, echo=FALSE, purl = FALSE}
question("The higher the minimum number of nodes to remove...",
         answer("the more resilient is the network.", correct = TRUE,
                message = learnr::random_praise()),
         answer("the less resilient the network.",
                message = learnr::random_encouragement()),
         random_answer_order = TRUE,
         allow_retry = TRUE
)
```

A cohesion of 1, as here, is the most fragile a connected network can be:
there is at least one single node whose departure would break the network
apart. Recall the star and scale-free networks from earlier —
highly centralised topologies buy efficiency at exactly this price.

::: {.callout}
<span class="callout-label">**Going further**:</span>
Cohesion and adhesion report a minimum _count_, regardless of network size.
For measures that weigh the removals against the fragments they create,
see `net_by_toughness()` (for nodes) and `net_by_strength()` (for ties).
:::

### Identifying cutpoints {#identifying-cutpoints}

But which are these nodes? Is there more than one?
Nodes that endanger fragmentation of the network are called `r gloss("cutpoints","cutpoint")`.
**Find and use a function to identify which, if any, of the nodes in the `ison_adolescents`
network are cutpoints.**

```{r idcuts, exercise = TRUE, purl=FALSE}

```

```{r idcuts-solution}
node_is_cutpoint(ison_adolescents)
```

Ok, so this results in a vector identifying which nodes are cutpoints (TRUE) or not (FALSE).
Somewhat more useful though would be to highlight these nodes on the network.
**Can you add a node attribute that highlights which nodes are cutpoints?**

```{r closerlook, exercise = TRUE, purl=FALSE}

```

```{r closerlook-solution}
ison_adolescents |>
  mutate_nodes(cut = node_is_cutpoint()) |>
  graphr(node_color = "cut")
```

The highlighted node(s) are the network's single points of failure:
each sits astride the only path between two otherwise-separate parts of the
network, so its removal would split the adolescents into disconnected groups.
These are the nodes you would protect first if you cared about keeping the
network together.

### Identifying bridges {#identifying-bridges}

Let's do something similar now, but with respect to ties rather than nodes.
Here we are interested in identifying which ties are `r gloss("bridges","bridge")`.
**Report the network's adhesion, then graph it with the bridging ties highlighted.**

```{r tieside, exercise = TRUE, purl=FALSE}

```

```{r tieside-solution}
net_by_adhesion(ison_adolescents)
ison_adolescents |>
  mutate_ties(cut = tie_is_bridge()) |>
  graphr(edge_color = "cut")
```

The adhesion tells you the minimum number of ties to cut to fragment the
network, and the highlighted ties show _which_ ones do that work.
As with cutpoints, a network held together by only one or two bridges is
fragile: losing a single highlighted tie would break it apart.

We could also investigate the opposite of a bridge,
the degree to which ties are deeply embedded in
`r gloss("triangles","triangle")`.
This is called (rather confusingly) tie cohesion.
**Size the ties by their cohesion to see which are most embedded.**

```{r tiecoh, exercise = TRUE, purl=FALSE}

```

```{r tiecoh-solution}
ison_adolescents |>
  mutate_ties(coh = tie_by_cohesion()) |>
  graphr(edge_size = "coh")
```

Where would you target your efforts if you wanted to fragment this network?
And conversely, if you wanted to make this network more resilient,
the map of bridges and cutpoints shows where redundancy is missing:
adding even one tie between the far sides of a bridge
would rob both the bridge and its cutpoints of their fragility.

::: {.callout}
<span class="callout-label">**In brief**:</span> `net_by_connectedness()`, `net_by_cohesion()`, and
`net_by_adhesion()` summarise how hard a network is to fragment — the share
of dyads that can reach each other, and the minimum nodes or ties whose
removal would fragment it. `node_is_cutpoint()` and `tie_is_bridge()` point
to _where_ it would break, and `tie_by_cohesion()` shows which ties are
buttressed by triangles.
:::

## Free play

Choose another dataset included in `{manynet}` (browse them with `table_data()`),
and ask of it the questions this tutorial has equipped you to answer.
Which ideal topology does it most resemble —
is it more lattice-like, small-world, or scale-free?
Does it have a discernible core and periphery?
And how resilient is it: where are its cutpoints and bridges,
and what would it take to fragment it?

If you are not sure where to start,
here is one suggestion per flavour:

| Classic (small, tidy) | Fiction (moderate) | Real-world (larger) |
|---|---|---|
| `ison_karateka` (the karate club that famously split in two — its degree distribution fits a power-law exponent close to 2, and its cutpoints and bridges foreshadowed where the split would come) | `fict_greys` (a sparse network of romantic entanglements in several separate components — find the cutpoints, and check the cohesion of its largest component with `to_giant()`) | `irps_books` (co-purchases of 105 political books — does polarised buying sort the market into a core and periphery, into communities, or around a few hub titles?) |

```{r freeplay, exercise = TRUE, purl=FALSE, fig.width=9}

```

## Summary

<img src="https://media1.tenor.com/m/gSsbNTouixUAAAAC/bravo-applause.gif" alt="gif of an enthusiastic standing ovation and cries of bravo"/>

Well done -- you have completed the tutorial on topology and resilience!
Along the way, you have learned to use these functions:

| Function | What it does |
|---|---|
| `create_empty()`, `create_filled()` | networks with no ties, or with all possible ties |
| `create_star()` | one universal node connected to all others |
| `create_tree()` | branching networks, with `width` branches per node |
| `create_lattice()` | balanced two-dimensional grids of a given number of nodes |
| `create_ring()` | circles, with `width` setting how far along the ring ties reach |
| `create_core()` | an ideal core-periphery network |
| `generate_random()` | ties placed at random, by probability or count (Erdös-Renyi) |
| `generate_smallworld()` | a ring rewired at some probability (Watts–Strogatz) |
| `generate_scalefree()` | growth with preferential attachment (Barabasi-Albert) |
| `generate_fire()`, `generate_islands()`, `generate_citations()` | forest-fire, modular-islands, and citation growth models |
| `net_by_density()` | proportion of possible ties present |
| `net_by_transitivity()` | clustering: how often two nodes' shared neighbour closes into a triangle |
| `net_by_diameter()`, `net_by_length()` | longest and average shortest path lengths |
| `net_by_smallworld()` | small-world coefficient (default 'omega', 1 = maximally small-world) |
| `net_by_scalefree()` | power-law exponent fitted to the degree distribution |
| `net_by_richclub()`, `net_by_assortativity()` | whether hubs interconnect, and whether like degrees attach to like |
| `node_is_core()`, `node_in_core()` | assigns nodes to core/periphery (or core/semi-periphery/periphery) |
| `net_by_core()` | correlation of the network with an ideal core-periphery model |
| `node_by_kcoreness()` | each node's _k_-coreness (depth in the network's successive cores) |
| `net_x_hierarchy()` | Krackhardt's four graph-theoretic dimensions of hierarchy |
| `net_by_connectedness()` | proportion of dyads that can reach each other |
| `net_by_cohesion()`, `net_by_adhesion()` | minimum nodes / ties to remove to fragment the network |
| `node_is_cutpoint()`, `tie_is_bridge()` | flags the nodes / ties whose removal would fragment the network |
| `tie_by_cohesion()` | how deeply each tie is embedded in triangles |
| `to_uniplex()`, `to_undirected()`, `to_redirected()`, `to_giant()` | extract one tie type, drop direction, reverse direction, keep the largest component |
| `mutate_nodes()`, `mutate_ties()`, `graphr()` | map marks and measures onto the graph |
| `chisq.test()` | (base R) association between core status and a categorical attribute |

This completes the `{netrics}` tutorials on describing network structure —
centrality, community, position, and now topology.
Run `run_tute()` at the console to see all available tutorials,
including those in the other `{stocnet}` packages.

### Glossary

Here are some of the terms that we have covered in this tutorial:

`r print_glossary()`

---
title: "Centrality"
author: "by James Hollway"
output:
  learnr::tutorial:
    theme: flatly
runtime: shiny_prerendered
description: >
  This tutorial covers how to measure and map degree, betweenness,
  closeness, eigenvector, and other types of centrality, explore their
  distributions, highlight the most central nodes, and summarise a whole
  network's centralisation.
---

```{r pkgs, include = FALSE, eval=TRUE}
library(netrics)
library(autograph)
library(patchwork)
```

```{r setup, include = FALSE, purl=FALSE, eval=TRUE}
library(learnr)
knitr::opts_chunk$set(echo = FALSE)
library(netrics)
library(autograph)
stocnet_theme("default")
clear_glossary()
learnr::random_phrases_add(language = "fr",
                           praise = c("C'est génial!",
                                      "Beau travail",
                                      "Excellent travail!",
                                      "Bravo!",
                                      "Super!",
                                      "Bien fait",
                                      "Bien joué",
                                      "Tu l'as fait!",
                                      "Je savais que tu pouvais le faire.",
                                      "Ça a l'air facile!",
                                      "C'était un travail de première classe.",
                                      "C'est ce que j'appelle un bon travail!"),
                          encouragement = c("Bon effort",
                                            "Vous l'avez presque maîtrisé!",
                                            "Ça avance bien.",
                                            "Continuez comme ça.",
                                            "Continuez à travailler dur!",
                                            "Vous apprenez vite!",
                                            "Vous faites un excellent travail aujourd'hui."))
learnr::random_phrases_add(language = "en",
                           praise = c("That's brilliant!",
                                      "Great work!",
                                      "Well done!",
                                      "Awesome!"),
                           encouragement = c("Good effort"))
ison_brandes2 <- ison_brandes %>% rename_nodes(type = twomode_type)
```

```{r detect-language, context="server-start", purl=FALSE}
sys_locale <- Sys.getlocale("LC_CTYPE")
if (grepl("fr", sys_locale, ignore.case = TRUE)) {
  options(tutorial.language = "fr")
} else {
  options(tutorial.language = "en")
}
```

<style>
html { font-size: 115%; }
.callout {
  border-left: 3px solid #5B9C78;
  padding: 0.1em 1em;
  margin: 1.2em 0;
}
.callout-label { color: #5B9C78; }
.page-toc { font-style: italic; }
.page-toc-link { color: #5B9C78; cursor: pointer; text-decoration: underline; }
.tutorialTitle { color: #5B9C78 !important; }
#tutorial-topic .nav-link { color: #5B9C78 !important; }
.section a:not(.btn) { color: #5B9C78; }
.section a:not(.btn):hover { color: #487E61; }
.btn-tutorial-run,
.btn-tutorial-run:hover,
.btn-tutorial-run:focus,
.btn-tutorial-run:active {
  background-color: #5B9C78;
  border-color: #5B9C78;
  color: #fff;
}
</style>

## Today's target

Centrality is one of the oldest and most-used ideas in network analysis:
the question of which `r gloss("nodes","node")` are, in some sense, the most
important, prominent, or well-placed in a `r gloss("network")`.
The catch is that "important" can mean several different things,
and each meaning has its own measure — indeed its own _family_ of measures.
This tutorial shows how to measure and map
`r gloss("degree")`, `r gloss("betweenness")`, `r gloss("closeness")`,
`r gloss("eigenvector")`, and related types of centrality,
explore their `r gloss("distributions","distribution")`,
and summarise the whole network's `r gloss("centralisation","centralization")`.

<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExbG5oYmVtYmcxMWs1d3MzOGh3Zng5dTZhc241ZHR2NTE0YThiZWp4NiZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/970Sr8vpwEbXG/giphy.gif" height="480px"/>

::: {.callout}
<span class="callout-label">**Catching up**:</span>
This tutorial assumes you can already load or make a network in R,
and draw it with `graphr()`.
If any of that is hazy, work through the earlier `{stocnet}` tutorials first:
run `run_tute("Making")` and `run_tute("Manipulating")` for network data,
and `run_tute("Visualising")` for graphing,
or read their static versions on the
[manynet](https://stocnet.github.io/manynet/) and
[autograph](https://stocnet.github.io/autograph/) websites.
:::

::: {.callout}
<span class="callout-label">**New to network vocabulary?**:</span>
Throughout this tutorial, key terms are italicised:
hover over them for a definition,
and a full glossary of the terms used appears at the end of the tutorial.
:::

### Aims

By the end of this tutorial, you should be able to:

- [ ] &nbsp; Calculate degree centrality by hand and with `node_by_degree()`, including in- and out-degree for directed networks and strength for weighted ones
- [ ] &nbsp; Compute betweenness, closeness, and eigenvector centrality and interpret what each one means
- [ ] &nbsp; Recognise the wider betweenness-, closeness-, and eigenvector-type families, and choose an appropriate variant (e.g. reach, pagerank, or power)
- [ ] &nbsp; Highlight the most central node(s) on a graph with `node_is_max()` and `graphr()`
- [ ] &nbsp; Read a measure's `r gloss("distribution")` with `plot()`, and see how its shape drives a network's `r gloss("centralisation","centralization")`, computed with `net_by_*()`
- [ ] &nbsp; Know when scores are normalised, and how to obtain the raw scores

**Choose your own data**: The worked examples below use `ison_brandes`,
a small, tidy teaching network bundled with `{manynet}`.
But wherever there is an exercise box, you are encouraged to swap in
a network that interests _you_.
Remember the three flavours of bundled data as a rough difficulty ladder —
**Classic** (`ison_*`, small & tidy),
**Fiction** (`fict_*`, mid-sized & fun),
**Real-world** (`irps_*`, larger & realistic) —
and that you can browse the full list with `table_data()`.

### Setting up {#setting-up}

For this exercise, we'll use the `ison_brandes` dataset in `{manynet}`.
This dataset is in a 'tidygraph' format,
but `manynet` makes it easy to coerce this into other forms
to be compatible with other packages.
We can create a `r gloss("two-mode","twomode")` version of the dataset
by renaming the nodal attribute "twomode_type" to just "type".
Let's begin by graphing these datasets using `graphr()`.

```{r coercion, exercise = TRUE, purl = FALSE}

```

```{r coercion-hint-1, purl = FALSE}
# Let's graph the one-mode version
graphr(____)
```

```{r coercion-hint-2, purl = FALSE}
# Now, let's create a two-mode version 'ison_brandes2' and graph it.
ison_brandes2 <- ison_brandes |> rename_nodes(type = twomode_type)
graphr(____)
```

```{r coercion-solution}
# plot the one-mode version
graphr(ison_brandes)
ison_brandes2 <- ison_brandes |> rename_nodes(type = twomode_type)
# plot the two-mode version
graphr(ison_brandes2, layout = "bipartite")
```

The network is anonymous, but I think it would be nice to add some names,
even if it's just pretend.
Luckily, `{manynet}` has a function for this: `to_named()`.
This makes plotting the network just a wee bit more accessible and interpretable:

```{r addingnames, exercise = TRUE, purl = FALSE}
ison_brandes <- to_named(ison_brandes)
```

```{r addingnames-hint-1, purl = FALSE}
# Now, let's graph using the object names: "ison_brandes"
graphr(____)
```

```{r addingnames-solution}
ison_brandes <- to_named(ison_brandes)
# plot network with names
graphr(ison_brandes)
```

Note that you will likely get a different set of names (though still alphabetical),
as they are assigned randomly from a pool of (American) first names.

## Degree centrality

<span class="page-toc">On this page:
<span class="page-toc-link" onclick="document.getElementById('section-counting-ties').scrollIntoView({behavior:'auto',block:'start'});">Counting ties</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-degrees-of-direction').scrollIntoView({behavior:'auto',block:'start'});">Direction</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-strength-of-ties').scrollIntoView({behavior:'auto',block:'start'});">Strength</span></span>

<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExM3d6eGd2OHRxb2FvbHYzc2oyenFzOG5qcjhzMm82Mm1ncTVpOHhreiZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/uVd5gVHAq63SjtjAOe/giphy.gif" height="480px"/>

### Counting ties {#counting-ties}

Let's start with calculating `r gloss("degree")`.
Remember that degree centrality is just the number of incident edges/ties to each node.
It is therefore easy to calculate yourself.
Just sum the rows or columns of the matrix!

```{r degreesum, exercise = TRUE, exercise.setup = "addingnames", purl = FALSE}

```

```{r degreesum-hint-1, purl = FALSE}
# We can calculate degree centrality like this:
(mat <- as_matrix(ison_brandes))
(degrees <- rowSums(mat))
rowSums(mat) == colSums(mat)
```

```{r degreesum-hint-2, purl = FALSE}
# Or by using a built in command in netrics like this:
node_by_degree(ison_brandes, normalized = FALSE)
# node_by_deg() is a shortcut for exactly this unnormalised version:
node_by_deg(ison_brandes)
```

```{r degreesum-solution}
# manually calculate degree centrality
mat <- as_matrix(ison_brandes)
degrees <- rowSums(mat)
rowSums(mat) == colSums(mat)
# You can also just use a built in command in netrics though:
node_by_degree(ison_brandes, normalized = FALSE)
# node_by_deg() is a shortcut for exactly this unnormalised version:
node_by_deg(ison_brandes)
```

```{r degreesum-Q, echo=FALSE, purl = FALSE}
question("Are the row sums the same as the column sums?",
  answer("Yes",
         correct = TRUE,
         message = "That's right, that's because this is an undirected network."),
  answer("No",
         message = "Actually they are. Because this network is undirected, its adjacency matrix is symmetric, so the sum of row _i_ always equals the sum of column _i_. (In a directed network the row sums give out-degree and the column sums give in-degree, and these can differ.)"),
  allow_retry = TRUE
)
```

A node's degree is the most immediate sense in which it can be "central":
a well-connected node has more opportunities to send or receive whatever
flows through the network, and more visibility to the nodes around it.
But degree is a _local_ measure — it counts a node's ties without regard to
where those ties lead — so a high-degree node is best read as an _active_ or
_busy_ one, not necessarily a powerful or well-positioned one.
The other measures in this tutorial each capture a different, less local sense
of importance.

```{r degreeinterpQ, purl = FALSE}
question("In what ways are higher degree nodes more 'central'?",
         answer("They have more power than other nodes",
                message = "Not necessarily -- degree counts how many ties a node has, but power can depend on *who* those ties are to, not just how many there are."),
         answer("They are more active than other nodes",
                correct = TRUE, message = learnr::random_praise()),
         answer("They would be located in the centre of the graph",
                message = "Not necessarily -- degree is about the number of ties, not where a node happens to be drawn; a high-degree node can sit anywhere in a layout."),
         answer("They would be able to control information flow in the network",
                message = "Not necessarily -- that is betweenness, not degree. A node can have many ties yet still not lie on the shortest paths between others."),
         allow_retry = TRUE,
         random_answer_order = TRUE)
```

::: {.callout}
<span class="callout-label">**Going further**:</span>
All centrality measures in `{netrics}` return _normalised_ scores by default,
so that values are comparable across networks of different sizes.
For the raw scores, add `normalized = FALSE` to any of these functions —
or, for degree, just use the truncated `node_by_deg()`,
which is an alias for the unnormalised version.
:::

### Degrees of direction {#degrees-of-direction}

So far each node has had a single degree, because `ison_brandes` is
`r gloss("undirected")`. In a `r gloss("directed")` network, ties are sent
_from_ one node _to_ another, so every node has two degrees:
an _in-degree_ (ties received) and an _out-degree_ (ties sent).
These can tell quite different stories:
in an advice network, for example, a high in-degree suggests a sought-after
source of advice, while a high out-degree marks an eager asker.

`ison_brandes` has no natural direction, but `{manynet}`'s `to_directed()`
will assign a random direction to each tie —
just as `to_named()` assigned random names —
which is all we need for a worked comparison.
**Run the code, and check whether the row and column sums still agree.**

```{r directing, exercise = TRUE, exercise.setup = "addingnames"}
ison_brandes_dir <- to_directed(ison_brandes)
node_by_indegree(ison_brandes_dir, normalized = FALSE)
node_by_outdegree(ison_brandes_dir, normalized = FALSE)
mat <- as_matrix(ison_brandes_dir)
rowSums(mat) == colSums(mat)
```

Now the row sums (out-degree) and column sums (in-degree) part ways —
the answer to the earlier quiz question flips as soon as direction enters.
Because the directions here are assigned at random,
your in- and out-degrees will differ from run to run;
with real directed data they would of course be meaningful and stable.
`node_by_indegree()` and `node_by_outdegree()` are shortcuts for
`node_by_degree()`'s `direction = "in"` and `direction = "out"` arguments.

### Strength of ties {#strength-of-ties}

What about `r gloss("weighted")` networks, where ties record not just presence
but amounts — messages exchanged, hours worked together, tonnes traded?
There, counting ties is only half the story:
two nodes with three ties each are hardly equivalent if one's ties are ten
times heavier. Summing the _weights_ of a node's ties instead of counting them
yields what is often called _strength_ centrality.

Again `{manynet}` can conjure the missing property for practice purposes:
`to_weighted()` assigns random weights to the ties.
The `alpha` argument to `node_by_deg()` then trades off between
counting ties and summing weights:
`alpha = 0` ignores the weights entirely (pure degree),
`alpha = 1` ignores the counts entirely (pure strength),
and values in between mix the two.
**Compare the two extremes.**

```{r weighting, exercise = TRUE, exercise.setup = "addingnames"}
ison_brandes_wtd <- to_weighted(ison_brandes)
node_by_deg(ison_brandes_wtd)             # alpha = 0: count ties
node_by_deg(ison_brandes_wtd, alpha = 1)  # alpha = 1: sum tie weights
```

Note that on a weighted network, `node_by_degree()` itself defaults to
strength (`alpha = 1`) — the assumption being that if you have measured
weights, you probably care about them.

::: {.callout}
<span class="callout-label">**Going further**:</span>
The degree family has several further members for special kinds of networks:
`node_by_multidegree()` compares tie types in a
`r gloss("multiplex")` network,
`node_by_posneg()` handles `r gloss("signed")` networks with positive and
negative ties,
`node_by_neighbours_degree()` averages the degrees of each node's neighbours,
and `tie_by_degree()` scores each _tie_ by the degrees of its endpoints.
See `?measure_central_degree` for definitions and references.
:::

::: {.callout}
<span class="callout-label">**In brief**:</span> Degree centrality counts each node's ties:
`node_by_degree()` for normalised scores, `node_by_deg()` for raw counts,
`node_by_indegree()`/`node_by_outdegree()` for directed networks,
and `alpha = 1` to sum tie weights (strength) in weighted networks.
It is a local measure of _activity_.
:::

## Betweenness centralities

<span class="page-toc">On this page:
<span class="page-toc-link" onclick="document.getElementById('section-betweenness-centrality').scrollIntoView({behavior:'auto',block:'start'});">Betweenness</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-ties-can-be-between-too').scrollIntoView({behavior:'auto',block:'start'});">Tie betweenness</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-induced-and-other-variants').scrollIntoView({behavior:'auto',block:'start'});">Variants</span></span>

The next three pages tour the three main _families_ of centrality measures
beyond degree.
What the members of the **betweenness family** all have in common is that they
measure how much of the traffic between _other_ nodes depends on a given node
or tie — importance as _control_ over flow.
Where they differ is in what they assume about how things flow:
along shortest paths only, along all paths, or in electrical-current fashion.

### Betweenness centrality {#betweenness-centrality}

`r gloss("Betweenness","betweenness")` measures how often a node lies on the
shortest paths (`r gloss("geodesics","geodesic")`) between other pairs of nodes.
A node with high betweenness sits at a crossroads: whatever passes between
otherwise-distant parts of the network has to go through it, giving it the
_opportunity_ to broker, filter, or bottleneck that flow.
**Calculate the betweenness centralities for `ison_brandes`.**

```{r betcalc, exercise = TRUE, exercise.setup = "addingnames", purl = FALSE}

```

```{r betcalc-hint-1, purl = FALSE}
# Use the node_by_betweenness() function to calculate the
# betweenness centralities of nodes in a network
node_by_betweenness(____)
```

```{r betcalc-solution}
node_by_betweenness(ison_brandes)
```

```{r betinterpQ, purl = FALSE}
question("Why would a node lying 'between' many other nodes be 'central'?",
         answer("They have more power than other nodes",
                message = "Not necessarily -- lying on many shortest paths gives a node the *opportunity* to broker, but that potential isn't automatically power."),
         answer("They would be able to control information flow in the network",
                correct = TRUE, message = learnr::random_praise()),
         answer("They would be located in the centre of the graph",
                message = "Not necessarily -- betweenness is about lying on shortest paths between others, not about a node's position in a layout."),
         answer("They can distribute a message to all other nodes in the network most quickly",
                message = "Not necessarily -- that describes closeness. A node can broker between others without being the quickest to reach everyone."),
         allow_retry = TRUE,
         random_answer_order = TRUE)
```

### Ties can be between too {#ties-can-be-between-too}

Nodes are not the only things that can lie between:
`tie_by_betweenness()` counts the shortest paths that run through each _tie_.
Ties with high betweenness often act as `r gloss("bridges","bridge")` between
otherwise separate parts of the network —
indeed this score is the basis of a famous community-detection algorithm
(Girvan-Newman), which you will meet in the community tutorial.
**Map tie betweenness onto tie width to see the network's main thoroughfares.**

```{r tiebet, exercise = TRUE, exercise.setup = "addingnames"}
ison_brandes %>%
  mutate_ties(bridgeness = tie_by_betweenness(ison_brandes)) %>%
  graphr(edge_size = "bridgeness")
```

The widest ties are the ones the network can least afford to lose:
cut one and many shortest paths between nodes lengthen — or disappear.

### Induced and other variants {#induced-and-other-variants}

The rest of the betweenness family keeps the "who depends on whom" logic
but varies the assumptions:

- `node_by_induced()` measures _induced_ or vitality betweenness:
how much the network's _total_ betweenness changes when the node is removed.
It answers "what would we lose without this node?" rather than
"how often is this node in between?".
- `node_by_flow()` measures _flow_ betweenness,
which models spreading as electrical current across _all_ paths,
not just the shortest ones —
useful when whatever flows does not know the shortest route.
- `node_by_stress()` counts the shortest paths through each node without
weighting them by how many alternatives exist —
an older, "rawer" cousin of betweenness.

**Compare classic and induced betweenness — do they agree here?**

```{r induced, exercise = TRUE, exercise.setup = "addingnames"}
node_by_betweenness(ison_brandes, normalized = FALSE)
node_by_induced(ison_brandes, normalized = FALSE)
```

For this network the rankings broadly agree, but on networks with more
redundancy they can diverge: a node can lie on few shortest paths itself yet
still be sorely missed when removed.
For formal definitions and references, see `?measure_central_between`.

::: {.callout}
<span class="callout-label">**In brief**:</span> The betweenness family measures control over flow:
`node_by_betweenness()` via shortest paths,
`tie_by_betweenness()` for ties rather than nodes,
`node_by_induced()` via a node's contribution to total betweenness,
`node_by_flow()` via current across all paths,
and `node_by_stress()` via raw path counts.
:::

## Closeness centralities

<span class="page-toc">On this page:
<span class="page-toc-link" onclick="document.getElementById('section-closeness-centrality').scrollIntoView({behavior:'auto',block:'start'});">Closeness</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-reach-and-distance').scrollIntoView({behavior:'auto',block:'start'});">Reach & distance</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-harmonic-and-other-variants').scrollIntoView({behavior:'auto',block:'start'});">Variants</span></span>

What the members of the **closeness family** have in common is that they
measure how easily a node can reach — or be reached by — the rest of the
network: importance as _access_ or _independence_ rather than control.
Where they differ is in how they summarise the distances involved:
their total, their average, their maximum, or how many nodes fall within a
given range.

### Closeness centrality {#closeness-centrality}

`r gloss("Closeness","closeness")` measures how short a node's paths are to
_all_ other nodes — the smaller the total `r gloss("geodesic")` distance, the
higher the closeness. A node with high closeness can reach the rest of the
network in few steps, so it is well placed to spread (or hear) something quickly.
**Calculate the closeness centralities for `ison_brandes`.**

```{r closecalc, exercise = TRUE, exercise.setup = "addingnames", purl = FALSE}

```

```{r closecalc-hint-1, purl = FALSE}
# Use the node_by_closeness() function to calculate the
# closeness centrality of nodes in a network
node_by_closeness(____)
```

```{r closecalc-solution}
node_by_closeness(ison_brandes)
```

```{r closeinterpQ, purl = FALSE}
question("In what way is a node with the smallest sum of geodesic distances to all other nodes 'central'?",
         answer("They have more power than other nodes",
                message = "Not necessarily -- closeness reflects how efficiently a node can reach everyone else, which isn't the same as holding power over them."),
         answer("They can distribute a message to all other nodes in the network most quickly",
                correct = TRUE, message = learnr::random_praise()),
         answer("They would be located in the centre of the graph",
                message = "Not necessarily -- closeness is defined by geodesic distances, not by where a node is placed in a drawing."),
         answer("They would be able to control information flow in the network",
                message = "Not necessarily -- that is betweenness. Being close to everyone doesn't mean you sit *between* them on their shortest paths."),
         allow_retry = TRUE,
         random_answer_order = TRUE)
```

### Reach and distance {#reach-and-distance}

Sometimes what matters is not the average distance to everyone,
but a more concrete question:
_how much of the network can this node reach within k steps?_
That is `node_by_reach()`, which by default counts the proportion of the
network within two steps — a good proxy for "who could mobilise the most
others quickly?".
And sometimes the question is about one node in particular:
`node_by_distance()` returns every node's `r gloss("geodesic")` distance
from (or to) a given node — the network equivalent of dropping a pin on a map.
**Try both: how far does the first-named node's reach extend?**

```{r reach, exercise = TRUE, exercise.setup = "addingnames"}
node_by_reach(ison_brandes)
node_by_distance(ison_brandes, from = node_names(ison_brandes)[1])
```

Note that `node_by_reach()` takes a `cutoff` argument to change the number of
steps considered: `cutoff = 1` reduces it to degree,
while a large cutoff approaches a measure of component membership.

### Harmonic and other variants {#harmonic-and-other-variants}

Classic closeness has a well-known weakness: in a disconnected network,
some distances are infinite, and the measure breaks down.
The family includes several responses to this, and other refinements:

- `node_by_harmonic()` sums the _reciprocals_ of distances,
so unreachable nodes simply contribute zero —
the recommended drop-in replacement for closeness on disconnected networks.
- `node_by_eccentricity()` reports each node's _worst case_:
the distance to whatever node is furthest from it.
- `node_by_information()` measures current-flow closeness,
crediting all paths (weighted by efficiency) rather than only the shortest.
- `node_by_vitality()` measures how much the network's total closeness
would change without the node — the closeness analogue of induced betweenness.

For formal definitions and references, see `?measure_central_close`.

::: {.callout}
<span class="callout-label">**In brief**:</span> The closeness family measures access:
`node_by_closeness()` via total distances,
`node_by_reach()` via the share of the network within _k_ steps,
`node_by_distance()` via distances from a chosen node,
`node_by_harmonic()` as the disconnection-proof variant,
and `node_by_eccentricity()` via each node's furthest distance.
:::

## Eigenvector centralities

<span class="page-toc">On this page:
<span class="page-toc-link" onclick="document.getElementById('section-eigenvector-centrality').scrollIntoView({behavior:'auto',block:'start'});">Eigenvector</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-power-and-influence').scrollIntoView({behavior:'auto',block:'start'});">Power</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-pagerank-hubs-and-authorities').scrollIntoView({behavior:'auto',block:'start'});">Pagerank & hubs</span></span>

What the members of the **eigenvector family** have in common is recursion:
a node's score depends on the scores of the nodes it is connected to,
which depend in turn on _their_ neighbours, and so on —
importance as _standing among the important_.
Where they differ is in how that recursion is tempered:
whether alters' scores always help, how far along walks influence travels,
and whether sending and receiving are scored separately.

### Eigenvector centrality {#eigenvector-centrality}

`r gloss("Eigenvector","eigenvector")` centrality weights a node's ties by how
central its neighbours are: you are important if you are connected to important
others. It captures influence rather than mere activity — a node with only a
few ties, but to very well-connected nodes, can outscore a node with many ties
to peripheral ones.
**Calculate the eigenvector centralities for `ison_brandes`.**

```{r eigencalc, exercise = TRUE, exercise.setup = "addingnames", purl = FALSE}

```

```{r eigencalc-hint-1, purl = FALSE}
# Use the node_by_eigenvector() function to calculate
# the eigenvector centrality of nodes in a network
node_by_eigenvector(____)
```

```{r eigencalc-solution}
node_by_eigenvector(ison_brandes)
```

```{r eiginterpQ, purl = FALSE}
question("Which of the following is true of eigenvector centrality?",
         answer("It is the only centrality measure that is correlated with power",
                message = "Not necessarily -- eigenvector centrality has no unique claim on power, and other measures can correlate with it too."),
         answer("It is a measure of influence, not activity",
                correct = TRUE, message = learnr::random_praise()),
         answer("It always gives the highest score to the node with the highest degree",
                message = "Not necessarily -- because it weights ties by the importance of their neighbours, a lower-degree node connected to influential nodes can outrank a higher-degree node connected to peripheral ones."),
         answer("It is only meaningful in networks with nodes of similar degree",
                message = "Not necessarily -- it is meaningful (and often most revealing) precisely when degrees vary, since it distinguishes nodes with equally many but differently-connected neighbours."),
         allow_retry = TRUE,
         random_answer_order = TRUE)
```

::: {.callout}
<span class="callout-label">**Going further**:</span>
A note of caution: eigenvector centrality is only well-behaved on a single
`r gloss("connected","connected")` `r gloss("component")` —
on a disconnected network the scores for the smaller components can be
misleading. The pagerank variant below is one common response.
:::

### Power and influence {#power-and-influence}

Bonacich famously argued that `r gloss("power")` and influence are _not_ the
same thing. Under eigenvector logic, well-connected neighbours always raise
your score. But in bargaining or exchange, the reverse can hold:
being connected to poorly-connected others makes _them_ dependent on _you_ —
that is power.
`node_by_power()` implements Bonacich's measure, and its `exponent` argument
sets the sign of the recursion:
a positive exponent rewards well-connected neighbours (influence-like),
while a negative one rewards _dependent_, poorly-connected neighbours (power).
**Compare the two.**

```{r power, exercise = TRUE, exercise.setup = "addingnames"}
node_by_power(ison_brandes, exponent = 1)
node_by_power(ison_brandes, exponent = -1)
```

Try to answer the following questions for yourself:

- which nodes swap fortunes when the exponent flips, and what do their
neighbourhoods look like?
- can you think of a real-world example when an actor might be central but not
powerful, or powerful but not central?

### Pagerank, hubs, and authorities {#pagerank-hubs-and-authorities}

Two further members of this family were developed for the web,
and shine on `r gloss("directed")` networks:

- `node_by_pagerank()` measures pagerank, the algorithm behind early Google:
a random surfer follows ties and occasionally teleports to a random node,
and a node's score is how often the surfer visits.
The teleporting makes it robust on disconnected and directed networks
where plain eigenvector centrality struggles.
- `node_by_hub()` and `node_by_authority()` split reputation in two:
`r gloss("authorities","authority")` are nodes that many point _to_,
while `r gloss("hubs","hub")` are nodes that point to many good authorities —
think review sites (hubs) versus the restaurants they recommend (authorities).
- `node_by_alpha()` measures alpha or Katz centrality,
which extends eigenvector logic to count longer walks at a discount —
see `?measure_central_eigen` for details on the whole family.

**Try these on a directed version of our network.**

```{r pagerank, exercise = TRUE, exercise.setup = "addingnames"}
ison_brandes_dir <- to_directed(ison_brandes)
node_by_pagerank(ison_brandes_dir)
node_by_hub(ison_brandes_dir)
node_by_authority(ison_brandes_dir)
```

Since the directions are again assigned at random,
run the code a few times: watch how a node's authority score rises and falls
with the arrows pointing at it.

You have now met all four families of centrality —
degree, betweenness, closeness, and eigenvector —
each ranking the nodes by a different notion of importance.
With four families (and many variants within each) to choose from,
how do you decide? That is the subject of the next page.

::: {.callout}
<span class="callout-label">**In brief**:</span> The eigenvector family measures standing among the
important: `node_by_eigenvector()` for influence,
`node_by_power()` for Bonacich power (with its sign-flipping `exponent`),
`node_by_pagerank()` for a robust directed-network variant,
and `node_by_hub()`/`node_by_authority()` for split sending/receiving
reputations.
:::

## Which centrality?

There are, by one count, [over 200 centrality indices in the
literature](https://www.r-bloggers.com/network-centrality-in-r-an-introduction/) —
so which should you reach for?
Linton Freeman's classic answer was that most of them boil down to just three
ideas — degree, closeness, and betweenness — with everything else a variant;
we add eigenvector as a fourth, giving the **four families** this tutorial is
built around.

Two questions organise the whole zoo, and locate each family within it.
First: does the measure count what radiates _from_ a node — its own ties, its
distances, its walks — or what passes _through_ it on the way between others?
Borgatti and Everett ([2006](https://doi.org/10.1016/j.socnet.2005.11.005)) call the first **radial** and the second **medial**:
degree, closeness, and eigenvector are radial; betweenness is medial.
Second: does it read only a node's immediate neighbourhood (**local**),
or its place in the whole network (**global**)?
Degree is local; the other three are global.
The four families are simply the most useful corners of that space,
and each variant you met earlier is a refinement or extension within its family.

| Family | Reads a node's… | Built on | Answers the question | Well-suited when |
|---|---|---|---|---|
| **Degree** | activity (radial, local) | direct ties | "Who is busiest, or most directly connected?" | only immediate ties matter, or influence spreads one step at a time; also the cheapest to compute on very large networks; outdegree associated with activity and indegree with popularity; tie weight and sign versions exist too |
| **Closeness** | reach (radial, global) | shortest-path distances to all others | "Who can reach — or be reached by — everyone else in the fewest steps?" | the network is connected and things travel by efficient routes; use `node_by_harmonic()` if it is disconnected |
| **Betweenness** | brokerage (medial, global) | shortest paths that pass through it | "Who sits between others, brokering, bridging, or bottlenecking flow?" | you care about gatekeeping, bridges between groups, or where flow is vulnerable; costly on huge networks (use a `cutoff`) |
| **Eigenvector** | standing (radial, global) | walks of all lengths, weighted by neighbours' scores | "Who is connected to other important, well-connected nodes?" | importance is recursive — prestige, status, influence; use `node_by_pagerank()` for directed or disconnected networks; use `node_by_power()` if influence depends on being connected to poorly-connected nodes |

Reading across a row tells you what a family is _for_;
reading down the last column tells you which _kind of network_ each suits best.

```{r whichcentQ, purl = FALSE}
question("A rumour spreads step-by-step along ties, and you want to find the person who could seed it to reach everyone else in the fewest steps. Which family is the most natural first choice?",
  answer("Closeness",
         correct = TRUE,
         message = "Yes -- closeness is precisely about short distances to all other nodes, i.e. reaching everyone quickly."),
  answer("Degree",
         message = "Degree finds who has the most direct contacts, but a well-connected node in a peripheral cluster can still be far from most of the network."),
  answer("Betweenness",
         message = "Betweenness finds brokers who sit between others -- great for controlling flow, but not the same as being able to reach everyone quickly yourself."),
  answer("Eigenvector",
         message = "Eigenvector finds nodes connected to important others (influence/prestige), which need not be the nodes that can reach everyone in few steps."),
  allow_retry = TRUE,
  random_answer_order = TRUE
)
```

::: {.callout}
<span class="callout-label">**Going further**:</span>
For the full landscape see David Schoch's
[periodic table of centrality](http://schochastics.net/sna/periodic.html)
and the `{netrankr}` package, which can even compare nodes _without_ committing
to a single index.
:::

Now let's see how to spot the most central nodes at a glance.

## Plotting centrality

<span class="page-toc">On this page:
<span class="page-toc-link" onclick="document.getElementById('section-highlighting-the-most-central-node').scrollIntoView({behavior:'auto',block:'start'});">Highlighting extremes</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-the-two-mode-version').scrollIntoView({behavior:'auto',block:'start'});">Two-mode</span></span>

### Highlighting the most central node {#highlighting-the-most-central-node}

It is straightforward in `{autograph}` to highlight nodes and ties
with maximum or minimum (e.g. degree) scores.
If the vector is numeric (i.e. a "measure"),
then this can be easily converted into a logical vector that
identifies the node/tie with the maximum/minimum score using
e.g. `node_is_max()` or `tie_is_min()`.
By passing this attribute to the `graphr()` argument "node_color"
we can highlight which node or nodes hold the maximum score in a different colour.

```{r ggid, exercise = TRUE, exercise.setup = "addingnames", purl = FALSE}

```

```{r ggid-solution}
# plot the network, highlighting the node with the highest centrality score with a different color
ison_brandes %>%
  mutate_nodes(color = node_is_max(node_by_degree())) %>%
  graphr(node_color = "color")

ison_brandes %>%
  mutate_nodes(color = node_is_max(node_by_betweenness())) %>%
  graphr(node_color = "color")

ison_brandes %>%
  mutate_nodes(color = node_is_max(node_by_closeness())) %>%
  graphr(node_color = "color")

ison_brandes %>%
  mutate_nodes(color = node_is_max(node_by_eigenvector())) %>%
  graphr(node_color = "color")
```

How neat! Notice whether the _same_ node lights up across the four measures or
whether different nodes do — because the measures capture different things,
a node can be top of one ranking and unremarkable on another.

### The two-mode version {#the-two-mode-version}

Try it with the two-mode version.
What can you see?

```{r ggid_twomode, exercise = TRUE, purl = FALSE}
# Instead of "ison_brandes", use "ison_brandes2"

```

```{r ggid_twomode-solution}
ison_brandes2 %>%
  add_node_attribute("color", node_is_max(node_by_degree(ison_brandes2))) %>%
  graphr(node_color = "color", layout = "bipartite")

ison_brandes2 %>%
  add_node_attribute("color", node_is_max(node_by_betweenness(ison_brandes2))) %>%
  graphr(node_color = "color", layout = "bipartite")

ison_brandes2 %>%
  add_node_attribute("color", node_is_max(node_by_closeness(ison_brandes2))) %>%
  graphr(node_color = "color", layout = "bipartite")

ison_brandes2 %>%
  add_node_attribute("color", node_is_max(node_by_eigenvector(ison_brandes2))) %>%
  graphr(node_color = "color", layout = "bipartite")
```

```{r brandes2quiz, purl = FALSE}
question("Select all that are true for the two-mode Brandes network.",
         answer("Only one node is selected in each plot.",
                message = "Not quite: `node_is_max()` flags every node tied for the highest score, so when there's a tie for the top several nodes light up at once."),
         answer("The maximum degree triangle has a higher degree than the maximum degree circle(s).",
                correct = TRUE,
                message = "That's right -- the busiest event (triangle) has degree 4, whereas the busiest actor (circle) only reaches degree 3."),
         answer("No node is ever the most central according to two or more different centrality measures.",
                message = "Actually the same node can top several measures at once -- centrality measures are often correlated, so watch for a node that stays highlighted across plots."),
         allow_retry = TRUE,
         random_answer_order = TRUE)
```

::: {.callout}
<span class="callout-label">**In brief**:</span> Turn any numeric measure into a logical flag with
`node_is_max()`/`node_is_min()`, then map it to `node_color` in `graphr()`
to spotlight the most (or least) central node(s) directly on the graph.
:::

## Centralisation

<span class="page-toc">On this page:
<span class="page-toc-link" onclick="document.getElementById('section-reading-a-distribution').scrollIntoView({behavior:'auto',block:'start'});">Distributions</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-from-distribution-to-centralisation').scrollIntoView({behavior:'auto',block:'start'});">Shape &amp; centralisation</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-measuring-centralisation').scrollIntoView({behavior:'auto',block:'start'});">Measuring</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-comparing-the-measures').scrollIntoView({behavior:'auto',block:'start'});">Comparing</span></span>

So far we have scored individual nodes.
But we can also ask a question about the _whole_ network:
is its centrality piled up on a few nodes, or shared out evenly?
That whole-network summary is called `r gloss("centralisation","centralization")`,
and the bridge to it runs through the _distribution_ of a nodal measure.

### Reading a distribution {#reading-a-distribution}

<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExanZ4OWhwM2k0eWU1ancza3NtYXlpZDFkdTJub2RvMnFkYnlwbWZ4dCZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/3o7aTHbH39h8xYePza/giphy.gif" height="400px"/>

Rather than reduce a measure to a single summary number,
we can look at its whole `r gloss("distribution")` across the nodes.
`{autograph}` offers a way to get a pretty good first look at this,
though there are more elaborate ways to do this in base and grid graphics.
**Plot the degree distribution of `ison_brandes`.**

```{r distrib, exercise = TRUE, exercise.setup = "addingnames", purl = FALSE}

```

```{r distrib-solution}
# distribution of degree centrality scores of nodes
plot(node_by_degree(ison_brandes))
```

What's plotted here by default is both the degree distribution as a histogram,
as well as a density plot overlaid on it in red.
Reading a distribution rather than a single summary number lets you see
_shape_: whether ties are spread fairly evenly across nodes, or concentrated
on a few.

```{r degdistshapeQ, purl = FALSE}
question("What kind of shape does this degree distribution have?",
         answer("Exponential",
                message = "An exponential (or power-law) shape would show many low-degree nodes and a long tail of a few very high-degree nodes. There isn't a clear tail like that here."),
         answer("Hard to tell",
                correct = TRUE,
                message = paste(learnr::random_praise(), "With only 11 nodes there simply aren't enough observations to identify a clear parametric shape -- degree distributions usually only reveal a recognisable form in larger networks.")),
         answer("Uniform",
                message = "A uniform distribution would have roughly equal numbers of nodes at every degree value (a flat histogram), which isn't quite what we see."),
         answer("Normal",
                message = "A normal distribution would be a symmetric bell centred on a modal degree. With so few nodes it's hard to justify calling this bell-shaped."),
         allow_retry = TRUE,
         random_answer_order = TRUE)
```

```{r degdistinterpQ, purl = FALSE}
question("What can the degree distribution tell us?",
         answer("Whether the network is centralized around a few key nodes",
                correct = TRUE, message = learnr::random_praise()),
         answer("Whether there are a few highly connected nodes and many poorly connected nodes",
                correct = TRUE, message = learnr::random_praise()),
         answer("Whether the network is hierarchical",
                message = "Not necessarily -- the degree distribution shows the *spread* of degrees, but not how nodes are ordered into levels. Hierarchy is about the (directed) arrangement of ties, which degree counts alone don't reveal."),
         answer("Whether the network is robust to random failure",
                message = "Not necessarily -- robustness also depends on how the network is connected and how failures cascade, so the degree distribution alone can't tell you this directly."),
         allow_retry = TRUE,
         random_answer_order = TRUE)
```

The same works for _any_ node measure, not just degree.
**Plot the distributions of the other three centralities you have met.**

```{r otherdist, exercise=TRUE, exercise.setup = "addingnames", purl=FALSE}

```

```{r otherdist-solution}
plot(node_by_betweenness(ison_brandes))
plot(node_by_closeness(ison_brandes))
plot(node_by_eigenvector(ison_brandes))
```

### From distribution to centralisation {#from-distribution-to-centralisation}

Here is the key idea. The _shape_ of a degree distribution and the network's
_centralisation_ are two views of the same thing.
When degrees are spread evenly — a symmetric, roughly _normal_
bell where most nodes are near-average — no node dominates, and centralisation
is low. When the distribution is skewed into a long, _exponential_
tail — a few hubs with very high degree, and many nodes with very little —
a handful of nodes dominate, and centralisation is high.

To see this, let's conjure two networks of the same size with deliberately
different degree distributions:
a _random_ network (whose degrees cluster around a typical
value) and a _scale-free_ network (which grows a few dominant hubs).
The seed just fixes the random draw so everyone sees the same picture.
**Run it, and compare both the plots and the two centralisation scores.**

```{r distcent, exercise = TRUE, fig.width=9}
set.seed(7)
rand <- generate_random(50, 0.1)     # bell-shaped degree distribution
scaf <- generate_scalefree(50, 1.3)  # heavy-tailed degree distribution
plot(node_by_degree(rand)) + plot(node_by_degree(scaf))
net_by_degree(rand)   # low centralisation
net_by_degree(scaf)   # much higher centralisation
```

The random network's degrees pile up in a bell, and its degree centralisation
is low — no node is far above the crowd.
The scale-free network's degrees trail off into a long tail, and its
centralisation is several times higher — a few hubs carry the network.
Our `ison_brandes` example sits somewhere between these two extremes.

```{r shapecentQ, purl = FALSE}
question("A network's degree distribution is a symmetric bell around its average. What can you say about its degree centralisation?",
  answer("It will be relatively low",
         correct = TRUE,
         message = "Right -- a symmetric bell means most nodes are near-average and none dominates, so there is little inequality for centralisation to register."),
  answer("It will be relatively high",
         message = "High centralisation goes with a skewed, hub-dominated distribution (a long tail), not a symmetric bell where nodes are similar."),
  answer("It cannot be determined from the distribution",
         message = "It can, at least roughly: centralisation is essentially a summary of how unequal (spread out and skewed) the distribution is."),
  allow_retry = TRUE,
  random_answer_order = TRUE
)
```

::: {.callout}
<span class="callout-label">**In brief**:</span> `plot()` on any node measure draws its distribution
(a histogram plus a density curve). An even, bell-shaped distribution signals
low centralisation; a skewed, heavy-tailed one signals high centralisation.
Centralisation just turns that distributional inequality into a single number.
:::

### Measuring centralisation {#measuring-centralisation}

`{netrics}` also implements network `r gloss("centralisation","centralization")` functions.
Here we are no longer interested in the level of the node,
but in the level of the whole network,
so the syntax replaces `node_` with `net_`:

```{r centzn, exercise = TRUE, exercise.setup = "addingnames", purl = FALSE}

```

```{r centzn-solution}
net_by_degree(ison_brandes)
net_by_betweenness(ison_brandes)
net_by_closeness(ison_brandes)
print(net_by_eigenvector(ison_brandes), digits = 5)
```

By default, scores are printed up to 3 decimal places,
but this can be modified and, in any case,
the unrounded values are retained internally.
This means that even if rounded values are printed (to respect console space),
as much precision as is available is used in further calculations.

::: {.callout}
<span class="callout-label">**Going further**:</span>
For centralisation in `r gloss("two-mode","twomode")` networks,
`mode_by_*()` functions are available to return two values, one per mode.
:::

### Comparing the measures {#comparing-the-measures}

What if we want to have a single image/figure with multiple plots?
This can be a little tricky with gg-based plots,
but fortunately the `{patchwork}` package is here to help.
We use `|` to place graphs side-by-side and `/` to stack them,
and `ggtitle()` to record each measure's centralisation score as a subtitle.

```{r multiplot, exercise = TRUE, exercise.setup = "addingnames", purl = FALSE, fig.width=9}

```

```{r multiplot-solution}
ison_brandes <- ison_brandes |>
  add_node_attribute("degree", node_is_max(node_by_degree(ison_brandes)))  |>
  add_node_attribute("betweenness", node_is_max(node_by_betweenness(ison_brandes))) |>
  add_node_attribute("closeness", node_is_max(node_by_closeness(ison_brandes)))  |>
  add_node_attribute("eigenvector", node_is_max(node_by_eigenvector(ison_brandes)))
gd <- graphr(ison_brandes, node_color = "degree") +
  ggtitle("Degree", subtitle = round(net_by_degree(ison_brandes), 2))
gc <- graphr(ison_brandes, node_color = "closeness") +
  ggtitle("Closeness", subtitle = round(net_by_closeness(ison_brandes), 2))
gb <- graphr(ison_brandes, node_color = "betweenness") +
  ggtitle("Betweenness", subtitle = round(net_by_betweenness(ison_brandes), 2))
ge <- graphr(ison_brandes, node_color = "eigenvector") +
  ggtitle("Eigenvector", subtitle = round(net_by_eigenvector(ison_brandes), 2))
(gd | gb) / (gc | ge)
# ggsave("brandes-centralities.pdf")
```

```{r centzdq, purl = FALSE}
question("How centralized is the ison_brandes network? Select all that apply.",
         answer("It is more degree centralised than betweenness centralised.",
                message = "Degree centralisation is at 0.18 for this network whereas betweenness centralisation is at 0.32. In other words, the network is better characterised as having 1 or 2 nodes lying on the shortest paths between others than one where 1 or 2 nodes have many more ties than the others."),
         answer("It is more closeness centralised than betweenness centralised.",
                message = "Closeness centralisation is at 0.23 for this network whereas betweenness centralisation is at 0.32. In other words, the network is better characterised as having 1 or 2 nodes lying on the shortest paths between others than one where 1 or 2 nodes can reach or access most other nodes."),
         answer("It is more eigenvector centralised than betweenness centralised.",
                correct = TRUE,
                message = "That's right, eigenvector centralisation is at 0.48 for this network whereas betweenness centralisation is at 0.32. In other words, the network is better characterised as having a core (or cores) of well-connected nodes rather than a wide network with only 1 or 2 nodes lying on the shortest paths between others."),
         random_answer_order = TRUE,
         allow_retry = TRUE)
```

```{r centvcent, echo=FALSE, purl = FALSE}
question("What is the difference between centrality and centralisation according to the literature?",
  answer("Centrality is for nodes and centralisation is for networks",
         correct = TRUE,
         message = "That's right -- centrality is a nodal property (how central each node is), while centralisation summarises the whole network (how unequally centrality is distributed across its nodes)."),
  answer("Centrality is a state and centralisation is a process",
         message = "This is a nice intuition, but it isn't the distinction drawn in the network literature -- both are measured on a fixed network, not as an unfolding process."),
  answer("Centrality is a '-ity' and centralisation is a '-isation'",
         message = "The spelling differs, but that's not the substantive distinction -- think about which level (node vs. network) each one describes."),
  answer("Centrality is to centralisation what polarity is to polarisation",
         message = "A tempting analogy, but it doesn't capture the actual difference, which is about the level of analysis: nodes for centrality, the whole network for centralisation."),
  allow_retry = TRUE,
  random_answer_order = TRUE
)
```

::: {.callout}
<span class="callout-label">**In brief**:</span> Swap `node_` for `net_` to move from a node's
centrality to the whole network's _centralisation_, summarising how unequally that centrality is
distributed. For multimodal networks, `mode_by_*()` returns one value per mode,
which can be compared to see which mode is more centralised with respect to the other.
:::

## Free play

<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExejh4dXBha3JhNW0yenJwMDd5aGpwd3Brc201ZXoxZnY2aWFna3Z4cCZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/7ROgX60TtTVK0mZwkJ/giphy.gif" height="400px"/>

Choose another dataset included in `{manynet}` (browse them with `table_data()`).
Name a plausible research question you could ask of the dataset relating to each
of the four main centrality measures (degree, betweenness, closeness, eigenvector).
Plot the network with nodes sized or coloured by each centrality measure,
using titles or subtitles to record the question and/or centralisation measure.

If you are not sure where to start,
here is one suggestion per flavour, each with real direction and/or weights
to practise the measures from this tutorial on:

| Classic (small, tidy) | Fiction (moderate) | Real-world (larger) |
|---|---|---|
| `ison_networkers` (directed, weighted messages among early network researchers — compare in-degree with strength: who receives from many, and who receives a lot?) | `fict_starwars` (directed, weighted interactions between Star Wars characters — who dominates the dialogue, and is that the same as being influential?) | `irps_blogs` (directed hyperlinks among 1,490 political blogs — made for pagerank, hubs, and authorities) |

And if betweenness is what intrigues you,
`irps_911` — Krebs' study of a covert network —
is a classic setting for asking who brokers while staying inconspicuous.

```{r freeplayend, exercise=TRUE, purl=FALSE, fig.width=9}

```

## Summary

<img src="https://media1.tenor.com/m/gSsbNTouixUAAAAC/bravo-applause.gif" alt="gif of an enthusiastic standing ovation and cries of bravo"/>

Well done -- you have completed the tutorial on centrality!
Along the way, you have learned to use these functions:

| Function | What it does |
|---|---|
| `node_by_degree()`, `node_by_deg()` | degree centrality (ties per node); `node_by_deg()` returns the raw, unnormalised counts |
| `node_by_indegree()`, `node_by_outdegree()` | ties received and sent in directed networks |
| `node_by_deg(..., alpha = 1)` | strength: sums tie weights instead of counting ties |
| `node_by_betweenness()` | betweenness centrality (lying on others' shortest paths) |
| `tie_by_betweenness()` | shortest paths through each tie (bridges) |
| `node_by_induced()`, `node_by_flow()`, `node_by_stress()` | betweenness variants: contribution to total, current flow, raw path counts |
| `node_by_closeness()` | closeness centrality (short paths to all others) |
| `node_by_reach()`, `node_by_distance()` | share of network within _k_ steps; distances from a chosen node |
| `node_by_harmonic()`, `node_by_eccentricity()` | closeness variants for disconnected networks and worst-case distance |
| `node_by_eigenvector()` | eigenvector centrality (connected to well-connected others) |
| `node_by_power()` | Bonacich power, with its sign-flipping `exponent` |
| `node_by_pagerank()`, `node_by_hub()`, `node_by_authority()` | web-style variants for directed networks |
| `node_by_*(..., normalized = FALSE)` | returns raw rather than normalised scores |
| `plot()` | draws the distribution of a node measure (histogram + density) |
| `generate_random()`, `generate_scalefree()` | (manynet) make example networks with contrasting degree distributions |
| `node_is_max()`, `node_is_min()` | flags the node(s) with the extreme score, for highlighting |
| `graphr(..., node_color = )`, `graphr(..., edge_size = )` | maps node or tie measures onto the graph |
| `net_by_degree()`, `net_by_betweenness()`, `net_by_closeness()`, `net_by_eigenvector()` | whole-network centralisation |
| `+`, `/` (patchwork) | place graphs beside or above one another in one figure |

When you are ready, continue with the other `{netrics}` tutorials —
on community, position, and topology —
where further ways of summarising network structure are introduced.
Run `run_tute()` at the console to see all available tutorials.

### Glossary

Here are some of the terms that we have covered in this tutorial:

`r print_glossary()`

---
title: "Cohesion and Community"
author: "by James Hollway"
output:
  learnr::tutorial:
    theme: flatly
runtime: shiny_prerendered
description: >
  This tutorial covers how to measure the cohesion of a network through
  density, reciprocity, and transitivity, project two-mode networks,
  identify components and factions, judge the fit of any grouping with
  modularity, and detect communities using a variety of algorithms.
---

```{r pkgs, include = FALSE, eval=TRUE}
library(netrics)
library(autograph)
library(patchwork)
```

```{r setup, include = FALSE, purl=FALSE, eval=TRUE}
library(learnr)
knitr::opts_chunk$set(echo = FALSE)
library(netrics)
library(autograph)
stocnet_theme("default")
clear_glossary()
learnr::random_phrases_add(language = "fr",
                           praise = c("C'est génial!",
                                      "Beau travail",
                                      "Excellent travail!",
                                      "Bravo!",
                                      "Super!",
                                      "Bien fait",
                                      "Bien joué",
                                      "Tu l'as fait!",
                                      "Je savais que tu pouvais le faire.",
                                      "Ça a l'air facile!",
                                      "C'était un travail de première classe.",
                                      "C'est ce que j'appelle un bon travail!"),
                          encouragement = c("Bon effort",
                                            "Vous l'avez presque maîtrisé!",
                                            "Ça avance bien.",
                                            "Continuez comme ça.",
                                            "Continuez à travailler dur!",
                                            "Vous apprenez vite!",
                                            "Vous faites un excellent travail aujourd'hui."))
learnr::random_phrases_add(language = "en",
                           praise = c("That's brilliant!",
                                      "Great work!",
                                      "Well done!",
                                      "Awesome!"),
                           encouragement = c("Good effort"))
friends <- to_uniplex(ison_algebra, "friends")
social <- to_uniplex(ison_algebra, "social")
tasks <- to_uniplex(ison_algebra, "tasks")
```

```{r detect-language, context="server-start", purl=FALSE}
sys_locale <- Sys.getlocale("LC_CTYPE")
if (grepl("fr", sys_locale, ignore.case = TRUE)) {
  options(tutorial.language = "fr")
} else {
  options(tutorial.language = "en")
}
```

<style>
html { font-size: 115%; }
.callout {
  border-left: 3px solid #5B9C78;
  padding: 0.1em 1em;
  margin: 1.2em 0;
}
.callout-label { color: #5B9C78; }
.page-toc { font-style: italic; }
.page-toc-link { color: #5B9C78; cursor: pointer; text-decoration: underline; }
.tutorialTitle { color: #5B9C78 !important; }
#tutorial-topic .nav-link { color: #5B9C78 !important; }
.section a:not(.btn) { color: #5B9C78; }
.section a:not(.btn):hover { color: #487E61; }
.btn-tutorial-run,
.btn-tutorial-run:hover,
.btn-tutorial-run:focus,
.btn-tutorial-run:active {
  background-color: #5B9C78;
  border-color: #5B9C78;
  color: #fff;
}
</style>

## Today's target

In the Centrality tutorial, we asked which `r gloss("nodes","node")` stand out
in a `r gloss("network")` — centrality as an expression of structural
inequality. But nodes don't always seek to differentiate themselves in a
network. Sometimes they are interested in being part of a group.
This tutorial considers various ways in which such 'groupness' might be
examined: how `r gloss("cohesive","cohesion")` a network is overall,
whether its ties close into groups,
how it breaks into `r gloss("components","component")` and factions,
and how well any proposed grouping fits —
its `r gloss("modularity")` — including groupings discovered by
`r gloss("community")` detection algorithms.

<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExbDNsNmZnNzJ2YzRyYzE5cDl4M2pxeGU4d2M1bnMwZm1qd3JvMWw3MyZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/N7AOIqqUpjXUc/giphy.gif" alt="gif of man not knowing how to study"/>

::: {.callout}
<span class="callout-label">**Catching up**:</span>
This tutorial assumes you can already load or make a network in R,
draw it with `graphr()`, and measure it with `node_by_*()` and `net_by_*()`
functions.
If any of that is hazy, work through the earlier `{stocnet}` tutorials first:
run `run_tute("Making")` and `run_tute("Manipulating")` for network data,
`run_tute("Visualising")` for graphing,
and `run_tute("Centrality")` for the measuring syntax used throughout,
or read their static versions on the
[manynet](https://stocnet.github.io/manynet/),
[autograph](https://stocnet.github.io/autograph/), and
[netrics](https://stocnet.github.io/netrics/) websites.
:::

::: {.callout}
<span class="callout-label">**New to network vocabulary?**:</span>
Throughout this tutorial, key terms are italicised:
hover over them for a definition,
and a full glossary of the terms used appears at the end of the tutorial.
:::

### Aims

By the end of this tutorial, you should be able to:

- [ ] &nbsp; Measure how cohesive a network is with `net_by_density()`, and interpret the result for the type of network at hand
- [ ] &nbsp; Measure closure with `net_by_reciprocity()` and `net_by_transitivity()`, and connect each to a mechanism of tie formation
- [ ] &nbsp; Project a two-mode network into its one-mode projections with `to_mode1()` and `to_mode2()`, choosing an appropriate similarity option — and know what projection obscures
- [ ] &nbsp; Count weak and strong components with `net_by_components()` and map their membership with `node_in_component()`
- [ ] &nbsp; Concentrate on a network's giant component with `to_giant()` and split it into factions with `node_in_partition()`
- [ ] &nbsp; Judge the fit of _any_ membership assignment with `net_by_modularity()`
- [ ] &nbsp; Detect communities with the walktrap, edge-betweenness, and fast-greedy algorithms, and let `node_in_community()` choose among them for you

**Choose your own data**: The worked examples below use `ison_algebra`,
`ison_southern_women`, and `irps_blogs`,
all bundled with `{manynet}`.
But wherever there is an exercise box, you are encouraged to swap in
a network that interests _you_.
Remember the three flavours of bundled data as a rough difficulty ladder —
**Classic** (`ison_*`, small & tidy),
**Fiction** (`fict_*`, mid-sized & fun),
**Real-world** (`irps_*`, larger & realistic) —
and that you can browse the full list with `table_data()`.

### Setting up {#setting-up}

The data we're going to use here, "ison_algebra", is included in the `{manynet}` package.
Do you remember how to call the data?
Can you find out some more information about it?

```{r data, exercise = TRUE, purl = FALSE}

```

```{r data-hint-1, purl = FALSE}
# Let's call and load the 'ison_algebra' dataset
data("ison_algebra", package = "manynet")
# Or you can retrieve like this:
ison_algebra <- manynet::ison_algebra
```

```{r data-hint-2, purl = FALSE}
# If you want to learn more about the 'ison_algebra' dataset, use the following function (below)
?manynet::ison_algebra
```

```{r data-solution}
data("ison_algebra", package = "manynet")
?manynet::ison_algebra
# If you want to see the network object, you can run the name of the object
ison_algebra
# or print the code with brackets at the front and end of the code
(ison_algebra <- manynet::ison_algebra)
```

We can see after printing the object that the dataset is
`r gloss("multiplex")`,
meaning that it contains several different types of ties:
friendship (friends), social (social) and task interactions (tasks).

### Adding names {#adding-names}

The network is also anonymous, but I think it would be nice to add some names,
even if it's just pretend.
Luckily, `{manynet}` has a function for this, `to_named()`.
This makes plotting the network just a wee bit more accessible and interpretable.
Let's try adding names and graphing the network now:

```{r addingnames, exercise=TRUE, exercise.setup = "data", purl = FALSE}

```

```{r addingnames-hint-1, purl = FALSE}
ison_algebra <- to_named(ison_algebra)
```

```{r addingnames-hint-2, purl = FALSE}
graphr(ison_algebra)
```

```{r addingnames-solution}
ison_algebra <- to_named(ison_algebra)
graphr(ison_algebra)
```

Note that you will likely get a different set of names,
as they are assigned randomly from a pool of (American) first names.

### Separating multiplex networks {#separating-networks}

As a multiplex network,
there are actually three different types of ties (friends, social, and tasks)
in this network.
We can extract them and graph them separately using `to_uniplex()`:

```{r separatingnets, exercise=TRUE, exercise.setup = "data", purl = FALSE}

```

```{r separatingnets-hint-1, purl = FALSE}
# to_uniplex extracts ties of a single type,
# focusing on the 'friends' tie attribute here
friends <- to_uniplex(ison_algebra, "friends")
gfriend <- graphr(friends) + ggtitle("Friendship")
```

```{r separatingnets-hint-2, purl = FALSE}
# now let's focus on the 'social' tie attribute
social <- to_uniplex(ison_algebra, "social")
gsocial <- graphr(social) + ggtitle("Social")
```

```{r separatingnets-hint-3, purl = FALSE}
# and the 'tasks' tie attribute
tasks <- to_uniplex(ison_algebra, "tasks")
gtask <- graphr(tasks) + ggtitle("Task")
```

```{r separatingnets-hint-4, purl = FALSE}
# now, let's compare each attribute's graph, side-by-side
gfriend + gsocial + gtask
# if you get an error here, you may need to install and load
# the package 'patchwork'.
# It's highly recommended for assembling multiple plots together.
# Otherwise you can just plot them separately on different lines.
```

```{r separatingnets-solution}
friends <- to_uniplex(ison_algebra, "friends")
gfriend <- graphr(friends) + ggtitle("Friendship")

social <- to_uniplex(ison_algebra, "social")
gsocial <- graphr(social) + ggtitle("Social")

tasks <- to_uniplex(ison_algebra, "tasks")
gtask <- graphr(tasks) + ggtitle("Task")

# We now have three separate networks depicting each type of tie from the ison_algebra network:
gfriend + gsocial + gtask
```

::: {.callout}
<span class="callout-label">**Beginner note**:</span>
Placing two or more gg-based plots side by side with `+`
relies on the `{patchwork}` package.
If `gfriend + gsocial + gtask` errors for you,
install and load `{patchwork}` —
or just plot each object on its own line.
:::

Note also that these are `r gloss("weighted")` networks.
`graphr()` automatically recognises these different weights and plots them.
Where useful (less dense `r gloss("directed")` networks),
`graphr()` also bends reciprocated arcs.
What (else) can we say about these three networks?

## Cohesion

<span class="page-toc">On this page:
<span class="page-toc-link" onclick="document.getElementById('section-defining-density').scrollIntoView({behavior:'auto',block:'start'});">Defining density</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-calculating-density').scrollIntoView({behavior:'auto',block:'start'});">Calculating density</span></span>

Let's concentrate on the **task** network for now and calculate
perhaps the most common generalised measure of cohesion:
`r gloss("density")`.

### Defining density {#defining-density}

Density represents a generalised measure of cohesion,
characterising how cohesive the network is in terms of how many
potential ties (i.e. `r gloss("dyads","dyad")`) are actualised.
Recall that there are different equations depending on the type of network.
Below are three equations:

$$A: \frac{|T|}{|N|(|N|-1)}$$
$$B: \frac{2|T|}{|N|(|N|-1)}$$
$$C: \frac{|T|}{|N||M|}$$

where $|T|$ is the number of ties in the network,
and $|N|$ and $|M|$ are the number of nodes in the first and second mode respectively.

```{r densq, echo=FALSE, purl = FALSE}
question("Which equation is used for measuring density for a directed network:",
         answer("A",
                correct = TRUE,
                message = '<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExb2M5cTNmZm1kdmszdmR6Yzc3MnczcWxvb3MzdHRic2N0ZncwYml2MSZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/YcMs3OGd89Pxu/giphy.gif" alt="gif of a thumbs up"/>'
),
         answer("B",
                message = "This is the equation for an undirected network."),
         answer("C",
                message = "This is the equation for a two-mode network."),
        allow_retry = TRUE
)
```

### Calculating density {#calculating-density}

One can calculate the density of the network using the number of nodes
and the number of ties using the functions `net_nodes()` and `net_ties()`,
respectively:

```{r dens-explicit, exercise=TRUE, exercise.setup = "separatingnets", purl = FALSE}

```

```{r dens-explicit-solution}
# calculating network density manually according to equation
net_ties(tasks)/(net_nodes(tasks)*(net_nodes(tasks)-1))
```

but we can also just use the `{netrics}` function for calculating the density,
which always uses the equation appropriate for the type of network...

```{r dens, exercise=TRUE, exercise.setup = "separatingnets", purl = FALSE}

```

```{r dens-solution}
net_by_density(tasks)
```

Note that the various measures in `{netrics}` print results to three decimal points
by default, but the underlying result retains the same recurrence.
So same result...

::: {.callout}
<span class="callout-label">**Beginner note**:</span>
The `net_by_` prefix signals a whole-network measure,
just as `node_by_` signalled a nodal measure in the Centrality tutorial.
The same grammar recurs throughout `{netrics}`:
`net_by_density()`, `net_by_reciprocity()`, `net_by_transitivity()`,
`net_by_components()`, `net_by_modularity()`, and so on.
:::

```{r dens-qa, echo=FALSE, purl = FALSE}
question("Is this network's density high or low in absolute terms?",
         answer("High",
                message = "The closer the value is to 1, the more dense the network and the more cohesive the network is as a whole."),
         answer("Low",
                correct = TRUE,
                message = 'The closer the value is to 0, the sparser the network and the less cohesive the network is as a whole. But this is still quite typical density for a relatively small, social network like this one.  <br><br>    <img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExOWs4Y292OXg5YmlvNTF3OWp4YWxmNjNpdm11c2kyMm55Z2U4eTJkeiZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/6agGMtS4rEVSo/giphy.gif" alt="gif of i knew it"/>'),
        allow_retry = TRUE
)
```

Density offers an important baseline measure for characterising
the network as a whole.
Here a little over a third of the possible task ties are present (0.367) —
far from complete, yet busy enough that task activity clearly spreads
beyond a handful of pairs.
Because density depends so strongly on network size
(each new node adds many more _possible_ ties than actual ones),
raw densities should only be compared across networks of similar size —
larger networks are almost always sparser.

::: {.callout}
<span class="callout-label">**Going further**:</span>
Density is not the only sense in which a network can be cohesive.
`net_by_cohesion()` reports the minimum number of nodes
that would need to be removed to disconnect the network,
and `net_by_adhesion()` the minimum number of ties —
`r gloss("adhesion")` is thus the tie-based counterpart to node-based
cohesion. See `?measure_cohesion` for the full set.
:::

::: {.callout}
<span class="callout-label">**In brief**:</span> Density is the proportion of possible ties that are
actually present: `net_by_density()` always applies the equation appropriate
to the network's type, or calculate it by hand from `net_ties()` and
`net_nodes()`. It is a size-sensitive baseline for how cohesive a network is
as a whole.
:::

## Closure

<span class="page-toc">On this page:
<span class="page-toc-link" onclick="document.getElementById('section-reciprocity').scrollIntoView({behavior:'auto',block:'start'});">Reciprocity</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-transitivity').scrollIntoView({behavior:'auto',block:'start'});">Transitivity</span></span>

In this section we're going to move from generalised measures of cohesion,
like density, to more localised measures of cohesion.
These are not only measures of cohesion though,
but are also often associated with certain mechanisms of closure.
Closure involves ties being more likely because other ties are present.
There are two common examples of this in the literature:
`r gloss("reciprocity")`,
where a directed tie is often likely to prompt a reciprocating tie,
and `r gloss("transitivity")`,
where a directed two-path is likely to be shortened
by an additional arc connecting the first and third nodes on that path.

<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExZ29idmxwMGNpb3Z6YzV0NGdremNoZ3A3OXdjeHlsd3UwNDdwY3ZwOCZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/dxVdo4ca7FkME/giphy.gif" alt="gif of ah ha gotcha"/>

### Reciprocity {#reciprocity}

First, let's calculate reciprocity in the task network.
While one could do this by hand,
it's more efficient to do this using the `{netrics}` package.
Can you guess the correct name of the function?

```{r recip, exercise=TRUE, exercise.setup = "separatingnets", purl = FALSE}

```

```{r recip-solution}
net_by_reciprocity(tasks)
# this function calculates the amount of reciprocity in the whole network
```

Wow, this seems quite high based on what we observed visually!
But if we look closer, this makes sense.
We can use `tie_is_reciprocated()` to identify those ties that are
reciprocated and not.

```{r recip-explanation, exercise = TRUE, exercise.setup = "separatingnets"}
tasks |> mutate_ties(rec = tie_is_reciprocated(tasks)) |>
  graphr(edge_color = "rec")
net_by_indegree(tasks)
```

So we can see that indeed there are very few asymmetric ties,
and yet node 16 is both the sender and receiver of most of the task activity.
So our reciprocity measure has taught us something about this network
that might not have been obvious visually:
task interactions here are nearly always mutual —
where one node works with another, the recognition flows both ways —
even though _how much_ each node participates remains very unequal.

### Transitivity {#transitivity}

And let's calculate transitivity in the task network.
Again, can you guess the correct name of this function?

```{r trans, exercise=TRUE, exercise.setup = "separatingnets", purl = FALSE}

```

```{r trans-solution}
net_by_transitivity(tasks)
# this function calculates the amount of transitivity in the whole network
```

```{r trans-interp, echo=FALSE, purl = FALSE}
question("What can we say about task closure in this network? Choose all that apply.",
  answer("Transitivity for the task network is 0.568",
         correct = TRUE,
         message = '<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExZXBpNDlsZzNraTJ1bHN6MW1scjA4aDVzdXo1NW11bGI5bWF0dzYxMyZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/Hg7gPa05K6bAc/giphy.gif" alt="gif of cool cool cool"/>'),
  answer("Transitivity for the task network is -0.568",
         message = "Transivitity must be between 0 and 1."),
  answer("Transitivity is quite low in this network",
         message = "Transitivity is often around 0.3 in most social networks."),
  answer("Transitivity is quite high in this network",
         correct = TRUE,
         message = "Yes -- at 0.568 it is well above the ~0.3 typical of many social networks, so a lot of this network's two-paths are closed into triangles."),
  answer("Transitivity is likely higher in the task network than the friendship network",
         correct = TRUE,
         message = "Yes -- task collaboration tends to be strongly clustered (if A works with B and B with C, A and C often end up working together too), so we'd expect higher transitivity here than among friendships."),
  random_answer_order = TRUE,
  allow_retry = TRUE
)
```

At 0.568, well over half of this network's two-paths are closed into
`r gloss("triangles","triangle")` —
if two students each work with a common third, they usually end up
working with each other too.
That is exactly what we would expect of task interactions,
which tend to happen within teams,
and it foreshadows this tutorial's destination:
where closure concentrates, groups emerge.

::: {.callout}
<span class="callout-label">**Going further**:</span>
Both closure measures also come in nodal versions:
`node_by_reciprocity()` and `node_by_transitivity()` score each node's
_local_ closure, and `tie_is_transitive()` flags the ties involved in
closed triads. See `?measure_closure` and `?measure_closure_node`
for definitions and references.
:::

::: {.callout}
<span class="callout-label">**In brief**:</span> Closure measures capture ties begetting ties:
`net_by_reciprocity()` for how often directed ties are returned
(with `tie_is_reciprocated()` to map which ones),
and `net_by_transitivity()` for how often two-paths close into triangles.
Both hint at the group structure explored in the rest of this tutorial.
:::

## Projection

<span class="page-toc">On this page:
<span class="page-toc-link" onclick="document.getElementById('section-a-two-mode-network').scrollIntoView({behavior:'auto',block:'start'});">A two-mode network</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-two-projections').scrollIntoView({behavior:'auto',block:'start'});">Two projections</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-projection-options').scrollIntoView({behavior:'auto',block:'start'});">Projection options</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-two-mode-closure').scrollIntoView({behavior:'auto',block:'start'});">Two-mode closure</span></span>

### A two-mode network {#a-two-mode-network}

The next dataset, 'ison_southern_women', is also available in `{manynet}`.
It's a classic `r gloss("two-mode","twomode")` network about the attendance
of 18 women at 14 social events in the 1930s American South.
For more details see `?ison_southern_women` for documentation in your R/RStudio.
Let's load and graph the data.

```{r setup-women, exercise=TRUE, exercise.setup = "data", purl = FALSE}

```

```{r setup-women-hint-1, purl = FALSE}
# let's load the data and analyze it
data("ison_southern_women")
ison_southern_women
```

```{r setup-women-hint-2, purl = FALSE}
graphr(ison_southern_women, node_color = "type")
graphr(ison_southern_women, "railway", node_color = "type")
```

```{r setup-women-solution}
data("ison_southern_women")
ison_southern_women
graphr(ison_southern_women, node_color = "type")
```

### Two projections {#two-projections}

Now what if we are only interested in one part of the network?
For that, we can obtain a `r gloss("projection")` of the two-mode network.
There are two ways of doing this.
The hard way...

```{r hardway, exercise=TRUE, exercise.setup = "setup-women", purl = FALSE}

```

```{r hardway-solution}
twomode_matrix <- as_matrix(ison_southern_women)
women_matrix <- twomode_matrix %*% t(twomode_matrix)
event_matrix <- t(twomode_matrix) %*% twomode_matrix
```

Or the easy way:

```{r easyway, exercise=TRUE, exercise.setup = "setup-women"}
# women-graph
# to_mode1(): Results in a weighted one-mode object that retains the row nodes from
# a two-mode object, and weights the ties between them on the basis of their joint
# ties to nodes in the second mode (columns)

women_graph <- to_mode1(ison_southern_women)
graphr(women_graph)

# note that projection `to_mode1` involves keeping one type of nodes
# this is different from to_uniplex above, which keeps one type of ties in the network

# event-graph
# to_mode2(): Results in a weighted one-mode object that retains the column nodes from
# a two-mode object, and weights the ties between them on the basis of their joint ties
# to nodes in the first mode (rows)

event_graph <- to_mode2(ison_southern_women)
graphr(event_graph)
```

The women's projection now records which women moved in the same social
circles, and the event projection which events drew a common crowd —
but note how much denser both look than the original two-mode network:
every event has been replaced by ties among _all_ of its attendees,
a point we will return to below.

### Projection options {#projection-options}

`{manynet}` also includes several other options for how to construct the projection.
The default ("count") might be interpreted as indicating the degree or amount of
shared ties to the other mode.
"jaccard" divides this count by the number of nodes in the other mode
to which either of the nodes are tied.
It can thus be interpreted as opportunity weighted by participation.
"rand" instead counts both shared ties and shared absences,
and can thus be interpreted as the degree of behavioural mirroring between the
nodes.
Lastly, "pearson" (Pearson's coefficient) and "yule" (Yule's Q) produce
correlations in ties for valued and binary data respectively.

```{r otherway, exercise=TRUE, exercise.setup = "setup-women", purl = FALSE}

```

```{r otherway-solution}
tie_weights(to_mode1(ison_southern_women, similarity = "count"))
tie_weights(to_mode1(ison_southern_women, similarity = "jaccard"))
tie_weights(to_mode1(ison_southern_women, similarity = "rand"))
tie_weights(to_mode1(ison_southern_women, similarity = "pearson"))
tie_weights(to_mode1(ison_southern_women, similarity = "yule"))
```

Practically speaking, most projection is conducted using the "count" method,
as it reflects the absolute overlap, the frequency of co-occurrence.
As such, it is most commonly used in social and organisational network analysis,
reflecting shared group membership or collaboration intensity.
But it is sensitive to asymmetries in the number of nodes in each node set and
the degree distribution of each node set, and ignores the absence of ties as
uninformative.
To normalise for these effects, "jaccard" and "rand" are often used.
"pearson" and "yule" are less commonly used, but can be useful when
interested in the correlation of ties between nodes.

### Closure in two-mode networks {#two-mode-closure}

Let's return to the question of closure.
First try one of the closure measures we have already treated that gives us
a sense of shared partners for one-mode networks.
Then compare this with `net_by_equivalency()`, which can be used on the original
two-mode network.

```{r twomode-cohesion, exercise=TRUE, exercise.setup = "easyway", purl = FALSE}

```

```{r twomode-cohesion-hint-1, purl = FALSE}
# net_by_transitivity(): Calculate transitivity in a network

net_by_transitivity(women_graph)
net_by_transitivity(event_graph)
```

```{r twomode-cohesion-hint-2, purl = FALSE}
# net_by_equivalency(): Calculate equivalence or reinforcement in a (usually two-mode) network

net_by_equivalency(ison_southern_women)
```

```{r twomode-cohesion-solution}
net_by_transitivity(women_graph)
net_by_transitivity(event_graph)
net_by_equivalency(ison_southern_women)
```

```{r equil-interp, echo=FALSE, purl = FALSE}
question("What do we learn from this? Choose all that apply.",
  answer("Transitivity in the women projection is very high.",
         correct = TRUE,
         message = "Yes -- transitivity in the women projection is about 0.93, close to the maximum of 1."),
  answer("Transitivity in the event projection is very high.",
         correct = TRUE,
         message = "Yes -- transitivity in the event projection is about 0.83, also very high."),
  answer("Equivalence for the two-mode Southern Women dataset is moderate.",
         correct = TRUE,
         message = '<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExZmdqZHFraGR5Ym9udTBrYzA1ejhwd3ljOXdma3ppNWp2cXozMTh0bCZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/xqjZLKH1iTBe0/giphy.gif" alt="gif of you just wrinkled my brain"/>'),
  answer("Transitivity will often be very high in projected networks.",
         correct = TRUE,
         message = "Yes -- projecting a two-mode network turns every event into a clique of its attendees, and cliques are perfectly transitive, so projections almost always inflate transitivity."),
  answer("Projection obscures which women are members of which events and vice versa.",
         correct = TRUE,
         message = "Yes -- once projected we only see woman-to-woman (or event-to-event) ties, losing the original membership information that connected the two modes."),
  # random_answer_order = TRUE,
  allow_retry = TRUE
)
```

The lesson here is that projection _manufactures_ closure:
every event becomes a `r gloss("clique")` of its attendees,
and since cliques are perfectly transitive,
the projections' transitivity scores (0.93 and 0.83) are inflated
well beyond anything the underlying behaviour requires.
The two-mode `r gloss("equivalency")` score (0.47) is the more honest summary
of how much the women's attendance patterns actually reinforce one another.

Try to explain in no more than a paragraph why projection can lead to misleading transitivity measures and what some consequences of this might be.

::: {.callout}
<span class="callout-label">**In brief**:</span> `to_mode1()`/`to_mode2()` project a two-mode network
into ties among one of its node sets, with a `similarity` argument
("count", "jaccard", "rand", "pearson", "yule") governing the weights.
Projection inflates transitivity and hides memberships, so where possible
measure the two-mode network directly, e.g. with `net_by_equivalency()`.
:::

## Components

<span class="page-toc">On this page:
<span class="page-toc-link" onclick="document.getElementById('section-counting-components').scrollIntoView({behavior:'auto',block:'start'});">Counting components</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-component-membership').scrollIntoView({behavior:'auto',block:'start'});">Component membership</span></span>

### Counting components {#counting-components}

Now let's look at the friendship network, 'friends'.
We're interested here in how many
`r gloss("components", "component")`
there are.
By default, the `net_by_components()` function will
return the number of _strong_ components for directed networks.
For _weak_ components, you will need to first make the network
`r gloss("undirected")`.
Remember the difference between weak and strong components?

```{r weak-strong, echo = FALSE, purl = FALSE}
question("Weak components...",
  answer("don't care about tie direction when establishing components.",
         correct = TRUE,
         message = "That's right -- a weak component treats every tie as if it were undirected, so two nodes are in the same weak component if a path connects them ignoring arrow direction."),
  answer("care about tie direction when establishing components.",
         message = "That describes *strong* components, where every node must be reachable from every other following tie direction. Weak components ignore direction."),
  allow_retry = TRUE
)
```

```{r comp-no, exercise=TRUE, exercise.setup = "separatingnets", purl = FALSE}

```

```{r comp-no-hint-1, purl = FALSE}
net_by_components(friends)
# note that friends is a directed network
# you can see this by calling the object 'friends'
# or by running `is_directed(friends)`
```

```{r comp-no-hint-2, purl = FALSE}
# Now let's look at the number of components for objects connected by an undirected edge
# Note: to_undirected() returns an object with all tie direction removed,
# so any pair of nodes with at least one directed edge
# will be connected by an undirected edge in the new network.
net_by_components(to_undirected(friends))
```

```{r comp-no-solution}
# note that friends is a directed network
net_by_components(friends)
net_by_components(to_undirected(friends))
```

```{r comp-interp, echo = FALSE, purl = FALSE}
question("How many components are there?",
  answer("2",
         message = "There are more than 2 components."),
  answer("3",
         message = "There are 3 _weak_ components.",
         correct = TRUE),
  answer("4",
         message = 'There are 4 _strong_ components. <img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExMDB5bzhjcGo3NHozZm5jdHcyMWk5emhnajNyZnFxc3Z2cmJnbGN0ZyZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/YpS0oFFC2VJ84/giphy.gif" alt="gif of two thumbs up"/>',
         correct = TRUE),
  answer("5",
         message = "There are fewer than 5 components."),
  allow_retry = TRUE
)
```

### Component membership {#component-membership}

So we know how many components there are,
but maybe we're also interested in which nodes are members of which components?
`node_in_component()` returns a membership vector
that can be used to color nodes in `graphr()`:

```{r comp-memb, exercise=TRUE, exercise.setup = "separatingnets", purl = FALSE}

```

```{r comp-memb-hint-1, purl = FALSE}
friends <- friends |>
  mutate_nodes(weak_comp = node_in_component(to_undirected(friends)),
               strong_comp = node_in_component(friends))
# node_in_component returns a vector of nodes' memberships to components in the network
# here, we are adding the nodes' membership to components as an attribute in the network
# alternatively, we can also use the function `add_node_attribute()`
# eg. `add_node_attribute(friends, "weak_comp", node_in_component(to_undirected(friends)))`
```

```{r comp-memb-hint-2, purl = FALSE}
graphr(friends, node_color = "weak_comp") + ggtitle("Weak components") +
graphr(friends, node_color = "strong_comp") + ggtitle("Strong components")
# by using the 'node_color' argument, we are telling graphr to color
# the nodes in the graph according to the values of the 'weak_comp' attribute in the network
```

```{r comp-memb-solution}
friends <- friends |>
  mutate_nodes(weak_comp = node_in_component(to_undirected(friends)),
               strong_comp = node_in_component(friends))
graphr(friends, node_color = "weak_comp") + ggtitle("Weak components") +
graphr(friends, node_color = "strong_comp") + ggtitle("Strong components")
```

```{r node-comp-interp, echo = FALSE, purl = FALSE}
question("Why is there a difference between the weak and strong components results?",
  answer("Because one node has only incoming ties.",
         correct = TRUE,
         message = "Yes -- a node that only receives ties can't reach anyone by following arrows, so it is stranded as its own *strong* component; ignore direction and it re-joins whoever points at it, which is why there is one fewer weak component."),
  answer("Because three nodes cannot reach any other nodes.",
         correct = TRUE,
         message = "Yes -- three nodes have no outgoing ties, so in the directed (strong) view each is stranded on its own. Dropping direction lets the connectable one rejoin the graph."),
  answer("Because there is an extra isolate.",
         message = "Not quite -- the isolates are identical in both analyses, so they can't account for the *difference* between the weak and strong counts."),
  answer("Because the tie strength matters.",
         message = "No -- components depend only on whether a path exists, not on how strong the ties are. Tie weights play no role here."),
  random_answer_order = TRUE,
  allow_retry = TRUE
)
```

Substantively, the difference between the two maps is one node that only
_receives_ friendship nominations: following the arrows it can reach no one,
so it counts as its own strong component,
but ignoring direction it rejoins the main friendship cluster.
Components thus give us a first, strict sense of groupness —
who is connected to whom _at all_ —
before we ask the subtler question of who clusters with whom.

::: {.callout}
<span class="callout-label">**In brief**:</span> Components partition a network by
`r gloss("reachability")`: `net_by_components()` counts them
(strong by default for directed networks; wrap in `to_undirected()` for weak),
and `node_in_component()` returns each node's membership,
ready to map onto `node_color` in `graphr()`.
:::

## Factions

<span class="page-toc">On this page:
<span class="page-toc-link" onclick="document.getElementById('section-a-bigger-network').scrollIntoView({behavior:'auto',block:'start'});">A bigger network</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-the-giant-component').scrollIntoView({behavior:'auto',block:'start'});">The giant component</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-finding-a-partition').scrollIntoView({behavior:'auto',block:'start'});">Finding a partition</span></span>

### A bigger network {#a-bigger-network}

Components offer a precise way of understanding groups in a network.
However, they can also ignore some 'groupiness' that is obvious to even a
cursory examination of the graph.
The `irps_blogs` network concerns the url links between political blogs
in the 2004 election.
It is a big network (you can check below),
so in our experience it can take a few seconds to graph —
we will concentrate on a random sample to speed things up.

```{r blogsize, exercise = TRUE}
# This is a large network
net_nodes(irps_blogs)
# Let's concentrate on just a sample of 240
# (by deleting a random sample of 1250)
blogs <- delete_nodes(irps_blogs, sample(1:1490, 1250))
graphr(blogs)
```

But are they all actually linked?
Even among the smaller sample, there seems to be a number of
`r gloss("isolates","isolate")`.
We can calculate the number of isolates by simply summing `node_is_isolate()`.

```{r blogisolates, exercise = TRUE, exercise.setup = "blogsize"}
sum(node_is_isolate(blogs))
```

Since there are many isolates, there will be many components,
even if we look at weak components and not just strong components.

```{r blogcomp, exercise = TRUE, exercise.setup = "blogsize"}
node_in_component(blogs)
node_in_component(to_undirected(blogs))
```

### The giant component {#the-giant-component}

So, it looks like most of the (weak) components are due to isolates!
How do we concentrate on the main component of this network?
Well, the main/largest component in a network is called the
`r gloss("giant component", "giant")`.

```{r blogtogiant, exercise=TRUE, warning=FALSE, fig.width=9, exercise.setup = "blogsize"}
blogs <- blogs |> to_giant()
sum(node_is_isolate(blogs))
graphr(blogs)
```

Finally, we have a single 'giant' component to examine.
However, now we have a different kind of challenge:
everything is one big hairball.
And yet, if we think about what we might expect of the structure of a network
of political blogs, we might not think it is so undifferentiated.
We might hypothesise that, despite the _graphical_ presentation of a hairball,
there is actually a reasonable `r gloss("partition")` of the network into
two `r gloss("factions", "faction")`.

### Finding a partition {#finding-a-partition}

To find a partition in a network, we use the `node_in_partition()` function.

::: {.callout}
<span class="callout-label">**Beginner note**:</span>
All `node_in_*()` functions return a string vector the length of the number
of nodes in the network — a string vector because membership is a
categorical result, unlike the numeric vectors the `node_by_*()` measures
return. Since it is the length of the nodes,
we can assign the result to the nodes as an attribute and graph with it.
:::

```{r bloggraph, exercise=TRUE, exercise.setup = "blogtogiant", warning=FALSE, fig.width=9}
blogs |> mutate_nodes(part = node_in_partition()) |>
  graphr(node_color = "part")
```

We see from this graph that indeed there seems to be an obvious separation
between the left and right 'hemispheres' of the network.
Where the components view saw a single undifferentiated blob,
the bipartition recovers a plausible two-camp structure —
but how good a description of the network is it really?
That question needs a measure of fit, which is where we turn next.

::: {.callout}
<span class="callout-label">**Going further**:</span>
`net_by_factions()` offers a whole-network summary of how factional
a network is: it measures the correlation between the observed network and
an idealised component model with the same dimensions,
using `node_in_partition()`'s bipartition when no membership is supplied.
See `?measure_features` for details.
:::

::: {.callout}
<span class="callout-label">**In brief**:</span> When a network is dominated by isolates and minor
components, `to_giant()` focuses analysis on the giant component
(count isolates first with `sum(node_is_isolate())`).
`node_in_partition()` then splits the network into two factions,
returning a categorical membership vector you can map onto `node_color`.
:::

## Modularity

<span class="page-toc">On this page:
<span class="page-toc-link" onclick="document.getElementById('section-measuring-fit').scrollIntoView({behavior:'auto',block:'start'});">Measuring fit</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-given-memberships').scrollIntoView({behavior:'auto',block:'start'});">Given memberships</span></span>

### Measuring fit {#measuring-fit}

But what is the 'fit' of this assignment of the blog nodes into two partitions?
The most common measure of the fit of a community assignment in a network is
`r gloss("modularity")`.

```{r blogmod, exercise=TRUE, exercise.setup = "blogtogiant"}
net_by_modularity(blogs, membership = node_in_partition(blogs))
```

Remember that modularity is conventionally described as ranging between 1 and -1,
though in practice the lowest value a real partition reaches is only about -0.5.
How can we interpret this result?

```{r blogQ, echo = FALSE, purl = FALSE}
question("Which statements about modularity are true?",
  answer("A community assignment with modularity > 0 means there are more ties within communities than between communities.",
         correct = TRUE,
         message = "Yes -- positive modularity means more ties fall within communities than we'd expect if ties were placed at random, i.e. more within than between."),
  answer("A community assignment with modularity = 0 means any tie is as likely to occur within a community as between assigned communities.",
         correct = TRUE,
         message = "Yes -- modularity of 0 is exactly the chance baseline: the within-community tie share matches what the null model expects, so the partition captures no extra structure."),
  answer("In a community assignment with modularity = 1, every community must have an internal density of 1 and an external density of 0.",
         message = "Not so -- modularity is defined relative to a random null model, not by raw densities, and 1 is an upper bound that real partitions approach but never actually reach."),
  # FLAG: correctness of this answer is arguable -- see message. Under the
  # strict/real-world reading (floor ~= -0.5) the statement is actually TRUE,
  # so you may want to mark it correct = TRUE or reword it. Left as-is for review.
  answer("A community assignment with modularity = -1 is impossible.",
         message = "It depends what we mean by 'range'. Modularity is often *quoted* as running from -1 to 1, and in that nominal sense -1 is just its stated lower boundary. In practice, though, the lowest value a real partition can reach is only about -0.5, so a modularity of exactly -1 is effectively unattainable. Keep that gap between the quoted range and the achievable minimum in mind."),
  random_answer_order = TRUE,
  allow_retry = TRUE
)
```

A clearly positive modularity like this one tells us the two-faction reading
of the blogosphere is not an artefact of the layout:
ties really do fall within the two camps more often than a random
rewiring of the network would produce.

### Given memberships {#given-memberships}

While the partition algorithm is useful for deriving a partition of the network
into the number of factions assigned,
it is still an algorithm that tries to maximise modularity.
Other times we might instead have an empirically collected grouping,
and we are keen to see how 'modular' the network is around this attribute.
This only works on categorical attributes, of course,
but is otherwise quite flexible.

```{r blogmodassign, exercise=TRUE, exercise.setup = "blogtogiant", warning=FALSE, fig.width=9}
graphr(blogs, node_color = "Leaning")
net_by_modularity(blogs, membership = node_attribute(blogs, "Leaning"))
```

<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExcXFwZWF2azV0YTlwdnM0bTZncXpuOTdxdHFyMjJqcXJod2V3b29vYiZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/PsBRTPKG71YVq/giphy.gif" alt="gif of Chevy Chase saying plot twist"/>

How interesting.
Perhaps the partitioning algorithm is not the algorithm that maximises
modularity after all...
The blogs' own declared political leanings describe the network's tie
pattern _better_ than the bipartition the algorithm found —
so there must be room for other solutions
that return an even greater modularity criterion.

::: {.callout}
<span class="callout-label">**In brief**:</span> `net_by_modularity()` scores the fit of _any_
membership vector — algorithmic (e.g. `node_in_partition()`) or empirical
(e.g. `node_attribute()`) — as the excess of within-group ties over a random
baseline. Positive values mean the grouping captures real structure,
and competing groupings can be compared on the same yardstick.
:::

## Communities

<span class="page-toc">On this page:
<span class="page-toc-link" onclick="document.getElementById('section-walktrap').scrollIntoView({behavior:'auto',block:'start'});">Walktrap</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-edge-betweenness').scrollIntoView({behavior:'auto',block:'start'});">Edge betweenness</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-fast-greedy').scrollIntoView({behavior:'auto',block:'start'});">Fast greedy</span> ·
<span class="page-toc-link" onclick="document.getElementById('section-detecting-communities').scrollIntoView({behavior:'auto',block:'start'});">Detecting communities</span></span>

Ok, the friendship network has 3-4 components, but how many 'groups' are there?
Visually, it looks like there are some denser clusters within the main component.

Today we'll use the 'friends' subgraph for exploring community detection methods.
For clarity and simplicity,
we will concentrate on the main component (the so-called 'giant' component)
and consider friendship undirected.

```{r manip-fri, exercise=TRUE, exercise.setup = "separatingnets"}
# to_giant() returns an object that includes only the main component without any smaller components or isolates
(friends <- to_giant(friends))
(friends <- to_undirected(friends))
graphr(friends)
```

Comparing `friends` before and after these operations,
you'll notice the number of ties decreases as reciprocated directed ties
are consolidated into single undirected ties,
and the number of nodes decreases as two isolates are removed.

There is no one single best community detection algorithm.
Instead there are several, each with their strengths and weaknesses.
Since this is a rather small network, we'll focus on the following methods:
walktrap, edge betweenness, and fast greedy.
(Others are included in `{netrics}`/`{igraph}`)
As you use them, consider how they portray communities and consider which one(s)
afford a sensible view of the social world as cohesively organized.

### Walktrap {#walktrap}

This algorithm detects communities through a series of short random walks,
with the idea that nodes encountered on any given random `r gloss("walk")`
are more likely to be within a community than not.
It was proposed by Pons and Latapy (2005).

The algorithm initially treats all nodes as communities of their own, then
merges them into larger communities, still larger communities, and so on.
In each step a new community is created from two other communities,
and its ID will be one larger than the largest community ID so far.
This means that before the first merge we have n communities
(the number of vertices in the graph) numbered from zero to n-1.
The first merge creates community n, the second community n+1, etc.
This merge history is returned by the function:
` # ?igraph::cluster_walktrap`

Note the "steps=" argument that specifies the length of the random walks.
While `{igraph}` sets this to 4 by default,
which is what is recommended by Pons and Latapy,
Waugh et al (2009) found that for many groups (Congresses),
these lengths did not provide the maximum modularity score.
To be thorough in their attempts to optimize modularity, they ran the walktrap
algorithm 50 times for each group (using random walks of lengths 1–50) and
selected the network partition with the highest modularity value from those 50.
They call this the "maximum modularity partition" and insert the parenthetical
"(though, strictly speaking, this cannot be proven to be the optimum without
computationally-prohibitive exhaustive enumeration (Brandes et al. 2008))."

So let's try and get a community classification using the walktrap algorithm, `node_in_walktrap()`,
with path lengths of the random walks specified to be 50.

```{r walk, exercise=TRUE, exercise.setup = "manip-fri"}
friend_wt <- node_in_walktrap(friends, times=50)
```

```{r walk-hint-1, purl = FALSE}
friend_wt # note that it prints pretty, but underlying its just a vector:
c(friend_wt)
```

```{r walk-hint-2, purl = FALSE}
# This says that dividing the graph into 3 communities maximises modularity,
# one with the nodes
which(friend_wt == "A")
# another with
which(friend_wt == "B")
# and the last with
which(friend_wt == "C")
```

```{r walk-hint-3, purl = FALSE}
# resulting in a modularity of
net_by_modularity(friends, friend_wt)
```

```{r walk-solution}
friend_wt <- node_in_walktrap(friends, times=50)
# results in a modularity of
net_by_modularity(friends, friend_wt)
```

We can also visualise the clusters on the original network
How does the following look? Plausible?

```{r walkplot, exercise=TRUE, exercise.setup = "walk", purl = FALSE}
```

```{r walkplot-hint-1, purl = FALSE}
# plot 1: groups by node color

friends <- friends |>
  mutate_nodes(walk_comm = friend_wt)
graphr(friends, node_color = "walk_comm")
```

```{r walkplot-hint-2, purl = FALSE}
#plot 2: groups by borders

# to be fancy, we could even draw the group borders around the nodes using the node_group argument
graphr(friends, node_group = "walk_comm")
```

```{r walkplot-hint-3, purl = FALSE}
# plot 3: group and node colors

# or both!
graphr(friends,
       node_color = "walk_comm",
       node_group = "walk_comm") +
  ggtitle("Walktrap",
    subtitle = round(net_by_modularity(friends, friend_wt), 3))
# the function `round()` rounds the values to a specified number of decimal places
# here, we are telling it to round the net_by_modularity score to 3 decimal places
```

```{r walkplot-solution}
friends <- friends |>
  mutate_nodes(walk_comm = friend_wt)
graphr(friends, node_color = "walk_comm")
# to be fancy, we could even draw the group borders around the nodes using the node_group argument
graphr(friends, node_group = "walk_comm")
# or both!
graphr(friends,
       node_color = "walk_comm",
       node_group = "walk_comm") +
  ggtitle("Walktrap",
    subtitle = round(net_by_modularity(friends, friend_wt), 3))
```

This can be helpful when polygons overlap to better identify membership
Or you can use node color and size to indicate other attributes...

The walktrap solution divides the giant component's friendships into three
communities at a modularity of about 0.41 —
comfortably positive, so these clusters hold together much more than a
random rewiring would suggest.

### Edge betweenness {#edge-betweenness}

Edge betweenness is like `r gloss("betweenness")` centrality but for ties not nodes.
The edge-betweenness score of an edge measures the number of
shortest paths from one vertex to another that go through it.

The idea of the edge-betweenness based community structure detection is that
it is likely that edges connecting separate clusters have high edge-betweenness,
as all the shortest paths from one cluster to another must traverse through them.
So if we iteratively remove the edge with the highest edge-betweenness score
we will get a hierarchical map (`r gloss("dendrogram")`) of the communities in the graph.

The following works similarly to walktrap, but no need to set a step length.

```{r eb, exercise=TRUE, exercise.setup = "manip-fri"}
friend_eb <- node_in_betweenness(friends)
friend_eb
```

How does community membership differ here from that found by walktrap?

We can see how the edge betweenness community detection method works
here: http://jfaganuk.github.io/2015/01/24/basic-network-analysis/

To visualise the result:

```{r ebplot, exercise=TRUE, exercise.setup = "eb", purl = FALSE}

```

```{r ebplot-hint-1, purl = FALSE}
# create an object

friends <- friends |>
  mutate_nodes(eb_comm = friend_eb)
```

```{r ebplot-hint-2, purl = FALSE}
# create a graph with a title and subtitle returning the modularity score

graphr(friends,
       node_color = "eb_comm",
       node_group = "eb_comm") +
  ggtitle("Edge-betweenness",
    subtitle = round(net_by_modularity(friends, friend_eb), 3))
```

```{r ebplot-solution}
friends <- friends |>
  mutate_nodes(eb_comm = friend_eb)
graphr(friends,
       node_color = "eb_comm",
       node_group = "eb_comm") +
  ggtitle("Edge-betweenness",
    subtitle = round(net_by_modularity(friends, friend_eb), 3))
```

For more on this algorithm, see M Newman and M Girvan: Finding and
evaluating community structure in networks, Physical Review E 69, 026113
(2004), https://arxiv.org/abs/cond-mat/0308217.

On this small network, cutting the bridges between clusters arrives at
essentially the same three friendship communities as chasing random walks
did — reassuring, since the two algorithms reason about
communities in quite different ways.

### Fast greedy {#fast-greedy}

This algorithm is the Clauset-Newman-Moore algorithm.
Whereas edge betweenness was `r gloss("divisive")` (top-down),
the fast greedy algorithm is `r gloss("agglomerative")` (bottom-up).

At each step, the algorithm seeks a merge that would most increase modularity.
This is very fast, but has the disadvantage of being a greedy algorithm,
so it might not produce the best overall community partitioning,
although I personally find it both useful and in many cases quite "accurate".

```{r fg, exercise=TRUE, exercise.setup = "manip-fri", purl = FALSE}

```

```{r fg-hint-1, purl = FALSE}
friend_fg <- node_in_greedy(friends)
friend_fg # Does this result in a different community partition?
net_by_modularity(friends, friend_fg) # Compare this to the edge betweenness procedure
```

```{r fg-hint-2, purl = FALSE}
# Again, we can visualise these communities in different ways:
friends <- friends |>
  mutate_nodes(fg_comm = friend_fg)
graphr(friends,
       node_color = "fg_comm",
       node_group = "fg_comm") +
  ggtitle("Fast-greedy",
    subtitle = round(net_by_modularity(friends, friend_fg), 3))
#
```

```{r fg-solution}
friend_fg <- node_in_greedy(friends)
friend_fg # Does this result in a different community partition?
net_by_modularity(friends, friend_fg) # Compare this to the edge betweenness procedure

# Again, we can visualise these communities in different ways:
friends <- friends |>
  mutate_nodes(fg_comm = friend_fg)
graphr(friends,
       node_color = "fg_comm",
       node_group = "fg_comm") +
  ggtitle("Fast-greedy",
    subtitle = round(net_by_modularity(friends, friend_fg), 3))
```

See A Clauset, MEJ Newman, C Moore:
Finding community structure in very large networks,
https://arxiv.org/abs/cond-mat/0408187

```{r comm-comp, echo=FALSE, purl = FALSE}
question("What is the difference between communities and components?",
         answer("Communities and components are just different terms for the same thing",
                message = "Not quite -- they answer different questions. Components ask whether nodes are connected at all; communities ask how densely they are connected relative to the rest of the network."),
         answer("Communities are a stricter form of component",
                message = "Not exactly -- a community need not be a component (it can have ties leaving it) and a component need not be a community, so one isn't simply a stricter version of the other."),
         answer("Components are about paths whereas communities are about the relationship between within-group and between-group ties",
                correct = TRUE,
                message = "That's right -- components are defined by reachability (is there a path?), whereas communities are defined by comparing the density of ties within a group to those between groups."),
  random_answer_order = TRUE,
         allow_retry = TRUE)
```

### Detecting communities {#detecting-communities}

Lastly, `{netrics}` includes a function to run through and find the membership
assignment that maximises modularity across any of the applicable community
detection procedures.
This is helpfully called `node_in_community()`.
This function pays attention to the size and other salient network properties,
such as whether it is directed, weighted, or connected.
It either uses the algorithm that maximises modularity by default
(`node_in_optimal()`) for smaller networks, or surveys the approximations
provided by other algorithms for the result that maximises the modularity.

```{r incomm, exercise = TRUE, exercise.setup = "manip-fri"}
node_in_community(friends)
```

It is important to name _which_ algorithm produced the membership assignment
that is then subsequently analysed though.

```{r alg-comp, echo=FALSE, purl = FALSE}
question("Which algorithm provides the 'best' membership assignment here?",
         answer("Walktrap",
                message = "Walktrap finds communities using short random walks. It's a fast heuristic, but not guaranteed to reach the highest possible modularity."),
         answer("Edge betweenness",
                message = "Edge betweenness (Girvan-Newman) removes high-betweenness ties one by one. It's intuitive but slow and only approximate."),
         answer("Leiden",
                message = "Leiden is a fast, high-quality heuristic, but on a small network it can still fall short of the exact maximum."),
         answer("Louvain",
                message = "Louvain is a popular fast greedy multi-level heuristic, but 'fast' comes at the cost of not being guaranteed optimal."),
         answer("Infomap",
                message = "Infomap optimises a description of information flow (the map equation) rather than modularity directly, so it isn't the 'best' by this modularity yardstick."),
         answer("Fluid",
                message = "Fluid communities is a heuristic that requires you to fix the number of communities in advance, so it's approximate."),
         answer("Fast Greedy",
                message = "Fast greedy maximises modularity greedily -- quick, but it can get stuck below the true maximum."),
         answer("Eigenvector",
                message = "The leading-eigenvector method is a spectral heuristic, again only an approximation to the optimum."),
         answer("Spinglass",
                message = "Spinglass uses simulated annealing, so it's stochastic and not guaranteed to land on the optimal assignment."),
         answer("Partition",
                message = "Partition splits the network structurally rather than maximising modularity, so it isn't the 'best' modularity assignment."),
         answer("Optimal",
                correct = TRUE,
                message = "That's right -- `node_in_optimal()` exhaustively maximises modularity, so on a network small enough for it to run it returns the highest possible modularity by definition. That's exactly why `node_in_community()` prefers it for smaller networks."),
  random_answer_order = TRUE,
         allow_retry = TRUE)
```

::: {.callout}
<span class="callout-label">**Going further**:</span>
The three algorithms treated here are only the start of the menagerie:
`{netrics}` also offers `node_in_louvain()` and `node_in_leiden()`
(fast multilevel modularity maximisers),
`node_in_infomap()` (information-flow based),
`node_in_spinglass()` (simulated annealing),
`node_in_fluid()` (fixed number of communities),
and `node_in_eigen()` (spectral).
See `?member_community` for the full list with definitions and references.
:::

::: {.callout}
<span class="callout-label">**In brief**:</span> Community detection algorithms cluster nodes by
tie density: `node_in_walktrap()` via random walks,
`node_in_betweenness()` divisively by cutting bridging ties,
and `node_in_greedy()` agglomeratively by modularity-improving merges.
`node_in_community()` surveys the applicable algorithms
(exhaustively via `node_in_optimal()` on small networks)
and returns the assignment that maximises modularity —
but always report which algorithm produced the result you analyse.
:::

## Free play

<img src="https://i.giphy.com/media/v1.Y2lkPTc5MGI3NjExc3Z1ZmlueXJhNzU5eHQ4eW14bHc3YWRiemdubnhkdzR5YzFvOHN4dCZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/mnLSsJi13Sgmc/giphy.gif" alt="gif of two dancing"/>

We've looked here at the `irps_blogs` dataset.
Now choose another dataset included in `{manynet}`
(browse them with `table_data()`).
What is the density? Does it make sense to investigate reciprocity,
transitivity, or equivalence? How can we interpret the results?
How many components are in the network? Is there a strong factional structure?
Which community detection algorithm returns the highest modularity score,
or corresponds best to what is in the data or what you see in the graph?

If you are not sure where to start,
here is one suggestion per flavour,
each well suited to the measures from this tutorial:

| Classic (small, tidy) | Fiction (moderate) | Real-world (larger) |
|---|---|---|
| `ison_dolphins` (62 dolphins observed associating off New Zealand — a classic community-detection benchmark: do the algorithms recover the two groups the observers saw?) | `fict_lotr` (36 Middle-Earth characters and their interactions — how modular is the fellowship around the `Race` attribute compared with detected communities?) | `irps_books` (105 co-purchased US politics books with a `Leaning` attribute — do purchasing communities align with political leaning, as the blogs did?) |

```{r freeplay, exercise = TRUE, fig.width=9}
irps_books
```

## Summary

<img src="https://media1.tenor.com/m/gSsbNTouixUAAAAC/bravo-applause.gif" alt="gif of an enthusiastic standing ovation and cries of bravo"/>

Well done -- you have completed the tutorial on cohesion and community!
Along the way, you have learned to use these functions:

| Function | What it does |
|---|---|
| `net_nodes()`, `net_ties()` | counts the nodes and ties in a network |
| `net_by_density()` | proportion of possible ties present, by the equation for the network's type |
| `net_by_reciprocity()` | how often directed ties are returned |
| `tie_is_reciprocated()` | flags each tie as reciprocated or not, for mapping with `edge_color` |
| `net_by_indegree()` | centralisation of incoming ties, to see how unequal participation is |
| `net_by_transitivity()` | how often two-paths close into triangles |
| `to_uniplex()`, `to_named()`, `to_undirected()`, `to_giant()` | extracts one tie type, adds names, drops direction, keeps the giant component |
| `as_matrix()` | coerces a network to a matrix, e.g. for manual projection |
| `to_mode1()`, `to_mode2()` | projects a two-mode network onto its row or column nodes, with a `similarity` option |
| `tie_weights()` | extracts the tie weights, e.g. of a projection |
| `net_by_equivalency()` | equivalence/reinforcement measured on the two-mode network itself |
| `net_by_components()` | number of (strong) components; wrap in `to_undirected()` for weak |
| `node_in_component()` | each node's component membership |
| `node_is_isolate()` | flags isolates (sum it to count them) |
| `delete_nodes()` | removes chosen (e.g. sampled) nodes |
| `node_in_partition()` | splits a network into two factions |
| `net_by_factions()` | how factional the network is around a (bi)partition |
| `net_by_modularity()` | fit of any membership assignment against a random baseline |
| `node_attribute()` | extracts a nodal attribute, e.g. as an empirical membership |
| `node_in_walktrap()`, `node_in_betweenness()`, `node_in_greedy()` | community detection via random walks, divisive tie removal, agglomerative merges |
| `node_in_community()` | surveys applicable algorithms and returns the best assignment by modularity |
| `mutate_nodes()`, `mutate_ties()` | adds measures or memberships to the network for graphing |
| `graphr(..., node_color = , node_group = , edge_color = )` | maps memberships onto colours and group outlines |

When you are ready, continue with the other `{netrics}` tutorials —
on position and topology —
where further ways of summarising network structure are introduced.
Run `run_tute()` at the console to see all available tutorials.

### Glossary

Here are some of the terms that we have covered in this tutorial:

`r print_glossary()`

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_centrality_closeness.R
\name{measure_centralities_close}
\alias{measure_centralities_close}
\alias{tie_by_closeness}
\title{Measuring ties closeness-like centrality}
\usage{
tie_by_closeness(.data, normalized = TRUE)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{normalized}{Logical scalar, whether scores are normalized.
Different denominators may be used depending on the measure,
whether the object is one-mode or two-mode, and other arguments.
By default TRUE.}
}
\value{
A \code{tie_measure} numeric vector the length of the ties in the network,
providing the scores for each tie.
If the network is labelled,
then the scores will be labelled with the ties' adjacent nodes' names.
}
\description{
\code{tie_by_closeness()} measures the closeness of each tie to other ties
in the network.

All measures attempt to use as much information as they are offered,
including whether the networks are directed, weighted, or multimodal.
If this would produce unintended results,
first transform the salient properties using e.g. \code{\link[manynet:to_undirected]{to_undirected()}} functions.
All centrality and centralization measures return normalized measures by default,
including for two-mode networks.
}
\examples{
(ec <- tie_by_closeness(ison_adolescents))
ison_adolescents |> mutate_ties(weight = ec)
}
\seealso{
Other closeness:
\code{\link{measure_central_close}},
\code{\link{measure_centralisation_close}}

Other centrality:
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralisation_between}},
\code{\link{measure_centralisation_close}},
\code{\link{measure_centralisation_degree}},
\code{\link{measure_centralisation_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}}

Other measures:
\code{\link{measure_assort_net}},
\code{\link{measure_assort_node}},
\code{\link{measure_breadth}},
\code{\link{measure_broker_node}},
\code{\link{measure_broker_tie}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}},
\code{\link{measure_closure}},
\code{\link{measure_closure_node}},
\code{\link{measure_cohesion}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_net}},
\code{\link{measure_diverse_node}},
\code{\link{measure_features}},
\code{\link{measure_fragmentation}},
\code{\link{measure_hierarchy}},
\code{\link{measure_periods}}

Other tie:
\code{\link{mark_dyads}},
\code{\link{mark_select_tie}},
\code{\link{mark_ties}},
\code{\link{mark_triangles}},
\code{\link{measure_broker_tie}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}}
}
\concept{centrality}
\concept{closeness}
\concept{measures}
\concept{tie}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mark_ties.R
\name{mark_ties}
\alias{mark_ties}
\alias{tie_is_loop}
\alias{tie_is_feedback}
\alias{tie_is_bridge}
\alias{tie_is_path}
\title{Marking ties based on structural properties}
\usage{
tie_is_loop(.data)

tie_is_feedback(.data)

tie_is_bridge(.data)

tie_is_path(.data, from, to, all_paths = FALSE)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{from}{The index or name of the node from which the path should start.}

\item{to}{The index or name of the node to which the path should be traced.}

\item{all_paths}{Whether to return a list of paths or sample just one.
By default FALSE, sampling just a single path.}
}
\value{
A \code{tie_mark} logical vector the length of the ties in the network,
giving either TRUE or FALSE for each tie depending on
whether the condition is matched.
}
\description{
These functions return logical vectors the length of the ties
in a network identifying which hold certain properties or positions in the network.
\itemize{
\item \code{tie_is_loop()} marks ties that are loops.
\item \code{tie_is_feedback()} marks ties that are feedback arcs causing the network to not be acyclic.
\item \code{tie_is_bridge()} marks ties that cut or act as articulation points in a network.
\item \code{tie_is_path()} marks ties on a path from one node to another.
}

They are most useful in highlighting parts of the network that
are particularly well- or poorly-connected.
}
\examples{
tie_is_loop(fict_marvel)
tie_is_feedback(ison_algebra)
tie_is_bridge(ison_brandes)
ison_adolescents |>
  mutate_ties(route = tie_is_path(from = "Jane", to = 7)) 
}
\seealso{
Other marks:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_dyads}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{mark_select_tie}},
\code{\link{mark_triangles}}

Other tie:
\code{\link{mark_dyads}},
\code{\link{mark_select_tie}},
\code{\link{mark_triangles}},
\code{\link{measure_broker_tie}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}}
}
\concept{marks}
\concept{tie}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/member_core.R
\name{measure_core}
\alias{measure_core}
\alias{node_by_kcoreness}
\alias{node_by_coreness}
\title{Measuring nodes' coreness}
\usage{
node_by_kcoreness(.data)

node_by_coreness(.data)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}
}
\value{
A \code{node_measure} numeric vector the length of the nodes in the network,
providing the scores for each node.
If the network is labelled,
then the scores will be labelled with the nodes' names.
}
\description{
These functions identify nodes belonging to (some level of) the core of a network:
\itemize{
\item \code{node_by_coreness()} returns a continuous measure of how closely each node
resembles a typical core node.
\item \code{node_by_kcoreness()} assigns nodes to their level of k-coreness.
}
}
\section{k-coreness}{

k-coreness captures the maximal subgraphs in which each vertex has at least
degree \emph{k}, where \emph{k} is also the order of the subgraph.
As described in \code{igraph::coreness},
a node's coreness is \emph{k} if it belongs to the \emph{k}-core
but not to the (\emph{k}+1)-core.
}

\examples{
node_by_kcoreness(ison_adolescents)
node_by_coreness(ison_adolescents)
}
\references{
\subsection{On k-coreness}{

Seidman, Stephen B. 1983.
"Network structure and minimum degree".
\emph{Social Networks}, 5(3), 269-287.
\doi{10.1016/0378-8733(83)90028-X}

Batagelj, Vladimir, and Matjaz Zaversnik. 2003.
"An O(m) algorithm for cores decomposition of networks".
\emph{arXiv preprint} cs/0310049.
\doi{10.48550/arXiv.cs/0310049}
}
}
\seealso{
Other core-periphery:
\code{\link{mark_core}},
\code{\link{member_core}}

Other measures:
\code{\link{measure_assort_net}},
\code{\link{measure_assort_node}},
\code{\link{measure_breadth}},
\code{\link{measure_broker_node}},
\code{\link{measure_broker_tie}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}},
\code{\link{measure_closure}},
\code{\link{measure_closure_node}},
\code{\link{measure_cohesion}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_net}},
\code{\link{measure_diverse_node}},
\code{\link{measure_features}},
\code{\link{measure_fragmentation}},
\code{\link{measure_hierarchy}},
\code{\link{measure_periods}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_node}},
\code{\link{motif_path}}
}
\concept{core-periphery}
\concept{measures}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netrics-defunct.R
\name{defunct}
\alias{defunct}
\title{Functions that have been renamed, superseded, or are no longer working}
\value{
Results as expected
along with a warning to use new function naming in the future.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}
Generally these functions have been superseded or renamed.
Upon using them, a message is provided directing the user to the new function.
However, at this stage of package development,
we generally clear older defunct functions at each minor release,
and so you are strongly encouraged to use the new functions/names/syntax
wherever possible and update your scripts accordingly.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_heterogeneity.R
\name{measure_assort_node}
\alias{measure_assort_node}
\alias{node_by_heterophily}
\alias{node_by_homophily}
\title{Measures of nodes assortativity}
\usage{
node_by_heterophily(.data, attribute)

node_by_homophily(
  .data,
  attribute,
  assortativity = c("ie", "ei", "yule", "geary")
)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{attribute}{Name of a nodal attribute, mark, measure, or membership vector.}

\item{assortativity}{Which method to use for \verb{*_homophily()}.
Either "ie" (negative E-I index), "ei" (E-I index),
"yule" (Yule's Q), or "geary" (Geary's C).
Default is "ie".
The E-I index is the number of ties between (or \emph{external}) nodes
grouped in some mutually exclusive categories
minus the number of ties within (or \emph{internal}) these groups
divided by the total number of ties.
This value can range from 1 to -1,
where 1 indicates ties only between categories/groups and -1 ties only within categories/groups.

Yule's Q is a measure of association for two binary variables, calculated as:
\deqn{Q = \frac{ad - bc}{ad + bc}}
where \eqn{a} is the number of ties between nodes in the same category,
\eqn{b} is the number of ties between nodes in different categories,
\eqn{c} is the number of non-ties between nodes in the same category,
and \eqn{d} is the number of non-ties between nodes in different categories.
This value can range from -1 to 1, where 1 indicates perfect association
(all ties are between nodes in the same category), -1 indicates perfect disassociation
(all ties are between nodes in different categories), and 0 indicates no association.

Geary's C is a measure of spatial autocorrelation, calculated as:
\deqn{C = \frac{(n - 1) \sum
\limits_{i=1}^n \sum\limits_{j=1}^n w_{ij} (x_i - x_j)^2}{2W \sum\limits_{i=1}^n (x_i - \bar{x})^2}}
where \eqn{n} is the number of nodes, \eqn{
w_{ij}} is the weight of the tie between nodes \eqn{i} and \eqn{j},
\eqn{x_i} is the attribute value of node \eqn{i},
\eqn{\bar{x}} is the mean attribute value across all nodes, and \eqn{W} is the sum of all tie weights.
This value can range from 0 to 2, where values less than 1
indicate positive autocorrelation (similar values are more likely to be connected),
values greater than 1 indicate negative autocorrelation (dissimilar values are more likely
to be connected), and a value of 1 indicates no autocorrelation.
If an incompatible method is chosen for the attribute type,
a suitable alternative will be used instead with a message.}
}
\value{
A \code{node_measure} numeric vector the length of the nodes in the network,
providing the scores for each node.
If the network is labelled,
then the scores will be labelled with the nodes' names.
}
\description{
These functions offer ways to measure nodes' assortativity in a network:
\itemize{
\item \code{node_by_heterophily()} measures each node's embeddedness within groups
of nodes with the same attribute.
\item \code{node_by_homophily()} measures each node's embeddedness within groups
of nodes with the same attribute, using a variety of methods.
}
}
\examples{
marvel_friends <- to_unsigned(to_uniplex(fict_marvel, "relationship"), "positive")
node_by_heterophily(marvel_friends, "Gender")
node_by_heterophily(marvel_friends, "Attractive")
}
\seealso{
Other diversity:
\code{\link{measure_assort_net}},
\code{\link{measure_diverse_net}},
\code{\link{measure_diverse_node}}

Other measures:
\code{\link{measure_assort_net}},
\code{\link{measure_breadth}},
\code{\link{measure_broker_node}},
\code{\link{measure_broker_tie}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}},
\code{\link{measure_closure}},
\code{\link{measure_closure_node}},
\code{\link{measure_cohesion}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_net}},
\code{\link{measure_diverse_node}},
\code{\link{measure_features}},
\code{\link{measure_fragmentation}},
\code{\link{measure_hierarchy}},
\code{\link{measure_periods}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_node}},
\code{\link{motif_path}}
}
\concept{diversity}
\concept{measures}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_centrality_eigen.R
\name{measure_central_eigen}
\alias{measure_central_eigen}
\alias{node_by_eigenvector}
\alias{node_by_power}
\alias{node_by_alpha}
\alias{node_by_pagerank}
\alias{node_by_authority}
\alias{node_by_hub}
\alias{node_by_subgraph}
\title{Measuring nodes eigenvector-like centrality}
\usage{
node_by_eigenvector(.data, normalized = TRUE, scale = TRUE)

node_by_power(.data, normalized = TRUE, scale = FALSE, exponent = 1)

node_by_alpha(.data, alpha = 0.85)

node_by_pagerank(.data)

node_by_authority(.data)

node_by_hub(.data)

node_by_subgraph(.data)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{normalized}{Logical scalar, whether scores are normalized.
Different denominators may be used depending on the measure,
whether the object is one-mode or two-mode, and other arguments.
By default TRUE.}

\item{scale}{Logical scalar, whether to rescale the vector so the maximum score is 1.}

\item{exponent}{Decay rate or attentuation factor for
the Bonacich power centrality score.
Can be positive or negative.}

\item{alpha}{A constant that trades off the importance of external influence against the importance of connection.
When \eqn{\alpha = 0}, only the external influence matters.
As \eqn{\alpha} gets larger, only the connectivity matters and we reduce to eigenvector centrality.
By default \eqn{\alpha = 0.85}.}
}
\value{
A \code{node_measure} numeric vector the length of the nodes in the network,
providing the scores for each node.
If the network is labelled,
then the scores will be labelled with the nodes' names.
}
\description{
These functions calculate common eigenvector-related centrality
measures, or walk-based eigenmeasures, for one- and two-mode networks:
\itemize{
\item \code{node_by_eigenvector()} measures the eigenvector centrality of nodes
in a network.
\item \code{node_by_power()} measures the Bonacich, beta, or power centrality of
nodes in a network.
\item \code{node_by_alpha()} measures the alpha or Katz centrality of nodes in a
network.
\item \code{node_by_pagerank()} measures the pagerank centrality of nodes in a network.
\item \code{node_by_hub()} measures how well nodes in a network serve as hubs pointing
to many authorities.
\item \code{node_by_authority()} measures how well nodes in a network serve as
authorities from many hubs.
}

All measures attempt to use as much information as they are offered,
including whether the networks are directed, weighted, or multimodal.
If this would produce unintended results,
first transform the salient properties using e.g. \code{\link[manynet:to_undirected]{to_undirected()}} functions.
All centrality and centralization measures return normalized measures
by default, including for two-mode networks.
}
\details{
We use \code{{igraph}} routines behind the scenes here for consistency and because they are often faster.
For example, \code{igraph::eigencentrality()} is approximately 25\% faster than \code{sna::evcent()}.
}
\section{Eigenvector centrality}{

Eigenvector centrality operates as a measure of a node's influence in a network.
The idea is that being connected to well-connected others results in a higher score.
Each node's eigenvector centrality can be defined as:
\deqn{x_i = \frac{1}{\lambda} \sum_{j \in N} a_{i,j} x_j}
where \eqn{a_{i,j} = 1} if \eqn{i} is linked to \eqn{j} and 0 otherwise,
and \eqn{\lambda} is a constant representing the principal eigenvalue.
Rather than performing this iteration,
most routines solve the eigenvector equation \eqn{Ax = \lambda x}.
Note that since \code{{igraph}} v2.1.1,
the values will always be rescaled so that the maximum is 1.
}

\section{Power or beta (or Bonacich) centrality}{

Power centrality includes an exponent that weights contributions to a node's
centrality based on how far away those other nodes are.
\deqn{c_b(i) = \sum A(i,j) (\alpha = \beta c(j))}
Where \eqn{\beta} is positive, this means being connected to central people
increases centrality.
Where \eqn{\beta} is negative, this means being connected to central people
decreases centrality
(and being connected to more peripheral actors increases centrality).
When \eqn{\beta = 0}, this is the outdegree.
\eqn{\alpha} is calculated to make sure the root mean square equals
the network size.
}

\section{Alpha centrality}{

Alpha or Katz (or Katz-Bonacich) centrality operates better than
eigenvector centrality for directed networks because eigenvector centrality
will return 0s for all nodes not in the main strongly-connected component.
Each node's alpha centrality can be defined as:
\deqn{x_i = \frac{1}{\lambda} \sum_{j \in N} a_{i,j} x_j + e_i}
where \eqn{a_{i,j} = 1} if \eqn{i} is linked to \eqn{j} and 0 otherwise,
\eqn{\lambda} is a constant representing the principal eigenvalue,
and \eqn{e_i} is some external influence used to ensure that even nodes beyond the main
strongly connected component begin with some basic influence.
Note that many equations replace \eqn{\frac{1}{\lambda}} with \eqn{\alpha},
hence the name.

For example, if \eqn{\alpha = 0.5}, then each direct connection (or alter) would be worth \eqn{(0.5)^1 = 0.5},
each secondary connection (or tertius) would be worth \eqn{(0.5)^2 = 0.25},
each tertiary connection would be worth \eqn{(0.5)^3 = 0.125}, and so on.

Rather than performing this iteration though,
most routines solve the equation \eqn{x = (I - \frac{1}{\lambda} A^T)^{-1} e}.
}

\section{Subgraph centrality}{

Subgraph centrality measures the participation of a node in all subgraphs
in the network, giving higher weight to smaller subgraphs.
It is defined as:
\deqn{C_S(i) = \sum_{k=0}^{\infty} \frac{(A^k)_{ii}}{k!}}
where \eqn{(A^k)_{ii}} is the \eqn{i}th diagonal element of the \eqn{k}th power
of the adjacency matrix \eqn{A}, representing the number of closed walks
of length \eqn{k} starting and ending at node \eqn{i}.
Weighting by \eqn{\frac{1}{k!}} ensures that shorter walks contribute more
to the centrality score than longer walks.

Subgraph centrality is a good choice of measure when the focus is on
local connectivity and clustering around a node,
as it captures the extent to which a node is embedded in tightly-knit
groups within the network.
Note though that because of the way spectral decomposition is used to
calculate this measure, this is not a good measure for very large graphs.
}

\examples{
node_by_eigenvector(ison_southern_women)
node_by_power(ison_southern_women, exponent = 0.5)
}
\references{
\subsection{On eigenvector centrality}{

Bonacich, Phillip. 1991.
“Simultaneous Group and Individual Centralities.”
\emph{Social Networks} 13(2):155–68.
\doi{10.1016/0378-8733(91)90018-O}
}

\subsection{On power centrality}{

Bonacich, Phillip. 1987.
“Power and Centrality: A Family of Measures.”
\emph{The American Journal of Sociology}, 92(5): 1170–82.
\doi{10.1086/228631}.
}

\subsection{On alpha centrality}{

Katz, Leo 1953.
"A new status index derived from sociometric analysis".
\emph{Psychometrika}. 18(1): 39–43.

Bonacich, P. and Lloyd, P. 2001.
“Eigenvector-like measures of centrality for asymmetric relations”
\emph{Social Networks}. 23(3):191-201.
}

\subsection{On pagerank centrality}{

Brin, Sergey and Page, Larry. 1998.
"The anatomy of a large-scale hypertextual web search engine".
\emph{Proceedings of the 7th World-Wide Web Conference}. Brisbane, Australia.
}

\subsection{On hub and authority centrality}{

Kleinberg, Jon. 1999.
"Authoritative sources in a hyperlinked environment".
\emph{Journal of the ACM} 46(5): 604–632.
\doi{10.1145/324133.324140}
}

\subsection{On subgraph centrality}{

Estrada, Ernesto and Rodríguez-Velázquez, Juan A. 2005.
"Subgraph centrality in complex networks".
\emph{Physical Review E} 71(5): 056103.
\doi{10.1103/PhysRevE.71.056103}
}
}
\seealso{
Other eigenvector:
\code{\link{measure_centralisation_eigen}},
\code{\link{measure_centralities_eigen}}

Other centrality:
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_centralisation_between}},
\code{\link{measure_centralisation_close}},
\code{\link{measure_centralisation_degree}},
\code{\link{measure_centralisation_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}}

Other measures:
\code{\link{measure_assort_net}},
\code{\link{measure_assort_node}},
\code{\link{measure_breadth}},
\code{\link{measure_broker_node}},
\code{\link{measure_broker_tie}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}},
\code{\link{measure_closure}},
\code{\link{measure_closure_node}},
\code{\link{measure_cohesion}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_net}},
\code{\link{measure_diverse_node}},
\code{\link{measure_features}},
\code{\link{measure_fragmentation}},
\code{\link{measure_hierarchy}},
\code{\link{measure_periods}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_node}},
\code{\link{motif_path}}
}
\concept{centrality}
\concept{eigenvector}
\concept{measures}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_change.R
\name{motif_periods}
\alias{motif_periods}
\alias{net_x_change}
\alias{net_x_stability}
\alias{net_x_correlation}
\title{Motifs of network change}
\usage{
net_x_change(.data, object2)

net_x_stability(.data, object2)

net_x_correlation(.data, object2)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{object2}{A network object.}
}
\value{
A \code{network_motif} named numeric vector or sometimes a data frame with
one row and a column for each motif type,
giving the count of each motif in the network.
This is printed as a tibble to avoid greedy printing of long vectors.
}
\description{
These functions measure certain topological features of networks:
\itemize{
\item \code{net_x_change()} measures the Hamming distance between two or more networks.
\item \code{net_x_stability()} measures the Jaccard index of stability between two or more networks.
\item \code{net_x_correlation()} measures the product-moment correlation between two networks.
}

These \verb{net_*()} functions return a numeric vector the length of the number
of networks minus one. E.g., the periods between waves.
}
\seealso{
Other change:
\code{\link{measure_periods}}

Other motifs:
\code{\link{motif_brokerage_net}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_hazard}},
\code{\link{motif_hierarchy}},
\code{\link{motif_net}},
\code{\link{motif_node}},
\code{\link{motif_path}}
}
\concept{change}
\concept{motifs}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_centrality_eigen.R
\name{measure_centralisation_eigen}
\alias{measure_centralisation_eigen}
\alias{net_by_eigenvector}
\alias{mode_by_eigenvector}
\title{Measuring networks eigenvector-like centralisation}
\usage{
net_by_eigenvector(.data, normalized = TRUE)

mode_by_eigenvector(.data, normalized = TRUE)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{normalized}{Logical scalar, whether scores are normalized.
Different denominators may be used depending on the measure,
whether the object is one-mode or two-mode, and other arguments.
By default TRUE.}
}
\value{
\code{net_by_eigenvector()} returns a \code{network_measure} scalar;
\code{mode_by_eigenvector()} returns a \code{mode_measure} numeric vector of length two,
giving one centralization score per mode.
}
\description{
\itemize{
\item \code{net_by_eigenvector()} measures the eigenvector centralization for a
network as a single score.
\item \code{mode_by_eigenvector()} measures eigenvector centralization separately for
each mode of a two-mode network (via projection to each mode), returning one
score per mode (following Borgatti and Everett, 1997).
}

All measures attempt to use as much information as they are offered,
including whether the networks are directed, weighted, or multimodal.
If this would produce unintended results,
first transform the salient properties using e.g. \code{\link[manynet:to_undirected]{to_undirected()}} functions.
All centrality and centralization measures return normalized measures
by default, including for two-mode networks.

For two-mode networks the two modes have different theoretical maxima, so
\code{net_by_eigenvector()} reports a single network-level score by applying
Freeman's general centralization index over the normalized node eigenvector
scores, whereas \code{mode_by_eigenvector()} reports the per-mode scores directly.
}
\examples{
net_by_eigenvector(ison_southern_women)
mode_by_eigenvector(ison_southern_women)
}
\references{
Borgatti, Stephen P., and Martin G. Everett. 1997.
"Network analysis of 2-mode data."
\emph{Social Networks} 19(3): 243-269.
\doi{10.1016/S0378-8733(96)00301-2}
}
\seealso{
Other eigenvector:
\code{\link{measure_central_eigen}},
\code{\link{measure_centralities_eigen}}

Other centrality:
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralisation_between}},
\code{\link{measure_centralisation_close}},
\code{\link{measure_centralisation_degree}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}}
}
\concept{centrality}
\concept{eigenvector}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/member_core.R
\name{member_core}
\alias{member_core}
\alias{node_in_core}
\title{Memberships in core-periphery categories}
\usage{
node_in_core(.data, groups = 3, cluster_by = c("bins", "quantiles", "kmeans"))
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{groups}{Number of categories to create. Must be at least 2 and at most
the number of nodes in the network. Default is 3.}

\item{cluster_by}{Method to use to create the categories.
One of "bins" (equal-width bins), "quantiles" (quantile-based bins),
or "kmeans" (k-means clustering). Default is "bins".}
}
\value{
A \code{node_member} character vector the length of the nodes in the network,
of group memberships "A", "B", etc for each node.
If the network is labelled,
then the assignments will be labelled with the nodes' names.
}
\description{
\code{node_in_core()} categorizes nodes into two or more core/periphery
categories based on their coreness.
}
\section{Core-periphery categories}{

This function categorizes nodes based on their coreness into a specified
number of groups. The groups are labeled as "Core", "Semi-core",
"Semi-periphery", and "Periphery" depending on the number of groups
specified.
The categorization can be done using different methods: equal-width bins,
quantile-based bins, or k-means clustering.
}

\examples{
node_in_core(ison_adolescents)
}
\references{
\subsection{On core-periphery categorization}{

Wallerstein, Immanuel. 1974.
"Dependence in an Interdependent World: The Limited Possibilities of Transformation Within the Capitalist World Economy."
\emph{African Studies Review}, 17(1), 1-26.
\doi{https://doi.org/10.2307/523574}
}
}
\seealso{
Other core-periphery:
\code{\link{mark_core}},
\code{\link{measure_core}}

Other memberships:
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_node}},
\code{\link{motif_path}}
}
\concept{core-periphery}
\concept{memberships}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/motif_census.R
\name{motif_exposure}
\alias{motif_exposure}
\alias{node_x_exposure}
\title{Motifs of nodes exposure}
\usage{
node_x_exposure(.data)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}
}
\value{
A \code{node_motif} matrix with one row for each node in the network and
a column for each motif type,
giving the count of each motif in which each node participates.
It is printed as a tibble, however, to avoid greedy printing.
If the network is labelled,
then the node names will be in a column named \code{names}.
}
\description{
\code{node_x_exposure()} produces a motif matrix of nodes' exposure to
infection/adoption by time step.
}
\examples{
node_x_exposure(play_diffusion(create_tree(12)))
}
\seealso{
Other diffusion:
\code{\link{mark_diff}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diffusion_node}},
\code{\link{member_diffusion}},
\code{\link{motif_hazard}}

Other motifs:
\code{\link{motif_brokerage_net}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_hazard}},
\code{\link{motif_hierarchy}},
\code{\link{motif_net}},
\code{\link{motif_node}},
\code{\link{motif_path}},
\code{\link{motif_periods}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_node}},
\code{\link{motif_path}}
}
\concept{diffusion}
\concept{motifs}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/motif_census.R
\name{motif_path}
\alias{motif_path}
\alias{node_x_tie}
\alias{node_x_path}
\title{Motifs of nodes pathing}
\usage{
node_x_tie(.data)

node_x_path(.data)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}
}
\value{
A \code{node_motif} matrix with one row for each node in the network and
a column for each motif type,
giving the count of each motif in which each node participates.
It is printed as a tibble, however, to avoid greedy printing.
If the network is labelled,
then the node names will be in a column named \code{names}.
}
\description{
These functions include ways to take a census of the positions of nodes
in a network:
\itemize{
\item \code{node_x_tie()} returns a census of the ties in a network.
For directed networks, out-ties and in-ties are bound together.
For multiplex networks, the various types of ties are bound together.
\item \code{node_x_path()} returns the shortest path lengths
of each node to every other node in the network.
}
}
\examples{
task_eg <- to_named(to_uniplex(ison_algebra, "tasks"))
(tie_cen <- node_x_tie(task_eg))
node_x_path(ison_adolescents)
node_x_path(ison_southern_women)
}
\references{
\subsection{On paths}{

Dijkstra, Edsger W. 1959.
"A note on two problems in connexion with graphs".
\emph{Numerische Mathematik} 1, 269-71.
\doi{10.1007/BF01386390}.

Opsahl, Tore, Filip Agneessens, and John Skvoretz. 2010.
"Node centrality in weighted networks: Generalizing degree and shortest paths".
\emph{Social Networks} 32(3): 245-51.
\doi{10.1016/j.socnet.2010.03.006}.
}
}
\seealso{
Other motifs:
\code{\link{motif_brokerage_net}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_hazard}},
\code{\link{motif_hierarchy}},
\code{\link{motif_net}},
\code{\link{motif_node}},
\code{\link{motif_periods}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_node}}
}
\concept{motifs}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mark_nodes.R
\name{mark_degree}
\alias{mark_degree}
\alias{node_is_isolate}
\alias{node_is_pendant}
\alias{node_is_universal}
\title{Marking nodes based on degree properties}
\usage{
node_is_isolate(.data)

node_is_pendant(.data)

node_is_universal(.data)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}
}
\value{
A \code{node_mark} logical vector the length of the nodes in the network,
giving either TRUE or FALSE for each node depending on
whether the condition is matched.
}
\description{
These functions return logical vectors the length of the
nodes in a network identifying which hold certain properties or positions in the network.
\itemize{
\item \code{node_is_isolate()} marks nodes that are isolates,
with neither incoming nor outgoing ties.
\item \code{node_is_pendant()} marks nodes that are pendants,
with exactly one incoming or outgoing tie.
\item \code{node_is_universal()} identifies whether nodes are adjacent to all other
nodes in the network.
}
}
\section{Universal/dominating node}{

A universal node is adjacent to all other nodes in the network.
It is also sometimes called the dominating vertex because it represents
a one-element dominating set.
A network with a universal node is called a cone, and its universal node
is called the apex of the cone.
A classic example of a cone is a star graph,
but friendship, wheel, and threshold graphs are also cones.
}

\examples{
node_is_isolate(ison_brandes)
node_is_universal(create_star(11))
}
\seealso{
Other degree:
\code{\link{measure_central_degree}},
\code{\link{measure_centralisation_degree}},
\code{\link{measure_centralities_degree}}

Other marks:
\code{\link{mark_core}},
\code{\link{mark_diff}},
\code{\link{mark_dyads}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{mark_select_tie}},
\code{\link{mark_ties}},
\code{\link{mark_triangles}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_node}},
\code{\link{motif_path}}
}
\concept{degree}
\concept{marks}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/member_community.R
\name{member_community}
\alias{member_community}
\alias{node_in_community}
\title{Memberships in communities}
\usage{
node_in_community(.data)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}
}
\value{
A \code{node_member} character vector the length of the nodes in the network,
of group memberships "A", "B", etc for each node.
If the network is labelled,
then the assignments will be labelled with the nodes' names.
}
\description{
\code{node_in_community()} runs through all available community detection algorithms
for a given network type, finds the algorithm that returns the
largest modularity score, and returns the corresponding membership
partition.
Where feasible (a small enough network), the optimal problem solving
technique is used to ensure the maximal modularity partition.
For larger networks, it identifies the applicable algorithms and
finds the algorithm that maximises modularity and
returns that membership vector.
}
\seealso{
Other community:
\code{\link{member_community_hier}},
\code{\link{member_community_non}}

Other memberships:
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_node}},
\code{\link{motif_path}}
}
\concept{community}
\concept{memberships}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/member_equivalence.R
\name{member_equivalence}
\alias{member_equivalence}
\alias{node_in_equivalence}
\alias{node_in_structural}
\alias{node_in_regular}
\alias{node_in_automorphic}
\title{Memberships in equivalent classes}
\source{
\url{https://github.com/aslez/concoR}
}
\usage{
node_in_equivalence(
  .data,
  motif,
  k = c("silhouette", "elbow", "strict"),
  cluster = c("hierarchical", "concor", "cosine"),
  distance = c("euclidean", "maximum", "manhattan", "canberra", "binary", "minkowski"),
  Kmax = 8L
)

node_in_structural(
  .data,
  k = c("silhouette", "elbow", "strict"),
  cluster = c("hierarchical", "concor", "cosine"),
  distance = c("euclidean", "maximum", "manhattan", "canberra", "binary", "minkowski"),
  Kmax = 8L
)

node_in_regular(
  .data,
  k = c("silhouette", "elbow", "strict"),
  cluster = c("hierarchical", "concor", "cosine"),
  distance = c("euclidean", "maximum", "manhattan", "canberra", "binary", "minkowski"),
  Kmax = 8L
)

node_in_automorphic(
  .data,
  k = c("silhouette", "elbow", "strict"),
  cluster = c("hierarchical", "concor", "cosine"),
  distance = c("euclidean", "maximum", "manhattan", "canberra", "binary", "minkowski"),
  Kmax = 8L
)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{motif}{A matrix returned by a \verb{node_x_*()} function.}

\item{k}{Typically a character string indicating which method
should be used to select the number of clusters to return.
By default \code{"silhouette"}, other options include \code{"elbow"} and \code{"strict"}.
\code{"strict"} returns classes with members only when strictly equivalent.
\code{"silhouette"} and \code{"elbow"} select classes based on the distance between
clusters or between nodes within a cluster.
Fewer, identifiable letters, e.g. \code{"e"} for elbow, is sufficient.
Alternatively, if \code{k} is passed an integer, e.g. \code{k = 3},
then all selection routines are skipped in favour of this number of clusters.}

\item{cluster}{Character string indicating whether clusters should be
clustered hierarchically (\code{"hierarchical"}) or
through convergence of correlations (\code{"concor"}).
Fewer, identifiable letters, e.g. \code{"c"} for CONCOR, is sufficient.}

\item{distance}{Character string indicating which distance metric
to pass on to \code{stats::dist}.
By default \code{"euclidean"}, but other options include
\code{"maximum"}, \code{"manhattan"}, \code{"canberra"}, \code{"binary"}, and \code{"minkowski"}.
Fewer, identifiable letters, e.g. \code{"e"} for Euclidean, is sufficient.}

\item{Kmax}{Integer indicating the maximum number of (k) clusters
to evaluate.
Ignored when \code{k = "strict"} or a discrete number is given for \code{k}.}
}
\value{
A \code{node_member} character vector the length of the nodes in the network,
of group memberships "A", "B", etc for each node.
If the network is labelled,
then the assignments will be labelled with the nodes' names.
}
\description{
These functions combine an appropriate \verb{node_x_*()} function
together with methods for calculating the hierarchical clusters
provided by a certain distance calculation.
\itemize{
\item \code{node_in_equivalence()} assigns nodes membership based on their equivalence
with respective to some motif/class.
The following functions call this function, together with an appropriate motif.
\item \code{node_in_structural()} assigns nodes membership based on their
having equivalent ties to the same other nodes.
\item \code{node_in_regular()} assigns nodes membership based on their
having equivalent patterns of ties.
\item \code{node_in_automorphic()} assigns nodes membership based on their
having equivalent distances to other nodes.
}

A \code{plot()} method exists for investigating the dendrogram
of the hierarchical cluster and showing the returned cluster
assignment.
}
\examples{
(nse <- node_in_structural(ison_algebra))
(nre <- node_in_regular(ison_southern_women,
  cluster = "concor"))
if(require("sna", quietly = TRUE)){
(nae <- node_in_automorphic(ison_southern_women,
  k = "elbow"))
}
}
\seealso{
Other memberships:
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_node}},
\code{\link{motif_path}}
}
\concept{memberships}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/method_k.R
\name{method_kselect}
\alias{method_kselect}
\alias{k_strict}
\alias{k_elbow}
\alias{k_silhouette}
\alias{k_gap}
\title{Methods for selecting clusters}
\usage{
k_strict(hc, .data)

k_elbow(hc, .data, motif, Kmax)

k_silhouette(hc, .data, Kmax)

k_gap(hc, motif, Kmax, sims = 100)
}
\arguments{
\item{hc}{A hierarchical clustering object.}

\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{motif}{A motif census object.}

\item{Kmax}{An integer indicating the maximum number of options to consider.
The minimum of this and the number of nodes in the network is used.}

\item{sims}{Integer of how many simulations should be generated as a
reference distribution.}
}
\value{
A single integer indicating the number of clusters to return.
}
\description{
Finding the optimal number of clusters is generally a balance between
optimal fit statistics, parsimony, and interpretability.
These functions help select the number of clusters to return from \code{hc},
some hierarchical clustering object:
\itemize{
\item \code{k_strict()} selects a number of clusters in which there is no
distance between cluster members.
\item \code{k_elbow()} selects a number of clusters in which there is
a fair trade-off between parsimony and fit according to the elbow method.
\item \code{k_silhouette()} selects a number of clusters that
maximises the silhouette score.
}

These functions are generally not user-facing but used internally
in e.g. the \verb{*_equivalence()} functions.
}
\section{Strict method}{

The strict method selects the number of clusters in which there is no
distance between cluster members.
This is a very conservative method that may be appropriate when the goal is
to identify clusters of nodes that are exactly the same.
However, it may not be appropriate in cases where the data is noisy or
when the clusters are not well-defined,
as it may result in a large number of small clusters.
}

\section{Elbow method}{

The elbow method is a heuristic used in cluster analysis to determine the optimal number of clusters.
It is based on the idea of plotting the within cluster correlation
as a function of the number of clusters and looking for an "elbow"
where there is a significant decrease in the rate of improvement in
correlation as the number of clusters increases.
The point at which the elbow occurs is often considered a good choice for
the number  of clusters, as it represents a balance between
model complexity and fit to the data.
}

\section{Silhouette method}{

The silhouette method is based on the concept of cohesion and separation.
Cohesion refers to how closely related the nodes within a cluster are,
while separation refers to how distinct the clusters are from each other.
The silhouette score combines these two concepts into a single metric that
can be used to evaluate the quality of a clustering solution.
The silhouette score is calculated as follows:
For each node, calculate the average distance to all other nodes in the same cluster (a)
and the average distance to all other nodes in the next nearest cluster (b).
The silhouette score for each node is then calculated as:
\deqn{S(i) = \frac{b - a}{\max(a, b)}}
A higher silhouette score indicates that the node is well-matched to its
own cluster and poorly matched to neighboring clusters.
The silhouette score for the entire clustering is the average silhouette score across all nodes.
Maximizing the silhouette score across a range of potential clusterings
allows researchers to identify the number of clusters that best captures
the underlying structure of the data.
It is particularly useful when the clusters are well-separated.
}

\references{
\subsection{On the elbow method}{

Thorndike, Robert L. 1953.
"Who Belongs in the Family?".
\emph{Psychometrika}, 18(4): 267–76.
\doi{10.1007/BF02289263}.
}

\subsection{On the silhouette method}{

Rousseeuw, Peter J. 1987.
“Silhouettes: A Graphical Aid to the Interpretation and Validation of Cluster Analysis.”
\emph{Journal of Computational and Applied Mathematics}, 20: 53–65.
\doi{10.1016/0377-0427(87)90125-7}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/member_cliques.R
\name{member_cliques}
\alias{member_cliques}
\alias{node_in_roulette}
\title{Memberships in maximally diverse cliques}
\usage{
node_in_roulette(.data, num_groups, group_size, times = NULL)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{num_groups}{An integer indicating the number of groups desired.}

\item{group_size}{An integer indicating the desired size of most of the groups.
Note that if the number of nodes is not divisible into groups of equal size,
there may be some larger or smaller groups.}

\item{times}{An integer of the number of search iterations the algorithm should complete.
By default this is the number of nodes in the network multiplied by the number of groups.
This heuristic may be insufficient for small networks and numbers of groups,
and burdensome for large networks and numbers of groups, but can be overwritten.
At every 10th iteration, a stronger perturbation of a number of successive changes,
approximately the number of nodes divided by the number of groups,
will take place irrespective of whether it improves the objective function.}
}
\value{
A \code{node_member} character vector the length of the nodes in the network,
of group memberships "A", "B", etc for each node.
If the network is labelled,
then the assignments will be labelled with the nodes' names.
}
\description{
These functions create a vector of nodes' memberships in
cliques:
\itemize{
\item \code{node_in_roulette()} assigns nodes to maximally diverse groups.
}
}
\section{Maximally diverse grouping problem}{

This well known computational problem is a NP-hard problem
with a number of relevant applications,
including the formation of groups of students that have encountered
each other least or least recently.
Essentially, the aim is to return a membership of nodes in cliques
that minimises the sum of their previous (weighted) ties:

\deqn{\sum_{g=1}^{m} \sum_{i=1}^{n-1} \sum_{j=i+1}^{n} x_{ij} y_{ig} y_{jg}}

where \eqn{y_{ig} = 1} if node \eqn{i} is in group \eqn{g}, and 0 otherwise.

\eqn{x_{ij}} is the existing network data.
If this is an empty network, the function will just return cliques.
To run this repeatedly, one can join a clique network of the membership result
with the original network, using this as the network data for the next round.

A form of the Lai and Hao (2016) iterated maxima search (IMS) is used here.
This performs well for small and moderately sized networks.
It includes both weak and strong perturbations to an initial solution
to ensure that a robust solution from the broader state space is identified.
The user is referred to Lai and Hao (2016) and Lai et al (2021) for more details.
}

\references{
\subsection{On the maximally diverse grouping problem}{

Lai, Xiangjing, and Jin-Kao Hao. 2016.
“Iterated Maxima Search for the Maximally Diverse Grouping Problem.”
\emph{European Journal of Operational Research} 254(3):780–800.
\doi{10.1016/j.ejor.2016.05.018}.

Lai, Xiangjing, Jin-Kao Hao, Zhang-Hua Fu, and Dong Yue. 2021.
“Neighborhood Decomposition Based Variable Neighborhood Search and Tabu Search for Maximally Diverse Grouping.”
\emph{European Journal of Operational Research} 289(3):1067–86.
\doi{10.1016/j.ejor.2020.07.048}.
}
}
\seealso{
Other memberships:
\code{\link{member_brokerage}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_node}},
\code{\link{motif_path}}
}
\concept{memberships}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_features.R
\name{measure_features}
\alias{measure_features}
\alias{net_by_core}
\alias{net_by_richclub}
\alias{net_by_factions}
\alias{net_by_modularity}
\alias{net_by_smallworld}
\alias{net_by_scalefree}
\alias{net_by_balance}
\title{Measuring network topological features}
\source{
\code{{signnet}} by David Schoch
}
\usage{
net_by_core(
  .data,
  mark = NULL,
  method = c("correlation", "ident", "ndiff", "diff")
)

net_by_richclub(.data)

net_by_factions(.data, membership = NULL)

net_by_modularity(.data, membership = NULL, resolution = 1)

net_by_smallworld(.data, method = c("omega", "sigma", "SWI"), times = 100)

net_by_scalefree(.data)

net_by_balance(.data)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{mark}{A logical vector indicating which nodes belong to the core.}

\item{method}{There are three small-world measures implemented:
\itemize{
\item "sigma" is the original equation from Watts and Strogatz (1998),
\deqn{\frac{\frac{C}{C_r}}{\frac{L}{L_r}}},
where \eqn{C} and \eqn{L} are the observed
clustering coefficient and path length, respectively,
and \eqn{C_r} and \eqn{L_r} are the averages obtained from
random networks of the same dimensions and density.
A \eqn{\sigma > 1} is considered to be small-world,
but this measure is highly sensitive to network size.
\item "omega" (the default) is an update from Telesford et al. (2011),
\deqn{\frac{L_r}{L} - \frac{C}{C_l}},
where \eqn{C_l} is the clustering coefficient for a lattice graph
with the same dimensions.
\eqn{\omega} ranges between 0 and 1,
where 1 is as close to a small-world as possible.
\item "SWI" is an alternative proposed by Neal (2017),
\deqn{\frac{L - L_l}{L_r - L_l} \times \frac{C - C_r}{C_l - C_r}},
where \eqn{L_l} is the average path length for a lattice graph
with the same dimensions.
\eqn{SWI} also ranges between 0 and 1 with the same interpretation,
but where there may not be a network for which \eqn{SWI = 1}.
}}

\item{membership}{A character string naming an existing node attribute in
the network, or a categorical vector of the same length as the number of
nodes in the network where each element indicates the group membership of
the corresponding node.
While this may often be a vector created using \verb{node_in_*()} functions,
it can be any character vector that assigns nodes to groups or categories.}

\item{resolution}{A proportion indicating the resolution scale.
By default 1, which returns the original definition of modularity.
The higher this parameter, the more smaller communities will be privileged.
The lower this parameter, the fewer larger communities are likely to be found.}

\item{times}{Integer of number of simulations.}
}
\value{
A \code{network_measure} numeric score.
}
\description{
These functions measure certain topological features of networks:
\itemize{
\item \code{net_by_core()} measures the correlation between a network
and a core-periphery model with the same dimensions.
\item \code{net_by_richclub()} measures the rich-club coefficient of a network.
\item \code{net_by_factions()} measures the correlation between a network
and a component model with the same dimensions.
If no 'membership' vector is given for the data,
\code{node_partition()} is used to partition nodes into two groups.
\item \code{net_by_modularity()} measures the modularity of a network
based on nodes' membership in defined clusters.
\item \code{net_by_smallworld()} measures the small-world coefficient for one- or
two-mode networks. Small-world networks can be highly clustered and yet
have short path lengths.
\item \code{net_by_scalefree()} measures the exponent of a fitted
power-law distribution. An exponent between 2 and 3 usually indicates
a power-law distribution.
\item \code{net_by_balance()} measures the structural balance index on
the proportion of balanced triangles,
ranging between \code{0} if all triangles are imbalanced and
\code{1} if all triangles are balanced.
}
}
\section{Core-Periphery}{

\code{net_core()} calculates the Pearson correlation between the given network,
where the nodes in the core are assigned by some given mark, and an ideal
typical core-periphery network with the same number of nodes in the core
and the periphery.
}

\section{Modularity}{

Modularity measures the difference between the number of ties within each community
from the number of ties expected within each community in a random graph
with the same degrees, and ranges between -1 and +1.
Modularity scores of +1 mean that ties only appear within communities,
while -1 would mean that ties only appear between communities.
A score of 0 would mean that ties are half within and half between communities,
as one would expect in a random graph.

Modularity faces a difficult problem known as the resolution limit
(Fortunato and Barthélemy 2007).
This problem appears when optimising modularity,
particularly with large networks or depending on the degree of interconnectedness,
can miss small clusters that 'hide' inside larger clusters.
In the extreme case, this can be where they are only connected
to the rest of the network through a single tie.
To help manage this problem, a \code{resolution} parameter is added.
Please see the argument definition for more details.
}

\examples{
net_by_core(ison_adolescents)
net_by_core(ison_southern_women)
net_by_richclub(ison_adolescents)
  net_by_factions(ison_southern_women)
net_by_modularity(ison_adolescents, 
  node_in_partition(ison_adolescents))
net_by_modularity(ison_southern_women, 
  node_in_partition(ison_southern_women))
net_by_smallworld(ison_brandes)
net_by_smallworld(ison_southern_women)
net_by_scalefree(ison_adolescents)
net_by_scalefree(generate_scalefree(50, 1.5))
net_by_scalefree(create_lattice(100))
net_by_balance(to_uniplex(fict_marvel, "relationship"))
}
\references{
\subsection{On core-periphery}{

Borgatti, Stephen P., and Martin G. Everett. 2000.
“Models of Core/Periphery Structures.”
\emph{Social Networks} 21(4):375–95.
\doi{10.1016/S0378-8733(99)00019-2}
}

\subsection{On the rich-club coefficient}{

Zhou, Shi, and Raul J. Mondragon. 2004.
"The Rich-Club Phenomenon in the Internet Topology".
\emph{IEEE Communications Letters}, 8(3): 180-182.
\doi{10.1109/lcomm.2004.823426}
}

\subsection{On modularity}{

Newman, Mark E.J. 2006.
"Modularity and community structure in networks",
\emph{Proceedings of the National Academy of Sciences} 103(23): 8577-8696.
\doi{https://doi.org/10.1073/pnas.0601602103}

Murata, Tsuyoshi. 2010.
"Modularity for Bipartite Networks".
In: Memon, N., Xu, J., Hicks, D., Chen, H. (eds)
\emph{Data Mining for Social Network Data. Annals of Information Systems}, Vol 12.
Springer, Boston, MA.
\doi{10.1007/978-1-4419-6287-4_7}
}

\subsection{On small-worldliness}{

Watts, Duncan J., and Steven H. Strogatz. 1998.
“Collective Dynamics of ‘Small-World’ Networks”.
\emph{Nature} 393(6684):440–42.
\doi{10.1038/30918}

Telesford QK, Joyce KE, Hayasaka S, Burdette JH, Laurienti PJ. 2011.
"The ubiquity of small-world networks".
\emph{Brain Connectivity} 1(5): 367–75.
\doi{10.1089/brain.2011.0038}

Neal, Zachary P. 2017.
"How small is it? Comparing indices of small worldliness".
\emph{Network Science}. 5 (1): 30–44.
\doi{10.1017/nws.2017.5}
}

\subsection{On scale-free networks}{

Barabasi, Albert-Laszlo, and Reka Albert. 1999.
"Emergence of scaling in random networks",
\emph{Science}, 286(5439): 509-512.
\doi{10.1126/science.286.5439.509}

Clauset, Aaron, Cosma Rohilla Shalizi, and Mark E.J. Newman. 2009.
"Power-law distributions in empirical data",
\emph{SIAM Review}, 51(4): 661-703.
\doi{10.1137/070710111}

Stumpf, Michael P.H., and Mason Porter. 2012.
"Critical truths about power laws",
\emph{Science}, 335(6069): 665-666.
\doi{10.1126/science.1216142}

Holme, Petter. 2019.
"Rare and everywhere: Perspectives on scale-free networks",
\emph{Nature Communications}, 10(1): 1016.
\doi{10.1038/s41467-019-09038-8}
}

\subsection{On balance theory}{

Heider, Fritz. 1946.
"Attitudes and cognitive organization".
\emph{The Journal of Psychology}, 21: 107-112.
\doi{10.1080/00223980.1946.9917275}

Cartwright, D., and Frank Harary. 1956.
"Structural balance: A generalization of Heider's theory".
\emph{Psychological Review}, 63(5): 277-293.
\doi{10.1037/h0046049}
}
}
\seealso{
\code{\link[=net_by_transitivity]{net_by_transitivity()}} and \code{\link[=net_by_equivalency]{net_by_equivalency()}}
for how clustering is calculated

Other measures:
\code{\link{measure_assort_net}},
\code{\link{measure_assort_node}},
\code{\link{measure_breadth}},
\code{\link{measure_broker_node}},
\code{\link{measure_broker_tie}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}},
\code{\link{measure_closure}},
\code{\link{measure_closure_node}},
\code{\link{measure_cohesion}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_net}},
\code{\link{measure_diverse_node}},
\code{\link{measure_fragmentation}},
\code{\link{measure_hierarchy}},
\code{\link{measure_periods}}
}
\concept{measures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_diffusion.R
\name{measure_diffusion_node}
\alias{measure_diffusion_node}
\alias{node_by_adopt_time}
\alias{node_by_adopt_threshold}
\alias{node_by_adopt_recovery}
\alias{node_by_adopt_exposure}
\title{Measures of nodes in a diffusion}
\usage{
node_by_adopt_time(.data)

node_by_adopt_threshold(.data, normalized = TRUE, lag = 1)

node_by_adopt_recovery(.data)

node_by_adopt_exposure(.data, mark, time = 0)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{normalized}{Logical scalar, whether scores are normalized.
Different denominators may be used depending on the measure,
whether the object is one-mode or two-mode, and other arguments.
By default TRUE.}

\item{lag}{The number of time steps back upon which the thresholds are
inferred.}

\item{mark}{A valid 'node_mark' object or
logical vector (TRUE/FALSE) of length equal to
the number of nodes in the network.}

\item{time}{A time point until which infections/adoptions should be
identified. By default \code{time = 0}.}
}
\value{
A \code{node_measure} numeric vector the length of the nodes in the network,
providing the scores for each node.
If the network is labelled,
then the scores will be labelled with the nodes' names.
}
\description{
These functions allow measurement of various features of
a diffusion process:
\itemize{
\item \code{node_by_adopt_time()}: Measures the number of time steps until
nodes adopt/become infected
\item \code{node_by_adopt_threshold()}: Measures nodes' thresholds from the amount
of exposure they had when they became infected
\item \code{node_by_adopt_recovery()}: Measures the average length nodes that become
infected remain infected in a compartmental model with recovery
\item \code{node_by_adopt_exposure()}: Measures how many exposures nodes have to
a given mark
}
}
\section{Adoption time}{

\code{node_by_adopt_time()} measures the time units it took
until each node became infected.
Note that an adoption time of 0 indicates that this was a seed node.
}

\section{Thresholds}{

\code{node_by_adopt_threshold()} infers nodes' thresholds based on how much
exposure they had when they were infected.
This inference is of course imperfect,
especially where there is a sudden increase in exposure,
but it can be used heuristically.
In a threshold model,
nodes activate when \eqn{\sum_{j:\text{active}} w_{ji} \geq \theta_i},
where \eqn{w} is some (potentially weighted) matrix,
\eqn{j} are some already activated nodes,
and \eqn{theta} is some pre-defined threshold value.
Where a fractional threshold is used, the equation is
\eqn{\frac{\sum_{j:\text{active}} w_{ji}}{\sum_{j} w_{ji}} \geq \theta_i}.
That is, \eqn{theta} is now a proportion,
and works regardless of whether \eqn{w} is weighted or not.
}

\section{Recovery}{

\code{node_by_adopt_recovery()} measures the average length of time that nodes
that become infected remain infected in a compartmental model with recovery.
Infections that are not concluded by the end of the study period are
calculated as infinite.
}

\section{Exposure}{

\code{node_exposure()} calculates the number of infected/adopting nodes
to which each susceptible node is exposed.
It usually expects network data and
an index or mark (TRUE/FALSE) vector of those nodes which are currently infected,
but if a diff_model is supplied instead it will return
nodes exposure at \eqn{t = 0}.
}

\examples{
  smeg <- generate_smallworld(15, 0.025)
  smeg_diff <- play_diffusion(smeg, recovery = 0.2)
  plot(smeg_diff)
  # To measure when nodes adopted a diffusion/were infected
  (times <- node_by_adopt_time(smeg_diff))
  # To infer nodes' thresholds
  node_by_adopt_threshold(smeg_diff)
  # To measure how long each node remains infected for
  node_by_adopt_recovery(smeg_diff)
  # To measure how much exposure nodes have to a given mark
  node_by_adopt_exposure(smeg, mark = c(1,3))
  node_by_adopt_exposure(smeg_diff)
}
\references{
\subsection{On diffusion measures}{

Valente, Tom W. 1995. \emph{Network models of the diffusion of innovations}
(2nd ed.). Cresskill N.J.: Hampton Press.
}
}
\seealso{
Other diffusion:
\code{\link{mark_diff}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_net}},
\code{\link{member_diffusion}},
\code{\link{motif_exposure}},
\code{\link{motif_hazard}}

Other measures:
\code{\link{measure_assort_net}},
\code{\link{measure_assort_node}},
\code{\link{measure_breadth}},
\code{\link{measure_broker_node}},
\code{\link{measure_broker_tie}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}},
\code{\link{measure_closure}},
\code{\link{measure_closure_node}},
\code{\link{measure_cohesion}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diverse_net}},
\code{\link{measure_diverse_node}},
\code{\link{measure_features}},
\code{\link{measure_fragmentation}},
\code{\link{measure_hierarchy}},
\code{\link{measure_periods}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_node}},
\code{\link{motif_path}}
}
\concept{diffusion}
\concept{measures}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_closure.R
\name{measure_closure}
\alias{measure_closure}
\alias{net_by_reciprocity}
\alias{net_by_transitivity}
\alias{net_by_equivalency}
\alias{net_by_congruency}
\title{Measuring network closure}
\usage{
net_by_reciprocity(.data, method = "default")

net_by_transitivity(.data)

net_by_equivalency(.data)

net_by_congruency(.data, object2)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{method}{For reciprocity, either \code{default} or \code{ratio}.
See \code{?igraph::reciprocity}}

\item{object2}{Optionally, a second (two-mode) matrix, igraph, or tidygraph}
}
\value{
A \code{network_measure} numeric score.
}
\description{
These functions offer methods for summarising the closure in configurations
in one-, two-, and three-mode networks:
\itemize{
\item \code{net_by_reciprocity()} measures reciprocity in a (usually directed) network.
\item \code{net_by_transitivity()} measures transitivity in a network.
\item \code{net_by_equivalency()} measures equivalence or reinforcement
in a (usually two-mode) network.
\item \code{net_by_congruency()} measures congruency across two two-mode networks.
}
}
\details{
For one-mode networks, shallow wrappers of igraph versions exist via
\code{net_reciprocity} and \code{net_transitivity}.

For two-mode networks, \code{net_equivalency} calculates the proportion of three-paths in the network
that are closed by fourth tie to establish a "shared four-cycle" structure.

For three-mode networks, \code{net_congruency} calculates the proportion of three-paths
spanning two two-mode networks that are closed by a fourth tie to establish a
"congruent four-cycle" structure.
}
\section{Equivalency}{

The \code{net_by_equivalency()} function calculates the Robins and Alexander (2004)
clustering coefficient for two-mode networks.
Note that for weighted two-mode networks, the result is divided by the average tie weight.
}

\examples{
net_by_reciprocity(ison_southern_women)
net_by_transitivity(ison_adolescents)
net_by_equivalency(ison_southern_women)
}
\references{
\subsection{On equivalency or four-cycles}{

Robins, Garry L, and Malcolm Alexander. 2004.
Small worlds among interlocking directors: Network structure and distance in bipartite graphs.
\emph{Computational & Mathematical Organization Theory} 10(1): 69–94.
\doi{10.1023/B:CMOT.0000032580.12184.c0}.
}

\subsection{On congruency}{

Knoke, David, Mario Diani, James Hollway, and Dimitris C Christopoulos. 2021.
\emph{Multimodal Political Networks}.
Cambridge University Press. Cambridge University Press.
\doi{10.1017/9781108985000}
}
}
\seealso{
Other measures:
\code{\link{measure_assort_net}},
\code{\link{measure_assort_node}},
\code{\link{measure_breadth}},
\code{\link{measure_broker_node}},
\code{\link{measure_broker_tie}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_cohesion}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_net}},
\code{\link{measure_diverse_node}},
\code{\link{measure_features}},
\code{\link{measure_fragmentation}},
\code{\link{measure_hierarchy}},
\code{\link{measure_periods}}
}
\concept{measures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mark_ties.R
\name{mark_select_tie}
\alias{mark_select_tie}
\alias{tie_is_random}
\alias{tie_is_max}
\alias{tie_is_min}
\title{Marking ties based on measures}
\usage{
tie_is_random(.data, select = 1)

tie_is_max(tie_measure)

tie_is_min(tie_measure)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{select}{Number of elements to select (as TRUE).}

\item{tie_measure}{An object created by a \code{tie_} measure.}
}
\value{
A \code{tie_mark} logical vector the length of the ties in the network,
giving either TRUE or FALSE for each tie depending on
whether the condition is matched.
}
\description{
These functions return logical vectors the length of the ties in a network:
\itemize{
\item \code{tie_is_random()} marks one or more ties at random.
\item \code{tie_is_max()} and \code{tie_is_min()} are more useful
for converting the results from some tie measure into a mark-class object.
They can be particularly useful for highlighting which tie or ties
are key because they minimise or, more often, maximise some measure.
}
}
\examples{
tie_is_max(tie_by_betweenness(ison_brandes))
tie_is_min(tie_by_betweenness(ison_brandes))
}
\seealso{
Other marks:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_dyads}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{mark_ties}},
\code{\link{mark_triangles}}

Other tie:
\code{\link{mark_dyads}},
\code{\link{mark_ties}},
\code{\link{mark_triangles}},
\code{\link{measure_broker_tie}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}}

Other selection:
\code{\link{mark_select_node}}
}
\concept{marks}
\concept{selection}
\concept{tie}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/motif_census.R
\name{motif_hazard}
\alias{motif_hazard}
\alias{net_x_hazard}
\title{Motifs of network hazard}
\source{
\code{{netdiffuseR}}
}
\usage{
net_x_hazard(.data)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}
}
\value{
A \code{network_motif} named numeric vector or sometimes a data frame with
one row and a column for each motif type,
giving the count of each motif in the network.
This is printed as a tibble to avoid greedy printing of long vectors.
}
\description{
\code{net_x_hazard()} measures the hazard rate or instantaneous probability that
nodes will adopt/become infected at that time.
}
\section{Hazard rate}{

The hazard rate is the instantaneous probability of adoption/infection
at each time point (Allison 1984).
In survival analysis, hazard rate is formally defined as:

\deqn{%
\lambda(t)=\lim_{h\to +0}\frac{F(t+h)-F(t)}{h}\frac{1}{1-F(t)} %
}{%
\lambda(t-1)= lim (t -> +0) [F(t+h)-F(t)]/h * 1/[1-F(t)] %
}

By approximating \eqn{h=1}, we can rewrite the equation as

\deqn{%
\lambda(t)=\frac{F(t+1)-F(t)}{1-F(t)} %
}{%
\lambda(t-1)= [F(t+1)-F(t)]/[1-F(t)] %
}

If we estimate \eqn{F(t)},
the probability of not having adopted the innovation in time \eqn{t},
from the proportion of adopters in that time,
such that \eqn{F(t) \sim q_t/n}{F(t) ~ q(t)/n}, we now have (ultimately for \eqn{t>1}):

\deqn{%
\lambda(t)=\frac{q_{t+1}/n-q_t/n}{1-q_t/n} = \frac{q_{t+1} - q_t}{n - q_t} = \frac{q_t - q_{t-1}}{n - q_{t-1}} %
}{%
\lambda(t-1)= [q(t+1)/n-q(t)/n]/[1-q(t)/n] = [q(t+1) - q(t)]/[n - q(t)] = [q(t) - q(t-1)]/[n - q(t-1)] %
}

where \eqn{q_i}{q(i)} is the number of adopters in time \eqn{t},
and \eqn{n} is the number of vertices in the graph.

The shape of the hazard rate indicates the pattern of new adopters over time.
Rapid diffusion with convex cumulative adoption curves will have
hazard functions that peak early and decay over time.
Slow concave cumulative adoption curves will have
hazard functions that are low early and rise over time.
Smooth hazard curves indicate constant adoption whereas
those that oscillate indicate variability in adoption behavior over time.
}

\examples{
# To calculate the hazard rates at each time point
  smeg <- generate_smallworld(15, 0.025)
net_x_hazard(play_diffusion(smeg, transmissibility = 0.3))
}
\references{
\subsection{On hazard rates}{

Allison, Paul D. 1984.
\emph{Event history analysis: Regression for longitudinal event data}.
London: Sage Publications.
\doi{10.4135/9781412984195}

Wooldridge, Jeffrey M. 2010.
\emph{Econometric Analysis of Cross Section and Panel Data} (2nd ed.).
Cambridge: MIT Press.
}
}
\seealso{
Other diffusion:
\code{\link{mark_diff}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diffusion_node}},
\code{\link{member_diffusion}},
\code{\link{motif_exposure}}

Other motifs:
\code{\link{motif_brokerage_net}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_hierarchy}},
\code{\link{motif_net}},
\code{\link{motif_node}},
\code{\link{motif_path}},
\code{\link{motif_periods}}
}
\concept{diffusion}
\concept{motifs}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_heterogeneity.R
\name{measure_diverse_net}
\alias{measure_diverse_net}
\alias{net_by_richness}
\alias{net_by_diversity}
\title{Measures of network diversity}
\usage{
net_by_richness(.data, attribute)

net_by_diversity(
  .data,
  attribute,
  diversity = c("blau", "teachman", "variation", "gini")
)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{attribute}{Name of a nodal attribute, mark, measure, or membership vector.}

\item{diversity}{Which method to use for \verb{*_diversity()}.
Either "blau" (Blau's index) or "teachman" (Teachman's index) for categorical attributes,
or "variation" (coefficient of variation) or "gini" (Gini coefficient) for numeric attributes.
Default is "blau".
If an incompatible method is chosen for the attribute type,
a suitable alternative will be used instead with a message.}
}
\value{
A \code{network_measure} numeric score.
}
\description{
These functions offer ways to measure the heterogeneity of an attribute
across a network, within groups of a network, or the distribution of ties
across this attribute:
\itemize{
\item \code{net_by_richness()} measures the number of unique categories
in a network attribute.
\item \code{net_by_diversity()} measures the heterogeneity of ties across a network.
}
}
\section{Richness}{

Richness is a simple count of the number of different categories
present for a given attribute.
}

\section{Diversity}{

Blau's index (1977) uses a formula known also in other disciplines
by other names
(Gini-Simpson Index, Gini impurity, Gini's diversity index,
Gibbs-Martin index, and probability of interspecific encounter (PIE)):
\deqn{1 - \sum\limits_{i = 1}^k {p_i^2 }}
where \eqn{p_i} is the proportion of group members in \eqn{i}th category
and \eqn{k} is the number of categories for an attribute of interest.
This index can be interpreted as the probability that two members
randomly selected from a group would be from different categories.
This index finds its minimum value (0) when there is no variety,
i.e. when all individuals are classified in the same category.
The maximum value depends on the number of categories and
whether nodes can be evenly distributed across categories.

Teachman's index (1980) is based on information theory
and is calculated as:
\deqn{- \sum\limits_{i = 1}^k {p_i \log(p_i)}}
where \eqn{p_i} is the proportion of group members in \eqn{i}th category
and \eqn{k} is the number of categories for an attribute of interest.
This index finds its minimum value (0) when there is no variety,
i.e. when all individuals are classified in the same category.
The maximum value depends on the number of categories and
whether nodes can be evenly distributed across categories.
It thus shares similar properties to Blau's index,
but includes also a notion of richness that tends to give more weight to
rare categories and thus tends to highlight imbalances more.

The coefficient of variation (CV) is a standardised measure of dispersion
of a probability distribution or frequency distribution.
It is defined as the ratio of the standard deviation \eqn{\sigma}
to the mean \eqn{\mu}:
\deqn{CV = \frac{\sigma}{\mu}}
It is often expressed as a percentage.
The CV is useful because the standard deviation of data must always be understood
in the context of the mean of the data.
The CV is particularly useful when comparing the degree of variation
from one data series to another,
even if the means are drastically different from each other.

The Gini coefficient is a measure of statistical dispersion
that is intended to represent the income or wealth distribution
of a nation's residents,
and is commonly used as a measure of inequality.
It is defined as a ratio with values between 0 and 1,
where 0 corresponds with perfect equality
(everyone has the same income) and 1 corresponds with perfect inequality
(one person has all the income, and everyone else has zero income).
The Gini coefficient can be calculated from the Lorenz curve,
which plots the proportion of the total income of the population
that is cumulatively earned by the bottom x\% of the population.
The Gini coefficient is defined as the area between the line of equality
and the Lorenz curve,
divided by the total area under the line of equality.
}

\examples{
net_by_richness(ison_networkers)
marvel_friends <- to_unsigned(to_uniplex(fict_marvel, "relationship"), "positive")
net_by_diversity(marvel_friends, "Gender")
net_by_diversity(marvel_friends, "Appearances")
}
\references{
\subsection{On richness}{

Magurran, Anne E. 1988.
\emph{Ecological Diversity and Its Measurement}.
Princeton: Princeton University Press.
\doi{10.1007/978-94-015-7358-0}
}

\subsection{On diversity}{

Blau, Peter M. 1977.
\emph{Inequality and heterogeneity}.
New York: Free Press.

Teachman, Jay D. 1980.
Analysis of population diversity: Measures of qualitative variation.
\emph{Sociological Methods & Research}, 8:341-362.
\doi{10.1177/004912418000800305}

Page, Scott E. 2010.
\emph{Diversity and Complexity}.
Princeton: Princeton University Press.
\doi{10.1515/9781400835140}
}
}
\seealso{
Other diversity:
\code{\link{measure_assort_net}},
\code{\link{measure_assort_node}},
\code{\link{measure_diverse_node}}

Other measures:
\code{\link{measure_assort_net}},
\code{\link{measure_assort_node}},
\code{\link{measure_breadth}},
\code{\link{measure_broker_node}},
\code{\link{measure_broker_tie}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}},
\code{\link{measure_closure}},
\code{\link{measure_closure_node}},
\code{\link{measure_cohesion}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{measure_features}},
\code{\link{measure_fragmentation}},
\code{\link{measure_hierarchy}},
\code{\link{measure_periods}}
}
\concept{diversity}
\concept{measures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/motif_brokerage.R
\name{member_brokerage}
\alias{member_brokerage}
\alias{node_in_brokering}
\title{Memberships in brokerage positions}
\usage{
node_in_brokering(.data, membership)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{membership}{A character string naming an existing node attribute in
the network, or a categorical vector of the same length as the number of
nodes in the network where each element indicates the group membership of
the corresponding node.
While this may often be a vector created using \verb{node_in_*()} functions,
it can be any character vector that assigns nodes to groups or categories.}
}
\value{
A \code{node_member} character vector the length of the nodes in the network,
of group memberships "A", "B", etc for each node.
If the network is labelled,
then the assignments will be labelled with the nodes' names.
}
\description{
\code{node_in_brokerage()} returns nodes membership as a powerhouse,
connector, linchpin, or sideliner according to Hamilton et al. (2020).
}
\references{
\subsection{On brokerage activity and exclusivity}{

Hamilton, Matthew, Jacob Hileman, and Orjan Bodin. 2020.
"Evaluating heterogeneous brokerage: New conceptual and methodological approaches
and their application to multi-level environmental governance networks"
\emph{Social Networks} 61: 1-10.
\doi{10.1016/j.socnet.2019.08.002}
}
}
\seealso{
Other brokerage:
\code{\link{measure_broker_node}},
\code{\link{measure_broker_tie}},
\code{\link{measure_brokerage}},
\code{\link{motif_brokerage_net}},
\code{\link{motif_brokerage_node}}

Other memberships:
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_node}},
\code{\link{motif_path}}
}
\concept{brokerage}
\concept{memberships}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_centrality_degree.R
\name{measure_central_degree}
\alias{measure_central_degree}
\alias{node_by_degree}
\alias{node_by_deg}
\alias{node_by_outdegree}
\alias{node_by_indegree}
\alias{node_by_multidegree}
\alias{node_by_posneg}
\alias{node_by_leverage}
\title{Measuring nodes degree-like centrality}
\usage{
node_by_degree(
  .data,
  normalized = TRUE,
  alpha = 1,
  direction = c("all", "out", "in")
)

node_by_deg(.data, alpha = 0, direction = c("all", "out", "in"))

node_by_outdegree(.data, normalized = TRUE, alpha = 0)

node_by_indegree(.data, normalized = TRUE, alpha = 0)

node_by_multidegree(.data, tie1, tie2)

node_by_posneg(.data)

node_by_leverage(.data)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{normalized}{Logical scalar, whether scores are normalized.
Different denominators may be used depending on the measure,
whether the object is one-mode or two-mode, and other arguments.
By default TRUE.}

\item{alpha}{Numeric scalar, the positive tuning parameter introduced in
Opsahl et al (2010) for trading off between degree and strength centrality measures.
By default, \code{alpha = 0}, which ignores tie weights and the measure is solely based
upon degree (the number of ties).
\code{alpha = 1} ignores the number of ties and provides the sum of the tie weights
as strength centrality.
Values between 0 and 1 reflect different trade-offs in the relative contributions of
degree and strength to the final outcome, with 0.5 as the middle ground.
Values above 1 penalise for the number of ties.
Of two nodes with the same sum of tie weights, the node with fewer ties will obtain
the higher score.
This argument is ignored except in the case of a weighted network.}

\item{direction}{Character string, “out” bases the measure on outgoing ties,
“in” on incoming ties, and "all" on either/the sum of the two.
By default "all".}

\item{tie1}{Character string indicating the first uniplex network.}

\item{tie2}{Character string indicating the second uniplex network.}
}
\value{
A \code{node_measure} numeric vector the length of the nodes in the network,
providing the scores for each node.
If the network is labelled,
then the scores will be labelled with the nodes' names.
}
\description{
These functions calculate common degree-related centrality measures for one- and two-mode networks:
\itemize{
\item \code{node_by_degree()} measures the degree centrality of nodes in an unweighted network,
or weighted degree/strength of nodes in a weighted network;
there are several related shortcut functions:
\itemize{
\item \code{node_by_deg()} returns the unnormalised results.
\item \code{node_by_indegree()} returns the \code{direction = 'in'} results.
\item \code{node_by_outdegree()} returns the \code{direction = 'out'} results.
}
\item \code{node_by_multidegree()} measures the ratio between types of ties in a multiplex network.
\item \code{node_by_posneg()} measures the PN (positive-negative) centrality of a signed network.
\item \code{node_by_leverage()} measures the leverage centrality of nodes in a network.
}

All measures attempt to use as much information as they are offered,
including whether the networks are directed, weighted, or multimodal.
If this would produce unintended results,
first transform the salient properties using e.g. \code{\link[manynet:to_undirected]{manynet::to_undirected()}} functions.
All centrality and centralization measures return normalized measures by default,
including for two-mode networks.
}
\section{Degree centrality}{

The degree of a node is the number of connections it has.
It is also sometimes called the valency of a node, \eqn{d(v)}.
The maximum degree in a network is often denoted \eqn{\Delta (G)} and
the minimum degree in a network \eqn{\delta (G)}.
The total degree of a network is the sum of all degrees, \eqn{\sum_v d(v)}.
The degree sequence is the set of all nodes' degrees,
ordered from largest to smallest.
Directed networks discriminate between
outdegree (degree of outgoing ties) and
indegree (degree of incoming ties).
}

\section{Leverage centrality}{

Leverage centrality concerns the degree of a node compared with that of its
neighbours, \eqn{J}:
\deqn{C_L(i) = \frac{1}{d(i)} \sum_{j \in J(i)} \frac{d(i) - d(j)}{d(i) + d(j)}}
}

\examples{
node_by_degree(ison_southern_women)
}
\references{
\subsection{On multimodal centrality}{

Faust, Katherine. 1997.
"Centrality in affiliation networks."
\emph{Social Networks} 19(2): 157-191.
\doi{10.1016/S0378-8733(96)00300-0}

Borgatti, Stephen P., and Martin G. Everett. 1997.
"Network analysis of 2-mode data."
\emph{Social Networks} 19(3): 243-270.
\doi{10.1016/S0378-8733(96)00301-2}

Borgatti, Stephen P., and Daniel S. Halgin. 2011.
"Analyzing affiliation networks."
In \emph{The SAGE Handbook of Social Network Analysis},
edited by John Scott and Peter J. Carrington, 417–33.
London, UK: Sage.
\doi{10.4135/9781446294413.n28}
}

\subsection{On strength centrality}{

Opsahl, Tore, Filip Agneessens, and John Skvoretz. 2010.
"Node centrality in weighted networks: Generalizing degree and shortest paths."
\emph{Social Networks} 32, 245-251.
\doi{10.1016/j.socnet.2010.03.006}
}

\subsection{On signed centrality}{

Everett, Martin G., and Stephen P. Borgatti. 2014.
“Networks Containing Negative Ties.”
\emph{Social Networks} 38:111–20.
\doi{10.1016/j.socnet.2014.03.005}
}

\subsection{On leverage centrality}{

Joyce, Karen E., Paul J. Laurienti, Jonathan H. Burdette, and Satoru Hayasaka. 2010.
"A New Measure of Centrality for Brain Networks".
\emph{PLoS ONE} 5(8): e12200.
\doi{10.1371/journal.pone.0012200}
}
}
\seealso{
Other degree:
\code{\link{mark_degree}},
\code{\link{measure_centralisation_degree}},
\code{\link{measure_centralities_degree}}

Other centrality:
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralisation_between}},
\code{\link{measure_centralisation_close}},
\code{\link{measure_centralisation_degree}},
\code{\link{measure_centralisation_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}}

Other measures:
\code{\link{measure_assort_net}},
\code{\link{measure_assort_node}},
\code{\link{measure_breadth}},
\code{\link{measure_broker_node}},
\code{\link{measure_broker_tie}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}},
\code{\link{measure_closure}},
\code{\link{measure_closure_node}},
\code{\link{measure_cohesion}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_net}},
\code{\link{measure_diverse_node}},
\code{\link{measure_features}},
\code{\link{measure_fragmentation}},
\code{\link{measure_hierarchy}},
\code{\link{measure_periods}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_node}},
\code{\link{motif_path}}
}
\concept{centrality}
\concept{degree}
\concept{measures}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mark_nodes.R
\name{mark_diff}
\alias{mark_diff}
\alias{node_is_latent}
\alias{node_is_infected}
\alias{node_is_recovered}
\alias{node_is_exposed}
\title{Marking nodes based on diffusion properties}
\usage{
node_is_latent(.data, time = 0)

node_is_infected(.data, time = 0)

node_is_recovered(.data, time = 0)

node_is_exposed(.data, mark, time = 0)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{time}{A time step at which nodes are identified.}

\item{mark}{vector denoting which nodes are infected}
}
\value{
A \code{node_mark} logical vector the length of the nodes in the network,
giving either TRUE or FALSE for each node depending on
whether the condition is matched.
}
\description{
These functions return logical vectors the length of the
nodes in a network identifying which hold certain properties or positions in the network.
\itemize{
\item \code{node_is_infected()} marks nodes that are infected by a particular time point.
\item \code{node_is_exposed()} marks nodes that are exposed to a given (other) mark.
\item \code{node_is_latent()} marks nodes that are latent at a particular time point.
\item \code{node_is_recovered()} marks nodes that are recovered at a particular time point.
}
}
\section{Exposed}{

\code{node_is_exposed()} is similar to \code{node_exposure()},
but returns a mark (TRUE/FALSE) vector indicating which nodes
are currently exposed to the diffusion content.
This diffusion content can be expressed in the 'mark' argument.
If no 'mark' argument is provided,
and '.data' is a diff_model object,
then the function will return nodes exposure to the seed nodes
in that diffusion.
}

\examples{
  # To mark nodes that are latent by a particular time point
  node_is_latent(play_diffusion(create_tree(6), latency = 1), time = 1)
  # To mark nodes that are infected by a particular time point
  node_is_infected(play_diffusion(create_tree(6)), time = 1)
  # To mark nodes that are recovered by a particular time point
  node_is_recovered(play_diffusion(create_tree(6), recovery = 0.5), time = 3)
  # To mark which nodes are currently exposed
  (expos <- node_is_exposed(manynet::create_tree(14), mark = c(1,3)))
  which(expos)
}
\seealso{
Other marks:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_dyads}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{mark_select_tie}},
\code{\link{mark_ties}},
\code{\link{mark_triangles}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_node}},
\code{\link{motif_path}}

Other diffusion:
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diffusion_node}},
\code{\link{member_diffusion}},
\code{\link{motif_exposure}},
\code{\link{motif_hazard}}
}
\concept{diffusion}
\concept{marks}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_diffusion.R
\name{member_diffusion}
\alias{member_diffusion}
\alias{node_in_adopter}
\title{Memberships in a diffusion process}
\usage{
node_in_adopter(.data)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}
}
\value{
A \code{node_member} character vector the length of the nodes in the network,
of group memberships "A", "B", etc for each node.
If the network is labelled,
then the assignments will be labelled with the nodes' names.
}
\description{
\code{node_in_adopter()} classifies membership of nodes into diffusion categories
by where on the distribution of adopters they fell.
Valente (1995) defines five memberships:
\itemize{
\item \emph{Early adopter}: those with an adoption time less than
the average adoption time minus one standard deviation of adoptions times
\item \emph{Early majority}: those with an adoption time between
the average adoption time and
the average adoption time minus one standard deviation of adoptions times
\item \emph{Late majority}: those with an adoption time between
the average adoption time and
the average adoption time plus one standard deviation of adoptions times
\item \emph{Laggard}: those with an adoption time greater than
the average adoption time plus one standard deviation of adoptions times
\item \emph{Non-adopter}: those without an adoption time,
i.e. never adopted
}
}
\examples{
  smeg <- generate_smallworld(15, 0.025)
  smeg_diff <- play_diffusion(smeg, recovery = 0.2)
  # To classify nodes by their position in the adoption curve
  (adopts <- node_in_adopter(smeg_diff))
  summary(adopts)
}
\references{
\subsection{On adopter classes}{

Valente, Tom W. 1995.
\emph{Network models of the diffusion of innovations}
(2nd ed.). Cresskill N.J.: Hampton Press.
}
}
\seealso{
Other memberships:
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_equivalence}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_node}},
\code{\link{motif_path}}

Other diffusion:
\code{\link{mark_diff}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diffusion_node}},
\code{\link{motif_exposure}},
\code{\link{motif_hazard}}
}
\concept{diffusion}
\concept{memberships}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/member_core.R
\name{mark_core}
\alias{mark_core}
\alias{node_is_core}
\title{Marking nodes as core or periphery}
\usage{
node_is_core(.data, centrality = c("degree", "eigenvector"))
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{centrality}{Which centrality measure to use to identify cores and periphery.
By default this is "degree",
which relies on the heuristic that high degree nodes are more likely to be in the core.
An alternative is "eigenvector", which instead begins with high eigenvector nodes.
Other methods, such as a genetic algorithm, CONCOR, and Rombach-Porter,
can be added if there is interest.}
}
\value{
A \code{node_mark} logical vector the length of the nodes in the network,
giving either TRUE or FALSE for each node depending on
whether the condition is matched.
}
\description{
\code{node_is_core()} identifies whether nodes belong to the core of the
network, as opposed to the periphery.
}
\section{Core-periphery}{

This function is used to identify which nodes should belong to the core,
and which to the periphery.
It seeks to minimize the following quantity:
\deqn{Z(S_1) = \sum_{(i<j)\in S_1} \textbf{I}_{\{A_{ij}=0\}} + \sum_{(i<j)\notin S_1} \textbf{I}_{\{A_{ij}=1\}}}
where nodes \eqn{\{i,j,...,n\}} are ordered in descending degree,
\eqn{A} is the adjacency matrix,
and the indicator function is 1 if the predicate is true or 0 otherwise.
Note that minimising this quantity maximises density in the core block
and minimises density in the periphery block;
it ignores ties between these blocks.
}

\examples{
node_is_core(ison_adolescents)
ison_adolescents |> 
   mutate(corep = node_is_core())
}
\references{
\subsection{On core-periphery partitioning}{

Borgatti, Stephen P., & Everett, Martin G. 1999.
"Models of core /periphery structures".
\emph{Social Networks}, 21, 375–395.
\doi{10.1016/S0378-8733(99)00019-2}

Lip, Sean Z. W. 2011.
“A Fast Algorithm for the Discrete Core/Periphery Bipartitioning Problem.”
\doi{10.48550/arXiv.1102.5511}
}
}
\seealso{
Other core-periphery:
\code{\link{measure_core}},
\code{\link{member_core}}

Other marks:
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_dyads}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{mark_select_tie}},
\code{\link{mark_ties}},
\code{\link{mark_triangles}}

Other nodal:
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_node}},
\code{\link{motif_path}}
}
\concept{core-periphery}
\concept{marks}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_closure.R
\name{measure_closure_node}
\alias{measure_closure_node}
\alias{node_by_reciprocity}
\alias{node_by_transitivity}
\alias{node_by_equivalency}
\title{Measuring node closure}
\usage{
node_by_reciprocity(.data)

node_by_transitivity(.data)

node_by_equivalency(.data)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}
}
\value{
A \code{node_measure} numeric vector the length of the nodes in the network,
providing the scores for each node.
If the network is labelled,
then the scores will be labelled with the nodes' names.
}
\description{
These functions offer methods for summarising the closure in configurations
in one- and two-mode networks:
\itemize{
\item \code{node_by_reciprocity()} measures nodes' reciprocity.
\item \code{node_by_transitivity()} measures nodes' transitivity.
\item \code{node_by_equivalency()} measures nodes' equivalence or reinforcement
in a (usually two-mode) network.
}
}
\details{
For one-mode networks, shallow wrappers of igraph versions exist via
\code{node_by_reciprocity} and \code{node_by_transitivity}.

For two-mode networks, \code{node_by_equivalency} calculates the proportion of three-paths in the network
that are closed by fourth tie to establish a "shared four-cycle" structure.
}
\examples{
node_by_reciprocity(to_unweighted(ison_networkers))
node_by_transitivity(ison_adolescents)
}
\seealso{
Other measures:
\code{\link{measure_assort_net}},
\code{\link{measure_assort_node}},
\code{\link{measure_breadth}},
\code{\link{measure_broker_node}},
\code{\link{measure_broker_tie}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}},
\code{\link{measure_closure}},
\code{\link{measure_cohesion}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_net}},
\code{\link{measure_diverse_node}},
\code{\link{measure_features}},
\code{\link{measure_fragmentation}},
\code{\link{measure_hierarchy}},
\code{\link{measure_periods}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_node}},
\code{\link{motif_path}}
}
\concept{measures}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_heterogeneity.R
\name{measure_diverse_node}
\alias{measure_diverse_node}
\alias{node_by_richness}
\alias{node_by_diversity}
\title{Measures of nodes diversity}
\usage{
node_by_richness(.data, attribute)

node_by_diversity(
  .data,
  attribute,
  diversity = c("blau", "teachman", "variation", "gini")
)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{attribute}{Name of a nodal attribute, mark, measure, or membership vector.}

\item{diversity}{Which method to use for \verb{*_diversity()}.
Either "blau" (Blau's index) or "teachman" (Teachman's index) for categorical attributes,
or "variation" (coefficient of variation) or "gini" (Gini coefficient) for numeric attributes.
Default is "blau".
If an incompatible method is chosen for the attribute type,
a suitable alternative will be used instead with a message.}
}
\value{
A \code{node_measure} numeric vector the length of the nodes in the network,
providing the scores for each node.
If the network is labelled,
then the scores will be labelled with the nodes' names.
}
\description{
These functions offer ways to measure the heterogeneity of an attribute
across a network, within groups of a network, or the distribution of ties
across this attribute:
\itemize{
\item \code{node_by_richness()} measures the number of unique categories
of an attribute to which each node is connected.
\item \code{node_by_diversity()} measures the heterogeneity of each node's
local neighbourhood.
}
}
\examples{
node_by_richness(ison_networkers, "Discipline")
marvel_friends <- to_unsigned(to_uniplex(fict_marvel, "relationship"), "positive")
node_by_diversity(marvel_friends, "Gender")
node_by_diversity(marvel_friends, "Attractive")
}
\seealso{
Other diversity:
\code{\link{measure_assort_net}},
\code{\link{measure_assort_node}},
\code{\link{measure_diverse_net}}

Other measures:
\code{\link{measure_assort_net}},
\code{\link{measure_assort_node}},
\code{\link{measure_breadth}},
\code{\link{measure_broker_node}},
\code{\link{measure_broker_tie}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}},
\code{\link{measure_closure}},
\code{\link{measure_closure_node}},
\code{\link{measure_cohesion}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_net}},
\code{\link{measure_features}},
\code{\link{measure_fragmentation}},
\code{\link{measure_hierarchy}},
\code{\link{measure_periods}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_node}},
\code{\link{motif_path}}
}
\concept{diversity}
\concept{measures}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_holes.R
\name{measure_broker_node}
\alias{measure_broker_node}
\alias{node_by_bridges}
\alias{node_by_redundancy}
\alias{node_by_effsize}
\alias{node_by_efficiency}
\alias{node_by_constraint}
\alias{node_by_hierarchy}
\alias{node_by_neighbours_degree}
\title{Measuring nodes brokerage}
\usage{
node_by_bridges(.data)

node_by_redundancy(.data)

node_by_effsize(.data)

node_by_efficiency(.data)

node_by_constraint(.data)

node_by_hierarchy(.data)

node_by_neighbours_degree(.data)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}
}
\value{
A \code{node_measure} numeric vector the length of the nodes in the network,
providing the scores for each node.
If the network is labelled,
then the scores will be labelled with the nodes' names.
}
\description{
These function provide different measures of the degree to which nodes
fill structural holes, as outlined in Burt (1992):
\itemize{
\item \code{node_by_bridges()} measures the sum of bridges to which each node
is adjacent.
\item \code{node_by_redundancy()} measures the redundancy of each nodes' contacts.
\item \code{node_by_effsize()} measures nodes' effective size.
\item \code{node_by_efficiency()} measures nodes' efficiency.
\item \code{node_by_constraint()} measures nodes' constraint scores for one-mode networks
according to Burt (1992) and for two-mode networks according to Hollway et al (2020).
\item \code{node_by_hierarchy()} measures nodes' exposure to hierarchy,
where only one or two contacts are the source of closure.
\item \code{node_by_neighbours_degree()} measures nodes' average nearest neighbors degree,
or \eqn{knn}, a measure of the type of local environment a node finds itself in
}

Burt's theory holds that while those nodes embedded in dense clusters
of close connections are likely exposed to the same or similar ideas and information,
those who fill structural holes between two otherwise disconnected groups
can gain some comparative advantage from that position.
}
\details{
A number of different ways of measuring these structural holes are available.
Note that we use Borgatti's reformulation for unweighted networks in
\code{node_redundancy()} and \code{node_effsize()}.
Redundancy is thus \eqn{\frac{2t}{n}},
where \eqn{t} is the sum of ties and \eqn{n} the sum of nodes in each node's neighbourhood,
and effective size is calculated as \eqn{n - \frac{2t}{n}}.
Node efficiency is the node's effective size divided by its degree.
}
\examples{
node_by_bridges(ison_adolescents)
node_by_bridges(ison_southern_women)
node_by_redundancy(ison_adolescents)
node_by_redundancy(ison_southern_women)
node_by_effsize(ison_adolescents)
node_by_effsize(ison_southern_women)
node_by_efficiency(ison_adolescents)
node_by_efficiency(ison_southern_women)
node_by_constraint(ison_southern_women)
node_by_hierarchy(ison_adolescents)
node_by_hierarchy(ison_southern_women)
}
\references{
\subsection{On structural holes}{

Burt, Ronald S. 1992.
\emph{Structural Holes: The Social Structure of Competition}.
Cambridge, MA: Harvard University Press.
}

Borgatti, Steven. 1997.
“\href{http://www.analytictech.com/connections/v20(1)/holes.htm}{Structural Holes: Unpacking Burt’s Redundancy Measures}”
\emph{Connections} 20(1):35-38.

Burchard, Jake, and Benjamin Cornwell. 2018.
“Structural Holes and Bridging in Two-Mode Networks.”
\emph{Social Networks} 55:11–20.
\doi{10.1016/j.socnet.2018.04.001}

Hollway, James, Jean-Frédéric Morin, and Joost Pauwelyn. 2020.
"Structural conditions for novelty: The introduction of new environmental clauses to the trade regime complex."
\emph{International Environmental Agreements: Politics, Law and Economics} 20 (1): 61–83.
\doi{10.1007/s10784-019-09464-5}

\subsection{On neighbours average degree}{

Barrat, Alain, Marc Barthelemy, Romualdo Pastor-Satorras, and Alessandro Vespignani. 2004.
"The architecture of complex weighted networks",
\emph{Proc. Natl. Acad. Sci.} 101: 3747.
}
}
\seealso{
Other brokerage:
\code{\link{measure_broker_tie}},
\code{\link{measure_brokerage}},
\code{\link{member_brokerage}},
\code{\link{motif_brokerage_net}},
\code{\link{motif_brokerage_node}}

Other measures:
\code{\link{measure_assort_net}},
\code{\link{measure_assort_node}},
\code{\link{measure_breadth}},
\code{\link{measure_broker_tie}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}},
\code{\link{measure_closure}},
\code{\link{measure_closure_node}},
\code{\link{measure_cohesion}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_net}},
\code{\link{measure_diverse_node}},
\code{\link{measure_features}},
\code{\link{measure_fragmentation}},
\code{\link{measure_hierarchy}},
\code{\link{measure_periods}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_node}},
\code{\link{motif_path}}
}
\concept{brokerage}
\concept{measures}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_hierarchy.R
\name{motif_hierarchy}
\alias{motif_hierarchy}
\alias{net_x_hierarchy}
\title{Motifs of network hierarchy}
\usage{
net_x_hierarchy(.data)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}
}
\value{
A \code{network_motif} named numeric vector or sometimes a data frame with
one row and a column for each motif type,
giving the count of each motif in the network.
This is printed as a tibble to avoid greedy printing of long vectors.
}
\description{
\code{net_x_hierarchy()} collects the measures of hierarchy into a single motif,
which can be used to compare the relative hierarchy of different networks.
The measures of hierarchy are:
\itemize{
\item \code{net_by_connectedness()} measures the proportion of dyads in the network
that are reachable to one another,
or the degree to which network is a single component.
\item \code{net_by_efficiency()} measures the Krackhardt efficiency score.
\item \code{net_by_upperbound()} measures the Krackhardt (least) upper bound
score.
\item \code{net_by_reciprocity()} measures the proportion of ties in the network that
are reciprocated,
which is a measure of the degree to which the network is non-hierarchical.
}
}
\examples{
net_x_hierarchy(ison_networkers)
}
\references{
\subsection{On hierarchy}{

Krackhardt, David. 1994.
Graph theoretical dimensions of informal organizations.
In Carley and Prietula (eds) \emph{Computational Organizational Theory},
Hillsdale, NJ: Lawrence Erlbaum Associates. Pp. 89-111.

Everett, Martin, and David Krackhardt. 2012.
“A second look at Krackhardt's graph theoretical dimensions of informal organizations.”
\emph{Social Networks}, 34: 159-163.
\doi{10.1016/j.socnet.2011.10.006}
}
}
\seealso{
Other hierarchy:
\code{\link{measure_hierarchy}}

Other motifs:
\code{\link{motif_brokerage_net}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_hazard}},
\code{\link{motif_net}},
\code{\link{motif_node}},
\code{\link{motif_path}},
\code{\link{motif_periods}}
}
\concept{hierarchy}
\concept{motifs}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_holes.R
\name{measure_broker_tie}
\alias{measure_broker_tie}
\alias{tie_by_cohesion}
\title{Measuring ties brokerage}
\usage{
tie_by_cohesion(.data)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}
}
\value{
A \code{tie_measure} numeric vector the length of the ties in the network,
providing the scores for each tie.
If the network is labelled,
then the scores will be labelled with the ties' adjacent nodes' names.
}
\description{
\code{tie_by_cohesion()} measures the ratio between common neighbors to ties'
adjacent nodes and the total number of adjacent nodes,
where high values indicate ties' embeddedness in dense local environments.
}
\seealso{
Other brokerage:
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{member_brokerage}},
\code{\link{motif_brokerage_net}},
\code{\link{motif_brokerage_node}}

Other measures:
\code{\link{measure_assort_net}},
\code{\link{measure_assort_node}},
\code{\link{measure_breadth}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}},
\code{\link{measure_closure}},
\code{\link{measure_closure_node}},
\code{\link{measure_cohesion}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_net}},
\code{\link{measure_diverse_node}},
\code{\link{measure_features}},
\code{\link{measure_fragmentation}},
\code{\link{measure_hierarchy}},
\code{\link{measure_periods}}

Other tie:
\code{\link{mark_dyads}},
\code{\link{mark_select_tie}},
\code{\link{mark_ties}},
\code{\link{mark_triangles}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}}
}
\concept{brokerage}
\concept{measures}
\concept{tie}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_centrality_degree.R
\name{measure_centralisation_degree}
\alias{measure_centralisation_degree}
\alias{net_by_degree}
\alias{mode_by_degree}
\alias{net_by_outdegree}
\alias{net_by_indegree}
\alias{mode_by_outdegree}
\alias{mode_by_indegree}
\title{Measuring networks degree-like centralisation}
\usage{
net_by_degree(.data, normalized = TRUE, direction = c("all", "out", "in"))

mode_by_degree(.data, normalized = TRUE, direction = c("all", "out", "in"))

net_by_outdegree(.data, normalized = TRUE)

net_by_indegree(.data, normalized = TRUE)

mode_by_outdegree(.data, normalized = TRUE)

mode_by_indegree(.data, normalized = TRUE)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{normalized}{Logical scalar, whether scores are normalized.
Different denominators may be used depending on the measure,
whether the object is one-mode or two-mode, and other arguments.
By default TRUE.}

\item{direction}{Character string, “out” bases the measure on outgoing ties,
“in” on incoming ties, and "all" on either/the sum of the two.
By default "all".}
}
\value{
\verb{net_by_*()} functions return a \code{network_measure} scalar;
\verb{mode_by_*()} functions return a \code{mode_measure} numeric vector of length two,
giving one centralization score per mode.
}
\description{
\itemize{
\item \code{net_by_degree()} measures a network's degree centralization as a single
score; there are several related shortcut functions:
\itemize{
\item \code{net_by_indegree()} returns the \code{direction = 'in'} results.
\item \code{net_by_outdegree()} returns the \code{direction = 'out'} results.
}
\item \code{mode_by_degree()} measures degree centralization separately for each mode
of a two-mode network, returning one score per mode
(following Borgatti and Everett, 1997); it has the same shortcuts:
\itemize{
\item \code{mode_by_indegree()} returns the \code{direction = 'in'} results.
\item \code{mode_by_outdegree()} returns the \code{direction = 'out'} results.
}
}

All measures attempt to use as much information as they are offered,
including whether the networks are directed, weighted, or multimodal.
If this would produce unintended results,
first transform the salient properties using e.g. \code{\link[manynet:to_undirected]{to_undirected()}} functions.
All centrality and centralization measures return normalized measures
by default, including for two-mode networks.

For two-mode networks, the two modes have different theoretical maxima,
so \code{net_by_degree()} reports a single network-level score by applying
Freeman's general centralization index over the mode-normalized node
degrees, whereas \code{mode_by_degree()} reports the per-mode centralization
scores directly. For the per-mode scores, "all" uses as numerator the sum
of differences between the maximum centrality score for the mode
against all other centrality scores in the network,
whereas "in" uses as numerator the sum of differences
between the maximum centrality score for the mode
against only the centrality scores of the other nodes in that mode.
}
\examples{
net_by_degree(ison_southern_women, direction = "in")
mode_by_degree(ison_southern_women, direction = "in")
}
\references{
Borgatti, Stephen P., and Martin G. Everett. 1997.
"Network analysis of 2-mode data."
\emph{Social Networks} 19(3): 243-269.
\doi{10.1016/S0378-8733(96)00301-2}
}
\seealso{
Other degree:
\code{\link{mark_degree}},
\code{\link{measure_central_degree}},
\code{\link{measure_centralities_degree}}

Other centrality:
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralisation_between}},
\code{\link{measure_centralisation_close}},
\code{\link{measure_centralisation_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}}
}
\concept{centrality}
\concept{degree}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_cohesion.R
\name{measure_fragmentation}
\alias{measure_fragmentation}
\alias{net_by_cohesion}
\alias{net_by_adhesion}
\alias{net_by_strength}
\alias{net_by_toughness}
\title{Measures of network fragmentation}
\usage{
net_by_cohesion(.data)

net_by_adhesion(.data)

net_by_strength(.data)

net_by_toughness(.data)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}
}
\value{
A \code{network_measure} numeric score.
}
\description{
These functions return values relating to how connected a network is
and the number of nodes or edges to remove that would increase fragmentation.
\itemize{
\item \code{net_by_cohesion()} measures the minimum number of nodes to remove
from the network needed to increase the number of components.
\item \code{net_by_toughness()} measures the number of nodes that would need to be
removed from a network to increase its number of components.
\item \code{net_by_adhesion()} measures the minimum number of ties to remove
from the network needed to increase the number of components.
\item \code{net_by_strength()} measures the number of ties that would need to be
removed from a network to increase its number of components.
}
}
\examples{
net_by_cohesion(fict_marvel)
net_by_cohesion(to_giant(fict_marvel))
net_by_adhesion(fict_marvel)
net_by_adhesion(to_giant(fict_marvel))
net_by_strength(ison_adolescents)
net_by_toughness(ison_adolescents)
}
\references{
\subsection{On cohesion}{

White, Douglas R and Frank Harary. 2001.
"The Cohesiveness of Blocks In Social Networks: Node Connectivity and Conditional Density."
\emph{Sociological Methodology} 31(1): 305-59.
\doi{10.1111/0081-1750.00098}
}
}
\seealso{
Other cohesion:
\code{\link{mark_triangles}},
\code{\link{measure_breadth}},
\code{\link{measure_cohesion}},
\code{\link{motif_net}},
\code{\link{motif_node}}

Other measures:
\code{\link{measure_assort_net}},
\code{\link{measure_assort_node}},
\code{\link{measure_breadth}},
\code{\link{measure_broker_node}},
\code{\link{measure_broker_tie}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}},
\code{\link{measure_closure}},
\code{\link{measure_closure_node}},
\code{\link{measure_cohesion}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_net}},
\code{\link{measure_diverse_node}},
\code{\link{measure_features}},
\code{\link{measure_hierarchy}},
\code{\link{measure_periods}}
}
\concept{cohesion}
\concept{measures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_diffusion.R
\name{measure_diffusion_net}
\alias{measure_diffusion_net}
\alias{net_by_transmissibility}
\alias{net_by_recovery}
\alias{net_by_reproduction}
\alias{net_by_immunity}
\title{Measures of network diffusion}
\usage{
net_by_transmissibility(.data)

net_by_recovery(.data, censor = TRUE)

net_by_reproduction(.data)

net_by_immunity(.data, normalized = TRUE)
}
\arguments{
\item{.data}{Network data with nodal changes,
as created by \code{play_diffusion()},
or a valid network diffusion model,
as created by \code{as_diffusion()}.}

\item{censor}{Where some nodes have not yet recovered by the end
of the simulation, right censored values can be replaced by the number of steps.
By default TRUE.
Note that this will likely still underestimate recovery.}

\item{normalized}{Logical scalar, whether scores are normalized.
Different denominators may be used depending on the measure,
whether the object is one-mode or two-mode, and other arguments.
By default TRUE.}
}
\value{
A \code{network_measure} numeric score.
}
\description{
These functions allow measurement of various features of
a diffusion process at the network level:
\itemize{
\item \code{net_by_transmissibility()} measures the average transmissibility observed
in a diffusion simulation, or the number of new infections over
the number of susceptible nodes.
\item \code{net_by_recovery()} measures the average number of time steps
nodes remain infected once they become infected.
\item \code{net_by_reproduction()} measures the observed reproductive number
in a diffusion simulation as the network's transmissibility over
the network's average infection length.
\item \code{net_by_immunity()} measures the proportion of nodes that would need
to be protected through vaccination, isolation, or recovery for herd immunity to be reached.
}
}
\section{Transmissibility}{

\code{net_transmissibility()} measures how many directly susceptible nodes
each infected node will infect in each time period, on average.
That is:
\deqn{T = \frac{1}{n}\sum_{j=1}^n \frac{i_{j}}{s_{j}}}
where \eqn{i} is the number of new infections in each time period, \eqn{j \in n},
and \eqn{s} is the number of nodes that could have been infected in that time period
(note that \eqn{s \neq S}, or
the number of nodes that are susceptible in the population).
\eqn{T} can be interpreted as the proportion of susceptible nodes that are
infected at each time period.
}

\section{Recovery time}{

\code{net_recovery()} measures the average number of time steps that
nodes in a network remain infected.
Note that in a diffusion model without recovery, average infection length
will be infinite.
This will also be the case where there is right censoring.
The longer nodes remain infected, the longer they can infect others.
}

\section{Reproduction number}{

\code{net_reproduction()} measures a given diffusion's reproductive number.
Here it is calculated as:
\deqn{R = \min\left(\frac{T}{1/L}, \bar{k}\right)}
where \eqn{T} is the observed transmissibility in a diffusion
and \eqn{L} is the observed recovery length in a diffusion.
Since \eqn{L} can be infinite where there is no recovery
or there is right censoring,
and since network structure places an upper limit on how many
nodes each node may further infect (their degree),
this function returns the minimum of \eqn{R_0}
and the network's average degree.

Interpretation of the reproduction number is oriented around R = 1.
Where \eqn{R > 1}, the 'disease' will 'infect' more and more
nodes in the network.
Where \eqn{R < 1}, the 'disease' will not sustain itself and eventually
die out.
Where \eqn{R = 1}, the 'disease' will continue as endemic,
if conditions allow.
}

\section{Herd immunity}{

\code{net_immunity()} estimates the proportion of a network
that need to be protected from infection for herd immunity
to be achieved.
This is known as the Herd Immunity Threshold or HIT:
\deqn{1 - \frac{1}{R}}
where \eqn{R} is the reproduction number from \code{net_reproduction()}.
The HIT indicates the threshold at which
the reduction of susceptible members of the network means
that infections will no longer keep increasing.
Note that there may still be more infections after this threshold has been reached,
but there should be fewer and fewer.
These excess infections are called the \emph{overshoot}.
This function does \emph{not} take into account the structure
of the network, instead using the average degree.

Interpretation is quite straightforward.
A HIT or immunity score of 0.75 would mean that 75\% of the nodes in the network
would need to be vaccinated or otherwise protected to achieve herd immunity.
To identify how many nodes this would be, multiply this proportion with the number
of nodes in the network.
}

\examples{
  smeg <- generate_smallworld(15, 0.025)
  smeg_diff <- play_diffusion(smeg, recovery = 0.2)
  plot(smeg_diff)
  # To calculate the average transmissibility for a given diffusion model
  net_by_transmissibility(smeg_diff)
  # To calculate the average infection length for a given diffusion model
  net_by_recovery(smeg_diff)
  # To calculate the reproduction number for a given diffusion model
  net_by_reproduction(smeg_diff)
  # Calculating the proportion required to achieve herd immunity
  net_by_immunity(smeg_diff)
  # To find the number of nodes to be vaccinated
  net_by_immunity(smeg_diff, normalized = FALSE)
}
\references{
\subsection{On epidemiological models}{

Kermack, William O., and Anderson Gray McKendrick. 1927.
"A contribution to the mathematical theory of epidemics".
\emph{Proc. R. Soc. London A} 115: 700-721.
\doi{10.1098/rspa.1927.0118}
}

\subsection{On the basic reproduction number}{

Diekmann, Odo, Hans J.A.P. Heesterbeek, and Hans J.A.J. Metz. 1990.
"On the definition and the computation of the basic reproduction ratio R0 in models for infectious diseases in heterogeneous populations".
\emph{Journal of Mathematical Biology}, 28(4): 365–82.
\doi{10.1007/BF00178324}

Kenah, Eben, and James M. Robins. 2007.
"Second look at the spread of epidemics on networks".
\emph{Physical Review E}, 76(3 Pt 2): 036113.
\doi{10.1103/PhysRevE.76.036113}
}

\subsection{On herd immunity}{

Garnett, G.P. 2005.
"Role of herd immunity in determining the effect of vaccines against sexually transmitted disease".
\emph{The Journal of Infectious Diseases}, 191(1): S97-106.
\doi{10.1086/425271}
}
}
\seealso{
Other measures:
\code{\link{measure_assort_net}},
\code{\link{measure_assort_node}},
\code{\link{measure_breadth}},
\code{\link{measure_broker_node}},
\code{\link{measure_broker_tie}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}},
\code{\link{measure_closure}},
\code{\link{measure_closure_node}},
\code{\link{measure_cohesion}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_net}},
\code{\link{measure_diverse_node}},
\code{\link{measure_features}},
\code{\link{measure_fragmentation}},
\code{\link{measure_hierarchy}},
\code{\link{measure_periods}}

Other diffusion:
\code{\link{mark_diff}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_node}},
\code{\link{member_diffusion}},
\code{\link{motif_exposure}},
\code{\link{motif_hazard}}
}
\concept{diffusion}
\concept{measures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/motif_brokerage.R
\name{motif_brokerage_net}
\alias{motif_brokerage_net}
\alias{net_x_brokerage}
\title{Motifs of network brokerage}
\usage{
net_x_brokerage(.data, membership, standardized = FALSE)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{membership}{A character string naming an existing node attribute in
the network, or a categorical vector of the same length as the number of
nodes in the network where each element indicates the group membership of
the corresponding node.
While this may often be a vector created using \verb{node_in_*()} functions,
it can be any character vector that assigns nodes to groups or categories.}

\item{standardized}{Whether the score should be standardized
into a \emph{z}-score indicating how many standard deviations above
or below the average the score lies.}
}
\value{
A \code{network_motif} named numeric vector or sometimes a data frame with
one row and a column for each motif type,
giving the count of each motif in the network.
This is printed as a tibble to avoid greedy printing of long vectors.
}
\description{
\code{net_x_brokerage()} returns the Gould-Fernandez brokerage
roles in a network.
}
\examples{
net_x_brokerage(ison_networkers, "Discipline")
}
\seealso{
Other brokerage:
\code{\link{measure_broker_node}},
\code{\link{measure_broker_tie}},
\code{\link{measure_brokerage}},
\code{\link{member_brokerage}},
\code{\link{motif_brokerage_node}}

Other motifs:
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_hazard}},
\code{\link{motif_hierarchy}},
\code{\link{motif_net}},
\code{\link{motif_node}},
\code{\link{motif_path}},
\code{\link{motif_periods}}
}
\concept{brokerage}
\concept{motifs}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/motif_brokerage.R
\name{motif_brokerage_node}
\alias{motif_brokerage_node}
\alias{node_x_brokerage}
\title{Motifs of nodes brokerage}
\usage{
node_x_brokerage(.data, membership, standardized = FALSE)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{membership}{A character string naming an existing node attribute in
the network, or a categorical vector of the same length as the number of
nodes in the network where each element indicates the group membership of
the corresponding node.
While this may often be a vector created using \verb{node_in_*()} functions,
it can be any character vector that assigns nodes to groups or categories.}

\item{standardized}{Whether the score should be standardized
into a \emph{z}-score indicating how many standard deviations above
or below the average the score lies.}
}
\value{
A \code{node_motif} matrix with one row for each node in the network and
a column for each motif type,
giving the count of each motif in which each node participates.
It is printed as a tibble, however, to avoid greedy printing.
If the network is labelled,
then the node names will be in a column named \code{names}.
}
\description{
\code{node_x_brokerage()} returns the Gould-Fernandez brokerage
roles played by nodes in a network.
}
\examples{
node_x_brokerage(ison_networkers, "Discipline")
}
\references{
\subsection{On brokerage motifs}{

Gould, Roger V., and Roberto M. Fernandez. 1989.
“Structures of Mediation: A Formal Approach to Brokerage in Transaction Networks.”
\emph{Sociological Methodology}, 19: 89-126.
\doi{10.2307/270949}

Jasny, Lorien, and Mark Lubell. 2015.
“Two-Mode Brokerage in Policy Networks.”
\emph{Social Networks} 41:36–47.
\doi{10.1016/j.socnet.2014.11.005}
}
}
\seealso{
Other brokerage:
\code{\link{measure_broker_node}},
\code{\link{measure_broker_tie}},
\code{\link{measure_brokerage}},
\code{\link{member_brokerage}},
\code{\link{motif_brokerage_net}}

Other motifs:
\code{\link{motif_brokerage_net}},
\code{\link{motif_exposure}},
\code{\link{motif_hazard}},
\code{\link{motif_hierarchy}},
\code{\link{motif_net}},
\code{\link{motif_node}},
\code{\link{motif_path}},
\code{\link{motif_periods}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_exposure}},
\code{\link{motif_node}},
\code{\link{motif_path}}
}
\concept{brokerage}
\concept{motifs}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_hierarchy.R
\name{measure_hierarchy}
\alias{measure_hierarchy}
\alias{net_by_connectedness}
\alias{net_by_efficiency}
\alias{net_by_upperbound}
\title{Measures of hierarchy}
\usage{
net_by_connectedness(.data)

net_by_efficiency(.data)

net_by_upperbound(.data)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}
}
\value{
A \code{network_measure} numeric score.
}
\description{
These functions, together with \code{net_reciprocity()}, are used jointly to
measure how hierarchical a network is:
\itemize{
\item \code{net_by_connectedness()} measures the proportion of dyads in the network
that are reachable to one another,
or the degree to which network is a single component.
\item \code{net_by_efficiency()} measures the Krackhardt efficiency score.
\item \code{net_by_upperbound()} measures the Krackhardt (least) upper bound score.
}
}
\examples{
net_by_connectedness(ison_networkers)
1 - net_by_reciprocity(ison_networkers)
net_by_efficiency(ison_networkers)
net_by_upperbound(ison_networkers)
}
\references{
\subsection{On hierarchy}{

Krackhardt, David. 1994.
Graph theoretical dimensions of informal organizations.
In Carley and Prietula (eds) \emph{Computational Organizational Theory},
Hillsdale, NJ: Lawrence Erlbaum Associates. Pp. 89-111.

Everett, Martin, and David Krackhardt. 2012.
“A second look at Krackhardt's graph theoretical dimensions of informal organizations.”
\emph{Social Networks}, 34: 159-163.
\doi{10.1016/j.socnet.2011.10.006}
}
}
\seealso{
Other measures:
\code{\link{measure_assort_net}},
\code{\link{measure_assort_node}},
\code{\link{measure_breadth}},
\code{\link{measure_broker_node}},
\code{\link{measure_broker_tie}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}},
\code{\link{measure_closure}},
\code{\link{measure_closure_node}},
\code{\link{measure_cohesion}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_net}},
\code{\link{measure_diverse_node}},
\code{\link{measure_features}},
\code{\link{measure_fragmentation}},
\code{\link{measure_periods}}

Other hierarchy:
\code{\link{motif_hierarchy}}
}
\concept{hierarchy}
\concept{measures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/method_cluster.R
\name{method_cluster}
\alias{method_cluster}
\alias{cluster_hierarchical}
\alias{cluster_cosine}
\alias{cluster_concor}
\title{Methods for equivalence clustering}
\usage{
cluster_hierarchical(motif, distance)

cluster_cosine(motif, distance)

cluster_concor(.data, motif)
}
\arguments{
\item{motif}{A matrix returned by a \verb{node_x_*()} function.}

\item{distance}{Character string indicating which distance metric
to pass on to \code{stats::dist}.
By default \code{"euclidean"}, but other options include
\code{"maximum"}, \code{"manhattan"}, \code{"canberra"}, \code{"binary"}, and \code{"minkowski"}.
Fewer, identifiable letters, e.g. \code{"e"} for Euclidean, is sufficient.}

\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}
}
\value{
A hierarchical clustering object created by \code{stats::hclust()},
with an additional \code{distances} attribute containing the distance matrix
used for clustering.
}
\description{
These functions are used to cluster some motif census object:
\itemize{
\item \code{cluster_hierarchical()} returns a hierarchical clustering object
created by \code{stats::hclust()}.
\item \code{cluster_concor()} returns a hierarchical clustering object
created from a convergence of correlations procedure (CONCOR).
\item \code{cluster_cosine()} returns a hierarchical clustering object
created by \code{stats::hclust()} on cosine dissimilarities,
rather than correlations, as created by \code{to_cosine()}.
}

These functions are not intended to be called directly,
but are called within \code{node_in_equivalence()} and related functions.
They are exported and listed here to provide more detailed documentation.
}
\section{Hierarchical clustering}{

This method uses \code{stats::hclust()} to create a hierarchical clustering object
from a distance matrix created from the correlations between nodes' profiles
in the given motif census.
First a matrix of Pearson correlation coefficients between each pair of
nodes' profiles in the given motif census is created.
Then a distance matrix is created by subtracting these correlations from \code{1},
and this is given to \code{stats::hclust()} to enable dendrogram construction etc.
}

\section{Cosine similarity}{

This method is similar to the hierarchical clustering method,
but uses cosine similarity rather than correlation as the clustering basis.
First a matrix of cosine similarities between each pair of nodes' profiles
in the given census is created.
Then a distance matrix is created by subtracting these similarities from \code{1},
and this is given to \code{stats::hclust} to enable dendrogram construction etc.
}

\section{CONCOR}{

First a matrix of Pearson correlation coefficients between each pair of
nodes' profiles in the given motif census is created.
Then, again, we find the correlations of this square, symmetric matrix,
and continue to do this iteratively until each entry is either \code{1} or \code{-1}.
These values are used to split the data into two partitions,
with members either holding the values \code{1} or \code{-1}.
This procedure from census to convergence is then repeated within each block,
allowing further partitions to be found.
Unlike UCINET, partitions are continued until there are single members in
each partition.
Then a distance matrix is constructed from records of in which partition phase
nodes were separated,
and this is given to \code{stats::hclust()} so that dendrograms etc can be returned.
}

\references{
\subsection{On CONCOR clustering}{

Breiger, Ronald L., Scott A. Boorman, and Phipps Arabie. 1975.
"An Algorithm for Clustering Relational Data with Applications to
Social Network Analysis and Comparison with Multidimensional Scaling".
\emph{Journal of Mathematical Psychology}, 12: 328-83.
\doi{10.1016/0022-2496(75)90028-0}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_centrality_closeness.R
\name{measure_central_close}
\alias{measure_central_close}
\alias{node_by_closeness}
\alias{node_by_harmonic}
\alias{node_by_reach}
\alias{node_by_information}
\alias{node_by_eccentricity}
\alias{node_by_distance}
\alias{node_by_vitality}
\alias{node_by_randomwalk}
\title{Measuring nodes closeness-like centrality}
\usage{
node_by_closeness(.data, normalized = TRUE, direction = "out", cutoff = NULL)

node_by_harmonic(.data, normalized = TRUE, cutoff = -1)

node_by_reach(.data, normalized = TRUE, cutoff = 2)

node_by_information(.data, normalized = TRUE)

node_by_eccentricity(.data, normalized = TRUE)

node_by_distance(.data, from, to, normalized = TRUE)

node_by_vitality(.data, normalized = TRUE)

node_by_randomwalk(.data, normalized = TRUE)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{normalized}{Logical scalar, whether scores are normalized.
Different denominators may be used depending on the measure,
whether the object is one-mode or two-mode, and other arguments.
By default TRUE.}

\item{direction}{Character string, “out” bases the measure on outgoing ties,
“in” on incoming ties, and "all" on either/the sum of the two.
By default "all".}

\item{cutoff}{Maximum path length to use during calculations.}

\item{from, to}{Index or name of a node to calculate distances from or to.}
}
\value{
A \code{node_measure} numeric vector the length of the nodes in the network,
providing the scores for each node.
If the network is labelled,
then the scores will be labelled with the nodes' names.
}
\description{
These functions calculate common closeness-related centrality measures
that rely on path-length for one- and two-mode networks:
\itemize{
\item \code{node_by_closeness()} measures the closeness centrality of nodes in a
network.
\item \code{node_by_harmonic()} measures nodes' harmonic centrality or valued
centrality, which is thought to behave better than reach centrality
for disconnected networks.
\item \code{node_by_reach()} measures nodes' reach centrality,
or how many nodes they can reach within \emph{k} steps.
\item \code{node_by_information()} measures nodes' information centrality or
current-flow closeness centrality.
\item \code{node_by_eccentricity()} measures nodes' eccentricity or maximum distance
from another node in the network.
\item \code{node_by_distance()} measures nodes' geodesic distance from or to a
given node.
\item \code{node_by_vitality()} measures a network's closeness vitality centrality,
or the change in closeness centrality between networks with and without a
given node.
}

All measures attempt to use as much information as they are offered,
including whether the networks are directed, weighted, or multimodal.
If this would produce unintended results,
first transform the salient properties using e.g. \code{\link[manynet:to_undirected]{to_undirected()}} functions.
All centrality and centralization measures return normalized measures by default,
including for two-mode networks.
}
\section{Closeness centrality}{

Closeness centrality, status centrality, or barycenter centrality is
defined as the reciprocal of the farness or distance, \eqn{d},
from a node to all other nodes in the network:
\deqn{C_C(i) = \frac{1}{\sum_j d(i,j)}}
When (more commonly) normalised, the numerator is instead \eqn{N-1}.
}

\section{Harmonic centrality}{

Harmonic centrality or valued centrality reverses the sum and reciprocal
operations compared to closeness centrality:
\deqn{C_H(i) = \sum_{i, i \neq j} \frac{1}{d(i,j)}}
where \eqn{\frac{1}{d(i,j)} = 0} where there is no path between \eqn{i} and
\eqn{j}. Normalization is by \eqn{N-1}.
Since the harmonic mean performs better than the arithmetic mean on
unconnected networks, i.e. networks with infinite distances,
harmonic centrality is to be preferred in these cases.
}

\section{Reach centrality}{

In some cases, longer path lengths are irrelevant and 'closeness' should
be defined as how many others are in a local neighbourhood.
How many steps out this neighbourhood should be defined as is given by
the 'cutoff' parameter.
This is usually termed \eqn{k} or \eqn{m} in equations,
which is why this is sometimes called (\eqn{m}- or)
\eqn{k}-step reach centrality:
\deqn{C_R(i) = \sum_j d(i,j) \leq k}
The maximum reach score is \eqn{N-1}, achieved when the node can reach all
other nodes in the network in \eqn{k} steps or less,
but the normalised version, \eqn{\frac{C_R}{N-1}}, is more common.
Note that if \eqn{k = 1} (i.e. cutoff = 1), then this returns the node's degree.
At higher cutoff reach centrality returns the size of the node's component.
}

\section{Information centrality}{

Information centrality, also known as current-flow centrality,
is a hybrid measure relating to both path-length and walk-based measures.
The information centrality of a node is the harmonic average of the
“bandwidth” or inverse path-length for all paths originating from the node.

As described in the \code{{sna}} package,
information centrality works on an undirected but potentially weighted
network excluding isolates (which take scores of zero).
It is defined as:
\deqn{C_I = \frac{1}{T + \frac{\sum T - 2 \sum C_1}{|N|}}}
where \eqn{C = B^-1} with \eqn{B} is a pseudo-adjacency matrix replacing
the diagonal of \eqn{1-A} with \eqn{1+k},
and \eqn{T} is the trace of \eqn{C} and \eqn{S_R} an arbitrary row sum
(all rows in \eqn{C} have the same sum).

Nodes with higher information centrality have a large number of short paths
to many others in the network, and are thus considered to have greater
control of the flow of information.
}

\section{Eccentricity centrality}{

Eccentricity centrality, graph centrality, or the Koenig number,
is the (if normalized, inverse of) the distance to the furthest node:
\deqn{C_E(i) = \frac{1}{max_{j \in N} d(i,j)}}
where the distance from \eqn{i} to \eqn{j} is \eqn{\infty} if unconnected.
As such it is only well defined for connected networks.
}

\section{Closeness vitality centrality}{

The closeness vitality of a node is the change in the sum of all distances
in a network, also known as the Wiener Index, when that node is removed.
Note that the closeness vitality may be negative infinity if
removing that node would disconnect the network.
Formally:
\deqn{C_V(i) = \sum_{j,k} d(j,k) - \sum_{j,k} d(j,k,G\ i)}
where \eqn{d(j,k,G\ i)} is the distance between nodes \eqn{j} and \eqn{k}
in the network with node \eqn{i} removed.
}

\section{Random walk closeness centrality}{

Random walk closeness centrality is based on the average length of
random walks starting at all other nodes to reach a given node.
It is defined as the inverse of the average hitting time to a node.
This means that higher values are given to nodes that can be reached
more quickly on average by random walks starting at other nodes.
Formally:
\deqn{C_{RW}(i) = \frac{1}{\frac{1}{N-1} \sum_{j \neq i} H_{ji}}}
where \eqn{H_{ji}} is the hitting time from node \eqn{j} to node \eqn{i}.
}

\examples{
node_by_closeness(ison_southern_women)
node_by_reach(ison_adolescents)
}
\references{
\subsection{On closeness centrality}{

Bavelas, Alex. 1950.
"Communication Patterns in Task‐Oriented Groups".
\emph{The Journal of the Acoustical Society of America}, 22(6): 725–730.
\doi{10.1121/1.1906679}

Harary, Frank. 1959.
"Status and Contrastatus".
\emph{Sociometry}, 22(1): 23–43.
\doi{10.2307/2785610}
}

\subsection{On harmonic centrality}{

Marchiori, Massimo, and Vito Latora. 2000.
"Harmony in the small-world".
\emph{Physica A} 285: 539-546.
\doi{10.1016/S0378-4371(00)00311-3}

Dekker, Anthony. 2005.
"Conceptual distance in social network analysis".
\emph{Journal of Social Structure} 6(3).
}

\subsection{On reach centrality}{

Borgatti, Stephen P., Martin G. Everett, and J.C. Johnson. 2013.
\emph{Analyzing social networks}.
London: SAGE Publications Limited.
}

\subsection{On information centrality}{

Stephenson, Karen, and Marvin Zelen. 1989.
"Rethinking centrality: Methods and examples".
\emph{Social Networks} 11(1):1-37.
\doi{10.1016/0378-8733(89)90016-6}

Brandes, Ulrik, and Daniel Fleischer. 2005.
"Centrality Measures Based on Current Flow".
\emph{Proc. 22nd Symp. Theoretical Aspects of Computer Science} LNCS 3404: 533-544.
\doi{10.1007/978-3-540-31856-9_44}
}

\subsection{On eccentricity centrality}{

Hage, Per, and Frank Harary. 1995.
"Eccentricity and centrality in networks".
\emph{Social Networks}, 17(1): 57-63.
\doi{10.1016/0378-8733(94)00248-9}
}

\subsection{On closeness vitality centrality}{

Koschuetzki, Dirk, Katharina Lehmann, Leon Peeters, Stefan Richter,
Dagmar Tenfelde-Podehl, and Oliver Zlotowski. 2005.
"Centrality Indices", in
Brandes, Ulrik, and Thomas Erlebach (eds.).
\emph{Network Analysis: Methodological Foundations}.
Springer: Berlin, pp. 16-61.
}

\subsection{On random walk closeness centrality}{

Noh, J.D. and R. Rieger. 2004.
"Random Walks on Complex Networks".
\emph{Physical Review Letters}, 92(11): 118701.
\doi{10.1103/PhysRevLett.92.118701}
}
}
\seealso{
Other closeness:
\code{\link{measure_centralisation_close}},
\code{\link{measure_centralities_close}}

Other centrality:
\code{\link{measure_central_between}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralisation_between}},
\code{\link{measure_centralisation_close}},
\code{\link{measure_centralisation_degree}},
\code{\link{measure_centralisation_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}}

Other measures:
\code{\link{measure_assort_net}},
\code{\link{measure_assort_node}},
\code{\link{measure_breadth}},
\code{\link{measure_broker_node}},
\code{\link{measure_broker_tie}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}},
\code{\link{measure_closure}},
\code{\link{measure_closure_node}},
\code{\link{measure_cohesion}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_net}},
\code{\link{measure_diverse_node}},
\code{\link{measure_features}},
\code{\link{measure_fragmentation}},
\code{\link{measure_hierarchy}},
\code{\link{measure_periods}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_node}},
\code{\link{motif_path}}
}
\concept{centrality}
\concept{closeness}
\concept{measures}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/member_community.R
\name{member_community_non}
\alias{member_community_non}
\alias{node_in_optimal}
\alias{node_in_partition}
\alias{node_in_infomap}
\alias{node_in_spinglass}
\alias{node_in_fluid}
\alias{node_in_louvain}
\alias{node_in_leiden}
\title{Memberships in non-hierarchical communities}
\usage{
node_in_optimal(.data)

node_in_partition(.data)

node_in_infomap(.data, times = 50)

node_in_spinglass(.data, max_k = 200, resolution = 1)

node_in_fluid(.data)

node_in_louvain(.data, resolution = 1)

node_in_leiden(.data, resolution = 1)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{times}{Integer indicating number of simulations/walks used.
By default, \code{times=50}.}

\item{max_k}{Integer constant, the number of spins to use as an upper limit
of communities to be found. Some sets can be empty at the end.}

\item{resolution}{The Reichardt-Bornholdt “gamma” resolution parameter for modularity.
By default 1, making existing and non-existing ties equally important.
Smaller values make existing ties more important,
and larger values make missing ties more important.}
}
\value{
A \code{node_member} character vector the length of the nodes in the network,
of group memberships "A", "B", etc for each node.
If the network is labelled,
then the assignments will be labelled with the nodes' names.
}
\description{
These functions offer algorithms for partitioning
networks into sets of communities:
\itemize{
\item \code{node_in_optimal()} is a problem-solving algorithm that seeks to maximise
modularity over all possible partitions.
\item \code{node_in_partition()} is a greedy, iterative, deterministic
partitioning algorithm that results in two equally-sized communities.
\item \code{node_in_infomap()} is an algorithm based on the information in random walks.
\item \code{node_in_spinglass()} is a greedy, iterative, probabilistic algorithm,
based on analogy to model from statistical physics.
\item \code{node_in_fluid()} is a propogation-based partitioning algorithm,
based on analogy to model from fluid dynamics.
\item \code{node_in_louvain()} is an agglomerative multilevel algorithm that seeks to maximise
modularity over all possible partitions.
\item \code{node_in_leiden()} is an agglomerative multilevel algorithm that seeks to maximise
the Constant Potts Model over all possible partitions.
}

The different algorithms offer various advantages in terms of computation time,
availability on different types of networks, ability to maximise modularity,
and their logic or domain of inspiration.
}
\section{Optimal}{

The general idea is to calculate the modularity of all possible partitions,
and choose the community structure that maximises this modularity measure.
Note that this is an NP-complete problem with exponential time complexity.
The guidance in the igraph package is networks of <50-200 nodes is probably fine.
}

\section{Partition}{

The general idea is to assign nodes to two groups, and then iteratively
swap pairs of nodes (one from each group) that give a positive sum of net tie costs,
where the net tie cost of a node is the difference between the sum
of the weights of ties to nodes in the other group (external costs) and
the sum of the weights of ties to nodes in the same group (internal costs).
This is a deterministic algorithm that will always return the same partition
for a given network, but it is not guaranteed to maximise modularity.
Note that this algorithm is only applicable to undirected, unipartite networks,
and will always return two communities of equal size (or as close to equal as possible).
}

\section{Infomap}{

Motivated by information theoretic principles, this algorithm tries to build
a grouping that provides the shortest description length for a random walk,
where the description length is measured by the expected number of bits
per node required to encode the path.
}

\section{Spin-glass}{

This is motivated by analogy to the Potts model in statistical physics.
Each node can be in one of \emph{k} "spin states",
and ties (particle interactions) provide information about which pairs of nodes
want similar or different spin states.
The final community definitions are represented by the nodes' spin states
after a number of updates.
A different implementation than the default is used in the case of signed networks,
such that nodes connected by negative ties will be more likely found in separate communities.
}

\section{Fluid}{

The general idea is to observe how a discrete number of fluids interact, expand and contract,
in a non-homogenous environment, i.e. the network structure.
Unlike the \code{{igraph}} implementation that this function wraps,
this function iterates over all possible numbers of communities and returns the membership
associated with the highest modularity.
}

\section{Louvain}{

The general idea is to take a hierarchical approach to optimising the modularity criterion.
Nodes begin in their own communities and are re-assigned in a local, greedy way:
each node is moved to the community where it achieves the highest contribution to modularity.
When no further modularity-increasing reassignments are possible,
the resulting communities are considered nodes (like a reduced graph),
and the process continues.
}

\section{Leiden}{

The general idea is to optimise the Constant Potts Model,
which does not suffer from the resolution limit, instead of modularity.
As outlined in the \code{{igraph}} package,
the Constant Potts Model object function is:

\deqn{\frac{1}{2m} \sum_{ij}(A_{ij}-\gamma n_i n_j)\delta(\sigma_i, \sigma_j)}

where \emph{m} is the total tie weight,
\eqn{A_{ij}} is the tie weight between \emph{i} and \emph{j},
\eqn{\gamma} is the so-called resolution parameter,
\eqn{n_i} is the node weight of node \emph{i},
and \eqn{\delta(\sigma_i, \sigma_j) = 1} if and only if
\emph{i} and \emph{j} are in the same communities and 0 otherwise.
Compared to the Louvain method, the Leiden algorithm additionally
tries to avoid unconnected communities.
}

\examples{
node_in_optimal(ison_adolescents)
node_in_partition(ison_adolescents)
node_in_partition(ison_southern_women)
node_in_infomap(ison_adolescents)
node_in_spinglass(ison_adolescents)
node_in_fluid(ison_adolescents)
node_in_louvain(ison_adolescents)
node_in_leiden(ison_adolescents)
}
\references{
\subsection{On optimal community detection}{

Brandes, Ulrik, Daniel Delling, Marco Gaertler, Robert Gorke, Martin Hoefer, Zoran Nikoloski, Dorothea Wagner. 2008.
"On Modularity Clustering",
\emph{IEEE Transactions on Knowledge and Data Engineering} 20(2):172-188.
}

\subsection{On partitioning community detection}{

Kernighan, Brian W., and Shen Lin. 1970.
"An efficient heuristic procedure for partitioning graphs."
\emph{The Bell System Technical Journal} 49(2): 291-307.
\doi{10.1002/j.1538-7305.1970.tb01770.x}
}

\subsection{On infomap community detection}{

Rosvall, M, and C. T. Bergstrom. 2008.
"Maps of information flow reveal community structure in complex networks",
\emph{PNAS} 105:1118.
\doi{10.1073/pnas.0706851105}

Rosvall, M., D. Axelsson, and C. T. Bergstrom. 2009.
"The map equation",
\emph{Eur. Phys. J. Special Topics} 178: 13.
\doi{10.1140/epjst/e2010-01179-1}
}

\subsection{On spinglass community detection}{

Reichardt, Jorg, and Stefan Bornholdt. 2006.
"Statistical Mechanics of Community Detection"
\emph{Physical Review E}, 74(1): 016110–14.
\doi{10.1073/pnas.0605965104}

Traag, Vincent A., and Jeroen Bruggeman. 2009.
"Community detection in networks with positive and negative links".
\emph{Physical Review E}, 80(3): 036115.
\doi{10.1103/PhysRevE.80.036115}
}

\subsection{On fluid community detection}{

Parés Ferran, Dario Garcia Gasulla, Armand Vilalta, Jonatan Moreno, Eduard Ayguade, Jesus Labarta, Ulises Cortes, and Toyotaro Suzumura. 2018.
"Fluid Communities: A Competitive, Scalable and Diverse Community Detection Algorithm".
In: \emph{Complex Networks & Their Applications VI}
Springer, 689: 229.
\doi{10.1007/978-3-319-72150-7_19}
}

\subsection{On Louvain community detection}{

Blondel, Vincent, Jean-Loup Guillaume, Renaud Lambiotte, Etienne Lefebvre. 2008.
"Fast unfolding of communities in large networks",
\emph{J. Stat. Mech.} P10008.
}

\subsection{On Leiden community detection}{

Traag, Vincent A., Ludo Waltman, and Nees Jan van Eck. 2019.
"From Louvain to Leiden: guaranteeing well-connected communities",
\emph{Scientific Reports}, 9(1):5233.
\doi{10.1038/s41598-019-41695-z}
}
}
\seealso{
Other community:
\code{\link{member_community}},
\code{\link{member_community_hier}}

Other memberships:
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_node}},
\code{\link{motif_path}}
}
\concept{community}
\concept{memberships}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_change.R
\name{measure_periods}
\alias{measure_periods}
\alias{net_by_waves}
\title{Measures of network change}
\usage{
net_by_waves(.data)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}
}
\value{
A \code{network_measure} numeric score.
}
\description{
\code{net_by_waves()} measures the number of waves in longitudinal network data.
}
\seealso{
Other change:
\code{\link{motif_periods}}

Other measures:
\code{\link{measure_assort_net}},
\code{\link{measure_assort_node}},
\code{\link{measure_breadth}},
\code{\link{measure_broker_node}},
\code{\link{measure_broker_tie}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}},
\code{\link{measure_closure}},
\code{\link{measure_closure_node}},
\code{\link{measure_cohesion}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_net}},
\code{\link{measure_diverse_node}},
\code{\link{measure_features}},
\code{\link{measure_fragmentation}},
\code{\link{measure_hierarchy}}
}
\concept{change}
\concept{measures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_centrality_between.R
\name{measure_central_between}
\alias{measure_central_between}
\alias{node_by_betweenness}
\alias{node_by_induced}
\alias{node_by_flow}
\alias{node_by_stress}
\title{Measuring nodes betweenness-like centrality}
\usage{
node_by_betweenness(.data, normalized = TRUE, cutoff = NULL)

node_by_induced(.data, normalized = TRUE, cutoff = NULL)

node_by_flow(.data, normalized = TRUE)

node_by_stress(.data, normalized = TRUE)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{normalized}{Logical scalar, whether scores are normalized.
Different denominators may be used depending on the measure,
whether the object is one-mode or two-mode, and other arguments.
By default TRUE.}

\item{cutoff}{The maximum path length to consider when calculating betweenness.
If negative or NULL (the default), there's no limit to the path lengths considered.}
}
\value{
A \code{node_measure} numeric vector the length of the nodes in the network,
providing the scores for each node.
If the network is labelled,
then the scores will be labelled with the nodes' names.
}
\description{
These functions calculate common betweenness-related centrality measures for one- and two-mode networks:
\itemize{
\item \code{node_by_betweenness()} measures the betweenness centralities of nodes in a network.
\item \code{node_by_induced()} measures the induced betweenness centralities of nodes in a network.
\item \code{node_by_flow()} measures the flow betweenness centralities of nodes in a network,
which uses an electrical current model for information spreading
in contrast to the shortest paths model used by normal betweenness centrality.
\item \code{node_by_stress()} measures the stress centrality of nodes in a network.
\item \code{tie_by_betweenness()} measures the number of shortest paths going through a tie.
}

All measures attempt to use as much information as they are offered,
including whether the networks are directed, weighted, or multimodal.
If this would produce unintended results,
first transform the salient properties using e.g. \code{\link[manynet:to_undirected]{to_undirected()}} functions.
All centrality and centralization measures return normalized measures by default,
including for two-mode networks.
}
\section{Betweenness centrality}{

Betweenness centrality is based on the number of shortest paths between
other nodes that a node lies upon:
\deqn{C_B(i) = \sum_{j,k:j \neq k, j \neq i, k \neq i} \frac{g_{jik}}{g_{jk}}}
}

\section{Induced centrality}{

Induced centrality or vitality centrality concerns the change in
total betweenness centrality between networks with and without a given node:
\deqn{C_I(i) = C_B(G) - C_B(G\ i)}
}

\section{Flow betweenness centrality}{

Flow betweenness centrality concerns the total maximum flow, \eqn{f},
between other nodes \eqn{j,k} in a network \eqn{G} that a given node mediates:
\deqn{C_F(i) = \sum_{j,k:j\neq k, j\neq i, k\neq i} f(j,k,G) - f(j,k,G\ i)}
When normalized (by default) this sum of differences is divided by the
sum of flows \eqn{f(i,j,G)}.
}

\section{Stress centrality}{

Stress centrality is the number of all shortest paths or geodesics, \eqn{g},
between other nodes that a given node mediates:
\deqn{C_S(i) = \sum_{j,k:j \neq k, j \neq i, k \neq i} g_{jik}}
High stress nodes lie on a large number of shortest paths between other
nodes, and thus associated with bridging or spanning boundaries.
}

\examples{
node_by_betweenness(ison_southern_women)
node_by_induced(ison_adolescents)
}
\references{
\subsection{On betweenness centrality}{

Freeman, Linton. 1977.
"A set of measures of centrality based on betweenness".
\emph{Sociometry}, 40(1): 35–41.
\doi{10.2307/3033543}
}

\subsection{On induced centrality}{

Everett, Martin and Steve Borgatti. 2010.
"Induced, endogenous and exogenous centrality"
\emph{Social Networks}, 32: 339-344.
\doi{10.1016/j.socnet.2010.06.004}
}

\subsection{On flow centrality}{

Freeman, Lin, Stephen Borgatti, and Douglas White. 1991.
"Centrality in Valued Graphs: A Measure of Betweenness Based on Network Flow".
\emph{Social Networks}, 13(2), 141-154.

Koschutzki, D., K.A. Lehmann, L. Peeters, S. Richter, D. Tenfelde-Podehl, and O. Zlotowski. 2005.
"Centrality Indices".
In U. Brandes and T. Erlebach (eds.), \emph{Network Analysis: Methodological Foundations}.
Berlin: Springer.
}

\subsection{On stress centrality}{

Shimbel, A. 1953.
"Structural Parameters of Communication Networks".
\emph{Bulletin of Mathematical Biophysics}, 15:501-507.
\doi{10.1007/BF02476438}
}
}
\seealso{
Other betweenness:
\code{\link{measure_centralisation_between}},
\code{\link{measure_centralities_between}}

Other centrality:
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralisation_between}},
\code{\link{measure_centralisation_close}},
\code{\link{measure_centralisation_degree}},
\code{\link{measure_centralisation_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}}

Other measures:
\code{\link{measure_assort_net}},
\code{\link{measure_assort_node}},
\code{\link{measure_breadth}},
\code{\link{measure_broker_node}},
\code{\link{measure_broker_tie}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}},
\code{\link{measure_closure}},
\code{\link{measure_closure_node}},
\code{\link{measure_cohesion}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_net}},
\code{\link{measure_diverse_node}},
\code{\link{measure_features}},
\code{\link{measure_fragmentation}},
\code{\link{measure_hierarchy}},
\code{\link{measure_periods}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_node}},
\code{\link{motif_path}}
}
\concept{betweenness}
\concept{centrality}
\concept{measures}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mark_ties.R
\name{mark_dyads}
\alias{mark_dyads}
\alias{tie_is_multiple}
\alias{tie_is_reciprocated}
\title{Marking ties based on dyadic properties}
\usage{
tie_is_multiple(.data)

tie_is_reciprocated(.data)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}
}
\value{
A \code{tie_mark} logical vector the length of the ties in the network,
giving either TRUE or FALSE for each tie depending on
whether the condition is matched.
}
\description{
These functions return logical vectors the length of the ties
in a network identifying which are embedded within particular dyads.
\itemize{
\item \code{tie_is_multiple()} marks ties that are multiples.
\item \code{tie_is_reciprocated()} marks ties that are mutual/reciprocated.
}

They are most useful in highlighting parts of the network where
relationships are denser.
}
\examples{
tie_is_multiple(fict_marvel)
tie_is_reciprocated(ison_algebra)
}
\seealso{
Other marks:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{mark_select_tie}},
\code{\link{mark_ties}},
\code{\link{mark_triangles}}

Other tie:
\code{\link{mark_select_tie}},
\code{\link{mark_ties}},
\code{\link{mark_triangles}},
\code{\link{measure_broker_tie}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}}
}
\concept{marks}
\concept{tie}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/motif_census.R
\name{motif_node}
\alias{motif_node}
\alias{node_x_dyad}
\alias{node_x_triad}
\alias{node_x_tetrad}
\title{Motifs of nodes cohesion}
\usage{
node_x_dyad(.data)

node_x_triad(.data)

node_x_tetrad(.data)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}
}
\value{
A \code{node_motif} matrix with one row for each node in the network and
a column for each motif type,
giving the count of each motif in which each node participates.
It is printed as a tibble, however, to avoid greedy printing.
If the network is labelled,
then the node names will be in a column named \code{names}.
}
\description{
These functions include ways to take a census of the positions of nodes
in a network:
\itemize{
\item \code{node_x_tie()} returns a census of the ties in a network.
For directed networks, out-ties and in-ties are bound together.
For multiplex networks, the various types of ties are bound together.
\item \code{node_x_triad()} returns a census of the triad configurations
nodes are embedded in.
\item \code{node_x_tetrad()} returns a census of nodes' positions
in motifs of four nodes.
\item \code{node_x_path()} returns the shortest path lengths
of each node to every other node in the network.
}
}
\section{Tetrad census}{

The nodal tetrad census counts the number of four-node configurations
that each node is embedded in.
The function returns a matrix with a special naming convention:
\itemize{
\item E4 (aka co-K4): This is an empty set of four nodes; no ties
\item I4 (aka co-diamond): This is a set of four nodes with just one tie
\item H4 (aka co-C4): This set of four nodes includes two non-adjacent ties
\item L4 (aka co-paw): This set of four nodes includes two adjacent ties
\item D4 (aka co-claw): This set of four nodes includes three adjacent ties,
in the form of a triangle with one isolate
\item U4 (aka P4, four-actor line): This set of four nodes includes three ties
arranged in a line
\item Y4 (aka claw): This set of four nodes includes three ties all adjacent
to a single node
\item P4 (aka paw, kite): This set of four nodes includes four ties arranged
as a triangle with an extra tie hanging off of one of the nodes
\item C4 (aka bifan): This is a symmetric box or 4-cycle or set of shared choices
\item Z4 (aka diamond): This resembles C4 but with an extra tie cutting across the box
\item X4 (aka K4): This resembles C4 but with two extra ties cutting across the box;
a realisation of all possible ties
}

Graphs of these motifs can be shown using
\code{plot(node_by_tetrad(ison_southern_women))}.
}

\examples{
node_x_dyad(ison_networkers)
task_eg <- to_named(to_uniplex(ison_algebra, "tasks"))
(triad_cen <- node_x_triad(task_eg))
node_x_tetrad(ison_southern_women)
}
\references{
\subsection{On the dyad census}{

Holland, Paul W., and Samuel Leinhardt. 1970.
"A Method for Detecting Structure in Sociometric Data".
\emph{American Journal of Sociology}, 76: 492-513.
\doi{10.1016/B978-0-12-442450-0.50028-6}
}

\subsection{On the triad census}{

Davis, James A., and Samuel Leinhardt. 1967.
“\href{https://files.eric.ed.gov/fulltext/ED024086.pdf}{The Structure of Positive Interpersonal Relations in Small Groups}.” 55.
}

\subsection{On the tetrad census}{

Ortmann, Mark, and Ulrik Brandes. 2017.
“Efficient Orbit-Aware Triad and Quad Census in Directed and Undirected Graphs.”
\emph{Applied Network Science} 2(1):13.
\doi{10.1007/s41109-017-0027-2}.

McMillan, Cassie, and Diane Felmlee. 2020.
"Beyond Dyads and Triads: A Comparison of Tetrads in Twenty Social Networks".
\emph{Social Psychology Quarterly} 83(4): 383-404.
\doi{10.1177/0190272520944151}
}
}
\seealso{
Other cohesion:
\code{\link{mark_triangles}},
\code{\link{measure_breadth}},
\code{\link{measure_cohesion}},
\code{\link{measure_fragmentation}},
\code{\link{motif_net}}

Other motifs:
\code{\link{motif_brokerage_net}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_hazard}},
\code{\link{motif_hierarchy}},
\code{\link{motif_net}},
\code{\link{motif_path}},
\code{\link{motif_periods}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_path}}
}
\concept{cohesion}
\concept{motifs}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mark_nodes.R
\name{mark_select_node}
\alias{mark_select_node}
\alias{node_is_random}
\alias{node_is_max}
\alias{node_is_min}
\alias{node_is_mean}
\title{Marking nodes based on measures}
\usage{
node_is_random(.data, select = 1)

node_is_max(node_measure, ranks = 1)

node_is_min(node_measure, ranks = 1)

node_is_mean(node_measure, ranks = 1)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{select}{Number of elements to select (as TRUE).}

\item{node_measure}{An object created by a \code{node_} measure.}

\item{ranks}{The number of ranks of max or min to return.
For example, \code{ranks = 3} will return TRUE for nodes with
scores equal to any of the top (or, for \code{node_is_min()}, bottom)
three scores.
By default, \code{ranks = 1}.}
}
\value{
A \code{node_mark} logical vector the length of the nodes in the network,
giving either TRUE or FALSE for each node depending on
whether the condition is matched.
}
\description{
These functions return logical vectors the length of the
nodes in a network identifying which hold certain properties or positions in the network.
\itemize{
\item \code{node_is_random()} marks one or more nodes at random.
\item \code{node_is_max()} and \code{node_is_min()} are more generally useful
for converting the results from some node measure into a mark-class object.
They can be particularly useful for highlighting which node or nodes
are key because they minimise or, more often, maximise some measure.
}
}
\examples{
node_is_random(ison_brandes, 2)
node_is_max(node_by_degree(ison_brandes))
node_is_min(node_by_degree(ison_brandes))
node_is_mean(node_by_degree(ison_brandes))
}
\seealso{
Other selection:
\code{\link{mark_select_tie}}

Other marks:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_dyads}},
\code{\link{mark_nodes}},
\code{\link{mark_select_tie}},
\code{\link{mark_ties}},
\code{\link{mark_triangles}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_node}},
\code{\link{motif_path}}
}
\concept{marks}
\concept{nodal}
\concept{selection}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/member_community.R
\name{member_community_hier}
\alias{member_community_hier}
\alias{node_in_betweenness}
\alias{node_in_greedy}
\alias{node_in_eigen}
\alias{node_in_walktrap}
\title{Memberships in hierarchical communities}
\usage{
node_in_betweenness(.data)

node_in_greedy(.data)

node_in_eigen(.data)

node_in_walktrap(.data, times = 50)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{times}{Integer indicating number of simulations/walks used.
By default, \code{times=50}.}
}
\value{
A \code{node_member} character vector the length of the nodes in the network,
of group memberships "A", "B", etc for each node.
If the network is labelled,
then the assignments will be labelled with the nodes' names.
}
\description{
These functions offer algorithms for hierarchically clustering
networks into communities. Since all of the following are hierarchical,
their dendrograms can be plotted:
\itemize{
\item \code{node_in_betweenness()} is a hierarchical, decomposition algorithm
where edges are removed in decreasing order of the number of
shortest paths passing through the edge.
\item \code{node_in_greedy()} is a hierarchical, agglomerative algorithm,
that tries to optimize modularity in a greedy manner.
\item \code{node_in_eigen()} is a top-down, hierarchical algorithm.
\item \code{node_in_walktrap()} is a hierarchical, agglomerative algorithm based on random walks.
}

The different algorithms offer various advantages in terms of computation time,
availability on different types of networks, ability to maximise modularity,
and their logic or domain of inspiration.
}
\section{Edge-betweenness}{

This is motivated by the idea that edges connecting different groups
are more likely to lie on multiple shortest paths when they are the
only option to go from one group to another.
This method yields good results but is very slow because of
the computational complexity of edge-betweenness calculations and
the betweenness scores have to be re-calculated after every edge removal.
Networks of ~700 nodes and ~3500 ties are around the upper size limit
that are feasible with this approach.
}

\section{Fast-greedy}{

Initially, each node is assigned a separate community.
Communities are then merged iteratively such that each merge
yields the largest increase in the current value of modularity,
until no further increases to the modularity are possible.
The method is fast and recommended as a first approximation
because it has no parameters to tune.
However, it is known to suffer from a resolution limit.
}

\section{Leading eigenvector}{

In each step, the network is bifurcated such that modularity increases most.
The splits are determined according to the leading eigenvector of the modularity matrix.
A stopping condition prevents tightly connected groups from being split further.
Note that due to the eigenvector calculations involved,
this algorithm will perform poorly on degenerate networks,
but will likely obtain a higher modularity than fast-greedy (at some cost of speed).
}

\section{Walktrap}{

The general idea is that random walks on a network are more likely to stay
within the same community because few edges lead outside a community.
By repeating random walks of 4 steps many times,
information about the hierarchical merging of communities is collected.
}

\examples{
node_in_betweenness(ison_adolescents)
node_in_greedy(ison_adolescents)
node_in_eigen(ison_adolescents)
node_in_walktrap(ison_adolescents)
}
\references{
\subsection{On edge-betweenness community detection}{

Newman, Mark, and Michelle Girvan. 2004.
"Finding and evaluating community structure in networks."
\emph{Physical Review E} 69: 026113.
\doi{10.1103/PhysRevE.69.026113}
}

\subsection{On fast-greedy community detection}{

Clauset, Aaron, Mark E.J. Newman, and Cristopher Moore. 2004.
"Finding community structure in very large networks."
\emph{Physical Review E}, 70: 066111.
\doi{10.1103/PhysRevE.70.066111}
}

\subsection{On leading eigenvector community detection}{

Newman, Mark E.J. 2006.
"Finding community structure using the eigenvectors of matrices"
\emph{Physical Review E} 74:036104.
\doi{10.1103/PhysRevE.74.036104}
}

\subsection{On walktrap community detection}{

Pons, Pascal, and Matthieu Latapy. 2005.
"Computing communities in large networks using random walks".
1-20.
\doi{10.48550/arXiv.physics/0512106}
}
}
\seealso{
Other memberships:
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_node}},
\code{\link{motif_path}}

Other community:
\code{\link{member_community}},
\code{\link{member_community_non}}
}
\concept{community}
\concept{memberships}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_heterogeneity.R
\name{measure_assort_net}
\alias{measure_assort_net}
\alias{net_by_heterophily}
\alias{net_by_homophily}
\alias{net_by_assortativity}
\alias{net_by_spatial}
\title{Measures of network assortativity}
\usage{
net_by_heterophily(.data, attribute)

net_by_homophily(
  .data,
  attribute,
  assortativity = c("ie", "ei", "yule", "geary")
)

net_by_assortativity(.data)

net_by_spatial(.data, attribute)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{attribute}{Name of a nodal attribute, mark, measure, or membership vector.}

\item{assortativity}{Which method to use for \verb{*_homophily()}.
Either "ie" (negative E-I index), "ei" (E-I index),
"yule" (Yule's Q), or "geary" (Geary's C).
Default is "ie".
The E-I index is the number of ties between (or \emph{external}) nodes
grouped in some mutually exclusive categories
minus the number of ties within (or \emph{internal}) these groups
divided by the total number of ties.
This value can range from 1 to -1,
where 1 indicates ties only between categories/groups and -1 ties only within categories/groups.

Yule's Q is a measure of association for two binary variables, calculated as:
\deqn{Q = \frac{ad - bc}{ad + bc}}
where \eqn{a} is the number of ties between nodes in the same category,
\eqn{b} is the number of ties between nodes in different categories,
\eqn{c} is the number of non-ties between nodes in the same category,
and \eqn{d} is the number of non-ties between nodes in different categories.
This value can range from -1 to 1, where 1 indicates perfect association
(all ties are between nodes in the same category), -1 indicates perfect disassociation
(all ties are between nodes in different categories), and 0 indicates no association.

Geary's C is a measure of spatial autocorrelation, calculated as:
\deqn{C = \frac{(n - 1) \sum
\limits_{i=1}^n \sum\limits_{j=1}^n w_{ij} (x_i - x_j)^2}{2W \sum\limits_{i=1}^n (x_i - \bar{x})^2}}
where \eqn{n} is the number of nodes, \eqn{
w_{ij}} is the weight of the tie between nodes \eqn{i} and \eqn{j},
\eqn{x_i} is the attribute value of node \eqn{i},
\eqn{\bar{x}} is the mean attribute value across all nodes, and \eqn{W} is the sum of all tie weights.
This value can range from 0 to 2, where values less than 1
indicate positive autocorrelation (similar values are more likely to be connected),
values greater than 1 indicate negative autocorrelation (dissimilar values are more likely
to be connected), and a value of 1 indicates no autocorrelation.
If an incompatible method is chosen for the attribute type,
a suitable alternative will be used instead with a message.}
}
\value{
A \code{network_measure} numeric score.
}
\description{
These functions offer ways to measure the distribution or assortativity
of ties in a network:
\itemize{
\item \code{net_by_heterophily()} measures how embedded nodes in the network
are within groups of nodes with the same attribute.
\item \code{net_by_homophily()} measures how embedded nodes in the network
are within groups of nodes with the same attribute, but with more options for method.
\item \code{net_by_assortativity()} measures the degree assortativity in a network.
\item \code{net_by_spatial()} measures the spatial association/autocorrelation
(global Moran's I) in a network.
}

Note that for two-mode networks,
homophily is calculated on the mode with the attribute of interest,
and the other mode is ignored.
If the attribute is present on both modes,
homophily is calculated on the first mode by default,
but a message is given and the user can choose to calculate homophily on the
other mode instead by subsetting the attribute vector and
converting the network to a one-mode network.
}
\section{Heterophily}{

Given a partition of a network into a number of mutually exclusive groups then
The E-I index is the number of ties between (or \emph{external}) nodes
grouped in some mutually exclusive categories
minus the number of ties within (or \emph{internal}) these groups
divided by the total number of ties.
This value can range from 1 to -1,
where 1 indicates ties only between categories/groups and -1 ties only within categories/groups.
}

\examples{
marvel_friends <- to_unsigned(to_uniplex(fict_marvel, "relationship"), "positive")
net_by_heterophily(marvel_friends, "Gender")
net_by_heterophily(marvel_friends, "Attractive")
net_by_homophily(marvel_friends, "Gender")
net_by_assortativity(ison_networkers)
net_by_spatial(ison_lawfirm, "age")
}
\references{
\subsection{On heterophily}{

Krackhardt, David, and Robert N. Stern. 1988.
Informal networks and organizational crises: an experimental simulation.
\emph{Social Psychology Quarterly} 51(2): 123-140.
\doi{10.2307/2786835}

McPherson, Miller, Lynn Smith-Lovin, and James M. Cook. 2001.
"Birds of a Feather: Homophily in Social Networks".
\emph{Annual Review of Sociology}, 27(1): 415-444.
\doi{10.1146/annurev.soc.27.1.415}
}

\subsection{On assortativity}{

Newman, Mark E.J. 2002.
"Assortative mixing in networks".
\emph{Physical Review Letters}, 89(20): 208701.
\doi{10.1103/physrevlett.89.208701}
}

\subsection{On spatial autocorrelation}{

Moran, Patrick Alfred Pierce. 1950.
"Notes on continuous stochastic phenomena".
\emph{Biometrika} 37(1): 17-23.
\doi{10.2307/2332142}
}
}
\seealso{
Other diversity:
\code{\link{measure_assort_node}},
\code{\link{measure_diverse_net}},
\code{\link{measure_diverse_node}}

Other measures:
\code{\link{measure_assort_node}},
\code{\link{measure_breadth}},
\code{\link{measure_broker_node}},
\code{\link{measure_broker_tie}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}},
\code{\link{measure_closure}},
\code{\link{measure_closure_node}},
\code{\link{measure_cohesion}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_net}},
\code{\link{measure_diverse_node}},
\code{\link{measure_features}},
\code{\link{measure_fragmentation}},
\code{\link{measure_hierarchy}},
\code{\link{measure_periods}}
}
\concept{diversity}
\concept{measures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_cohesion.R
\name{measure_breadth}
\alias{measure_breadth}
\alias{net_by_diameter}
\alias{net_by_length}
\title{Measures of network breadth}
\usage{
net_by_diameter(.data)

net_by_length(.data)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}
}
\value{
A \code{network_measure} numeric score.
}
\description{
These functions return values or vectors relating to how broad a network is.
\itemize{
\item \code{net_by_diameter()} measures the maximum path length in the network.
\item \code{net_by_length()} measures the average path length in the network.
}
}
\examples{
net_by_diameter(fict_marvel)
net_by_diameter(to_giant(fict_marvel))
net_by_length(fict_marvel)
net_by_length(to_giant(fict_marvel))
}
\seealso{
Other cohesion:
\code{\link{mark_triangles}},
\code{\link{measure_cohesion}},
\code{\link{measure_fragmentation}},
\code{\link{motif_net}},
\code{\link{motif_node}}

Other measures:
\code{\link{measure_assort_net}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_broker_tie}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}},
\code{\link{measure_closure}},
\code{\link{measure_closure_node}},
\code{\link{measure_cohesion}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_infection}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_net}},
\code{\link{measure_diverse_node}},
\code{\link{measure_features}},
\code{\link{measure_fragmentation}},
\code{\link{measure_hierarchy}},
\code{\link{measure_periods}}
}
\concept{cohesion}
\concept{measures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mark_ties.R
\name{mark_triangles}
\alias{mark_triangles}
\alias{tie_is_triangular}
\alias{tie_is_transitive}
\alias{tie_is_triplet}
\alias{tie_is_cyclical}
\alias{tie_is_simmelian}
\alias{tie_is_imbalanced}
\title{Marking ties based on triangular properties}
\usage{
tie_is_triangular(.data)

tie_is_transitive(.data)

tie_is_triplet(.data)

tie_is_cyclical(.data)

tie_is_simmelian(.data)

tie_is_imbalanced(.data)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}
}
\value{
A \code{tie_mark} logical vector the length of the ties in the network,
giving either TRUE or FALSE for each tie depending on
whether the condition is matched.
}
\description{
These functions return logical vectors the length of the ties
in a network identifying which hold certain properties or positions in the network.
\itemize{
\item \code{tie_is_triangular()} marks ties that are part of triangles.
\item \code{tie_is_cyclical()} marks ties that are part of cycles.
\item \code{tie_is_triplet()} marks ties that are part of transitive triplets.
\item \code{tie_is_simmelian()} marks ties that are both in a triangle
and fully reciprocated.
\item \code{tie_is_imbalanced()} marks ties that are part of imbalanced triads.
\item \code{tie_is_transitive()} marks ties that complete transitive closure.
}

They are most useful in highlighting parts of the network that
are cohesively connected.
}
\examples{
ison_monks |> to_uniplex("like") |> 
  mutate_ties(tri = tie_is_triangular())
ison_adolescents |> to_directed() |> 
  mutate_ties(trans = tie_is_transitive())
ison_adolescents |> to_directed() |> 
  mutate_ties(trip = tie_is_triplet())
ison_adolescents |> to_directed() |> 
  mutate_ties(cyc = tie_is_cyclical())
ison_monks |> to_uniplex("like") |> 
  mutate_ties(simmel = tie_is_simmelian())
fict_marvel |> to_uniplex("relationship") |> tie_is_imbalanced()
}
\seealso{
Other marks:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_dyads}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{mark_select_tie}},
\code{\link{mark_ties}}

Other tie:
\code{\link{mark_dyads}},
\code{\link{mark_select_tie}},
\code{\link{mark_ties}},
\code{\link{measure_broker_tie}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}}

Other cohesion:
\code{\link{measure_breadth}},
\code{\link{measure_cohesion}},
\code{\link{measure_fragmentation}},
\code{\link{motif_net}},
\code{\link{motif_node}}
}
\concept{cohesion}
\concept{marks}
\concept{tie}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_centrality_closeness.R
\name{measure_centralisation_close}
\alias{measure_centralisation_close}
\alias{net_by_closeness}
\alias{mode_by_closeness}
\alias{net_by_reach}
\alias{net_by_harmonic}
\title{Measuring networks closeness-like centralisation}
\usage{
net_by_closeness(.data, normalized = TRUE, direction = c("all", "out", "in"))

mode_by_closeness(.data, normalized = TRUE, direction = c("all", "out", "in"))

net_by_reach(.data, normalized = TRUE, cutoff = 2)

net_by_harmonic(.data, normalized = TRUE, cutoff = 2)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{normalized}{Logical scalar, whether scores are normalized.
Different denominators may be used depending on the measure,
whether the object is one-mode or two-mode, and other arguments.
By default TRUE.}

\item{direction}{Character string, “out” bases the measure on outgoing ties,
“in” on incoming ties, and "all" on either/the sum of the two.
By default "all".}

\item{cutoff}{The maximum path length to consider when calculating betweenness.
If negative or NULL (the default), there's no limit to the path lengths considered.}
}
\value{
\verb{net_by_*()} functions return a \code{network_measure} scalar;
\code{mode_by_closeness()} returns a \code{mode_measure} numeric vector of length two,
giving one centralization score per mode.
}
\description{
\itemize{
\item \code{net_by_closeness()} measures a network's closeness centralization as a
single score.
\item \code{mode_by_closeness()} measures closeness centralization separately for each
mode of a two-mode network, returning one score per mode
(following Borgatti and Everett, 1997).
\item \code{net_by_reach()} measures a network's reach centralization.
\item \code{net_by_harmonic()} measures a network's harmonic centralization.
}

All measures attempt to use as much information as they are offered,
including whether the networks are directed, weighted, or multimodal.
If this would produce unintended results,
first transform the salient properties using e.g. \code{\link[manynet:to_undirected]{to_undirected()}} functions.
All centrality and centralization measures return normalized measures by default,
including for two-mode networks.

For two-mode networks the two modes have different theoretical maxima, so
\code{net_by_closeness()} reports a single network-level score by applying
Freeman's general centralization index over the normalized node closeness
scores, whereas \code{mode_by_closeness()} reports the per-mode scores directly.
}
\examples{
net_by_closeness(ison_southern_women, direction = "in")
mode_by_closeness(ison_southern_women, direction = "in")
}
\references{
Borgatti, Stephen P., and Martin G. Everett. 1997.
"Network analysis of 2-mode data."
\emph{Social Networks} 19(3): 243-269.
\doi{10.1016/S0378-8733(96)00301-2}
}
\seealso{
Other closeness:
\code{\link{measure_central_close}},
\code{\link{measure_centralities_close}}

Other centrality:
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralisation_between}},
\code{\link{measure_centralisation_degree}},
\code{\link{measure_centralisation_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}}
}
\concept{centrality}
\concept{closeness}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mark_nodes.R
\name{mark_nodes}
\alias{mark_nodes}
\alias{node_is_independent}
\alias{node_is_cutpoint}
\alias{node_is_fold}
\alias{node_is_mentor}
\alias{node_is_neighbor}
\title{Marking nodes based on structural properties}
\usage{
node_is_independent(.data)

node_is_cutpoint(.data)

node_is_fold(.data)

node_is_mentor(.data, elites = 0.1)

node_is_neighbor(.data, node)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{elites}{The proportion of nodes to be selected as mentors.
By default this is set at 0.1.
This means that the top 10\% of nodes in terms of degree,
or those equal to the highest rank degree in the network,
whichever is the higher, will be used to select the mentors.

Note that if nodes are equidistant from two mentors,
they will choose one at random.
If a node is without a path to a mentor,
for example because they are an isolate,
a tie to themselves (a loop) will be created instead.
Note that this is a different default behaviour than that
described in Valente and Davis (1999).}

\item{node}{A node ID or name for which neighbors are to be marked.}
}
\value{
A \code{node_mark} logical vector the length of the nodes in the network,
giving either TRUE or FALSE for each node depending on
whether the condition is matched.
}
\description{
These functions return logical vectors the length of the
nodes in a network identifying which hold certain properties or positions in the network.
\itemize{
\item \code{node_is_independent()} marks nodes that are members of the largest independent set,
aka largest internally stable set.
\item \code{node_is_cutpoint()} marks nodes that cut or act as articulation points in a network,
increasing the number of connected components when removed.
\item \code{node_is_fold()} marks nodes that are in a structural fold between two or more
triangles that are only connected by that node.
\item \code{node_is_mentor()} marks a proportion of high indegree nodes as 'mentors' (see details).
\item \code{node_is_neighbor()} marks nodes that are neighbours of a given node.
}
}
\examples{
node_is_independent(ison_adolescents)
node_is_cutpoint(ison_brandes)
node_is_fold(create_explicit(A-B, B-C, A-C, C-D, C-E, D-E))
}
\references{
\subsection{On independent sets}{

Tsukiyama, Shuji, Mikio Ide, Hiromu Ariyoshi, and Isao Shirawaka. 1977.
"A new algorithm for generating all the maximal independent sets".
\emph{SIAM Journal on Computing}, 6(3):505–517.
\doi{10.1137/0206036}
}

\subsection{On articulation or cut-points}{

Tarjan, Robert E. and Uzi Vishkin. 1985.
"An Efficient Parallel Biconnectivity Algorithm",
\emph{SIAM Journal on Computing} 14(4): 862-874.
\doi{10.1137/0214061}
}

\subsection{On structural folds}{

Vedres, Balazs, and David Stark. 2010.
"Structural folds: Generative disruption in overlapping groups",
\emph{American Journal of Sociology} 115(4): 1150-1190.
\doi{10.1086/649497}
}

\subsection{On mentoring}{

Valente, Thomas, and Rebecca Davis. 1999.
"Accelerating the Diffusion of Innovations Using Opinion Leaders",
\emph{Annals of the American Academy of Political and Social Science} 566: 56-67.
}
}
\seealso{
Other marks:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_dyads}},
\code{\link{mark_select_node}},
\code{\link{mark_select_tie}},
\code{\link{mark_ties}},
\code{\link{mark_triangles}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_components}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_node}},
\code{\link{motif_path}}
}
\concept{marks}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/member_components.R
\name{member_components}
\alias{member_components}
\alias{node_in_component}
\alias{node_in_weak}
\alias{node_in_strong}
\title{Memberships in components}
\usage{
node_in_component(.data)

node_in_weak(.data)

node_in_strong(.data)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}
}
\value{
A \code{node_member} character vector the length of the nodes in the network,
of group memberships "A", "B", etc for each node.
If the network is labelled,
then the assignments will be labelled with the nodes' names.
}
\description{
These functions create a vector of nodes' memberships in components:
\itemize{
\item \code{node_in_component()} assigns nodes' component membership
using edge direction where available.
\item \code{node_in_weak()} assigns nodes' component membership
ignoring edge direction.
\item \code{node_in_strong()} assigns nodes' component membership
based on edge direction.
}

In graph theory, components, sometimes called connected components,
are induced subgraphs from partitioning the nodes into disjoint sets.
All nodes that are members of the same partition as \emph{i} are reachable
from \emph{i}.

For directed networks,
strongly connected components consist of subgraphs where there are paths
in each direction between member nodes.
Weakly connected components consist of subgraphs where there is a path
in either direction between member nodes.
}
\examples{
ison_monks |> to_uniplex("esteem") |>
  mutate_nodes(comp = node_in_component())
}
\seealso{
Other memberships:
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}}

Other nodal:
\code{\link{mark_core}},
\code{\link{mark_degree}},
\code{\link{mark_diff}},
\code{\link{mark_nodes}},
\code{\link{mark_select_node}},
\code{\link{measure_assort_node}},
\code{\link{measure_broker_node}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_closure_node}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_node}},
\code{\link{member_brokerage}},
\code{\link{member_cliques}},
\code{\link{member_community}},
\code{\link{member_community_hier}},
\code{\link{member_community_non}},
\code{\link{member_core}},
\code{\link{member_diffusion}},
\code{\link{member_equivalence}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_node}},
\code{\link{motif_path}}
}
\concept{memberships}
\concept{nodal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/measure_diffusion.R
\name{measure_diffusion_infection}
\alias{measure_diffusion_infection}
\alias{net_by_infection_complete}
\alias{net_by_infection_total}
\alias{net_by_infection_peak}
\title{Measures of network infection}
\usage{
net_by_infection_complete(.data)

net_by_infection_total(.data, normalized = TRUE)

net_by_infection_peak(.data)
}
\arguments{
\item{.data}{Network data with nodal changes,
as created by \code{play_diffusion()},
or a valid network diffusion model,
as created by \code{as_diffusion()}.}

\item{normalized}{Logical scalar, whether scores are normalized.
Different denominators may be used depending on the measure,
whether the object is one-mode or two-mode, and other arguments.
By default TRUE.}
}
\value{
A \code{network_measure} numeric score.
}
\description{
These functions allow measurement of various features of
a diffusion process at the network level:
\itemize{
\item \code{net_by_infection_complete()} measures the number of time steps until
(the first instance of) complete infection.
For diffusions that are not observed to complete,
this function returns the value of \code{Inf} (infinity).
This makes sure that at least ordinality is respected.
\item \code{net_by_infection_total()} measures the proportion or total number of nodes
that are infected/activated at some time by the end of the diffusion process.
This includes nodes that subsequently recover.
Where reinfection is possible, the proportion may be higher than 1.
\item \code{net_by_infection_peak()} measures the number of time steps until the
highest infection rate is observed.
}
}
\examples{
  smeg <- generate_smallworld(15, 0.025)
  smeg_diff <- play_diffusion(smeg)
  net_by_infection_complete(smeg_diff)
  net_by_infection_total(smeg_diff)
  net_by_infection_peak(smeg_diff)
}
\seealso{
Other diffusion:
\code{\link{mark_diff}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diffusion_node}},
\code{\link{member_diffusion}},
\code{\link{motif_exposure}},
\code{\link{motif_hazard}}

Other measures:
\code{\link{measure_assort_net}},
\code{\link{measure_assort_node}},
\code{\link{measure_breadth}},
\code{\link{measure_broker_node}},
\code{\link{measure_broker_tie}},
\code{\link{measure_brokerage}},
\code{\link{measure_central_between}},
\code{\link{measure_central_close}},
\code{\link{measure_central_degree}},
\code{\link{measure_central_eigen}},
\code{\link{measure_centralities_between}},
\code{\link{measure_centralities_close}},
\code{\link{measure_centralities_degree}},
\code{\link{measure_centralities_eigen}},
\code{\link{measure_closure}},
\code{\link{measure_closure_node}},
\code{\link{measure_cohesion}},
\code{\link{measure_core}},
\code{\link{measure_diffusion_net}},
\code{\link{measure_diffusion_node}},
\code{\link{measure_diverse_net}},
\code{\link{measure_diverse_node}},
\code{\link{measure_features}},
\code{\link{measure_fragmentation}},
\code{\link{measure_hierarchy}},
\code{\link{measure_periods}}
}
\concept{diffusion}
\concept{measures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/motif_census.R
\name{motif_net}
\alias{motif_net}
\alias{net_x_dyad}
\alias{net_x_triad}
\alias{net_x_tetrad}
\alias{net_x_mixed}
\title{Motifs of network cohesion}
\source{
Alejandro Espinosa 'netmem'
}
\usage{
net_x_dyad(.data)

net_x_triad(.data)

net_x_tetrad(.data)

net_x_mixed(.data, object2)
}
\arguments{
\item{.data}{A network object of class \code{mnet}, \code{igraph}, \code{tbl_graph}, \code{network}, or similar.
For more information on the standard coercion possible,
see \code{\link[manynet:as_tidygraph]{manynet::as_tidygraph()}}.}

\item{object2}{A second, two-mode network object.}
}
\value{
A \code{network_motif} named numeric vector or sometimes a data frame with
one row and a column for each motif type,
giving the count of each motif in the network.
This is printed as a tibble to avoid greedy printing of long vectors.
}
\description{
These functions include ways to take a census of the graphlets
in a network:
\itemize{
\item \code{net_x_dyad()} returns a census of dyad motifs in a network.
\item \code{net_x_triad()} returns a census of triad motifs in a network.
\item \code{net_x_tetrad()} returns a census of tetrad motifs in a network.
\item \code{net_x_mixed()} returns a census of triad motifs that span
a one-mode and a two-mode network.
}

See also \href{https://www.graphclasses.org/smallgraphs.html}{graph classes}.
}
\section{Dyad census}{

The dyad census counts the number of mutual, asymmetric, and null dyads
in a network.
For directed networks,
\itemize{
\item Mutual dyads have ties in both directions
\item Asymmetric dyads have a tie in one direction only
\item Null dyads have no ties
}

Note that for undirected and two-mode networks,
only mutual and null dyads are possible,
as the concept of an asymmetric dyad does not apply.
}

\section{Triad census}{

The triad census counts the number of three-node configurations in the network.
The function returns a matrix with a special naming convention:
\itemize{
\item 003: This is an empty triad; no ties
\item 012: This triad includes one tie
\item 102: This triad includes two ties, but they are not reciprocated
\item 021D: This triad includes two ties, one of which is reciprocated, and the other is directed towards the reciprocated tie
\item 021U: This triad includes two ties, one of which is
reciprocated, and the other is directed away from the reciprocated tie
\item 021C: This triad includes two ties, one of which is reciprocated, and the other is directed between the two non-reciprocated nodes
\item 111D: This triad includes three ties, two of which are
reciprocated, and the other is directed towards the reciprocated ties
\item 111U: This triad includes three ties, two of which are
reciprocated, and the other is directed away from the reciprocated ties
\item 030T: This triad includes three ties, all of which are
directed in a transitive manner (i.e. A->B, B->C, A->C)
\item 030C: This triad includes three ties, all of which are
directed in a cyclic manner (i.e. A->B, B->C
A->C)
\item 201: This triad includes three ties, all of which are reciproc
ated (i.e. A<->B, B<->C, A<->C)
\item 120D: This triad includes four ties, three of which are
reciprocated, and the other is directed towards the reciprocated ties
\item 120U: This triad includes four ties, three of which are
reciprocated, and the other is directed away from the reciprocated ties
\item 120C: This triad includes four ties, three of which are
reciprocated, and the other is directed between the two non-reciprocated
\item 210: This triad includes five ties, four of which are reciprocated, and the other is directed between the two non-reciprocated
\item 300: This triad includes six ties, all of which are reciprocated
}

Note that for undirected and two-mode networks, only 003, 102, and 201 are possible,
as the other configurations rely on the concept of directionality.
}

\section{Tetrad census}{

The tetrad census counts the number of four-node configurations in the network.
The function returns a matrix with a special naming convention:
\itemize{
\item E4 (aka co-K4): This is an empty set of four nodes; no ties
\item I4 (aka co-diamond): This is a set of four nodes with just one tie
\item H4 (aka co-C4): This set of four nodes includes two non-adjacent ties
\item L4 (aka co-paw): This set of four nodes includes two adjacent ties
\item D4 (aka co-claw): This set of four nodes includes three adjacent ties,
in the form of a triangle with one isolate
\item U4 (aka P4, four-actor line): This set of four nodes includes three ties
arranged in a line
\item Y4 (aka claw): This set of four nodes includes three ties all adjacent
to a single node
\item P4 (aka paw, kite): This set of four nodes includes four ties arranged
as a triangle with an extra tie hanging off of one of the nodes
\item C4 (aka bifan): This is a symmetric box or 4-cycle or set of shared choices
\item Z4 (aka diamond): This resembles C4 but with an extra tie cutting across the box
\item X4 (aka K4): This resembles C4 but with two extra ties cutting across the box;
a realisation of all possible ties
}

Graphs of these motifs can be shown using
\code{plot(net_x_tetrad(ison_southern_women))}.
}

\examples{
net_x_dyad(manynet::ison_algebra)
net_x_triad(manynet::ison_adolescents)
net_x_tetrad(ison_southern_women)
net_x_mixed(fict_marvel)
}
\references{
\subsection{On the dyad census}{

Holland, Paul W., and Samuel Leinhardt. 1970.
"A Method for Detecting Structure in Sociometric Data".
\emph{American Journal of Sociology}, 76: 492-513.
\doi{10.1016/B978-0-12-442450-0.50028-6}

Wasserman, Stanley, and Katherine Faust. 1994.
"Social Network Analysis: Methods and Applications".
Cambridge: Cambridge University Press.
}

\subsection{On the triad census}{

Davis, James A., and Samuel Leinhardt. 1967.
“\href{https://files.eric.ed.gov/fulltext/ED024086.pdf}{The Structure of Positive Interpersonal Relations in Small Groups}.” 55.
}

\subsection{On the tetrad census}{

Ortmann, Mark, and Ulrik Brandes. 2017.
“Efficient Orbit-Aware Triad and Quad Census in Directed and Undirected Graphs.”
\emph{Applied Network Science} 2(1):13.
\doi{10.1007/s41109-017-0027-2}.

McMillan, Cassie, and Diane Felmlee. 2020.
"Beyond Dyads and Triads: A Comparison of Tetrads in Twenty Social Networks".
\emph{Social Psychology Quarterly} 83(4): 383-404.
\doi{10.1177/0190272520944151}
}

\subsection{On the mixed census}{

Hollway, James, Alessandro Lomi, Francesca Pallotti, and Christoph Stadtfeld. 2017.
“Multilevel Social Spaces: The Network Dynamics of Organizational Fields.”
\emph{Network Science} 5(2): 187–212.
\doi{10.1017/nws.2017.8}
}
}
\seealso{
Other cohesion:
\code{\link{mark_triangles}},
\code{\link{measure_breadth}},
\code{\link{measure_cohesion}},
\code{\link{measure_fragmentation}},
\code{\link{motif_node}}

Other motifs:
\code{\link{motif_brokerage_net}},
\code{\link{motif_brokerage_node}},
\code{\link{motif_exposure}},
\code{\link{motif_hazard}},
\code{\link{motif_hierarchy}},
\code{\link{motif_node}},
\code{\link{motif_path}},
\code{\link{motif_periods}}
}
\concept{cohesion}
\concept{motifs}
